@@ -1,7 +1,7 @@
 #ifndef ALGO_BLAST_API___BLAST_RPS_OPTIONS__HPP
 #define ALGO_BLAST_API___BLAST_RPS_OPTIONS__HPP
 
-/*  $Id: blast_rps_options.hpp 303736 2011-06-13 12:22:17Z fongah2 $
+/*  $Id: blast_rps_options.hpp 383659 2012-12-17 17:37:15Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -99,6 +99,18 @@ public:
     /// @param hicut seg filtering parameter hicut [in]
     void SetSegFilteringHicut(double hicut) { m_Opts->SetSegFilteringHicut(hicut); }
 
+    /// Get composition based statistics mode
+    /// @return True if composition based statistics mode is enabled, false
+    /// otherwise
+    bool GetCompositionBasedStats(void) const
+    {return !(m_Opts->GetCompositionBasedStats() == eNoCompositionBasedStats);}
+
+    /// Set composition based statistics mode
+    /// @param mode Composition based statistics mode
+    void SetCompositionBasedStats(bool mode)
+    {m_Opts->SetCompositionBasedStats(mode ? eCompositionBasedStats
+                                      : eNoCompositionBasedStats);}
+
     /************************ Scoring options ************************/
     /// Returns GapOpeningCost
     int GetGapOpeningCost() const { return m_Opts->GetGapOpeningCost(); }
