@@ -1,4 +1,4 @@
-/*  $Id: blast_input_aux.hpp 309428 2011-06-24 11:40:15Z fongah2 $
+/*  $Id: blast_input_aux.hpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -243,10 +243,11 @@ ParseSequenceRangeOpenEnd(const string& range_str, const char* error_prefix = NU
 /** Retrieve the appropriate batch size for the specified task 
  * @param program BLAST task [in]
  * @param is_ungapped true if ungapped BLAST search is requested [in]
+ * @param remote true if remote BLAST search is requested [in]
  */
 NCBI_BLASTINPUT_EXPORT
 int
-GetQueryBatchSize(EProgram program, bool is_ungapped = false);
+GetQueryBatchSize(EProgram program, bool is_ungapped = false, bool remote = false);
 
 /** Read sequence input for BLAST 
  * @param in input stream from which to read [in]
