@@ -1,4 +1,4 @@
-/*  $Id: blastfmtutil.hpp 309006 2011-06-21 14:12:33Z fongah2 $
+/*  $Id: blastfmtutil.hpp 389291 2013-02-14 18:36:09Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -162,6 +162,20 @@ public:
     								  align_format::CDisplaySeqalign::SeqLocCharOption mask_char,
     								  int query_frame);
 
+    /*
+     *  @brief Get aggregrated score for each subject
+     *  @input param: alignSet -- seq aligns for 1 query
+     *  @ouput param: alignSet -- insert % query coverage per subject
+     *  @note: this function will return if the first seqalign in the
+     *         set contains score for query coverage %
+     *         (i.e. seqalign set from getreq contains query coverage %
+     *          already, we only need to do the calcaultion if the format r
+     *          request is from SB)
+     *
+     */
+
+    static void InsertSubjectScores (objects::CSeq_align_set & org_align_set,
+    					      const objects::CBioseq_Handle & query_handle);
 };
 
 
@@ -178,6 +192,18 @@ public:
     CBlastFormattingMatrix(int** data, unsigned int nrows, unsigned int ncols); 
 };
 
+/// Structure to hold data for incremental XML formatting.
+struct SBlastXMLIncremental : public CObject
+{
+    /// Default ctor()
+    SBlastXMLIncremental();
+
+    /// ctor sets to true, set to false for first chunk.
+    int m_IterationNum;
+
+    /// tag to be printed at end.
+    string m_SerialXmlEnd;
+};
 
 END_NCBI_SCOPE
 
