@@ -1,4 +1,4 @@
-/*  $Id: igblast.hpp 359118 2012-04-10 19:00:13Z maning $
+/*  $Id: igblast.hpp 383536 2012-12-14 21:12:01Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -66,6 +66,8 @@ public:
     bool m_IsProtein;                // search molecular type
     string m_Origin;                 // the origin of species
     string m_DomainSystem;           // domain system for annotation
+    string m_SequenceType;           //ig or tcr?
+    int m_Min_D_match;                 //the word size for D gene search
     string m_AuxFilename;            // auxulary file name
     string m_IgDataPath;             // internal data path
     CRef<CLocalDbAdapter> m_Db[4];   // user specified germline database
@@ -82,6 +84,9 @@ public:
     bool m_MinusStrand;              // hit is on minus strand of the query
     vector<string> m_TopGeneIds;     // Top match germline gene ID
     vector<string> m_ChainType;      // chain types of the query ([0]) and subjects ([1:])
+    string m_ChainTypeToShow;        // chain type to show to user.  Normally this is 
+                                     //the same as m_ChainType[0] but could be different
+                                     // in case o TCRA/D chains which can use both JA and JD
     int m_GeneInfo[6];               // The (start) and (end offset + 1) for VDJ
     int m_FrameInfo[3];              // Coding frame start offset for V start, V end,
                                      // and V start.
@@ -134,11 +139,19 @@ public:
         return -1;
     }
 
+    const string GetDJChainType(const string sid) {
+        if (m_DJChainType.find(sid) != m_DJChainType.end()) {
+            return m_DJChainType[sid];
+        }
+        return "N/A";
+    }
+
 private:
     map<string, int> m_DomainIndex;
     vector<int> m_DomainData;
     map<string, string> m_DomainChainType;
     map<string, int> m_FrameOffset;
+    map<string, string> m_DJChainType;
 };
 
 class CIgBlastResults : public CSearchResults 
@@ -253,9 +266,10 @@ private:
                         CRef<CBlastOptionsHandle>     &opts_hndl);
 
     /// Prepare blast option handle and query for D, J germline database search
-    void x_SetupDJSearch(vector<CRef <CIgAnnotation> > &annots,
+    void x_SetupDJSearch(const vector<CRef <CIgAnnotation> > &annots,
                          CRef<IQueryFactory>           &qf,
-                         CRef<CBlastOptionsHandle>     &opts_hndl);
+                         CRef<CBlastOptionsHandle>     &opts_hndl,
+                         int db_type);
 
     /// Prepare blast option handle and query for specified database search
     void x_SetupDbSearch(vector<CRef <CIgAnnotation> > &annot,
@@ -296,6 +310,26 @@ private:
     static void s_SetAnnotation(vector<CRef <CIgAnnotation> > &annot,
                                 CRef<CSearchResultSet> &final_results);
 
+    void x_FindDJ(CRef<CSearchResultSet>& results_D,
+                  CRef<CSearchResultSet>& results_J,
+                  CRef <CIgAnnotation> & annot,
+                  CRef<CSeq_align_set>& align_D,
+                  CRef<CSeq_align_set>& align_J,
+                  string q_ct,
+                  bool q_ms,
+                  ENa_strand q_st,
+                  int q_ve,
+                  int iq);
+
+    void x_FindDJAln(CRef<CSeq_align_set>& align_D,
+                     CRef<CSeq_align_set>& align_J,
+                     string q_ct,
+                     bool q_ms,
+                     ENa_strand q_st,
+                     int q_ve,
+                     int iq,
+                     bool va_or_vd_as_heavy_chain);
+    
 };
 
 END_SCOPE(blast)
