@@ -1,7 +1,7 @@
 #ifndef CGI___SESSION__HPP
 #define CGI___SESSION__HPP
 
-/*  $Id: cgi_session.hpp 338954 2011-09-23 15:31:09Z ucko $
+/*  $Id: cgi_session.hpp 367927 2012-06-29 14:08:31Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -63,9 +63,9 @@ class NCBI_XCGI_EXPORT CCgiSession
 public:
     typedef list<string> TNames;
 
-    static const string kDefaultSessionIdName;
-    static const string kDefaultSessionCookieDomain;
-    static const string kDefaultSessionCookiePath;
+    static const char* kDefaultSessionIdName;
+    static const char* kDefaultSessionCookieDomain;
+    static const char* kDefaultSessionCookiePath;
 
     /// Session status
     enum EStatus {
