@@ -1,7 +1,7 @@
 #ifndef NCBI_CGI_CTX__HPP
 #define NCBI_CGI_CTX__HPP
 
-/*  $Id: cgictx.hpp 372363 2012-08-16 18:59:19Z rafanovi $
+/*  $Id: cgictx.hpp 371313 2012-08-07 18:35:25Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -103,7 +103,7 @@ public:
     virtual ~CCtxMsgString(void);
     virtual CNcbiOstream& Write(CNcbiOstream& os) const;
 
-    static string sm_nl;
+    static const char* sm_nl;
 
 private:
     string m_Message;
