@@ -1,7 +1,7 @@
 #ifndef NCBICGI__HPP
 #define NCBICGI__HPP
 
-/*  $Id: ncbicgi.hpp 357979 2012-03-28 14:45:54Z ivanov $
+/*  $Id: ncbicgi.hpp 384670 2012-12-29 03:52:00Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -627,6 +627,16 @@ class CNcbiEnvironment;
 class CTrackingEnvHolder;
 
 
+// Base helper class for building query string from request arguments.
+class CEntryCollector_Base {
+public:
+    virtual ~CEntryCollector_Base(void) {}
+    virtual void AddEntry(const string& name,
+                          const string& value,
+                          bool          is_index = false) = 0;
+};
+
+
 ///////////////////////////////////////////////////////
 ///
 ///  CCgiRequest::
@@ -832,6 +842,10 @@ public:
     /// Get full set of arguments (both GET and POST), URL-encoded.
     /// A &-separated list of exclusions can be set in CGI_LOG_EXCLUDE_ARGS
     /// variable or [CGI] LOG_EXCLUDE_ARGS value in ini file.
+    void GetCGIEntries(CEntryCollector_Base& collector) const;
+
+    /// Shortcut for collecting arguments into a URL-style string.
+    /// @sa GetCGIEntries
     string GetCGIEntriesStr(void) const;
 
     bool CalcChecksum(string& checksum, string& content) const;
