@@ -1,4 +1,4 @@
-/* $Id: ncbiconf_msvc.h 281326 2011-05-03 16:47:04Z grichenk $
+/* $Id: ncbiconf_msvc.h 361821 2012-05-04 14:28:13Z ucko $
  * By Denis Vakatov, NCBI (vakatov@ncbi.nlm.nih.gov)
  *
  * MS-Win 32/64, MSVC++ 6.0/.NET
@@ -78,6 +78,7 @@
 #define NCBI_FORCEINLINE __forceinline
 #define NCBI_RESTRICT_C
 #define NCBI_RESTRICT_CXX
+#define NCBI_TLS_VAR __declspec(thread)
 
 #ifdef _WIN64
 #  define HOST         "i386-pc-win64"
@@ -98,6 +99,7 @@ typedef   int   ssize_t;
 #  define NCBI_PLATFORM_BITS  32
 #endif
 
+#define HAVE_LIBKRB5                    1
 
 /* FreeTDS v0.63 */
 
