@@ -1,4 +1,4 @@
-/* $Id: ncbiconf_xcode.h 346160 2011-12-05 15:35:08Z ucko $
+/* $Id: ncbiconf_xcode.h 361821 2012-05-04 14:28:13Z ucko $
  * By Vlad Lebedev, NCBI (lebedev@ncbi.nlm.nih.gov)
  *
  * Mac OS X - xCode Build
@@ -295,6 +295,9 @@
    the standard libraries. */
 #define HAVE_LIBICONV 1
 
+/* Define to 1 if libgssapi_krb5 is available. */
+#define HAVE_LIBKRB5 1
+
 /* Define to 1 if KSTAT is available, either in its own library or as part of
    the standard libraries. */
 /* #undef HAVE_LIBKSTAT */
@@ -646,6 +649,10 @@
 /* Define to 1 if SQLColAttribute's last argument is an SQLLEN * */
 /* #undef NCBI_SQLCOLATTRIBUTE_SQLLEN */
 
+/* Define to whatever syntax your compiler supports for declaring thread-local
+   variables, or leave undefined if it doesn't. */
+/* #undef NCBI_TLS_VAR */
+
 /* Define to 1 if building universal (multi-architecture) binaries. */
 /* (Not necessarily the case, but we shouldn't hardcode anything
  *  PowerPC-specific either.) */
