@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBI_EXPORT__H
 #define COMMON___NCBI_EXPORT__H
 
-/*  $Id: ncbi_export.h 368190 2012-07-03 20:42:50Z rafanovi $
+/*  $Id: ncbi_export.h 374128 2012-09-06 17:57:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -201,6 +201,7 @@
 #  define NCBI_XALGOSEGMASK_EXPORTS
 #  define NCBI_XALGOCONTIG_ASSEMBLY_EXPORTS
 #  define NCBI_XBLASTFORMAT_EXPORTS
+#  define NCBI_XPRIMER_EXPORTS
 #endif
 
 
@@ -1145,7 +1146,7 @@
 #  define NCBI_XLOADER_SRA_EXPORT NCBI_DLL_IMPORT
 #endif
 
-/* Export specifier for library ncbi_xloader_sra
+/* Export specifier for library ncbi_xloader_csra
  */
 #ifdef NCBI_XLOADER_CSRA_EXPORTS
 #  define NCBI_XLOADER_CSRA_EXPORT NCBI_DLL_EXPORT
@@ -1153,6 +1154,14 @@
 #  define NCBI_XLOADER_CSRA_EXPORT NCBI_DLL_IMPORT
 #endif
 
+/* Export specifier for library ncbi_xloader_wgs
+ */
+#ifdef NCBI_XLOADER_WGS_EXPORTS
+#  define NCBI_XLOADER_WGS_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_XLOADER_WGS_EXPORT NCBI_DLL_IMPORT
+#endif
+
 /* Export specifier for library ncbi_xloader_bam
  */
 #ifdef NCBI_XLOADER_BAM_EXPORTS
@@ -1233,6 +1242,14 @@
 #  define NCBI_XOMSSA_EXPORT NCBI_DLL_IMPORT
 #endif
 
+/* Export specifier for library xprimer
+ */
+#ifdef NCBI_XPRIMER_EXPORTS
+#  define NCBI_XPRIMER_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_XPRIMER_EXPORT NCBI_DLL_IMPORT
+#endif
+
 /* Export specifier for library xreader
  */
 #ifdef NCBI_XREADER_EXPORTS
