@@ -1,7 +1,7 @@
 #ifndef CONNECT___SERVER_CONNECTION__HPP
 #define CONNECT___SERVER_CONNECTION__HPP
 
-/* $Id: server_connection.hpp 311064 2011-07-07 23:06:34Z ucko $
+/* $Id: server_connection.hpp 355488 2012-03-06 17:12:11Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,8 +33,6 @@
 /// @file server_connection.hpp
 /// Internal header for threaded server connections.
 
-#include <util/thread_pool.hpp>
-
 #include <connect/server.hpp>
 
 
@@ -46,6 +44,20 @@
 
 BEGIN_NCBI_SCOPE
 
+
+enum EServerConnType {
+    eInactiveSocket,
+    eActiveSocket,
+    eListener,
+    ePreDeferredSocket,
+    eDeferredSocket,
+    ePreClosedSocket,
+    eClosedSocket
+};
+
+
+class CServer_ConnectionPool;
+
 class IServer_ConnectionBase
 {
 public:
@@ -62,6 +74,14 @@ public:
     virtual void OnOverflow(EOverflowReason) { }
     virtual void Activate(void) { }
     virtual void Passivate(void) { }
+
+private:
+    friend class CServer_ConnectionPool;
+    friend class IServer_MessageHandler;
+
+    CTime expiration;
+    CFastMutex type_lock;
+    volatile EServerConnType type;
 };
 
 class NCBI_XCONNECT_EXPORT CServer_Connection : public IServer_ConnectionBase,
@@ -86,10 +106,11 @@ public:
     virtual bool IsOpen(void);
     // connection-specific methods
     void OnSocketEvent(EServIO_Event event);
+    virtual ~CServer_Connection();
 private:
     auto_ptr<IServer_ConnectionHandler> m_Handler;
     bool m_Open;
-} ;
+};
 
 class CServer_Listener : public IServer_ConnectionBase,
                          public CListeningSocket // CPollable
