@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_STREAM__HPP
 #define CONNECT___NCBI_CONN_STREAM__HPP
 
-/* $Id: ncbi_conn_stream.hpp 362481 2012-05-09 17:23:43Z rafanovi $
+/* $Id: ncbi_conn_stream.hpp 376887 2012-10-04 18:10:28Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -69,8 +69,8 @@
  */
 
 #include <connect/ncbi_ftp_connector.h>
-#include <connect/ncbi_namedpipe_connector.hpp>
 #include <connect/ncbi_memory_connector.h>
+#include <connect/ncbi_namedpipe_connector.hpp>
 #include <connect/ncbi_pipe_connector.hpp>
 #include <connect/ncbi_service_connector.h>
 #include <connect/ncbi_socket_connector.h>
@@ -111,9 +111,18 @@ const size_t kConn_DefaultBufSize = 4096;
 /// @sa
 ///   CONN_SetCallback, eCONN_OnClose
 
-class NCBI_XCONNECT_EXPORT CConn_IOStream : public    CNcbiIostream,
-                                            protected CConnIniter
+class NCBI_XCONNECT_EXPORT CConn_IOStream : public            CNcbiIostream,
+                                            virtual protected CConnIniter
 {
+public:
+    /// Must be compatible by values with TCONN_Flags.
+    enum {
+        fConn_Untie         = 1,  ///< do not flush before reading
+        fConn_ReadBuffered  = 2,  ///< read buffer is to be allocated
+        fConn_WriteBuffered = 4   ///< write buffer is to be allocated
+    } EConn_Flag;
+    typedef unsigned int TConn_Flags;  ///< bitwise OR of EConn_Flag
+
 public:
     /// Create a stream based on a CONN, which is to be closed upon
     /// stream dtor only if "close" parameter is passed as "true".
@@ -127,7 +136,7 @@ public:
     ///  Default I/O timeout
     /// @param buf_size
     ///  Default size of underlying stream buffer's I/O arena
-    /// @param tie
+    /// @param flags
     ///  Specifies whether to tie input and output -- a tied stream flushes
     ///  all pending output prior to doing any input.
     /// @sa
@@ -138,7 +147,7 @@ public:
      bool            close    = false,
      const STimeout* timeout  = kDefaultTimeout,
      size_t          buf_size = kConn_DefaultBufSize,
-     bool            tie      = true,
+     TConn_Flags     flags    = fConn_ReadBuffered | fConn_WriteBuffered,
      CT_CHAR_TYPE*   ptr      = 0,
      size_t          size     = 0);
 
@@ -154,7 +163,7 @@ protected:
     ///  Default I/O timeout
     /// @param buf_size
     ///  Default size of underlying stream buffer's I/O arena
-    /// @param tie
+    /// @param flags
     ///  Specifies whether to tie input and output -- a tied stream flushes
     ///  all pending output prior to doing any input.
     /// @sa
@@ -163,7 +172,7 @@ protected:
     (CONNECTOR       connector,
      const STimeout* timeout  = kDefaultTimeout,
      size_t          buf_size = kConn_DefaultBufSize,
-     bool            tie      = true,
+     TConn_Flags     flags    = fConn_ReadBuffered | fConn_WriteBuffered,
      CT_CHAR_TYPE*   ptr      = 0,
      size_t          size     = 0);
 
@@ -214,7 +223,7 @@ public:
     ///   CONN_Close
     EIO_Status      Close(void);
 
-    /// Cancellation support
+    /// Cancellation support.
     /// @note ICanceled implementation must be derived from CObject as its
     /// first superclass.
     /// @sa
@@ -222,10 +231,13 @@ public:
     EIO_Status      SetCanceledCallback(const ICanceled* canceled);
 
     /// @return
-    ///   Internal CONNection handle (NULL if unset)
+    ///   Internal CONNection handle, which is still owned and used by
+    ///   the stream (or NULL if no such connection exists)
+    /// @note
+    ///   Connection can have additional flags set for I/O processing.
     /// @sa
-    ///   CONN, ncbi_connection.h
-    CONN GetCONN(void) const;
+    ///   CONN, ncbi_connection.h, CONN_GetFlags
+    CONN            GetCONN(void) const;
 
 protected:
     void x_Cleanup(void);
@@ -361,10 +373,12 @@ public:
     ///  Host to connect to
     /// @param port
     ///  ... and port number
-    /// @data
+    /// @param data
     ///  Pointer to block of data to send once connection is ready
-    /// @size
+    /// @param size
     ///  Size of the data block to send (or 0 if to send nothing)
+    /// @param flags
+    ///  Socket flags
     /// @param max_try
     ///  Number of attempts
     /// @param timeout
@@ -389,20 +403,24 @@ public:
     ///
     /// scheme                          -- must be http or unspecified, checked
     /// host:port                       -- target server
-    /// http_proxy_host:http_proxy_port -- proxy server to tunnel via
+    /// http_proxy_host:http_proxy_port -- HTTP proxy server to tunnel via
     /// http_proxy_user:http_proxy_pass -- credentials for the proxy, if needed
     /// http_proxy_leak                 -- ignore bad proxy and connect direct
-    /// timeout                         -- I/O timeout (NB: no kDefaultTimeout)
+    /// timeout                         -- timeout to connect to HTTP proxy
     /// firewall                        -- if true then look at proxy_server
     /// proxy_server                    -- use as "host" if non-empty and FW
-    /// debug_printout                  -- how to log socket data
+    /// debug_printout                  -- how to log socket data by default
     ///
     /// @param net_info
     ///  Connection point and proxy tunnel location
-    /// @data
+    /// @param data
     ///  Pointer to block of data to send once connection is ready
-    /// @size
+    /// @param size
     ///  Size of the data block to send (or 0 if to send nothing)
+    /// @param flags
+    ///  Socket flags
+    /// @param timeout
+    ///  Default I/O timeout
     /// @param buf_size
     ///  Default buffer size
     /// @sa
@@ -412,6 +430,7 @@ public:
      const void*         data     = 0,
      size_t              size     = 0,
      TSOCK_Flags         flags    = fSOCK_LogDefault,
+     const STimeout*     timeout  = kDefaultTimeout,
      size_t              buf_size = kConn_DefaultBufSize);
 
     /// This variant uses an existing socket "sock" to build a stream upon it.
@@ -434,14 +453,14 @@ public:
      const STimeout* timeout  = kDefaultTimeout,
      size_t          buf_size = kConn_DefaultBufSize);
 
-    /// This variant uses existing CSocket to build a stream upon it.
+    /// This variant uses an existing CSocket to build a stream up on it.
     /// NOTE:  it revokes all ownership of the "socket"'s internals
     /// (effectively leaving the CSocket empty);  CIO_Exception(eInvalidArg)
     /// is thrown if the internal SOCK is not owned by the passed CSocket.
     /// More details:  <ncbi_socket_connector.h>::SOCK_CreateConnectorOnTop().
     ///
     /// @param socket
-    ///  Socket to build the stream on
+    ///  Socket to build the stream up on
     /// @sa
     ///  CSocket, ncbi_socket.hpp
     CConn_SocketStream
@@ -449,9 +468,6 @@ public:
      const STimeout* timeout  = kDefaultTimeout,
      size_t          buf_size = kConn_DefaultBufSize);
 
-protected:
-    SOCK* m_SockPtr;
-
 private:
     // Disable copy constructor and assignment.
     CConn_SocketStream(const CConn_SocketStream&);
@@ -657,10 +673,10 @@ public:
      );
     virtual ~CConn_PipeStream();
 
-    CPipe& GetPipe(void) { return m_Pipe; }
+    CPipe& GetPipe(void) { return *m_Pipe; }
 
 protected:
-    CPipe m_Pipe; ///< Underlying pipe.
+    CPipe* m_Pipe; ///< Underlying pipe.
 
 private:
     // Disable copy constructor and assignment.
@@ -711,11 +727,12 @@ public:
     (const string&        host,
      const string&        user,
      const string&        pass,
-     const string&        path    = kEmptyStr,
-     unsigned short       port    = 0,
-     TFTP_Flags           flag    = 0,
-     const SFTP_Callback* cmcb    = 0,
-     const STimeout*      timeout = kDefaultTimeout
+     const string&        path     = kEmptyStr,
+     unsigned short       port     = 0,
+     TFTP_Flags           flag     = 0,
+     const SFTP_Callback* cmcb     = 0,
+     const STimeout*      timeout  = kDefaultTimeout,
+     size_t               buf_size = kConn_DefaultBufSize
      );
 
     /// Abort any command in progress, read and discard all input data,
@@ -740,15 +757,16 @@ class NCBI_XCONNECT_EXPORT CConn_FTPDownloadStream : public CConn_FtpStream
 public:
     CConn_FTPDownloadStream
     (const string&        host,
-     const string&        file    = kEmptyStr,
-     const string&        user    = "ftp",
-     const string&        pass    = "-none@", // "-" helps make login quieter
-     const string&        path    = kEmptyStr,
-     unsigned short       port    = 0, ///< 0 means default (21 for FTP)
-     TFTP_Flags           flag    = 0,
-     const SFTP_Callback* cmcb    = 0,
-     Uint8                offset  = 0, ///< file offset to begin download from
-     const STimeout*      timeout = kDefaultTimeout
+     const string&        file     = kEmptyStr,
+     const string&        user     = "ftp",
+     const string&        pass     = "-none@", // "-" helps make login quieter
+     const string&        path     = kEmptyStr,
+     unsigned short       port     = 0, ///< 0 means default (21 for FTP)
+     TFTP_Flags           flag     = 0,
+     const SFTP_Callback* cmcb     = 0,
+     Uint8                offset   = 0, ///< file offset to begin download from
+     const STimeout*      timeout  = kDefaultTimeout,
+     size_t               buf_size = kConn_DefaultBufSize
      );
 
 private:
@@ -791,7 +809,8 @@ private:
 /// Writing to the stream is undefined.
 ///
 extern NCBI_XCONNECT_EXPORT
-CConn_IOStream* NcbiOpenURL(const string& url);
+CConn_IOStream* NcbiOpenURL(const string& url,
+                            size_t        buf_size = kConn_DefaultBufSize);
 
 #endif //NCBI_CONN_STREAM_EXPERIMENTAL_API
 
