@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_TEST__HPP
 #define CONNECT___NCBI_CONN_TEST__HPP
 
-/* $Id: ncbi_conn_test.hpp 366508 2012-06-14 18:04:58Z rafanovi $
+/* $Id: ncbi_conn_test.hpp 371606 2012-08-09 15:55:12Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -65,8 +65,8 @@ public:
 
     /// Create test suite
     /// @param timeout
-    ///  non-NULL pointer to a finite timeout, or NULL (infinite); or
-    ///  default timeout (which is 30 seconds).
+    ///  non-NULL pointer to a finite timeout, or kInfiniteTimeout (NULL);
+    ///  or kDefaultTimeout (which is 30 seconds).
     /// @param out
     ///  test results get posted to the stream pointed to by this parameter;
     ///  no output is produced if "out" is NULL.
@@ -75,9 +75,6 @@ public:
     CConnTest(const STimeout* timeout = kDefaultTimeout,
               CNcbiOstream* output = 0, SIZE_TYPE width = 72);
 
-    void     SetEmail(const string& email)
-    { m_Email = email; }
-
     void     SetWidth(SIZE_TYPE width = 72)
     { m_Width = width; }
 
@@ -86,6 +83,12 @@ public:
 
     void     SetTimeout(const STimeout* timeout = kDefaultTimeout);
 
+    void     SetDebugPrintout(EDebugPrintout debug_printout)
+    { m_DebugPrintout = debug_printout; }
+
+    void     SetEmail(const string& email)
+    { m_Email = email; }
+
     // NB: ICanceled-impl must be derived from CObject (as a first subclass)
     void     SetCanceledCallback(const ICanceled* canceled)
     { m_Canceled = canceled; }
@@ -102,7 +105,7 @@ public:
     /// check to likely succeed).
     ///
     /// @param stage
-    ///  final stage requested or performed, when the call returns
+    ///  the last stage requested or performed, when the call returns
     /// @param reason
     ///  a pointer to a string to get a failure explanation
     /// @return
@@ -119,7 +122,7 @@ public:
     ///  behavior of service-related stateful data retrievals.  That is, when
     ///  the test is complete, only the safest route found operational will be
     ///  used to connect to NCBI for the life of the program.  Re-running the
-    ///  test may clobber these results and, thus, is highly unrecommended!
+    ///  test may reset these results and, thus, is highly unrecommended!
     ///
     virtual EIO_Status Execute(EStage& stage, string* reason = 0);
 
@@ -214,7 +217,10 @@ protected:
     const STimeout*       m_Timeout;
     CNcbiOstream*         m_Output;
     SIZE_TYPE             m_Width;
+
+    /// Appearance
     string                m_Email;
+    EDebugPrintout        m_DebugPrintout;
 
     /// Certain properties of communication as determined by configuration
     bool                  m_HttpProxy;
