@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNECTION__H
 #define CONNECT___NCBI_CONNECTION__H
 
-/* $Id: ncbi_connection.h 362482 2012-05-09 17:24:01Z rafanovi $
+/* $Id: ncbi_connection.h 363842 2012-05-19 20:34:38Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,14 +61,16 @@ extern "C" {
 
 
 struct SConnectionTag;
-typedef struct SConnectionTag* CONN;      /* connection handle */
+typedef struct SConnectionTag* CONN;  /**< connection handle */
 
 
+/** CONN flags should be kept compatible with CConn_IOStream::TConn_Flags.
+ */
 enum ECONN_Flag {
-    fCONN_Untie      = 1,  /* do not call flush method prior to every read */
-    fCONN_Supplement = 2   /* supplement I/O with extended return codes    */
+    fCONN_Untie      = 1,  /**< do not call flush method prior to every read */
+    fCONN_Supplement = 64  /**< supplement I/O with extended return codes    */
 };
-typedef unsigned int TCONN_Flags;  /* bitwise OR of ECONN_Flag */
+typedef unsigned int TCONN_Flags;  /**< bitwise OR of ECONN_Flag */
 
 
 /** Create all data necessary to establish a new connection (merely bind it to
@@ -85,15 +87,15 @@ typedef unsigned int TCONN_Flags;  /* bitwise OR of ECONN_Flag */
  *  CONN_Close
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_CreateEx
-(CONNECTOR   connector,  /* [in]  connector                        */
- TCONN_Flags flags,      /* [in]  connection flags                 */
- CONN*       conn        /* [out] handle of the created connection */
+(CONNECTOR   connector,  /**< [in]  connector                        */
+ TCONN_Flags flags,      /**< [in]  connection flags                 */
+ CONN*       conn        /**< [out] handle of the created connection */
  );
 
 /** Same as CONN_CreateEx() called with 0 in the "flags" parameter */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Create
-(CONNECTOR   connector,  /* [in]  connector                        */
- CONN*       conn        /* [out] handle of the created connection */
+(CONNECTOR   connector,  /**< [in]  connector                        */
+ CONN*       conn        /**< [out] handle of the created connection */
  );
 
 
@@ -108,8 +110,8 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Create
  *  CONN_Create, CONN_Close
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_ReInit
-(CONN      conn,      /* [in] connection handle */
- CONNECTOR connector  /* [in] new connector     */
+(CONN      conn,      /**< [in] connection handle */
+ CONNECTOR connector  /**< [in] new connector     */
  );
 
 
@@ -118,7 +120,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_ReInit
  * the connection.  Return value NULL denotes unknown connection type.
  */
 extern NCBI_XCONNECT_EXPORT const char* CONN_GetType
-(CONN conn  /* [in] connection handle */ 
+(CONN conn  /**< [in] connection handle */ 
  );
 
 
@@ -131,8 +133,8 @@ extern NCBI_XCONNECT_EXPORT const char* CONN_GetType
  * with 0, and to return 0.
  */
 extern NCBI_XCONNECT_EXPORT TNCBI_BigCount CONN_GetPosition
-(CONN      conn,  /* [in] connection handle */ 
- EIO_Event event  /* [in] see description   */
+(CONN      conn,  /**< [in] connection handle */ 
+ EIO_Event event  /**< [in] see description   */
  );
 
 
@@ -145,7 +147,7 @@ extern NCBI_XCONNECT_EXPORT TNCBI_BigCount CONN_GetPosition
  * by the returned string when the description is no longer needed.
  */
 extern NCBI_XCONNECT_EXPORT char* CONN_Description
-(CONN conn  /* [in] connection handle */
+(CONN conn  /**< [in] connection handle */
  );
 
 
@@ -156,9 +158,9 @@ extern NCBI_XCONNECT_EXPORT char* CONN_Description
  *         connector-specific value is used (this is the default).
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_SetTimeout
-(CONN            conn,    /* [in] connection handle */
- EIO_Event       event,   /* [in] I/O direction     */
- const STimeout* timeout  /* [in] new timeout       */
+(CONN            conn,    /**< [in] connection handle */
+ EIO_Event       event,   /**< [in] I/O direction     */
+ const STimeout* timeout  /**< [in] new timeout       */
  );
 
 
@@ -168,22 +170,23 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_SetTimeout
  * or CONN_Close().
  */
 extern NCBI_XCONNECT_EXPORT const STimeout* CONN_GetTimeout
-(CONN      conn,  /* [in] connection handle                  */
- EIO_Event event  /* [in] I/O direction, not "eIO_ReadWrite" */
+(CONN      conn,  /**< [in] connection handle                   */
+ EIO_Event event  /**< [in] I/O direction, not "eIO_ReadWrite"! */
  );
 
 
 /** Block on the connection until it becomes available for either reading or
  * writing (depending on "event"), until timeout expires, or until any error.
- * @note  "timeout" can also be one of two special values:
- *        NULL (means infinite), kDefaultTimeout (connector-defined).
+ * @note  "timeout" can also be one of the two special values:
+ *        * NULL (for infinite timeout, also as kInfiniteTimeout);
+ *        * kDefaultTimeout (connector-defined).
  * @sa
  *  CONN_Read, CONN_Write
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Wait
-(CONN            conn,    /* [in] connection handle                         */
- EIO_Event       event,   /* [in] can be either eIO_Read or eIO_Write only! */
- const STimeout* timeout  /* [in] the maximal wait time                     */
+(CONN            conn,    /**< [in] connection handle                        */
+ EIO_Event       event,   /**< [in] can only be either of eIO_Read, IO_Write */
+ const STimeout* timeout  /**< [in] the maximal wait time                    */
  );
 
 
@@ -192,13 +195,16 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Wait
  * May not return eIO_Success if no data at all can be written before
  * the write timeout expires or an error occurs.
  * Parameter "how" modifies the write behavior:
- * eIO_WritePlain      -- return immediately after having written as little
+ * * @var eIO_WritePlain
+ *                        return immediately after having written as little
  *                        as 1 byte of data (return eIO_Success), or if an
  *                        error has occurred (and "*n_written == 0");
- * eIO_WritePersist    -- return only after having written all of the data from
+ * * @var eIO_WritePersist
+ *                        return only after having written all of the data from
  *                        "buf" (return eIO_Success), or if an error has
- *                        occurred (fewer bytes written, non-eIO_Success).
- * eIO_WriteSupplement -- same as eIO_WritePlain but can return non-eIO_Success
+ *                        occurred (fewer bytes written, non-eIO_Success);
+ * * @var eIO_WriteSupplement
+ *                        same as eIO_WritePlain but can return non-eIO_Success
  *                        even with some data written (as indicated by
  *                        "*n_written"), to signify that an error has occurred
  *                        past the just written block of data.
@@ -207,11 +213,11 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Wait
  *  CONN_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Write
-(CONN            conn,       /* [in]  connection handle                     */ 
- const void*     buf,        /* [in]  pointer to the data buffer to write   */ 
- size_t          size,       /* [in]  # of bytes to write                   */ 
- size_t*         n_written,  /* [out, non-NULL] # of actually written bytes */
- EIO_WriteMethod how         /* [in]  eIO_WritePlain or eIO_WritePersist    */
+(CONN            conn,       /**< [in]  connection handle                    */
+ const void*     buf,        /**< [in]  pointer to the data buffer to write  */
+ size_t          size,       /**< [in]  # of bytes to write                  */
+ size_t*         n_written,  /**< [out] non-NULL, # of actually written bytes*/
+ EIO_WriteMethod how         /**< [in]  eIO_WritePlain or eIO_WritePersist   */
  );
 
 
@@ -225,9 +231,9 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Write
  *  CONN_Read, CONN_Write
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_PushBack
-(CONN        conn,  /* [in] connection handle                     */
- const void* buf,   /* [in] pointer to the data being pushed back */
- size_t      size   /* [in] # of bytes to push back               */
+(CONN        conn,  /**< [in] connection handle                     */
+ const void* buf,   /**< [in] pointer to the data being pushed back */
+ size_t      size   /**< [in] # of bytes to push back               */
  );
 
 
@@ -242,7 +248,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_PushBack
  *  CONN_Read, CONN_Write, CONN_Close
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Flush
-(CONN        conn  /* [in] connection handle */
+(CONN        conn  /**< [in] connection handle */
  );
 
 
@@ -250,15 +256,20 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Flush
  * Return the number of actually read bytes in "*n_read".
  * May not return eIO_Success if no data at all can be read before
  * the read timeout expires or an error occurs.
+ *
  * Parameter "how" modifies the read behavior:
- *   eIO_ReadPlain      -- return immediately after having read as many as
+ * * @var eIO_ReadPlain
+ *                         return immediately after having read as many as
  *                         1 byte from connection (return eIO_Success), or
  *                         if an error has occurred (and "*n_read == 0");
- *   eIO_ReadPeek       -- eIO_ReadPlain but don't discard data read from CONN;
- *   eIO_ReadPersist    -- return only after having filled full "buf" with data
+ * * @var eIO_ReadPeek
+ *                         eIO_ReadPlain but don't discard data read from CONN;
+ * * @var eIO_ReadPersist
+ *                         return only after having filled full "buf" with data
  *                         (exactly "size" bytes, eIO_Success), or if an error
- *                         has occurred (fewer bytes, non-eIO_Success).
- *   eIO_ReadSupplement -- same as eIO_ReadPlain but can return non-eIO_Success
+ *                         has occurred (fewer bytes, non-eIO_Success);
+ * * @var eIO_ReadSupplement
+ *                         same as eIO_ReadPlain but can return non-eIO_Success
  *                         even with some read data (as indicated by
  *                         "*n_read"), to show that an error has been following
  *                         the just read block of data (eg eIO_Closed for EOF).
@@ -267,11 +278,11 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Flush
  *  CONN_SetTimeout, CONN_ReadLine
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Read
-(CONN           conn,    /* [in]  connection handle                  */
- void*          buf,     /* [out] memory buffer to read to           */
- size_t         size,    /* [in]  max. # of bytes to read            */
- size_t*        n_read,  /* [out, non-NULL] # of actually read bytes */
- EIO_ReadMethod how      /* [in]  peek/read/persist                  */
+(CONN           conn,    /**< [in]  connection handle                  */
+ void*          buf,     /**< [out] memory buffer to read to           */
+ size_t         size,    /**< [in]  max. # of bytes to read            */
+ size_t*        n_read,  /**< [out] non-NULL, # of actually read bytes */
+ EIO_ReadMethod how      /**< [in]  peek/read/persist                  */
  );
 
 
@@ -282,46 +293,47 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Read
  * If not enough space provided in "line" to accomodate the '\0'-terminated
  * line, then all "size" bytes are used up and "*n_read" is equal to "size"
  * upon return - this is the only case when "line" will not be '\0'-terminated.
+ *
  * Return code advises the caller whether another read can be attempted:
- *   eIO_Success -- read completed successfully, keep reading;
- *   other code  -- an error occurred, and further read attempt may fail.
+ *   * eIO_Success -- read completed successfully, keep reading;
+ *   * other code  -- an error occurred, and further read attempt may fail.
  *
  * This call utilizes eIO_Read timeout as set by CONN_SetTimeout().
  * @sa
  *  CONN_Read, CONN_SetTimeout
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_ReadLine
-(CONN    conn,   /* [in]  connection handle */
- char*   line,   /* [out] buffer to read to */
- size_t  size,   /* [in]  buffer size       */
- size_t* n_read  /* [out] line length       */
+(CONN    conn,   /**< [in]  connection handle */
+ char*   line,   /**< [out] buffer to read to */
+ size_t  size,   /**< [in]  buffer size       */
+ size_t* n_read  /**< [out] line length       */
  );
 
 
 /** Obtain status of the last I/O operation.  This is NOT a completion
  * code of the last CONN call, but rather some status from a lower level
  * connector's layer (if available).
- * Special case:  eIO_Open as "dir" checks whether the connection is in an open
- * state (which means the underlying connector has been successfully opened,
- * but does not assure/check availability for any data or I/O), and returns
- * eIO_Success if it is, or an error code otherwise.
+ * @par Special case:  eIO_Open as "dir" checks whether the connection is in
+ * an open state (which means the underlying connector has been successfully
+ * opened, but does not assure/check availability for any data or I/O), and
+ * returns eIO_Success if it is, or an error code otherwise.
  * @sa
  *  CONN_Create, CONN_Read, CONN_Write, CONN_Flush
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Status
-(CONN      conn,  /* [in] connection handle     */
- EIO_Event dir    /* [in] eIO_Read or eIO_Write */
+(CONN      conn,  /**< [in] connection handle     */
+ EIO_Event dir    /**< [in] eIO_Read or eIO_Write */
  );
 
 
 /** Close the connection and destroy all relevant internal data.
  * @note  Whatever the error code is returned, the connection handle "conn"
- *        will have become invalid (so, it should not be used it anymore).
+ *        will have become invalid (so, it should not be used anymore).
  * @sa
  *  CONN_Create
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Close
-(CONN conn  /* [in] connection handle */
+(CONN conn  /**< [in] connection handle */
  );
 
 
@@ -329,58 +341,92 @@ extern NCBI_XCONNECT_EXPORT EIO_Status CONN_Close
  * callback type.  Note that the callback function is always called prior
  * to the event to happen, e.g. the eCONN_OnClose callback is called when
  * the connection is about to close, but has not yet been closed.
+ * @par
  * The callback function is supplied with 3 arguments: the connection handle,
  * a type of event, and a user data (specified when the callback was set).
  * CONN_SetCallback() stores previous callback in "old_cb" (if it is not NULL).
+ * @par
  * The callbacks remain valid until they are explicitly changed / de-activated
  * or the connection becomes closed.
+ * @par
  * This means that if a callback is intercepted and then relayed to the old
  * handler, the interceptor may not assume the callback remains set, and
  * must re-instate itself upon each upcall of the old handler (in general).
+ * @par
  * Normally, callback would return eIO_Success and let the operation continue;
  * non-eIO_Success return value causes it to be returned to the caller level
  * (but possibly with some processing already completed by then, e.g. such as
  * a partial read for eCONN_OnRead from an internal connection buffer).
- * NOTE:  eIO_Interrupt returned from the callback switches connection into a
+ * @note  eIO_Interrupt returned from the callback switches connection into a
  * cancelled state irreversibly, causing any further I/O for this handle to
  * fail with eIO_Interrupt.
- * NOTE:  non-eIO_Success from an eCONN_OnClose callback cannot postpone the
+ * @note  non-eIO_Success from an eCONN_OnClose callback cannot postpone the
  * connection closure (but the error code is still passed through to the user).
- * NOTE:  eCONN_OnTimeout can restart the I/O that has timed out by returning
+ * @note  eCONN_OnTimeout can restart the I/O that has timed out by returning
  * eIO_Success.
  * @sa
  *  CONN_Read, CONN_Write, CONN_Close
  */
 typedef enum {
-    eCONN_OnClose   = 0,  /** NB: CONN has been flushed prior to the call    */
-    eCONN_OnRead    = 1,  /** Read from connector is about to occur          */
-    eCONN_OnWrite   = 2,  /** Write to connector is about to occur           */
-    eCONN_OnFlush   = 3,  /** Connector is about to be flushed               */
-    eCONN_OnTimeout = 4   /** Connector I/O has timed out                    */
+    eCONN_OnClose   = 0,  /**< NB: CONN has been flushed prior to the call   */
+    eCONN_OnRead    = 1,  /**< Read from connector is about to occur         */
+    eCONN_OnWrite   = 2,  /**< Write to connector is about to occur          */
+    eCONN_OnFlush   = 3,  /**< Connector is about to be flushed              */
+    eCONN_OnTimeout = 4   /**< Connector I/O has timed out                   */
 } ECONN_Callback;
 #define CONN_N_CALLBACKS  5
 
 typedef EIO_Status (*FCONN_Callback)(CONN conn,ECONN_Callback type,void* data);
 
 typedef struct {
-    FCONN_Callback func;  /** Function to call on the event                  */
-    void*          data;  /** Data to pass to the callback as its last arg   */
+    FCONN_Callback func;  /**< function to call on the event                 */
+    void*          data;  /**< data to pass to the callback as its last arg  */
 } SCONN_Callback;
 
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_SetCallback
-(CONN                  conn,    /* [in]  connection to set callback for      */
- ECONN_Callback        type,    /* [in]  callback type                       */
- const SCONN_Callback* new_cb,  /* [in]  callback to set (may be 0 to reset) */
-       SCONN_Callback* old_cb   /* [out] to save old callback at (may be 0)  */
+(CONN                  conn,    /**< [in]  connection to set callback for    */
+ ECONN_Callback        type,    /**< [in]  callback type                     */
+ const SCONN_Callback* new_cb,  /**< [in]  callback to set (NULL to reset)   */
+       SCONN_Callback* old_cb   /**< [out] to save old callback at (may be 0)*/
 );
 
 
+/** Get an underlying SOCK handle for connection that is implemented as a
+ * socket.  Not a eIO_Success return code guarantees "*sock" is NULL.
+ * Set "*sock" to NULL when no socket handle can be obtained.
+ * @note  The returned SOCK object remains in use by the connection.
+ * @sa
+ *  SOCK, SOCK_GetOSHandleEx
+ */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_GetSOCK
-(CONN  conn,
- SOCK* sock
+(CONN  conn,  /**< [in]  connection handle            */
+ SOCK* sock   /**< [out] non-NULL, to get the SOCK to */
  );
     
 
+/** Set connection processing flags.
+ * @return
+ *  eIO_Success on success, other error code on error.
+ * @sa
+ *  CONN_CreateEx, CONN_GetFlags
+ */
+extern NCBI_XCONNECT_EXPORT EIO_Status CONN_SetFlags
+(CONN        conn,  /**< [in]  connection handle    */
+ TCONN_Flags flags  /**< [in]  new connection flags */
+ );
+
+
+/** Get connection processing flags currently in effect.
+ * @return
+ *  Current value of the flags.
+ * @sa
+ *  CONN_CreateEx, CONN_SetFlags
+ */
+extern NCBI_XCONNECT_EXPORT TCONN_Flags CONN_GetFlags
+(CONN conn  /**< [in]  connection handle */
+ );
+
+
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif
