@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/* $Id: ncbi_connutil.h 366365 2012-06-13 20:36:46Z rafanovi $
+/* $Id: ncbi_connutil.h 371018 2012-08-03 18:36:06Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,7 +79,6 @@
 
 #include <connect/ncbi_buffer.h>
 #include <connect/ncbi_connection.h>
-#include <connect/ncbi_socket.h>
 
 
 /* Well-known port values */
@@ -485,14 +484,12 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_SetupStandardArgs
 
 /* Log the contents of "*info" into specified "log" with severity "sev".
  */
-extern NCBI_XCONNECT_EXPORT void ConnNetInfo_LogEx
+extern NCBI_XCONNECT_EXPORT void ConnNetInfo_Log
 (const SConnNetInfo* info,
  ELOG_Level          sev,
  LOG                 log
  );
 
-#define ConnNetInfo_Log(i, l) ConnNetInfo_LogEx((i), eLOG_Trace, (l))
-
 
 /* Reconstruct text URL out of the SConnNetInfo's components
  * (excluding username:password for safety reasons).
