@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_TYPES__H
 #define CONNECT___NCBI_TYPES__H
 
-/* $Id: ncbi_types.h 236260 2011-01-31 18:46:12Z lavr $
+/* $Id: ncbi_types.h 361560 2012-05-02 19:14:36Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -104,18 +104,42 @@ extern NCBI_XCONNECT_EXPORT STimeout*     NcbiMsToTimeout
  * type, please apply equivalent changes to both definitions.
  */
 
-/** Aux. enum to set/unset/default various features
+/** Aux. enum to set/unset/default various features.
  */
-typedef enum {
+typedef enum ENcbiSwitch {
     eOff = 0,
     eOn,
     eDefault
 } ESwitch;
 
-#endif  /* !defined(NCBI_ESWITCH_DEFINED) */
+#endif /*!NCBI_ESWITCH_DEFINED*/
 
 
-/** Fixed size analogs of size_t and time_t (mainly for IPC)
+#ifndef NCBI_EOWNERSHIP_DEFINED
+#define NCBI_EOWNERSHIP_DEFINED
+
+/*
+ * ATTENTION!   Do not change this enumeration!
+ *
+ * It must always be kept in sync with its C++ counterpart defined in
+ * "corelib/ncbimisc.hpp". If you absolutely(sic!) need to alter this
+ * type, please apply equivalent changes to both definitions.
+ */
+
+/** Ownership relations between objects.
+ *
+ * Can be used to define or transfer ownership of objects.
+ * For example, specify if a CSocket object owns its underlying SOCK object.
+ */
+typedef enum ENcbiOwnership {
+    eNoOwnership,       /** No ownership is assumed                 */
+    eTakeOwnership      /** An object can take ownership of another */
+} EOwnership;
+
+#endif /*!NCBI_EOWNERSHIP_DEFINED*/
+
+
+/** Fixed-size analogs of size_t and time_t (mainly for IPC)
  */
 typedef unsigned int TNCBI_Size;
 typedef unsigned int TNCBI_Time;
