@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIARGS__HPP
 #define CORELIB___NCBIARGS__HPP
 
-/*  $Id: ncbiargs.hpp 360817 2012-04-26 15:09:00Z gouriano $
+/*  $Id: ncbiargs.hpp 369471 2012-07-19 17:45:26Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -238,7 +238,7 @@ public:
         fAppend   = (1 <<  2),  ///< Open file in append mode.
         fTruncate = (1 << 12)   ///< Open file in truncate mode.
     };
-    typedef int TFileFlags;   // binary OR of "EFileFlags"
+    typedef unsigned int TFileFlags;   ///< Bitwise OR of "EFileFlags"
 
     /// Get the argument as an input file stream.
     virtual CNcbiIstream& AsInputFile (TFileFlags flags = 0) const = 0;
@@ -519,7 +519,7 @@ public:
     /// Must match the argument type, or an exception will be thrown.
     /// ( File related are for eInputFile and eOutputFiler argument types.)
     enum EFlags {
-        // file related flags:
+        // File related flags:
 
         /// Open file right away for eInputFile, eOutputFile
         fPreOpen = (1 << 0),
@@ -537,7 +537,7 @@ public:
         /// Repeated key arguments are legal (use with AddKey)
         fAllowMultiple = (1 << 3),
 
-        // Error handling flags
+        // Error handling flags:
 
         /// Ignore invalid argument values. If not set, exceptions will be
         /// thrown on invalid values.
@@ -564,7 +564,7 @@ public:
         /// Hide it in Usage
         fHidden = (1 << 10)
     };
-    typedef unsigned int TFlags;  ///< Binary OR of "EFlags"
+    typedef unsigned int TFlags;  ///< Bitwise OR of "EFlags"
 
     /// Add description for mandatory key.
     ///
@@ -584,7 +584,7 @@ public:
                 const string& synopsis,   ///< Synopsis for argument
                 const string& comment,    ///< Argument description
                 EType         type,       ///< Argument type
-                TFlags        flags = 0   ///< Optional file related flags
+                TFlags        flags = 0   ///< Optional flags
                );
 
     /// Add description for optional key without default value.
@@ -605,7 +605,7 @@ public:
                         const string& synopsis, ///< Synopsis for argument
                         const string& comment,  ///< Argument description
                         EType         type,     ///< Argument type
-                        TFlags        flags = 0 ///< Optional file flags
+                        TFlags        flags = 0 ///< Optional flags
                        );
 
     /// Add description for optional key with default value.
@@ -627,7 +627,7 @@ public:
                        const string& comment,       ///< Argument description
                        EType         type,          ///< Argument type
                        const string& default_value, ///< Default value
-                       TFlags        flags = 0,     ///< Optional file flags
+                       TFlags        flags = 0,     ///< Optional flags
     /// Optional name of environment variable that contains default value
                        const string& env_var = kEmptyStr
                       );
@@ -673,7 +673,7 @@ public:
     void AddOpening(const string& name,     ///< Name of argument
                     const string& comment,  ///< Argument description
                     EType         type,     ///< Argument type
-                    TFlags        flags = 0 ///< Optional file flags
+                    TFlags        flags = 0 ///< Optional flags
                     );
 
     /// Add description for mandatory postional argument.
@@ -698,7 +698,7 @@ public:
     void AddPositional(const string& name,     ///< Name of argument
                        const string& comment,  ///< Argument description
                        EType         type,     ///< Argument type
-                       TFlags        flags = 0 ///< Optional file flags
+                       TFlags        flags = 0 ///< Optional flags
                       );
 
     /// Add description for optional positional argument without default
@@ -721,7 +721,7 @@ public:
     void AddOptionalPositional(const string& name,     ///< Name of argument
                                const string& comment,  ///< Argument descr.
                                EType         type,     ///< Argument type
-                               TFlags        flags = 0 ///< Optional file flgs
+                               TFlags        flags = 0 ///< Optional flags
                               );
 
     /// Add description for optional positional argument with default value.
@@ -743,7 +743,7 @@ public:
                               const string& comment,///< Argument description
                               EType         type,   ///< Argument type
                               const string& default_value, ///< Default value
-                              TFlags        flags = 0, ///< Optional file flags
+                              TFlags        flags = 0, ///< Optional flags
     /// Optional name of environment variable that contains default value
                               const string& env_var = kEmptyStr
                              );
@@ -764,7 +764,7 @@ public:
                   unsigned      n_optional,  ///< Number of optional args
                   const string& comment,     ///< Argument description
                   EType         type,        ///< Argument type
-                  TFlags        flags = 0    ///< Optional file flags
+                  TFlags        flags = 0    ///< Optional flags
                  );
 
     /// Add argument alias. The alias can be used in the command line instead
@@ -967,10 +967,6 @@ private:
     /// Helper method for doing pre-processing consistency checks.
     void x_PreCheck(void) const; 
 
-    /// Helper method for checking if auto help requested and throw
-    /// CArgHelpException if help requested.
-    void x_CheckAutoHelp(const string& arg) const;
-
     void x_PrintComment(list<string>&   arr,
                         const CArgDesc& arg,
                         SIZE_TYPE       width) const;
@@ -1014,6 +1010,11 @@ private:
     bool x_IsMultiArg(const string& name) const;
 
 protected:
+
+    /// Helper method for checking if auto help requested and throw
+    /// CArgHelpException if help requested.
+    void x_CheckAutoHelp(const string& arg) const;
+
 // PrintUsage helpers
     class CPrintUsage
     {
@@ -1160,11 +1161,13 @@ class NCBI_XNCBI_EXPORT CCommandArgDescriptions : public CArgDescriptions
 {
 public:
 
-    /// Command presence requirement
-    enum ECommandPresence {
-        eCommandOptional,    ///< Command is not necessary
-        eCommandMandatory    ///< Nonempty command is required
+    enum ECommandArgFlags {
+        eCommandMandatory = 0,      ///< Nonempty command is required
+        eCommandOptional  = 1,      ///< Command is not necessary
+        eNoSortCommands   = (1<<1), ///< On PrintUsage, keep commands unsorted
+        eNoSortGroups     = (1<<2)  ///< On PrintUsage, keep command groups unsorted
     };
+    typedef unsigned int TCommandArgFlags;   ///< Bitwise OR of ECommandArgFlags
 
     /// Constructor.
     ///
@@ -1175,7 +1178,7 @@ public:
     /// If not set the default handler is used.
     CCommandArgDescriptions(bool auto_help = true,
                           CArgErrorHandler* err_handler = 0,
-                          ECommandPresence cmd_req = eCommandMandatory);
+                          TCommandArgFlags cmd_flags = eCommandMandatory);
     /// Destructor.
     virtual ~CCommandArgDescriptions(void);
 
@@ -1213,17 +1216,19 @@ public:
 
 private:
     // not allowed
-    void SetArgsType(EArgSetType args_type) {}
+    void SetArgsType(EArgSetType /*args_type*/) { }
 
-    size_t x_GetCommandGroupIndex(const string& group);
+    bool x_IsCommandMandatory(void) const;
+    size_t x_GetCommandGroupIndex(const string& group) const;
     string x_IdentifyCommand(const string& command) const;
 
     typedef map<string, AutoPtr<CArgDescriptions> > TDescriptions;
-    ECommandPresence m_Cmd_req;
+    TCommandArgFlags m_Cmd_req;
     TDescriptions m_Description;    ///< command to ArgDescriptions
     map<string, size_t > m_Groups;  ///< command to group #
     map<string,string> m_Aliases;   ///< command to alias; one alias only
-    vector<string> m_CmdGroups;     ///< group names, and order
+    list<string> m_Commands;        ///< command names, and order
+    list<string> m_CmdGroups;       ///< group names, and order
     size_t m_CurrentGroup;          ///< current group #
     mutable string m_Command;       ///< current command
 };
