@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 361567 2012-05-02 19:34:29Z lavr $
+/*  $Id: ncbimisc.hpp 371900 2012-08-13 18:07:13Z vakatov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -944,6 +944,12 @@ const TSeqPos kInvalidSeqPos = ((TSeqPos) (-1));
 /// Use this typedef rather than its expansion, which may change.
 typedef int TSignedSeqPos;
 
+/// Type for sequence GI.
+///
+/// Use this typedef rather than its expansion, which may change.
+typedef int TGi;
+
+
 /// Helper address class
 class CRawPointer
 {
