@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIREG__HPP
 #define CORELIB___NCBIREG__HPP
 
-/*  $Id: ncbireg.hpp 337189 2011-09-09 13:04:25Z lavr $
+/*  $Id: ncbireg.hpp 377094 2012-10-09 15:00:23Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -347,12 +347,15 @@ public:
     ///   the new one. If "fTransient" flag is set, then store the newly
     ///   retrieved parameters as transient;  otherwise, store them as
     ///   persistent.
+    /// @param path
+    ///   Where to look for base registries listed with relative paths.
     /// @return
     ///   A pointer to a newly created subregistry, if any, directly
     ///   containing the entries loaded from is.
     /// @sa
     ///   Write()
-    IRWRegistry* Read(CNcbiIstream& is, TFlags flags = 0);
+    IRWRegistry* Read(CNcbiIstream& is, TFlags flags = 0,
+                      const string& path = kEmptyStr);
 
     /// Set the configuration parameter value.
     ///
@@ -425,7 +428,8 @@ protected:
 
     /// Most implementations should not override this, but
     /// CNcbiRegistry must, to handle some special cases properly.
-    virtual IRWRegistry* x_Read(CNcbiIstream& is, TFlags flags);
+    virtual IRWRegistry* x_Read(CNcbiIstream& is, TFlags flags,
+                                const string& path);
 
     // for use by implementations
     static bool MaybeSet(string& target, const string& value, TFlags flags);
@@ -697,8 +701,15 @@ public:
 
     /// Load any base registries listed in [NCBI].Inherits; returns
     /// true if able to load at least one, false otherwise.
+    /// @param flags
+    ///   Registry flags to apply.
+    /// @param metareg_flags
+    ///   Metaregistry flags to apply.
+    /// @param path
+    ///   Where to look for base registries listed with relative paths.
     bool LoadBaseRegistries(TFlags flags = 0,
-                            int /* CMetaRegistry::TFlags */ metareg_flags = 0);
+                            int /* CMetaRegistry::TFlags */ metareg_flags = 0,
+                            const string& path = kEmptyStr);
 
     /// Predefined subregistry's name.
     static const char* sm_MainRegName;
@@ -725,7 +736,7 @@ protected:
                const string& comment);
     bool x_SetComment(const string& comment, const string& section,
                       const string& name, TFlags flags);
-    IRWRegistry* x_Read(CNcbiIstream& is, TFlags flags);
+    IRWRegistry* x_Read(CNcbiIstream& is, TFlags flags, const string& path);
 
     /// Add an internal high-priority subregistry.
     void x_Add(const IRegistry& reg,
@@ -780,9 +791,12 @@ public:
     ///   How parameters are stored. The default is to store all parameters as
     ///   persistent unless the  "eTransient" flag is set in which case the
     ///   newly retrieved parameters are stored as transient.
+    /// @param path
+    ///   Where to look for base registries listed with relative paths.
     /// @sa
     ///   Read()
-    CNcbiRegistry(CNcbiIstream& is, TFlags flags = 0);
+    CNcbiRegistry(CNcbiIstream& is, TFlags flags = 0,
+                  const string& path = kEmptyStr);
 
     ~CNcbiRegistry();
 
@@ -811,7 +825,7 @@ public:
 
 protected:
     void x_Clear(TFlags flags);
-    IRWRegistry* x_Read(CNcbiIstream& is, TFlags flags);
+    IRWRegistry* x_Read(CNcbiIstream& is, TFlags flags, const string& path);
 
 private:
     void x_Init(void);
