@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBISTL__HPP
 #define CORELIB___NCBISTL__HPP
 
-/*  $Id: ncbistl.hpp 337189 2011-09-09 13:04:25Z lavr $
+/*  $Id: ncbistl.hpp 355319 2012-03-05 16:52:10Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -172,6 +172,9 @@ namespace NCBI_NS_NCBI { /* the fake one */ }
 #endif
 
 #if defined(NCBI_COMPILER_GCC) || defined(NCBI_COMPILER_WORKSHOP)
+#  if defined(NCBI_COMPILER_GCC)  &&  NCBI_COMPILER_VERSION >= 400
+#    include <algorithm>
+#  endif
 // This template is used by some stl algorithms (sort, reverse...)
 // We need to have our own implementation because some C++ Compiler vendors 
 // implemented it by using a temporary variable and an assignment operator 
@@ -189,6 +192,11 @@ void iter_swap( Iter it1, Iter it2 )
 
 END_STD_SCOPE
 
+#if defined(_GLIBCXX_DEBUG)
+/* STL iterators are non-POD types */
+# define NCBI_NON_POD_TYPE_STL_ITERATORS  1
+#endif
+
 #endif
 
 /* @} */
