@@ -1,7 +1,7 @@
 #ifndef CORELIB___PLUGIN_MANAGER__HPP
 #define CORELIB___PLUGIN_MANAGER__HPP
 
-/* $Id: plugin_manager.hpp 193147 2010-06-01 18:08:33Z ucko $
+/* $Id: plugin_manager.hpp 355803 2012-03-08 16:20:29Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -735,7 +735,9 @@ bool operator==(const SDriverInfo& i1, const SDriverInfo& i2)
 inline
 bool operator<(const SDriverInfo& i1, const SDriverInfo& i2)
 {
-    return i1.name == i2.name && i1.version < i2.version;
+    return (i1.name < i2.name ||
+            (i1.name == i2.name &&
+             (i1.version < i2.version)));
 }
 
 
@@ -920,12 +922,12 @@ bool CPluginManager<TClass>::WillExtendCapabilities
 
         typename TClassFactory::TDriverList drv_list;
         cf->GetDriverVersions(drv_list);
+        drv_list.sort();
         full_drv_list.merge(drv_list);
+        // Remove duplicates.
+        full_drv_list.unique();
     }
 
-    // Remove duplicates.
-    full_drv_list.unique();
-
     ITERATE(typename TClassFactory::TDriverList, it2, full_drv_list) {
         const typename TClassFactory::SDriverInfo& drv_info = *it2;
 
@@ -1200,7 +1202,7 @@ void CPluginManager<TClass>::ResolveFile(const string&       driver,
 
             if (epoint.entry_point.func) {
                 FNCBI_EntryPoint ep =
-                   (FNCBI_EntryPoint)epoint.entry_point.func;
+                   (FNCBI_EntryPoint)(void*)epoint.entry_point.func;
 
                 if ( RegisterWithEntryPoint(ep, driver, version) ) {
                     m_RegisteredEntries.push_back(entry);
