@@ -1,7 +1,7 @@
 #ifndef CORELIB___STREAM_UTILS__HPP
 #define CORELIB___STREAM_UTILS__HPP
 
-/* $Id: stream_utils.hpp 329065 2011-08-07 03:49:08Z lavr $
+/* $Id: stream_utils.hpp 356161 2012-03-12 18:43:50Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,8 +36,9 @@
  *
  *   Reader-writer utilities:
  *   1. Construct an IReader object from an istream;
- *   2. Append entire IReader's contents into a string;
- *   3. Construct an IReader object from a string.
+ *   2. Construct an IWriter object from an ostream;
+ *   3. Construct an IReader object from a string;
+ *   4. Append entire IReader's contents to a string.
  *
  */
 
@@ -87,7 +88,7 @@ struct NCBI_XNCBI_EXPORT CStreamUtils
                                CT_CHAR_TYPE*       buf,
                                streamsize          buf_size,
                                void*               del_ptr)
-    { x_Pushback(is, buf, buf_size, del_ptr, ePushback_NoCopy); }
+    { x_Pushback(is, buf, (size_t) buf_size, del_ptr, ePushback_NoCopy); }
 
 // Acts just like its counterpart with 4 args (above), but this variant
 // always copies (if necessary) the "pushback data" into an internal buffer,
@@ -95,7 +96,7 @@ struct NCBI_XNCBI_EXPORT CStreamUtils
     static void       Pushback(CNcbiIstream&       is,
                                const CT_CHAR_TYPE* buf,
                                streamsize          buf_size)
-    { x_Pushback(is, const_cast<CT_CHAR_TYPE*> (buf), buf_size); }
+    { x_Pushback(is, const_cast<CT_CHAR_TYPE*> (buf), (size_t) buf_size); }
 
 // Unsafe but fast API that tries to backup "buf_size" bytes in the
 // internal stream buffer,but if that is not possible, will use
@@ -106,7 +107,7 @@ struct NCBI_XNCBI_EXPORT CStreamUtils
                                CT_CHAR_TYPE*       buf,
                                streamsize          buf_size,
                                void*               del_ptr = 0)
-    { x_Pushback(is, buf, buf_size, del_ptr, ePushback_Stepback); }
+    { x_Pushback(is, buf, (size_t) buf_size, del_ptr, ePushback_Stepback); }
 
 // Read at most "buf_size" bytes from the stream "is" into a buffer pointed
 // to by "buf".  This call tries its best to be non-blocking.
@@ -124,15 +125,22 @@ private:
         ePushback_Stepback
     } EPushback_How;
 
-    static void     x_Pushback(CNcbiIstream&       is,
-                               CT_CHAR_TYPE*       buf,
-                               streamsize          buf_size,
-                               void*               del_ptr = 0,
-                               EPushback_How       how = ePushback_Copy);
+    static void x_Pushback(CNcbiIstream&       is,
+                           CT_CHAR_TYPE*       buf,
+                           streamsize          buf_size,
+                           void*               del_ptr = 0,
+                           EPushback_How       how = ePushback_Copy);
 };
 
 
 /// istream-based IReader
+///
+/// Besides other uses, this class can help unite two uni-derectional streams
+/// into a single bi-directional one, e.g.:
+///     CRWStream rw(new CStreamReader(cin), new CStreamWriter(cout),
+///                  0, 0, CRWStreambuf::fOwnAll);
+/// @sa
+///   CStreamWriter
 class NCBI_XNCBI_EXPORT CStreamReader : public IReader
 {
 public:
@@ -143,18 +151,45 @@ public:
     virtual ERW_Result Read(void* buf, size_t count, size_t* bytes_read);
     virtual ERW_Result PendingCount(size_t* count);
 
-private:
+protected:
     AutoPtr<CNcbiIstream> m_Stream;
 
-private: // prevent copy
+private:
+    // prevent copy
     CStreamReader(const CStreamReader&);
     void operator=(const CStreamReader&);
 };
 
 
-/// Appends to the given string.
-void NCBI_XNCBI_EXPORT ExtractReaderContents(IReader& reader, string& s);
+/// ostream-based IWriter
+///
+/// Can help create single bi-directional stream from two uni-derectional ones:
+///     CRWStream rw(new CStreamReader(cin), new CStreamWriter(cout),
+///                  0, 0, CRWStreambuf::fOwnAll);
+/// @sa
+///   CStreamReader
+class NCBI_XNCBI_EXPORT CStreamWriter : public IWriter
+{
+public:
+    CStreamWriter(CNcbiOstream& os, EOwnership own = eNoOwnership)
+        : m_Stream(&os, own)
+    { }
 
+    virtual ERW_Result Write(const void* buf,
+                             size_t count, size_t* bytes_written);
+    virtual ERW_Result Flush(void);
+
+protected:
+    AutoPtr<CNcbiOstream> m_Stream;
+
+private:
+    // prevent copy
+    CStreamWriter(const CStreamWriter&);
+    void operator=(const CStreamWriter&);
+};
+
+
+/// string-based IReader
 class NCBI_XNCBI_EXPORT CStringReader : public IReader
 {
 public:
@@ -171,6 +206,10 @@ private:
 };
 
 
+/// Append all IReader contents to a given string.
+void NCBI_XNCBI_EXPORT g_ExtractReaderContents(IReader& reader, string& s);
+
+
 END_NCBI_SCOPE
 
 
