@@ -1,7 +1,7 @@
 #ifndef DBAPI___DBAPI__HPP
 #define DBAPI___DBAPI__HPP
 
-/* $Id: dbapi.hpp 154862 2009-03-16 18:17:49Z ivanovp $
+/* $Id: dbapi.hpp 334257 2011-09-02 19:10:14Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -855,6 +855,9 @@ public:
     ///        DB API), then set the timeout to infinite.
     virtual void SetTimeout(size_t nof_secs) = 0;
 
+    /// Set timeout for command cancellation and connection closing
+    virtual void SetCancelTimeout(size_t nof_secs) {}
+
     /// If enabled, redirects all error messages
     /// to CDB_MultiEx object (see below).
     virtual void MsgToEx(bool v) = 0;
