@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___DBAPI_DRIVER_CONVERT__HPP
 #define DBAPI_DRIVER___DBAPI_DRIVER_CONVERT__HPP
 
-/* $Id: dbapi_driver_convert.hpp 213628 2010-11-24 17:16:13Z gouriano $
+/* $Id: dbapi_driver_convert.hpp 373396 2012-08-29 14:57:13Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NTOICE
@@ -985,10 +985,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        throw CInvalidConversionException();
     }
 
 private:
@@ -1039,10 +1040,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        throw CInvalidConversionException();
     }
 
 private:
@@ -1094,10 +1096,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        throw CInvalidConversionException();
     }
 
 private:
@@ -1148,10 +1151,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        throw CInvalidConversionException();
     }
 
 private:
