@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___EXCEPTION__HPP
 #define DBAPI_DRIVER___EXCEPTION__HPP
 
-/* $Id: exception.hpp 343769 2011-11-09 16:51:52Z ivanovp $
+/* $Id: exception.hpp 355800 2012-03-08 16:15:06Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -468,6 +468,14 @@ public:
     /// @sa HandleAll(), CException::Throw()
     virtual bool HandleIt(CDB_Exception* ex) = 0;
 
+    /// Handle message resulting from a native API call.
+    /// Method MUST NOT throw any exceptions.
+    /// 
+    /// @return
+    ///   TRUE if message is processed and shouldn't be saved for later
+    ///   appearance as CDB_Exception, FALSE otherwise (default value is FALSE)
+    virtual bool HandleMessage(int severity, int msgnum, const string& message);
+
     // Get current global "last-resort" error handler.
     // If not set, then the default will be "CDB_UserHandler_Default".
     // This handler is guaranteed to be valid up to the program termination,
