@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___PUBLIC__HPP
 #define DBAPI_DRIVER___PUBLIC__HPP
 
-/* $Id: public.hpp 201469 2010-08-18 17:34:13Z ivanovp $
+/* $Id: public.hpp 333164 2011-09-02 16:04:31Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -104,6 +104,9 @@ public:
     ///   Maximal data size.
     /// @param log_it 
     ///   Log LOB operation if this value is set to true.
+    /// @param discard_results
+    ///   Discard all resultsets that might be returned from server
+    ///   if this value is set to true.
     /// 
     /// @return 
     ///   Newly created send-data object.
@@ -111,7 +114,8 @@ public:
     /// @sa SendData
     virtual CDB_SendDataCmd* SendDataCmd(I_ITDescriptor& desc,
                                          size_t          data_size,
-                                         bool            log_it = true);
+                                         bool            log_it = true,
+                                         bool            discard_results = true);
 
     /// @brief 
     ///   Shortcut to send text and image to the server without using the
@@ -288,6 +292,9 @@ public:
     ///   (depends on the underlying DB API), then set the timeout to infinite.
     virtual void SetTimeout(size_t nof_secs);
 
+    /// Set timeout for command cancellation and connection closing
+    virtual void SetCancelTimeout(size_t nof_secs);
+
     /// Get interface for extra features that could be implemented in the driver.
     virtual I_ConnectionExtra& GetExtraFeatures(void);
 
@@ -802,8 +809,25 @@ public:
     virtual bool Update(const string& table_name, const string& upd_query);
     virtual bool UpdateTextImage(unsigned int item_num, CDB_Stream& data,
                                  bool log_it = true);
+    /// Make "send-data" command
+    /// @brief 
+    ///   Create send-data command.
+    /// 
+    /// @param item_num 
+    ///   Column number to rewrite.
+    /// @param size 
+    ///   Maximal data size.
+    /// @param log_it 
+    ///   Log LOB operation if this value is set to true.
+    /// @param discard_results
+    ///   Discard all resultsets that might be returned from server
+    ///   if this value is set to true.
+    /// 
+    /// @return 
+    ///   Newly created send-data object.
     virtual CDB_SendDataCmd* SendDataCmd(unsigned int item_num, size_t size,
-                                         bool log_it = true);
+                                         bool log_it = true,
+                                         bool discard_results = true);
 
     /// Delete the last fetched row.
     /// NOTE: the cursor must be declared for delete in CDB_Connection::Cursor()
@@ -851,6 +875,15 @@ public:
     virtual size_t SendChunk(const void* data, size_t size);
     virtual bool Cancel(void);
 
+    /// Get result set
+    virtual CDB_Result* Result();
+    virtual bool HasMoreResults() const;
+
+    /// Dump the results of the command
+    /// If result processor is installed for this connection, then it will be
+    /// called for each result set
+    virtual void DumpResults();
+
     // Destructor
     virtual ~CDB_SendDataCmd();
 
