@@ -1,7 +1,7 @@
 #ifndef DBAPI___VARIANT__HPP
 #define DBAPI___VARIANT__HPP
 
-/* $Id: variant.hpp 103491 2007-05-04 17:18:18Z kazimird $
+/* $Id: variant.hpp 356104 2012-03-12 14:57:08Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include <corelib/ncbitype.h>
 #include <corelib/ncbitime.hpp>
 #include <dbapi/driver/types.hpp>
+#include <dbapi/driver/interfaces.hpp>
 
 
 /** @addtogroup DbVariant
@@ -196,6 +197,10 @@ public:
     // Moves the internal position pointer
     bool MoveTo(size_t pos) const;
 
+    void SetITDescriptor(I_ITDescriptor* descr);
+    I_ITDescriptor& GetITDescriptor(void) const;
+    I_ITDescriptor* ReleaseITDescriptor(void) const;
+
 protected:
     // Set methods
     void SetData(CDB_Object* o);
@@ -209,6 +214,7 @@ private:
     void x_Inapplicable_Method(const char* method) const;
 
     class CDB_Object* m_data;
+    mutable auto_ptr<I_ITDescriptor> m_descr;
 };
 
 bool NCBI_DBAPI_EXPORT operator==(const CVariant& v1, const CVariant& v2);
@@ -242,6 +248,24 @@ EDB_Type CVariant::GetType() const
 //}
 
 
+inline void
+CVariant::SetITDescriptor(I_ITDescriptor* descr)
+{
+    m_descr.reset(descr);
+}
+
+inline I_ITDescriptor&
+CVariant::GetITDescriptor(void) const
+{
+    return *m_descr;
+}
+
+inline I_ITDescriptor*
+CVariant::ReleaseITDescriptor(void) const
+{
+    return m_descr.release();
+}
+
 inline
 void CVariant::x_Verify_AssignType(EDB_Type db_type, const char* cxx_type) const
 {
