@@ -1,7 +1,7 @@
 #ifndef ANNOT_SELECTOR__HPP
 #define ANNOT_SELECTOR__HPP
 
-/*  $Id: annot_selector.hpp 208693 2010-10-20 13:48:46Z mcelhany $
+/*  $Id: annot_selector.hpp 330204 2011-08-10 18:15:22Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 #include <objmgr/annot_name.hpp>
 #include <objmgr/annot_type_selector.hpp>
 #include <objmgr/tse_handle.hpp>
+#include <objmgr/bioseq_handle.hpp>
 
 #include <bitset>
 #include <vector>
@@ -550,7 +551,7 @@ struct NCBI_XOBJMGR_EXPORT SAnnotSelector : public SAnnotTypeSelector
     ///   ExcludeNamedAnnots()
     ///   SetAllNamedAnnots()
     typedef vector<CAnnotName> TAnnotsNames;
-    typedef set<string> TNamedAnnotAccessions;
+    typedef map<string, int> TNamedAnnotAccessions;
     /// Select annotations from all Seq-annots
     SAnnotSelector& ResetAnnotsNames(void);
     /// Reset special processing of unnamed annots (added or excluded)
@@ -606,7 +607,8 @@ struct NCBI_XOBJMGR_EXPORT SAnnotSelector : public SAnnotTypeSelector
     bool ExcludedAnnotName(const CAnnotName& name) const;
 
     /// Add named annot accession (NA*) in the search.
-    SAnnotSelector& IncludeNamedAnnotAccession(const string& acc);
+    SAnnotSelector& IncludeNamedAnnotAccession(const string& acc,
+                                               int zoom_level = 0);
     /// 
     const TNamedAnnotAccessions& GetNamedAnnotAccessions(void) const
         {
@@ -677,6 +679,19 @@ struct NCBI_XOBJMGR_EXPORT SAnnotSelector : public SAnnotTypeSelector
     /// Reset filter for source location of annotations
     SAnnotSelector& ResetSourceLoc(void);
 
+    /// Set handle used for determining what locations are "near".
+    /// You can set it to an "unset" or "blank" CBioseq_Handle to effectively unset this.
+    SAnnotSelector& SetIgnoreFarLocationsForSorting( const CBioseq_Handle &handle )
+    {
+        m_IgnoreFarLocationsForSorting = handle;
+        return *this;
+    }
+
+    const CBioseq_Handle &GetIgnoreFarLocationsForSorting( void ) const
+    {
+        return m_IgnoreFarLocationsForSorting;
+    }
+
 protected:
     friend class CAnnot_Collector;
 
@@ -721,9 +736,38 @@ protected:
     TTSE_Limits           m_ExcludedTSE;
     TAnnotTypesBitset     m_AnnotTypesBitset;
     AutoPtr<CHandleRangeMap> m_SourceLoc;
+    CBioseq_Handle        m_IgnoreFarLocationsForSorting;
 };
 
 
+/// Named annotations zoom level can be encoded in the accession string
+/// with @@ suffix, for example: NA000000001.1@@1000
+/// zoom level is the number of bases covered by single value in a annotation
+/// density graph.
+
+#define NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX "@@"
+
+
+/// Extract optional zoom level suffix from named annotation string.
+/// returns true if zoom level explicitly defined in the full_name argument.
+/// The accession string without zoom level will be written
+/// by acc_ptr pointer if it's not null.
+/// Zoom level will be written by zoom_level_ptr pointer if it's not null.
+/// Absent zoom level will be represented by value 0.
+/// Wildcard zoom level will be represented by value -1.
+NCBI_XOBJMGR_EXPORT
+bool ExtractZoomLevel(const string& full_name,
+                      string* acc_ptr, int* zoom_level_ptr);
+
+/// Combine accession string and zoom level into a string with separator.
+/// If the argument string already contains zoom level verify it's the same
+/// as the zoom_level argument.
+/// Zoom level value of -1 can be used to add wildcard @@*.
+NCBI_XOBJMGR_EXPORT
+string CombineWithZoomLevel(const string& acc, int zoom_level);
+NCBI_XOBJMGR_EXPORT
+void AddZoomLevel(string& acc, int zoom_level);
+
 /* @} */
 
 
