@@ -1,7 +1,7 @@
 #ifndef SEQ_ANNOT_INFO__HPP
 #define SEQ_ANNOT_INFO__HPP
 
-/*  $Id: seq_annot_info.hpp 267328 2011-03-25 14:30:31Z vasilche $
+/*  $Id: seq_annot_info.hpp 382535 2012-12-06 19:21:37Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -143,6 +143,15 @@ public:
 
     void Update(TAnnotIndex index);
 
+    void AddFeatId(TAnnotIndex index,
+                   const CObject_id& id,
+                   EFeatIdType id_type);
+    void RemoveFeatId(TAnnotIndex index,
+                      const CObject_id& id,
+                      EFeatIdType id_type);
+    void ClearFeatIds(TAnnotIndex index,
+                      EFeatIdType id_type);
+
     const CAnnotObject_Info& GetInfo(TAnnotIndex index) const;
 
     const CSeqTableInfo& GetTableInfo(void) const;
@@ -161,6 +170,8 @@ public:
                                 const CAnnotObject_Info& feat_info) const;
     bool IsTableFeatPartial(const CAnnotObject_Info& feat_info) const;
 
+    virtual string GetDescription(void) const;
+
 protected:
     friend class CDataSource;
     friend class CTSE_Info;
@@ -201,8 +212,12 @@ protected:
 
     void x_MapFeatIds(CAnnotObject_Info& info);
     void x_UnmapFeatIds(CAnnotObject_Info& info);
-    void x_MapFeatById(const CFeat_id& id, CAnnotObject_Info& info, bool xref);
-    void x_UnmapFeatById(const CFeat_id& id, CAnnotObject_Info& info, bool xref);
+    void x_MapFeatById(const CFeat_id& id,
+                       CAnnotObject_Info& info,
+                       EFeatIdType id_type);
+    void x_UnmapFeatById(const CFeat_id& id,
+                         CAnnotObject_Info& info,
+                         EFeatIdType id_type);
     void x_MapFeatByGene(const CGene_ref& gene, CAnnotObject_Info& info);
     void x_UnmapFeatByGene(const CGene_ref& gene, CAnnotObject_Info& info);
 
