@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 #define OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 
-/*  $Id: tse_info.hpp 369613 2012-07-20 18:37:00Z rafanovi $
+/*  $Id: tse_info.hpp 387293 2013-01-28 20:43:07Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -168,6 +168,13 @@ private:
 
 class CTSEAnnotObjectMapper;
 
+
+class CBioseqUpdater : public CObject {
+public:
+    virtual void Update(CBioseq_Info& seq) = 0;
+};
+
+
 class NCBI_XOBJMGR_EXPORT CTSE_Info : public CSeq_entry_Info
 {
     typedef CSeq_entry_Info TParent;
@@ -277,18 +284,18 @@ public:
 
     struct SFeatIdInfo
     {
-        SFeatIdInfo(bool xref, CAnnotObject_Info* info)
-            : m_Xref(xref), m_IsChunk(false)
+        SFeatIdInfo(EFeatIdType type, CAnnotObject_Info* info)
+            : m_Type(type), m_IsChunk(false)
             {
                 m_Info = info;
             }
-        SFeatIdInfo(bool xref, TChunkId chunk_id)
-            : m_Xref(xref), m_IsChunk(true)
+        SFeatIdInfo(EFeatIdType type, TChunkId chunk_id)
+            : m_Type(type), m_IsChunk(true)
             {
                 m_ChunkId = chunk_id;
             }
 
-        bool               m_Xref;
+        Int1               m_Type;
         bool               m_IsChunk;
         union {
             CAnnotObject_Info* m_Info;
@@ -319,7 +326,8 @@ public:
     CConstRef<CBioseq_Info> FindBioseq(const CSeq_id_Handle& id) const;
 
     // find bioseq with matching id
-    bool ContainsMatchingBioseq(const CSeq_id_Handle& id) const;
+    // returns matching Seq-id handle
+    CSeq_id_Handle ContainsMatchingBioseq(const CSeq_id_Handle& id) const;
     CConstRef<CBioseq_Info> FindMatchingBioseq(const CSeq_id_Handle& id) const;
     SSeqMatch_TSE GetSeqMatch(const CSeq_id_Handle& id) const;
 
@@ -332,12 +340,19 @@ public:
     // fill ids with all Annot Seq-ids from this TSE
     // the result will be sorted and contain no duplicates
     virtual void GetAnnotIds(TSeqIds& ids) const;
+
+    // patch loaded CBioseq objects now, and split CBioseq objects when loaded
+    void SetBioseqUpdater(CRef<CBioseqUpdater> updater);
     
     void UpdateAnnotIndex(const CSeq_id_Handle& id) const;
     void UpdateAnnotIndex(void) const;
     void UpdateAnnotIndex(const CTSE_Info_Object& object) const;
     void UpdateAnnotIndex(void);
     void UpdateAnnotIndex(CTSE_Info_Object& object);
+    void UpdateFeatIdIndex(CSeqFeatData::E_Choice type,
+                           EFeatIdType id_type) const;
+    void UpdateFeatIdIndex(CSeqFeatData::ESubtype subtype,
+                           EFeatIdType id_type) const;
 
     void x_UpdateAnnotIndexContents(CTSE_Info& tse);
 
@@ -379,43 +394,43 @@ public:
     void x_AddFeaturesById(TAnnotObjects& objects,
                            const SFeatIdIndex& index,
                            TFeatIdInt id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            CSeqFeatData::ESubtype subtype,
                            TFeatIdInt id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddAllFeaturesById(TAnnotObjects& objects,
                               TFeatIdInt id,
-                              bool xref) const;
+                              EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::E_Choice type,
                                     TFeatIdInt id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                                     TFeatIdInt id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            const SFeatIdIndex& index,
                            const TFeatIdStr& id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            CSeqFeatData::ESubtype subtype,
                            const TFeatIdStr& id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddAllFeaturesById(TAnnotObjects& objects,
                               const TFeatIdStr& id,
-                              bool xref) const;
+                              EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::E_Choice type,
                                     const TFeatIdStr& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                                     const TFeatIdStr& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::E_Choice type,
                                     const TFeatId& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                                     const TFeatId& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesByLocus(const string& locus, bool tag) const;
 
     typedef pair<CConstRef<CSeq_annot_Info>, CTSE_Lock> TSeq_annot_Lock;
@@ -424,6 +439,8 @@ public:
                                   TSeqPos loc_pos,
                                   const CSeq_feat& feat) const;
 
+    virtual string GetDescription(void) const;
+
 private:
     friend class CTSE_Guard;
     friend class CDataSource;
@@ -516,22 +533,22 @@ private:
 
     void x_MapFeatById(TFeatIdInt id,
                        CAnnotObject_Info& info,
-                       bool xref);
+                       EFeatIdType type);
     void x_UnmapFeatById(TFeatIdInt id,
                          CAnnotObject_Info& info,
-                         bool xref);
+                         EFeatIdType type);
     void x_MapFeatById(const TFeatIdStr& id,
                        CAnnotObject_Info& info,
-                       bool xref);
+                       EFeatIdType type);
     void x_UnmapFeatById(const TFeatIdStr& id,
                          CAnnotObject_Info& info,
-                         bool xref);
+                         EFeatIdType type);
     void x_MapFeatById(const TFeatId& id,
                        CAnnotObject_Info& info,
-                       bool xref);
+                       EFeatIdType type);
     void x_UnmapFeatById(const TFeatId& id,
                          CAnnotObject_Info& info,
-                         bool xref);
+                         EFeatIdType type);
     void x_MapFeatByLocus(const string& locus, bool tag,
                           CAnnotObject_Info& info);
     void x_UnmapFeatByLocus(const string& locus, bool tag,
@@ -544,42 +561,42 @@ private:
     void x_MapChunkByFeatType(const SAnnotTypeSelector& type,
                               TChunkId chunk_id);
 
-    void x_MapChunkByFeatId(TFeatIdInt feat_id,
+    void x_MapChunkByFeatId(TFeatIdInt id,
                             CSeqFeatData::ESubtype subtype,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(TFeatIdInt feat_id,
+                            EFeatIdType type);
+    void x_MapChunkByFeatId(TFeatIdInt id,
                             CSeqFeatData::E_Choice type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(TFeatIdInt feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(TFeatIdInt id,
                             const SAnnotTypeSelector& type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatIdStr& id,
                             CSeqFeatData::ESubtype subtype,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatIdStr& id,
                             CSeqFeatData::E_Choice type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatIdStr& id,
                             const SAnnotTypeSelector& type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatId& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatId& id,
                             CSeqFeatData::ESubtype subtype,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatId& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatId& id,
                             CSeqFeatData::E_Choice type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatId& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatId& id,
                             const SAnnotTypeSelector& type,
                             TChunkId chunk_id,
-                            bool xref);
+                            EFeatIdType id_type);
 
     friend class CTSEAnnotObjectMapper;
 
@@ -698,6 +715,8 @@ private:
     mutable bool m_MasterSeqSegmentsLoaded;
     mutable CConstRef<CMasterSeqSegments> m_MasterSeqSegments;
 
+    CRef<CBioseqUpdater> m_BioseqUpdater;
+
 private:
     // Hide copy methods
     CTSE_Info(const CTSE_Info&);
