@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___TSE_SPLIT_INFO__HPP
 #define OBJECTS_OBJMGR_IMPL___TSE_SPLIT_INFO__HPP
 
-/*  $Id: tse_split_info.hpp 255785 2011-02-28 13:41:06Z vasilche $
+/*  $Id: tse_split_info.hpp 390318 2013-02-26 21:04:57Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -56,6 +56,7 @@ class CDataLoader;
 class CTSE_SetObjectInfo;
 
 class ITSE_Assigner;
+class CBioseqUpdater;
 
 struct SAnnotObjectsIndex;
 
@@ -126,6 +127,10 @@ public:
 
     void x_UpdateAnnotIndex(void);
     void x_UpdateAnnotIndex(CTSE_Chunk_Info& chunk);
+    void x_UpdateFeatIdIndex(CSeqFeatData::E_Choice type,
+                             EFeatIdType id_type);
+    void x_UpdateFeatIdIndex(CSeqFeatData::ESubtype subtype,
+                             EFeatIdType id_type);
     
     // append ids with all Bioseqs Seq-ids from this Split-Info
     void GetBioseqsIds(TSeqIds& ids) const;
@@ -148,6 +153,7 @@ public:
                         const TAssembly& assembly);
     void x_LoadSeq_entry(CSeq_entry& entry, CTSE_SetObjectInfo* set_info = 0);
 
+    void x_SetBioseqUpdater(CRef<CBioseqUpdater> updater);
 
 protected:
     TSeqIdToChunks::const_iterator x_FindChunk(const CSeq_id_Handle& id) const;
