@@ -1,7 +1,7 @@
 #ifndef OBJMGR_SCOPE__HPP
 #define OBJMGR_SCOPE__HPP
 
-/*  $Id: scope.hpp 366487 2012-06-14 16:53:31Z rafanovi $
+/*  $Id: scope.hpp 373109 2012-08-24 20:48:13Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -105,9 +105,16 @@ public:
     /// Get bioseq handle by seq-id handle
     CBioseq_Handle GetBioseqHandle(const CSeq_id_Handle& id);
 
-    /// Get bioseq handle by seq-id location
-    /// @deprecated  This method is hard to use correctly.
-    NCBI_DEPRECATED CBioseq_Handle GetBioseqHandle(const CSeq_loc& loc);
+    /// Get bioseq handle by seq-loc.
+    /// If the seq-loc contains no seq-ids or there's a single seq-id which
+    /// can not be resolved, returns empty handle.
+    /// If the seq-loc references only parts of a segmented set, the method
+    /// returns bioseq handle for the master bioseq.
+    /// If the seq-loc contains a single seq-id, the bioseq handle for this
+    /// id is returned.
+    /// If there are multiple seq-ids not belonging to the same seg-set,
+    /// the method throws CObjMgrException.
+    CBioseq_Handle GetBioseqHandle(const CSeq_loc& loc);
 
     enum EGetBioseqFlag {
         eGetBioseq_Resolved, ///< Search only in already resolved ids
