@@ -1,7 +1,7 @@
 #ifndef SEQ_LOC_MAPPER__HPP
 #define SEQ_LOC_MAPPER__HPP
 
-/*  $Id: seq_loc_mapper.hpp 360367 2012-04-23 14:02:03Z grichenk $
+/*  $Id: seq_loc_mapper.hpp 386189 2013-01-16 19:44:37Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -272,7 +272,8 @@ private:
     void x_InitGCSequence(const CGC_Sequence& gc_seq,
                           ESeqMapDirection    direction,
                           SSeqMapSelector     selector,
-                          const CGC_Sequence* parent_seq);
+                          const CGC_Sequence* parent_seq,
+                          CRef<CSeq_id>       override_id);
 
     // Creates and adds to the scope a virtual bioseq. If delta is not null,
     // the bioseq uses it for the instance. Otherwise a virtual bioseq without
