@@ -1,7 +1,7 @@
 #ifndef OBJMGR_TSE_HANDLE__HPP
 #define OBJMGR_TSE_HANDLE__HPP
 
-/*  $Id: tse_handle.hpp 183267 2010-02-16 17:24:39Z vasilche $
+/*  $Id: tse_handle.hpp 257766 2011-03-16 14:41:24Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -497,7 +497,7 @@ private: // to prevent copying
 };
 
 
-class CScopeInfoLocker : protected CObjectCounterLocker
+class CScopeInfoLocker : public CObjectCounterLocker
 {
 public:
     void Lock(CScopeInfo_Base* info) const
