@@ -1,4 +1,4 @@
-/*  $Id: align_format_util.hpp 372466 2012-08-17 17:40:12Z rafanovi $
+/*  $Id: align_format_util.hpp 373569 2012-08-30 17:46:53Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -87,9 +87,11 @@ using the Entrez Genomes MapViewer</td></tr></table><p>";
 ///unigene
 // .ncbirc alias: UNIGEN
 static const char kUnigeneUrl[] = "<a href=\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=<@db@>&cmd=Display&dopt=<@dopt@>_unigene&from_uid=<@gi@>&RID=<@rid@>&log$=unigene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
+
 //substitues <@lnk_displ@>
 static const char kUnigeneImg[] = "<img border=0 height=16 width=16 src=\"images/U.gif\" alt=\"UniGene info linked to <@label@>\">";
-
+//For text link <@lnk@> is substituted by formatted url
+static const string kUnigeneDispl =  "<div><@lnk@>-<span class=\"rlLink\">clustered expressed sequence tags</span></div>";
 
 ///structure
 // .ncbirc alias: STRUCTURE_URL
@@ -97,23 +99,33 @@ static const char kStructureUrl[] = "<a href=\"http://www.ncbi.nlm.nih.gov/Struc
 &blast_view=<@blast_view@>&hsp=0&taxname=<@taxname@>&client=blast&log$=structure<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
 //substitues <@lnk_displ@>
 static const char kStructureImg[] = "<img border=0 height=16 width=16 src=\"http://www.ncbi.nlm.nih.gov/Structure/cblast/str_link.gif\" alt=\"Structure related to <@label@>\">";
+//For text link <@lnk@> is substituted by formatted url
+static const string kStructureDispl =  "<div><@lnk@>-<span class=\"rlLink\">3D structure displays</span></div>";
 
 ///structure overview
 static const char kStructure_Overview[] = "<a href=\"http://www.ncbi.nlm.nih.\
 gov/Structure/cblast/cblast.cgi?blast_RID=%s&blast_rep_gi=%d&hit=%d&%s\
 &blast_view=%s&hsp=0&taxname=%s&client=blast\">Related Structures</a>";
 
+
 ///Geo
 // .ncbirc alias: GEO
-static const char kGeoUrl[] =  "<a href=\"http://www.ncbi.nlm.nih.gov/geoprofiles/?term=<@gi@>[gi]&RID=<@rid@>&log$=geo<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
+static const char kGeoUrl[] =  "<a href=\"http://www/geoprofiles?LinkName=nuccore_geoprofiles&from_uid=<@gi@>&RID=<@rid@>&log$=geo<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
+
+
 //substitues <@lnk_displ@>
 static const char kGeoImg[] = "<img border=0 height=16 width=16 src=\"images/E.gif\" alt=\"GEO profiles info linked to <@label@>\">";
+//For text link <@lnk@> is substituted by formatted url
+static const string kGeoDispl =  "<div><@lnk@>-<span class=\"rlLink\">microarray expression data</span></div>";
 
 ///Gene
 // .ncbirc alias: GENE
-static const char kGeneUrl[] = "<a href=\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=gene&cmd=search&term=<@gi@>[<@uid@>]&RID=<@rid@>&log$=gene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
+static const char kGeneUrl[] = "<a href=\"http://www.ncbi.nlm.nih.gov/gene?term=<@gi@>[<@uid@>]&RID=<@rid@>&log$=gene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
+
 //substitues <@lnk_displ@>
 static const char kGeneImg[] = "<img border=0 height=16 width=16 src=\"images/G.gif\" alt=\"Gene info linked to <@label@>\">";
+//For text link <@lnk@> is substituted by formatted url
+static const string kGeneDispl =  "<div><@lnk@>-<span class=\"rlLink\">associated gene details</span></div>";
 
 ///Bioassay for proteins
 // .ncbirc alias: BIOASSAY_PROT
@@ -126,15 +138,21 @@ static const char kBioAssayProtImg[] = "<img border=0 height=16 width=16 src=\"i
 static const char kBioAssayNucURL[] = "<a href=\"http://www.ncbi.nlm.nih.gov/entrez?db=pcassay&term=<@gi@>[RNATargetGI]&RID=<@rid@>&log$=pcassay<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
 static const char kBioAssayNucImg[] = "<img border=0 height=16 width=16 src=\"images/Bioassay.gif\" alt=\"PubChem BioAssay Info linked to <@label@>\">";
 
+//For text link <@lnk@> is substituted by formatted url for both BioAssay Nuc and Prot
+static const string kBioAssayDispl =  "<div><@lnk@>-<span class=\"rlLink\">bioactivity screening</span></div>";
+
 ///mapviewer linkout
 // .ncbirc alias: MAPVIEWER
 static const char kMapviwerUrl[] = "<a href=\"http://www.ncbi.nlm.nih.gov/mapview/map_search.cgi?direct=on&gbgi=<@gi@>&THE_BLAST_RID=<@rid@>&log$=map<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@>><@lnk_displ@></a>";
 //substitues <@lnk_displ@>
 static const char kMapviwerImg[] = "<img border=0 height=16 width=16 src=\"images/M.gif\" alt=\"Genome view with mapviewer linked to <@label@>\">";
+//For text link <@lnk@> is substituted by formatted url
+static const string kMapviwerDispl =  "<div><@lnk@>-<span class=\"rlLink\">aligned genomic context</span></div>";
 
 ///mapviewer linkout
 //for used for NT/NW/NC
-static const char kMapviewBlastHitUrl[] = "<a href=\"<@user_url@>&db=<@db@>&na=<@is_na@>&gnl=<@gnl@>&gi=<@gi@>&term=<@gi@>[gi]&taxid=<@taxid@>&RID=<@rid@>&QUERY_NUMBER=<@query_number@>&log$=nucl<@log@>\"<@lnkTitle@>><@lnk_displ@></a>";
+static const string kMapviewBlastHitUrl = "http://www.ncbi.nlm.nih.gov/mapview/maps.cgi?maps=blast_set";
+static const string kMapviewBlastHitParams = "<a href=\"<@user_url@>&db=<@db@>&na=<@is_na@>&gnl=<@gnl@>&gi=<@gi@>&term=<@gi@>[gi]&taxid=<@taxid@>&RID=<@rid@>&QUERY_NUMBER=<@query_number@>&log$=nucl<@log@>\"<@lnkTitle@>><@lnk_displ@></a>";
 
 ///dumpgnl
 static const char kDownloadUrl[] = "/blast/dumpgnl.cgi";
@@ -142,11 +160,12 @@ static const char kDownloadLink[] = "<a href=\"<@download_url@>&segs=<@segs@>\">
 //substitues <@lnk_displ@>
 static const char kDownloadImg[] = "<img border=0 height=16 width=16 src=\"images/D.gif\" alt=\"Download subject sequence <@label@> spanning the HSP\">";
 
-static const char kSeqViewerUrl[] = "http://www.ncbi.nlm.nih.gov/<@dbtype@>/<@gi@>?report=graph&rid=<@rid@>&tracks=[key:gene_model_track],[key:alignment_track]&v=<@from@>:<@to@>";
+static const char kSeqViewerUrl[] = "http://www.ncbi.nlm.nih.gov/<@dbtype@>/<@gi@>?report=graph&rid=<@rid@>&<@seqViewerParams@>&v=<@from@>:<@to@>";
+static const string kSeqViewerParams = "tracks=[key:gene_model_track,CDSProductFeats:false][key:alignment_track,name:other alignments,annots:NG Alignments|Refseq Alignments|Gnomon Alignments|Unnamed,shown:false]";
 //to test ranges use:
 //static const char kSeqViewerUrl[] = "http://www.ncbi.nlm.nih.gov/<@dbtype@>/<@gi@>?report=graph&rid=<@rid@>&tracks=[key:gene_model_track],[key:alignment_track]&v=<@from@>:<@to@>,<@fromTest@>:<@toTest@>&flip=<@flip@>";
 
-static const char kCustomLinkTemplate[] = "<a href=\"<@custom_url@>\" class=\"<@custom_cls@>\" title=\"<@custom_title@>\"><@custom_lnk_displ@></a>"; 
+static const char kCustomLinkTemplate[] = "<a href=\"<@custom_url@>\" class=\"<@custom_cls@>\" target=\"<@custom_trg@>\" title=\"<@custom_title@>\"><@custom_lnk_displ@></a>"; 
 static const char kCustomLinkTitle[]="Show <@custom_report_type@> report for <@seqid@>";
 ///Sub-sequence
 // .ncbirc alias: ENTREZ_SUBSEQ
@@ -694,6 +713,12 @@ public:
     ///
     static CRef<objects::CSeq_align_set> HitListToHspList(list< CRef<objects::CSeq_align_set> >& source);
 
+    ///extract seq_align_set coreesponding to seqid list
+    ///@param all_aln_set: CSeq_align_set source/target list
+    ///@param alignSeqList: string of seqIds separated by comma
+    ///
+    static void ExtractSeqAlignForSeqList(CRef<objects::CSeq_align_set> &all_aln_set, string alignSeqList);
+
     ///return the custom url (such as mapview)
     ///@param ids: the id list
     ///@param taxid
@@ -970,8 +995,7 @@ public:
                                                  const string& rid,
                                                  const string& cdd_rid, 
                                                  const string& entrez_term,
-                                                 bool is_na,           
-                                                 int first_gi,
+                                                 bool is_na,                                                            
                                                  bool structure_linkout_as_group,
                                                  bool for_alignment, 
                                                  int cur_align,
@@ -1078,6 +1102,26 @@ public:
 
     static list<string>  GetGiLinksList(SSeqURLInfo *seqUrlInfo,bool hspRange = false);
 
+    ///Create URL showing aligned regions info
+    ///@param seqUrlInfo: struct SSeqURLInfo containing data for URL construction
+    ///@param id: CSeq_id object    
+    ///@param scope: scope to fetch this sequence
+    ///@return: string containing URL
+    ///
+    static string  GetFASTALinkURL(SSeqURLInfo *seqUrlInfo,
+                                   const objects::CSeq_id& id,
+                                   objects::CScope &scope);
+
+    ///Create URL to FASTA info
+    ///@param seqUrlInfo: struct SSeqURLInfo containing data for URL construction
+    ///@param id: CSeq_id object    
+    ///@param scope: scope to fetch this sequence
+    ///@return: string containing URL
+    ///
+    static string GetAlignedRegionsURL(SSeqURLInfo *seqUrlInfo,
+                                const objects::CSeq_id& id,
+                                objects::CScope &scope);
+
     ///Set the database as gi type
     ///@param actual_aln_list: the alignment
     ///@param scope: scope to fetch sequences
@@ -1093,6 +1137,17 @@ public:
 
     static map < string, CRef<objects::CSeq_align_set>  >  HspListToHitMap(vector <string> seqIdList, const objects::CSeq_align_set& source);
 
+    ///Scan the the list of blast deflines and find seqID to be use in display    
+    ///@param handle: CBioseq_Handle [in]
+    ///@param aln_id: CSeq_id object for alignment seq [in]
+    ///@param use_this_gi: list<int> list of gis to use [in]
+    ///@param gi: gi to be used for display if exists or 0    
+    ///@return: CSeq_id object to be used for display
+    static CRef<objects::CSeq_id> GetDisplayIds(const objects::CBioseq_Handle& handle,
+                                                const objects::CSeq_id& aln_id,
+                                                list<int>& use_this_gi,
+                                                int& gi);
+
     ///Get Gene symobol for gi
     ///@param  giForGeneLookup: gi
     ///@return: string gene symbol
