@@ -1,4 +1,4 @@
-/*  $Id: showalign.hpp 372466 2012-08-17 17:40:12Z rafanovi $
+/*  $Id: showalign.hpp 372408 2012-08-16 20:57:57Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,10 +51,9 @@
 #include <objmgr/object_manager.hpp>
 
 
+
 BEGIN_NCBI_SCOPE
-
 class CCgiContext;
-
 BEGIN_SCOPE(align_format)
 
 /// Auxiliary type to embed a CConstRef<objects::CSeq_id> in STL containers that require
@@ -256,12 +255,14 @@ class NCBI_ALIGN_FORMAT_EXPORT CDisplaySeqalign {
     }
 
     //Display pariwise seqalign for the set of seq IDS (for future use)
-    void DisplayPairwiseSeqalign(CNcbiOstream& out,hash_set <string> selectedIDs);
+    void DisplayPairwiseSeqalign(CNcbiOstream& out,hash_set <string> selectedIDs);        
     //Data representing templates for defline display 
     struct SAlignTemplates {
         string alignHeaderTmpl; ///< Template for displaying header,deflines and gene info  - BLAST_ALIGN_HEADER
         string sortInfoTmpl;    ///< Template for displaying  Sort by header - SORT_ALIGNS_SEQ
         string alnDefLineTmpl;    ///< Template for displaying one defline ALN_DEFLINE_ROW    
+        string alnTitlesLinkTmpl;    ///< Template for displaying link for more defline titles
+        string alnTitlesTmpl;    ///< Template for displaying multiple defline titles
         string alnSeqInfoTmpl;  ///< Template for displaying sequnce link in defline
         string alignInfoTmpl;   ///< Template for displaying singe align params - BLAST_ALIGN_PARAMS_NUC,BLAST_ALIGN_PARAMS_PROT
         string alignFeatureTmpl; ///< Template for displaying  align features -ALN_FEATURES 
@@ -435,6 +436,11 @@ class NCBI_ALIGN_FORMAT_EXPORT CDisplaySeqalign {
     void SetSequencePropertyLabel(const vector<string>* SequencePropertyLabel) {
         m_SeqPropertyLabel = SequencePropertyLabel;
     }
+ 
+    //set and add result index in front of seqid in <a name=seqid> for quick link (for multiple result case)
+    void SetResultPositionIndex(int index) {
+        m_ResultPositionIndex = index;
+    }
 
 private:
 
@@ -443,8 +449,8 @@ private:
     /// Prohibit assignment operator
     CDisplaySeqalign& operator=(const CDisplaySeqalign& rhs);
 
-protected: 
-
+protected:
+ 
     ///internal insert information
     ///aln_start. insert right after this position
     struct SInsertInformation : public CObject {
@@ -595,7 +601,11 @@ protected:
     
     map < string, struct SAlnLinksParams > m_AlnLinksParams;
     list <string> m_CustomLinksList;
+    list<string> m_LinkoutList;
     list <string> m_HSPLinksList;
+    string m_FASTAlinkUrl;
+    string m_AlignedRegionsUrl;
+    
 
     CRef < objects::CObjectManager > m_FeatObj;  // used for fetching feature
     CRef < objects::CScope > m_featScope;        // used for fetching feature
@@ -621,9 +631,18 @@ protected:
     /// mapviewer build name associated with the sequences in the BLAST
     /// database out of which the results are being formatted by this class.
     string m_MapViewerBuildName;
+    ///gi(if exists) that is used for html formatting otherwise id without db part
+    string m_CurrAlnID_Lbl;
+    ///accession that is displayed
+    string m_CurrAlnAccession;
+    ///gi(if exists) that is used for html formatting otherwise id with db part like ti:xxxxxxx or GNOMON:XXXXXX
+    string m_CurrAlnID_DbLbl;
 
     TranslatedFrameForLocalSeq m_TranslatedFrameForLocalSeq;
 
+    ///result position index for multiple query case
+    int m_ResultPositionIndex;
+
     string x_PrintDynamicFeatures(void); 
     ///Display the current alnvec
     ///@param out: stream for display
@@ -875,7 +894,7 @@ protected:
     SAlnRowInfo *x_PrepareRowData(void);
 
     string x_FormatSingleAlign(SAlnInfo* aln_vec_info);  
-    string x_FormatAlignSortInfo(string id_label);
+    string x_FormatAlignSortInfo();
     string x_FormatAlnBlastInfo(SAlnInfo* aln_vec_info);
     string x_FormatIdentityInfo(string alignInfo, SAlnInfo* aln_vec_info);
     string x_FormatDynamicFeaturesInfo(string alignInfo, SAlnInfo* aln_vec_info);
@@ -908,7 +927,8 @@ protected:
     
 	SAlnDispParams *x_FillAlnDispParams(const objects::CBioseq_Handle& bsp_handle);	
 	string x_FormatDefLinesHeader(const objects::CBioseq_Handle& bsp_handle,SAlnInfo* aln_vec_info);
-	string	x_MapDefLine(SAlnDispParams *alnDispParams,bool isFisrt, bool linkout,bool hideDefline);
+    string x_InitDefLinesHeader(const objects::CBioseq_Handle& bsp_handle,SAlnInfo* aln_vec_info);
+	string	x_MapDefLine(SAlnDispParams *alnDispParams,bool isFisrt, bool linkout,bool hideDefline,int seqLength);
 	void x_ShowAlnvecInfoTemplate(CNcbiOstream& out, SAlnInfo* aln_vec_info,bool show_defline,bool showSortControls);
 	void x_ShowAlnvecInfo(CNcbiOstream& out, SAlnInfo* aln_vec_info,bool show_defline);    
 
