@@ -1,4 +1,4 @@
-/*  $Id: showdefline.hpp 354621 2012-02-28 18:29:04Z zaretska $
+/*  $Id: showdefline.hpp 373569 2012-08-30 17:46:53Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -99,6 +99,7 @@ public:
     ///Data Representing each defline
     struct SDeflineInfo {
         CConstRef<objects::CSeq_id> id;         //best accession type id
+        string alnIDFasta;
         int gi;                        //gi 
         string defline;                //defline
         list<string> linkout_list;     //linkout urls
@@ -234,6 +235,12 @@ public:
         m_BlastType = type;
     }
 
+    ///Sets the start index of defline for PSI blast description table
+    /// @param int startIndex [in]
+    void SetStartIndex(int startIndex) {
+        m_StartIndex = startIndex;
+    }
+
     void SetCgiContext (CCgiContext& ctx) {
         m_Ctx = &ctx;
     }
@@ -251,6 +258,11 @@ public:
         m_PreComputedResID = preComputedResID;
     }
     
+    ///set and add result position index to <name=seqid> in score quick link for multiple result case
+    void SetResultPosIndex(int index) {
+        m_PositionIndex = index;
+    }
+
     /// Creates a '|' delimited string, corresponding to a list of Seq-ids
     /// @param id List of Seq-ids [in]
     /// @param show_gi Should gi ids be shown or skipped?
@@ -432,6 +444,12 @@ protected:
     /// database out of which the results are being formatted by this class.
     string m_MapViewerBuildName;
 
+    ///The start index of defline for PSI blast description table
+    int    m_StartIndex;
+
+    ///result position index for multiple query case
+    int m_PositionIndex;
+
     ///Internal function to return score info
     ///@param aln seq-align we are working with [in]
     ///@param blast_rank ordinal nubmer of defline [in]
