@@ -1,4 +1,4 @@
-/* $Id: tabular.hpp 367113 2012-06-20 19:26:08Z rafanovi $ 
+/* $Id: tabular.hpp 386699 2013-01-22 22:08:28Z rafanovi $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -39,13 +39,16 @@
 #include <objects/seqloc/Seq_id.hpp>
 #include <objmgr/scope.hpp>
 #include <objtools/align_format/align_format_util.hpp>
+#include <objtools/blast/seqdb_reader/seqdb.hpp>
 #include <algo/blast/igblast/igblast.hpp>
+#include <objects/blastdb/Blast_def_line_set.hpp>
 
 #include <algorithm>
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(align_format)
 
+
 /// Class containing information needed for tabular formatting of BLAST 
 /// results.
 class NCBI_ALIGN_FORMAT_EXPORT CBlastTabularInfo : public CObject 
@@ -121,7 +124,7 @@ public:
     /// @param scope Scope for Bioseq retrieval [in]
     /// @param matrix Matrix to calculate positives; NULL if not applicable. [in]
     /// @return 0 on success, 1 if query or subject Bioseq is not found.
-    virtual int SetFields(const objects::CSeq_align& sal, 
+    int SetFields(const objects::CSeq_align& sal, 
                           objects::CScope& scope, 
                           CNcbiMatrix<int>* matrix=0);
 
@@ -136,7 +139,7 @@ public:
     /// @param iteration Iteration number (for PSI-BLAST), use default
     /// parameter value when not applicable [in]
     /// @param align_set All alignments for this query [in]
-    virtual void PrintHeader(const string& program, 
+    void PrintHeader(const string& program, 
                              const objects::CBioseq& bioseq, 
                              const string& dbname, 
                              const string& rid = kEmptyStr,
@@ -262,6 +265,21 @@ protected:
     void x_PrintQueryLength();
     /// Print the subject sequence length
     void x_PrintSubjectLength();
+    /// Print subject tax info
+    void x_PrintSubjectTaxIds();
+    void x_PrintSubjectSciNames();
+    void x_PrintSubjectCommonNames();
+    void x_PrintSubjectBlastNames();
+    void x_PrintSubjectSuperKingdoms();
+    void x_PrintSubjectTitle();
+    void x_PrintSubjectAllTitles();
+    void x_PrintSubjectStrand();
+    void x_PrintSeqalignCoverage();
+    void x_PrintSubjectCoverage();
+    void x_SetTaxInfo(const objects::CBioseq_Handle & handle, const CRef<objects::CBlast_def_line_set> & bdlRef);
+    void x_SetSubjectId(const objects::CBioseq_Handle& bh, const CRef<objects::CBlast_def_line_set> & bdlRef);
+    void x_SetQueryCovSubject(const objects::CSeq_align & align);
+    void x_SetQueryCovSeqalign(const CSeq_align & align, int query_len);
 
     CNcbiOstream& m_Ostream; ///< Stream to write output to
     char m_FieldDelimiter;   ///< Delimiter character for fields to print.
@@ -296,8 +314,21 @@ private:
     /// Should the query deflines be parsed for local IDs?
     bool m_ParseLocalIds;
     string m_BTOP;            /// Blast-traceback-operations.
+
+    //TaxInfo
+    set<int>			m_SubjectTaxIds;
+    vector<string>		m_SubjectSciNames;
+    vector<string>		m_SubjectCommonNames;
+    set<string>		m_SubjectBlastNames;
+    set<string>		m_SubjectSuperKingdoms;
+    CRef<CBlast_def_line_set> m_SubjectDefline;
+
+    string m_SubjectStrand;
+    pair<string, int>  m_QueryCovSubject;
+    int m_QueryCovSeqalign;
 };
 
+
 inline void CBlastTabularInfo::x_PrintQuerySeq(void)
 {
     m_Ostream << m_QuerySeq;
@@ -508,12 +539,14 @@ public:
     int SetMasterFields(const objects::CSeq_align& align, 
                         objects::CScope&           scope, 
                         const string&              chain_type,
+                        const string& master_chain_type_to_show,
                         CNcbiMatrix<int>*          matrix=0);
 
     /// Set fields for all other alignments
     int SetFields(const objects::CSeq_align& align,
                   objects::CScope&           scope,
                   const string&              chain_type,
+                  const string& master_chain_type_to_show,
                   CNcbiMatrix<int>*          matrix=0);
 
     /// Override the print method
@@ -582,10 +615,12 @@ private:
     bool m_IsMinusStrand;
     string m_FrameInfo;                                                           
     string m_ChainType;
+    string m_MasterChainTypeToShow;
     SIgGene m_VGene;
     SIgGene m_DGene;
     SIgGene m_JGene;
     vector<SIgDomain *> m_IgDomains;                                        
+    vector<string> m_OtherInfo;
 };
 
 END_SCOPE(align_format)
