@@ -1,4 +1,4 @@
-/*  $Id: blastdb_dataextract.hpp 319614 2011-07-22 19:20:26Z camacho $
+/*  $Id: blastdb_dataextract.hpp 389294 2013-02-14 18:43:48Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -89,9 +89,13 @@ public:
     string ExtractLinkoutInteger();
     string ExtractMembershipInteger();
     string ExtractLinkoutTokens();
-
+    string ExtractAsn1Defline();
+    string ExtractAsn1Bioseq();
     string ExtractFasta(const CBlastDBSeqId &seq_id);
 
+    // Call before ExtractFasta or SetSeqId
+    void SetConfig(TSeqRange range, objects::ENa_strand strand, int filt_algo_id);
+
 protected:
     /// underlying Blast database
     CSeqDB& m_BlastDb;
