@@ -1,4 +1,4 @@
-/*  $Id: seq_writer.hpp 218360 2010-12-28 14:33:02Z maning $
+/*  $Id: seq_writer.hpp 386876 2013-01-23 23:13:13Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -95,6 +95,8 @@ public:
     /// @throws CExcpetion derived classes on error
     void DumpAll(CSeqDB& blastdb, CSeqFormatterConfig config = CSeqFormatterConfig());
 
+    /// Set range, strand and filter algo for each seq id
+    void SetConfig(TSeqRange range, objects::ENa_strand strand, int filt_algo_id);
 private:
     /// Stream to write output
     CNcbiOstream& m_Out;
