@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 
-/*  $Id: seqdbcommon.hpp 211164 2010-11-10 23:19:50Z camacho $
+/*  $Id: seqdbcommon.hpp 389294 2013-02-14 18:43:48Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1366,8 +1366,9 @@ typedef map< string, TSeqDBAliasFileVersions > TSeqDBAliasFileValues;
 
 struct SSeqDBTaxInfo {
     /// Default constructor
-    SSeqDBTaxInfo()
-        : taxid(0)
+    /// @param t the taxonomy ID to set for this structure
+    SSeqDBTaxInfo(int t = 0)
+        : taxid(t)
     {
     }
     
@@ -1385,6 +1386,15 @@ struct SSeqDBTaxInfo {
     
     /// A string of length 1 indicating the "Super Kingdom".
     string s_kingdom;
+
+    friend ostream& operator<<(ostream& out, const SSeqDBTaxInfo& rhs) {
+        out << "Taxid=" << rhs.taxid
+            << "\tSciName=" << rhs.scientific_name
+            << "\tCommonName=" << rhs.common_name
+            << "\tBlastName=" << rhs.blast_name
+            << "\tSuperKingdom=" << rhs.s_kingdom;
+        return out;
+    }
 };
 
 
