@@ -2,7 +2,7 @@
 #define PROCESSOR__HPP_INCLUDED
 /* */
 
-/*  $Id: processor.hpp 330387 2011-08-11 16:49:59Z vasilche $
+/*  $Id: processor.hpp 390318 2013-02-26 21:04:57Z vasilche $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -51,6 +51,8 @@ class CWriter;
 class CID2_Reply_Data;
 class CLoadLockBlob;
 class CTSE_SetObjectInfo;
+class CTSE_Chunk_Info;
+class CDataLoader;
 struct STimeSizeStatistics;
 
 class NCBI_XREADER_EXPORT CProcessor : public CObject 
@@ -85,6 +87,7 @@ public:
 
     enum {
         kMain_ChunkId       = -1, // not a chunk, but main Seq-entry
+        kMasterWGS_ChunkId  = kMax_Int-1, // chunk with master WGS descr
         kDelayedMain_ChunkId= kMax_Int // main Seq-entry with delayed ext annot
     };
 
@@ -111,13 +114,20 @@ public:
     static void SetSeqEntryReadHooks(CObjectIStream& in);
     static void SetSNPReadHooks(CObjectIStream& in);
 
-    static bool IsLoaded(const TBlobId& blob_id,
+    static bool IsLoaded(CReaderRequestResult& result,
+                         const TBlobId& blob_id,
                          TChunkId chunk_id,
                          CLoadLockBlob& blob);
     static void SetLoaded(CReaderRequestResult& result,
                           const TBlobId& blob_id,
                           TChunkId chunk_id,
                           CLoadLockBlob& blob);
+    static void AddWGSMaster(CReaderRequestResult& result,
+                             const TBlobId& blob_id,
+                             TChunkId chunk_id,
+                             CLoadLockBlob& blob);
+    static void LoadWGSMaster(CDataLoader* loader,
+                              CRef<CTSE_Chunk_Info> chunk);
 
     static void SetSeq_entry(CReaderRequestResult& /*result*/,
                              const TBlobId& /*blob_id*/,
@@ -157,6 +167,7 @@ NCBI_PARAM_DECL(bool, GENBANK, SNP_TABLE);
 NCBI_PARAM_DECL(bool, GENBANK, USE_MEMORY_POOL);
 NCBI_PARAM_DECL(int, GENBANK, READER_STATS);
 NCBI_PARAM_DECL(bool, GENBANK, CACHE_RECOMPRESS);
+NCBI_PARAM_DECL(bool, GENBANK, ADD_WGS_MASTER);
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
