@@ -1,6 +1,6 @@
 #ifndef READER__HPP_INCLUDED
 #define READER__HPP_INCLUDED
-/*  $Id: reader.hpp 347885 2011-12-21 18:20:59Z vasilche $
+/*  $Id: reader.hpp 389585 2013-02-19 17:21:14Z vasilche $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -294,9 +294,14 @@ public:
     
     void Release(void);
     void Restart(void);
+
+    bool IsAllocated(void) const {
+        return m_Result != 0;
+    }
     
     operator TConn(void) const
         {
+            _ASSERT(IsAllocated());
             return m_Conn;
         }
     
