@@ -1,7 +1,7 @@
 #ifndef GBLOADER_REQUEST_RESULT__HPP_INCLUDED
 #define GBLOADER_REQUEST_RESULT__HPP_INCLUDED
 
-/*  $Id: request_result.hpp 347885 2011-12-21 18:20:59Z vasilche $
+/*  $Id: request_result.hpp 390834 2013-03-02 19:33:56Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -59,6 +59,7 @@ class CBlob_id;
 class CTSE_Info;
 class CTimer;
 
+class CGBDataLoader;
 class CLoadInfo;
 class CLoadInfoLock;
 class CLoadLock_Base;
@@ -507,6 +508,7 @@ private:
 class NCBI_XREADER_EXPORT CLoadLockBlob : public CTSE_LoadLock
 {
 public:
+    CLoadLockBlob(void);
     CLoadLockBlob(CReaderRequestResult& src, const CBlob_id& blob_id);
 
     typedef int TBlobState;
@@ -580,6 +582,8 @@ public:
     typedef int TBlobVersion;
     typedef int TBlobState;
 
+    virtual CGBDataLoader* GetLoaderPtr(void);
+
     virtual CRef<TInfoSeq_ids>  GetInfoSeq_ids(const TKeySeq_ids& seq_id) = 0;
     virtual CRef<TInfoSeq_ids>  GetInfoSeq_ids(const TKeySeq_ids2& seq_id) = 0;
     virtual CRef<TInfoBlob_ids> GetInfoBlob_ids(const TKeyBlob_ids& seq_id) = 0;
@@ -669,7 +673,7 @@ private:
 
     void ReleaseLoadLock(const CRef<CLoadInfo>& info);
 
-    typedef map<CRef<CLoadInfo>, CLoadInfoLock*> TLockMap;
+    typedef map<CRef<CLoadInfo>, CRef<CLoadInfoLock> > TLockMap;
     typedef pair<TBlobVersion, CTSE_LoadLock> TBlobLoadInfo;
     typedef map<CBlob_id, TBlobLoadInfo> TBlobLoadLocks;
 
