@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___CONTEXT__HPP
 #define OBJTOOLS_FORMAT___CONTEXT__HPP
 
-/*  $Id: context.hpp 372931 2012-08-23 16:50:20Z rafanovi $
+/*  $Id: context.hpp 377677 2012-10-15 16:03:18Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -132,6 +132,8 @@ public:
     // top-level Seq-entry info
     CTopLevelSeqEntryContext& GetTLSeqEntryCtx(void) { return *m_TLSeqEntryCtx; }
     bool CanGetTLSeqEntryCtx                  (void) const { return m_TLSeqEntryCtx.NotNull(); }
+    bool IsBioseqInSameTopLevelSeqEntry(const CBioseq_Handle & seq);
+    bool IsSeqIdInSameTopLevelSeqEntry(const CSeq_id & seq_id );
 
     // delta sequence
     bool IsDelta(void) const { return m_Repr == CSeq_inst::eRepr_delta; }
@@ -449,6 +451,18 @@ void CBioseqContext::SetMaster(CMasterContext& mctx) {
     m_Master.Reset(&mctx);
 }
 
+inline
+bool CBioseqContext::IsBioseqInSameTopLevelSeqEntry(const CBioseq_Handle & seq)
+{
+    return seq.GetTopLevelEntry() == m_Handle.GetTopLevelEntry();
+}
+
+inline
+bool CBioseqContext::IsSeqIdInSameTopLevelSeqEntry(const CSeq_id & seq_id)
+{
+    return GetScope().GetBioseqHandleFromTSE(seq_id, m_Handle);
+}
+
 inline
 bool CBioseqContext::IsRSCompleteGenomic(void)  const
 {
@@ -512,7 +526,8 @@ bool CBioseqContext::IsRSPredictedProtein(void)  const
 inline
 bool CBioseqContext::IsRSWGSNuc(void)  const
 {
-    return m_RefseqInfo == CSeq_id::eAcc_refseq_wgs_nuc;  // NZ_
+    // ignore "master" bit if it's there
+    return (m_RefseqInfo & ~CSeq_id::fAcc_master) == CSeq_id::eAcc_refseq_wgs_nuc;  // NZ_
 }
 
 inline
