@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 
-/*  $Id: flat_file_config.hpp 328426 2011-08-03 15:11:31Z kornbluh $
+/*  $Id: flat_file_config.hpp 390834 2013-03-02 19:33:56Z dicuccio $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,11 +32,39 @@
  *    Configuration class for flat-file generator
  */
 #include <corelib/ncbistd.hpp>
+#include <corelib/ncbiobj.hpp>
 
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
+class CBioseqContext;
+class CStartSectionItem;
+class CHtmlAnchorItem;
+class CLocusItem;
+class CDeflineItem;
+class CAccessionItem;
+class CVersionItem;
+class CGenomeProjectItem;
+class CDBSourceItem;
+class CKeywordsItem;
+class CSegmentItem;
+class CSourceItem;
+class CReferenceItem;
+class CCommentItem;
+class CPrimaryItem;
+class CFeatHeaderItem;
+class CSourceFeatureItem;
+class CFeatureItem;
+class CGapItem;
+class CBaseCountItem;
+class COriginItem;
+class CSequenceItem;
+class CContigItem;
+class CWGSItem;
+class CTSAItem;
+class CEndSectionItem;
+class IFlatItem;
 
 // --- Flat File configuration class
 
@@ -133,6 +161,44 @@ public:
         fGffForFlybase         = 1 << 4, ///< Flybase flavour of GFF3.
     };
 
+    // These flags are used to select the GenBank sections to print or skip.
+    enum FGenbankBlocks {
+        // default is all sections
+        fGenbankBlocks_All        = (~0u),
+        fGenbankBlocks_None       = 0,
+
+        // or, specify individual sections to print.
+        // or, specify individual sections to skip
+        // (via (fGenbankBlocks_All & ~fGenbankBlocks_Locus), for example)
+        // Note that these flags do NOT have a one-to-one relationship
+        // with the notify() functions in CGenbankBlockCallback.
+        // For example, fGenbankBlocks_FeatAndGap is used to turn
+        // on both the CFeatureItem and CGapItem notify functions.
+        fGenbankBlocks_Head       = (1u <<  0),
+        fGenbankBlocks_Locus      = (1u <<  1),
+        fGenbankBlocks_Defline    = (1u <<  2),
+        fGenbankBlocks_Accession  = (1u <<  3),
+        fGenbankBlocks_Version    = (1u <<  4),
+        fGenbankBlocks_Project    = (1u <<  5),
+        fGenbankBlocks_Dbsource   = (1u <<  6),
+        fGenbankBlocks_Keywords   = (1u <<  7),
+        fGenbankBlocks_Segment    = (1u <<  8),
+        fGenbankBlocks_Source     = (1u <<  9),
+        fGenbankBlocks_Reference  = (1u << 10),
+        fGenbankBlocks_Comment    = (1u << 11),
+        fGenbankBlocks_Primary    = (1u << 12), 
+        fGenbankBlocks_Featheader = (1u << 13),
+        fGenbankBlocks_Sourcefeat = (1u << 14),
+        fGenbankBlocks_FeatAndGap = (1u << 15),
+        fGenbankBlocks_Basecount  = (1u << 16),
+        fGenbankBlocks_Origin     = (1u << 17),
+        fGenbankBlocks_Sequence   = (1u << 18),
+        fGenbankBlocks_Contig     = (1u << 19),
+        fGenbankBlocks_Wgs        = (1u << 20),
+        fGenbankBlocks_Tsa        = (1u << 21),
+        fGenbankBlocks_Slash      = (1u << 22)
+    };
+
     // types
     typedef EFormat         TFormat;
     typedef EMode           TMode;
@@ -140,6 +206,111 @@ public:
     typedef unsigned int    TFlags; // binary OR of "EFlatFileFlags"
     typedef EView           TView;
     typedef unsigned int    TGffOptions;
+    typedef unsigned int    TGenbankBlocks;
+    
+    class NCBI_FORMAT_EXPORT CGenbankBlockCallback : public CObject {
+    public:
+        enum EAction {
+            // the normal case
+            eAction_Default,
+            // skip this block (i.e. don't print it)
+            eAction_Skip,
+            // if for some reason you don't want the rest of the flatfile generated, 
+            // this will trigger a CFlatException of type eHaltRequested
+            eAction_HaltFlatfileGeneration
+        };
+
+        virtual ~CGenbankBlockCallback(void) { }
+
+        // please note that these notify functions let you change the block_text
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CStartSectionItem & head_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CHtmlAnchorItem & anchor_item );
+        virtual EAction notify( string & block_text, 
+                                const CBioseqContext& ctx,
+                                const CLocusItem &locus_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CDeflineItem & defline_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CAccessionItem & accession_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CVersionItem & version_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CGenomeProjectItem & project_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CDBSourceItem & dbsource_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CKeywordsItem & keywords_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CSegmentItem & segment_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CSourceItem & source_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CReferenceItem & ref_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CCommentItem & comment_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CPrimaryItem & primary_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CFeatHeaderItem & featheader_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CSourceFeatureItem & sourcefeat_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CFeatureItem & feature_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CGapItem & feature_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CBaseCountItem & basecount_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const COriginItem & origin_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CSequenceItem & sequence_chunk_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CContigItem & contig_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CWGSItem & wgs_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CTSAItem & tsa_item );
+        virtual EAction notify( string & block_text,
+                                const CBioseqContext& ctx,
+                                const CEndSectionItem & slash_item );
+
+        // add more overridable functions if more blocks are invented
+
+
+        // ...or override
+        // this if you want only a single entry-point for
+        // notifications.
+        virtual EAction unified_notify( 
+            string & block_text,
+            const CBioseqContext& ctx, 
+            const IFlatItem & flat_item, 
+            FGenbankBlocks which_block ) { return eAction_Default; }
+    };
 
     // constructors
     CFlatFileConfig(TFormat format = eFormat_GenBank,
@@ -147,7 +318,9 @@ public:
                     TStyle  style = eStyle_Normal,
                     TFlags  flags = 0,
                     TView   view = fViewNucleotides,
-                    TGffOptions gff_options = fGffGTFCompat );
+                    TGffOptions gff_options = fGffGTFCompat,
+                    TGenbankBlocks genbank_blocks = fGenbankBlocks_All,
+                    CGenbankBlockCallback* pGenbankBlockCallback = NULL );
     // destructor
     ~CFlatFileConfig(void);
 
@@ -265,7 +438,7 @@ public:
     bool UseEmblMolType      (void) const;
     bool HideBankItComment   (void) const;
     bool CheckCDSProductId   (void) const;
-    bool SuppressSegLoc      (void) const;
+    bool FrequencyToNote      (void) const;
     bool SrcQualsToNote      (void) const;
     bool HideEmptySource     (void) const;
     bool GoQualsToNote       (void) const;
@@ -367,6 +540,34 @@ public:
         m_GffOptions |= fGffForFlybase;
     };
 
+    // check if the given section is shown
+    bool IsShownGenbankBlock(FGenbankBlocks fTGenbankBlocksMask) const
+    {
+        return (m_fGenbankBlocks & fTGenbankBlocksMask) == fTGenbankBlocksMask;
+    }
+
+    // set the given section to be shown
+    void ShowGenbankBlock(FGenbankBlocks fTGenbankBlocksMask)
+    {
+        m_fGenbankBlocks |= (fTGenbankBlocksMask);
+    }
+
+    // set the given section to be skipped
+    // (that is, not shown or even processed)
+    void SkipGenbankBlock(FGenbankBlocks fTGenbankBlocksMask)
+    {
+        m_fGenbankBlocks &= (~fTGenbankBlocksMask);
+    }
+
+    // throws on error
+    static FGenbankBlocks StringToGenbankBlock(const string & str);
+    // returns the set of all possible genbank strings
+    // "head, "locus", etc.  Guaranteed to be sorted.
+    static const vector<string> & GetAllGenbankStrings(void);
+
+    // return non-const callback even if the CFlatFileConfig is const
+    CRef<CGenbankBlockCallback> GetGenbankBlockCallback(void) const { return m_GenbankBlockCallback; }
+
 public:
     static const size_t SMARTFEATLIMIT = 1000000;
 
@@ -382,7 +583,8 @@ private:
     TFlags      m_Flags;  // custom flags
     bool        m_RefSeqConventions;
     TGffOptions m_GffOptions;
-
+    TGenbankBlocks m_fGenbankBlocks;
+    CRef<CGenbankBlockCallback> m_GenbankBlockCallback;
 };
 
 
