@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___GENBANK_GATHER__HPP
 #define OBJTOOLS_FORMAT___GENBANK_GATHER__HPP
 
-/*  $Id: genbank_gather.hpp 360035 2012-04-19 13:43:48Z kornbluh $
+/*  $Id: genbank_gather.hpp 379632 2012-11-02 15:56:26Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -53,8 +53,8 @@ public:
     virtual void x_DoSingleSection(CBioseqContext& ctx) const;
 
 private:
-    void x_GatherWGS(CBioseqContext& ctx) const;
-    void x_GatherTSA(CBioseqContext& ctx) const;
+    void x_GatherWGS(void) const;
+    void x_GatherTSA(void) const;
 };
 
 
