@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___COMMENT_ITEM__HPP
 #define OBJTOOLS_FORMAT_ITEMS___COMMENT_ITEM__HPP
 
-/*  $Id: comment_item.hpp 370808 2012-08-01 20:00:57Z rafanovi $
+/*  $Id: comment_item.hpp 379630 2012-11-02 15:55:46Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -111,6 +111,9 @@ public:
 
     void AddPeriod(void);
 
+    void RemoveExcessNewlines( const CCommentItem & next_comment );
+    void RemovePeriodAfterURL(void);
+
     enum ECommentFormat
     {
         eFormat_Text,
