@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 
-/*  $Id: feature_item.hpp 372931 2012-08-23 16:50:20Z rafanovi $
+/*  $Id: feature_item.hpp 381315 2012-11-20 20:42:10Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -172,33 +172,6 @@ protected:
 
     void x_GatherInfoWithParent(CBioseqContext& ctx, CConstRef<CFeatureItem> parentFeatureItem );
 
-    bool x_ExceptionIsLegalForFeature() const;
-    void x_GetAssociatedGeneInfo( CBioseqContext& ctx, const CGene_ref*&,
-        CConstRef<CSeq_feat>&, CConstRef<CFeatureItem> parentFeatureItem ) const;
-    CSeq_feat_Handle x_ResolveGeneXref( const CGene_ref *xref_g_ref, CBioseqContext& ctx ) const;
-    bool x_CanUseExtremesToFindGene( CBioseqContext& ctx, const CSeq_loc &location ) const;
-    CConstRef<CSeq_feat> 
-        x_GetFeatViaSubsetThenExtremesIfPossible( 
-            CBioseqContext& ctx, CSeqFeatData::E_Choice feat_type,
-            CSeqFeatData::ESubtype feat_subtype,
-            const CSeq_loc &location, CSeqFeatData::E_Choice sought_type,
-            const CGene_ref* filtering_gene_xref ) const ;
-
-    CConstRef<CSeq_feat> 
-        x_GetFeatViaSubsetThenExtremesIfPossible_Helper(
-            CBioseqContext& ctx, CScope *scope, const CSeq_loc &location, CSeqFeatData::E_Choice sought_type,
-            const CGene_ref* filtering_gene_xref ) const;
-    // These 2 functions could just be folded into x_GetFeatViaSubsetThenExtremesIfPossible_Helper,
-    // but they're separate to make it easier to profile the different paths.
-    CConstRef<CSeq_feat> 
-        x_GetFeatViaSubsetThenExtremesIfPossible_Helper_subset(
-            CBioseqContext& ctx, CScope *scope, const CSeq_loc &location, CSeqFeatData::E_Choice sought_type,
-            const CGene_ref* filtering_gene_xref ) const;
-    CConstRef<CSeq_feat> 
-        x_GetFeatViaSubsetThenExtremesIfPossible_Helper_extremes(
-            CBioseqContext& ctx, CScope *scope, const CSeq_loc &location, CSeqFeatData::E_Choice sought_type,
-            const CGene_ref* filtering_gene_xref ) const;
-
     void x_GetAssociatedProtInfo( CBioseqContext&, CBioseq_Handle&,
         const CProt_ref*&, CMappedFeat& protFeat, CConstRef<CSeq_id>& );
     void x_AddQualPartial( CBioseqContext& );
