@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___REFERENCE_ITEM__HPP
 #define OBJTOOLS_FORMAT_ITEMS___REFERENCE_ITEM__HPP
 
-/*  $Id: reference_item.hpp 213605 2010-11-24 15:12:46Z kornbluh $
+/*  $Id: reference_item.hpp 375501 2012-09-20 18:07:20Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -192,6 +192,9 @@ private:
     void x_CleanData(void);
     void x_CapitalizeTitleIfNecessary(void);
 
+    void x_ChangeMedlineAuthorsToISO( CRef<CPub> pub );
+    bool x_StringIsJustCapitalLetters( const string & str );
+
     // data
     TPubType              m_PubType;
     TCategory             m_Category;
