@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.hpp 332908 2011-08-31 14:52:41Z ludwigf $
+/*  $Id: bed_reader.hpp 355049 2012-03-02 12:12:44Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,6 +44,11 @@ BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects) // namespace ncbi::objects::
 
 //  ----------------------------------------------------------------------------
+/// CReaderBase implementation that reads BED data files, either a single object
+/// or all objects found. For the purpose of CBedReader, an object consists of
+/// a run of records all with the same ID (BED comlumn 1), and all contained
+/// within a single track.
+///
 class NCBI_XOBJREAD_EXPORT CBedReader
 //  ----------------------------------------------------------------------------
     : public CReaderBase
@@ -53,7 +58,7 @@ class NCBI_XOBJREAD_EXPORT CBedReader
     //
 public:
     CBedReader( 
-        int =fDefaults );
+        unsigned int =fNormal );
     virtual ~CBedReader();
     
     //
@@ -61,50 +66,78 @@ public:
     //
 public:
     enum EBedFlags {
-        fDefaults = 0,
-        fAllIdsAsLocal = 1<<0,
-        fNumericIdsAsLocal = 1<<1,
-        // open slot
-        fDumpStats = 1<<3,
+        // currentry, only what's inherited from CReaderBase
     };
     typedef int TFlags;
 
+    /// Read object from line reader containing BED data, rendering it as a
+    /// Seq-annot
+    /// @param lr
+    ///   line reader to read from.
+    /// @param pErrors
+    ///   pointer to optional error container object. 
+    ///
     virtual CRef< CSerialObject >
     ReadObject(
-        ILineReader&,
-        IErrorContainer* =0 );
+        ILineReader& lr,
+        IErrorContainer* pErrors=0 );
                 
+    /// Read a single object from given line reader containing BED data. The
+    /// resulting Seq-annot will contain a feature table.
+    /// @param lr
+    ///   line reader to read from.
+    /// @param pErrors
+    ///   pointer to optional error container object. 
+    ///  
     virtual CRef< CSeq_annot >
     ReadSeqAnnot(
-        ILineReader&,
-        IErrorContainer* =0 );
+        ILineReader& lr,
+        IErrorContainer* pErrors=0 );
 
+    /// Read all objects from given insput stream, returning them as a vector of
+    /// Seq-annots, each containing a feature table.
+    /// @param annots
+    ///   (out) vector containing read Seq-annots
+    /// @param istr
+    ///   input stream to read from.
+    /// @param pErrors
+    ///   pointer to optional error container object. 
+    ///  
     virtual void
     ReadSeqAnnots(
-        vector< CRef<CSeq_annot> >&,
-        CNcbiIstream&,
-        IErrorContainer* =0 );
+        vector< CRef<CSeq_annot> >& annots,
+        CNcbiIstream& istr,
+        IErrorContainer* pErrors=0 );
                         
+    /// Read all objects from given insput stream, returning them as a vector of
+    /// Seq-annots, each containing a feature table.
+    /// @param annots
+    ///   (out) vector containing read Seq-annots
+    /// @param lr
+    ///   line reader to read from.
+    /// @param pErrors
+    ///   pointer to optional error container object. 
+    ///  
     virtual void
     ReadSeqAnnots(
-        vector< CRef<CSeq_annot> >&,
-        ILineReader&,
-        IErrorContainer* =0 );
+        vector< CRef<CSeq_annot> >& annots,
+        ILineReader& lr,
+        IErrorContainer* pErrors=0 );
                         
-    //
-    //  helpers:
-    //
 protected:
-    virtual bool x_ParseTrackLine(
+    virtual bool xParseTrackLine(
         const string&,
-        vector< CRef< CSeq_annot > >&,
         CRef< CSeq_annot >& );
         
-    bool x_ParseFeature(
+    bool xParseFeature(
         const string&,
-        CRef<CSeq_annot>& );
+        CRef<CSeq_annot>&);
     /* throws CObjReaderLineException */
 
+    bool xParseComment(
+        const string&,
+        CRef<CSeq_annot>&);
+
     void x_SetFeatureLocation(
         CRef<CSeq_feat>&,
         const vector<string>& );
@@ -119,33 +152,23 @@ protected:
         const string&,
         const string& );
 
-    CRef<CSeq_id> x_ResolvedId(
-        const string& );
-
     CRef< CSeq_annot > x_AppendAnnot(
         vector< CRef< CSeq_annot > >& );
                     
     void
-    x_ProcessError(
+    xProcessError(
         CObjReaderLineException&,
         IErrorContainer* );
 
-    void x_CountRecord(
-        const string& );
-
-    void x_DumpStats(
-        CNcbiOstream& );
-        
     //
     //  data:
     //
 protected:
     CErrorContainerLenient m_ErrorsPrivate;
-    
+    string m_currentId;
+
     vector<string>::size_type m_columncount;
     bool m_usescore;
-//    int m_iFlags;
-    map< string, unsigned int > m_RecordCounts;
 };
 
 END_SCOPE(objects)
