@@ -1,4 +1,4 @@
-/*  $Id: error_container.hpp 311628 2011-07-12 17:23:34Z whlavina $
+/*  $Id: error_container.hpp 352531 2012-02-07 19:02:19Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -72,7 +72,7 @@ public:
 //  ============================================================================
 class CErrorContainerBase:
 //  ============================================================================
-    public IErrorContainer
+    public IErrorContainer, public CObject
 {
 public:
     CErrorContainerBase() {};
