@@ -1,4 +1,4 @@
- /*  $Id: gff2_reader.hpp 256444 2011-03-04 17:45:13Z rafanovi $
+ /*  $Id: gff2_reader.hpp 337492 2011-09-13 11:28:54Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -54,14 +54,16 @@ class SRecord;
 //  ----------------------------------------------------------------------------
 class NCBI_XOBJREAD_EXPORT CGff2Reader
 //  ----------------------------------------------------------------------------
-    : protected CGFFReader, public CReaderBase
+    : public CReaderBase
 {
 public:
     typedef enum {
         fNormal =       0,
         fNewCode =      0x1000, // for now don't clobber CGFFReader flags
     } TFlags;
-    
+
+    typedef map< string, CRef< CSeq_feat > > IdToFeatureMap;
+   
 public:
     typedef vector< CRef< CSeq_annot > > TAnnots;
     typedef TAnnots::iterator TAnnotIt;
@@ -72,7 +74,7 @@ public:
     //
 public:
     CGff2Reader(
-        unsigned int uFlags,
+        int iFlags,
         const string& name = "",
         const string& title = "" );
 
@@ -164,7 +166,7 @@ public:
         CGff2Record::TAttrCit,
         CRef< CSeq_feat > );
   
-    bool x_FeatureSetLocation(
+    virtual bool x_FeatureSetLocation(
         const CGff2Record&,
         CRef< CSeq_feat > );
     
@@ -213,7 +215,7 @@ public:
         const string& );
 
     CErrorContainerLenient m_ErrorsPrivate;
-    map< string, CRef< CSeq_feat > > m_MapIdToFeature;
+    IdToFeatureMap m_MapIdToFeature;
 
     static string GenbankKey(
         CSeqFeatData::ESubtype );
@@ -266,7 +268,7 @@ protected:
     //  data:
     //
 protected:
-    TFlags             m_uFlags;
+//    TFlags             m_uFlags;
     IErrorContainer*   m_pErrors;
     CRef< CAnnotdesc > m_CurrentTrackInfo;
     CRef< CAnnotdesc > m_CurrentBrowserInfo;
