@@ -1,4 +1,4 @@
-/*  $Id: gff3_sofa.hpp 340818 2011-10-13 11:43:21Z ludwigf $
+/*  $Id: gff3_sofa.hpp 369200 2012-07-17 15:33:42Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,9 +49,8 @@ class NCBI_XOBJREAD_EXPORT CGff3SofaTypes
 {
     friend CGff3SofaTypes& SofaTypes();
 
-protected:
-    CGff3SofaTypes();
 public:
+    CGff3SofaTypes();
     ~CGff3SofaTypes();
 
     CSeqFeatData::ESubtype MapSofaTermToGenbankType(
@@ -61,7 +60,7 @@ public:
         const string& );
 
 protected:
-    static TLookupSofaToGenbank m_Lookup;
+    static CSafeStaticPtr<TLookupSofaToGenbank> m_Lookup;
 };
 
 //  ----------------------------------------------------------------------------
