@@ -1,4 +1,4 @@
-/*  $Id: microarray_reader.hpp 167193 2009-07-30 12:40:57Z ludwigf $
+/*  $Id: microarray_reader.hpp 332908 2011-08-31 14:52:41Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -106,7 +106,7 @@ protected:
     //
 protected:
     bool m_usescore;
-    int m_flags;
+//    int m_flags;
     string m_strExpNames;
     int m_iExpScale;
     int m_iExpStep;
