@@ -1,7 +1,7 @@
 #ifndef MEMBER__HPP
 #define MEMBER__HPP
 
-/*  $Id: member.hpp 282780 2011-05-16 16:02:27Z gouriano $
+/*  $Id: member.hpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -160,9 +160,7 @@ public:
     void SetPathWriteHook(CObjectOStream* out, const string& path,
                           CWriteClassMemberHook* hook);
 
-    void SetGlobalSkipHook(CSkipClassMemberHook* hook);
     void SetLocalSkipHook(CObjectIStream& in, CSkipClassMemberHook* hook);
-    void ResetGlobalSkipHook(void);
     void ResetLocalSkipHook(CObjectIStream& in);
     void SetPathSkipHook(CObjectIStream* in, const string& path,
                          CSkipClassMemberHook* hook);
