@@ -1,7 +1,7 @@
 #ifndef OBJSTACK__HPP
 #define OBJSTACK__HPP
 
-/*  $Id: objstack.hpp 347300 2011-12-15 19:22:48Z vasilche $
+/*  $Id: objstack.hpp 361377 2012-05-01 17:30:53Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -141,6 +141,8 @@ public:
     void ClearStack(void);
 
     string GetStackTraceASN(void) const;
+    virtual string GetStackTrace(void) const = 0;
+    virtual string GetPosition(void) const = 0;
 
     const TFrame& TopFrame(void) const;
     TFrame& TopFrame(void);
