@@ -1,7 +1,7 @@
 #ifndef VARIANT__HPP
 #define VARIANT__HPP
 
-/*  $Id: variant.hpp 347300 2011-12-15 19:22:48Z vasilche $
+/*  $Id: variant.hpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -145,9 +145,7 @@ public:
     void SetPathWriteHook(CObjectOStream* out, const string& path,
                           CWriteChoiceVariantHook* hook);
 
-    void SetGlobalSkipHook(CSkipChoiceVariantHook* hook);
     void SetLocalSkipHook(CObjectIStream& in, CSkipChoiceVariantHook* hook);
-    void ResetGlobalSkipHook(void);
     void ResetLocalSkipHook(CObjectIStream& in);
     void SetPathSkipHook(CObjectIStream* in, const string& path,
                          CSkipChoiceVariantHook* hook);
