@@ -1,7 +1,7 @@
 #ifndef OBJECTITER__HPP
 #define OBJECTITER__HPP
 
-/*  $Id: objectiter.hpp 327855 2011-07-29 12:38:44Z gouriano $
+/*  $Id: objectiter.hpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -265,8 +265,7 @@ public:
 
     void SetLocalReadHook(CObjectIStream& stream,
                           CReadClassMemberHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalReadHook(CReadClassMemberHook* hook) const;
+    void SetGlobalReadHook(CReadClassMemberHook* hook) const;
     void ResetLocalReadHook(CObjectIStream& stream) const;
     void ResetGlobalReadHook(void) const;
     void SetPathReadHook(CObjectIStream* in, const string& path,
@@ -274,8 +273,7 @@ public:
 
     void SetLocalWriteHook(CObjectOStream& stream,
                           CWriteClassMemberHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalWriteHook(CWriteClassMemberHook* hook) const;
+    void SetGlobalWriteHook(CWriteClassMemberHook* hook) const;
     void ResetLocalWriteHook(CObjectOStream& stream) const;
     void ResetGlobalWriteHook(void) const;
     void SetPathWriteHook(CObjectOStream* stream, const string& path,
@@ -283,17 +281,13 @@ public:
 
     void SetLocalSkipHook(CObjectIStream& stream,
                           CSkipClassMemberHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalSkipHook(CSkipClassMemberHook* hook) const;
     void ResetLocalSkipHook(CObjectIStream& stream) const;
-    void ResetGlobalSkipHook(void) const;
     void SetPathSkipHook(CObjectIStream* stream, const string& path,
                          CSkipClassMemberHook* hook) const;
 
     void SetLocalCopyHook(CObjectStreamCopier& stream,
                           CCopyClassMemberHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalCopyHook(CCopyClassMemberHook* hook) const;
+    void SetGlobalCopyHook(CCopyClassMemberHook* hook) const;
     void ResetLocalCopyHook(CObjectStreamCopier& stream) const;
     void ResetGlobalCopyHook(void) const;
     void SetPathCopyHook(CObjectStreamCopier* stream, const string& path,
@@ -346,8 +340,7 @@ public:
 
     void SetLocalReadHook(CObjectIStream& stream,
                           CReadChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalReadHook(CReadChoiceVariantHook* hook) const;
+    void SetGlobalReadHook(CReadChoiceVariantHook* hook) const;
     void ResetLocalReadHook(CObjectIStream& stream) const;
     void ResetGlobalReadHook(void) const;
     void SetPathReadHook(CObjectIStream* stream, const string& path,
@@ -355,8 +348,7 @@ public:
 
     void SetLocalWriteHook(CObjectOStream& stream,
                           CWriteChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalWriteHook(CWriteChoiceVariantHook* hook) const;
+    void SetGlobalWriteHook(CWriteChoiceVariantHook* hook) const;
     void ResetLocalWriteHook(CObjectOStream& stream) const;
     void ResetGlobalWriteHook(void) const;
     void SetPathWriteHook(CObjectOStream* stream, const string& path,
@@ -364,17 +356,13 @@ public:
 
     void SetLocalSkipHook(CObjectIStream& stream,
                           CSkipChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalSkipHook(CSkipChoiceVariantHook* hook) const;
     void ResetLocalSkipHook(CObjectIStream& stream) const;
-    void ResetGlobalSkipHook(void) const;
     void SetPathSkipHook(CObjectIStream* stream, const string& path,
                          CSkipChoiceVariantHook* hook) const;
 
     void SetLocalCopyHook(CObjectStreamCopier& stream,
                           CCopyChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalCopyHook(CCopyChoiceVariantHook* hook) const;
+    void SetGlobalCopyHook(CCopyChoiceVariantHook* hook) const;
     void ResetLocalCopyHook(CObjectStreamCopier& stream) const;
     void ResetGlobalCopyHook(void) const;
     void SetPathCopyHook(CObjectStreamCopier* stream, const string& path,
@@ -512,8 +500,7 @@ public:
 
     void SetLocalReadHook(CObjectIStream& stream,
                           CReadChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalReadHook(CReadChoiceVariantHook* hook) const;
+    void SetGlobalReadHook(CReadChoiceVariantHook* hook) const;
     void ResetLocalReadHook(CObjectIStream& stream) const;
     void ResetGlobalReadHook(void) const;
     void SetPathReadHook(CObjectIStream* stream, const string& path,
@@ -521,8 +508,7 @@ public:
 
     void SetLocalWriteHook(CObjectOStream& stream,
                           CWriteChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalWriteHook(CWriteChoiceVariantHook* hook) const;
+    void SetGlobalWriteHook(CWriteChoiceVariantHook* hook) const;
     void ResetLocalWriteHook(CObjectOStream& stream) const;
     void ResetGlobalWriteHook(void) const;
     void SetPathWriteHook(CObjectOStream* stream, const string& path,
@@ -530,8 +516,7 @@ public:
 
     void SetLocalCopyHook(CObjectStreamCopier& stream,
                           CCopyChoiceVariantHook* hook) const;
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalCopyHook(CCopyChoiceVariantHook* hook) const;
+    void SetGlobalCopyHook(CCopyChoiceVariantHook* hook) const;
     void ResetLocalCopyHook(CObjectStreamCopier& stream) const;
     void ResetGlobalCopyHook(void) const;
     void SetPathCopyHook(CObjectStreamCopier* stream, const string& path,
