@@ -1,7 +1,7 @@
 #ifndef SERIAL___OBJISTR__HPP
 #define SERIAL___OBJISTR__HPP
 
-/*  $Id: objistr.hpp 363042 2012-05-11 19:25:07Z rafanovi $
+/*  $Id: objistr.hpp 381682 2012-11-27 20:30:49Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -319,6 +319,8 @@ public:
     ///
     /// @param skip
     ///   Skip unknown choice variants parameter
+    /// @note
+    ///   Skipping unknown variants can result in invalid object - with unset choice
     void SetSkipUnknownVariants(ESerialSkipUnknown skip);
 
     /// Get skip unknown choice variants parameter
@@ -1026,12 +1028,8 @@ private:
     static CRef<CByteSource> GetSource(CNcbiIstream& inStream,
                                        bool deleteInStream = false);
 
-    static ESerialVerifyData ms_VerifyDataDefault;
-    static ESerialVerifyData x_GetVerifyDataDefault(void);
-
-    static ESerialSkipUnknown ms_SkipUnknownDefault;
+    static ESerialVerifyData  x_GetVerifyDataDefault(void);
     static ESerialSkipUnknown x_GetSkipUnknownDefault(void);
-
     static ESerialSkipUnknown x_GetSkipUnknownVariantsDefault(void);
 
 
