@@ -1,7 +1,7 @@
 #ifndef TYPEINFO__HPP
 #define TYPEINFO__HPP
 
-/*  $Id: typeinfo.hpp 339061 2011-09-26 14:09:07Z gouriano $
+/*  $Id: typeinfo.hpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -205,12 +205,8 @@ public:
     void SetPathWriteHook(CObjectOStream* out, const string& path,
                           CWriteObjectHook* hook);
 
-    /// Set global (for all input streams) skip hook
-    void SetGlobalSkipHook(CSkipObjectHook* hook);
     /// Set local (for a specific input stream) skip hook
     void SetLocalSkipHook(CObjectIStream& in, CSkipObjectHook* hook);
-    /// Reset global skip hooks
-    void ResetGlobalSkipHook(void);
     /// Reset local skip hook
     void ResetLocalSkipHook(CObjectIStream& in);
     /// Set local context-specific skip hook
