@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__TAR__HPP
 #define UTIL_COMPRESS__TAR__HPP
 
-/* $Id: tar.hpp 354259 2012-02-24 15:23:57Z lavr $
+/* $Id: tar.hpp 370846 2012-08-02 15:10:28Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -267,7 +267,7 @@ public:
     enum EFlags {
         // --- Extract/List/Test ---
         /// Ignore blocks of zeros in archive.
-        //  Generally, 2 or more consecutive zero blocks indicate EOF.
+        //  Generally, 2 or more consecutive zero blocks indicate EOT.
         fIgnoreZeroBlocks   = (1<<1),
 
         // --- Extract/Append/Update ---
@@ -293,7 +293,6 @@ public:
         fPreserveAll        = fPreserveOwner | fPreserveMode | fPreserveTime,
 
         // --- Extract/List ---
-        fMaskNocase         = (1<<10),
         /// Skip unsupported entries rather than making files out of them
         /// when extracting (the latter is the default POSIX requirement).
         fSkipUnsupported    = (1<<15),
@@ -303,7 +302,7 @@ public:
         fSlowSkipWithRead   = (1<<21),
 
         // --- Miscellaneous ---
-        /// Stream pipes tar data through
+        /// Stream tar data through
         fStreamPipeThrough  = (1<<24),
         /// Do not trim tar file size after append/update
         fTarfileNoTruncate  = (1<<26),
@@ -315,6 +314,19 @@ public:
     };
     typedef unsigned int TFlags;  ///< Bitwise OR of EFlags
 
+    /// Mask type enumerator.
+    /// @enum eExtractMask
+    ///   CMask can select both inclusions and exclusions (in this order) of
+    ///   fully-qualified archive entries for list or extraction, so that e.g.
+    ///   the mask ".svn" does not match an entry like "a/.svn" for processing.
+    /// @enum eExcludeMask
+    ///   CMask can select both exclusions and inclusions (in this order) of
+    ///   patterns of archive entries for all operations (excepting eTest),
+    ///   and so the mask ".svn" matches "a/b/c/.svn" for processing.
+    enum EMaskType {
+        eExtractMask = 0,  ///< for list or extract
+        eExcludeMask       ///< applied as a pattern
+    };
 
     /// Constructors
     CTar(const string& filename, size_t blocking_factor = 20);
@@ -373,6 +385,7 @@ public:
     /// have only forward slashes in the paths, and drive letter, if any on
     /// MS-Windows, stripped).  All entries will be added at the logical end
     /// (not always EOF) of the archive, when appending to a non-empty one.
+    ///
     /// @note Adding to a stream archive does not seek to the logical end of
     /// the archive but begins at the current position right away.
     ///
@@ -382,7 +395,7 @@ public:
     ///   Create, Update, SetBaseDir
     auto_ptr<TEntries> Append(const string& name);
 
-    /// Append an entry from stream (exactly entry.GetSize() bytes)
+    /// Append an entry from a stream (exactly entry.GetSize() bytes).
     /// @return
     ///   A list (containing one entry) with full acrhive info filled in
     /// @sa
@@ -392,12 +405,12 @@ public:
     /// Look whether more recent copies of archive members are available in
     /// the file system, and if so, append them to the archive:
     ///
-    /// if fUpdate is set in processing flags, only the existing archive
+    /// - if fUpdate is set in processing flags, only the existing archive
     /// entries (including directories) will be updated;  that is, Update(".")
     /// won't recursively add "." if "." is not an archive member;  it will,
-    /// however, do the recursive update should "." be found in the archive.
+    /// however, do the recursive update should "." be found in the archive;
     ///
-    /// if fUpdate is unset, the existing entries will be updated (if their
+    /// - if fUpdate is unset, the existing entries will be updated (if their
     /// file system counterparts are newer), and inexistent entries will be
     /// added to the archive;  that is, Update(".") will recursively scan "."
     /// to update both existing entries (if newer files found), and also add
@@ -416,6 +429,8 @@ public:
     /// a directory otherwise specified by SetBaseDir()).
     ///
     /// Extract all archive entries, whose names match the pre-set mask.
+    /// @return
+    ///   A list of entries that have been actually extracted.
     /// @sa
     ///   SetMask, SetBaseDir
     auto_ptr<TEntries> Extract(void);
@@ -433,7 +448,7 @@ public:
     ///
     /// Read through the archive without actually extracting anything from it.
     /// Flag fDumpEntryHeaders causes most of archive headers to be dumped to
-    /// the log (as eDiag_Info) as Test() advances through the archive.
+    /// the log (as eDiag_Info) as the Test() advances through the archive.
     /// @sa
     ///   SetFlags
     void Test(void);
@@ -449,30 +464,27 @@ public:
     /// Set processing flags.
     void   SetFlags(TFlags flags);
 
+    /// Get current stream position.
+    Uint8  GetCurrentPosition(void) const;
+
     /// Set name mask.
     ///
     /// The set of masks is used to process existing entries in the archive,
     /// and apply to list and extract operations only.
     /// If masks are not defined then all archive entries will be processed.
-    /// By default, the masks are used case sensitively.  To cancel this and
-    /// use the masks case-insensitively, SetFlags() can be called with
-    /// fMaskNocase flag set.
-    /// @note Unset mask means wildcard processing (all entries match).
-    /// @param mask
-    ///   Set of masks.
-    /// @param own
-    ///   Flag to take ownership on the masks (delete upon CTar destruction).
-    /// @sa
-    //    SetFlags, UnsetMask
-    void SetMask(CMask* mask, EOwnership own = eNoOwnership);
-
-    /// Unset name mask.
     ///
-    /// Upon mask reset, all entries become subject to archive processing in
-    /// the list and extract operations.
+    /// @note Unset mask means wildcard processing (all entries match).
+    ///
+    /// @param mask
+    ///   Set of masks (0 to unset the current set without setting a new one).
+    /// @param own
+    ///   Whether to take ownership on the mask (delete upon CTar destruction).
     /// @sa
-    ///   SetMask
-    void UnsetMask(void);
+    //    SetFlags
+    void SetMask(CMask*      mask,
+                 EOwnership  own   = eNoOwnership,
+                 EMaskType   type  = eExtractMask,
+                 NStr::ECase acase = NStr::eNocase);
 
     /// Get base directory to use for files while extracting from/adding to
     /// the archive, and in the latter case used only for relative paths.
@@ -489,7 +501,7 @@ public:
     /// Return archive size as if all specified input entries were put in it.
     /// Note that the return value is not the exact but the upper bound of
     /// what the archive size can be expected.  This call does not recurse
-    /// into any subdirectries but relies solely upon the information as
+    /// into any subdirectories but relies solely upon the information as
     /// passed via the parameter.
     ///
     /// The returned size includes all necessary alignments and padding.
@@ -548,6 +560,7 @@ public:
     /// CTarEntryInfo::GetPosition(ePos_Header)'s result possibly off-set
     /// with some fixed archive base position, e.g. if there is any preamble).
     /// The extraction is done at the first matching entry only, then stops.
+    /// @note fStreamPipeThrough will be ignored if passed in flags.
     /// See test suite (in test/test_tar.cpp) for a usage example.
     /// @return
     ///   IReader interface to read the file contents with;  0 on error.
@@ -598,6 +611,16 @@ private:
         eZeroBlock,
         eEOF
     };
+    /// Mask storage
+    struct SMask {
+        CMask*      mask;
+        NStr::ECase acase;
+        EOwnership  owned;
+
+        SMask(void)
+            : mask(0), acase(NStr::eNocase), owned(eNoOwnership)
+        { }
+    };
 
     // Common part of initialization.
     void x_Init(void);
@@ -606,12 +629,12 @@ private:
     void x_Open(EAction action);
     void x_Close(bool truncate);  // NB: "truncate" effects file archives only
 
-    // Flush the archive (adding EOT) if necessary;  return "true" if done
+    // Flush the archive (w/EOT);  return "true" if it is okay to truncate
     bool x_Flush(bool nothrow = false);
 
-    // Backspace and skip the archive (in terms of blocks).
-    void x_Backspace(EAction action, Uint8 blocks);
-    void x_Skip(Uint8 blocks);  // Can do either skip or read
+    // Backspace and skip the archive.
+    void x_Backspace(EAction action);  // NB: m_ZeroBlockCount blocks back
+    void x_Skip(Uint8 blocks);         // NB: Can do by either skip or read
 
     // Parse in extended entry information (POSIX) for the current entry.
     EStatus x_ParsePAXData(const string& buffer);
@@ -629,7 +652,7 @@ private:
     auto_ptr<TEntries> x_ReadAndProcess(EAction action);
 
     // Process current entry from the archive (the actual size passed in).
-    // If extract == FALSE, then just skip the entry without any processing.
+    // If "extract" is FALSE, then just skip the entry without any processing.
     bool x_ProcessEntry(bool extract, Uint8 size, const TEntries* done);
 
     // Extract current entry (the actual size passed in) from the archive into
@@ -638,9 +661,9 @@ private:
                         const CDirEntry* dst, const CDirEntry* src);
 
     // Restore attributes of an entry in the file system.
-    // If 'path' not specified, then the destination path will be constructed
-    // from 'info', and the base directory (if any).  Otherwise, 'path' will
-    // be used "as is", assuming it corresponds to the specified 'info'.
+    // If "path" is not specified, then the destination path will be
+    // constructed from "info", and the base directory (if any).  Otherwise,
+    // "path" will be used "as is", assuming it corresponds to "info".
     void x_RestoreAttrs(const CTarEntryInfo& info,
                         TFlags               what,
                         const CDirEntry*     path = 0,
@@ -653,32 +676,32 @@ private:
     // Append an entry from the file system to the archive.
     auto_ptr<TEntries> x_Append(const string& name, const TEntries* toc = 0);
 
-    // Append an entry from istream to the archive.
+    // Append an entry from an istream to the archive.
     auto_ptr<TEntries> x_Append(const CTarUserEntryInfo& entry, istream& is);
 
-    // Append data from istream to the archive.
+    // Append data from an istream to the archive.
     void x_AppendStream(const string& name, istream& is);
 
     // Append a regular file to the archive.
     void x_AppendFile(const string& file);
 
 private:
-    string        m_FileName;     ///< Tar archive file name.
-    CNcbiFstream* m_FileStream;   ///< File stream of the archive.
-    CNcbiIos&     m_Stream;       ///< Archive stream (used for all I/O).
-    const size_t  m_BufferSize;   ///< Buffer(record) size for I/O operations.
-    size_t        m_BufferPos;    ///< Position within the record.
-    Uint8         m_StreamPos;    ///< Position in stream (0-based).
-    char*         m_BufPtr;       ///< Page unaligned buffer pointer.
-    char*         m_Buffer;       ///< I/O buffer (page-aligned).
-    CMask*        m_Mask;         ///< Masks for list/test/extract.
-    EOwnership    m_MaskOwned;    ///< Flag of m_Mask's ownership.
-    EOpenMode     m_OpenMode;     ///< What was it opened for.
-    bool          m_Modified;     ///< True after at least one write.
-    bool          m_Bad;          ///< True if a fatal output error occurred.
-    TFlags        m_Flags;        ///< Bitwise OR of flags.
-    string        m_BaseDir;      ///< Base directory for relative paths.
-    CTarEntryInfo m_Current;      ///< Current entry being processed.
+    string        m_FileName;       ///< Tar archive file name (only if file)
+    CNcbiFstream* m_FileStream;     ///< File stream of the archive (if file)
+    CNcbiIos&     m_Stream;         ///< Archive stream (used for all I/O)
+    size_t        m_ZeroBlockCount; ///< Zero blocks seen in between entries
+    const size_t  m_BufferSize;     ///< Buffer(record) size for I/O operations
+    size_t        m_BufferPos;      ///< Position within the record
+    Uint8         m_StreamPos;      ///< Position in stream (0-based)
+    char*         m_BufPtr;         ///< Page-unaligned buffer pointer
+    char*         m_Buffer;         ///< I/O buffer (page-aligned)
+    SMask         m_Mask[2];        ///< Entry masks for operations
+    EOpenMode     m_OpenMode;       ///< What was it opened for
+    bool          m_Modified;       ///< True after at least one write
+    bool          m_Bad;            ///< True if a fatal output error occurred
+    TFlags        m_Flags;          ///< Bitwise OR of flags
+    string        m_BaseDir;        ///< Base directory for relative paths
+    CTarEntryInfo m_Current;        ///< Current entry being processed
 
 private:
     // Prohibit assignment and copy
@@ -754,21 +777,9 @@ void CTar::SetFlags(TFlags flags)
     m_Flags = flags;
 }
 
-inline
-void CTar::SetMask(CMask* mask, EOwnership own)
+inline Uint8 CTar::GetCurrentPosition(void) const
 {
-    UnsetMask();
-    m_Mask      = mask;
-    m_MaskOwned = own;
-}
-
-inline
-void CTar::UnsetMask(void)
-{
-    if ( m_MaskOwned ) {
-        delete m_Mask;
-    }
-    m_Mask = 0;
+    return m_StreamPos;
 }
 
 inline
