@@ -1,7 +1,7 @@
 #ifndef FORMATGUESS__HPP
 #define FORMATGUESS__HPP
 
-/*  $Id: format_guess.hpp 359371 2012-04-12 16:38:52Z vakatov $
+/*  $Id: format_guess.hpp 375160 2012-09-17 19:29:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -316,12 +316,16 @@ private:
 
     bool x_TestFormat(EFormat format, EMode mode);
 
+    // to test for a table we check each of the most common delimiter combitions,
+    // ' ' ' \t' '\t' ',' '|'
+    bool x_TestTableDelimiter(const string& delims);
+
     // data:
     static const char* const sm_FormatNames[eFormat_max];
 protected:
     static int s_CheckOrder[];
     
-    static const streamsize s_iTestBufferSize = 1024;
+    static const streamsize s_iTestBufferSize = 4096;
 
     CNcbiIstream& m_Stream;
     bool m_bOwnsStream;
