@@ -1,7 +1,7 @@
 #ifndef UTIL___VALUE_CONV_POLICY__HPP
 #define UTIL___VALUE_CONV_POLICY__HPP
 
-/* $Id: value_convert_policy.hpp 363039 2012-05-11 19:24:09Z rafanovi $
+/* $Id: value_convert_policy.hpp 380343 2012-11-09 21:09:27Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NTOICE
@@ -159,8 +159,9 @@ struct SNumericLimits : public std::numeric_limits<T>
 {
     static T min(void)
     {
-        return std::numeric_limits<T>::min() > 0
-            ? T(-std::numeric_limits<T>::max()) : std::numeric_limits<T>::min();
+        typedef std::numeric_limits<T> TBase;
+        return (( !TBase::is_integer  ||  TBase::min() > 0)
+                ? T(-TBase::max()) : TBase::min());
     }
 }; 
 
