@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_exec_guard.sh 117476 2008-01-16 16:44:07Z ucko $
+# $Id: check_exec_guard.sh 279760 2011-05-02 18:55:38Z ucko $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -32,7 +32,7 @@ while [ $timeout -gt 0 ]; do
     sleep_time=$timeout
   fi
   timeout="`expr $timeout - $sleep_time`"
-  sleep $sleep_time
+  sleep $sleep_time >/dev/null 2>&1
 done
 
 # Time out, kill the process
@@ -41,7 +41,7 @@ echo "Maximum execution time of $1 seconds is exceeded"
 echo
 
 kill $pid > /dev/null 2>&1
-sleep $sleep_time
+sleep $sleep_time >/dev/null 2>&1
 kill -9 $pid > /dev/null 2>&1
 
 exit 1
