@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix.sh 363590 2012-05-17 17:08:32Z rafanovi $
+# $Id: check_make_unix.sh 385873 2013-01-14 19:17:51Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -312,6 +312,9 @@ fi
 # Valgrind configuration
 VALGRIND_SUP="\${root_dir}/scripts/common/check/valgrind.supp"
 VALGRIND_CMD="--tool=memcheck --suppressions=\$VALGRIND_SUP"
+if valgrind --ncbi --help >/dev/null 2>&1; then                                
+    VALGRIND_CMD="--ncbi \$VALGRIND_CMD" # --ncbi must be the first option!
+fi                                                                             
 
 # Export some global vars
 top_srcdir="\$root_dir"
@@ -373,6 +376,16 @@ EOF
 else
    echo "WARNING:  Cannot find path to the library dir."
 fi
+# Add additional path for imported projects to point to local /lib first
+if test "$x_import_prj" = "yes"; then
+    local_lib=`(cd "$x_compile_dir/../lib"; pwd | sed -e 's/\/$//g')`
+    if test -n "$local_lib"  -a  -d "$local_lib";  then
+   cat >> $x_out <<EOF
+COMMON_AddRunpath "$local_lib"
+EOF
+    fi
+fi
+
 
 cat >> $x_out <<EOF
 
@@ -390,7 +403,7 @@ fi
 
 if test "\$NCBI_CHECK_SETLIMITS" != "0"; then
    ulimit -c 1000000
-   ulimit -v 2000000
+   ulimit -v 4000000
 fi
 
 
