@@ -1,6 +1,6 @@
 #! /bin/sh
 #################################
-# $Id: if_diff.sh 327756 2011-07-28 18:28:10Z ucko $
+# $Id: if_diff.sh 358078 2012-03-28 19:59:25Z ucko $
 # Author:  Denis Vakatov (vakatov@ncbi.nlm.nih.gov)
 #################################
 
@@ -54,7 +54,9 @@ ExecHelper()
   test "$quiet" = yes || echo "$cmd"
   PATH=$orig_PATH
   "$@" "$dest"
+  status=$?
   PATH=/bin:/usr/bin
+  return $status
 }
 
 ExecAction()
