@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_aux_priv.cpp 214207 2010-12-02 16:11:26Z maning $";
+    "$Id: blast_aux_priv.cpp 354756 2012-02-29 17:40:28Z morgulis $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
@@ -145,7 +145,7 @@ BlastSetupPreliminarySearchEx(CRef<IQueryFactory> qf,
 
     // 0. Initialize the megablast database index.
     if (options->GetUseIndex()) {
-        CSetupFactory::InitializeMegablastDbIndex(seqsrc, options);
+        CSetupFactory::InitializeMegablastDbIndex(options);
     }
 
     // 1. Initialize the query data (borrow it from the factory)
