@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_objmgr_tools.cpp 350677 2012-01-23 19:46:06Z maning $";
+    "$Id: blast_objmgr_tools.cpp 390750 2013-03-01 18:22:37Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
@@ -36,7 +36,7 @@ static char const rcsid[] =
 
 #include <ncbi_pch.hpp>
 #include <objmgr/seq_vector.hpp>
-#include <objmgr/util/sequence.hpp>
+#include <objmgr/util/create_defline.hpp>
 #include <objects/seqalign/seqalign__.hpp>
 #include <objects/seqloc/seqloc__.hpp>
 
@@ -351,8 +351,12 @@ CBlastQuerySourceOM::GetTitle(int i) const
     }
     _ASSERT(seqloc.NotEmpty());
     _ASSERT(scope.NotEmpty());
-    CBioseq_Handle bh = scope->GetBioseqHandle(*seqloc);
-    return bh ? sequence::GetTitle(bh) : kEmptyStr;
+    if ( !seqloc->GetId() ) {
+        return kEmptyStr;
+    }
+
+    CBioseq_Handle bh = scope->GetBioseqHandle(*seqloc->GetId());
+    return bh ? sequence::CDeflineGenerator().GenerateDefline(bh) : kEmptyStr;
 }
 
 TSeqPos
@@ -420,7 +424,12 @@ public:
     virtual void GetStrandData(objects::ENa_strand strand, unsigned char* buf) {
         x_FixStrand(strand);
         for (CSeqVector_CI itr(m_SeqVector, strand); itr; ++itr) {
-            *buf++ = *itr;
+            // treat gap '-' as 'N'
+            if (itr.IsInGap()) {
+                *buf++ = 0xf;
+            } else {
+                *buf++ = *itr;
+            }
         }
     }
 
