@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_options_local_priv.cpp 167759 2009-08-06 14:28:02Z avagyanv $";
+    "$Id: blast_options_local_priv.cpp 347205 2011-12-14 20:08:44Z boratyng $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
@@ -64,6 +64,7 @@ CBlastOptionsLocal::CBlastOptionsLocal()
     BlastEffectiveLengthsOptionsNew(&m_EffLenOpts);
     BlastDatabaseOptionsNew(&m_DbOpts);
     PSIBlastOptionsNew(&m_PSIBlastOpts);
+    PSIBlastOptionsNew(&m_DeltaBlastOpts);
     m_Program = eBlastNotSet;
     m_UseMBIndex = false;
     m_ForceMBIndex = false;
@@ -399,6 +400,7 @@ CBlastOptionsLocal::DebugDump(CDebugDumpContext ddc, unsigned int depth) const
     m_ExtnOpts.DebugDump(ddc, depth);
     m_HitSaveOpts.DebugDump(ddc, depth);
     m_PSIBlastOpts.DebugDump(ddc, depth);
+    m_DeltaBlastOpts.DebugDump(ddc, depth);
     m_DbOpts.DebugDump(ddc, depth);
     m_ScoringOpts.DebugDump(ddc, depth);
     m_EffLenOpts.DebugDump(ddc, depth);
@@ -471,6 +473,8 @@ x_BlastScoringOptions_cmp(const BlastScoringOptions* a,
     if (a->reward != b->reward) return false;
     if (a->penalty != b->penalty) return false;
     if (a->gapped_calculation != b->gapped_calculation) return false;
+    // Added to support complexity adjusted scoring in RMBlastN -RMH-
+    if (a->complexity_adjusted_scoring != b->complexity_adjusted_scoring) return false;
     if (a->gap_open != b->gap_open) return false;
     if (a->gap_extend != b->gap_extend) return false;
     if (a->is_ooframe != b->is_ooframe) return false;
@@ -530,6 +534,11 @@ CBlastOptionsLocal::operator==(const CBlastOptionsLocal& rhs) const
     if ( x_safe_memcmp(a, b, sizeof(PSIBlastOptions)) != 0 )
          return false;
 
+    a = static_cast<void*>( (PSIBlastOptions*) m_DeltaBlastOpts);
+    b = static_cast<void*>( (PSIBlastOptions*) rhs.m_DeltaBlastOpts);
+    if ( x_safe_memcmp(a, b, sizeof(PSIBlastOptions)) != 0 )
+         return false;
+
     if ( !x_BlastDatabaseOptions_cmp(m_DbOpts, rhs.m_DbOpts) )
         return false;
 
