@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: local_blast.cpp 303807 2011-06-13 18:22:23Z camacho $";
+    "$Id: local_blast.cpp 388612 2013-02-08 20:29:41Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -42,6 +42,7 @@ static char const rcsid[] =
 #include "blast_aux_priv.hpp"
 #include <objects/scoremat/PssmWithParameters.hpp>
 #include <algo/blast/api/seqinfosrc_seqdb.hpp>
+#include <algo/blast/api/blast_dbindex.hpp>
 
 /** @addtogroup AlgoBlast
  *
@@ -52,6 +53,58 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 BEGIN_SCOPE(blast)
 
+size_t
+SplitQuery_GetChunkSize(EProgram program)
+{
+    size_t retval = 0;
+
+    // used for experimentation purposes
+    char* chunk_sz_str = getenv("CHUNK_SIZE");
+    if (chunk_sz_str && !NStr::IsBlank(chunk_sz_str)) {
+        retval = NStr::StringToInt(chunk_sz_str);
+        _TRACE("DEBUG: Using query chunk size " << retval);
+    } else {
+
+        switch (program) {
+        case eBlastn:
+            retval = 1000000;
+            break;
+        case eMegablast:
+        case eDiscMegablast:
+            retval = 5000000;
+            break;
+        case eTblastn:
+            retval = 20000;
+            break;
+        // if the query will be translated, round the chunk size up to the next
+        // multiple of 3, that way, when the nucleotide sequence(s) get(s)
+        // split, context N%6 in one chunk will have the same frame as context
+        // N%6 in the next chunk
+        case eBlastx:
+        case eTblastx:
+            // N.B.: the splitting is done on the nucleotide query sequences,
+            // then each of these chunks is translated
+            retval = 10002;
+            break;
+        case eBlastp:
+        default:
+            retval = 10000;
+            break;
+        }
+
+        _TRACE("Using query chunk size " << retval);
+    }
+
+    const EBlastProgramType prog_type(EProgramToEBlastProgramType(program));
+    if (Blast_QueryIsTranslated(prog_type) && !Blast_SubjectIsPssm(prog_type) &&
+        (retval % CODON_LENGTH) != 0) {
+        NCBI_THROW(CBlastException, eInvalidArgument, 
+                   "Split query chunk size must be divisible by 3");
+    }
+
+    return retval;
+}
+
 CLocalBlast::CLocalBlast(CRef<IQueryFactory> qf,
                          CRef<CBlastOptionsHandle> opts_handle,
                          const CSearchDatabase& dbinfo)
@@ -125,6 +178,8 @@ CLocalBlast::Run()
     try {
         m_PrelimSearch->SetNumberOfThreads(GetNumberOfThreads());
         m_InternalData = m_PrelimSearch->Run();
+    } catch( CIndexedDbException & e ) { 
+        throw;
     } catch (...) {
     }
     if (status != 0)
@@ -194,9 +249,22 @@ CLocalBlast::Run()
     CRef<CSearchResultSet> retval = m_TbackSearch->Run();
     retval->SetFilteredQueryRegions(m_PrelimSearch->GetFilteredQueryRegions());
     m_Messages = m_TbackSearch->GetSearchMessages();
+
     return retval;
 }
 
+Int4 CLocalBlast::GetNumExtensions()
+{
+    Int4 retv = 0;
+    if (m_InternalData) {
+        BlastDiagnostics * diag = m_InternalData->m_Diagnostics->GetPointer();
+        if (diag && diag->ungapped_stat) {
+             retv = diag->ungapped_stat->good_init_extends;
+        }
+    }
+    return retv;
+}
+
 END_SCOPE(blast)
 END_NCBI_SCOPE
 
