@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: psiblast_aux_priv.cpp 161402 2009-05-27 17:35:47Z camacho $";
+    "$Id: psiblast_aux_priv.cpp 327673 2011-07-28 14:30:03Z camacho $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
@@ -469,7 +469,7 @@ void PsiBlastComputePssmScores(CRef<objects::CPssmWithParameters> pssm,
                                const CBlastOptions& opts)
 {
     CConstRef<CBioseq> query(&pssm->GetQuery().GetSeq());
-    CRef<IQueryFactory> seq_fetcher(new CObjMgrFree_QueryFactory(query));
+    CRef<IQueryFactory> seq_fetcher(new CObjMgrFree_QueryFactory(query)); /* NCBI_FAKE_WARNING */
 
     CRef<ILocalQueryData> query_data(seq_fetcher->MakeLocalQueryData(&opts));
     BLAST_SequenceBlk* seqblk = query_data->GetSequenceBlk();
@@ -560,8 +560,9 @@ CPsiBlastAlignmentProcessor::operator()
         // Look for HSP with score less than inclusion_ethresh
         double e = GetLowestEvalue((*hsp)->GetScore());
         if (e < evalue_inclusion_threshold) {
-            CRef<CSeq_id> id(const_cast<CSeq_id*>(&(*hsp)->GetSeq_id(1)));
-            output.insert(id);
+            CSeq_id_Handle sid =
+                CSeq_id_Handle::GetHandle((*hsp)->GetSeq_id(1));
+            output.insert(sid);
         }
     }
 }
