@@ -1,4 +1,4 @@
-/*  $Id: remote_blast.cpp 368805 2012-07-12 16:14:48Z rafanovi $ 
+/*  $Id: remote_blast.cpp 391263 2013-03-06 18:02:05Z rafanovi $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -800,11 +800,9 @@ void CRemoteBlast::x_Init(CNcbiIstream& f)
             m_ObjectStream.reset(new CObjectIStreamAsn(f));
             break;
 
-/* What's up here?
         case CFormatGuess::eXml:
-            m_ObjectStream.reset(new CObjectIStreamXml(f));
+            m_ObjectStream.reset(CObjectIStream::Open(eSerial_Xml, f));
             break;
-*/
 
          default:
             NCBI_THROW(CBlastException, eInvalidArgument,
@@ -1123,7 +1121,7 @@ CRemoteBlast::x_QueryMaskingLocationsToNetwork()
         CRemoteBlast::ConvertToRemoteMasks(m_QueryMaskingLocations,
                                            program, &m_Warn);
     ITERATE(CBlast4_get_search_results_reply::TMasks, itr, network_masks) {
-        x_SetOneParam(B4Param_LCaseMask, *itr);
+        x_SetOneParam(CBlast4Field::Get(eBlastOpt_LCaseMask), *itr);
     }
 
 }
@@ -1311,8 +1309,8 @@ void CRemoteBlast::x_InitQueries(CRef<IQueryFactory> queries)
                 bss->GetSeq_set().front()->GetSeq().GetInst().GetLength();
 
             if (kFullLength != kRangeLength) {
-                x_SetOneParam(B4Param_RequiredStart, &kStart);
-                x_SetOneParam(B4Param_RequiredEnd, &kStop);
+                x_SetOneParam(CBlast4Field::Get(eBlastOpt_RequiredStart), &kStart);
+                x_SetOneParam(CBlast4Field::Get(eBlastOpt_RequiredEnd), &kStop);
             }
         }
     
@@ -1439,7 +1437,7 @@ void CRemoteBlast::SetGIList(const list<Int4> & gi_list)
         NCBI_THROW(CBlastException, eNotSupported, 
            "Submitting gi lists remotely is currently not supported");
     }
-    x_SetOneParam(B4Param_GiList, & gi_list);
+    x_SetOneParam(CBlast4Field::Get(eBlastOpt_GiList), & gi_list);
     
     m_GiList.clear();
     copy(gi_list.begin(), gi_list.end(), back_inserter(m_GiList));
@@ -1450,7 +1448,7 @@ void CRemoteBlast::SetDbFilteringAlgorithmId(int algo_id)
     if (algo_id == -1) 
         return;
 
-    x_SetOneParam(B4Param_DbFilteringAlgorithmId, &algo_id);
+    x_SetOneParam(CBlast4Field::Get(eBlastOpt_DbFilteringAlgorithmId), &algo_id);
     m_DbFilteringAlgorithmId = algo_id;
 }
 
@@ -1462,7 +1460,7 @@ void CRemoteBlast::SetNegativeGIList(const list<Int4> & gi_list)
         NCBI_THROW(CBlastException, eNotSupported, 
            "Submitting negative gi lists remotely is currently not supported");
     }
-    x_SetOneParam(B4Param_NegativeGiList, & gi_list);
+    x_SetOneParam(CBlast4Field::Get(eBlastOpt_NegativeGiList), & gi_list);
     
     m_NegativeGiList.clear();
     copy(gi_list.begin(), gi_list.end(), back_inserter(m_NegativeGiList));
@@ -1547,7 +1545,7 @@ void CRemoteBlast::SetEntrezQuery(const char * x)
     }
     
     if (*x) { // Ignore empty strings.
-        x_SetOneParam(B4Param_EntrezQuery, &x);
+        x_SetOneParam(CBlast4Field::Get(eBlastOpt_EntrezQuery), &x);
         m_EntrezQuery.assign(x);
     }
 }
@@ -1621,8 +1619,8 @@ CRemoteBlast::x_GetRequestInfoFromFile()
         m_Service   = strategy.GetService();
         m_CreatedBy = strategy.GetCreatedBy();
         m_Queries    = strategy.GetQueries();
-        m_AlgoOpts.Reset( & strategy.GetAlgoOptions() );
-        m_ProgramOpts.Reset( & strategy.GetProgramOptions() );
+        m_AlgoOpts.Reset( strategy.GetAlgoOptions() );
+        m_ProgramOpts.Reset( strategy.GetProgramOptions() );
 
         if (strategy.GetSubject()->IsDatabase())
             x_SetDatabase(strategy.GetSubject()->GetDatabase());
@@ -1632,9 +1630,7 @@ CRemoteBlast::x_GetRequestInfoFromFile()
         if(m_Service == "psi")
         {
         	// Would have errored out in CImportStrategy if we can't get queue search
-        	 CBlast4_queue_search_request& qs = request->SetBody().SetQueue_search();
-        	 if(qs.CanGetFormat_options())
-        	        	m_FormatOpts.Reset(&qs.SetFormat_options());
+        	m_FormatOpts.Reset( strategy.GetWebFormatOptions());
         }
         // Ignore return value, want side effect of setting fields.
         GetSearchOptions();
@@ -1924,7 +1920,8 @@ CRef<CBlastOptionsHandle> CRemoteBlast::GetSearchOptions()
         
         CBlastOptionsBuilder bob(program_s, service_s, CBlastOptions::eRemote );
         
-        m_CBOH = bob.GetSearchOptions(m_AlgoOpts, m_ProgramOpts, &m_Task);
+        m_CBOH = bob.GetSearchOptions(m_AlgoOpts, m_ProgramOpts, m_FormatOpts,
+                                      &m_Task);
         
         if (bob.HaveEntrezQuery()) {
             m_EntrezQuery = bob.GetEntrezQuery();
@@ -2254,12 +2251,13 @@ string CRemoteBlast::GetTitle(void)
 {
 	    // Build the request
 	    CRef<CBlast4_request_body> request_body =
-	        s_BuildSearchInfoRequest(m_RID, kBlast4SearchInfoReqName_Search,
-	                                 kBlast4SearchInfoReqValue_Title);
+	        s_BuildSearchInfoRequest(m_RID,
+                                     CTempString(kBlast4SearchInfoReqName_Search),
+	                                 CTempString(kBlast4SearchInfoReqValue_Title));
 	    CRef<CBlast4_reply> reply = x_SendRequest(request_body);
 	    return x_GetStringFromSearchInfoReply(reply,
-	                                          kBlast4SearchInfoReqName_Search,
-	                                          kBlast4SearchInfoReqValue_Title);
+	                                          CTempString(kBlast4SearchInfoReqName_Search),
+	                                          CTempString(kBlast4SearchInfoReqValue_Title));
 
 }
 // Disk Cache version: x_CheckResults
@@ -2447,8 +2445,8 @@ void CRemoteBlast::x_GetSubjects(void)
     // Build the request
     CRef<CBlast4_get_search_info_request> info_request( new CBlast4_get_search_info_request );
     info_request->SetRequest_id( m_RID );
-    info_request->SetInfo().Add(kBlast4SearchInfoReqName_Search, 
-                                kBlast4SearchInfoReqValue_Subjects);
+    info_request->SetInfo().Add(CTempString(kBlast4SearchInfoReqName_Search), 
+                                CTempString(kBlast4SearchInfoReqValue_Subjects));
 
     CRef<CBlast4_request_body> body(new CBlast4_request_body);
     body->SetGet_search_info( *info_request );
@@ -2493,8 +2491,8 @@ void CRemoteBlast::x_GetSubjects(void)
                 if( info_reply.CanGetInfo() ){
                     const CBlast4_parameters &params = info_reply.GetInfo();
                     string reply_name =
-                          Blast4SearchInfo_BuildReplyName(kBlast4SearchInfoReqName_Search,
-                                                          kBlast4SearchInfoReqValue_Subjects);
+                          Blast4SearchInfo_BuildReplyName(CTempString(kBlast4SearchInfoReqName_Search),
+                                                          CTempString(kBlast4SearchInfoReqValue_Subjects));
                     CRef< CBlast4_parameter > search_param = params.GetParamByName (reply_name);
                     // reply could have string, seq-loc-list or 
                     // bioseq-list, but we don't care about string result for bl2seq
@@ -2525,7 +2523,7 @@ unsigned int CRemoteBlast::GetPsiNumberOfIterations(void)
 	unsigned int iter_num = 0;
 	if(!m_FormatOpts.Empty())
 	{
-		CRef< CBlast4_parameter > param = m_FormatOpts->GetParamByName (B4Param_Web_StepNumber.GetName());
+		CRef< CBlast4_parameter > param = m_FormatOpts->GetParamByName (CBlast4Field::GetName(eBlastOpt_Web_StepNumber));
         if( param.NotEmpty())
         {
         	iter_num  = param->GetValue().GetInteger();
@@ -2544,12 +2542,13 @@ unsigned int CRemoteBlast::x_GetPsiIterationsFromServer()
 	unsigned int retval=0;
 
      CRef<CBlast4_request_body> request_body =
-         s_BuildSearchInfoRequest(m_RID, kBlast4SearchInfoReqName_Search,
-                                  kBlast4SearchInfoReqValue_PsiIterationNum);
+         s_BuildSearchInfoRequest(m_RID,
+                                  CTempString(kBlast4SearchInfoReqName_Search),
+                                  CTempString(kBlast4SearchInfoReqValue_PsiIterationNum));
      CRef<CBlast4_reply> reply = x_SendRequest(request_body);
      string num = x_GetStringFromSearchInfoReply(reply,
-                                                 kBlast4SearchInfoReqName_Search,
-                                                 kBlast4SearchInfoReqValue_PsiIterationNum);
+                                                 CTempString(kBlast4SearchInfoReqName_Search),
+                                                 CTempString(kBlast4SearchInfoReqValue_PsiIterationNum));
       if ( !num.empty() ) {
          try { retval = NStr::StringToUInt(num); }
          catch (...) {}  // ignore errors and leave as unset
