@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: rps_aux.cpp 347205 2011-12-14 20:08:44Z boratyng $";
+    "$Id: rps_aux.cpp 382128 2012-12-03 19:47:32Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -458,6 +458,56 @@ CRpsObsrFile::operator()() const
     return m_Data;
 }
 
+/////////////////////////////////////////////////////////////////////////////
+//
+// CRpsFreqRatiosFile
+//
+/////////////////////////////////////////////////////////////////////////////
+
+/// This class represents the .freq file in a RPS-BLAST file, which contains
+/// the frequency ratios for the database
+class CRpsFreqRatiosFile : public CRpsMmappedFile {
+public:
+    /// Extension associated with the RPS-BLAST database PSSM file
+    static const string kExtension;
+
+    /// Parametrized constructor
+    /// @param filename_no_extn name of the file without extension
+    CRpsFreqRatiosFile(const string& filename_no_extn);
+
+    virtual ~CRpsFreqRatiosFile(){};
+
+    /// Lend the caller the pointer to the data structure this object manages.
+    /// Caller MUST NOT deallocate the return value.
+    const BlastRPSFreqRatiosHeader* operator()() const;
+private:
+    /// The data this class manages
+    BlastRPSFreqRatiosHeader* m_Data;
+};
+
+const string CRpsFreqRatiosFile::kExtension(".freq");
+
+CRpsFreqRatiosFile::CRpsFreqRatiosFile(const string& filename_no_extn)
+    : CRpsMmappedFile(filename_no_extn + kExtension), m_Data(NULL)
+{
+    m_Data = (BlastRPSFreqRatiosHeader *) m_MmappedFile->GetPtr();
+    if (m_Data->magic_number != RPS_MAGIC_NUM &&
+            m_Data->magic_number != RPS_MAGIC_NUM_28) {
+            m_Data = NULL;
+            NCBI_THROW(CBlastException, eRpsInit,
+                   "RPS BLAST freq ratios file (" + filename_no_extn + kExtension +
+                   ") is either corrupt or constructed for an incompatible "
+                   "architecture");
+    }
+
+}
+
+const BlastRPSFreqRatiosHeader*
+CRpsFreqRatiosFile::operator()() const
+{
+    return m_Data;
+}
+
 
 CBlastRPSInfo::CBlastRPSInfo(const string& rps_dbname)
 {
@@ -501,6 +551,7 @@ void CBlastRPSInfo::x_Init(const string& rps_dbname, int flags)
     m_RpsInfo->profile_header = NULL;
     m_RpsInfo->freq_header = NULL;
     m_RpsInfo->obsr_header = NULL;
+    m_RpsInfo->freq_ratios_header = NULL;
 
         // Load the various files
     if (flags & fAuxInfoFile) {
@@ -547,6 +598,19 @@ void CBlastRPSInfo::x_Init(const string& rps_dbname, int flags)
         m_RpsInfo->obsr_header =
             const_cast<BlastRPSProfileHeader*>((*m_ObsrFile)());
     }
+
+    if (flags & fFreqRatiosFile) {
+        try {
+            // read frequency ratios data
+            m_FreqRatiosFile.Reset(new CRpsFreqRatiosFile(path));
+        } catch (const CBlastException& e) {
+        	string msg = rps_dbname + " contains no frequency ratios needed for composition-based statistics.\n" \
+        			     "Please disable composition-based statistics when searching against " + rps_dbname + ".";
+            NCBI_RETHROW(e, CBlastException, eRpsInit, msg);
+        }
+        m_RpsInfo->freq_ratios_header =
+            const_cast<BlastRPSFreqRatiosHeader*>((*m_FreqRatiosFile)());
+    }
 }
 
 CBlastRPSInfo::~CBlastRPSInfo()
