@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: setup_factory.cpp 364124 2012-05-22 21:03:27Z rafanovi $";
+    "$Id: setup_factory.cpp 369420 2012-07-19 13:41:19Z boratyng $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -73,7 +73,10 @@ CRef<CBlastRPSInfo>
 CSetupFactory::CreateRpsStructures(const string& rps_dbname,
                                    CRef<CBlastOptions> options)
 {
-    CRef<CBlastRPSInfo> retval(new CBlastRPSInfo(rps_dbname));
+    CBlastRPSInfo::EOpenFlags mode = 
+             (options->GetCompositionBasedStats() == eNoCompositionBasedStats) 
+             ? CBlastRPSInfo::fRpsBlast : CBlastRPSInfo::fRpsBlastWithCBS;
+    CRef<CBlastRPSInfo> retval(new CBlastRPSInfo(rps_dbname, mode));
     options->SetMatrixName(retval->GetMatrixName());
     options->SetGapOpeningCost(retval->GetGapOpeningCost());
     options->SetGapExtensionCost(retval->GetGapExtensionCost());
