@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: split_query_aux_priv.cpp 214207 2010-12-02 16:11:26Z maning $";
+    "$Id: split_query_aux_priv.cpp 388612 2013-02-08 20:29:41Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -73,58 +73,6 @@ SplitQuery_GetOverlapChunkSize(EBlastProgramType program)
     return retval;
 }
 
-size_t
-SplitQuery_GetChunkSize(EProgram program)
-{
-    size_t retval = 0;
-
-    // used for experimentation purposes
-    char* chunk_sz_str = getenv("CHUNK_SIZE");
-    if (chunk_sz_str && !NStr::IsBlank(chunk_sz_str)) {
-        retval = NStr::StringToInt(chunk_sz_str);
-        _TRACE("DEBUG: Using query chunk size " << retval);
-    } else {
-
-        switch (program) {
-        case eBlastn:
-            retval = 1000000;
-            break;
-        case eMegablast:
-        case eDiscMegablast:
-            retval = 5000000;
-            break;
-        case eTblastn:
-            retval = 20000;
-            break;
-        // if the query will be translated, round the chunk size up to the next
-        // multiple of 3, that way, when the nucleotide sequence(s) get(s)
-        // split, context N%6 in one chunk will have the same frame as context
-        // N%6 in the next chunk
-        case eBlastx:
-        case eTblastx:
-            // N.B.: the splitting is done on the nucleotide query sequences,
-            // then each of these chunks is translated
-            retval = 10002;
-            break;
-        case eBlastp:
-        default:
-            retval = 10000;
-            break;
-        }
-
-        _TRACE("Using query chunk size " << retval);
-    }
-
-    const EBlastProgramType prog_type(EProgramToEBlastProgramType(program));
-    if (Blast_QueryIsTranslated(prog_type) && !Blast_SubjectIsPssm(prog_type) &&
-        (retval % CODON_LENGTH) != 0) {
-        NCBI_THROW(CBlastException, eInvalidArgument, 
-                   "Split query chunk size must be divisible by 3");
-    }
-
-    return retval;
-}
-
 bool
 SplitQuery_ShouldSplit(EBlastProgramType program,
                        size_t chunk_size,
