@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: uniform_search.cpp 272713 2011-04-11 14:49:23Z camacho $";
+    "$Id: uniform_search.cpp 372583 2012-08-20 18:02:56Z maning $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -154,7 +154,7 @@ void
 CSearchDatabase::SetFilteringAlgorithm(const string &filt_algorithm,
                                        ESubjectMaskingType mask_type)
 {
-    m_FilteringAlgorithmId = NStr::StringToNumeric(filt_algorithm);
+    m_FilteringAlgorithmId = NStr::StringToInt(filt_algorithm);
     m_MaskType = mask_type;
     if (m_FilteringAlgorithmId < 0) {
         // This is a string id, must translate to numeric id first
