@@ -1,4 +1,4 @@
-/*  $Id: blastn_args.cpp 314890 2011-07-21 19:52:27Z camacho $
+/*  $Id: blastn_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] 
-    = "$Id: blastn_args.cpp 314890 2011-07-21 19:52:27Z camacho $";
+    = "$Id: blastn_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -137,7 +137,8 @@ CBlastnAppArgs::x_CreateOptionsHandle(CBlastOptions::EAPILocality locality,
 int
 CBlastnAppArgs::GetQueryBatchSize() const
 {
-    return blast::GetQueryBatchSize(ProgramNameToEnum(GetTask()), m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(ProgramNameToEnum(GetTask()), m_IsUngapped, is_remote);
 }
 
 END_SCOPE(blast)
