@@ -1,4 +1,4 @@
-/*  $Id: blastp_args.cpp 174882 2009-11-02 16:46:46Z madden $
+/*  $Id: blastp_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blastp_args.cpp 174882 2009-11-02 16:46:46Z madden $";
+static char const rcsid[] = "$Id: blastp_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -132,7 +132,8 @@ CBlastpAppArgs::x_CreateOptionsHandle(CBlastOptions::EAPILocality locality,
 int
 CBlastpAppArgs::GetQueryBatchSize() const
 {
-    return blast::GetQueryBatchSize(eBlastp, m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(eBlastp, m_IsUngapped, is_remote);
 }
 
 END_SCOPE(blast)
