@@ -1,4 +1,4 @@
-/*  $Id: deltablast_args.cpp 354265 2012-02-24 15:53:39Z boratyng $
+/*  $Id: deltablast_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: deltablast_args.cpp 354265 2012-02-24 15:53:39Z boratyng $";
+static char const rcsid[] = "$Id: deltablast_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -57,6 +57,7 @@ CDeltaBlastAppArgs::CDeltaBlastAppArgs()
      bool const kQueryIsProtein = true;
      bool const kFilterByDefault = false;
      bool const kIsDeltaBlast = true;
+     bool const kIsCBS2and3Supported = false;
      m_ClientId = kProgram + " " + CBlastVersion().Print();
 
      static const string kDefaultTask = "deltablast";
@@ -105,7 +106,8 @@ CDeltaBlastAppArgs::CDeltaBlastAppArgs()
      arg.Reset(m_RemoteArgs);
      m_Args.push_back(arg);
 
-     arg.Reset(new CCompositionBasedStatsArgs(kIsDeltaBlast));
+     arg.Reset(new CCompositionBasedStatsArgs(kIsCBS2and3Supported,
+                                              kDfltArgCompBasedStatsDelta));
      m_Args.push_back(arg);
 
      arg.Reset(new CGapTriggerArgs(kQueryIsProtein));
@@ -131,7 +133,8 @@ CDeltaBlastAppArgs::CDeltaBlastAppArgs()
 int
 CDeltaBlastAppArgs::GetQueryBatchSize(void) const
 {
-    return blast::GetQueryBatchSize(eDeltaBlast, m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(eDeltaBlast, m_IsUngapped, is_remote);
 }
 
 CRef<CBlastOptionsHandle>
