@@ -1,4 +1,4 @@
-/*  $Id: psiblast_args.cpp 334322 2011-09-06 14:50:26Z fongah2 $
+/*  $Id: psiblast_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: psiblast_args.cpp 334322 2011-09-06 14:50:26Z fongah2 $";
+static char const rcsid[] = "$Id: psiblast_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -161,7 +161,8 @@ CPsiBlastAppArgs::SetNumberOfIterations(unsigned int num_iters)
 int 
 CPsiBlastAppArgs::GetQueryBatchSize() const
 {
-    return blast::GetQueryBatchSize(ePSIBlast, m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(ePSIBlast, m_IsUngapped, is_remote);
 }
 
 bool
