@@ -1,4 +1,4 @@
-/*  $Id: rpsblast_args.cpp 344511 2011-11-16 13:21:44Z fongah2 $
+/*  $Id: rpsblast_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: rpsblast_args.cpp 344511 2011-11-16 13:21:44Z fongah2 $";
+static char const rcsid[] = "$Id: rpsblast_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -50,6 +50,7 @@ CRPSBlastAppArgs::CRPSBlastAppArgs()
 {
     const bool kQueryIsProtein = true;
     const bool kIsRpsBlast = true;
+    const bool kIsCBS2and3Supported = false;
     CRef<IBlastCmdLineArgs> arg;
     static const string kProgram("rpsblast");
     arg.Reset(new CProgramDescriptionArgs(kProgram,
@@ -100,6 +101,13 @@ CRPSBlastAppArgs::CRPSBlastAppArgs()
     m_DebugArgs.Reset(new CDebugArgs);
     arg.Reset(m_DebugArgs);
     m_Args.push_back(arg);
+
+    string cbs_opt_zero_descr = "Simplified Composition-based statistics as in"
+        " Bioinformatics 15:1000-1011, 1999";
+    arg.Reset(new CCompositionBasedStatsArgs(kIsCBS2and3Supported,
+                                             kDfltArgCompBasedStatsRPS,
+                                             cbs_opt_zero_descr));
+    m_Args.push_back(arg);
 }
 
 CRef<CBlastOptionsHandle> 
@@ -114,7 +122,8 @@ CRPSBlastAppArgs::x_CreateOptionsHandle(CBlastOptions::EAPILocality locality,
 int
 CRPSBlastAppArgs::GetQueryBatchSize() const
 {
-    return blast::GetQueryBatchSize(eRPSBlast, m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(eRPSBlast, m_IsUngapped, is_remote);
 }
 
 END_SCOPE(blast)
