@@ -1,4 +1,4 @@
-/*  $Id: rpstblastn_args.cpp 345047 2011-11-21 19:17:57Z camacho $
+/*  $Id: rpstblastn_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] 
-    = "$Id: rpstblastn_args.cpp 345047 2011-11-21 19:17:57Z camacho $";
+    = "$Id: rpstblastn_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -95,10 +95,9 @@ CRPSTBlastnAppArgs::CRPSTBlastnAppArgs()
     arg.Reset(m_FormattingArgs);
     m_Args.push_back(arg);
 
-    /* RPS blast does not support MT
-    m_MTArgs.Reset(new CMTArgs);
+    m_MTArgs.Reset(new CMTArgs(true));
     arg.Reset(m_MTArgs);
-    m_Args.push_back(arg); */
+    m_Args.push_back(arg); 
 
     m_RemoteArgs.Reset(new CRemoteArgs);
     arg.Reset(m_RemoteArgs);
@@ -119,7 +118,8 @@ CRPSTBlastnAppArgs::x_CreateOptionsHandle(CBlastOptions::EAPILocality locality,
 int
 CRPSTBlastnAppArgs::GetQueryBatchSize() const
 {
-    return blast::GetQueryBatchSize(eRPSTblastn, m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(eRPSTblastn, m_IsUngapped, is_remote);
 }
 
 END_SCOPE(blast)
