@@ -1,4 +1,4 @@
-/*  $Id: tblastn_args.cpp 373344 2012-08-28 20:44:16Z rafanovi $
+/*  $Id: tblastn_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] 
-    = "$Id: tblastn_args.cpp 373344 2012-08-28 20:44:16Z rafanovi $";
+    = "$Id: tblastn_args.cpp 389884 2013-02-21 16:37:10Z rafanovi $";
 #endif
 
 #include <ncbi_pch.hpp>
@@ -159,7 +159,8 @@ CTblastnAppArgs::SetInputPssm(CRef<objects::CPssmWithParameters> pssm)
 int
 CTblastnAppArgs::GetQueryBatchSize() const
 {
-    return blast::GetQueryBatchSize(eTblastn, m_IsUngapped);
+    bool is_remote = (m_RemoteArgs.NotEmpty() && m_RemoteArgs->ExecuteRemotely());
+    return blast::GetQueryBatchSize(eTblastn, m_IsUngapped, is_remote);
 }
 
 END_SCOPE(blast)
