@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 370872 2012-08-02 17:19:24Z rafanovi $
+/* $Id: blast_engine.c 391966 2013-03-12 20:48:27Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,7 +55,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_engine.c 370872 2012-08-02 17:19:24Z rafanovi $";
+    "$Id: blast_engine.c 391966 2013-03-12 20:48:27Z camacho $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_engine.h>
@@ -84,8 +84,8 @@ static char const rcsid[] =
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
 NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 27;
-NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "Aug-12-2012";
+NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 28;
+NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "March-19-2013";
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers 
     to various preallocated structures and arrays. */
@@ -785,10 +785,10 @@ s_BlastSearchEngineCore(EBlastProgramType program_number,
             isRPS = TRUE;
             scale_factor = score_params->scale_factor;
         }
-        status = Blast_HSPListGetEvalues(query_info, stat_length, hsp_list_out, 
+        status = Blast_HSPListGetEvalues(program_number, query_info,
+                                         stat_length, hsp_list_out, 
                                          score_options->gapped_calculation, 
                                          isRPS, gap_align->sbp, 0, scale_factor);
-        sbp->matrix_only_scoring = FALSE;
     }
     
    /* Use score threshold rather than evalue if 
@@ -1079,6 +1079,36 @@ s_RPSPreliminarySearchEngine(EBlastProgramType program_number,
     return status;
 }
 
+static void 
+s_AdjustSubjectForSraSearch(BlastHSPList* hsp_list, Uint1 offset )
+{
+	int i = 0;
+	BlastHSP ** hsp_array = hsp_list->hsp_array;
+	for(i=0; i < hsp_list->hspcnt; i++)
+	{
+		BlastHSP * hsp = hsp_array[i];
+		if(hsp->subject.offset <= offset)
+		{
+			hsp->subject.offset = 0;
+			hsp->query.offset += offset;
+
+			if(hsp->subject.gapped_start <= offset)
+			{
+				hsp->subject.gapped_start =  0;
+				hsp->query.gapped_start = hsp->query.offset;
+			}
+		}
+		else
+		{
+			hsp->subject.offset -= offset;
+		}
+
+		hsp->subject.end -= offset;
+
+		ASSERT(hsp->subject.offset < hsp->subject.end);
+		ASSERT(hsp->query.offset < hsp->query.end);
+	}
+}
 
 Int4 
 BLAST_PreliminarySearchEngine(EBlastProgramType program_number, 
@@ -1167,6 +1197,7 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
 
        if (BlastSeqSrcGetSequence(seq_src, &seq_arg) < 0)
            continue;
+
        if (db_length == 0) {
            /* This is not a database search, hence need to recalculate and save
             the effective search spaces and length adjustments for all 
@@ -1246,8 +1277,9 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
                                       hit_params->link_hsp_params, 
                                       gapped_calculation);
                } else {
-                  Blast_HSPListGetEvalues(query_info, stat_length,
-                                          hsp_list, gapped_calculation, FALSE,
+                  Blast_HSPListGetEvalues(program_number, query_info,
+                                          stat_length, hsp_list,
+                                          gapped_calculation, FALSE,
                                           sbp, 0, 1.0);
                }
                /* Use score threshold rather than evalue if 
@@ -1267,6 +1299,12 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
             Blast_HSPListGetBitScores(hsp_list, gapped_calculation, sbp);
          } 
          
+         // This should only happen for sra searches
+         if(seq_arg.seq->bases_offset > 0)
+         {
+        	s_AdjustSubjectForSraSearch(hsp_list, seq_arg.seq->bases_offset);
+         }
+
          /* Save the results. */
          status = BlastHSPStreamWrite(hsp_stream, &hsp_list);
          if (status != 0)
