@@ -1,4 +1,4 @@
-/* $Id: blast_setup.c 351770 2012-02-01 14:34:56Z maning $
+/* $Id: blast_setup.c 367865 2012-06-28 19:22:11Z madden $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_setup.c 351770 2012-02-01 14:34:56Z maning $";
+    "$Id: blast_setup.c 367865 2012-06-28 19:22:11Z madden $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_setup.h>
@@ -145,6 +145,7 @@ s_PHIScoreBlkFill(BlastScoreBlk* sbp, const BlastScoringOptions* options,
    Blast_KarlinBlk* kbp;
    char buffer[1024];
    Int2 status = 0;
+   int index;
 
    sbp->read_in_matrix = TRUE;
    if ((status = Blast_ScoreBlkMatrixFill(sbp, get_path)) != 0)
@@ -316,9 +317,17 @@ s_PHIScoreBlkFill(BlastScoreBlk* sbp, const BlastScoringOptions* options,
    if (status) 
        Blast_MessageWrite(blast_message, eBlastSevWarning, kBlastMessageNoContext, buffer);
    else {
-       /* Put a copy the Karlin block into the kbp_std array */
-       sbp->kbp_std[0] = (Blast_KarlinBlk*) 
+
+       /* fill in the rest of kbp_gap_std */
+       for(index=1;index<sbp->number_of_contexts;index++)
+       sbp->kbp_gap_std[index] = (Blast_KarlinBlk*)
            BlastMemDup(sbp->kbp_gap_std[0], sizeof(Blast_KarlinBlk));
+
+       /* copy kbp_gap_std to kbp_std */
+       for(index=0;index<sbp->number_of_contexts;index++)
+       sbp->kbp_std[index] = (Blast_KarlinBlk*)
+           BlastMemDup(sbp->kbp_gap_std[0], sizeof(Blast_KarlinBlk));
+
        sbp->kbp = sbp->kbp_std;
    }
 
@@ -345,6 +354,7 @@ Blast_ScoreBlkMatrixInit(EBlastProgramType program_number,
     if (program_number == eBlastTypeBlastn) {
 
         BLAST_ScoreSetAmbigRes(sbp, 'N');
+        BLAST_ScoreSetAmbigRes(sbp, '-');
 
         /* If reward/penalty are both zero the calling program is
          * indicating that a matrix must be used to score both the
