@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c 358499 2012-04-03 14:48:04Z coulouri $
+/* $Id: blast_traceback.c 390750 2013-03-01 18:22:37Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_traceback.c 358499 2012-04-03 14:48:04Z coulouri $";
+    "$Id: blast_traceback.c 390750 2013-03-01 18:22:37Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_traceback.h>
@@ -308,7 +308,8 @@ s_HSPListPostTracebackUpdate(EBlastProgramType program_number,
          are rounded down to the nearest even number. */
       Blast_HSPListAdjustOddBlastnScores(hsp_list, kGapped, sbp);
 
-      Blast_HSPListGetEvalues(query_info, subject_length, hsp_list, kGapped, FALSE, sbp, 0,
+      Blast_HSPListGetEvalues(program_number, query_info, subject_length,
+                              hsp_list, kGapped, FALSE, sbp, 0,
                               scale_factor);
    }
 
@@ -328,114 +329,6 @@ s_HSPListPostTracebackUpdate(EBlastProgramType program_number,
    return 0;
 }
 
-
-/** Split the HSP if it contains "non-bases"
- * @param hsp_list HSPList obtained after a traceback alignment [in] [out]
- * @param query_blk Query sequence block [in]
- * @param query_info Query sequence information [in]
- */
-static void
-s_SplitHsp(BlastHSPList* hsp_list, 
-           BLAST_SequenceBlk* query_blk, 
-           BlastQueryInfo* query_info)
-{
-    Int4 index, hspcnt;
-    hspcnt = hsp_list->hspcnt;
-    for (index=0; index < hspcnt; index++) {
-
-        Boolean found = FALSE;
-        BlastHSP *hsp = hsp_list->hsp_array[index];
-        Uint1 *query;
-        Int4 qid;
-
-        if (!hsp) continue;
-
-        query = query_blk->sequence +
-                   query_info->contexts[hsp->context].query_offset;
-
-        for (qid=hsp->query.offset; qid< hsp->query.end; ++qid) {
-            if (query[qid] == 0xf) {
-                found = TRUE;
-                break;
-            }
-        }
-
-        if (found) {
-            Int4 eid, oid, qid, sid;
-            Int4 qs, qe, ss, se, es, ee, os;
-            BlastHSP *new_hsp; 
-            GapEditScript *esp, *new_esp;
-            Boolean seek_start;
- 
-            esp = hsp->gap_info;
-            qid = hsp->query.offset;
-            sid = hsp->subject.offset;
-            qs = qid;
-            ss = sid;
-            se = 0;
-            es = 0;
-            ee = 0;
-            os = 0;
-            seek_start = FALSE;
-
-            for (eid=0; eid < esp->size; eid++) {
-                for(oid=0; oid < esp->num[eid];) {
-                    if (!seek_start && query[qid] == 0xf) {
-                        new_esp = GapEditScriptNew(1 + ee - es);
-                        GapEditScriptPartialCopy(new_esp, 0, esp, es, ee);
-                        new_esp->num[0] -= os;
-                        if (eid == ee) {
-                            ASSERT(oid>0);
-                            new_esp->num[ee-es] = oid;
-                            qe = qid;
-                            se = sid;
-                        }
-                        Blast_HSPInit(qs, qe, ss, se, qs, ss,
-                              hsp->context,  hsp->query.frame, 
-                              hsp->subject.frame, 0, &new_esp, &new_hsp);
-                        Blast_HSPListSaveHSP(hsp_list, new_hsp);
-                        seek_start = TRUE;
-                    } 
-
-                    if (esp->op_type[eid] == eGapAlignSub) {
-                        if (seek_start && query[qid] != 0xf) {
-                            qs = qid;
-                            ss = sid;
-                            es = eid;
-                            ee = eid;
-                            os = oid;
-                            seek_start = FALSE;
-                        } 
-                        qid++;
-                        sid++;
-                        oid++;
-                        if (oid == esp->num[eid] && !seek_start) {
-                            qe = qid;
-                            se = sid;
-                            ee = eid;
-                        }
-                    } else if (esp->op_type[eid] == eGapAlignDel) {
-                        sid+=esp->num[eid];
-                        oid+=esp->num[eid];
-                    } else if (esp->op_type[eid] == eGapAlignIns) {
-                        qid++;
-                        oid++;
-                    }     
-                }
-            }
-            ASSERT(!seek_start);
-            new_esp = GapEditScriptNew(1 + ee - es);
-            GapEditScriptPartialCopy(new_esp, 0, esp, es, ee);
-            new_esp->num[0] -= os;
-            Blast_HSPInit(qs, qe, ss, se, qs, ss,
-                          hsp->context,  hsp->query.frame, 
-                          hsp->subject.frame, 0, &new_esp, &new_hsp);
-            Blast_HSPListSaveHSP(hsp_list, new_hsp);
-            hsp_list->hsp_array[index] = Blast_HSPFree(hsp);
-        }
-    }
-}
-
 /*
     Comments in blast_traceback.h
  */
@@ -629,14 +522,14 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
             if(kIsOutOfFrame) {
                /* Code below should be investigated for possible
                   optimization for OOF */
-               s_start = hsp->subject.gapped_start;
-               q_start = hsp->query.gapped_start;
                gap_align->subject_start = 0;
                gap_align->query_start = 0;
-            } else {
-               q_start = hsp->query.gapped_start;
-               s_start = hsp->subject.gapped_start;
-            }
+            } else if (program_number == eBlastTypeBlastn) {
+               /* Find the optimal starting offset */
+               BlastGetStartForGappedAlignmentNucl(query, subject, hsp);
+            } 
+            q_start = hsp->query.gapped_start;
+            s_start = hsp->subject.gapped_start;
          }
          
          adjusted_s_length = subject_length;
@@ -789,6 +682,8 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
       }
    } /* End loop on HSPs */
 
+   Blast_HSPListPurgeNullHSPs(hsp_list);
+
    if (program_number != eBlastTypeRpsTblastn) {
       if (translation_buffer) {
          sfree(translation_buffer);
@@ -814,11 +709,6 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
        Int4 extra_start =
            Blast_HSPListPurgeHSPsWithCommonEndpoints(program_number, hsp_list, FALSE);
 
-       /* Check for possible non-base region and break HSP into segments */
-       if (program_number == eBlastTypeBlastn) {
-           s_SplitHsp(hsp_list, query_blk, query_info);
-       }
-
        /* Low level greedy algorithm ignores ambiguities, so the score
         * needs to be reevaluated. */
        if (kGreedyTraceback) {
@@ -1057,6 +947,7 @@ void RPSPsiMatrixDetach(BlastScoreBlk* sbp)
     ASSERT(sbp);
     sbp->psi_matrix->pssm->data = NULL;
     sfree(sbp->psi_matrix->pssm);
+    sfree(sbp->psi_matrix->freq_ratios);
     sfree(sbp->psi_matrix);
 }
 
@@ -1069,13 +960,16 @@ void RPSPsiMatrixDetach(BlastScoreBlk* sbp)
 static Int2 
 s_RPSGapAlignDataPrepare(BlastQueryInfo* concat_db_info, 
                          BlastGapAlignStruct* gap_align, 
-                         const BlastRPSInfo* rps_info)
+                         const BlastRPSInfo* rps_info,
+                         Int4 *** rps_freq)
 {
    Int4** rps_pssm = NULL;
    Int4 num_profiles;
    Int4 num_pssm_rows;
    Int4* pssm_start;
+   Int4 * freq_start = NULL;
    BlastRPSProfileHeader *profile_header;
+   BlastRPSFreqRatiosHeader *freq_header;
    Int4 index;
    Int4 alphabet_size;
 
@@ -1085,6 +979,7 @@ s_RPSGapAlignDataPrepare(BlastQueryInfo* concat_db_info,
    ASSERT(concat_db_info);
 
    profile_header = rps_info->profile_header;
+   freq_header = rps_info->freq_ratios_header;
    num_profiles = profile_header->num_profiles;
 
    /* force the alphabet size to match up to the on-disk format */
@@ -1103,17 +998,42 @@ s_RPSGapAlignDataPrepare(BlastQueryInfo* concat_db_info,
    rps_pssm = (Int4 **)malloc((num_pssm_rows+1) * sizeof(Int4 *));
    pssm_start = profile_header->start_offsets + num_profiles + 1;
 
+   if (freq_header) {
+      *rps_freq = (Int4 **)malloc((num_pssm_rows+1) * sizeof(Int4 *));
+      freq_start = freq_header->start_offsets +num_profiles + 1;
+   }
+
    for (index = 0; index < num_pssm_rows + 1; index++) {
       rps_pssm[index] = pssm_start;
       pssm_start += alphabet_size;
+      if (freq_header) {
+          (*rps_freq)[index] = freq_start;
+          freq_start += alphabet_size;
+      }
    }
 
    gap_align->positionBased = TRUE;
    RPSPsiMatrixAttach(gap_align->sbp, rps_pssm, alphabet_size);
-
    return 0;
 }
 
+static void s_RPSFillFreqRatiosInPsiMatrix(SPsiBlastScoreMatrix* psi_matrix,
+                            Int4 ** freq, Int4 ncol)
+{
+   Int4 ic, ir;
+   psi_matrix->pssm->ncols = ncol;
+   psi_matrix->freq_ratios = (double **) 
+           _PSIAllocateMatrix(ncol, BLASTAA_SIZE, sizeof(double));
+   for (ic=0; ic<ncol; ic++) {
+       for (ir=0; ir<psi_matrix->pssm->nrows; ir++) {
+           psi_matrix->freq_ratios[ic][ir] = (double) freq[ic][ir]/ (double) FREQ_RATIO_SCALE;
+       }
+       for (;  ir<BLASTAA_SIZE; ir++) {
+           psi_matrix->freq_ratios[ic][ir] = 0.0;
+       }
+   }
+}
+
 /** Factor to multiply the Karlin-Altschul K parameter by for RPS BLAST, to make
  * e-values more conservative.
  */
@@ -1155,12 +1075,14 @@ static
 Int2 s_RPSComputeTraceback(EBlastProgramType program_number, 
                            BlastHSPStream* hsp_stream, 
                            const BlastSeqSrc* seq_src, 
+                           Int4  default_db_genetic_code,
                            BLAST_SequenceBlk* query, BlastQueryInfo* query_info, 
                            BlastGapAlignStruct* gap_align,
-                           const BlastScoringParameters* score_params,
+                           BlastScoringParameters* score_params,
                            const BlastExtensionParameters* ext_params,
                            BlastHitSavingParameters* hit_params,
                            const BlastRPSInfo* rps_info,                
+                           const PSIBlastOptions* psi_options,
                            BlastHSPResults* results,
                            TInterruptFnPtr interrupt_search, 
                            SBlastProgress* progress_info)
@@ -1169,6 +1091,7 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
    BlastHSPList* hsp_list;
    BlastScoreBlk* sbp;
    Int4 **rpsblast_pssms = NULL;
+   Int4 **rpsblast_freqs = NULL;
    Int4 db_seq_start;
    EBlastEncoding encoding;
    BlastSeqSrcGetSeqArg seq_arg;
@@ -1185,9 +1108,15 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
    
    concat_db_info = BlastQueryInfoNew(program_number,
                                       rps_info->profile_header->num_profiles);
+   // Note that if rpsblast_freqs is not NULL, memory has been allocated in s_RPSGAPAlignDataPrepare
    if ((status = 
-        s_RPSGapAlignDataPrepare(concat_db_info, gap_align, rps_info)) != 0)
+        s_RPSGapAlignDataPrepare(concat_db_info, gap_align, rps_info, &rpsblast_freqs)) != 0)
+   {
+	  if(NULL != rpsblast_freqs)
+		  sfree(rpsblast_freqs);
+
       return status;
+   }
       
    sbp = gap_align->sbp;
    rpsblast_pssms = gap_align->sbp->psi_matrix->pssm->data;
@@ -1223,6 +1152,15 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
       }
    }
 
+   /* for CBS the scaling of PSSM is ignored.  So we scale the gap parameters back */
+   if (ext_params->options->compositionBasedStats > 0) {
+      Int4 scale_factor = (Int4) (score_params->scale_factor);
+      ASSERT(scale_factor);
+      score_params->gap_open /= scale_factor;
+      score_params->gap_extend /= scale_factor;
+      score_params->shift_pen /= scale_factor;
+   }
+
    while (BlastHSPStreamRead(hsp_stream, &hsp_list) 
           != kBlastHSPStream_Eof) {
 
@@ -1241,7 +1179,11 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
       if (Blast_GetOneQueryStructs(&one_query_info, &one_query, 
                                    query_info, query, 
                                    hsp_list->query_index) != 0)
+      {
+    	  if(NULL != rpsblast_freqs)
+    	  		  sfree(rpsblast_freqs);
           return -1;
+      }
 
       /* Pick out one of the sequences from the concatenated DB (given by the 
          OID of this HSPList). The sequence length does not include the 
@@ -1262,37 +1204,69 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
          sbp->psi_matrix->pssm->data = rpsblast_pssms + db_seq_start;
       } else {
          const double* karlin_k = rps_info->aux_info.karlin_k;
-         /* replace the PSSM and the Karlin values for this DB sequence
-            and this query sequence. */
-         sbp->psi_matrix->pssm->data = 
-            RPSRescalePssm(score_params->scale_factor,
-                           one_query->length, one_query->sequence, 
-                           seq_arg.seq->length,
-                           rpsblast_pssms + db_seq_start, sbp);
-         /* The composition of the query could have caused this one
-            subject sequence to produce a bad PSSM. This should
-            not be a fatal error, so just go on to the next subject
-            sequence */
-         if (sbp->psi_matrix->pssm->data == NULL) {
-            /** @todo FIXME Results should not be silently skipped,
-             *        need a warning here
-             */
-            hsp_list = Blast_HSPListFree(hsp_list);
-            BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
-            continue;
+
+
+         /* for CBS only allocate memory for PSSM matrix */
+         if (ext_params->options->compositionBasedStats > 0) {
+
+             sbp->psi_matrix->pssm->data = (Int4**)_PSIAllocateMatrix(
+                                                          seq_arg.seq->length,
+                                                          BLASTAA_SIZE,
+                                                          sizeof(Int4));
          }
-         
+         else {
+
+             /* for no CBS option rescale PSSM matrix with the half-way CBS */
+             /* replace the PSSM and the Karlin values for this DB sequence
+                and this query sequence. */
+             sbp->psi_matrix->pssm->data = 
+                 RPSRescalePssm(score_params->scale_factor,
+                                one_query->length, one_query->sequence, 
+                                seq_arg.seq->length,
+                                rpsblast_pssms + db_seq_start, sbp);
+             /* The composition of the query could have caused this one
+                subject sequence to produce a bad PSSM. This should
+                not be a fatal error, so just go on to the next subject
+                sequence */
+             if (sbp->psi_matrix->pssm->data == NULL) {
+                 /** @todo FIXME Results should not be silently skipped,
+                  *        need a warning here
+                  */
+                 hsp_list = Blast_HSPListFree(hsp_list);
+                 BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
+                 continue;
+             }
+         }
+
          sbp->kbp_gap[0]->K = RPS_K_MULT * karlin_k[hsp_list->oid];
          sbp->kbp_gap[0]->logK = log(RPS_K_MULT * karlin_k[hsp_list->oid]);
       }
 
-
       /* compute the traceback information and calculate E values
          for all HSPs in the list */
-      
-      Blast_TracebackFromHSPList(program_number, hsp_list, seq_arg.seq, 
-         one_query, one_query_info, gap_align, sbp, score_params, 
-         ext_params->options, hit_params, NULL, NULL);
+
+      if (ext_params->options->compositionBasedStats > 0) {
+          Int4 offsets[2], query_index;
+          s_RPSFillFreqRatiosInPsiMatrix(sbp->psi_matrix, rpsblast_freqs + db_seq_start, seq_arg.seq->length);
+          one_query_info->first_context = one_query_info->last_context = 0;
+          one_query_info->num_queries = 1;
+          offsets[0] = 0;
+          offsets[1] = seq_arg.seq->length + 1;
+          one_query_info->max_length = seq_arg.seq->length;
+          OffsetArrayToContextOffsets(one_query_info, offsets, program_number);
+          query_index = hsp_list->query_index;
+          hsp_list->query_index = 0;
+          Blast_RedoAlignmentCore(program_number, seq_arg.seq, one_query_info, sbp,
+                                  one_query, NULL, default_db_genetic_code,
+                                  hsp_list, NULL, score_params, ext_params, 
+                                  hit_params, psi_options, NULL);
+          hsp_list->query_index = query_index;
+          s_BlastHSPListRPSUpdate(program_number, hsp_list);
+      } else {
+          Blast_TracebackFromHSPList(program_number, hsp_list, seq_arg.seq, 
+             one_query, one_query_info, gap_align, sbp, score_params, 
+             ext_params->options, hit_params, NULL, NULL);
+      }
 
       BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
 
@@ -1332,6 +1306,9 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
 
    /* Free the allocated array of memory mapped matrix columns and restore
       the original settings in the gapped alignment structure. */
+   if(NULL != rpsblast_freqs)
+	   sfree(rpsblast_freqs);
+
    sfree(rpsblast_pssms);
    gap_align->positionBased = FALSE;
    RPSPsiMatrixDetach(sbp);
@@ -1435,9 +1412,9 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
 
    if (Blast_ProgramIsRpsBlast(program_number)) {
        status =
-           s_RPSComputeTraceback(program_number, hsp_stream, seq_src, query,
-                                 query_info, gap_align, score_params,
-                                 ext_params, hit_params, rps_info, results,
+           s_RPSComputeTraceback(program_number, hsp_stream, seq_src, default_db_genetic_code,
+                                 query, query_info, gap_align, score_params,
+                                 ext_params, hit_params, rps_info, psi_options, results,
                                  interrupt_search, progress_info);
    } else if (ext_params->options->compositionBasedStats > 0 ||
               ext_params->options->eTbackExt == eSmithWatermanTbck) {
@@ -1445,9 +1422,9 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
          and seg fault */
       status =
           Blast_RedoAlignmentCore(program_number, query, query_info, sbp,
-                                  hsp_stream, seq_src, default_db_genetic_code,
-                                  score_params, ext_params, hit_params, 
-                                  psi_options, results);
+                                  NULL, seq_src, default_db_genetic_code, 
+                                  NULL, hsp_stream, score_params, ext_params, 
+                                  hit_params, psi_options, results);
    } else {
       Int4 i;
       BlastSeqSrcGetSeqArg seq_arg;
@@ -1610,6 +1587,7 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
       BlastSequenceBlkFree(seq_arg.seq);
    }
 
+
    // -RMH-: Apply masklevel filter
    if ( results && hit_params->mask_level < 101 )
    {
