@@ -1,4 +1,4 @@
-/* $Id: greedy_align.c 109471 2007-08-27 14:26:13Z papadopo $
+/* $Id: greedy_align.c 356969 2012-03-19 17:33:39Z coulouri $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,7 +36,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: greedy_align.c 109471 2007-08-27 14:26:13Z papadopo $";
+    "$Id: greedy_align.c 356969 2012-03-19 17:33:39Z coulouri $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/greedy_align.h>
@@ -315,7 +315,7 @@ s_GetNextNonAffineTback(Int4 **last_seq2_off, Int4 d,
  *            is uncompressed [in]
  * @return Number of exact matches found before a mismatch was encountered
  */
-static Int4 s_FindFirstMismatch(const Uint1 *seq1, const Uint1 *seq2,
+static NCBI_INLINE Int4 s_FindFirstMismatch(const Uint1 *seq1, const Uint1 *seq2,
                                 Int4 len1, Int4 len2, 
                                 Int4 seq1_index, Int4 seq2_index,
                                 Boolean *fence_hit,
