@@ -1,4 +1,4 @@
-/* $Id: na_ungapped.c 369385 2012-07-18 19:41:03Z rafanovi $
+/* $Id: na_ungapped.c 377712 2012-10-15 18:09:53Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,7 +30,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: na_ungapped.c 369385 2012-07-18 19:41:03Z rafanovi $";
+    "$Id: na_ungapped.c 377712 2012-10-15 18:09:53Z rafanovi $";
 #endif                          /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/na_ungapped.h>
@@ -718,7 +718,7 @@ s_BlastnDiagTableExtendInitialHit(BLAST_SequenceBlk * query,
              * -RMH-
              */
             if ( word_params->options->program_number == eBlastTypeBlastn &&
-                 word_params->matrix_only_scoring )
+                 (word_params->matrix_only_scoring || word_length < 11))
             {
                s_NuclUngappedExtendExact(query, subject, matrix, q_off,
                                   s_off, -(cutoffs->x_dropoff), ungapped_data);
