@@ -1,5 +1,5 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blast_format.cpp 367113 2012-06-20 19:26:08Z rafanovi $";
+static char const rcsid[] = "$Id: blast_format.cpp 389291 2013-02-14 18:36:09Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /*
@@ -53,6 +53,8 @@ Author: Jason Papadopoulos
 #include <algo/blast/format/build_archive.hpp>
 #include <serial/objostrxml.hpp>
 
+#include <corelib/ncbistre.hpp>
+
 #ifndef SKIP_DOXYGEN_PROCESSING
 USING_NCBI_SCOPE;
 USING_SCOPE(blast);
@@ -97,7 +99,8 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& options,
           m_Megablast(is_megablast),
           m_IndexedMegablast(is_indexed), 
           m_CustomOutputFormatSpec(custom_output_format),
-          m_IgOptions(ig_opts)
+          m_IgOptions(ig_opts),
+          m_Options(&options)
 {
     m_DbName = db_adapter.GetDatabaseName();
     m_IsBl2Seq = (m_DbName == kEmptyStr ? true : false);
@@ -109,6 +112,7 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& options,
     }
     if (m_FormatType == CFormattingArgs::eXml) {
         m_AccumulatedQueries.Reset(new CBlastQueryVector());
+        m_BlastXMLIncremental.Reset(new SBlastXMLIncremental());
     }
     if (use_sum_statistics && m_IsUngappedSearch) {
         m_ShowLinkedSetSize = true;
@@ -176,7 +180,8 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& opts,
           m_Megablast(opts.GetProgram() == eMegablast ||
                       opts.GetProgram() == eDiscMegablast),
           m_IndexedMegablast(opts.GetMBIndexLoaded()), 
-          m_CustomOutputFormatSpec(custom_output_format)
+          m_CustomOutputFormatSpec(custom_output_format),
+          m_Options(&opts)
 {
     m_DbInfo.assign(dbinfo_list.begin(), dbinfo_list.end());
     vector< CBlastFormatUtil::SDbInfo >::const_iterator itInfo;
@@ -189,6 +194,7 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& opts,
 
     if (m_FormatType == CFormattingArgs::eXml) {
         m_AccumulatedQueries.Reset(new CBlastQueryVector());
+        m_BlastXMLIncremental.Reset(new SBlastXMLIncremental());
     }
 
     if (opts.GetSumStatisticsMode() && m_IsUngappedSearch) {
@@ -347,13 +353,17 @@ s_HasLocalIDs(CConstRef<CBlastQueryVector> queries)
 
 void 
 CBlastFormat::x_ConfigCShowBlastDefline(CShowBlastDefline& showdef, 
-                                        int skip_from, int skip_to)
+                                        int skip_from, int skip_to, int index)
 {
     int flags = 0;
     if (m_ShowLinkedSetSize)
         flags |= CShowBlastDefline::eShowSumN;
-    if (m_IsHTML)
+    if (m_IsHTML){
         flags |= CShowBlastDefline::eHtml;
+        if (index >= 0) {
+            showdef.SetResultPosIndex(index); 
+        }
+    }
     if (m_ShowGi)
         flags |= CShowBlastDefline::eShowGi;
 
@@ -405,9 +415,11 @@ s_IsGlobalSeqAlign(CConstRef<objects::CSeq_align_set> seqalign_set)
 
 void
 CBlastFormat::x_DisplayDeflines(CConstRef<CSeq_align_set> aln_set, 
-                        unsigned int itr_num,
-                        blast::CPsiBlastIterationState::TSeqIds& prev_seqids,
-                        int additional)
+                                unsigned int itr_num,
+                                blast::CPsiBlastIterationState::TSeqIds& prev_seqids,
+                                int additional,
+                                int index,
+                                int defline_length )
 {
 
     if (itr_num != numeric_limits<unsigned int>::max() && 
@@ -437,10 +449,11 @@ CBlastFormat::x_DisplayDeflines(CConstRef<CSeq_align_set> aln_set,
         }}
 
     } else {
+        
         CShowBlastDefline showdef(*aln_set, *m_Scope, 
-                                  kFormatLineLength,
+                                  defline_length == -1 ? kFormatLineLength:defline_length,
                                   m_NumSummary + additional);
-        x_ConfigCShowBlastDefline(showdef, -1, -1);
+        x_ConfigCShowBlastDefline(showdef, -1, -1, index);
         showdef.DisplayBlastDefline(m_Outfile);
     }
     m_Outfile << "\n";
@@ -528,6 +541,16 @@ CBlastFormat::x_PrintStructuredReport(const blast::CSearchResults& results,
                 break;
             }
         }
+        CCmdLineBlastXMLReportData report_data(m_AccumulatedQueries, 
+                                               m_AccumulatedResults,
+                                               *m_Options, m_DbName, m_DbIsAA,
+                                               m_QueryGenCode, m_DbGenCode,
+                                               m_IsRemoteSearch);
+        objects::CBlastOutput xml_output;
+        BlastXML_FormatReport(xml_output, &report_data, &m_Outfile, 
+           m_BlastXMLIncremental.GetPointer());
+        m_AccumulatedResults.clear();
+        m_AccumulatedQueries->clear();
         return;
     }
 }
@@ -552,6 +575,7 @@ CBlastFormat::x_PrintTabularReport(const blast::CSearchResults& results,
         const CBlastTabularInfo::EFieldDelimiter kDelim =
             (m_FormatType == CFormattingArgs::eCommaSeparatedValues
              ? CBlastTabularInfo::eComma : CBlastTabularInfo::eTab);
+
         CBlastTabularInfo tabinfo(m_Outfile, m_CustomOutputFormatSpec, kDelim);
         tabinfo.SetParseLocalIds(m_BelieveQuery);
         if (ncbi::NStr::ToLower(m_Program) == string("blastn"))
@@ -566,10 +590,13 @@ CBlastFormat::x_PrintTabularReport(const blast::CSearchResults& results,
                                 subject_bioseq);
         }
 
-
         if (results.HasAlignments()) {
     	    CSeq_align_set copy_aln_set;
             CBlastFormatUtil::PruneSeqalign(*aln_set, copy_aln_set, m_HitlistSize);
+
+            if(string::npos != m_CustomOutputFormatSpec.find("qcovs"))
+            		CBlastFormatUtil::InsertSubjectScores (copy_aln_set, bhandle);
+
             ITERATE(CSeq_align_set::Tdata, itr, copy_aln_set.Get()) {
                     const CSeq_align& s = **itr;
                     tabinfo.SetFields(s, *m_Scope, &m_ScoringMatrix);
@@ -617,6 +644,7 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results)
         CSeq_align_set::Tdata::const_iterator itr = aln_set->Get().begin();
         tabinfo.SetMasterFields(**itr, *m_Scope, 
                                 annots->m_ChainType[0], 
+                                annots->m_ChainTypeToShow, 
                                 &m_ScoringMatrix);
         tabinfo.SetIgAnnotation(annots, m_IgOptions->m_IsProtein);
 
@@ -631,6 +659,7 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results)
         for (; itr != aln_set->Get().end(); ++itr) {
             tabinfo.SetFields(**itr, *m_Scope, 
                               annots->m_ChainType[j++], 
+                              annots->m_ChainTypeToShow,
                               &m_ScoringMatrix);
             tabinfo.Print();
         }
@@ -673,12 +702,30 @@ CConstRef<objects::CBioseq> CBlastFormat::x_CreateSubjectBioseq()
     return bhandle.GetBioseqCore();
 }
 
+/// Auxiliary function to print the BLAST Archive in multiple output formats
+static void s_PrintArchive(CRef<objects::CBlast4_archive> archive,
+                         CNcbiOstream& out)
+{
+    if (archive.Empty()) {
+        return;
+    }
+    string outfmt = CNcbiEnvironment().Get("ARCHIVE_FORMAT");
+    if (outfmt.empty()) {
+        out << MSerial_AsnText << *archive;
+    } else if (!NStr::CompareNocase(outfmt, "xml")) {
+        out << MSerial_Xml << *archive;
+    } else if (NStr::StartsWith(outfmt, "bin", NStr::eNocase)) {
+        out << MSerial_AsnBinary << *archive;
+    }
+}
+
 void 
 CBlastFormat::WriteArchive(blast::IQueryFactory& queries,
                            blast::CBlastOptionsHandle& options_handle,
                            const CSearchResultSet& results,
                            unsigned int num_iters)
 {
+    CRef<objects::CBlast4_archive>  archive;
     if (m_IsBl2Seq)
     {
 	CRef<CBlastQueryVector> query_vector(new CBlastQueryVector);
@@ -692,20 +739,18 @@ CBlastFormat::WriteArchive(blast::IQueryFactory& queries,
                 query_vector->AddQuery(search_query);
         }
         CObjMgr_QueryFactory subjects(*query_vector);
-        m_Outfile << MSerial_AsnText << *(BlastBuildArchive(queries, options_handle, results, subjects));
+        archive = BlastBuildArchive(queries, options_handle, results, subjects);
         
     }
     else
     {
-   		CRef<objects::CBlast4_archive>  archive;
     	// Use only by psi blast
     	if(num_iters != 0)
     		archive = BlastBuildArchive(queries, options_handle, results,  m_DbName, num_iters);
     	else
     		archive = BlastBuildArchive(queries, options_handle, results,  m_DbName);
-
-    	m_Outfile << MSerial_AsnText << *archive;
     }
+    s_PrintArchive(archive, m_Outfile);
 }
 
 void
@@ -714,7 +759,8 @@ CBlastFormat::WriteArchive(objects::CPssmWithParameters & pssm,
                            const CSearchResultSet& results,
                            unsigned int num_iters)
 {
-       m_Outfile << MSerial_AsnText << *(BlastBuildArchive(pssm, options_handle, results,  m_DbName, num_iters));
+    CRef<objects::CBlast4_archive> archive(BlastBuildArchive(pssm, options_handle, results,  m_DbName, num_iters));
+    s_PrintArchive(archive, m_Outfile);
 }
 
 
@@ -871,8 +917,10 @@ CBlastFormat::PrintOneResultSet(const blast::CSearchResults& results,
 
 void
 CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
-                        CConstRef<blast::CBlastQueryVector> queries)
+                                CConstRef<blast::CBlastQueryVector> queries, 
+                                int index)
 {
+
     // For remote searches, we don't retrieve the sequence data for the query
     // sequence when initially sending the request to the BLAST server (if it's
     // a GI/accession/TI), so we flush the scope so that it can be retrieved
@@ -913,6 +961,13 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
         x_PrintIgTabularReport(results);
         return;
     }
+
+    
+    if (m_IsHTML){
+        m_Outfile << "<link rel=\"stylesheet\" type=\"text/css\" href=\"css/align.css\" media=\"screen\" />"
+                  << "<link rel=\"stylesheet\" type=\"text/css\" href=\"css/alignIE.css\" media=\"screen\" />"
+                  << "<link rel=\"stylesheet\" type=\"text/css\" href=\"css/print.css\" media=\"print\" />";
+    }
     const bool kIsTabularOutput = false;
 
     // other output types will need a bioseq handle
@@ -926,6 +981,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     // print the preamble for this query
 
     m_Outfile << "\n\n";
+
     CBlastFormatUtil::AcknowledgeBlastQuery(*bioseq, kFormatLineLength,
                                             m_Outfile, m_BelieveQuery,
                                             m_IsHTML, kIsTabularOutput,
@@ -951,7 +1007,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     if ( !m_IsBl2Seq ) {
         CPsiBlastIterationState::TSeqIds prev_ids = CPsiBlastIterationState::TSeqIds();
         int additional =  results.m_NumActualV +results.m_NumActualD + results.m_NumActualJ;
-        x_DisplayDeflines(aln_set, numeric_limits<unsigned int>::max(), prev_ids, additional);
+        x_DisplayDeflines(aln_set, numeric_limits<unsigned int>::max(), prev_ids, additional, index, 100);
     }
 
     //-------------------------------------------------
@@ -971,6 +1027,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
         CSeq_align_set::Tdata::const_iterator itr = aln_set->Get().begin();
         tabinfo.SetMasterFields(**itr, *m_Scope, 
                                 annots->m_ChainType[0], 
+                                annots->m_ChainTypeToShow, 
                                 &m_ScoringMatrix);
         tabinfo.SetIgAnnotation(annots, m_IgOptions->m_IsProtein);
         m_Outfile << "Domain classification requested: " << m_IgOptions->m_DomainSystem << endl << endl;
@@ -986,7 +1043,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
 
     int flags = CDisplaySeqalign::eMergeAlign
         + CDisplaySeqalign::eShowIdentity
-        // + CDisplaySeqalign::eShowGapOnlyLines
+        + CDisplaySeqalign::eNewTargetWindow
         + CDisplaySeqalign::eShowEndGaps
         + CDisplaySeqalign::eShowAlignStatsForMultiAlignView;
 
@@ -1027,21 +1084,23 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
 
 
     CDisplaySeqalign display(*aln_set, *m_Scope, &masklocs, NULL,  m_MatrixName);
-    int num_align_to_show = m_NumAlignments + results.m_NumActualD + 
+    int num_align_to_show = results.m_NumActualV + results.m_NumActualD + 
         results.m_NumActualJ;
     if (m_DbName != m_IgOptions->m_Db[0]->GetDatabaseName()){
-        num_align_to_show += results.m_NumActualV;
+        num_align_to_show += m_NumAlignments;
     }
     display.SetNumAlignToShow(num_align_to_show);
     display.SetMasterDomain(&domain);
     display.SetDbName(m_DbName);
     display.SetDbType(!m_DbIsAA);
-    display.SetLineLen(70);
+    display.SetLineLen(90);
+
     if (annots->m_FrameInfo[0] >= 0 && m_IgOptions->m_Translate) {
-        display.SetTranslatedFrameForLocalSeq((CDisplaySeqalign::TranslatedFrameForLocalSeq) annots->m_FrameInfo[0]); 
+        display.SetTranslatedFrameForLocalSeq((CDisplaySeqalign::TranslatedFrameForLocalSeq) (annots->m_FrameInfo[0]%3)); 
         flags += CDisplaySeqalign::eShowTranslationForLocalSeq;
     }
     flags += CDisplaySeqalign::eShowSequencePropertyLabel;
+    flags += CDisplaySeqalign::eShowInfoOnMouseOverSeqid;
     vector<string> chain_type_list;
     ITERATE(vector<string>, iter, annots->m_ChainType) {
         if (*iter=="N/A"){
@@ -1066,7 +1125,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     display.SetSubjectMasks(subj_masks);
 
     if (m_IsHTML) {
-     
+        display.SetResultPositionIndex(index); 
         m_Outfile << "\n<CENTER><b><FONT color=\"green\">Alignments</FONT></b></CENTER>" 
                   << endl;
 
@@ -1098,7 +1157,7 @@ CBlastFormat::x_ReverseQuery(blast::CIgBlastResults& results)
     CRef<CBioseq> q_new(new CBioseq(loc, new_id));
     CConstRef<CSeq_id> new_qid = m_Scope->AddBioseq(*q_new).GetSeqId();
     if (qid->IsLocal()) {
-        string title = sequence::GetTitle(q_bh);
+        string title = sequence::CDeflineGenerator().GenerateDefline(q_bh);
         if (title != "") {
             CRef<CSeqdesc> des(new CSeqdesc());
             des->SetTitle("reversed|" + title);
@@ -1322,22 +1381,10 @@ CBlastFormat::PrintEpilog(const blast::CBlastOptions& options)
     } else if (m_FormatType >= CFormattingArgs::eTabular) 
         return;  // No footer for these.
 
-    // XML can only be printed once (i.e.: not in batches), so we print it as
-    // the epilog of the report
+    // Most of XML is printed as it's finished.
+    // the epilog closes the report.
     if (m_FormatType == CFormattingArgs::eXml) {
-        CCmdLineBlastXMLReportData report_data(m_AccumulatedQueries, 
-                                               m_AccumulatedResults,
-                                               options, m_DbName, m_DbIsAA,
-                                               m_QueryGenCode, m_DbGenCode,
-                                               m_IsRemoteSearch);
-        objects::CBlastOutput xml_output;
-        BlastXML_FormatReport(xml_output, &report_data);
-
-	CObjectOStreamXml l_xml_stream( m_Outfile, false );
-	l_xml_stream.SetDefaultDTDFilePrefix("http://www.ncbi.nlm.nih.gov/dtd/");
-	l_xml_stream.WriteFileHeader(ObjectType( xml_output ));
-        l_xml_stream.WriteObject( ObjectInfo( xml_output ));
-
+        m_Outfile << m_BlastXMLIncremental->m_SerialXmlEnd << endl; 
         m_AccumulatedResults.clear();
         m_AccumulatedQueries->clear();
         return;
