@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: igblast.cpp 359118 2012-04-10 19:00:13Z maning $";
+    "$Id: igblast.cpp 383539 2012-12-14 21:13:12Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -54,6 +54,10 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 BEGIN_SCOPE(blast)
 
+static int max_allowed_VJ_distance_with_D = 90;
+static int max_allowed_VJ_distance_without_D = 40;
+static int max_allowed_VD_distance = 40;
+
 static void s_ReadLinesFromFile(const string& fn, vector<string>& lines)
 {
     CNcbiIfstream fs(fn.c_str(), IOS_BASE::in);
@@ -118,6 +122,9 @@ CIgAnnotationInfo::CIgAnnotationInfo(CConstRef<CIgBlastOptions> &ig_opt)
             if (frame != -1) {
                 m_FrameOffset[tokens[0]] = frame;
             }
+            if (tokens.size() == 3) { //just backward compatible as there was no such field
+                m_DJChainType[tokens[0]] = tokens[2];
+            }
         }
     }
 };
@@ -153,8 +160,9 @@ CIgBlast::Run()
     /*** search DJ germline */
     int num_genes =  (m_IgOptions->m_IsProtein) ? 1 : 3;
     if (num_genes > 1) {
-        x_SetupDJSearch(annots, qf, opts_hndl);
+        
         for (int gene = 1; gene < num_genes; ++gene) {
+            x_SetupDJSearch(annots, qf, opts_hndl, gene);
             CLocalBlast blast(qf, opts_hndl, m_IgOptions->m_Db[gene]);
             results[gene] = blast.Run();
             x_ConvertResultType(results[gene]);
@@ -222,25 +230,34 @@ void CIgBlast::x_SetupVSearch(CRef<IQueryFactory>       &qf,
     }
     opts_hndl->SetEvalueThreshold(10.0);
     opts_hndl->SetFilterString("F");
-    opts_hndl->SetHitlistSize(5);
+    opts_hndl->SetHitlistSize(5+ m_IgOptions->m_NumAlign[0]);
     qf.Reset(new CObjMgr_QueryFactory(*m_Query));
 
 };
 
-void CIgBlast::x_SetupDJSearch(vector<CRef <CIgAnnotation> > &annots,
+void CIgBlast::x_SetupDJSearch(const vector<CRef <CIgAnnotation> > &annots,
                                CRef<IQueryFactory>           &qf,
-                               CRef<CBlastOptionsHandle>     &opts_hndl)
+                               CRef<CBlastOptionsHandle>     &opts_hndl,
+                               int db_type)
 {
     // Only igblastn will search DJ
     CBlastOptions & opts = opts_hndl->SetOptions();
     opts.SetMatchReward(1);
     opts.SetMismatchPenalty(-4);
-    opts.SetWordSize(5);
+    if (db_type == 2){ //J genes are longer so if can afford more reliable identification
+        opts.SetWordSize(7);
+        opts.SetMismatchPenalty(-3);
+    } else {
+        opts.SetWordSize(m_IgOptions->m_Min_D_match);
+    }
+
     opts.SetGapOpeningCost(5);
     opts.SetGapExtensionCost(2);
-    opts_hndl->SetEvalueThreshold(1000.0);
+    opts_hndl->SetEvalueThreshold((db_type == 2) ? 1000.0 : 100000.0);
     opts_hndl->SetFilterString("F");
-    opts_hndl->SetHitlistSize(50);
+    opts_hndl->SetHitlistSize(max(max(50, 
+               m_IgOptions->m_NumAlign[1]), 
+               m_IgOptions->m_NumAlign[2]));
 
     // Mask query for D, J search
     int iq = 0;
@@ -386,33 +403,114 @@ static bool s_DJNotCompatible(const CSeq_align &d, const CSeq_align &j, bool ms)
     return false;
 };
 
-void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
-                            CRef<CSearchResultSet>        &results_J,
-                            vector<CRef <CIgAnnotation> > &annots)
-{
-    int iq = 0;
-    NON_CONST_ITERATE(vector<CRef <CIgAnnotation> >, annot, annots) {
-
-        string q_ct = (*annot)->m_ChainType[0];
-        bool q_ms = (*annot)->m_MinusStrand;
-        ENa_strand q_st = (q_ms) ? eNa_strand_minus : eNa_strand_plus;
-        int q_ve = (q_ms) ? (*annot)->m_GeneInfo[0] : (*annot)->m_GeneInfo[1] - 1;
-
-        CRef<CSeq_align_set> align_D, align_J;
-
+/*
+static bool s_IsTopMatchJD(CSearchResults& res_J, CIgAnnotationInfo& annotation_info){
+    bool result = true; //default
+    CRef<CSeq_align_set> align_J;
+    if (res_J.HasAlignments()) {
+        align_J.Reset(const_cast<CSeq_align_set *>
+                      (&*(res_J.GetSeqAlign())));
+        CSeq_align_set::Tdata & align_list = align_J->Set();
+        CSeq_align_set::Tdata::iterator it = align_list.begin();
+        int prev_score = 0;
+        result = false;
+        while (it != align_list.end()) {
+            int current_score;
+            (*it)->GetNamedScore(CSeq_align::eScore_Score, current_score);
+            if(current_score >= prev_score){
+                string j_id;
+                (*it)->GetSeq_id(1).GetLabel(&j_id, CSeq_id::eContent);
+                string j_chain_type = annotation_info.GetDJChainType(j_id);
+                if (j_chain_type == "N/A"){
+                    //assume J gene id style 
+                    
+                    string sid = NStr::ToUpper(j_id);
+                    if (sid.substr(0, 2) == "TR" && sid[3] == 'J') {
+                        j_chain_type = "J" + sid.substr(2,1);
+                    } else if (sid[0] == 'J') {
+                        j_chain_type = sid.substr(0,2);
+                    }
+                }
+                if (j_chain_type == "JD"){
+                    result = true;
+                    break;
+                }
+                
+            } else {
+                break;
+            } 
+            prev_score = current_score;
+            ++it;
+        }
+           
+    }
+    return result;
+};
+*/
+void CIgBlast::x_FindDJAln(CRef<CSeq_align_set>& align_D,
+                           CRef<CSeq_align_set>& align_J,
+                           string q_ct,
+                           bool q_ms,
+                           ENa_strand q_st,
+                           int q_ve,
+                           int iq,
+                           bool va_or_vd_as_heavy_chain) {
+    
+    int allowed_VJ_distance = max_allowed_VJ_distance_with_D;
         /* preprocess D */
-        CSearchResults& res_D = (*results_D)[iq];
-        if (res_D.HasAlignments()) {
-            align_D.Reset(const_cast<CSeq_align_set *>
-                                           (&*(res_D.GetSeqAlign())));
+        if (align_D && !align_D->Get().empty()) {
             CSeq_align_set::Tdata & align_list = align_D->Set();
             CSeq_align_set::Tdata::iterator it = align_list.begin();
             /* chain type test */
-            if (q_ct!="VH" && q_ct!="N/A") {
+            if (q_ct!="VH" && q_ct!="VD" && q_ct!="VA" && q_ct!="VB" && q_ct!="N/A") {
                 while (it != align_list.end()) {
                     it = align_list.erase(it);
                 }
+                allowed_VJ_distance = max_allowed_VJ_distance_without_D;
+            } else if (q_ct =="VA" || q_ct =="VD") {
+                if (va_or_vd_as_heavy_chain) {
+                    //VA could behave like VD and is allowed to rearrange to JA or DD/JD
+                    q_ct = "VD";
+                    //annot->m_ChainType[0] = "VD";
+                } else {
+                    q_ct = "VA";
+                    while (it != align_list.end()) {
+                        it = align_list.erase(it);
+                    } 
+                    allowed_VJ_distance = max_allowed_VJ_distance_without_D;
+                }
             }
+            //test compatability between V and D
+            it = align_list.begin();
+            while (it != align_list.end()) {
+                bool keep = true;
+                /* chain type test */
+                if (q_ct!="N/A") {
+                    char s_ct = q_ct[1];
+                    string d_id;
+                    (*it)->GetSeq_id(1).GetLabel(&d_id, CSeq_id::eContent);
+                    string d_chain_type = m_AnnotationInfo.GetDJChainType(d_id);
+                    if (d_chain_type != "N/A"){
+                        if (d_chain_type[1] != q_ct[1]) keep = false;
+                    } else { //assume D gene id style 
+                        string sid = (*it)->GetSeq_id(1).AsFastaString();
+                        sid = NStr::ToUpper(sid);
+                        if (sid.substr(0, 4) == "LCL|") sid = sid.substr(4, sid.length());
+                        if ((sid.substr(0, 2) == "IG" || sid.substr(0, 2) == "TR")
+                            && sid[3] == 'D') {
+                            s_ct = sid[2];
+                        }
+                        if (s_ct!='B' && s_ct!='D') s_ct = q_ct[1];
+                        if (s_ct != q_ct[1]) keep = false;
+                    }
+                }
+                
+                /* remove failed seq_align */
+                if (!keep) it = align_list.erase(it);
+                else ++it;
+            }
+
+
             /* strand test */
             bool strand_found = false;
             ITERATE(CSeq_align_set::Tdata, it, align_list) {
@@ -435,8 +533,8 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
                 bool keep = false;
                 int q_ds = (*it)->GetSeqStart(0);
                 int q_de = (*it)->GetSeqStop(0);
-                if (q_ms) keep = (q_de >= q_ve - 30 && q_ds <= q_ve);
-                else      keep = (q_ds <= q_ve + 30 && q_de >= q_ve);
+                if (q_ms) keep = (q_de >= q_ve - max_allowed_VD_distance && q_ds <= q_ve - 3);
+                else      keep = (q_ds <= q_ve + max_allowed_VD_distance && q_de >= q_ve + 3);
                 if (!keep) it = align_list.erase(it);
                 else ++it;
             }
@@ -445,14 +543,34 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
         }
 
         /* preprocess J */
-        CSearchResults& res_J = (*results_J)[iq];
-        if (res_J.HasAlignments()) {
-            align_J.Reset(const_cast<CSeq_align_set *>
-                                           (&*(res_J.GetSeqAlign())));
+        if (align_J && !align_J->Get().empty()) {
             CSeq_align_set::Tdata & align_list = align_J->Set();
             CSeq_align_set::Tdata::iterator it = align_list.begin();
             while (it != align_list.end()) {
                 bool keep = true;
+                /* chain type test */
+                if (q_ct!="N/A") {
+                    char s_ct = q_ct[1];
+                    string j_id;
+                    (*it)->GetSeq_id(1).GetLabel(&j_id, CSeq_id::eContent);
+                    string j_chain_type = m_AnnotationInfo.GetDJChainType(j_id);
+                    if (j_chain_type != "N/A"){
+                        if (j_chain_type[1] != q_ct[1]) keep = false;
+                    } else { //assume J gene id style 
+                        string sid = (*it)->GetSeq_id(1).AsFastaString();
+                        sid = NStr::ToUpper(sid);
+                        if (sid.substr(0, 4) == "LCL|") sid = sid.substr(4, sid.length());
+                        if ((sid.substr(0, 2) == "IG" || sid.substr(0, 2) == "TR")
+                            && sid[3] == 'J') {
+                            s_ct = sid[2];
+                        } else if (sid[0] == 'J') {
+                            s_ct = sid[1];
+                        }
+                        if (s_ct!='H' && s_ct!='L' && s_ct!='K' &&
+                            s_ct!='A' && s_ct!='B' && s_ct!='D' && s_ct!='G') s_ct = q_ct[1];
+                        if (s_ct != q_ct[1]) keep = false;
+                    }
+                }
                 /* strand test */
                 if ((*it)->GetSeqStrand(0) != q_st) keep = false;
                 /* subject start test */
@@ -461,9 +579,9 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
                 int q_js = (*it)->GetSeqStart(0);
                 int q_je = (*it)->GetSeqStop(0);
                 if (q_ms) { 
-                    if (q_je < q_ve - 60 || q_js > q_ve) keep = false;
+                    if (q_je < q_ve - allowed_VJ_distance  || q_js > q_ve) keep = false;
                 } else {
-                    if (q_js > q_ve + 60 || q_je < q_ve) keep = false;
+                    if (q_js > q_ve + allowed_VJ_distance || q_je < q_ve) keep = false;
                 }
                 /* remove failed seq_align */
                 if (!keep) it = align_list.erase(it);
@@ -487,6 +605,15 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
                         it = al_D.erase(it);
                     } else ++it;
                 }
+
+                if (align_D.NotEmpty() && !align_D->IsEmpty()){
+                    it = al_J.begin();
+                    while (it != al_J.end()) {
+                        if (s_DJNotCompatible(**(al_D.begin()), **it, q_ms)) {
+                            it = al_J.erase(it);
+                        } else ++it;
+                    }
+                }
             } else {
                 it = al_J.begin();
                 while (it != al_J.end()) {
@@ -494,9 +621,105 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
                         it = al_J.erase(it);
                     } else ++it;
                 }
+                if (align_J.NotEmpty() && !align_J->IsEmpty()) {
+                    it = al_D.begin();
+                    while (it != al_D.end()) {
+                        if (s_DJNotCompatible(**it, **(al_J.begin()), q_ms)) {
+                            it = al_D.erase(it);
+                        } else ++it;
+                    }
+                    
+                }
             }
         }
+                   
+
+}
+
+
+void CIgBlast::x_FindDJ(CRef<CSearchResultSet>& results_D,
+                        CRef<CSearchResultSet>& results_J,
+                        CRef <CIgAnnotation>& annot,
+                        CRef<CSeq_align_set>& align_D,
+                        CRef<CSeq_align_set>& align_J,
+                        string q_ct,
+                        bool q_ms,
+                        ENa_strand q_st,
+                        int q_ve,
+                        int iq) {
+    
+    CRef<CSeq_align_set> original_align_D(new CSeq_align_set);
+    CRef<CSeq_align_set> original_align_J(new CSeq_align_set);
+    
+        /* preprocess D */
+        CSearchResults& res_D = (*results_D)[iq];
+        if (res_D.HasAlignments()) {
+
+            align_D.Reset(const_cast<CSeq_align_set *>
+                                           (&*(res_D.GetSeqAlign())));
+            original_align_D->Assign(*align_D);
+;
+        }
+
+        /* preprocess J */
+        CSearchResults& res_J = (*results_J)[iq];
+        if (res_J.HasAlignments()) {
+            align_J.Reset(const_cast<CSeq_align_set *>
+                                           (&*(res_J.GetSeqAlign())));
+            original_align_J->Assign(*align_J);
+           
+        } 
+        //try as VA
+        x_FindDJAln(align_D, align_J, q_ct, q_ms, q_st, q_ve, iq, false);
+        if (q_ct =="VA" || q_ct =="VD") {
+            annot->m_ChainType[0] = "VA";
+            //try as VD
+            x_FindDJAln(original_align_D, original_align_J, q_ct, q_ms, q_st, q_ve, iq, true);
+            int as_heavy_chain_score = 0;
+            int as_light_chain_score = 0;
+            int d_score = 0;
+            if(original_align_J.NotEmpty() && !original_align_J->Get().empty()){
+                original_align_J->Get().front()->GetNamedScore(CSeq_align::eScore_Score, as_heavy_chain_score);
+            }
+           
+            if(original_align_D.NotEmpty() && !original_align_D->Get().empty()){
+                original_align_D->Get().front()->GetNamedScore(CSeq_align::eScore_Score, d_score);
+            }
+            if (align_J.NotEmpty() && !align_J->Get().empty()){
+                align_J->Get().front()->GetNamedScore(CSeq_align::eScore_Score, as_light_chain_score);
+            }
+            if (as_heavy_chain_score + d_score> as_light_chain_score){
+                if (align_D.NotEmpty() && original_align_D.NotEmpty()){
+                    align_D->Assign(*original_align_D);
+                }
+                if (align_J.NotEmpty() && original_align_J.NotEmpty()){
+                    align_J->Assign(*original_align_J);
+                }
                 
+                annot->m_ChainType[0] = "VD";
+            }
+            
+        }
+        
+}
+
+void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
+                            CRef<CSearchResultSet>        &results_J,
+                            vector<CRef <CIgAnnotation> > &annots)
+{
+    int iq = 0;
+    NON_CONST_ITERATE(vector<CRef <CIgAnnotation> >, annot, annots) {
+
+        string q_ct = (*annot)->m_ChainType[0];
+        bool q_ms = (*annot)->m_MinusStrand;
+        ENa_strand q_st = (q_ms) ? eNa_strand_minus : eNa_strand_plus;
+        int q_ve = (q_ms) ? (*annot)->m_GeneInfo[0] : (*annot)->m_GeneInfo[1] - 1;
+
+        CRef<CSeq_align_set> align_D, align_J;
+
+        x_FindDJ( results_D, results_J, *annot, align_D, align_J, q_ct, q_ms, q_st, q_ve, iq); 
+
+       
         /* annotate D */    
         if (align_D.NotEmpty() && !align_D->IsEmpty()) {
             const CSeq_align & it = **(align_D->Get().begin());
@@ -555,7 +778,7 @@ void CIgBlast::x_AnnotateDomain(CRef<CSearchResultSet>        &gl_results,
 
         CIgAnnotation *annot = &*(annots[iq]);
         annot->m_ChainType.push_back("NON");  // Assuming non-ig sequence first
-
+        annot->m_ChainTypeToShow = "NON";
         if ((*result)->HasAlignments() && (*gl_results)[iq].HasAlignments()) {
 
 
@@ -587,7 +810,7 @@ void CIgBlast::x_AnnotateDomain(CRef<CSearchResultSet>        &gl_results,
                 string sid = (*it)->GetSeq_id(1).AsFastaString();
                 if (sid.substr(0, 4) == "lcl|") sid = sid.substr(4, sid.length());
                 annot->m_ChainType[0] = m_AnnotationInfo.GetDomainChainType(sid);
-
+                annot->m_ChainTypeToShow = annot->m_ChainType[0];
                 int domain_info[10];
 
                 if (m_AnnotationInfo.GetDomainInfo(sid, domain_info)) {
@@ -722,8 +945,8 @@ void CIgBlast::x_AnnotateDomain(CRef<CSearchResultSet>        &gl_results,
                         int q_stop = (*it)->GetSeqStop(0); 
                         int q_mid = q_start + q_stop;
                         int q_dif = q_stop - q_start;
-                        int frame_adj = ((*it)->GetSeqStart(1) + 3 - frame_offset) % 3;
-                        annot->m_FrameInfo[0] = ((q_mid - q_dir *q_dif)/2 + 3 - q_dir * frame_adj) % 3;
+                        int frame_adj = (3 - ((*it)->GetSeqStart(1) + 3 - frame_offset) % 3) %3;
+                        annot->m_FrameInfo[0] = (q_mid - q_dir *q_dif)/2 + q_dir * frame_adj;
                         frame_adj = ((*it)->GetSeqStop(1) + 3 - frame_offset) % 3;
                         annot->m_FrameInfo[1] = (q_mid + q_dir *q_dif)/2 - q_dir * frame_adj;
                     }
