@@ -1,4 +1,4 @@
-/*  $Id: remote_blast_unit_test.cpp 372443 2012-08-17 14:53:31Z rafanovi $
+/*  $Id: remote_blast_unit_test.cpp 382374 2012-12-05 15:02:09Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -198,8 +198,9 @@ BOOST_AUTO_TEST_CASE(CheckRemoteRPSBlastOptionsHandle) {
     BOOST_REQUIRE_EQUAL(true, rmt_blaster.Submit());
 }
 
+// Search of GIs 555 and 3090 against ecoli
 BOOST_AUTO_TEST_CASE(CheckBlastnMasks) {
-    const string rid("1143488952-21447-118405783159.BLASTQ1");
+    const string rid("BW3U058R01R");
     CRemoteBlast rmt_blaster(rid);
 
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
@@ -224,10 +225,7 @@ BOOST_AUTO_TEST_CASE(CheckBlastnMasks) {
 
     size_t index = 0;
     vector<TSeqRange> expected_masks;
-    expected_masks.push_back(TSeqRange(10, 40));
     expected_masks.push_back(TSeqRange(78, 89));
-    expected_masks.push_back(TSeqRange(178, 190));
-    expected_masks.push_back(TSeqRange(330, 370));
     BOOST_REQUIRE_EQUAL(expected_masks.size(), masks.front().size());
 
     ITERATE(TMaskedQueryRegions, seqlocinfo, masks.front()) {
@@ -257,9 +255,11 @@ BOOST_AUTO_TEST_CASE(CheckBlastnMasks) {
 
     index = 0;
     expected_masks.clear();
-    expected_masks.push_back(TSeqRange(90, 114));
-    expected_masks.push_back(TSeqRange(165, 249));
-    expected_masks.push_back(TSeqRange(258, 267));
+    expected_masks.push_back(TSeqRange(25, 31));
+    expected_masks.push_back(TSeqRange(35, 101));
+    expected_masks.push_back(TSeqRange(116, 123));
+    expected_masks.push_back(TSeqRange(131, 195));
+    expected_masks.push_back(TSeqRange(2022, 2337));
     BOOST_REQUIRE_EQUAL(expected_masks.size(), masks.back().size());
     ITERATE(TMaskedQueryRegions, seqlocinfo, masks.back()) {
         BOOST_REQUIRE(seqlocinfo->NotEmpty());
@@ -291,9 +291,9 @@ BOOST_AUTO_TEST_CASE(CheckBlastpMasks) {
     const string rid("1143488953-21461-10186755953.BLASTQ1");
     CRemoteBlast rmt_blaster(rid);
 
-    BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
-    BOOST_REQUIRE_EQUAL(true, rmt_blaster.CheckDone());
-    BOOST_REQUIRE_EQUAL(kEmptyStr, rmt_blaster.GetErrors());
+    BOOST_REQUIRE_MESSAGE(rid == rmt_blaster.GetRID(), "RID=" << rid);
+    BOOST_REQUIRE_MESSAGE(rmt_blaster.CheckDone(), "RID=" << rid);
+    BOOST_REQUIRE_MESSAGE(kEmptyStr == rmt_blaster.GetErrors(), "RID=" << rid);
 
     const EBlastProgramType prog = 
         NetworkProgram2BlastProgramType(rmt_blaster.GetProgram(),
@@ -371,8 +371,9 @@ BOOST_AUTO_TEST_CASE(CheckBlastpMasks) {
     }
 }
 
+// Search of GIs 555 and 3090 against ecoli
 BOOST_AUTO_TEST_CASE(CheckBlastxMasks) {
-    const string rid("1143488953-21475-125029367489.BLASTQ1");
+    const string rid("BW41NPVB014");
     CRemoteBlast rmt_blaster(rid);
 
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
@@ -390,7 +391,7 @@ BOOST_AUTO_TEST_CASE(CheckBlastxMasks) {
     BOOST_REQUIRE(!masks.empty());
     BOOST_REQUIRE(!network_masks.empty());
     const size_t kNumQueries = 2;
-    const size_t kNumNetMasks = 8;
+    const size_t kNumNetMasks = 7;
     BOOST_REQUIRE_EQUAL(kNumQueries, masks.size());
     BOOST_REQUIRE_EQUAL(kNumNetMasks, network_masks.size());
 
@@ -399,20 +400,8 @@ BOOST_AUTO_TEST_CASE(CheckBlastxMasks) {
     typedef pair<TSeqRange, CSeqLocInfo::ETranslationFrame> TMask;
     typedef vector<TMask> TQueryMasks;
     TQueryMasks expected_masks;
-    expected_masks.push_back(make_pair(TSeqRange(9, 39),
-                                       CSeqLocInfo::eFramePlus1));
-    expected_masks.push_back(make_pair(TSeqRange(177, 189),
-                                       CSeqLocInfo::eFramePlus1));
-    expected_masks.push_back(make_pair(TSeqRange(330, 369),
-                                       CSeqLocInfo::eFramePlus1));
-    expected_masks.push_back(make_pair(TSeqRange(333, 371),
-                                       CSeqLocInfo::eFrameMinus1));
-    expected_masks.push_back(make_pair(TSeqRange(180, 191),
-                                       CSeqLocInfo::eFrameMinus1));
     expected_masks.push_back(make_pair(TSeqRange(114, 155),
                                        CSeqLocInfo::eFrameMinus1));
-    expected_masks.push_back(make_pair(TSeqRange(12, 41),
-                                       CSeqLocInfo::eFrameMinus1));
     BOOST_REQUIRE_EQUAL(expected_masks.size(), query1_masks.size());
 
     ITERATE(TMaskedQueryRegions, seqlocinfo, query1_masks) {
@@ -425,7 +414,7 @@ BOOST_AUTO_TEST_CASE(CheckBlastxMasks) {
         BOOST_REQUIRE_EQUAL((int)mask.second, (*seqlocinfo)->GetFrame());
     }
     index = 0;
-    BOOST_REQUIRE_EQUAL(eBlast4_frame_type_plus1,
+    BOOST_REQUIRE_EQUAL(eBlast4_frame_type_minus1,
                         network_masks.front()->GetFrame());
     CBlast4_mask::TLocations const* net_masks =
         &network_masks.front()->GetLocations();
@@ -443,28 +432,69 @@ BOOST_AUTO_TEST_CASE(CheckBlastxMasks) {
 
     index = 0;
     expected_masks.clear();
-    expected_masks.push_back(make_pair(TSeqRange(90, 123),
-                                       CSeqLocInfo::eFramePlus1));
-    expected_masks.push_back(make_pair(TSeqRange(198, 231),
-                                       CSeqLocInfo::eFramePlus1));
-    expected_masks.push_back(make_pair(TSeqRange(258, 267),
-                                       CSeqLocInfo::eFramePlus1));
-    expected_masks.push_back(make_pair(TSeqRange(196, 232),
-                                       CSeqLocInfo::eFramePlus2));
-    expected_masks.push_back(make_pair(TSeqRange(101, 125),
-                                       CSeqLocInfo::eFramePlus3));
-    expected_masks.push_back(make_pair(TSeqRange(197, 233),
-                                       CSeqLocInfo::eFramePlus3));
-    expected_masks.push_back(make_pair(TSeqRange(259, 267),
-                                       CSeqLocInfo::eFrameMinus1));
-    expected_masks.push_back(make_pair(TSeqRange(199, 252),
-                                       CSeqLocInfo::eFrameMinus1));
-    expected_masks.push_back(make_pair(TSeqRange(91, 114),
-                                       CSeqLocInfo::eFrameMinus1));
-    expected_masks.push_back(make_pair(TSeqRange(198, 233),
-                                       CSeqLocInfo::eFrameMinus2));
-    expected_masks.push_back(make_pair(TSeqRange(200, 232),
-                                       CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(36, 66), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(129, 240), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(363, 393), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(423, 471), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(933, 972), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(1092, 1137), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(1158, 1206), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(1224, 1260), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(1665, 1734), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(1842, 1899), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(1971, 2010), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(2058, 2226), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(2256, 2334), CSeqLocInfo::eFramePlus1));
+    expected_masks.push_back(make_pair(TSeqRange(37, 64), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(607, 652), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(1153, 1192), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(1702, 1744), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(2014, 2092), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(2104, 2194), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(2251, 2278), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(2305, 2335), CSeqLocInfo::eFramePlus2));
+    expected_masks.push_back(make_pair(TSeqRange(35, 56), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(92, 173), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(239, 275), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(359, 398), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(1679, 1733), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(2072, 2135), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(2159, 2294), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(2309, 2333), CSeqLocInfo::eFramePlus3));
+    expected_masks.push_back(make_pair(TSeqRange(2311, 2337), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(2221, 2280), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(2155, 2202), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(2035, 2148), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(1816, 1857), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(1684, 1761), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(1348, 1389), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(1249, 1287), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(982, 1014), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(613, 654), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(514, 552), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(256, 279), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(121, 174), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(22, 84), CSeqLocInfo::eFrameMinus1));
+    expected_masks.push_back(make_pair(TSeqRange(2274, 2336), CSeqLocInfo::eFrameMinus2));
+    expected_masks.push_back(make_pair(TSeqRange(2004, 2261), CSeqLocInfo::eFrameMinus2));
+    expected_masks.push_back(make_pair(TSeqRange(222, 242), CSeqLocInfo::eFrameMinus2));
+    expected_masks.push_back(make_pair(TSeqRange(183, 203), CSeqLocInfo::eFrameMinus2));
+    expected_masks.push_back(make_pair(TSeqRange(132, 164), CSeqLocInfo::eFrameMinus2));
+    expected_masks.push_back(make_pair(TSeqRange(30, 83), CSeqLocInfo::eFrameMinus2));
+    expected_masks.push_back(make_pair(TSeqRange(2255, 2335), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(2192, 2239), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(2060, 2185), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(1964, 2011), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(1850, 1888), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(1673, 1741), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(1226, 1261), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(1166, 1204), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(1097, 1135), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(431, 469), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(365, 394), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(242, 289), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(131, 208), CSeqLocInfo::eFrameMinus3));
+    expected_masks.push_back(make_pair(TSeqRange(38, 70), CSeqLocInfo::eFrameMinus3));
 
     BOOST_REQUIRE_EQUAL(expected_masks.size(), query2_masks.size());
     ITERATE(TMaskedQueryRegions, seqlocinfo, masks.back()) {
@@ -571,7 +601,7 @@ BOOST_AUTO_TEST_CASE(CheckColoRID) {
 BOOST_AUTO_TEST_CASE(GetErrorsFromFailedRID) {
     // Uncomment to redirect to test system
     //CAutoEnvironmentVariable tmp_env("BLAST4_CONN_SERVICE_NAME", "blast4_test");
-    const string rid("2MSX0X8M01N" );
+    const string rid("A7CZ5K5N016");
     CRemoteBlast rmt_blaster(rid);
     //rmt_blaster.SetVerbose();
     
@@ -583,6 +613,7 @@ BOOST_AUTO_TEST_CASE(GetErrorsFromFailedRID) {
     BOOST_REQUIRE_EQUAL(kEmptyStr, rmt_blaster.GetWarnings());
     
     const string error("CPU usage limit was exceeded");
+    BOOST_REQUIRE(rmt_blaster.GetErrors().empty() == false);
     BOOST_REQUIRE(NStr::FindNoCase(rmt_blaster.GetErrors(), error) != NPOS);
     BOOST_REQUIRE_EQUAL(CRemoteBlast::eStatus_Failed, 
                         rmt_blaster.CheckStatus());
@@ -608,28 +639,31 @@ BOOST_AUTO_TEST_CASE(RetrieveInvalidRID) {
 BOOST_AUTO_TEST_CASE(RetrieveRIDWithError) {
     // Uncomment to redirect to test system
     //CAutoEnvironmentVariable tmp_env("BLAST4_CONN_SERVICE_NAME", "blast4_test");
-    const string rid("2RCK3CZR01N");
+    const string rid("A844TTAM014");
     CRemoteBlast rmt_blaster(rid);
     //rmt_blaster.SetVerbose();
 
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
     BOOST_REQUIRE_EQUAL(true, rmt_blaster.CheckDone());
-    BOOST_REQUIRE(NStr::Find(rmt_blaster.GetErrors(), "CPU usage limit was exceeded, resulting in SIGXCPU") != NPOS);
+    BOOST_REQUIRE_MESSAGE(NStr::Find(rmt_blaster.GetErrors(), 
+                                     "CPU usage limit was exceeded, resulting in SIGXCPU") != NPOS,
+                          "RID=" << rid);
     BOOST_REQUIRE_EQUAL(CRemoteBlast::eStatus_Failed, rmt_blaster.CheckStatus());
 }
 
 BOOST_AUTO_TEST_CASE(RetrieveRIDWithSIGXCPU) {
     // Uncomment to redirect to test system
     //CAutoEnvironmentVariable tmp_env("BLAST4_CONN_SERVICE_NAME", "blast4_test");
-    const string rid("2RCWHBTS01N");
+    const string rid("A84J535H016");
     CRemoteBlast rmt_blaster(rid);
     //rmt_blaster.SetVerbose();
 
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
     BOOST_REQUIRE_EQUAL(true, rmt_blaster.CheckDone());
     //cerr << "Errors: '" << rmt_blaster.GetErrors() << "'" << endl;
-    BOOST_REQUIRE(NStr::Find(rmt_blaster.GetErrors(), 
-                             "Error: CPU usage limit was exceeded") != NPOS);
+    BOOST_REQUIRE_MESSAGE(NStr::Find(rmt_blaster.GetErrors(), 
+                             "Error: CPU usage limit was exceeded") != NPOS,
+                          "RID=" << rid);
     BOOST_REQUIRE_EQUAL(CRemoteBlast::eStatus_Failed,
                         rmt_blaster.CheckStatus());
 }
@@ -712,7 +746,7 @@ BOOST_AUTO_TEST_CASE(RetrieveQuerySet)
 {
     // Another preserved query, with mixed ID types.
     
-    string rid("1121716447-26774-190697041164.BLASTQ2");
+    string rid("BWX50RMX016");  // GIs 104501, 129295, and FASTA for 400260645 vs swissprot   
     
     CRemoteBlast rb(rid);
     
@@ -727,9 +761,9 @@ BOOST_AUTO_TEST_CASE(RetrieveQuerySet)
         ids.push_back(L);
     }
     
-    BOOST_REQUIRE_EQUAL(string("gi|4507012"), ids[0]);
-    BOOST_REQUIRE_EQUAL(string("lcl|1_26774"), ids[1]);
-    BOOST_REQUIRE_EQUAL(string("lcl|2_26774"), ids[2]);
+    BOOST_REQUIRE_EQUAL(string("gi|104501"), ids[0]);
+    BOOST_REQUIRE_EQUAL(string("gi|129295"), ids[1]);
+    BOOST_REQUIRE_EQUAL(string("lcl|47622"), ids[2]);
 }
 
 BOOST_AUTO_TEST_CASE(GetRequestInfo)
@@ -755,78 +789,6 @@ BOOST_AUTO_TEST_CASE(GetRequestInfo)
 
 }
 
-BOOST_AUTO_TEST_CASE(GetDatabaseInfo)
-{
-    CBlastServices remote_svc;
-
-    CRef<CBlast4_database> blastdb(new CBlast4_database);
-    blastdb->SetName("nr");
-    blastdb->SetType(eBlast4_residue_type_nucleotide);
-
-    CRef<CBlast4_database_info> dbinfo = remote_svc.GetDatabaseInfo(blastdb);
-    BOOST_REQUIRE(dbinfo.NotEmpty());
-    BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
-
-    const string nt_title =
-        "All GenBank+EMBL+DDBJ+PDB sequences (but no EST, STS, "
-        "GSS,environmental samples or phase 0, 1 or 2 HTGS sequences)";
-    BOOST_REQUIRE_EQUAL(nt_title, dbinfo->GetDescription());
-
-    BOOST_REQUIRE(dbinfo->GetTotal_length() > (Int8)15e+9);
-    BOOST_REQUIRE(dbinfo->GetNum_sequences() > (Int8)35e+5);
-
-    // Try fetching swissprot
-    blastdb->SetName("swissprot");
-    blastdb->SetType(eBlast4_residue_type_protein);
-
-    dbinfo = remote_svc.GetDatabaseInfo(blastdb);
-    BOOST_REQUIRE(dbinfo.NotEmpty());
-    BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
-
-    const string swissprot_title("Non-redundant UniProtKB/SwissProt sequences.");
-    BOOST_REQUIRE_EQUAL(swissprot_title, dbinfo->GetDescription());
-
-    BOOST_REQUIRE(dbinfo->GetTotal_length() > (Int8)7e+7);
-    BOOST_REQUIRE(dbinfo->GetNum_sequences() > (Int8)15e+4);
-
-    // Try fetching a non-existent database
-    blastdb->SetName("junk");
-    blastdb->SetType(eBlast4_residue_type_protein);
-    dbinfo = remote_svc.GetDatabaseInfo(blastdb);
-    BOOST_REQUIRE(dbinfo.Empty());
-
-    // Try a frozen database
-    blastdb->SetName("ecoli");
-    blastdb->SetType(eBlast4_residue_type_nucleotide);
-    dbinfo = remote_svc.GetDatabaseInfo(blastdb);
-    BOOST_REQUIRE(dbinfo.NotEmpty());
-
-    BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
-    //cerr << MSerial_AsnText << * dbinfo << endl;
-    BOOST_REQUIRE_EQUAL("ecoli", dbinfo->GetDescription());
-    // Last_updated is now the date in which the statistics are collected?
-    //BOOST_REQUIRE_EQUAL(string("12/12/2002 14:17:00"),
-    //                     dbinfo->GetLast_updated());
-    BOOST_REQUIRE_EQUAL((Int8)400, dbinfo->GetNum_sequences());
-    BOOST_REQUIRE_EQUAL((Int8)4662239, dbinfo->GetTotal_length());
-
-    // Try unknown residue type
-    blastdb->SetName("patnt");
-    blastdb->SetType(eBlast4_residue_type_unknown);
-    dbinfo = remote_svc.GetDatabaseInfo(blastdb);
-    BOOST_REQUIRE(dbinfo.Empty());
-
-    // Fetch the structure group's favorite RPS-BLAST database
-    blastdb->SetName("cdd");
-    blastdb->SetType(eBlast4_residue_type_protein);
-    dbinfo = remote_svc.GetDatabaseInfo(blastdb);
-    BOOST_REQUIRE(dbinfo.NotEmpty());
-    BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
-
-// This is no longer true - (why?)
-    //BOOST_REQUIRE(dbinfo->GetDescription().find("cdd.v") != NPOS);
-}
-
 BOOST_AUTO_TEST_CASE(FetchQuerySequence)
 {
     // Uncomment to redirect to test system
@@ -837,7 +799,7 @@ BOOST_AUTO_TEST_CASE(FetchQuerySequence)
     // sequence (data) associated with the described sequence,
     // and checks that the length is correct.
     
-    string rid("1138907364-9206-202454460245.BLASTQ1");
+    string rid("BWY0XPAV014");  // P38398.2 and 129295 vs swissprot
     
     CRemoteBlast rb(rid);
     
@@ -868,7 +830,7 @@ BOOST_AUTO_TEST_CASE(FetchQuerySequence)
     
     // Assuming it is a "whole" Seq-loc, make a vector of Seq-ids.
     
-    CRef<CSeq_id> seqid(& query1->SetWhole());
+    CRef<CSeq_id> seqid(&query1->SetWhole());
     
     CBlastServices::TSeqIdVector getseq_queries;
     getseq_queries.push_back(seqid);
@@ -890,8 +852,9 @@ BOOST_AUTO_TEST_CASE(FetchQuerySequence)
     BOOST_REQUIRE(results[0]->CanGetInst());
 
     int length = results[0]->GetInst().GetLength();
-    
-    BOOST_REQUIRE_EQUAL(length, 320);
+    BOOST_REQUIRE_EQUAL(length, 1863);
+    //length = results[1]->GetInst().GetLength();
+    //BOOST_REQUIRE_EQUAL(length, 232);
 }
 
 BOOST_AUTO_TEST_CASE(FetchQuerySequence_NotFound)
@@ -1067,8 +1030,8 @@ BOOST_AUTO_TEST_CASE(GetSearchStrategy_FullQuery) {
 
     // These are the parameters that we are looking for
     vector<string> param_names;
-    param_names.push_back(B4Param_RequiredStart.GetName());
-    param_names.push_back(B4Param_RequiredEnd.GetName());
+    param_names.push_back(CBlast4Field::GetName(eBlastOpt_RequiredStart));
+    param_names.push_back(CBlast4Field::GetName(eBlastOpt_RequiredEnd));
 
     // Get the program options 
     if (qsr.CanGetProgram_options()) {
@@ -1129,8 +1092,8 @@ BOOST_AUTO_TEST_CASE(GetSearchStrategy_QueryWithRange) {
 
     // These are the parameters that we are looking for
     vector<string> param_names;
-    param_names.push_back(B4Param_RequiredStart.GetName());
-    param_names.push_back(B4Param_RequiredEnd.GetName());
+    param_names.push_back(CBlast4Field::GetName(eBlastOpt_RequiredStart));
+    param_names.push_back(CBlast4Field::GetName(eBlastOpt_RequiredEnd));
 
     // Get the program options 
     if (qsr.CanGetProgram_options()) {
@@ -1140,11 +1103,11 @@ BOOST_AUTO_TEST_CASE(GetSearchStrategy_QueryWithRange) {
             if (p.NotEmpty()) {
                 BOOST_REQUIRE(p->CanGetValue());
                 found_query_range = true;
-                if (*pname == B4Param_RequiredStart.GetName()) {
+                if (*pname == CBlast4Field::GetName(eBlastOpt_RequiredStart)) {
                     BOOST_REQUIRE_EQUAL((int)query_range.GetFrom(), 
                                         (int)p->GetValue().GetInteger());
                 }
-                if (*pname == B4Param_RequiredEnd.GetName()) {
+                if (*pname == CBlast4Field::GetName(eBlastOpt_RequiredEnd)) {
                     BOOST_REQUIRE_EQUAL((int)query_range.GetTo(), 
                                         (int)p->GetValue().GetInteger());
                 }
@@ -1352,4 +1315,27 @@ BOOST_AUTO_TEST_CASE(ReadArchiveFormatMultipleQueries)
     BOOST_REQUIRE(result_set->GetNumResults() == 3);
 }
 
+class CDiagLevelGuard {
+public:
+    CDiagLevelGuard(EDiagSev target)    { m_Orig = SetDiagPostLevel(target); }
+    ~CDiagLevelGuard()                  { SetDiagPostLevel(m_Orig); }
+private:
+    EDiagSev m_Orig;
+};
+
+BOOST_AUTO_TEST_CASE(GetBlast4Parameters)
+{
+    const string kUnknown("-");
+    CBlast4Field p = CBlast4Field::Get(eBlastOpt_Web_ExclModels);
+    BOOST_REQUIRE(p.GetName() != kUnknown);
+    p = CBlast4Field::Get(eBlastOpt_MbIndexName);
+    BOOST_REQUIRE(p.GetName() != kUnknown);
+
+    // These shouldn't be found, supress diagnostics
+    CDiagLevelGuard g(eDiag_Error);
+    p = CBlast4Field::Get(eBlastOpt_MaxValue);
+    BOOST_REQUIRE(p.GetName() == kUnknown);
+    p = CBlast4Field::Get(eBlastOpt_Program);
+    BOOST_REQUIRE(p.GetName() == kUnknown);
+}
 BOOST_AUTO_TEST_SUITE_END()
