@@ -1,4 +1,4 @@
-/*  $Id: win_mask_config.cpp 245014 2011-02-10 17:14:32Z mozese2 $
+/*  $Id: win_mask_config.cpp 345116 2011-11-22 15:31:45Z morgulis $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -176,7 +176,7 @@ void CWinMaskConfig::AddWinMaskArgs(CArgDescriptions &arg_desc,
                                  ->Allow( "oascii" )
                                  ->Allow( "obinary" ) );
         arg_desc.AddDefaultKey( "smem", "available_memory",
-                                 "memory available for masking stage in megabytes",
+                                "target size of the output file containing the unit counts",
                                  CArgDescriptions::eInteger, "512" );
     }
     if(type == eAny || type >= eGenerateMasksWithDuster)
@@ -261,18 +261,31 @@ CWinMaskConfig::EAppType CWinMaskConfig::s_DetermineAppType(
 }
 
 CMaskWriter*
-CWinMaskConfig::x_GetWriter(const CArgs& args, 
-                            CNcbiOstream& output, 
-                            const string& format)
+CWinMaskConfig::x_GetWriter(const CArgs& args)
 {
+    const string & format( args[kOutputFormat].AsString() );
     CMaskWriter* retval = NULL;
+
     if (format == "interval") {
+        CNcbiOstream& output = args[kOutput].AsOutputFile();
         retval = new CMaskWriterInt(output);
     } else if (format == "fasta") {
+        CNcbiOstream& output = args[kOutput].AsOutputFile();
         retval = new CMaskWriterFasta(output);
-    } else if (NStr::StartsWith(format, "seqloc_")) {
+    } else if (NStr::StartsWith(format, "seqloc_asn1_binary")) {
+        CNcbiOstream& output = args[kOutput].AsOutputFile(CArgValue::fBinary);
         retval = new CMaskWriterSeqLoc(output, format);
+    } else if (NStr::StartsWith(format, "seqloc_")) {
+        CNcbiOstream& output = args[kOutput].AsOutputFile();
+        retval = new CMaskWriterSeqLoc(output, format);
+    } else if (NStr::StartsWith(format, "maskinfo_asn1_binary")) {
+        CNcbiOstream& output = args[kOutput].AsOutputFile(CArgValue::fBinary);
+        retval = 
+            new CMaskWriterBlastDbMaskInfo(output, format, 3,
+                               eBlast_filter_program_windowmasker,
+                               BuildAlgorithmParametersString(args));
     } else if (NStr::StartsWith(format, "maskinfo_")) {
+        CNcbiOstream& output = args[kOutput].AsOutputFile();
         retval = 
             new CMaskWriterBlastDbMaskInfo(output, format, 3,
                                eBlast_filter_program_windowmasker,
@@ -290,11 +303,7 @@ CWinMaskConfig::CWinMaskConfig( const CArgs & args, EAppType type, bool determin
                                      && determine_input ? 
           ( !(args[kInput].AsString() == "-")
             ? new CNcbiIfstream( args[kInput].AsString().c_str() ) 
-            : static_cast<CNcbiIstream*>(&NcbiCin) ) : NULL ), reader( NULL ), 
-      os( app_type >= eGenerateMasks ?
-          ( !(args[kOutput].AsString() == "-")
-            ? new CNcbiOfstream( args[kOutput].AsString().c_str() )
-            : static_cast<CNcbiOstream*>(&NcbiCout) ) : NULL ), writer( NULL ),
+            : static_cast<CNcbiIstream*>(&NcbiCin) ) : NULL ), reader( NULL ), writer( NULL ),
       lstat_name( app_type >= eGenerateMasks ? args["ustat"].AsString() : "" ),
       textend( app_type >= eGenerateMasks && args["t_extend"] ? args["t_extend"].AsInteger() : 0 ), 
       cutoff_score( app_type >= eGenerateMasks && args["t_thres"] ? args["t_thres"].AsInteger() : 0 ),
@@ -359,9 +368,7 @@ CWinMaskConfig::CWinMaskConfig( const CArgs & args, EAppType type, bool determin
             }
         }
 
-        string oformatstr = args[kOutputFormat].AsString();
-
-        writer = x_GetWriter(args, *os, oformatstr);
+        writer = x_GetWriter(args);
 
         set_max_score = args["set_t_high"]  ? args["set_t_high"].AsInteger()
                                             : 0;
@@ -449,7 +456,7 @@ void CWinMaskConfig::FillIdList( const string & file_name,
 }
 
 //----------------------------------------------------------------------------
-const char * CWinMaskConfig::CWinMaskConfigException::GetErrCodeString() const
+const char * CWinMaskConfigException::GetErrCodeString() const
 {
     switch( GetErrCode() )
     {
