@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.cpp 368006 2012-06-29 19:48:05Z rafanovi $
+/*  $Id: blast_app_util.cpp 391263 2013-03-06 18:02:05Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_app_util.cpp 368006 2012-06-29 19:48:05Z rafanovi $";
+    "$Id: blast_app_util.cpp 391263 2013-03-06 18:02:05Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -64,6 +64,27 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 USING_SCOPE(blast);
 
+Int4 CBatchSizeMixer::GetBatchSize(Int4 hits) 
+{
+     if (hits > 0) {
+         double ratio = 1.0 * hits / m_BatchSize;
+         m_Ratio = (m_Ratio < 0) ? ratio 
+                 : k_MixIn * ratio + (1.0 - k_MixIn) * m_Ratio;
+         m_BatchSize = (Int4) (1.0 * k_TargetHits / m_Ratio);
+         if (m_BatchSize > k_MaxBatchSize) {
+             m_BatchSize = k_MaxBatchSize;
+             m_Ratio = -1.0;  // reset the history
+         } else if (m_BatchSize < 100) {
+             m_BatchSize = 100;
+             m_Ratio = -1.0;  // reset the history
+         }
+     } else if (hits == 0) {
+         m_BatchSize = k_MaxBatchSize;
+         m_Ratio = -1.0;
+     }
+     return m_BatchSize;
+}
+
 CRef<blast::CRemoteBlast> 
 InitializeRemoteBlast(CRef<blast::IQueryFactory> queries,
                       CRef<blast::CBlastDatabaseArgs> db_args,
@@ -124,6 +145,41 @@ static string s_FindBlastDbDataLoaderName(const string& dbname, bool is_protein)
     return kEmptyStr;
 }
 
+blast::SDataLoaderConfig 
+InitializeQueryDataLoaderConfiguration(bool query_is_protein, 
+                                       CRef<blast::CLocalDbAdapter> db_adapter)
+{
+    SDataLoaderConfig retval(query_is_protein);
+    retval.OptimizeForWholeLargeSequenceRetrieval();
+
+    /* Load the BLAST database into the data loader configuration for the query
+     * so that if the query sequence(s) are specified as seq-ids, these can be 
+     * fetched from the BLAST database being searched */
+    if (db_adapter->IsBlastDb() &&  /* this is a BLAST database search */
+        retval.m_UseBlastDbs &&   /* the BLAST database data loader is requested */
+        (query_is_protein == db_adapter->IsProtein())) { /* the same database type is used for both queries and subjects */
+        // Make sure we don't add the same database more than once
+        vector<string> default_dbs;
+        NStr::Tokenize(retval.m_BlastDbName, " ", default_dbs);
+        if (default_dbs.size() &&
+            (find(default_dbs.begin(), default_dbs.end(),
+                 db_adapter->GetDatabaseName()) == default_dbs.end())) {
+        CNcbiOstrstream oss;
+        oss << db_adapter->GetDatabaseName() << " " << retval.m_BlastDbName;
+        retval.m_BlastDbName = CNcbiOstrstreamToString(oss);
+    }
+    }
+    if (retval.m_UseBlastDbs) {
+        _TRACE("Initializing query data loader to '" << retval.m_BlastDbName 
+               << "' (" << (query_is_protein ? "protein" : "nucleotide") 
+               << " BLAST database)");
+    }
+    if (retval.m_UseGenbank) {
+        _TRACE("Initializing query data loader to use GenBank data loader");
+    }
+    return retval;
+}
+
 void
 InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args, 
                   CRef<blast::CBlastOptionsHandle> opts_hndl,
@@ -194,7 +250,8 @@ InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
             scope->AddDataLoader(dbloader_name, 
                              CBlastDatabaseArgs::kSubjectsDataLoaderPriority);
             _TRACE("Setting " << dbloader_name << " priority to "
-                   << (int)CBlastDatabaseArgs::kSubjectsDataLoaderPriority);
+                   << (int)CBlastDatabaseArgs::kSubjectsDataLoaderPriority
+                   << " for subjects");
         }
     }
 }
@@ -210,7 +267,8 @@ string RegisterOMDataLoader(CRef<CSeqDB> db_handle)
     CBlastDbDataLoader::SBlastDbParam param(db_handle);
     string retval(CBlastDbDataLoader::GetLoaderNameFromArgs(param));
     _TRACE("Registering " << retval << " at priority " <<
-           (int)CBlastDatabaseArgs::kSubjectsDataLoaderPriority);
+           (int)CBlastDatabaseArgs::kSubjectsDataLoaderPriority 
+           << " for subjects");
     return retval;
 }
 
@@ -369,9 +427,11 @@ s_ImportQueries(const CBlast4_queries& queries,
         CRef<CScope> scope(scope_src.NewScope());
 
         ITERATE(CBlast4_queries::TSeq_loc_list, itr, seqlocs) {
-            CBioseq_Handle bh = scope->GetBioseqHandle(**itr);
-            CConstRef<CBioseq> bioseq = bh.GetCompleteBioseq();
-            out.Write(*bioseq);
+            if ((*itr)->GetId()) {
+                CBioseq_Handle bh = scope->GetBioseqHandle(*(*itr)->GetId());
+                CConstRef<CBioseq> bioseq = bh.GetCompleteBioseq();
+                out.Write(*bioseq);
+            }
         }
         scope.Reset();
         scope_src.RevokeBlastDbDataLoader();
@@ -477,6 +537,14 @@ s_ImportSearchStrategy(CNcbiIstream* in,
     cmdline_args->SetOptionsHandle(opts_hndl);
     const EBlastProgramType prog = opts_hndl->GetOptions().GetProgramType();
     cmdline_args->SetTask(strategy.GetTask());
+#if _DEBUG
+    {
+        char* program_string = 0;
+        BlastNumber2Program(prog, &program_string);
+        _TRACE("EBlastProgramType=" << program_string << " task=" << strategy.GetTask());
+        sfree(program_string);
+    }
+#endif
 
     // Get the subject
     if (override_subject) {
@@ -487,10 +555,8 @@ s_ImportSearchStrategy(CNcbiIstream* in,
         const bool subject_is_protein = Blast_SubjectIsProtein(prog) ? true : false;
 
         if (subj->IsDatabase()) {
-            CBlastOptionsBuilder bob(strategy.GetProgram(), strategy.GetService(), CBlastOptions::eBoth);
-            bob.GetSearchOptions(&strategy.GetAlgoOptions(), &strategy.GetProgramOptions());
-            db_args = s_ImportDatabase(*subj, bob, subject_is_protein,
-                                       is_remote_search);
+            db_args = s_ImportDatabase(*subj, strategy.GetOptionsBuilder(),
+                                       subject_is_protein, is_remote_search);
         } else {
             db_args = s_ImportSubjects(*subj, subject_is_protein);
         }
@@ -724,5 +790,43 @@ ExtractPssmAncillaryData(const CPssmWithParameters& pssm)
                                                              true));
 }
 
+void
+CheckForFreqRatioFile(const string& rps_dbname, CRef<CBlastOptionsHandle>  & opt_handle, bool isRpsblast)
+{
+    bool use_cbs = (opt_handle->GetOptions().GetCompositionBasedStats() == eNoCompositionBasedStats) ? false : true;
+    if(use_cbs) {
+        vector<string> db;
+        NStr::Tokenize(rps_dbname, " ", db);
+        list<string> failed_db;
+        for (unsigned int i=0; i < db.size(); i++) {
+    	    string path;
+    	    try {
+                vector<string> dbpath;
+       	        CSeqDB::FindVolumePaths(db[i], CSeqDB::eProtein, dbpath);
+                path = *dbpath.begin();
+            } catch (const CSeqDBException& e) {
+                 NCBI_RETHROW(e, CBlastException, eRpsInit,
+                              "Cannot retrieve path to RPS database");
+            }
+
+    	    CFile f(path + ".freq");
+            if(!f.Exists()) {
+            	failed_db.push_back(db[i]);
+            }
+
+        }
+        if(!failed_db.empty()) {
+        	opt_handle->SetOptions().SetCompositionBasedStats(eNoCompositionBasedStats);
+        	string all_failed = NStr::Join(failed_db, ", ");
+        	string prog_str = isRpsblast ? "RPSBLAST": "DELTABLAST";
+        	string msg = all_failed + " contain(s) no freq ratios " \
+                     	 + "needed for composition-based statistics.\n" \
+                     	 + prog_str + " will be run without composition-based statistics.";
+        	ERR_POST(Warning << msg);
+        }
+
+    }
+    return;
+}
 
 END_NCBI_SCOPE
