@@ -1,4 +1,4 @@
-/*  $Id: blast_formatter.cpp 334322 2011-09-06 14:50:26Z fongah2 $
+/*  $Id: blast_formatter.cpp 382303 2012-12-04 20:47:00Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_formatter.cpp 334322 2011-09-06 14:50:26Z fongah2 $";
+    "$Id: blast_formatter.cpp 382303 2012-12-04 20:47:00Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -105,18 +105,19 @@ void CBlastFormatterApp::Init()
                   + CBlastVersion().Print());
 
     arg_desc->SetCurrentGroup("Input options");
-    arg_desc->AddDefaultKey("rid", "BLAST_RID", "BLAST Request ID (RID), not compatiable with archive arg.", 
-                     CArgDescriptions::eString, "");
+    arg_desc->AddOptionalKey(kArgRid, "BLAST_RID", "BLAST Request ID (RID)", 
+                     CArgDescriptions::eString);
 
     // add input file for seq-align here?
-    arg_desc->AddOptionalKey("archive", "ArchiveFile", "Archive file of results, not compatiable with rid arg.", 
+    arg_desc->AddOptionalKey(kArgArchive, "ArchiveFile", "File containing BLAST Archive format in ASN.1 (i.e.: output format 11)", 
                      CArgDescriptions::eInputFile);
+    arg_desc->SetDependency(kArgRid, CArgDescriptions::eExcludes, kArgArchive);
 
     CFormattingArgs fmt_args;
     fmt_args.SetArgumentDescriptions(*arg_desc);
 
     arg_desc->SetCurrentGroup("Output configuration options");
-    arg_desc->AddDefaultKey("out", "output_file", "Output file name", 
+    arg_desc->AddDefaultKey(kArgOutput, "output_file", "Output file name", 
                             CArgDescriptions::eOutputFile, "-");
 
     arg_desc->SetCurrentGroup("Miscellaneous options");
@@ -137,12 +138,14 @@ CBlastFormatterApp::x_QueryBioseqToSSeqLoc(const CBioseq& bioseq,
     static bool first_time = true;
     _ASSERT(scope);
 
-    scope->AddBioseq(bioseq);
     if ( !HasRawSequenceData(bioseq) && first_time ) {
         _ASSERT(m_QueryScopeSource);
         m_QueryScopeSource->AddDataLoaders(scope);
         first_time = false;
     }
+    else {
+        scope->AddBioseq(bioseq);
+    }
     CRef<CSeq_loc> seqloc(new CSeq_loc);
     seqloc->SetWhole().Assign(*bioseq.GetFirstId());
     return SSeqLoc(seqloc, scope);
@@ -240,8 +243,9 @@ int CBlastFormatterApp::PrintFormattedOutput(void)
 {
     int retval = 0;
     const CArgs& args = GetArgs();
-    const string& kRid = args["rid"].AsString();
-    CNcbiOstream& out = args["out"].AsOutputFile();
+    const string& kRid = args[kArgRid].HasValue() 
+        ? args[kArgRid].AsString() : kEmptyStr;
+    CNcbiOstream& out = args[kArgOutput].AsOutputFile();
     CFormattingArgs fmt_args;
 
     CRef<CBlastOptionsHandle> opts_handle = m_RmtBlast->GetSearchOptions();
@@ -358,19 +362,28 @@ int CBlastFormatterApp::Run(void)
     const CArgs& args = GetArgs();
 
     try {
-        const string& kRid = args["rid"].AsString();
+        if (args[kArgArchive].HasValue()) {
+            CNcbiIstream& istr = args[kArgArchive].AsInputFile();
+            try { m_RmtBlast.Reset(new CRemoteBlast(istr)); }
+            catch (const CBlastException& e) {
+                if (e.GetErrCode() == CBlastException::eInvalidArgument) {
+                    NCBI_RETHROW(e, CInputException, eInvalidInput,
+                                 "Invalid input format for BLAST Archive.");
+                }
+            }
 
-        if (kRid == "")
-        {
-            CNcbiIstream& istr = args["archive"].AsInputFile();
-    		m_RmtBlast.Reset(new CRemoteBlast(istr));
-
-            while (m_RmtBlast->LoadFromArchive())
-                status = PrintFormattedOutput();
+            try {
+                while (m_RmtBlast->LoadFromArchive())
+                    status = PrintFormattedOutput();
+            } catch (const CSerialException& e) {
+                NCBI_RETHROW(e, CInputException, eInvalidInput,
+                             "Invalid input format for BLAST Archive.");
+            }
 
     		return status;
         }
 
+        const string kRid = args[kArgRid].AsString();
         m_RmtBlast.Reset(new CRemoteBlast(kRid));
         {{
             CDebugArgs debug_args;
