@@ -1,4 +1,4 @@
-/*  $Id: blastn_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
+/*  $Id: blastn_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: blastn_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
+	"$Id: blastn_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -103,19 +103,6 @@ int CBlastnApp::Run(void)
         }
         const CBlastOptions& opt = opts_hndl->GetOptions();
 
-        /*** Get the query sequence(s) ***/
-        CRef<CQueryOptionsArgs> query_opts = 
-            m_CmdLineArgs->GetQueryOptionsArgs();
-        SDataLoaderConfig dlconfig(query_opts->QueryIsProtein());
-        dlconfig.OptimizeForWholeLargeSequenceRetrieval();
-        CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
-                                     query_opts->UseLowercaseMasks(),
-                                     query_opts->GetParseDeflines(),
-                                     query_opts->GetRange());
-        iconfig.SetQueryLocalIdMode();
-        CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
-        CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
-
         /*** Initialize the database/subject ***/
         CRef<CBlastDatabaseArgs> db_args(m_CmdLineArgs->GetBlastDatabaseArgs());
         CRef<CLocalDbAdapter> db_adapter;
@@ -124,6 +111,20 @@ int CBlastnApp::Run(void)
                          db_adapter, scope);
         _ASSERT(db_adapter && scope);
 
+        /*** Get the query sequence(s) ***/
+        CRef<CQueryOptionsArgs> query_opts = 
+            m_CmdLineArgs->GetQueryOptionsArgs();
+        SDataLoaderConfig dlconfig =
+            InitializeQueryDataLoaderConfiguration(query_opts->QueryIsProtein(),
+                                                   db_adapter);
+        CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
+                                     query_opts->UseLowercaseMasks(),
+                                     query_opts->GetParseDeflines(),
+                                     query_opts->GetRange());
+        iconfig.SetQueryLocalIdMode();
+        CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
+        CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
+
         // Initialize the megablast database index now so we can know whether an indexed search will be run.
         // This is only important for the reference in the report, but would be done anyway.
         if (opt.GetUseIndex() && !m_CmdLineArgs->ExecuteRemotely()) {
@@ -155,6 +156,12 @@ int CBlastnApp::Run(void)
         
         formatter.PrintProlog();
         
+        CBatchSizeMixer mixer(2000000,   // targeted at 2M hits per batch
+                              SplitQuery_GetChunkSize(opt.GetProgram())-1000);
+        if (!m_CmdLineArgs->ExecuteRemotely()) {
+            input.SetBatchSize(mixer.GetBatchSize());
+        }
+
         /*** Process the input ***/
         for (; !input.End(); formatter.ResetScopeHistory()) {
 
@@ -175,6 +182,7 @@ int CBlastnApp::Run(void)
                 CLocalBlast lcl_blast(queries, opts_hndl, db_adapter);
                 lcl_blast.SetNumberOfThreads(m_CmdLineArgs->GetNumThreads());
                 results = lcl_blast.Run();
+                input.SetBatchSize(mixer.GetBatchSize(lcl_blast.GetNumExtensions()));
             }
 
             if (fmt_args->ArchiveFormatRequested(args)) {
