@@ -1,4 +1,4 @@
-/*  $Id: deltablast_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
+/*  $Id: deltablast_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-        "$Id: deltablast_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
+        "$Id: deltablast_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -161,6 +161,32 @@ static void s_AddSeqTitleToPssm(CRef<CPssmWithParameters> pssm,
     }
 }
 
+// Add sequence data to pssm query
+static void s_AddSeqDataToPssm(CRef<CPssmWithParameters> pssm,
+                               CRef<CBlastQueryVector> query_batch,
+                               CRef<CScope> scope)
+{
+    CConstRef<CSeq_id> query_id =
+        query_batch->GetBlastSearchQuery(0)->GetQueryId();
+
+    // first make sure that query id and pssm query id are the same
+    if (!pssm->GetPssm().GetQuery().GetSeq().GetFirstId()->Match(*query_id)) {
+        NCBI_THROW(CException, eInvalid, "Query and PSSM sequence ids do not "
+                   "match");
+    }
+    
+    CBioseq_Handle bhandle = scope->GetBioseqHandle(*query_id);
+    CConstRef<CBioseq> scope_bioseq = bhandle.GetCompleteBioseq();
+
+    // set sequence data only if query bioseq has them and pssm does not
+    if (scope_bioseq->GetInst().IsSetSeq_data()
+        && !pssm->GetPssm().GetQuery().GetSeq().GetInst().IsSetSeq_data()) {
+        const CSeq_data& seq_data = scope_bioseq->GetInst().GetSeq_data();
+        pssm->SetQuery().SetSeq().SetInst().SetSeq_data(
+                                          const_cast<CSeq_data&>(seq_data));
+    }
+}
+
 int CDeltaBlastApp::Run(void)
 {
     int status = BLAST_EXIT_SUCCESS;
@@ -176,23 +202,6 @@ int CDeltaBlastApp::Run(void)
         CRef<CBlastOptionsHandle> opts_hndl(&*m_CmdLineArgs->SetOptions(args));
         const CBlastOptions& opt = opts_hndl->GetOptions();
 
-        /*** Get the query sequence(s) ***/
-        CRef<CQueryOptionsArgs> query_opts = 
-            m_CmdLineArgs->GetQueryOptionsArgs();
-        SDataLoaderConfig dlconfig(query_opts->QueryIsProtein());
-        dlconfig.OptimizeForWholeLargeSequenceRetrieval();
-        CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
-                                     query_opts->UseLowercaseMasks(),
-                                     query_opts->GetParseDeflines(),
-                                     query_opts->GetRange());
-        iconfig.SetQueryLocalIdMode();
-        CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
-        size_t query_batch_size = m_CmdLineArgs->GetQueryBatchSize();
-        if (m_CmdLineArgs->GetNumberOfPsiBlastIterations() > 1) {
-            query_batch_size = 1;
-        }
-        CBlastInput input(&fasta, query_batch_size);
-
         /*** Initialize the database/subject ***/
         CRef<CBlastDatabaseArgs> db_args(m_CmdLineArgs->GetBlastDatabaseArgs());
         CRef<CLocalDbAdapter> db_adapter;
@@ -201,6 +210,26 @@ int CDeltaBlastApp::Run(void)
                          db_adapter, scope);
         _ASSERT(db_adapter && scope);
 
+        /*** Get the query sequence(s) ***/
+        CRef<CQueryOptionsArgs> query_opts = 
+            m_CmdLineArgs->GetQueryOptionsArgs();
+        SDataLoaderConfig dlconfig =
+            InitializeQueryDataLoaderConfiguration(query_opts->QueryIsProtein(),
+                                                   db_adapter);
+        CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
+                                     query_opts->UseLowercaseMasks(),
+                                     query_opts->GetParseDeflines(),
+                                     query_opts->GetRange());
+        iconfig.SetQueryLocalIdMode();
+        CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
+        size_t query_batch_size = m_CmdLineArgs->GetQueryBatchSize();
+        if (m_CmdLineArgs->GetNumberOfPsiBlastIterations() > 1
+            || m_CmdLineArgs->ExecuteRemotely()) {
+
+            query_batch_size = 1;
+        }
+        CBlastInput input(&fasta, query_batch_size);
+
         /*** Initialize the domain database ***/
         CRef<CLocalDbAdapter> domain_db_adapter(new CLocalDbAdapter(
                                     *m_CmdLineArgs->GetDomainDatabase()));
@@ -291,6 +320,15 @@ int CDeltaBlastApp::Run(void)
                 s_AddSeqTitleToPssm(pssm, query_batch, scope);
             }
 
+            // remote blast remves sequence data from pssm for known ids
+            // the data must be added if pssm is requested after remote search
+            if (m_CmdLineArgs->ExecuteRemotely()
+                && (m_CmdLineArgs->SaveCheckpoint()
+                    || m_CmdLineArgs->SaveAsciiPssm())) {
+
+                s_AddSeqDataToPssm(pssm, query_batch, scope);
+            }
+
             // only one PSI-BLAST iteration requested, then print results
             // (the first PIS-BLAST iteration is done by DELTA-BLAST)
             if (m_CmdLineArgs->GetNumberOfPsiBlastIterations() == 1) {
