@@ -1,4 +1,4 @@
-/*  $Id: tblastx_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
+/*  $Id: tblastx_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: tblastx_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
+	"$Id: tblastx_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -103,19 +103,6 @@ int CTblastxApp::Run(void)
         }
         const CBlastOptions& opt = opts_hndl->GetOptions();
 
-        /*** Get the query sequence(s) ***/
-        CRef<CQueryOptionsArgs> query_opts = 
-            m_CmdLineArgs->GetQueryOptionsArgs();
-        SDataLoaderConfig dlconfig(query_opts->QueryIsProtein());
-        dlconfig.OptimizeForWholeLargeSequenceRetrieval();
-        CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
-                                     query_opts->UseLowercaseMasks(),
-                                     query_opts->GetParseDeflines(),
-                                     query_opts->GetRange());
-        iconfig.SetQueryLocalIdMode();
-        CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
-        CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
-
         /*** Initialize the database/subject ***/
         CRef<CBlastDatabaseArgs> db_args(m_CmdLineArgs->GetBlastDatabaseArgs());
         CRef<CLocalDbAdapter> db_adapter;
@@ -124,6 +111,20 @@ int CTblastxApp::Run(void)
                          db_adapter, scope);
         _ASSERT(db_adapter && scope);
 
+        /*** Get the query sequence(s) ***/
+        CRef<CQueryOptionsArgs> query_opts = 
+            m_CmdLineArgs->GetQueryOptionsArgs();
+        SDataLoaderConfig dlconfig =
+            InitializeQueryDataLoaderConfiguration(query_opts->QueryIsProtein(),
+                                                   db_adapter);
+        CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
+                                     query_opts->UseLowercaseMasks(),
+                                     query_opts->GetParseDeflines(),
+                                     query_opts->GetRange());
+        iconfig.SetQueryLocalIdMode();
+        CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
+        CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
+
         /*** Get the formatting options ***/
         CRef<CFormattingArgs> fmt_args(m_CmdLineArgs->GetFormattingArgs());
         CBlastFormat formatter(opt, *db_adapter,
