@@ -1,5 +1,5 @@
 #! /usr/bin/perl -w
-# $Id: update_blastdb.pl 348105 2011-12-23 16:13:19Z camacho $
+# $Id: update_blastdb.pl 354828 2012-02-29 20:39:04Z camacho $
 # ===========================================================================
 #
 #                            PUBLIC DOMAIN NOTICE
@@ -80,7 +80,7 @@ my $exit_code = 0;
 # Connect and download files
 my $ftp = &connect_to_ftp() unless ($opt_show_version);
 if ($opt_show_version) {
-    my $revision = '$Revision: 348105 $';
+    my $revision = '$Revision: 354828 $';
     $revision =~ s/\$Revision: | \$//g;
     print "$0 version $revision\n";
 } elsif ($opt_showall) {
@@ -325,6 +325,8 @@ update_blastdb.pl [options] blastdb ...
 Downloads, decompresses the archives in the current working directory, and
 deletes the downloaded archive to save disk space, while preserving the
 archive checksum files (default: false).
+B<Note>: Using this option may require more computing resources than using
+your system's native decompression tool(s).
 
 =item B<--showall>
 
