@@ -1,4 +1,4 @@
-/*  $Id: blastdbcmd.cpp 361739 2012-05-03 19:39:02Z camacho $
+/*  $Id: blastdbcmd.cpp 389735 2013-02-20 18:16:58Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blastdbcmd.cpp 361739 2012-05-03 19:39:02Z camacho $";
+    "$Id: blastdbcmd.cpp 389735 2013-02-20 18:16:58Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -83,6 +83,8 @@ private:
     ENa_strand m_Strand;
     /// output is FASTA
     bool m_FASTA;
+    /// output is ASN.1 defline
+    bool m_Asn1DeflineOutput;
     /// should we find duplicate entries? 
     bool m_GetDuplicates;
     /// should we output target sequence only?
@@ -118,6 +120,12 @@ private:
     /// @param retval the return value where the queries will be added [in|out]
     /// @param entry the user's query [in]
     void x_AddOid(CBlastDBCmdApp::TQueries& retval, const int oid, bool check=false) const;
+
+    /// Process batch entry with range, strand and filter id
+    /// @param args program input args
+    /// @param seq_fmt sequence formatter object
+    /// @return 0 on sucess; 1 if some queries were not processed
+    int x_ProcessBatchEntry(const CArgs& args, CSeqFormatter & seq_fmt);
 };
 
 int
@@ -166,7 +174,7 @@ CBlastDBCmdApp::x_AddOid(CBlastDBCmdApp::TQueries& retval,
     } 
 
     // FASTA output just need one id
-    if (m_FASTA) {
+    if (m_FASTA || m_Asn1DeflineOutput) {
         CRef<CBlastDBSeqId> blastdb_seqid(new CBlastDBSeqId());
         blastdb_seqid->SetOID(oid);
         retval.push_back(blastdb_seqid);
@@ -224,18 +232,6 @@ CBlastDBCmdApp::x_GetQueries(CBlastDBCmdApp::TQueries& retval) const
             err_found += x_AddSeqId(retval, entry);
         }
 
-    } else if (args["entry_batch"].HasValue()) {
-
-        CNcbiIstream& input = args["entry_batch"].AsInputFile();
-        retval.reserve(256); // arbitrary value
-        while (input) {
-            string line;
-            NcbiGetlineEOL(input, line);
-            if ( !line.empty() ) {
-                err_found += x_AddSeqId(retval, line);
-            }
-        }
-
     } else {
         NCBI_THROW(CInputException, eInvalidInput, 
                    "Must specify query type: one of 'entry', 'entry_batch', or 'pig'");
@@ -335,11 +331,24 @@ CBlastDBCmdApp::x_ProcessSearchRequest()
     if (args["outfmt"].HasValue()) {
         outfmt = args["outfmt"].AsString();
         m_FASTA = false;
+        m_Asn1DeflineOutput = false;
+
+        if (outfmt.find("%f") != string::npos && 
+            outfmt.find("%d") != string::npos) {
+            NCBI_THROW(CInputException, eInvalidInput, 
+                "The %d and %f output format options cannot be specified together.");
+        }
+
         // If "%f" is found within outfmt, discard everything else
         if (outfmt.find("%f") != string::npos) {
             outfmt = "%f";
             m_FASTA = true;
         }
+        // If "%d" is found within outfmt, discard everything else
+        if (outfmt.find("%d") != string::npos) {
+            outfmt = "%d";
+            m_Asn1DeflineOutput = true;
+        }
         if (outfmt.find("%m") != string::npos) {
             int algo_id = 0;
             size_t i = outfmt.find("%m") + 2;
@@ -379,6 +388,10 @@ CBlastDBCmdApp::x_ProcessSearchRequest()
         return errors_found ? 1 : 0;
     }
 
+    if (args["entry_batch"].HasValue()) {
+       	return x_ProcessBatchEntry(args, seq_fmt);
+    }
+
     TQueries queries;
 	errors_found = (x_GetQueries(queries) > 0 ? true : false);
     _ASSERT( !queries.empty() );
@@ -397,6 +410,65 @@ CBlastDBCmdApp::x_ProcessSearchRequest()
     return errors_found ? 1 : 0;
 }
 
+int CBlastDBCmdApp::x_ProcessBatchEntry(const CArgs& args, CSeqFormatter & seq_fmt)
+{
+    CNcbiIstream& input = args["entry_batch"].AsInputFile();
+    bool err_found = false;
+    while (input) {
+        string line;
+        NcbiGetlineEOL(input, line);
+        if ( !line.empty() ) {
+        	vector<string> tmp;
+        	NStr::Tokenize(line, " \t", tmp, NStr::fSplit_MergeDelims);
+        	if(tmp.empty())
+        		continue;
+
+        	TQueries queries;
+        	if(x_AddSeqId(queries, tmp[0]) > 0 )
+        			err_found = true;
+
+            if(queries.empty())
+            	continue;
+
+           	TSeqRange seq_range(TSeqRange::GetEmpty());
+            ENa_strand seq_strand = eNa_strand_plus;
+           	int seq_algo_id = -1;
+
+           	for(unsigned int i=1; i < tmp.size(); i++) {
+
+           		if(tmp[i].find('-')!= string::npos)
+            	{
+            		try {
+            			seq_range = ParseSequenceRangeOpenEnd(tmp[i]);
+            		} catch (...) {
+            			seq_range = TSeqRange::GetEmpty();
+            		}
+            	}
+            	else if (!m_DbIsProtein && NStr::EqualNocase(tmp[i].c_str(), "minus")) {
+            		seq_strand = eNa_strand_minus;
+            	}
+            	else {
+            		seq_algo_id = NStr::StringToNonNegativeInt(tmp[i]);
+            	}
+            }
+
+           	seq_fmt.SetConfig(seq_range, seq_strand, seq_algo_id);
+            NON_CONST_ITERATE(TQueries, itr, queries) {
+            	try {
+            		seq_fmt.Write(**itr);
+            	} catch (const CException& e) {
+                     ERR_POST(Error << e.GetMsg());
+                     err_found = true;
+            	} catch (...) {
+                  	ERR_POST(Error << "Failed to retrieve requested item");
+                   	err_found = true;
+            	}
+            }
+        }
+    }
+    return err_found ? 1:0;
+}
+
 void CBlastDBCmdApp::Init()
 {
     HideStdArgs(fHideConffile | fHideFullVersion | fHideXmlHelp | fHideDryRun);
@@ -425,9 +497,13 @@ void CBlastDBCmdApp::Init()
                      CArgDescriptions::eString);
 
     arg_desc->AddOptionalKey("entry_batch", "input_file", 
-                 "Input file for batch processing (Format: one entry per line)",
+                 "Input file for batch processing (Format: one entry per line, seq id \n"
+		 "followed by optional space-delimited specifier(s) [range|strand|mask_algo_id]",
                  CArgDescriptions::eInputFile);
     arg_desc->SetDependency("entry_batch", CArgDescriptions::eExcludes, "entry");
+    arg_desc->SetDependency("entry_batch", CArgDescriptions::eExcludes, "range");
+    arg_desc->SetDependency("entry_batch", CArgDescriptions::eExcludes, "strand");
+    arg_desc->SetDependency("entry_batch", CArgDescriptions::eExcludes, "mask_sequence_with");
 
     arg_desc->AddOptionalKey("pig", "PIG", "PIG to retrieve", 
                              CArgDescriptions::eInteger);
@@ -484,12 +560,21 @@ void CBlastDBCmdApp::Init()
             "\t\t%L means common taxonomic name\n"
             "\t\t%S means scientific name\n"
             "\t\t%P means PIG\n"
+#if _BLAST_DEBUG
+            "\t\t%d means defline in text ASN.1 format\n"
+            "\t\t%b means Bioseq in text ASN.1 format\n"
+#endif /* _BLAST_DEBUG */
             "\t\t%m means sequence masking data.\n"
             "\t\t   Masking data will be displayed as a series of 'N-M' values\n"
             "\t\t   separated by ';' or the word 'none' if none are available.\n"
+#if _BLAST_DEBUG
+            "\tIf '%f' or '%d' are specified, all other format specifiers are ignored.\n"
+            "\tFor every format except '%f' and '%d', each line of output will "
+#else
             "\tIf '%f' is specified, all other format specifiers are ignored.\n"
             "\tFor every format except '%f', each line of output will "
-            "correspond to\n\ta sequence.\n",
+#endif /* _BLAST_DEBUG */
+            "correspond\n\tto a sequence.\n",
             CArgDescriptions::eString, "%f");
 
     //arg_desc->AddDefaultKey("target_only", "value",
@@ -572,6 +657,9 @@ int CBlastDBCmdApp::Run(void)
     int status = 0;
     const CArgs& args = GetArgs();
 
+    // Silences warning in CSeq_id for CSeq_id::fParse_PartialOK
+    SetDiagFilter(eDiagFilter_Post, "!(1306.10)");
+
     try {
         CNcbiOstream& out = args["out"].AsOutputFile();
         if (args["show_blastdb_search_path"]) {
