@@ -1,4 +1,4 @@
-/*  $Id: makeprofiledb.cpp 373329 2012-08-28 19:33:38Z rafanovi $
+/*  $Id: makeprofiledb.cpp 387634 2013-01-30 22:56:11Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -27,7 +27,7 @@
  *
  */
 
-/** @file makeblastrpsdb.cpp
+/** @file makeprofiledb.cpp
  * Command line tool to create RPS,COBALT & DELTA BLAST databases.
  * This is the successor to formatrpsdb from the C toolkit
  */
@@ -58,6 +58,7 @@ static char const rcsid[] = "";
 #include <objects/scoremat/LoopConstraint.hpp>
 #include <algo/blast/core/blast_aalookup.h>
 #include <algo/blast/core/blast_options.h>
+#include <algo/blast/core/ncbi_math.h>
 #include <objtools/blast/seqdb_writer/writedb.hpp>
 #include <objtools/blast/seqdb_writer/taxid_set.hpp>
 
@@ -210,7 +211,7 @@ private:
         		gap_open(0), gap_extend(0),scale_factor(0), curr_seq_offset(0),
         		query_options(NULL), lookup_options(NULL), lookup(NULL)
         { };
-    };
+    }s;
 
     enum CheckInputScoremat_RV
     {
@@ -239,6 +240,8 @@ private:
     bool x_ValidateCd(const list<double>& freqs, const list<double>& observ, unsigned int alphabet_size);
     void x_WrapUpDelta(CTmpFile & tmp_obsr_file, CTmpFile & tmp_freq_file);
     vector<string> x_CreateDeltaList(void);
+    void x_UpdateFreqRatios(const CPssmWithParameters & pssm_p, Int4 seq_index, Int4 seq_size);
+    bool x_IsUpdateFreqRatios(const CPssm & p);
 
     // Data
     CNcbiOstream * m_LogFile;
@@ -268,6 +271,8 @@ private:
 	list<Int4> m_ObsrOffsets;
 	Int4 m_CurrFreqOffset;
 	Int4 m_CurrObsrOffset;
+
+	bool m_UpdateFreqRatios;
 };
 
 CMakeProfileDBApp::CMakeProfileDBApp(void)
@@ -276,7 +281,8 @@ CMakeProfileDBApp::CMakeProfileDBApp(void)
                   m_OutDbType(kEmptyStr), m_CreateIndexFile(false),m_GapOpenPenalty(0),
                   m_GapExtPenalty(0), m_PssmScaleFactor(0),m_Matrix(kEmptyStr),  m_op_mode(op_invalid),
                   m_binary_scoremat(false), m_Taxids(new CTaxIdSet()), m_Done(false),
-                  m_ObsrvThreshold(0), m_ExcludeInvalid(false), m_CurrFreqOffset(0), m_CurrObsrOffset(0)
+                  m_ObsrvThreshold(0), m_ExcludeInvalid(false), m_CurrFreqOffset(0), m_CurrObsrOffset(0),
+                  m_UpdateFreqRatios(true)
 {
 	CRef<CVersion> version(new CVersion());
 	version->SetVersionInfo(new CBlastVersion());
@@ -302,16 +308,16 @@ CMakeProfileDBApp::~CMakeProfileDBApp()
 			string rps_str = m_OutDbName + ".rps";
 		 	string lookup_str = m_OutDbName + ".loo";
 		 	string aux_str = m_OutDbName + ".aux";
+		 	string freq_str = m_OutDbName + ".freq";
 		 	CFile(rps_str).Remove();
 		 	CFile(lookup_str).Remove();
 		 	CFile(aux_str).Remove();
+		 	CFile(freq_str).Remove();
 
 		 	if(op_cobalt == m_op_mode)
 		 	{
 				string blocks_str = m_OutDbName + ".blocks";
-		 		string freq_str = m_OutDbName + ".freq";
 		 		CFile(blocks_str).Remove();
-		 		CFile(freq_str).Remove();
 		 	}
 
 		 	if(op_delta == m_op_mode)
@@ -324,6 +330,7 @@ CMakeProfileDBApp::~CMakeProfileDBApp()
 
 		 	if(m_OutputDb.NotEmpty())
 		 	{
+		 		m_OutputDb->Close();
 		 		vector<string> tmp_files;
 		 		m_OutputDb->ListFiles(tmp_files);
 		 		for(vector<string>::iterator itr = tmp_files.begin();
@@ -562,6 +569,18 @@ CMakeProfileDBApp::x_CheckInputScoremat(const CPssmWithParameters & pssm_w_param
 			NCBI_THROW(CInputException, eInvalidInput,  err);
 		}
 
+		// First time around
+		if(NULL == m_RpsDbInfo.lookup)
+		{
+			m_UpdateFreqRatios = x_IsUpdateFreqRatios(pssm);
+		}
+
+		if(m_UpdateFreqRatios && (!pssm.IsSetIntermediateData()|| !pssm.GetIntermediateData().IsSetFreqRatios()))
+		{
+			string err = filename + " contains no frequence ratios for building database";
+			NCBI_THROW(CInputException, eInvalidInput,  err);
+		}
+
 		if(op_cobalt == m_op_mode)
 		{
 			if(!pssm_w_parameters.IsSetParams() || !pssm_w_parameters.GetParams().IsSetConstraints() ||
@@ -570,13 +589,6 @@ CMakeProfileDBApp::x_CheckInputScoremat(const CPssmWithParameters & pssm_w_param
 				string err = filename + " contains no core block to build cobalt database";
 				NCBI_THROW(CInputException, eInvalidInput,  err);
 			}
-
-			if(!pssm.IsSetIntermediateData()|| !pssm.GetIntermediateData().IsSetFreqRatios())
-			{
-				string err = filename + " contains no frequence ratios for building cobalt database";
-				NCBI_THROW(CInputException, eInvalidInput,  err);
-			}
-
 		}
 
 		if(pssm.IsSetFinalData())
@@ -606,6 +618,17 @@ CMakeProfileDBApp::x_CheckInputScoremat(const CPssmWithParameters & pssm_w_param
 	return sm;
 }
 
+bool CMakeProfileDBApp::x_IsUpdateFreqRatios(const CPssm & p)
+{
+	if(op_cobalt == m_op_mode)
+		return true;
+
+	if(!p.IsSetIntermediateData()|| !p.GetIntermediateData().IsSetFreqRatios())
+		return false;
+
+	return true;
+}
+
 void CMakeProfileDBApp::x_InitOutputDb(void)
 {
 	CWriteDB::EIndexType index_type = (m_CreateIndexFile == true ? CWriteDB::eDefault : CWriteDB::eNoIndex);
@@ -634,18 +657,28 @@ void CMakeProfileDBApp::x_InitRPSDbInfo(Int4 num_files)
      if (!m_RpsDbInfo.aux_file.is_open())
     	 NCBI_THROW(CSeqDBException, eFileErr,"Failed to open output .aux file");
 
+	 string freq_str = m_OutDbName + ".freq";
+	 m_RpsDbInfo.freq_file.open(freq_str.c_str(), IOS_BASE::out|IOS_BASE::binary);
+	 if (!m_RpsDbInfo.freq_file.is_open())
+		 NCBI_THROW(CSeqDBException, eFileErr,"Failed to open output .freq file");
+
      /* Write the magic numbers to the PSSM file */
 
      Int4 version = RPS_DATABASE_VERSION;
       m_RpsDbInfo.pssm_file.write ((char *)&version , sizeof(Int4));
+      m_RpsDbInfo.freq_file.write ((char *)&version , sizeof(Int4));
 
      /* Fill in space for the sequence offsets. The PSSM
         data gets written after this list of integers. Also
         write the number of sequences to the PSSM file */
 
       m_RpsDbInfo.pssm_file.write((char *) &num_files, sizeof(Int4));
+      m_RpsDbInfo.freq_file.write((char *) &num_files, sizeof(Int4));
      for (Int4 i = 0; i <= num_files; i++)
+     {
     	 m_RpsDbInfo.pssm_file.write((char *)&i, sizeof(Int4));
+    	 m_RpsDbInfo.freq_file.write((char *)&i, sizeof(Int4));
+     }
 
      if(op_cobalt == m_op_mode)
      {
@@ -653,13 +686,9 @@ void CMakeProfileDBApp::x_InitRPSDbInfo(Int4 num_files)
     	 m_RpsDbInfo.blocks_file.open(blocks_str.c_str());
     	 if (!m_RpsDbInfo.blocks_file.is_open())
     		 NCBI_THROW(CSeqDBException, eFileErr,"Failed to open output .blocks file");
-
-    	 string freq_str = m_OutDbName + ".freq";
-    	 m_RpsDbInfo.freq_file.open(freq_str.c_str(), IOS_BASE::out|IOS_BASE::binary);
-    	 if (!m_RpsDbInfo.freq_file.is_open())
-    		 NCBI_THROW(CSeqDBException, eFileErr,"Failed to open output .freq file");
      }
 
+
      m_RpsDbInfo.curr_seq_offset = 0;
      //Init them to input arg values first , may change after reading in the first sequence
      m_RpsDbInfo.gap_extend = m_GapExtPenalty;
@@ -884,28 +913,36 @@ void CMakeProfileDBApp::x_RPSUpdateStatistics(CPssmWithParameters & seq, Int4 se
 		 count++;
 		 ++itr;
 	 }
+	 return;
+ }
+void CMakeProfileDBApp::x_UpdateFreqRatios(const CPssmWithParameters & pssm_p, Int4 seq_index, Int4 seq_size)
+ {
+	if (!m_UpdateFreqRatios)
+		return;
 
+	 const CPssm & pssm = pssm_p.GetPssm();
 	 // Update .freq file
 	 Int4 i = 0;
 	 Int4 j = 0;
-	 double row[BLASTAA_SIZE];
+	 Int4 row[BLASTAA_SIZE];
 	 Int4 alphabet_size = pssm.GetNumRows();
 
 	 const list<double> & freq_ratios = pssm.GetIntermediateData().GetFreqRatios();
 	 list<double>::const_iterator itr_fr = freq_ratios.begin();
+    m_RpsDbInfo.freq_file.seekp(0, ios_base::end);
 
 	 if (pssm.GetByRow() == FALSE) {
 	    for (i = 0; i < seq_size; i++) {
 	        for (j = 0; j < alphabet_size; j++) {
 	            if (itr_fr == freq_ratios.end())
 	                break;
-	            row[j] = *itr_fr;
+	            row[j] = (Int4) BLAST_Nint(*itr_fr * FREQ_RATIO_SCALE);
 	            ++itr_fr;
 	        }
 	        for ( ;j < BLASTAA_SIZE; j++) {
 	        	row[j] = 0;
 	        }
-	        m_RpsDbInfo.freq_file.write((const char *)row, sizeof(double)*BLASTAA_SIZE);
+	        m_RpsDbInfo.freq_file.write((const char *)row, sizeof(Int4)*BLASTAA_SIZE);
 	    }
     }
     else {
@@ -913,15 +950,17 @@ void CMakeProfileDBApp::x_RPSUpdateStatistics(CPssmWithParameters & seq, Int4 se
 
 	    for (i = 0; i < seq_size; i++) {
 	        for (j = 0; j < BLASTAA_SIZE; j++) {
-	            row[j] = (*matrix)(i,j );
+	            row[j] = (Int4) BLAST_Nint((*matrix)(i,j ) * FREQ_RATIO_SCALE);
 	        }
-	        m_RpsDbInfo.freq_file.write((const char *)row, sizeof(double)*BLASTAA_SIZE);
+	        m_RpsDbInfo.freq_file.write((const char *)row, sizeof(Int4)*BLASTAA_SIZE);
 	    }
     }
 
-	 memset(row, 0, sizeof(row));
-	m_RpsDbInfo.freq_file.write((const char *)row, sizeof(double)*BLASTAA_SIZE);
+	memset(row, 0, sizeof(row));
+	m_RpsDbInfo.freq_file.write((const char *)row, sizeof(Int4)*BLASTAA_SIZE);
 
+    m_RpsDbInfo.freq_file.seekp( 8 + (seq_index) * sizeof(Int4), ios_base::beg);
+    m_RpsDbInfo.freq_file.write((const char *) &m_RpsDbInfo.curr_seq_offset, sizeof(Int4));
 	return;
  }
 
@@ -1059,6 +1098,8 @@ void CMakeProfileDBApp::x_RPS_DbClose(void)
 
     m_RpsDbInfo.pssm_file.seekp(8 + (m_RpsDbInfo.num_seqs) * sizeof(Int4), ios::beg);
     m_RpsDbInfo.pssm_file.write((const char *) &m_RpsDbInfo.curr_seq_offset, sizeof(Int4));
+    m_RpsDbInfo.freq_file.seekp(8 + (m_RpsDbInfo.num_seqs) * sizeof(Int4), ios::beg);
+    m_RpsDbInfo.freq_file.write((const char *) &m_RpsDbInfo.curr_seq_offset, sizeof(Int4));
 
     /* Pack the lookup table into its compressed form */
     if(NULL == m_RpsDbInfo.lookup)
@@ -1153,13 +1194,18 @@ void CMakeProfileDBApp::x_RPS_DbClose(void)
     m_RpsDbInfo.pssm_file.close();
     m_RpsDbInfo.aux_file.flush();
     m_RpsDbInfo.aux_file.close();
+	m_RpsDbInfo.freq_file.flush();
+	m_RpsDbInfo.freq_file.close();
 
     if(op_cobalt == m_op_mode)
     {
     	m_RpsDbInfo.blocks_file.flush();
     	m_RpsDbInfo.blocks_file.close();
-    	m_RpsDbInfo.freq_file.flush();
-    	m_RpsDbInfo.freq_file.close();
+    }
+    else if(!m_UpdateFreqRatios)
+    {
+    	string freq_str = m_OutDbName + ".freq";
+	 	CFile(freq_str).Remove();
     }
 
 }
@@ -1238,6 +1284,7 @@ int CMakeProfileDBApp::Run(void)
 
 		x_RPSUpdatePSSM(pssm, seq_index, seq_size);
 		x_RPSUpdateLookup(seq_size);
+		x_UpdateFreqRatios(pssm_w_parameters, seq_index, seq_size);
 
 		m_RpsDbInfo.aux_file << seq_size << "\n";
 		m_RpsDbInfo.aux_file << scientific << pssm.GetFinalData().GetKappa() << "\n";
