@@ -1,4 +1,4 @@
-/* $Id: msvc_dlls_info.cpp 340094 2011-10-05 15:16:19Z gouriano $
+/* $Id: msvc_dlls_info.cpp 354436 2012-02-27 14:14:59Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -282,9 +282,7 @@ static void s_AddProjItemToDll(const CProjectItemsTree& tree_src,
         string makefile_name = 
             SMakeProjectT::CreateMakeAppLibFileName(lib.m_SourcesBaseDir,
                                                     lib.m_Name);
-        CSimpleMakeFileContents makefile(
-            CDirEntry::ConcatPath(lib.m_SourcesBaseDir,makefile_name),
-            eMakeType_Undefined);
+        CSimpleMakeFileContents makefile(makefile_name,eMakeType_Undefined);
         CSimpleMakeFileContents::TContents::const_iterator p = 
             makefile.m_Contents.find("NCBI_C_LIBS");
 
