@@ -1,7 +1,7 @@
 #ifndef PROJECT_TREE_BUILDER__TOOLS_IMPLEMENT__HPP
 #define PROJECT_TREE_BUILDER__TOOLS_IMPLEMENT__HPP
 
-/* $Id: msvc_tools_implement.hpp 217367 2010-12-16 17:05:53Z gouriano $
+/* $Id: msvc_tools_implement.hpp 278245 2011-04-19 15:36:03Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -85,6 +85,11 @@ public:
     }
     virtual string IntermediateDirectory(void) const
     {
+        if (CMsvc7RegSettings::GetMsvcPlatform() < CMsvc7RegSettings::eUnix) {
+	        return CDirEntry::AddTrailingPathSeparator(
+                CDirEntry::ConcatPath(
+                    CMsvc7RegSettings::GetConfigNameKeyword(),"$(TargetName)"));
+        }
 	    return CDirEntry::AddTrailingPathSeparator(
                 CMsvc7RegSettings::GetConfigNameKeyword());
     }
@@ -401,7 +406,7 @@ public:
     virtual string GenerateDebugInformation(void) const
     {
         if (m_Config.m_VTuneAddon) {
-            return "TRUE";
+            return "true";
         }
         return GetLinkerOpt(m_MsvcMetaMakefile,
                             m_MsvcProjectMakefile,
@@ -461,6 +466,8 @@ public:
                             "FixedBaseAddress",
                             m_Config );
     }
+    SUPPORT_LINKER_OPTION(GenerateManifest)
+    SUPPORT_LINKER_OPTION(EmbedManifest)
 
 private:
     string      m_AdditionalOptions;
@@ -516,7 +523,7 @@ public:
 
     virtual string IgnoreAllDefaultLibraries(void) const
     {
-        return "FALSE";
+        return "false";
     }
     virtual string IgnoreDefaultLibraryNames(void) const
     {
@@ -525,6 +532,8 @@ public:
 
     SUPPORT_DUMMY_OPTION(AdditionalLibraryDirectories)
     SUPPORT_DUMMY_OPTION(FixedBaseAddress)
+    SUPPORT_DUMMY_OPTION(GenerateManifest)
+    SUPPORT_DUMMY_OPTION(EmbedManifest)
 
 private:
     CLinkerToolDummyImpl(const CLinkerToolDummyImpl&);
