@@ -1,4 +1,4 @@
-/* $Id: proj_tree.cpp 256258 2011-03-03 14:56:31Z gouriano $
+/* $Id: proj_tree.cpp 354436 2012-02-27 14:14:59Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -134,9 +134,8 @@ void CProjectItemsTree::CreateFrom(const string& root_src,
                 }
         
                 string applib_mfilepath = 
-                    CDirEntry::ConcatPath(source_base_dir,
                     SMakeProjectT::CreateMakeAppLibFileName(source_base_dir, 
-                                                            proj_name, info.m_Type));
+                                                            proj_name, info.m_Type);
                 if ( applib_mfilepath.empty() )
                     continue;
             
