@@ -1,4 +1,4 @@
-/*  $Id: cgiapp.cpp 372363 2012-08-16 18:59:19Z rafanovi $
+/*  $Id: cgiapp.cpp 390562 2013-02-28 15:13:28Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -63,19 +63,19 @@ BEGIN_NCBI_SCOPE
 NCBI_PARAM_DECL(bool, CGI, Print_Http_Referer);
 NCBI_PARAM_DEF_EX(bool, CGI, Print_Http_Referer, true, eParam_NoThread,
                   CGI_PRINT_HTTP_REFERER);
-static NCBI_PARAM_TYPE(CGI, Print_Http_Referer) s_PrintRefererParam;
+typedef NCBI_PARAM_TYPE(CGI, Print_Http_Referer) TPrintRefererParam;
 
 
 NCBI_PARAM_DECL(bool, CGI, Print_User_Agent);
 NCBI_PARAM_DEF_EX(bool, CGI, Print_User_Agent, true, eParam_NoThread,
                   CGI_PRINT_USER_AGENT);
-static NCBI_PARAM_TYPE(CGI, Print_User_Agent) s_PrintUserAgentParam;
+typedef NCBI_PARAM_TYPE(CGI, Print_User_Agent) TPrintUserAgentParam;
 
 
 NCBI_PARAM_DECL(bool, CGI, Print_Self_Url);
 NCBI_PARAM_DEF_EX(bool, CGI, Print_Self_Url, true, eParam_NoThread,
                   CGI_PRINT_SELF_URL);
-static NCBI_PARAM_TYPE(CGI, Print_Self_Url) s_PrintSelfUrlParam;
+typedef NCBI_PARAM_TYPE(CGI, Print_Self_Url) TPrintSelfUrlParam;
 
 NCBI_PARAM_DECL(bool, CGI, Allow_Sigpipe);
 NCBI_PARAM_DEF_EX(bool, CGI, Allow_Sigpipe, false, eParam_NoThread,
@@ -354,7 +354,7 @@ int CCgiApplication::Run(void)
         // Exception reporting. Use different severity for broken connection.
         ios_base::failure* fex = dynamic_cast<ios_base::failure*>(&e);
         CNcbiOstream* os = m_Context.get() ? m_Context->GetResponse().GetOutput() : NULL;
-        if (fex  &&  os  &&  !os->good()) {
+        if ((fex  &&  os  &&  !os->good())  ||  m_OutputBroken) {
             if ( !TClientConnIntOk::GetDefault() ) {
                 ERR_POST_X(13, Severity(TClientConnIntSeverity::GetDefault()) <<
                     "Connection interrupted");
@@ -405,7 +405,7 @@ void CCgiApplication::LogRequest(void) const
 {
     const CCgiContext& ctx = GetContext();
     string str;
-    if ( s_PrintSelfUrlParam.Get() ) {
+    if ( TPrintSelfUrlParam::GetDefault() ) {
         // Print script URL
         string self_url = ctx.GetSelfURL();
         if ( !self_url.empty() ) {
@@ -434,14 +434,14 @@ void CCgiApplication::LogRequest(void) const
         }
     }
     // Print HTTP_REFERER
-    if ( s_PrintRefererParam.Get() ) {
+    if ( TPrintRefererParam::GetDefault() ) {
         str = ctx.GetRequest().GetProperty(eCgi_HttpReferer);
         if ( !str.empty() ) {
             GetDiagContext().Extra().Print("HTTP_REFERER", str);
         }
     }
     // Print USER_AGENT
-    if ( s_PrintUserAgentParam.Get() ) {
+    if ( TPrintUserAgentParam::GetDefault() ) {
         str = ctx.GetRequest().GetProperty(eCgi_HttpUserAgent);
         if ( !str.empty() ) {
             GetDiagContext().Extra().Print("USER_AGENT", str);
@@ -481,12 +481,12 @@ CNcbiResource& CCgiApplication::x_GetResource( void ) const
 NCBI_PARAM_DECL(bool, CGI, Merge_Log_Lines);
 NCBI_PARAM_DEF_EX(bool, CGI, Merge_Log_Lines, true, eParam_NoThread,
                   CGI_MERGE_LOG_LINES);
-static NCBI_PARAM_TYPE(CGI, Merge_Log_Lines) s_MergeLogLines;
+typedef NCBI_PARAM_TYPE(CGI, Merge_Log_Lines) TMergeLogLines;
 
 
 void CCgiApplication::Init(void)
 {
-    if ( s_MergeLogLines.Get() ) {
+    if ( TMergeLogLines::GetDefault() ) {
         // Convert multi-line diagnostic messages into one-line ones by default.
         SetDiagPostFlag(eDPF_PreMergeLines);
         SetDiagPostFlag(eDPF_MergeLines);
@@ -520,7 +520,7 @@ CCgiServerContext* CCgiApplication::LoadServerContext(CCgiContext& /*context*/)
 
 
 NCBI_PARAM_DECL(bool, CGI, Count_Transfered);
-NCBI_PARAM_DEF_EX(bool, CGI, Count_Transfered, false, eParam_NoThread,
+NCBI_PARAM_DEF_EX(bool, CGI, Count_Transfered, true, eParam_NoThread,
                   CGI_COUNT_TRANSFERED);
 typedef NCBI_PARAM_TYPE(CGI, Count_Transfered) TCGI_Count_Transfered;
 
@@ -546,6 +546,8 @@ CCgiContext* CCgiApplication::CreateContextWithFlags
  int               ofd,
  int               flags)
 {
+    m_OutputBroken = false; // reset failure flag
+
     int errbuf_size =
         GetConfig().GetInt("CGI", "RequestErrBufSize", 256, 0,
                            CNcbiRegistry::eReturn);
@@ -558,6 +560,7 @@ CCgiContext* CCgiApplication::CreateContextWithFlags
                                 CRWStreambuf::fOwnReader));
             }
             inp = m_InputStream.get();
+            ifd = 0;
         }
         if ( !out ) {
             if ( !m_OutputStream.get() ) {
@@ -566,6 +569,7 @@ CCgiContext* CCgiApplication::CreateContextWithFlags
                                 CRWStreambuf::fOwnWriter));
             }
             out = m_OutputStream.get();
+            ofd = 1;
             if ( m_InputStream.get() ) {
                 // If both streams are created by the application, tie them.
                 inp->tie(out);
@@ -623,6 +627,7 @@ CCgiApplication::CCgiApplication(void)
    m_HostIP(0), 
    m_Iteration(0),
    m_ArgContextSync(false),
+   m_OutputBroken(false),
    m_IsResultReady(true),
    m_ShouldExit(false),
    m_RequestStartPrinted(false),
@@ -692,7 +697,7 @@ int CCgiApplication::OnException(exception& e, CNcbiOstream& os)
     }
 
     // Don't try to write to a broken output
-    if ( !os.good() ) {
+    if (!os.good()  ||  m_OutputBroken) {
         return -1;
     }
 
@@ -754,6 +759,26 @@ const CArgs& CCgiApplication::GetArgs(void) const
 }
 
 
+class CExtraEntryCollector : public CEntryCollector_Base {
+public:
+    CExtraEntryCollector(void) {}
+    virtual ~CExtraEntryCollector(void) {}
+    
+    virtual void AddEntry(const string& name,
+                          const string& value,
+                          bool          is_index)
+    {
+        _ASSERT(!is_index  ||  value.empty());
+        m_Args.push_back(CDiagContext_Extra::TExtraArg(name, value));
+    }
+
+    CDiagContext_Extra::TExtraArgs& GetArgs(void) { return m_Args; }
+
+private:
+    CDiagContext_Extra::TExtraArgs m_Args;
+};
+
+
 void CCgiApplication::x_OnEvent(EEvent event, int status)
 {
     switch ( event ) {
@@ -764,7 +789,9 @@ void CCgiApplication::x_OnEvent(EEvent event, int status)
 
             // Print request start message
             if ( !CDiagContext::IsSetOldPostFormat() ) {
-                GetDiagContext().PrintRequestStart(req.GetCGIEntriesStr());
+                CExtraEntryCollector collector;
+                req.GetCGIEntries(collector);
+                GetDiagContext().PrintRequestStart().Print(collector.GetArgs());
                 m_RequestStartPrinted = true;
             }
 
@@ -808,6 +835,7 @@ void CCgiApplication::x_OnEvent(EEvent event, int status)
             try {
                 if ( m_OutputStream.get() ) {
                     if ( !m_OutputStream->good() ) {
+                        m_OutputBroken = true; // set flag to indicate broken output
                         m_OutputStream->clear();
                     }
                     rctx.SetBytesWr(NcbiStreamposToInt8(m_OutputStream->tellp()));
@@ -827,9 +855,14 @@ void CCgiApplication::x_OnEvent(EEvent event, int status)
                 // Log broken connection as 299/499 status
                 CNcbiOstream* os = m_Context.get() ?
                     m_Context->GetResponse().GetOutput() : NULL;
-                if (os  &&  !os->good()) {
+                if ((os  &&  !os->good())  ||  m_OutputBroken) {
+                    // 'Accept-Ranges: bytes' indicates a request for
+                    // content length, broken connection is OK.
+                    // If Content-Range is also set, the client was downloading
+                    // partial data. Broken connection is not OK in this case.
                     if (TClientConnIntOk::GetDefault()  ||
-                        m_Context->GetResponse().AcceptRangesBytes()) {
+                        (m_Context->GetResponse().AcceptRangesBytes()  &&
+                        !m_Context->GetResponse().HaveContentRange())) {
                         rctx.SetRequestStatus(
                             CRequestStatus::e299_PartialContentBrokenConnection);
                     }
