@@ -1,4 +1,4 @@
-/*  $Id: fcgi_run.cpp 353682 2012-02-17 20:09:28Z ucko $
+/*  $Id: fcgi_run.cpp 390561 2013-02-28 15:12:57Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -243,9 +243,10 @@ extern "C" {
 // has changed since the last iteration)
 const int kSR_Executable = 111;
 const int kSR_WatchFile  = 112;
+
 static int s_ShouldRestart(CTime& mtime, CCgiWatchFile* watcher, int delay)
 {
-    static CTime restart_time(CTime::eEmpty, CTime::eGmt);
+    static CSafeStaticPtr<CTime> restart_time;
     static int   restart_reason;
 
     // Check if this CGI executable has been changed
@@ -264,13 +265,14 @@ static int s_ShouldRestart(CTime& mtime, CCgiWatchFile* watcher, int delay)
     }
 
     if (restart_reason) {
-        if (restart_time.IsEmpty()) {
-            restart_time.SetCurrent();
-            restart_time.AddSecond(delay);
+        if (restart_time->IsEmpty()) {
+            restart_time->SetTimeZone(CTime::eGmt);
+            restart_time->SetCurrent();
+            restart_time->AddSecond(delay);
             _TRACE("Will restart Fast-CGI in " << delay << " seconds, at "
-                   << restart_time.GetLocalTime().AsString("h:m:s"));
+                   << restart_time->GetLocalTime().AsString("h:m:s"));
         }
-        if (CurrentTime(CTime::eGmt) >= restart_time) {
+        if (CurrentTime(CTime::eGmt) >= *restart_time) {
             return restart_reason;
         }
     }
@@ -536,6 +538,8 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
             CNcbiArguments    args(0, 0);  // no cmd.-line ars
 
             m_Context.reset(CreateContext(&args, &env, &istr, &ostr));
+            _ASSERT(m_Context.get());
+            m_Context->CheckStatus();
 
             CNcbiOstream* orig_stream = NULL;
             //int orig_fd = -1;
@@ -657,6 +661,14 @@ bool CCgiApplication::x_RunFastCGI(int* result, unsigned int def_iter)
         }
         catch (exception& e) {
             // Reset stream pointers since the streams have been destroyed.
+            try {
+                CNcbiOstream* os = m_Context->GetResponse().GetOutput();
+                if (os && !os->good()) {
+                    m_OutputBroken = true;
+                }
+            }
+            catch (exception&) {
+            }
             m_Context->GetResponse().SetOutput(0);
             m_Context->GetRequest().SetInputStream(0);
 
