@@ -1,4 +1,4 @@
-/*  $Id: ncbicgi.cpp 359344 2012-04-12 14:57:12Z lavr $
+/*  $Id: ncbicgi.cpp 384670 2012-12-29 03:52:00Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -475,13 +475,13 @@ typedef NCBI_PARAM_TYPE(CGI, Cookie_Name_Banned_Symbols) TCookieNameBannedSymbol
 
 const char* s_GetCookieNameBannedSymbols(void)
 {
-    static string s_BannedSymbols;
+    static CSafeStaticPtr<string> s_BannedSymbols;
     static bool s_BannedSymbolsSet = false;
     if ( !s_BannedSymbolsSet ) {
-        s_BannedSymbols = TCookieNameBannedSymbols::GetDefault();
+        *s_BannedSymbols = TCookieNameBannedSymbols::GetDefault();
         s_BannedSymbolsSet = true;
     }
-    return s_BannedSymbols.c_str();
+    return s_BannedSymbols.Get().c_str();
 }
 
 
@@ -1549,21 +1549,19 @@ NCBI_PARAM_DEF_EX(string, CGI, LOG_LIMIT_ARGS, "*:1000000", eParam_NoThread,
 typedef NCBI_PARAM_TYPE(CGI, LOG_LIMIT_ARGS) TCGI_LogLimitArgs;
 
 
-string CCgiRequest::GetCGIEntriesStr(void) const
+void CCgiRequest::GetCGIEntries(CEntryCollector_Base& collector) const
 {
-    string args;
+    typedef CDiagContext_Extra::TExtraArg TExtraArg;
+
     // If there are any indexes, ignore entries and limits
     if ( !m_Indexes.empty() ) {
         ITERATE(TCgiIndexes, idx, m_Indexes) {
             if ( idx->empty() ) {
                 continue;
             }
-            if ( !args.empty() ) {
-                args += '+';
-            }
-            args += NStr::URLEncode(*idx, NStr::eUrlEnc_PercentOnly);
+            collector.AddEntry(*idx, kEmptyStr, true);
         }
-        return args;
+        return;
     }
 
     list<string> excluded, limited;
@@ -1604,7 +1602,7 @@ string CCgiRequest::GetCGIEntriesStr(void) const
     NStr::Split(TCGI_LogExcludeArgs::GetDefault(), "&", excluded);
     ITERATE(list<string>, it, excluded) {
         if (*it == "*") {
-            return kEmptyStr;
+            return;
         }
         arg_limits[*it] = -2;
     }
@@ -1619,17 +1617,55 @@ string CCgiRequest::GetCGIEntriesStr(void) const
             // Excluded argument
             continue;
         }
-        if ( !args.empty() ) {
-            args += '&';
-        }
-        args += NStr::URLEncode(entry->first, NStr::eUrlEnc_URIQueryName);
-        args += '=';
-        args += NStr::URLEncode(lim >= 0 ?
-                                entry->second.substr(0, lim) :
-                                string(entry->second),
-                                NStr::eUrlEnc_URIQueryValue);
+        collector.AddEntry(entry->first,
+            lim >= 0 ? entry->second.substr(0, lim) : string(entry->second),
+            false);
     }
-    return args;
+}
+
+
+class CStringEntryCollector : public CEntryCollector_Base {
+public:
+    CStringEntryCollector(void) {}
+    virtual ~CStringEntryCollector(void) {}
+    
+    virtual void AddEntry(const string& name,
+                          const string& value,
+                          bool          is_index);
+
+    const string& GetArgs(void) const { return m_Args; }
+
+private:
+    string m_Args;
+};
+
+
+void CStringEntryCollector::AddEntry(const string& name,
+                                     const string& value,
+                                     bool          is_index)
+{
+    if ( is_index ) {
+        if ( !m_Args.empty() ) {
+            m_Args += '+';
+        }
+        m_Args += NStr::URLEncode(name, NStr::eUrlEnc_PercentOnly);
+    }
+    else {
+        if ( !m_Args.empty() ) {
+            m_Args += '&';
+        }
+        m_Args += NStr::URLEncode(name, NStr::eUrlEnc_URIQueryName);
+        m_Args += '=';
+        m_Args += NStr::URLEncode(value, NStr::eUrlEnc_URIQueryValue);
+    }
+}
+
+
+string CCgiRequest::GetCGIEntriesStr(void) const
+{
+    CStringEntryCollector collector;
+    GetCGIEntries(collector);
+    return collector.GetArgs();
 }
 
 
