@@ -1,4 +1,4 @@
-/*  $Id: ncbicgir.cpp 354606 2012-02-28 17:00:34Z grichenk $
+/*  $Id: ncbicgir.cpp 390562 2013-02-28 15:13:28Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 #include <cgi/cgi_exception.hpp>
 #include <cgi/cgi_session.hpp>
 #include <cgi/cgiapp.hpp>
+#include <corelib/ncbi_safe_static.hpp>
 #include <cgi/error_codes.hpp>
 #include <time.h>
 
@@ -53,20 +54,21 @@
 BEGIN_NCBI_SCOPE
 
 
-const string CCgiResponse::sm_ContentTypeName    = "Content-Type";
-const string CCgiResponse::sm_LocationName       = "Location";
-const string CCgiResponse::sm_ContentTypeDefault = "text/html";
-const string CCgiResponse::sm_ContentTypeMixed   = "multipart/mixed";
-const string CCgiResponse::sm_ContentTypeRelated = "multipart/related";
-const string CCgiResponse::sm_ContentTypeXMR     = "multipart/x-mixed-replace";
-const string CCgiResponse::sm_ContentDispoName   = "Content-Disposition";
-const string CCgiResponse::sm_FilenamePrefix     = "attachment; filename=\"";
-const string CCgiResponse::sm_HTTPStatusName     = "Status";
-const string CCgiResponse::sm_HTTPStatusDefault  = "200 OK";
-const string CCgiResponse::sm_BoundaryPrefix     = "NCBI_CGI_Boundary_";
-const string CCgiResponse::sm_CacheControl       = "Cache-Control";
-const string CCgiResponse::sm_AcceptRanges       = "Accept-Ranges";
-const string CCgiResponse::sm_AcceptRangesBytes  = "bytes";
+const char* CCgiResponse::sm_ContentTypeName    = "Content-Type";
+const char* CCgiResponse::sm_LocationName       = "Location";
+const char* CCgiResponse::sm_ContentTypeDefault = "text/html";
+const char* CCgiResponse::sm_ContentTypeMixed   = "multipart/mixed";
+const char* CCgiResponse::sm_ContentTypeRelated = "multipart/related";
+const char* CCgiResponse::sm_ContentTypeXMR     = "multipart/x-mixed-replace";
+const char* CCgiResponse::sm_ContentDispoName   = "Content-Disposition";
+const char* CCgiResponse::sm_FilenamePrefix     = "attachment; filename=\"";
+const char* CCgiResponse::sm_HTTPStatusName     = "Status";
+const char* CCgiResponse::sm_HTTPStatusDefault  = "200 OK";
+const char* CCgiResponse::sm_BoundaryPrefix     = "NCBI_CGI_Boundary_";
+const char* CCgiResponse::sm_CacheControl       = "Cache-Control";
+const char* CCgiResponse::sm_AcceptRanges       = "Accept-Ranges";
+const char* CCgiResponse::sm_AcceptRangesBytes  = "bytes";
+const char* CCgiResponse::sm_ContentRange       = "Content-Range";
 
 NCBI_PARAM_DEF_IN_SCOPE(bool, CGI, ThrowOnBadOutput, true, CCgiResponse);
 
@@ -458,4 +460,10 @@ bool CCgiResponse::AcceptRangesBytes(void) const
 }
 
 
+bool CCgiResponse::HaveContentRange(void) const
+{
+    return HaveHeaderValue(sm_ContentRange);
+}
+
+
 END_NCBI_SCOPE
