@@ -1,4 +1,4 @@
-/*  $Id: user_agent.cpp 357979 2012-03-28 14:45:54Z ivanov $
+/*  $Id: user_agent.cpp 391099 2013-03-05 16:12:17Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -939,7 +939,8 @@ void CCgiUserAgent::x_Parse(const string& user_agent)
     }
     // Make additional check if platform is still undefined
     if ( m_Platform == ePlatform_Unknown ) {
-        if (m_UserAgent.find(USTR("Nokia"))        != NPOS  ||  // Nokia
+        if (m_UserAgent.find(USTR("Android "))     != NPOS  ||  // All Android based devices
+            m_UserAgent.find(USTR("Nokia"))        != NPOS  ||  // Nokia
             //m_UserAgent.find(USTR("HTC-"))       != NPOS  ||  // HTC
             //m_UserAgent.find(USTR("HTC_"))       != NPOS  ||  // HTC
             m_UserAgent.find(USTR("iPod"))         != NPOS  ||  // Apple iPod 
