@@ -1,4 +1,4 @@
-/* $Id: ncbi_dblb.c 213586 2010-11-24 02:43:49Z lavr $
+/* $Id: ncbi_dblb.c 358310 2012-03-30 19:19:35Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -149,7 +149,7 @@ const char* DBLB_GetServer(const char*             lb_name,
     if (strchr(lb_name, '.')) {
         cp.host = SOCK_gethostbyname(lb_name);
         if (cp.host == SOCK_GetLoopbackAddress())
-            cp.host = SERV_LOCALHOST;
+            cp.host = /*FIXME?*/SERV_LOCALHOST;
     } else
         cp.host = 0;
 
