@@ -1,4 +1,4 @@
-/* $Id: ncbi_iprange.c 328918 2011-08-04 17:45:10Z lavr $
+/* $Id: ncbi_iprange.c 360937 2012-04-27 13:47:28Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -239,6 +239,9 @@ int/*bool*/ NcbiParseIPRange(SIPRange* range, const char* s)
                 return 1/*success*/;
             }
         }
+        /* forbid misleading IP representations */
+        if (!SOCK_isipEx(s, 1/*fullquad*/))
+            return 0/*failure*/;
     }
     if (!(range->a = SOCK_gethostbyname(s)))
         return 0/*failure*/;
