@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_stream.cpp 362481 2012-05-09 17:23:43Z rafanovi $
+/* $Id: ncbi_conn_stream.cpp 371110 2012-08-04 18:58:23Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -48,34 +48,47 @@ BEGIN_NCBI_SCOPE
 
 
 CConn_IOStream::CConn_IOStream(CONNECTOR connector, const STimeout* timeout,
-                               size_t buf_size, bool tie,
-                               CT_CHAR_TYPE* ptr, size_t size) :
-    CNcbiIostream(0), m_CSb(0)
+                               size_t buf_size, TConn_Flags flags,
+                               CT_CHAR_TYPE* ptr, size_t size)
+    : CNcbiIostream(0), m_CSb(0)
 {
     auto_ptr<CConn_Streambuf>
-        csb(new CConn_Streambuf(connector, timeout, buf_size, tie,
+        csb(new CConn_Streambuf(connector, timeout, buf_size, flags,
                                 ptr, size));
-    if (csb->GetCONN()) {
-        init(csb.get());
-        m_CSb = csb.release();
-    } else
-        init(0); // according to the standard (27.4.4.1.3), badbit is set here
+    CONN conn = csb->GetCONN();
+    if (conn) {
+        SOCK s/*dummy*/;
+        // CONN_Write(0 bytes) could have done the same effect as GetSOCK
+        (void) CONN_GetSOCK(conn, &s);  // Prompt connection to actually open
+        if (CONN_Status(conn, eIO_Open) == eIO_Success) {
+            init(csb.get());
+            m_CSb = csb.release();
+            return;
+        }
+    }
+    init(0); // according to the standard (27.4.4.1.3), badbit is set here
 }
 
 
 CConn_IOStream::CConn_IOStream(CONN conn, bool close, const STimeout* timeout,
-                               size_t buf_size, bool tie,
-                               CT_CHAR_TYPE* ptr, size_t size) :
-    CNcbiIostream(0), m_CSb(0)
+                               size_t buf_size, TConn_Flags flags,
+                               CT_CHAR_TYPE* ptr, size_t size)
+    : CNcbiIostream(0), m_CSb(0)
 {
+    auto_ptr<CConn_Streambuf>
+        csb(new CConn_Streambuf(conn, close, timeout, buf_size, flags,
+                                ptr, size));
     if (conn) {
-        auto_ptr<CConn_Streambuf>
-            csb(new CConn_Streambuf(conn, close, timeout, buf_size, tie,
-                                    ptr, size));
-        init(csb.get());
-        m_CSb = csb.release();
-    } else
-        init(0);
+        SOCK s/*dummy*/;
+        // CONN_Write(0 bytes) should have done the same effect as GetSOCK
+        (void) CONN_GetSOCK(conn, &s);  // Prompt connection to open (if not)
+        if (CONN_Status(conn, eIO_Open) == eIO_Success) {
+            init(csb.get());
+            m_CSb = csb.release();
+            return;
+        }
+    }
+    init(0); // according to the standard (27.4.4.1.3), badbit is set here
 }
 
 
@@ -234,10 +247,11 @@ CConn_SocketStream::CConn_SocketStream(SOCK            sock,
 }
 
 
-static CONNECTOR s_TunneledSocketConnector(const SConnNetInfo* net_info,
-                                           const void*         init_data,
-                                           size_t              init_size,
-                                           TSOCK_Flags         flags)
+static CONNECTOR s_SocketConnectorBuilder(const SConnNetInfo* net_info,
+                                          const STimeout*     timeout,
+                                          const void*         data,
+                                          size_t              size,
+                                          TSOCK_Flags         flags)
 {
     EIO_Status status;
     SOCK       sock = 0;
@@ -253,25 +267,13 @@ static CONNECTOR s_TunneledSocketConnector(const SConnNetInfo* net_info,
         status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
         _ASSERT(!sock ^ !(status != eIO_Success));
         if (status == eIO_Success
-            &&  ((flags & ~(fSOCK_LogOn | fSOCK_LogDefault))  ||  init_size)) {
-            size_t size   = SOCK_OSHandleSize();
-            char*  handle = new char[size];
-            status = SOCK_GetOSHandleEx(sock, handle, size, eTakeOwnership);
-            if (status == eIO_Success) {
-                SOCK_Close(sock);
-                status = SOCK_CreateOnTopEx(handle, size, &sock,
-                                            init_data, init_size, flags);
-                if (status != eIO_Success) {
-                    SOCK_CloseOSHandle(handle, size);
-                    _ASSERT(!sock);
-                } else
-                    _ASSERT(sock);
-            } else {
-                SOCK_Abort(sock);
-                SOCK_Close(sock);
-                sock = 0;
-            }
-            delete[] handle;
+            &&  ((flags & ~(fSOCK_LogOn | fSOCK_LogDefault))  ||  size)) {
+            SOCK s;
+            status = SOCK_CreateOnTopEx(sock, 0, &s,
+                                        data, size, flags);
+            _ASSERT(!s ^ !(status != eIO_Success));
+            SOCK_Destroy(sock);
+            sock = s;
         }
         proxy = true;
     }
@@ -279,9 +281,9 @@ static CONNECTOR s_TunneledSocketConnector(const SConnNetInfo* net_info,
         const char* host = (net_info->firewall  &&  *net_info->proxy_host
                             ? net_info->proxy_host : net_info->host);
         if (!proxy  &&  net_info->debug_printout)
-            ConnNetInfo_LogEx(net_info, eLOG_Note, CORE_GetLOG());
-        status = SOCK_CreateEx(host, net_info->port, net_info->timeout, &sock,
-                               init_data, init_size, flags);
+            ConnNetInfo_Log(net_info, eLOG_Note, CORE_GetLOG());
+        status = SOCK_CreateEx(host, net_info->port, timeout, &sock,
+                               data, size, flags);
         _ASSERT(!sock ^ !(status != eIO_Success));
     }
     string hostport(net_info->host);
@@ -300,9 +302,11 @@ CConn_SocketStream::CConn_SocketStream(const SConnNetInfo& net_info,
                                        const void*         data,
                                        size_t              size,
                                        TSOCK_Flags         flags,
+                                       const STimeout*     timeout,
                                        size_t              buf_size)
-    : CConn_IOStream(s_TunneledSocketConnector(&net_info, data, size, flags),
-                     net_info.timeout, buf_size)
+    : CConn_IOStream(s_SocketConnectorBuilder(&net_info, timeout,
+                                              data, size, flags),
+                     timeout, buf_size)
 {
     return;
 }
@@ -586,7 +590,8 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
 
 
 CConn_MemoryStream::CConn_MemoryStream(size_t buf_size)
-    : CConn_IOStream(MEMORY_CreateConnector(), 0, buf_size, true),
+    : CConn_IOStream(MEMORY_CreateConnector(),
+                     kInfiniteTimeout/*0*/, buf_size),
       m_Ptr(0)
 {
     return;
@@ -598,7 +603,8 @@ CConn_MemoryStream::CConn_MemoryStream(BUF        buf,
                                        size_t     buf_size)
     : CConn_IOStream(MEMORY_CreateConnectorEx(buf, owner == eTakeOwnership
                                               ? 1/*true*/
-                                              : 0/*false*/), 0, buf_size, true,
+                                              : 0/*false*/),
+                     0, buf_size, fConn_ReadBuffered | fConn_WriteBuffered,
                      0, BUF_Size(buf)),
       m_Ptr(0)
 {
@@ -610,7 +616,8 @@ CConn_MemoryStream::CConn_MemoryStream(const void* ptr,
                                        size_t      size,
                                        EOwnership  owner,
                                        size_t      buf_size)
-    : CConn_IOStream(MEMORY_CreateConnector(), 0, buf_size, true,
+    : CConn_IOStream(MEMORY_CreateConnector(),
+                     0, buf_size, fConn_ReadBuffered | fConn_WriteBuffered,
                      (CT_CHAR_TYPE*) ptr, size),
       m_Ptr(owner == eTakeOwnership ? ptr : 0)
 {
@@ -623,7 +630,7 @@ CConn_MemoryStream::~CConn_MemoryStream()
     // Explicitly call x_Cleanup() to avoid using deleted m_Ptr otherwise.
     x_Cleanup();
     rdbuf(0);
-    delete[] (char*) m_Ptr;
+    delete[] (CT_CHAR_TYPE*) m_Ptr;
 }
 
 
@@ -671,13 +678,23 @@ void CConn_MemoryStream::ToVector(vector<char>* vec)
 }
 
 
+static CONNECTOR s_PipeConnectorBuilder(const string&         cmd,
+                                        const vector<string>& args,
+                                        CPipe::TCreateFlags   create_flags,
+                                        CPipe*&               pipe)
+{
+    pipe = new CPipe;
+    return PIPE_CreateConnector(cmd, args, create_flags,
+                                pipe, eNoOwnership);
+}
+
+
 CConn_PipeStream::CConn_PipeStream(const string&         cmd,
                                    const vector<string>& args,
                                    CPipe::TCreateFlags   create_flags,
                                    const STimeout*       timeout,
                                    size_t                buf_size)
-    : CConn_IOStream(PIPE_CreateConnector(cmd, args, create_flags,
-                                          &m_Pipe, eNoOwnership),
+    : CConn_IOStream(s_PipeConnectorBuilder(cmd, args, create_flags, m_Pipe),
                      timeout, buf_size)
 {
     return;
@@ -686,9 +703,10 @@ CConn_PipeStream::CConn_PipeStream(const string&         cmd,
 
 CConn_PipeStream::~CConn_PipeStream()
 {
-    // Explicitly call x_Cleanup() to avoid using dead m_Pipe otherwise.
+    // Explicitly do x_Cleanup() to avoid using dead m_Pipe in base class dtor
     x_Cleanup();
     rdbuf(0);
+    delete m_Pipe;
 }
 
 
@@ -717,11 +735,12 @@ CConn_FtpStream::CConn_FtpStream(const string&        host,
                                  unsigned short       port,
                                  TFTP_Flags           flag,
                                  const SFTP_Callback* cmcb,
-                                 const STimeout*      timeout)
+                                 const STimeout*      timeout,
+                                 size_t               buf_size)
     : CConn_IOStream(FTP_CreateConnectorSimple(host.c_str(), port,
                                                user.c_str(), pass.c_str(),
                                                path.c_str(), flag, cmcb),
-                     timeout, 0/*must be unbuffered*/, false/*thus untied*/)
+                     timeout, buf_size, fConn_Untie | fConn_ReadBuffered)
 {
     return;
 }
@@ -770,8 +789,10 @@ CConn_FTPDownloadStream::CConn_FTPDownloadStream(const string&        host,
                                                  TFTP_Flags           flag,
                                                  const SFTP_Callback* cmcb,
                                                  Uint8                offset,
-                                                 const STimeout*      timeout)
-    : CConn_FtpStream(host, user, pass, path, port, flag, cmcb, timeout)
+                                                 const STimeout*      timeout,
+                                                 size_t               buf_size)
+    : CConn_FtpStream(host, user, pass, path, port, flag, cmcb,
+                      timeout, buf_size)
 {
     // Use '\n' here instead of NcbiFlush to avoid (and thus make silent)
     // flush errors on retrieval of inexistent (or bad) files / directories..
@@ -841,7 +862,7 @@ public:
                      SFILE_ConnAttr* attr   = 0)
         : CConn_IOStream(FILE_CreateConnectorEx(ifname.c_str(),
                                                 ofname.c_str(), attr),
-                         0/*timeout*/, 0/*unbuffered*/, false/*untied*/)
+                         0/*timeout*/, 0/*unbuffered*/, fConn_Untie)
     {
         return;
     }
@@ -868,7 +889,7 @@ static bool x_IsIdentifier(const string& str)
 
 extern
 NCBI_XCONNECT_EXPORT  // FIXME: To remove once the API is fully official
-CConn_IOStream* NcbiOpenURL(const string& url)
+CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
 {
     class CPrivateIniter : public CConnIniter {
     public:
@@ -890,34 +911,48 @@ CConn_IOStream* NcbiOpenURL(const string& url)
         net_info->req_method = eReqMethod_Connect;
 
     if (ConnNetInfo_ParseURL(net_info.get(), url.c_str())) {
-        if (net_info->req_method == eReqMethod_Connect)
-            return new CConn_SocketStream(*net_info);
+        if (net_info->req_method == eReqMethod_Connect) {
+            return new CConn_SocketStream(*net_info, 0, 0,
+                                          fSOCK_LogDefault, net_info->timeout,
+                                          buf_size);
+        }
         switch (net_info->scheme) {
         case eURL_Https:
         case eURL_Http:
-            return new CConn_HttpStream(net_info.get());
+            return new CConn_HttpStream(net_info.get(), kEmptyStr, 0, 0, 0, 0,
+                                        fHTTP_AutoReconnect, kDefaultTimeout,
+                                        buf_size);
         case eURL_File:
             if (*net_info->host  ||  net_info->port)
                 break; /*not supported*/
             if (net_info->debug_printout)
-                ConnNetInfo_LogEx(net_info.get(), eLOG_Note, CORE_GetLOG());
+                ConnNetInfo_Log(net_info.get(), eLOG_Note, CORE_GetLOG());
             return new CConn_FileStream(net_info->path);
         case eURL_Ftp:
             if (net_info->debug_printout)
-                ConnNetInfo_LogEx(net_info.get(), eLOG_Note, CORE_GetLOG());
+                ConnNetInfo_Log(net_info.get(), eLOG_Note, CORE_GetLOG());
             return new CConn_FTPDownloadStream(net_info->host,
                                                net_info->path,
                                                net_info->user,
                                                net_info->pass,
                                                kEmptyStr/*path*/,
                                                net_info->port,
-                                               net_info->debug_printout
-                                               == eDebugPrintout_Data
-                                               ? fFTP_LogAll
-                                               : net_info->debug_printout
-                                               == eDebugPrintout_Some
-                                               ? fFTP_LogControl
-                                               : 0);
+                                               (net_info->debug_printout
+                                                == eDebugPrintout_Some
+                                                ? fFTP_LogControl
+                                                : net_info->debug_printout
+                                                == eDebugPrintout_Data
+                                                ? fFTP_LogAll
+                                                : 0) |
+                                               (net_info->req_method
+                                                == eReqMethod_Post
+                                                ? fFTP_UsePassive
+                                                : net_info->req_method
+                                                == eReqMethod_Get
+                                                ? fFTP_UseActive
+                                                : 0), 0, 0,
+                                               net_info->timeout,
+                                               buf_size);
         default:
             break;
         }
