@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_test.cpp 366508 2012-06-14 18:04:58Z rafanovi $
+/* $Id: ncbi_conn_test.cpp 371606 2012-08-09 15:55:12Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,14 +39,15 @@
 #include "ncbi_priv.h"
 #include "ncbi_servicep.h"
 #include <iterator>
+#include <stdlib.h>
 
 #define _STR(x)           #x
 #define STRINGIFY(x)  _STR(x)
 
-#define HELP_EMAIL  (m_Email.empty()                                    \
-                     ? string("NCBI Help Desk info@ncbi.nlm.nih.gov")   \
-                     : m_Email)
-#define NCBI_FW_URL                                                     \
+#define HELP_EMAIL    (m_Email.empty()                                  \
+                       ? string("NCBI Help Desk info@ncbi.nlm.nih.gov") \
+                       : m_Email)
+#define NCBI_FWDOC_URL                                                  \
     "http://www.ncbi.nlm.nih.gov/IEB/ToolBox/NETWORK/firewall.html#Settings"
 
 
@@ -162,9 +163,19 @@ EIO_Status CConnTest::ConnStatus(bool failure, CConn_IOStream* io)
 }
 
 
+static SConnNetInfo* ConnNetInfo_Create(const char*    svc_name,
+                                        EDebugPrintout dbg_printout)
+{
+    SConnNetInfo* net_info = ::ConnNetInfo_Create(svc_name);
+    if (net_info  &&  (EDebugPrintout) net_info->debug_printout < dbg_printout)
+        net_info->debug_printout = dbg_printout;
+    return net_info;
+}
+
+
 EIO_Status CConnTest::HttpOkay(string* reason)
 {
-    SConnNetInfo* net_info = ConnNetInfo_Create(0);
+    SConnNetInfo* net_info = ConnNetInfo_Create(0, m_DebugPrintout);
     if (net_info) {
         if (*net_info->http_proxy_host  &&  net_info->http_proxy_port)
             m_HttpProxy = true;
@@ -263,7 +274,7 @@ static int s_ParseHeader(const char* header, void* data, int server_error)
 
 EIO_Status CConnTest::DispatcherOkay(string* reason)
 {
-    SConnNetInfo* net_info = ConnNetInfo_Create(kTest);
+    SConnNetInfo* net_info = ConnNetInfo_Create(0, m_DebugPrintout);
     ConnNetInfo_SetupStandardArgs(net_info, kTest);
 
     PreCheck(eDispatcher, 0/*main*/,
@@ -318,7 +329,7 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
 {
     static const char kService[] = "bounce";
 
-    SConnNetInfo* net_info = ConnNetInfo_Create(kService);
+    SConnNetInfo* net_info = ConnNetInfo_Create(kService, m_DebugPrintout);
     if (net_info)
         net_info->lb_disable = 1/*no local LB to use even if available*/;
 
@@ -331,7 +342,7 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
     svc << kTest << NcbiEndl;
     string temp;
     svc >> temp;
-    bool responded = temp.size() > 0;
+    bool responded = temp.size() > 0 ? true : false;
     EIO_Status status = ConnStatus(NStr::Compare(temp, kTest) != 0, &svc);
 
     if (status == eIO_Interrupt)
@@ -458,7 +469,7 @@ EIO_Status CConnTest::x_GetFirewallConfiguration(const SConnNetInfo* net_info)
 
 EIO_Status CConnTest::GetFWConnections(string* reason)
 {
-    SConnNetInfo* net_info = ConnNetInfo_Create(0);
+    SConnNetInfo* net_info = ConnNetInfo_Create(0, m_DebugPrintout);
     if (net_info) {
         const char* user_header;
         net_info->req_method = eReqMethod_Post;
@@ -483,7 +494,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             " and 165.112.7.12\n"
             "To set that up correctly, please have your network administrator"
             " read the following (if they have not already done so):"
-            " " NCBI_FW_URL "\n";
+            " " NCBI_FWDOC_URL "\n";
     } else {
         temp += "This is an obsolescent mode that requires keeping a wide port"
             " range [4444..4544] (inclusive) open to let through connections"
@@ -495,8 +506,9 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
         _ASSERT(net_info);
         switch (net_info->firewall) {
         case eFWMode_Adaptive:
-            temp += "There are also fallback connection ports such as 22 and"
-                " 443 at 130.14.29.112.  They will be used if connections to"
+            temp += "There are also fallback connection ports such as "
+                STRINGIFY(CONN_PORT_SSH) " and " STRINGIFY(CONN_PORT_HTTPS)
+                " at 130.14.29.112.  They will be used if connections to"
                 " the ports in the range described above have failed\n";
             break;
         case eFWMode_Firewall:
@@ -626,10 +638,10 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
 }
 
 
-static SOCK x_GetSOCK(const CConn_IOStream* fw)
+static inline SOCK x_GetSOCK(const CConn_IOStream* s)
 {
-    SOCK s;
-    return CONN_GetSOCK(fw->GetCONN(), &s) == eIO_Success ? s : 0;
+    SOCK sock;
+    return CONN_GetSOCK(s->GetCONN(), &sock) == eIO_Success ? sock : 0;
 }
 
 
@@ -637,11 +649,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
 {
     static const STimeout kZeroTmo = { 0, 0 };
 
-    SConnNetInfo* net_info = ConnNetInfo_Create(0);
-
-    TSOCK_Flags flags =
-        (net_info  &&  net_info->debug_printout == eDebugPrintout_Data
-         ? fSOCK_LogOn : fSOCK_LogDefault);
+    SConnNetInfo* net_info = ConnNetInfo_Create(0, m_DebugPrintout);
 
     string temp("Checking individual connection points..\n");
     if (net_info) {
@@ -658,8 +666,12 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                 " firewall ports may fail to operate.  Only those ports found"
                 " in working order will be used to access NCBI services\n";
         }
+        net_info->path[0] = '\0';
+        net_info->args[0] = '\0';
     }
 
+    SERV_InitFirewallMode();
+
     PreCheck(eFirewallConnections, 0/*main*/, temp);
 
     vector<CFWConnPoint>* fwd[] = { &m_Fwd, &m_FwdFB };
@@ -686,8 +698,9 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
             net_info->port = cp->port;
             CConn_SocketStream* fw;
             if (cp->status == eIO_Success) {
-                fw = new CConn_SocketStream(*net_info, "\r\n"/*data*/,
-                                            2/*size*/, flags);
+                fw = new CConn_SocketStream(*net_info,
+                                            "\r\n"/*data*/, 2/*size*/,
+                                            fSOCK_LogDefault, &kZeroTmo);
                 if (!fw->good()  ||  !fw->GetCONN()) {
                     delete fw;
                     fw = 0;
@@ -744,8 +757,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
             vector<CSocketAPI::SPoll>   poll;
             ITERATE(vector<CFWCheck>, ck, v) {
                 CConn_SocketStream* fw = ck->first.get();
-                CFWConnPoint*       cp = ck->second;
-                if (!fw  ||  cp->status != eIO_Success)
+                if (!fw  ||  ck->second->status != eIO_Success)
                     continue;
                 CSocket* s = new CSocket;
                 s->Reset(x_GetSOCK(fw), eNoOwnership, eCopyTimeoutsFromSOCK);
@@ -756,6 +768,14 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
             if (!poll.size())
                 break;
             status = CSocketAPI::Poll(poll, net_info->timeout);
+            if (status != eIO_Timeout)
+                continue;
+            ITERATE(vector<CFWCheck>, ck, v)
+                ck->second->status = eIO_Timeout;
+            if (n) {
+                status = eIO_Success;
+                break;
+            }
         } while (status == eIO_Success);
 
         // Report results:
@@ -831,7 +851,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                     temp += n ? "fallback" : "non-conventional";
                     temp += " ports; please see your network administrator";
                     if (!url) {
-                        temp += " and let them read: " NCBI_FW_URL;
+                        temp += " and let them read: " NCBI_FWDOC_URL;
                         url = true;
                     }
                     temp += '\n';
@@ -843,7 +863,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                         " please check with your network administrator"
                         " that the proxy has been configured correctly";
                     if (!url) {
-                        temp += ": " NCBI_FW_URL;
+                        temp += ": " NCBI_FWDOC_URL;
                         url = true;
                     }
                     temp += '\n';
@@ -854,7 +874,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                     temp += " please see your network administrator";
                     if (!url) {
                         temp += " and have them read the following: "
-                            NCBI_FW_URL;
+                            NCBI_FWDOC_URL;
                         url = true;
                     }
                     temp += '\n';
@@ -863,7 +883,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                         " configuration to use a more narrow"
                         " port range";
                     if (!url) {
-                        temp += " per: " NCBI_FW_URL;
+                        temp += " per: " NCBI_FWDOC_URL;
                         url = true;
                     }
                     temp += '\n';
@@ -907,7 +927,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
         }
         if (note  &&  status != eIO_Interrupt) {
             temp += "\nYou may want to read this link for more information: "
-                NCBI_FW_URL;
+                NCBI_FWDOC_URL;
         }
     } else {
         if (m_Firewall)
@@ -927,40 +947,75 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
 }
 
 
+static inline unsigned int ud(time_t one, time_t two)
+{
+    return (unsigned int)(one > two ? one - two : two - one);
+}
+
+
+static size_t rnd(size_t minimal, size_t maximal)
+{
+    return minimal <= maximal ? minimal + rand() % (maximal - minimal + 1) : 0;
+}
+
+
 EIO_Status CConnTest::StatefulOkay(string* reason)
 {
-    static const char kId2Init[] =
-        "0\2000\200\242\200\240\200\5\0\0\0\0\0\0\0\0\0";
-    static const char kId2[] = "ID2";
-    char ry[sizeof(kFWSign)];
-
-    SConnNetInfo* net_info = ConnNetInfo_Create(kId2);
+    static const char kEcho[] = "ECHO";
 
     PreCheck(eStatefulService, 0/*main*/,
              "Checking reachability of a stateful service");
 
-    CConn_ServiceStream id2(kId2, fSERV_Any, net_info, 0/*params*/, m_Timeout);
-    id2.SetCanceledCallback(m_Canceled);
+    SConnNetInfo* net_info = ConnNetInfo_Create(kEcho, m_DebugPrintout);
+
+    CTime  time(CTime::eCurrent, CTime::eLocal);
+    time_t seed = time.GetTimeT();
+
+    char buf[(1 << 10) + (8 + 1)];
+    sprintf(buf, "%08X", (unsigned int) seed);
+    size_t size = rnd(2, (sizeof(buf) - (8 + 1)) >> 3);
+
+    size_t i;
+    for (i = 1;  i < size;  i++)
+        sprintf(buf + (i << 3), "%08X", (unsigned int) rand());
+    memset(&buf[size <<= 3], 0, 8);
+    size += 8;
+
+    CConn_ServiceStream echo(kEcho, fSERV_Any, net_info, 0, m_Timeout);
+    echo.SetCanceledCallback(m_Canceled);
 
     streamsize n = 0;
-    bool iofail = !id2.write(kId2Init, sizeof(kId2Init) - 1)  ||  !id2.flush()
-        ||  !(n = CStreamUtils::Readsome(id2, ry, sizeof(ry)-1));
-    EIO_Status status = ConnStatus
-        (iofail  ||  n < 4  ||  memcmp(ry, "0\200\240\200", 4) != 0, &id2);
-    ry[n] = '\0';
+    bool iofail = !echo.write(buf, size)  ||  !echo.flush()
+        ||  !(n = CStreamUtils::Readsome(echo, buf, size));
+    if (!iofail) {
+        if (n < (streamsize) size) {
+            if (!echo.read(buf + n, (streamsize) size - n))
+                iofail = true;
+            else
+                n += echo.gcount();
+        }
+        if (n == (streamsize) size) {
+            time_t now = (time_t) NStr::StringToNumeric<unsigned int>
+                (CTempString(buf, 8), NStr::fConvErr_NoThrow, 16/*radix*/);
+            time.SetTimeT(now);
+        } else
+            iofail = true;
+    }
+    EIO_Status status = ConnStatus(iofail, &echo);
+    buf[n] = '\0';
 
     string temp;
     if (status == eIO_Interrupt)
         temp = kCanceled;
     else if (status == eIO_Success)
-        temp = "OK";
+        temp = "OK (RTT "+NStr::NumericToString(ud(seed,time.GetTimeT()))+')';
     else {
-        char* str = SERV_ServiceName(kId2);
-        if (str  &&  NStr::CompareNocase(str, kId2) != 0) {
+        char* str = SERV_ServiceName(kEcho);
+        if (str  &&  NStr::CompareNocase(str, kEcho) != 0) {
             temp = n ? "Unrecognized" : "No";
             temp += " response received from substituted service;"
                 " please remove [";
-            string upper(kId2);
+            string upper(kEcho);
             temp += NStr::ToUpper(upper);
             temp += "]CONN_SERVICE_NAME=\"";
             temp += str;
@@ -983,7 +1038,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                     " preventing this stateful service from operating"
                     " properly; try to remove [";
                 if (!m_Stateless) {
-                    string upper(kId2);
+                    string upper(kEcho);
                     temp += NStr::ToUpper(upper);
                     temp += "]CONN_STATELESS\n";
                 } else
@@ -991,7 +1046,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
             } else if (!str) {
                 SERV_ITER iter = 0;
                 if (status != eIO_Timeout
-                    &&  (!(iter = SERV_OpenSimple(kId2))
+                    &&  (!(iter = SERV_OpenSimple(kEcho))
                          ||  !SERV_GetNextInfo(iter))) {
                     temp += "The service is currently unavailable;"
                         " you may want to contact " + HELP_EMAIL + '\n';
@@ -1007,7 +1062,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
             }
         } else if (!str) {
             if (n  &&  net_info  &&  net_info->http_proxy_port
-                &&  NStr::strncasecmp(ry, kFWSign, (size_t) n) == 0) {
+                &&  NStr::strncasecmp(buf, kFWSign, (size_t) n) == 0) {
                 temp += "NCBI Firewall";
                 if (!net_info->firewall)
                     temp += " (Connection Relay)";
