@@ -1,4 +1,4 @@
-/* $Id: ncbi_connutil.c 362495 2012-05-09 17:27:00Z rafanovi $
+/* $Id: ncbi_connutil.c 378021 2012-10-17 18:46:38Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -329,7 +329,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     REG_VALUE(REG_CONN_TIMEOUT, str, 0);
     len = strlen(str);
     if (len < 3  ||  8 < len  ||  strncasecmp(str, "infinite", len) != 0) {
-        if (!*str  ||  (dbl = atof(str)) < 0.0)
+        if (!*str || (dbl = NCBI_simple_atof(str, &e)) < 0.0 || errno || *e)
             dbl = DEF_CONN_TIMEOUT;
         info->tmo.sec      = (unsigned int)  dbl;
         info->tmo.usec     = (unsigned int)((dbl - info->tmo.sec) * 1000000.0);
@@ -551,11 +551,12 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
         }
         memcpy(info->args, args, argslen);
         info->args[argslen + len] = '\0';
-    } else if (!(args = strchr(info->args, '#'))) {
+    } else if ((scheme == eURL_Https  ||  scheme == eURL_Http)
+               &&  (args = strchr(info->args, '#'))) {
         /* keep the old fragment, if any, but drop all args */
-        info->args[0] = '\0';
-    } else
         memmove(info->args, args, strlen(args) + 1);
+    } else
+        info->args[0] = '\0';
     if (path) {
         memcpy(p, path, pathlen);
         p[pathlen] = '\0';
@@ -1150,7 +1151,7 @@ static void s_SaveUserHeader(char* s, const char* name,
         memcpy(s, "NULL\n", 6);
 }
 
-extern void ConnNetInfo_LogEx(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
+extern void ConnNetInfo_Log(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
 {
     char   buf[40];
     size_t uhlen;
@@ -1211,7 +1212,11 @@ extern void ConnNetInfo_LogEx(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
                                            ? x_Scheme((EURLScheme)info->scheme,
                                                       buf)
                                            : "(unspec)"));
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
     s_SaveString    (s, "user",            info->user);
+#else
+    s_SaveKeyval    (s, "user",           *info->user ? "(set)" : "\"\"");
+#endif /*_DEBUG && !NDEBUG*/
     if (*info->pass)
         s_SaveKeyval(s, "pass",           *info->user ? "(set)" : "(ignored)");
     else
@@ -1228,7 +1233,12 @@ extern void ConnNetInfo_LogEx(const SConnNetInfo* info, ELOG_Level sev, LOG lg)
     s_SaveKeyval    (s, "http_proxy_port",(info->http_proxy_port
                                            ? x_Port(info->http_proxy_port, buf)
                                            : "(none)"));
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
     s_SaveString    (s, "http_proxy_user", info->http_proxy_user);
+#else
+    s_SaveKeyval    (s, "http_proxy_user",(info->http_proxy_user[0] ?
+                                           "(set)" : "\"\""));
+#endif /*_DEBUG && !NDEBUG*/
     if (*info->http_proxy_pass)
         s_SaveKeyval(s, "http_proxy_pass", "(set)");
     else
@@ -1533,24 +1543,10 @@ extern EIO_Status URL_ConnectEx
     BUF_Destroy(buf);
 
     if (s) {
-        /* resuse connection to HTTPD */
-        size_t size   = SOCK_OSHandleSize();
-        void*  handle = malloc(size);
-        status = SOCK_GetOSHandleEx(s, handle, size, eTakeOwnership);
-        if (status == eIO_Success) {
-            SOCK_Close(s);
-            status  = SOCK_CreateOnTopEx(handle, size, sock,
-                                         hdr, hdr_len, flags);
-            if (status != eIO_Success) {
-                SOCK_CloseOSHandle(handle, size);
-                assert(!*sock);
-            }
-        } else {
-            SOCK_Abort(s);
-            SOCK_Close(s);
-        }
-        if (handle)
-            free(handle);
+        /* resuse connection */
+        status = SOCK_CreateOnTopEx(s, 0, sock,
+                                    hdr, hdr_len, flags);
+        SOCK_Destroy(s);
     } else {
         /* connect to HTTPD */
         status = SOCK_CreateEx(host, port, o_timeout, sock,
