@@ -1,4 +1,4 @@
-/* $Id: ncbi_core.c 349631 2012-01-11 21:12:09Z rafanovi $
+/* $Id: ncbi_core.c 361886 2012-05-04 18:26:31Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,6 @@
  */
 
 #include "ncbi_ansi_ext.h"
-#include "ncbi_assert.h"
 #include "ncbi_priv.h"
 #include <stdlib.h>
 
@@ -368,11 +367,11 @@ extern void LOG_WriteInternal
 
     /* unconditional exit/abort on fatal error */
     if (call_data->level == eLOG_Fatal) {
-#if defined(NDEBUG)
+#ifdef NDEBUG
         exit(1);
 #else
         abort();
-#endif
+#endif /*NDEBUG*/
     }
 }
 
