@@ -1,4 +1,4 @@
-/* $Id: ncbi_core_cxx.cpp 354817 2012-02-29 20:12:06Z lavr $
+/* $Id: ncbi_core_cxx.cpp 372798 2012-08-22 17:08:00Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -164,8 +164,9 @@ static void s_LOG_Handler(void*       /*user_data*/,
                 call_data->raw_size <<
                 " byte" << (call_data->raw_size != 1 ? "s" : "") << ")\n" <<
                 NStr::PrintableString
-                (string(static_cast<const char*> (call_data->raw_data),
-                        call_data->raw_size), NStr::eNewLine_Passthru) <<
+                (CTempString(static_cast<const char*>(call_data->raw_data),
+                             call_data->raw_size),
+                 NStr::fNewLine_Passthru | NStr::fNonAscii_Quote) <<
                 "\n#################### [END] Raw Data";
         }
     }
