@@ -1,4 +1,4 @@
-/* $Id: ncbi_file_connector.c 362482 2012-05-09 17:24:01Z rafanovi $
+/* $Id: ncbi_file_connector.c 373957 2012-09-05 15:27:28Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,8 +34,8 @@
  */
 
 #include "ncbi_ansi_ext.h"
+#include "ncbi_assert.h"
 #include <connect/ncbi_file_connector.h>
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
