@@ -1,4 +1,4 @@
-/* $Id: ncbi_host_info.c 303552 2011-06-10 03:44:13Z lavr $
+/* $Id: ncbi_host_info.c 373959 2012-09-05 15:28:02Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,16 +31,15 @@
  */
 
 #include "ncbi_lbsmd.h"
-#include <math.h>
+#include <math.h>  /* NB: pull only M_PI */
 #include <stdlib.h>
 #include <string.h>
 
-#ifndef M_PI
-/* Not defined on MacOS.9 :-( */
-#  define M_PI 3.14159265358979323846
-#endif
-
-#define HINFO_MAGIC M_PI
+#ifdef    M_PI
+#  define HINFO_MAGIC  M_PI
+#else /* Not defined on MacOS.9 :-( */
+#  define HINFO_MAGIC  3.14159265358979323846
+#endif /*!M_PI*/
 
 
 HOST_INFO HINFO_Create(unsigned int addr, const void* hinfo, size_t hinfo_size,
@@ -122,7 +121,7 @@ extern int HINFO_TaskCount(const HOST_INFO host_info)
 
 extern int HINFO_Memusage(const HOST_INFO host_info, double memusage[5])
 {
-    memset(memusage, 0, sizeof(memusage[0]) * 5);
+    memset(memusage, 0, 5 * sizeof(memusage[0]));
     if (!host_info  ||  host_info->pad != HINFO_MAGIC)
         return 0;
     return LBSM_HINFO_Memusage(host_info, memusage);
@@ -140,7 +139,7 @@ extern int HINFO_MachineParams(const HOST_INFO host_info, SHINFO_Params* p)
 
 extern int/*bool*/ HINFO_LoadAverage(const HOST_INFO host_info, double lavg[2])
 {
-    memset(lavg, 0, sizeof(lavg[0]) * 2);
+    memset(lavg, 0, 2 * sizeof(lavg[0]));
     if (!host_info  ||  host_info->pad != HINFO_MAGIC)
         return 0;
     return LBSM_HINFO_LoadAverage(host_info, lavg);
@@ -149,7 +148,7 @@ extern int/*bool*/ HINFO_LoadAverage(const HOST_INFO host_info, double lavg[2])
 
 extern int/*bool*/ HINFO_Status(const HOST_INFO host_info, double status[2])
 {
-    memset(status, 0, sizeof(status[0]) * 2);
+    memset(status, 0, 2 * sizeof(status[0]));
     if (!host_info  ||  host_info->pad != HINFO_MAGIC)
         return 0;
     return LBSM_HINFO_Status(host_info, status);
