@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_connector.c 362482 2012-05-09 17:24:01Z rafanovi $
+/* $Id: ncbi_http_connector.c 373575 2012-08-30 17:57:43Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -252,7 +252,7 @@ static EIO_Status s_Adjust(SHttpConnector* uuu,
                     ||  (uuu->flags & fHTTP_InsecureRedirect)
                     ||  !uuu->w_len) {
                     int secure = uuu->net_info->scheme == eURL_Https ? 1 : 0;
-                    *uuu->net_info->args = '\0'/*arguments not inherited*/;
+                    uuu->net_info->args[0] = '\0'/*arguments not inherited*/;
                     fail = !ConnNetInfo_ParseURL(uuu->net_info, retry->data);
                     if (!fail  &&  secure
                         &&  uuu->net_info->scheme != eURL_Https
@@ -297,7 +297,6 @@ static EIO_Status s_Adjust(SHttpConnector* uuu,
                     fail = -1;
             }
             if (fail) {
-                char* url = ConnNetInfo_URL(uuu->net_info);
                 CORE_LOGF_X(3, eLOG_Error,
                             ("[HTTP%s%s]  %s %s %c%s%c",
                              url  &&  *url ? "; " : "",
@@ -309,8 +308,6 @@ static EIO_Status s_Adjust(SHttpConnector* uuu,
                              "(["[!retry->data],
                              retry->data ? retry->data : "NULL",
                              ")]"[!retry->data]));
-                if (url)
-                    free(url);
                 status = fail < 0 ? eIO_NotSupported : eIO_Closed;
             } else {
                 CORE_LOGF_X(18, eLOG_Trace,
@@ -546,7 +543,7 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
                 reset_user_header = 0;
 
             if (uuu->net_info->debug_printout)
-                ConnNetInfo_Log(uuu->net_info, CORE_GetLOG());
+                ConnNetInfo_Log(uuu->net_info, eLOG_Note, CORE_GetLOG());
 
             status = URL_ConnectEx(host, port, path, args,
                                    (EReqMethod) uuu->net_info->req_method, len,
@@ -575,7 +572,7 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
         } else
             assert(!sock);
 
-        /* connection failed */
+        /* connection failed, try another server */
         if (s_Adjust(uuu, 0, extract) != eIO_Success)
             break;
     }
@@ -647,12 +644,12 @@ static EIO_Status s_ConnectAndSend(SHttpConnector* uuu,
                 uuu->w_len -= n_written;
                 off        += n_written;
             } while (uuu->w_len);
-        } else if (!uuu->read_state)
+        } else if (uuu->read_state == eRS_WriteRequest)
             status = SOCK_Write(uuu->sock, 0, 0, 0, eIO_WritePlain);
 
         if (status == eIO_Success) {
             assert(uuu->w_len == 0);
-            if (!uuu->read_state) {
+            if (uuu->read_state == eRS_WriteRequest) {
                 /* 10/07/03: While this call here is perfectly legal, it could
                  * cause connection severed by a buggy CISCO load-balancer. */
                 /* 10/28/03: CISCO's beta patch for their LB shows that the
@@ -757,20 +754,22 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
     int        server_error = 0;
     int        http_status;
     EIO_Status status;
-    char*      header;
     size_t     size;
+    char*      url;
+    char*      str;
+    size_t     n;
 
     assert(uuu->sock  &&  uuu->read_state == eRS_ReadHeader);
-    retry->mode = eRetry_None;
-    retry->data = 0;
+    memset(retry, 0, sizeof(*retry));
+    /*retry->mode = eRetry_None;
+      retry->data = 0;*/
 
     /* line by line HTTP header input */
     for (;;) {
-        size_t n;
         /* do we have full header yet? */
         size = BUF_Size(uuu->http);
-        if (!(header = (char*) malloc(size + 1))) {
-            char* url = ConnNetInfo_URL(uuu->net_info);
+        if (!(str = (char*) malloc(size + 1))) {
+            url = ConnNetInfo_URL(uuu->net_info);
             CORE_LOGF_X(7, eLOG_Error,
                         ("[HTTP%s%s]  Cannot allocate header, %lu byte%s",
                          url  &&  *url ? "; " : "",
@@ -780,16 +779,15 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                 free(url);
             return eIO_Unknown;
         }
-        verify(BUF_Peek(uuu->http, header, size) == size);
-        header[size] = '\0';
-        if (size >= 4  &&  strcmp(&header[size - 4], "\r\n\r\n") == 0)
+        verify(BUF_Peek(uuu->http, str, size) == size);
+        str[size] = '\0';
+        if (size >= 4  &&  strcmp(&str[size - 4], "\r\n\r\n") == 0)
             break/*full header captured*/;
-        free(header);
+        free(str);
 
         status = SOCK_StripToPattern(uuu->sock, "\r\n", 2, &uuu->http, &n);
 
         if (status != eIO_Success  ||  size + n != BUF_Size(uuu->http)) {
-            char* url;
             ELOG_Level level;
             if (status == eIO_Timeout) {
                 const STimeout* tmo = SOCK_GetTimeout(uuu->sock, eIO_Read);
@@ -823,7 +821,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
     BUF_Erase(uuu->http);
 
     /* HTTP status must come on the first line of the response */
-    if (sscanf(header, "HTTP/%*d.%*d %d ", &http_status) != 1 || !http_status)
+    if (sscanf(str, "HTTP/%*d.%*d %d ", &http_status) != 1  ||  !http_status)
         http_status = -1;
     if (http_status < 200  ||  299 < http_status) {
         server_error = http_status;
@@ -856,14 +854,14 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
             header_header = "HTTP header (unrecoverable error)";
         else
             header_header = "HTTP header (server error, retry available)";
-        CORE_DATA_X(9, header, size, header_header);
+        CORE_DATA_X(9, str, size, header_header);
     }
 
     {{
         /* parsing "NCBI-Message" tag */
         static const char kNcbiMessageTag[] = "\n" HTTP_NCBI_MESSAGE " ";
         const char* s;
-        for (s = strchr(header, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
+        for (s = strchr(str, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
             if (strncasecmp(s, kNcbiMessageTag, sizeof(kNcbiMessageTag)-1)==0){
                 const char* message = s + sizeof(kNcbiMessageTag) - 1;
                 while (*message  &&  isspace((unsigned char)(*message)))
@@ -895,8 +893,8 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
 
     if (uuu->flags & fHTTP_KeepHeader) {
         retry->mode = eRetry_None;
-        if (!BUF_Write(&uuu->r_buf, header, size)) {
-            char* url = ConnNetInfo_URL(uuu->net_info);
+        if (!BUF_Write(&uuu->r_buf, str, size)) {
+            url = ConnNetInfo_URL(uuu->net_info);
             CORE_LOGF_X(11, eLOG_Error,
                         ("[HTTP%s%s]  Cannot keep HTTP header",
                          url  &&  *url ? "; " : "",
@@ -905,12 +903,12 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                 free(url);
             status = eIO_Unknown;
         }
-        free(header);
+        free(str);
         return status;
     }
 
     if (uuu->parse_header
-        &&  !uuu->parse_header(header, uuu->user_data, server_error)) {
+        &&  !uuu->parse_header(str, uuu->user_data, server_error)) {
         server_error = 1/*fake, but still boolean true*/;
     }
 
@@ -918,7 +916,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
         /* parsing "Location" pointer */
         static const char kLocationTag[] = "\nLocation: ";
         char* s;
-        for (s = strchr(header, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
+        for (s = strchr(str, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
             if (strncasecmp(s, kLocationTag, sizeof(kLocationTag) - 1) == 0) {
                 char* location = s + sizeof(kLocationTag) - 1;
                 while (*location  &&  isspace((unsigned char)(*location)))
@@ -932,10 +930,10 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                         break;
                 } while (--s > location);
                 if (s != location) {
-                    size_t len = (size_t)(s - location);
-                    memmove(header, location, len);
-                    header[len] = '\0';
-                    retry->data = header;
+                    n = (size_t)(s - location);
+                    memmove(str, location, n);
+                    str[n] = '\0';
+                    retry->data = str;
                 }
                 break;
             }
@@ -944,7 +942,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
         /* parsing "Authenticate" tags */
         static const char kAuthenticateTag[] = "-Authenticate: ";
         char* s;
-        for (s = strchr(header, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
+        for (s = strchr(str, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
             if (strncasecmp(s + (retry->mode == eRetry_Authenticate ? 4 : 6),
                             kAuthenticateTag, sizeof(kAuthenticateTag)-1)==0){
                 if ((retry->mode == eRetry_Authenticate
@@ -966,11 +964,11 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                             break;
                     } while (--e > challenge);
                     if (e != challenge) {
-                        size_t len = (size_t)(e - challenge);
-                        if (s_IsValidAuth(challenge, len)) {
-                            memmove(header, challenge, len);
-                            header[len] = '\0';
-                            retry->data = header;
+                        n = (size_t)(e - challenge);
+                        if (s_IsValidAuth(challenge, n)) {
+                            memmove(str, challenge, n);
+                            str[n] = '\0';
+                            retry->data = str;
                             break;
                         }
                     }
@@ -980,7 +978,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
     } else if (!server_error) {
         static const char kContentLengthTag[] = "\nContent-Length: ";
         const char* s;
-        for (s = strchr(header, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
+        for (s = strchr(str, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
             if (!strncasecmp(s,kContentLengthTag,sizeof(kContentLengthTag)-1)){
                 const char* expected = s + sizeof(kContentLengthTag) - 1;
                 while (*expected  &&  isspace((unsigned char)(*expected)))
@@ -1005,24 +1003,25 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
         }
     }
     if (!retry->data)
-        free(header);
+        free(str);
 
-    /* skip & printout the content, if a server error was flagged */
-    if (server_error && uuu->net_info->debug_printout == eDebugPrintout_Some) {
-        char* url = ConnNetInfo_URL(uuu->net_info);
-        BUF   buf = 0;
-        char* body;
+    if (!server_error)
+        return status;
 
-        SOCK_SetTimeout(uuu->sock, eIO_Read, 0);
-        /* read until EOF */
-        SOCK_StripToPattern(uuu->sock, 0, 0, &buf, 0);
-        if (!(size = BUF_Size(buf))) {
+    /* skip & printout the content by reading until EOF */
+    if (uuu->net_info->debug_printout) {
+        SOCK_SetTimeout(uuu->sock, eIO_Read, kInfiniteTimeout);
+        SOCK_StripToPattern(uuu->sock, 0, 0, &uuu->http, &size);
+    }
+    if (uuu->net_info->debug_printout == eDebugPrintout_Some) {
+        url = ConnNetInfo_URL(uuu->net_info);
+        if (!size) {
             CORE_LOGF_X(12, eLOG_Trace,
                         ("[HTTP%s%s]  No HTTP body received with server error",
                          url  &&  *url ? "; " : "",
                          url           ? url  : ""));
-        } else if ((body = (char*) malloc(size)) != 0) {
-            size_t n = BUF_Read(buf, body, size);
+        } else if ((str = (char*) malloc(size)) != 0) {
+            n = BUF_Read(uuu->http, str, size);
             if (n != size) {
                 CORE_LOGF_X(13, eLOG_Error,
                             ("[HTTP%s%s]  Cannot read server error body"
@@ -1032,12 +1031,12 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                              (unsigned long) n, (unsigned long) size));
             }
             if (n) {
-                CORE_DATAF_X(14, body, n,
+                CORE_DATAF_X(14, str, n,
                              ("[HTTP%s%s] Server error body",
                               url  &&  *url ? "; " : "",
                               url           ? url  : ""));
             }
-            free(body);
+            free(str);
         } else {
             CORE_LOGF_X(15, eLOG_Error,
                         ("[HTTP%s%s]  Cannot allocate server error body,"
@@ -1046,12 +1045,12 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                          url           ? url  : "",
                          (unsigned long) size, &"s"[size == 1]));
         }
-        BUF_Destroy(buf);
         if (url)
             free(url);
     }
+    BUF_Erase(uuu->http);
 
-    return server_error ? eIO_Unknown : status;
+    return eIO_Unknown;
 }
 
 
@@ -1073,17 +1072,12 @@ static EIO_Status s_PreRead(SHttpConnector* uuu,
         SRetry     retry;
 
         status = s_ConnectAndSend(uuu, extract);
-        if (!uuu->sock) {
-            assert(status != eIO_Success);
-            break;
-        }
-        if (status != eIO_Success) {
-            if (status != eIO_Timeout  ||
-                status == SOCK_Status(uuu->sock, eIO_Read)/*pending*/)
-                break;
-        }
-        if (extract == eEM_Flush)
+        if (status != eIO_Success  ||  extract == eEM_Flush)
             break;
+        assert(uuu->sock  &&  uuu->read_state > eRS_WriteRequest);
+
+        if (uuu->read_state == eRS_DoneBody  &&  extract == eEM_Wait)
+            return eIO_Closed;
 
         /* set read timeout before leaving (s_Read() expects it set) */
         SOCK_SetTimeout(uuu->sock, eIO_Read, timeout);
@@ -1091,12 +1085,14 @@ static EIO_Status s_PreRead(SHttpConnector* uuu,
         if (uuu->read_state & eRS_ReadBody)
             break;
 
+        assert(uuu->read_state == eRS_ReadHeader);
         if ((status = s_ReadHeader(uuu, &retry, extract)) == eIO_Success) {
             assert((uuu->read_state & eRS_ReadBody)  &&  !retry.mode);
             /* pending output data no longer needed */
             BUF_Erase(uuu->w_buf);
             break;
         }
+
         assert(status != eIO_Timeout  ||  !retry.mode);
         /* if polling then bail out with eIO_Timeout */
         if (status == eIO_Timeout
@@ -1187,17 +1183,18 @@ static EIO_Status s_Read(SHttpConnector* uuu, void* buf,
 
     if (uuu->expected) {
         const char* how = 0;
-        if (uuu->received > uuu->expected) {
-             status = eIO_Unknown/*received too much*/;
-             how = "too much";
+        if (uuu->expected <= uuu->received) {
+            uuu->read_state = eRS_DoneBody;
+            if (uuu->expected < uuu->received) {
+                status = eIO_Unknown/*received too much*/;
+                how = "too much";
+            }
         } else if (uuu->expected != (TNCBI_BigCount)(-1L)) {
-            if (status == eIO_Closed  &&  uuu->expected > uuu->received) {
+            assert(uuu->expected > uuu->received);
+            if (status == eIO_Closed) {
                 status  = eIO_Unknown/*received too little*/;
                 how = "premature EOF in";
             }
-        } else if (uuu->received) {
-            status = eIO_Unknown/*received too much*/;
-            how = "too much";
         }
         if (how) {
             char* url = ConnNetInfo_URL(uuu->net_info);
@@ -1637,8 +1634,8 @@ static EIO_Status s_CreateHttpConnector
     ConnNetInfo_OverrideUserHeader(xxx, user_header);
     if (tunnel) {
         xxx->req_method = eReqMethod_Connect;
-        *xxx->path = '\0';
-        *xxx->args = '\0';
+        xxx->path[0] = '\0';
+        xxx->args[0] = '\0';
         if (xxx->http_referer) {
             free((void*) xxx->http_referer);
             xxx->http_referer = 0;
@@ -1761,8 +1758,8 @@ extern CONNECTOR HTTP_CreateConnectorEx
 extern EIO_Status HTTP_CreateTunnelEx
 (const SConnNetInfo* net_info,
  THTTP_Flags         flags,
- const void*         init_data,
- size_t              init_size,
+ const void*         data,
+ size_t              size,
  SOCK*               sock)
 {
     EIO_Status      status;
@@ -1780,9 +1777,9 @@ extern EIO_Status HTTP_CreateTunnelEx
         return status;
     }
     assert(uuu  &&  !BUF_Size(uuu->w_buf));
-    if (!init_size  ||  BUF_Prepend(&uuu->w_buf, init_data, init_size)) {
-        if (init_size)
-            sprintf(uuu->net_info->args, "[%lu]", (unsigned long) init_size);
+    if (!size  ||  BUF_Prepend(&uuu->w_buf, data, size)) {
+        if (size)
+            sprintf(uuu->net_info->args, "[%lu]", (unsigned long) size);
         status = s_PreRead(uuu, uuu->net_info->timeout, eEM_Wait);
         if (status == eIO_Success) {
             assert(uuu->read_state == eRS_ReadBody);
