@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsm.c 351842 2012-02-01 18:31:22Z lavr $
+/* $Id: ncbi_lbsm.c 378027 2012-10-17 18:48:13Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,18 +43,23 @@
 
 #ifdef   max
 #  undef max
-#endif
+#endif /*max*/
 #define  max(a, b)  ((a) < (b) ? (b) : (a))
 
 #ifdef   min
 #  undef min
-#endif
+#endif /*min*/
 #define  min(a, b)  ((a) > (b) ? (b) : (a))
 
+#ifdef   fabs
+#  undef fabs
+#endif /*fabs*/
+#define  fabs(v)    ((v) < 0.0 ? -(v) : (v))
+
 #define NCBI_USE_ERRCODE_X   Connect_LBSM
 
 /* Alias for service lookups by type */
-#define eLBSM_AnyService eLBSM_Invalid
+#define eLBSM_PendingOrService eLBSM_Invalid
 
 
 int/*bool*/ LBSM_SetVersion(HEAP heap, const SLBSM_Version* v)
@@ -66,15 +71,16 @@ int/*bool*/ LBSM_SetVersion(HEAP heap, const SLBSM_Version* v)
     if (!v  ||  v->entry.type != eLBSM_Version)
         return 0/*false: bad version*/;
 
-    if ((e = (SLBSM_Entry*) HEAP_Walk(heap, 0))  &&
-        ((short) e->head.flag  ||  HEAP_Walk(heap, &e->head))) {
+    if ((e = (SLBSM_Entry*) HEAP_Walk(heap, 0))
+        &&  ((short) e->head.flag  ||  HEAP_Walk(heap, &e->head))) {
         return 0/*false: heap is not empty*/;
     }
 
     size = sizeof(*v) - sizeof(v->entry.head);
-    if (!(b = (SLBSM_Version*) HEAP_Alloc(heap, size)))
-        return 0/*false: allocation failed*/;
-    assert((void*) b == (void*) HEAP_Base(heap));
+    if (!(b = (SLBSM_Version*) HEAP_Alloc(heap, size))
+        ||  (void*) b != (void*) HEAP_Base(heap)) {
+        return 0/*false: bad allocation*/;
+    }
 
     /* Copy version to the heap, preserving block header */
     memcpy((char*) b + sizeof(b->entry.head),
@@ -88,6 +94,7 @@ const SLBSM_Version* LBSM_GetVersion(HEAP heap)
     const SLBSM_Entry* e = (const SLBSM_Entry*) HEAP_Walk(heap, 0);
     if (!e  ||  (short) e->head.flag == 0  ||  e->type != eLBSM_Version)
         return 0;
+    assert((void*) e == (void*) HEAP_Base(heap));
     return (const SLBSM_Version*) e;
 }
 
@@ -145,8 +152,9 @@ static SLBSM_Service* s_Lookup(HEAP heap, const char* name, int/*bool*/ mask,
             continue/*unused block*/;
         if (e->type != eLBSM_Service  &&  e->type != eLBSM_Pending)
             continue/*not a server entry*/;
-        if (type == eLBSM_AnyService  ||  type == e->type) {
+        if (type == eLBSM_PendingOrService  ||  type == e->type) {
             SLBSM_Service* t = (SLBSM_Service*) e;
+            assert(t->info.host);
             if (!name  ||
                 (!mask && strcasecmp((const char*) t + t->name, name) == 0) ||
                 ( mask && UTIL_MatchesMask((const char*) t + t->name, name))) {
@@ -182,7 +190,8 @@ const SLBSM_Host* LBSM_LookupHost(HEAP               heap,
                 break;
             if ((short) e->head.flag  &&  e->type == eLBSM_Host) {
                 const SLBSM_Host* h = (const SLBSM_Host*) e;
-                if (h->addr == addr)
+                assert(h->addr);
+                if (!addr  ||  h->addr == addr)
                     return h;
             }
         }
@@ -191,12 +200,6 @@ const SLBSM_Host* LBSM_LookupHost(HEAP               heap,
 }
 
 
-#ifdef   MAX
-#  undef MAX
-#endif /*MAX*/
-#define  MAX(a, b) ((a) > (b) ? (a) : (b))
-
-
 ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
 {
     SLBSM_Service* t = 0;
@@ -205,11 +208,11 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
     ELBSM_Type type;
     size_t size;
 
-    if (!s  ||  s->entry.type != eLBSM_Service)
+    if (!s  ||  s->entry.type != eLBSM_Service  ||  !s->info.host)
         return eLBSM_Invalid/*bad call*/;
 
     name = (const char*) s + s->name;
-    type = s->info.quorum ? eLBSM_AnyService : eLBSM_Service;
+    type = s->info.quorum ? eLBSM_PendingOrService : eLBSM_Service;
     while ((t = s_Lookup(heap, name, 0, type, t)) != 0) {
         char host1[40], host2[40];
         int/*bool*/ equal;
@@ -242,7 +245,7 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
                 /* Unowned service is changing its state, so to avoid excessive
                  * flapping make sure state is sticky for at least half-life */
                 if (s->entry.good <
-                    t->entry.good + (MAX(t->info.time, s->info.time) >> 1)) {
+                    t->entry.good + (max(t->info.time, s->info.time) >> 1)) {
                     return type/*fake success*/;
                 }
             }
@@ -307,14 +310,22 @@ TNCBI_Time LBSM_UnpublishHost(HEAP heap, unsigned int addr)
             continue/*free block*/;
         if (e->type == eLBSM_Host) {
             SLBSM_Host* h = (SLBSM_Host*) e;
+            TNCBI_Time jt;
+            assert(h->addr);
             if (h->addr != addr)
                 continue;
-            joined =
-                h->sys.data.joined ? h->sys.data.joined : NCBI_TIME_INFINITE;
+            jt = h->sys.data.joined ? h->sys.data.joined : NCBI_TIME_INFINITE;
+            if (joined > jt  ||  !joined)
+                joined = jt;
         } else if (e->type == eLBSM_Service  ||  e->type == eLBSM_Pending) {
             SLBSM_Service* s = (SLBSM_Service*) e;
+            assert(s->info.host);
             if (s->addr != addr)
                 continue/*not by this host*/;
+            if (s->entry.type == eLBSM_Service
+                &&  (s->info.rate < 0.0  ||  s->info.quorum)) {
+                continue/*keep static/active backup entry*/;
+            }
             if (s->entry.type == eLBSM_Service  &&  s->info.quorum)
                 continue/*keep active backup entry*/;
         } else
@@ -332,7 +343,7 @@ int/*bool*/ LBSM_PublishHost(HEAP heap, const SLBSM_Host* h)
     SLBSM_Host* b;
     size_t size;
 
-    if (!h  ||  h->entry.type != eLBSM_Host)
+    if (!h  ||  h->entry.type != eLBSM_Host  ||  !h->addr)
         return 0/*failure*/;
 
     assert(!LBSM_LookupHost(heap, h->addr, 0));
@@ -366,6 +377,7 @@ size_t LBSM_Expire(HEAP heap, TNCBI_Time time, size_t count)
         }
         if (e->type == eLBSM_Service) {
             const SLBSM_Service* s = (const SLBSM_Service*) e;
+            assert(s->info.host);
             if (s->info.quorum) {
                 e->good = 0;
                 need_backup_rescan = 1;
@@ -373,6 +385,7 @@ size_t LBSM_Expire(HEAP heap, TNCBI_Time time, size_t count)
                           as yet, but later at rescan time*/;
             }
         } else if (e->type == eLBSM_Host) {
+            assert(((const SLBSM_Host*) e)->addr);
             if (count) {
                 const SLBSM_Host* h = (const SLBSM_Host*) e;
                 char host[64];
@@ -411,7 +424,7 @@ static int s_BackupWatchByName(HEAP heap, const char* name, int/*bool*/ log)
     int quorum = 0;
 
     assert(sizeof(s->info.reserved) >= sizeof(double));
-    while ((s = s_Lookup(heap, name, 0, eLBSM_AnyService, s)) != 0) {
+    while ((s = s_Lookup(heap, name, 0, eLBSM_PendingOrService, s)) != 0) {
         if (s->info.quorum) {
             const SLBSM_Service* t = 0;
             if (s->entry.type == eLBSM_Service) {
@@ -476,7 +489,7 @@ static int s_BackupWatchByName(HEAP heap, const char* name, int/*bool*/ log)
         return 0/*false*/;
 
     assert(s == 0);
-    while ((s = s_Lookup(heap, name, 0, eLBSM_AnyService, s)) != 0) {
+    while ((s = s_Lookup(heap, name, 0, eLBSM_PendingOrService, s)) != 0) {
         if (!s->info.rate)
             continue;
         if (s->entry.type == eLBSM_Service) {
@@ -526,11 +539,11 @@ int/*bool*/ LBSM_BackupWatch(HEAP heap, int/*bool*/ log)
     int backed_up = 0/*false*/;
     const SLBSM_Service* s = 0;
 
-    while ((s = s_Lookup(heap, 0/*all*/, 0, eLBSM_AnyService, s)) != 0) {
+    while ((s = s_Lookup(heap, 0/*any*/, 0, eLBSM_PendingOrService, s)) != 0) {
         if (s->info.quorum) {
             const char* name = (const char*) s + s->name;
             const SLBSM_Service* t = 0;
-            while ((t = s_Lookup(heap, name, 0, eLBSM_AnyService, t)) != 0) {
+            while ((t = s_Lookup(heap,name,0,eLBSM_PendingOrService,t)) != 0) {
                 if (t->info.quorum)
                     break;
             }
@@ -549,13 +562,13 @@ double LBSM_CalculateStatus(double rate, double fine, ESERV_Flag flag,
 
     if (!rate)
         return 0.0;
-    if (rate < 0.01)
+    if (rate < LBSM_STANDBY_THRESHOLD)
         status = rate < 0.0 ? -LBSM_DEFAULT_RATE : LBSM_DEFAULT_RATE;
     else
         status = flag == fSERV_Blast ? load->statusBLAST : load->status;
     status *= rate/LBSM_DEFAULT_RATE;
-    /* apply fine */
-    status *= 1.0 - (fine < 0.0 ? 0.0 : (fine > 100.0 ? 100.0 : fine))/100.0;
+    /* accurately apply fine: avoid any imperfections with 100% */
+    status *= (100.0 - (fine < 0.0 ? 0.0 : fine > 100.0 ? 100.0 : fine))/100.0;
     return fabs(status); /*paranoid but safe*/
 }
 
@@ -594,12 +607,12 @@ int/*bool*/ LBSM_SubmitPenaltyOrRerate(const char*    name,
         return 0/*failure*/;
     }
     if (fine)
-        sprintf(value, "%#.0f %u", min(max(0.0, rate), 100.0), fine);
+        NCBI_simple_ftoa(value, min(max(0.0, rate), 100.0), 0);
     else if (rate <= -LBSM_RERATE_DEFAULT  ||  LBSM_RERATE_DEFAULT <= rate)
-        strcpy (value, "DEFAULT");
+        strcpy(value, "DEFAULT");
     else { /*NB: avoid -0.0*/
-        sprintf(value, "%.3f", (!rate ? 0.0 :
-                                min(max(-100000.0, rate), 100000.0)));
+        NCBI_simple_ftoa(value, (!rate ? 0.0 :
+                                 min(max(-100000.0, rate), 100000.0)), 3);
     }
     retval = 0; /*assume worst: failure*/
     len = sprintf(msg, "%u %s %s%s %s %s\n", (unsigned int) geteuid(),
