@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsm_ipc.c 341681 2011-10-21 16:16:41Z lavr $
+/* $Id: ncbi_lbsm_ipc.c 368257 2012-07-05 16:25:00Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -146,7 +146,7 @@ static int/*tri-state*/ s_Shmem_RLock(int which)
 #ifdef LBSM_DEBUG
     CORE_LOG(eLOG_Trace, "LBSM R-lock acquire (%d)", which + 1);
 #endif /*LBSM_DEBUG*/
-    /* For block 1 then block 0: check [1] first, then continue with [2] */
+    /* For block 1 then block 0: check [1] first, then continue with [2]++ */
     if (s_Shmem_Lock(which, 1, which/*1:no-wait,0:wait*/) == 0)
         return which/*block locked*/;
     return !which  ||  errno == EINVAL ? -1 : s_Shmem_Lock(0, 1, 0/*wait*/);
@@ -184,7 +184,7 @@ static int/*tri-state-bool, inverted*/ s_Shmem_TryWLock(int which)
         if (semop(s_Muxid, rwlock, sizeof(rwlock)/sizeof(rwlock[0])) >= 0) {
             /* No new read accesses are allowed after this point */
             s_NoUndo[semnum - 1] = no_undo;
-            /* Look at [2] first, then continue with it */
+            /* Look at [2] first, then continue with it ([2]++) */
             if (s_Shmem_Lock(which, 2, 0/*wait*/) != 0)
                 return 1/*partially locked*/;
             return 0/*okay*/;
@@ -235,7 +235,7 @@ static int/*bool*/ s_Shmem_WLock(int which, int/*bool*/ wait)
         /* even [1] was not successfully locked, so we have either
          *   a/ a hanging writer, or
          *   b/ a hanging old reader (which doesn't change [2] in block 0 only)
-         * In either case, we can try to obtain a PID of the offender.
+         * In either case, we can try to obtain PID of the offender.
          */
         if ((pid = semctl(s_Muxid, (which << 1) | 1, GETPID, dummy)) > 0) {
             int self   = (pid_t) pid == getpid();
@@ -263,7 +263,7 @@ static int/*bool*/ s_Shmem_WLock(int which, int/*bool*/ wait)
     } else {
         pid = 0;
         /* [1] was free (now locked) but [2] was taken by someone else,
-         * we have a hanging reader, no additional info is available.
+         * we have a hanging reader, and no additional info can be obtained.
          */
         if (!wait) {
             union semun arg;
@@ -283,18 +283,16 @@ static int/*bool*/ s_Shmem_WLock(int which, int/*bool*/ wait)
     }
 
     if (!pid) {
-        int val;
-        if ((val = semctl(s_Muxid, (which << 1) + 2, GETVAL, dummy)) > 1) {
-            CORE_LOGF_X(20, eLOG_Warning,
-                        ("%u hanging readers in LBSM shmem[%d]%s",
-                         (unsigned int) val, which + 1, wait ? "" :
-                         locked < 0 ? ", revoking lock" : ", lock revoked"));
-        } else {
-            CORE_LOGF_X(21, eLOG_Warning,
-                        ("A hanging reader in LBSM shmem[%d]%s",
-                         which + 1, wait ? "" :
-                         locked < 0 ? ", revoking lock" : ", lock revoked"));
-        }
+        int  val;
+        char num[32];
+        if ((val = semctl(s_Muxid, (which << 1) + 2, GETVAL, dummy)) > 1)
+            sprintf(num, "%d", val);
+        else
+            strcpy(num, "A");
+        CORE_LOGF_X(20, eLOG_Warning,
+                    ("%s hanging reader%s in LBSM shmem[%d]%s",
+                     num, val > 1 ? "s" : "", which + 1, wait ? "" :
+                     locked < 0 ? ", revoking lock" : ", lock revoked"));
     }
 
     if (wait)
@@ -345,9 +343,8 @@ static HEAP s_Shmem_Attach(int which)
 }
 
 
-HEAP LBSM_Shmem_Attach(void)
+HEAP LBSM_Shmem_Attach(int/*bool*/ fallback)
 {
-    int  fallback = 0; /*FIXME: to become a parameter*/
     int  which;
     HEAP heap;
 
@@ -515,7 +512,7 @@ unsigned int LBSM_Shmem_Update(HEAP heap, int/*bool*/ wait)
 
     assert(heapbase  &&  heapsize);
     for (i = 0;  i < 2;  i++) {
-        int can_wait = wait ||  (s_ShmemSize[i]  &&  (!i  ||  updated));
+        int can_wait = wait  ||  (s_ShmemSize[i]  &&  (!i  ||  updated));
         if (!s_Shmem_WLock(i, can_wait))
             continue;
 
@@ -539,7 +536,7 @@ unsigned int LBSM_Shmem_Update(HEAP heap, int/*bool*/ wait)
         }
         memcpy(s_Shmem[i], heapbase, heapsize);
         if (!s_Shmem_WUnlock(i)) {
-            CORE_LOGF_ERRNO_X(23, eLOG_Warning, errno,
+            CORE_LOGF_ERRNO_X(21, eLOG_Warning, errno,
                               ("Update failed to unlock shmem[%d]", i + 1));
         }
         updated |= 1 << i;
