@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsmd.c 254619 2011-02-16 15:16:38Z lavr $
+/* $Id: ncbi_lbsmd.c 389593 2013-02-19 18:34:16Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,10 +40,18 @@
 #include "ncbi_priv.h"
 #include "ncbi_version.h"
 #include <errno.h>
-#include <math.h>
 #include <stdlib.h>
 #include <time.h>
 
+#ifdef   fabs
+#  undef fabs
+#endif /*fabs*/
+#define  fabs(v)         ((v) < 0.0 ? -(v) : (v))
+
+#define  copysign(x, y)  ((y) < 0.0                     \
+                          ? ((x) < 0.0 ?  (x) : -(x))   \
+                          : ((x) < 0.0 ? -(x) :  (x)))
+
 #define NCBI_USE_ERRCODE_X   Connect_LBSMD
 
 #define MAX_IP_ADDR_LEN 16      /* sizeof("255.255.255.255") */
@@ -60,7 +68,7 @@ extern "C" {
     static void        s_Close      (SERV_ITER);
 
     static const SSERV_VTable s_op = {
-        0/*Reset*/, s_GetNextInfo, 0/*Update*/, s_Feedback, s_Close, "LBSMD"
+        s_GetNextInfo, s_Feedback, 0/*Update*/, 0/*Reset*/, s_Close, "LBSMD"
     };
 #ifdef __cplusplus
 } /* extern "C" */
@@ -84,11 +92,14 @@ typedef enum {
 
 static unsigned int s_Localhost(const SLBSM_Version* v)
 {
-    if (v->entry.head.size >= 32) { /*FIXME: remove this condition*/
-        if (!memchr(&v->local, '\x00', sizeof(v->local))  &&
-            !memchr(&v->local, '\xFF', sizeof(v->local))) {
-            return v->local;
-        }
+    if (v->major > LBSM_HEAP_VERSION_MAJ_CRC32
+        ||  (v->major == LBSM_HEAP_VERSION_MAJ_CRC32
+             &&  v->minor > LBSM_HEAP_VERSION_MIN_CRC32)) {
+        return v->local;
+    }
+    if (v->entry.head.size >= 32/*FIXME: remove this condition*/
+        &&  v->local  &&  v->local != (unsigned int)(-1)) {
+        return v->local;
     }
     return 0;
 }
@@ -242,7 +253,7 @@ static int s_SortStandbys(const void* p1, const void* p2)
         if (c2->cand.status > 0.0)
             return  1;
     }
-    assert(c1->svc->info.rate * c2->svc->info.rate > 0.0); /* same sign */
+    assert(c1->svc->info.rate * c2->svc->info.rate >= 0.0); /* same sign */
     /* randomization is done by allocation of services on the heap */
     return (int) copysign(1.0,
                           fabs(c2->svc->info.rate) - fabs(c1->svc->info.rate));
@@ -318,28 +329,27 @@ static int/*bool*/ s_VerifyChecksum(const HEAP heap, unsigned int cksum)
  *
  * In order to make heap accesses as efficient as possible:
  *
- * 1. "ifcopy1" governs whether a copy of actual LBSM heap v1.1-
- *    is made (instead of returning the live LBSM shmem;  neither
- *    is cached since there's no way to check for freshness of v1.1-).
- *    For v1.2+ a copy is always made and retained in "s_Heap"
- *    (regardless of "ifcopy1").
+ * If s_FastHeapAccess is non-zero, each iterator has its own (cached) copy of
+ * the LBSM heap, and keeps re-using it (w/o expiration checks) until closed.
  *
- *    Cached copies have negative serial numbers, non-cached
- *    copies (or live heaps) have positive serial numbers.
+ * If s_FastHeapAccess is zero, s_Heap (which then may be expired and renewed
+ * in-between searches) is a cached copy and is used for service look-ups.
  *
- *    The absolute value of a serial number corresponds to the
- *    actual LBSM shmem block number [1] or [2].
- *
- * 2. "s_Heap" never retains any live heap (but a ref-counted copy of v1.2+).
+ * Serial numbers of original (attached) heaps are positive, while those of
+ * copied heaps (including the global s_Heap) are negative (negated original).
  */
 
-static HEAP s_GetHeap(TNCBI_Time now, int/*bool*/ ifcopy1)
+static HEAP s_GetHeapCopy(TNCBI_Time now)
 {
-    HEAP result = 0;
-    HEAP heap;
-    int n;
+    enum {
+        eNone     = 0,
+        eAgain    = 1,
+        eFallback = 2
+    } retry   = eNone;
+    HEAP heap = 0;
+    HEAP lbsm;
 
-    for (n = 0; ; n = 1) {
+    for (;;) {
         const SLBSM_Version *c, *v;
         int serial = 0;
 
@@ -348,7 +358,7 @@ static HEAP s_GetHeap(TNCBI_Time now, int/*bool*/ ifcopy1)
         if (s_Heap) {
             c = LBSM_GetVersion(s_Heap);
             assert(c  &&  c->major == LBSM_HEAP_VERSION_MAJ);
-            assert(!n  &&  (void*) c == (void*) HEAP_Base(s_Heap));
+            assert((void*) c == (void*) HEAP_Base(s_Heap));
             if (c->entry.good < now) {
 #ifdef LBSM_DEBUG
                 CORE_LOGF(eLOG_Trace,
@@ -367,56 +377,58 @@ static HEAP s_GetHeap(TNCBI_Time now, int/*bool*/ ifcopy1)
 #endif /*LBSM_DEBUG*/
         } else
             c = 0/*dummy for compiler not to complain*/;
-
-        if (!(heap = LBSM_Shmem_Attach()) || (serial = HEAP_Serial(heap)) <=0){
-            if (heap) {
+        
+        if (!(lbsm = LBSM_Shmem_Attach(retry == eFallback))
+            ||  (serial = HEAP_Serial(lbsm)) <= 0) {
+            if (lbsm) {
                 CORE_LOGF_X(1, eLOG_Error,
                             ("Bad serial (%d) from LBSM heap attach", serial));
             } /* else, an error has already been posted */
             break;
         }
 
-        if (!(v = LBSM_GetVersion(heap)) || v->major < LBSM_HEAP_VERSION_MAJ) {
+        if (!(v = LBSM_GetVersion(lbsm))
+            ||  (v->major < LBSM_HEAP_VERSION_MAJ
+                 ||  (v->major == LBSM_HEAP_VERSION_MAJ
+                      &&  v->minor < LBSM_HEAP_VERSION_MIN))) {
             if (v) {
                 CORE_LOGF_X(2, eLOG_Error,
                             ("LBSM heap[%d] version mismatch"
-                             " (current=%hu.%hu, expected=%u+)",
+                             " (current=%hu.%hu, expected=%u.%u+)",
                              serial, v->major, v->minor,
-                             LBSM_HEAP_VERSION_MAJ));
+                             LBSM_HEAP_VERSION_MAJ, LBSM_HEAP_VERSION_MIN));
             } else {
-                CORE_LOG_X(3, eLOG_Error,
-                           "LBSM heap[%d] has no version");
+                CORE_LOGF_X(3, eLOG_Error,
+                            ("LBSM heap[%d] has no version", serial));
             }
             break;
         }
 
-        if (now > v->entry.good) {
-            /* expired v1.2+ */
+        if (v->entry.good < now) {
             CORE_LOGF_X(4, eLOG_Warning,
                         ("LBSM heap[%d] is out-of-date"
-                         " (current=%lu, expiry=%lu, delta=%lu)", serial,
+                         " (current=%lu, expiry=%lu, delta=%lu)%s", serial,
                          (unsigned long) now, (unsigned long) v->entry.good,
-                         (unsigned long) now -(unsigned long) v->entry.good));
-            if (!(result = HEAP_Copy(heap, 0, serial))) {
-                CORE_LOGF_ERRNO_X(5, eLOG_Error, errno,
-                                  ("Cannot copy LBSM heap[%d]", serial));
-            }
-#ifdef LBSM_DEBUG
-            else {
-                CORE_LOGF(eLOG_Trace,
-                          ("LBSM heap[%d] copied", serial));
+                         (unsigned long) now -(unsigned long) v->entry.good,
+                         !retry  &&  serial > 1 ? ", re-trying" : ""));
+            if (!retry  &&   serial > 1) {
+                LBSM_Shmem_Detach(heap);
+                retry = eFallback;
+                CORE_UNLOCK;
+                continue;
             }
             if (s_Heap) {
+#ifdef LBSM_DEBUG
                 CORE_LOGF(eLOG_Trace,
                           ("Cached LBSM heap[%p, %p, %d] dropped",
                            s_Heap, HEAP_Base(s_Heap), HEAP_Serial(s_Heap)));
-            }
 #endif /*LBSM_DEBUG*/
-            HEAP_Destroy(s_Heap);
-            s_Heap = 0;
+                HEAP_Destroy(s_Heap);
+                s_Heap = 0;
+            }
             break;
         }
-        assert((void*) v == (void*) HEAP_Base(heap));
+        assert((void*) v == (void*) HEAP_Base(lbsm));
 
         if (s_Heap) {
             if (c->count == v->count  &&  c->cksum == v->cksum) {
@@ -425,7 +437,7 @@ static HEAP s_GetHeap(TNCBI_Time now, int/*bool*/ ifcopy1)
                           ("Cached LBSM heap[%p, %p, %d] used",
                            s_Heap, HEAP_Base(s_Heap), HEAP_Serial(s_Heap)));
 #endif /*LBSM_DEBUG*/
-                result = s_Heap;
+                heap = s_Heap;
                 break;
             }
 #ifdef LBSM_DEBUG
@@ -437,62 +449,55 @@ static HEAP s_GetHeap(TNCBI_Time now, int/*bool*/ ifcopy1)
             s_Heap = 0;
         }
 
-        if (!(result = HEAP_Copy(heap, 0, -serial))) {
+        if (!(heap = HEAP_Copy(lbsm, 0, -serial))) {
             CORE_LOGF_ERRNO_X(6, eLOG_Error, errno,
                               ("Unable to copy LBSM heap[%d]", serial));
             break;
         }
 
-        if (s_VerifyChecksum(result, v->cksum)) {
+        if (s_VerifyChecksum(heap, v->cksum)) {
 #ifdef LBSM_DEBUG
             CORE_LOGF(eLOG_Trace,
                       ("Cached LBSM heap[%p, %p, %d] renewed",
-                       result, HEAP_Base(result), HEAP_Serial(result)));
+                       heap, HEAP_Base(heap), HEAP_Serial(heap)));
 #endif /*LBSM_DEBUG*/
-            s_Heap = result;
+            s_Heap = heap;
             break;
         }
 
-        CORE_LOGF_X(7, n ? eLOG_Error : eLOG_Warning,
-                    ("Cached LBSM heap[%p, %p, %d]%s checksum failure%s",
-                     (void*) result, HEAP_Base(result), HEAP_Serial(result),
-                     n ? " persistent" : "",
-                     n ? ""            : ", trying to copy again"));
-        errno = EAGAIN;
+        CORE_LOGF_X(7, retry ? eLOG_Error : eLOG_Warning,
+                    ("LBSM heap[%p, %p, %d]%s checksum failure%s",
+                     (void*) heap, HEAP_Base(heap), HEAP_Serial(heap),
+                     retry == eAgain ? " persistent" : "",
+                     retry           ? ""            : ", re-trying"));
 
-        HEAP_Destroy(result);
-        result = 0;
-        if (n)
+        verify(HEAP_Destroy(heap) == 0);
+        heap = 0;
+        if (retry)
             break;
 
-        LBSM_Shmem_Detach(heap);
-
+        LBSM_Shmem_Detach(lbsm);
+        retry = eAgain;
         CORE_UNLOCK;
     }
 
-    if (!result) {
-        LBSM_Shmem_Detach(heap);
-    } else if (result == s_Heap) {
-        LBSM_Shmem_Detach(heap);
-        HEAP_AddRef(s_Heap);
-    } else if (ifcopy1) {
-        LBSM_Shmem_Detach(heap);
-    }
+    assert(!heap  ||  heap != lbsm);
+    if (heap  &&  heap == s_Heap)
+        verify(HEAP_AddRef(s_Heap) > 1);
 
+    LBSM_Shmem_Detach(lbsm);
     CORE_UNLOCK;
-
-    return result;
+    return heap;
 }
 
 
 static const SLBSM_Host* s_LookupHost(HEAP heap, const SERV_ITER iter,
-                                      const SLBSM_Service* svc,
-                                      const SLBSM_Service* hint)
+                                      const SLBSM_Service* svc)
 {
     unsigned int addr =
         svc->info.host ? svc->info.host : s_GetLocalHostAddress(heap);
-    const SLBSM_Host* host = LBSM_LookupHost(heap, addr, &hint->entry);
-    if (!host  ||  (hint  &&  host->entry.good < iter->time)) {
+    const SLBSM_Host* host = LBSM_LookupHost(heap, addr, &svc->entry);
+    if (!host  ||  host->entry.good < iter->time) {
         if (svc->info.rate > 0.0) {
             char buf[40];
             if (SOCK_ntoa(addr, buf, sizeof(buf)) != 0)
@@ -511,7 +516,7 @@ static const SLBSM_Host* s_LookupHost(HEAP heap, const SERV_ITER iter,
 
 static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
 {
-    const TSERV_Type type = iter->type & ~fSERV_Firewall;
+    const TSERV_Type types = iter->types & ~fSERV_Firewall;
     size_t i, n, idx[eHost_NoMatch], n_cand, a_cand;
     EHost_Match best_match, match;
     struct SLBSM_Candidate* cand;
@@ -520,9 +525,9 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
     const SLBSM_Host* host;
     const char* env, *a;
     SSERV_Info* info;
+    const char* name;
     double status;
     int standby;
-    int serial;
     HEAP heap;
     char* v;
 
@@ -534,14 +539,13 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
                    heap, HEAP_Base(heap), HEAP_Serial(heap), iter->name));
 #endif /*LBSM_DEBUG*/
         /*noop*/;
-    } else if (!(heap = s_GetHeap(iter->time, !s_FastHeapAccess))) {
-        return iter->external  ||  iter->data == iter  ||  !(type & fSERV_Dns)
+    } else if (!(heap = s_GetHeapCopy(iter->time))) {
+        return iter->external  ||  iter->data == iter  ||  !(types & fSERV_Dns)
             ? 0 : s_FakeDnsReturn(iter, host_info, 0, NCBI_TIME_INFINITE);
     }
 
     best_match = eHost_InitMatch;
     memset(idx, 0, sizeof(idx));
-    serial = HEAP_Serial(heap);
     standby = -1/*unassigned*/;
     dns_info_time = 0/*none*/;
     n = n_cand = a_cand = 0;
@@ -549,14 +553,16 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
     cand = 0;
     svc = 0;
 
-    while ((svc = LBSM_LookupService(heap, iter->name, iter->ismask, svc))) {
-        if (serial < 0  &&  svc->entry.good < iter->time)
+    name = *iter->name ? iter->name : 0;
+    assert(name  ||  iter->ismask); /*NB: ismask ignored for NULL*/
+    while ((svc = LBSM_LookupService(heap, name, iter->ismask, svc))) {
+        if (svc->entry.good < iter->time)
             continue; /* out-of-date entry */
 
         if (!svc->info.time)
             continue; /* off */
 
-        if (type != fSERV_Any  &&  !(type & svc->info.type))
+        if (types != fSERV_Any  &&  !(types & svc->info.type))
             continue; /* type doesn't match */
 
         if (iter->external  &&  svc->info.locl)
@@ -571,7 +577,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
         }
 
         if (svc->info.type == fSERV_Dns) {
-            if (type == fSERV_Any)
+            if (types == fSERV_Any)
                 continue; /* DNS entries have to be requested explicitly */
             if (!iter->ismask) {
                 if (dns_info_time < svc->info.time)
@@ -590,7 +596,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
         }
 
         if (svc->info.rate > 0.0  ||  host_info) {
-            if (!(host = s_LookupHost(heap, iter, svc, serial < 0 ? svc : 0))
+            if (!(host = s_LookupHost(heap, iter, svc))
                 &&  svc->info.rate > 0.0) {
                 continue; /* no host information for non-static server */
             }
@@ -619,6 +625,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
                 break;
             }
         }
+        /*FIXME*//*CORE_LOG(eLOG_Note, (char*) svc + svc->name);*/
         if (n >= iter->n_skip) {
             status = LBSM_CalculateStatus(svc->info.rate, svc->fine,
                                           svc->info.flag, &host->sys.load);
@@ -646,7 +653,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
         } else if (iter->arglen) {
             assert(iter->arg);
             if (!host)
-                host = s_LookupHost(heap, iter, svc, serial < 0 ? svc : 0);
+                host = s_LookupHost(heap, iter, svc);
             if ( host  &&  host->env)
                 env = (const char*) host + host->env;
             match = s_Match(env,
@@ -672,7 +679,8 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
                         continue;
                     /* this has to be given out as a suppressed one */
                     status = copysign(svc->info.rate, -1.0);
-                }
+                } else if (standby < 0)
+                    standby = 1;
             } else if (standby) {
                 standby = 0/*cancel*/;
                 if (!iter->ok_suppressed) {
@@ -724,11 +732,11 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
             CORE_LOGF(eLOG_Note, ("%s%s%s%s: %s%s%s%s%s%s", s,
                                   env ? " <" : "", env ? env : "",
                                   env ? ">"  : "", m,
-                                  a   ? ", arg="          : "",
-                                  a   ? a                 : "",
-                                  v   ? ", val="          : "",
-                                  v   ? (*v ? v : "\"\"") : "",
-                                  standby   ? ", standby" : ""));
+                                  a   ? ", arg="             : "",
+                                  a   ? a                    : "",
+                                  v   ? ", val="             : "",
+                                  v   ? (*v ? v : "\"\"")    : "",
+                                  standby > 0 ? ", standby"  : ""));
             free(s);
         }
 #endif /*NCBI_LB_DEBUG*/
@@ -779,7 +787,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
     if (n_cand) {
         assert(cand);
         do {
-            if (!standby) {
+            if (standby <= 0) {
                 struct SLBSM_Data data;
                 data.cand   = cand;
                 data.n_cand = n_cand;
@@ -841,10 +849,8 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
                             (const char*) svc + svc->name : "");
         if ((info = SERV_CopyInfoEx(&svc->info, name)) != 0) {
             info->rate = cand[n].cand.status;
-            if (info->time != NCBI_TIME_INFINITE) {
-                info->time = (serial < 0 ? cand[n].svc->entry.good :
-                              cand[n].svc->info.time + iter->time);
-            }
+            if (info->time != NCBI_TIME_INFINITE)
+                info->time  = cand[n].svc->entry.good;
             if (host_info) {
                 if ((host = cand[n].host) != 0) {
                     *host_info =
@@ -872,28 +878,25 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
     if (cand)
         free(cand);
 
-    if (serial < 0  ||  !s_FastHeapAccess) {
-        if (!s_FastHeapAccess) {
+    if (!s_FastHeapAccess) {
 #ifdef LBSM_DEBUG
-            CORE_LOGF(eLOG_Trace,
-                      ("LBSM heap[%p, %p, %d] for \"%s\" released",
-                       heap, HEAP_Base(heap), HEAP_Serial(heap), iter->name));
+        CORE_LOGF(eLOG_Trace,
+                  ("LBSM heap[%p, %p, %d] for \"%s\" released",
+                   heap, HEAP_Base(heap), HEAP_Serial(heap), iter->name));
 #endif /*LBSM_DEBUG*/
-            CORE_LOCK_WRITE;
-            HEAP_Detach(heap);
-            CORE_UNLOCK;
-            heap = 0;
-        }
+        CORE_LOCK_WRITE;
+        HEAP_Detach(heap);
+        CORE_UNLOCK;
+        heap = 0;
+    }
 #ifdef LBSM_DEBUG
-        else {
-            CORE_LOGF(eLOG_Trace,
-                      ("LBSM heap[%p, %p, %d] for \"%s\" retained",
-                       heap, HEAP_Base(heap), HEAP_Serial(heap), iter->name));
-        }
+    else {
+        CORE_LOGF(eLOG_Trace,
+                  ("LBSM heap[%p, %p, %d] for \"%s\" retained",
+                   heap, HEAP_Base(heap), HEAP_Serial(heap), iter->name));
+    }
 #endif /*LBSM_DEBUG*/
-        iter->data = heap;
-    } else
-        LBSM_Shmem_Detach(heap);
+    iter->data = heap;
 
     return info;
 }
@@ -901,9 +904,10 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
 
 static int/*bool*/ s_Feedback(SERV_ITER iter, double rate, int/*bool*/ fine)
 {
-    assert(iter->last);
-    return LBSM_SubmitPenaltyOrRerate(iter->name, iter->last->type, rate, fine,
-                                      iter->last->host, iter->last->port, 0);
+    SSERV_InfoCPtr last = iter->last;
+    assert(last);
+    return LBSM_SubmitPenaltyOrRerate(SERV_CurrentName(iter), last->type,
+                                      rate, fine, last->host, last->port, 0);
 }
 
 
@@ -917,7 +921,7 @@ static void s_Close(SERV_ITER iter)
                    HEAP_Serial((HEAP) iter->data), iter->name));
 #endif /*LBSM_DEBUG*/
         CORE_LOCK_WRITE;
-        HEAP_Destroy((HEAP) iter->data);
+        HEAP_Detach((HEAP) iter->data);
         CORE_UNLOCK;
         iter->data = 0;
     }
@@ -1007,7 +1011,7 @@ ESwitch LBSMD_FastHeapAccess(ESwitch OnOff)
 
 HEAP LBSMD_GetHeapCopy(TNCBI_Time now)
 {
-    return s_GetHeap(now, 1/*do copy*/);
+    return s_GetHeapCopy(now);
 }
 
 
@@ -1017,20 +1021,17 @@ const char* LBSMD_GetConfig(void)
     HEAP heap;
 
     if (LBSM_LBSMD(0) > 0  &&  errno == EAGAIN) {
-        if ((heap = s_GetHeap(time(0), !s_FastHeapAccess)) != 0) {
+        if ((heap = s_GetHeapCopy(time(0))) != 0) {
             if ((s = LBSM_GetConfig(heap)) != 0)
                 s = strdup(s);
-            if (HEAP_Serial(heap) < 0  ||  !s_FastHeapAccess) {
 #ifdef LBSM_DEBUG
-                CORE_LOGF(eLOG_Trace,
-                          ("LBSM heap[%p, %p, %d] released",
-                           heap, HEAP_Base(heap), HEAP_Serial(heap)));
+            CORE_LOGF(eLOG_Trace,
+                      ("LBSM heap[%p, %p, %d] released",
+                       heap, HEAP_Base(heap), HEAP_Serial(heap)));
 #endif /*LBSM_DEBUG*/
-                CORE_LOCK_WRITE;
-                HEAP_Detach(heap);
-                CORE_UNLOCK;
-            } else
-                LBSM_Shmem_Detach(heap);
+            CORE_LOCK_WRITE;
+            HEAP_Detach(heap);
+            CORE_UNLOCK;
         }
     }
     if (!s_FastHeapAccess) {
@@ -1050,11 +1051,11 @@ const char* LBSMD_GetHostParameter(unsigned int addr,
 
     if (!namelen
         ||  LBSM_LBSMD(0) <= 0  ||  errno != EAGAIN
-        ||  !(heap = LBSMD_GetHeapCopy(time(0)))) {
+        ||  !(heap = s_GetHeapCopy(time(0)))) {
         return 0;
     }
     if (addr == SERV_ANYHOST  ||  addr == SERV_LOCALHOST)
-        addr = s_GetLocalHostAddress(heap);
+        addr  = s_GetLocalHostAddress(heap);
     if ((host = LBSM_LookupHost(heap, addr, 0)) != 0  &&  host->env) {
         const char* e = (char*) host + host->env;
         const char* env;
@@ -1078,7 +1079,7 @@ const char* LBSMD_GetHostParameter(unsigned int addr,
         }
     }
     CORE_LOCK_WRITE;
-    HEAP_Destroy(heap);
+    HEAP_Detach(heap);
     CORE_UNLOCK;
     return 0;
 }
@@ -1100,14 +1101,14 @@ static const SLBSM_Sysinfo* s_GetSysinfo(const HOST_INFO hinfo,
                 strncpy0(addr, "unknown", sizeof(addr) - 1);
             CORE_LOGF(s_Warn++ < 5 ? eLOG_Warning : eLOG_Trace,
                       ("HINFO may be incorrect for obsolete daemon on %s"
-                       " (detected=%u.%u.%u, expected=%s+)", addr,
+                       " (detected=%hu.%hu.%hu, expected=%s+)", addr,
                        NETDAEMONS_MAJOR_OF(si->data.version),
                        NETDAEMONS_MINOR_OF(si->data.version),
                        NETDAEMONS_PATCH_OF(si->data.version),
                        NETDAEMONS_VERSION_STR));
         }
     }
-#endif /*NETDAEMONS_VERSION_INT*/
+#endif /*_DEBUG && !NDEBUG && NETDAEMONS_VERSION_INT*/
     return si;
 }
 
@@ -1122,7 +1123,7 @@ int LBSM_HINFO_CpuCount(const HOST_INFO hinfo)
 int LBSM_HINFO_CpuUnits(const HOST_INFO hinfo)
 {
     const SLBSM_Sysinfo* si = s_GetSysinfo(hinfo, 1/*warn*/);
-    return (si->data.nrproc >> 8) & 0xFF;
+    return si->data.nrproc >> 8;
 }
 
 
@@ -1156,15 +1157,19 @@ int/*bool*/ LBSM_HINFO_Memusage(const HOST_INFO hinfo, double memusage[5])
 int/*bool*/ LBSM_HINFO_MachineParams(const HOST_INFO hinfo, SHINFO_Params* p)
 {
     const SLBSM_Sysinfo* si = s_GetSysinfo(hinfo, 1/*warn*/);
-    p->kernel.major = (si->data.kernel >> 11) & 0x0F;
-    p->kernel.minor = (si->data.kernel >> 6)  & 0x1F;
-    p->kernel.patch =  si->data.kernel        & 0x3F;
-    p->pgsize       =  si->data.pgsize << 10;
-    p->bootup       =  si->data.sys_uptime;
-    p->start        =  si->data.start_time;
-    p->daemon.major = NETDAEMONS_MAJOR_OF(si->data.version);
-    p->daemon.minor = NETDAEMONS_MINOR_OF(si->data.version);
-    p->daemon.patch = NETDAEMONS_PATCH_OF(si->data.version);
+    unsigned short div   =  si->data.kernel & 0x8000 ? 10 : 1;
+    unsigned short major = (si->data.kernel >> 24) & 0xFF;
+    unsigned short minor = (si->data.kernel >> 16) & 0xFF;
+    p->kernel.patch      =  si->data.kernel        & 0x7FFF;
+    p->pgsize            =  si->data.pgsize << 10;
+    p->bootup            =  si->data.sys_uptime;
+    p->startup           =  si->data.start_time;
+    p->daemon.major      = NETDAEMONS_MAJOR_OF(si->data.version);
+    p->daemon.minor      = NETDAEMONS_MINOR_OF(si->data.version);
+    p->daemon.patch      = NETDAEMONS_PATCH_OF(si->data.version);
+    p->kernel.major      =   major / div;
+    p->kernel.minor      =   minor / div;
+    p->svcpack           = ((major % div) << 8) | (minor % div);
     return 1/*success*/;
 }
 
