@@ -1,4 +1,4 @@
-/* $Id: ncbi_namedpipe.cpp 344089 2011-11-11 17:47:02Z lavr $
+/* $Id: ncbi_namedpipe.cpp 356160 2012-03-12 18:38:34Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,7 +35,6 @@
 #include <connect/ncbi_namedpipe.hpp>
 #include <connect/ncbi_util.h>
 #include <corelib/ncbifile.hpp>
-#include <corelib/ncbi_system.hpp>
 
 #if defined(NCBI_OS_MSWIN)
 
@@ -97,7 +96,7 @@ inline void s_AdjustPipeBufSize(size_t& bufsize)
     if (!bufsize) {
         bufsize = kDefaultPipeBufSize;
     } else if (bufsize == (size_t) CNamedPipe::eDefaultSysBufSize) {
-        bufsize = 0 /* use system default buffer size */;
+        bufsize = 0/*use system default buffer size*/;
     }
 }
 
@@ -225,7 +224,7 @@ EIO_Status CNamedPipeHandle::Open(const string&   pipename,
         }
         // Save parameters
         m_PipeName    = pipename;
-        m_PipeBufSize = 0/*pipebufsize is not used on client side*/;
+        m_PipeBufSize = 0/*pipebufsize not used on client side*/;
 
         // Set the base security attributes
         SECURITY_ATTRIBUTES attr;
@@ -288,6 +287,9 @@ EIO_Status CNamedPipeHandle::Create(const string& pipename,
         if (m_Pipe != INVALID_HANDLE_VALUE) {
             throw string("Named pipe already open");
         }
+        if (pipebufsize > numeric_limits<DWORD>::max()) {
+            throw string("Buffer size too large");
+        }
         // Save parameters
         m_PipeName    = pipename;
         m_PipeBufSize = pipebufsize;
@@ -304,8 +306,8 @@ EIO_Status CNamedPipeHandle::Create(const string& pipename,
              PIPE_ACCESS_DUPLEX,            // read/write access 
              PIPE_TYPE_BYTE | PIPE_NOWAIT,  // byte-type, nonblocking mode 
              1,                             // one instance only 
-             pipebufsize,                   // output buffer size 
-             pipebufsize,                   // input buffer size 
+             (DWORD) pipebufsize,           // output buffer size 
+             (DWORD) pipebufsize,           // input buffer size 
              INFINITE,                      // client time-out by default
              &attr);                        // security attributes
 
@@ -497,8 +499,8 @@ EIO_Status CNamedPipeHandle::Read(void* buf, size_t count, size_t* n_read,
             _ASSERT(bytes_avail);
             // We must read only "count" bytes of data regardless of the amount
             // available to read.
-            if (bytes_avail > count) {
-                bytes_avail = count;
+            if (bytes_avail >         count) {
+                bytes_avail = (DWORD) count;
             }
             if ( !::ReadFile(m_Pipe, buf, bytes_avail, &bytes_avail, NULL) ) {
                 if ( !bytes_avail ) {
@@ -548,13 +550,16 @@ EIO_Status CNamedPipeHandle::Write(const void* buf, size_t count,
 
         status = eIO_Unknown;
         DWORD x_timeout = timeout ? NcbiTimeoutToMs(timeout) : INFINITE;
+        DWORD to_write  = (count > numeric_limits<DWORD>::max()
+                           ? numeric_limits<DWORD>::max()
+                           : (DWORD) count);
         DWORD bytes_written = 0;
 
         // Wait for data from the pipe with timeout.
         // NOTE:  WaitForSingleObject() does not work with pipes.
 
         for (;;) {
-            if ( !::WriteFile(m_Pipe, buf, count, &bytes_written, NULL) ) {
+            if ( !::WriteFile(m_Pipe, buf, to_write, &bytes_written, NULL) ) {
                 // NB:  status == eIO_Unknown
                 if ( !bytes_written ) {
                     DWORD error = ::GetLastError();
@@ -904,7 +909,7 @@ EIO_Status CNamedPipeHandle::Read(void* buf, size_t count, size_t* n_read,
 
         status = SOCK_SetTimeout(m_IoSocket, eIO_Read, timeout);
         if (status == eIO_Success) {
-            status = SOCK_Read(m_IoSocket, buf, count, n_read, eIO_ReadPlain);
+            status  = SOCK_Read(m_IoSocket, buf, count, n_read, eIO_ReadPlain);
         }
     }
     catch (string& what) {
@@ -932,8 +937,8 @@ EIO_Status CNamedPipeHandle::Write(const void* buf, size_t count,
 
         status = SOCK_SetTimeout(m_IoSocket, eIO_Write, timeout);
         if (status == eIO_Success) {
-            status = SOCK_Write(m_IoSocket, buf, count, n_written,
-                                eIO_WritePlain);
+            status  = SOCK_Write(m_IoSocket, buf, count, n_written,
+                                 eIO_WritePlain);
         }
     }
     catch (string& what) {
