@@ -1,4 +1,4 @@
-/* $Id: ncbi_sendmail.c 360593 2012-04-24 17:54:30Z lavr $
+/* $Id: ncbi_sendmail.c 373967 2012-09-05 15:30:20Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,7 +40,7 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_Sendmail
 
-#define MX_MAGIC_NUMBER 0xBA8ADEDA
+#define MX_MAGIC_COOKIE 0xBA8ADEDA
 #define MX_CRLF         "\r\n"
 
 #define SMTP_READERR    -1      /* Read error from socket               */
@@ -84,7 +84,7 @@ static int s_SockRead(SOCK sock, char* response, size_t max_response_len)
         assert(m == 4);
 
         if (buf[3] == '-'  ||  (done = isspace((unsigned char) buf[3]))) {
-            buf[3] = '\0';
+            buf[3]  = '\0';
             if (!code) {
                 char* e;
                 errno = 0;
@@ -214,16 +214,16 @@ static STimeout       s_MxTmo;
 
 static void x_Sendmail_InitEnv(void)
 {
-    char         buf[sizeof(s_MxHost)];
+    char         buf[sizeof(s_MxHost)], *e;
     unsigned int port;
     double       tmo;
 
     if (*s_MxHost)
         return;
 
-    if (!ConnNetInfo_GetValue(0, "MX_TIMEOUT", buf, sizeof(buf), 0)
-        ||  (tmo = atof(buf)) < 0.000001) {
-        tmo = 120.0;
+    if (!ConnNetInfo_GetValue(0, "MX_TIMEOUT", buf, sizeof(buf), 0)  ||  !*buf
+        ||  (tmo = NCBI_simple_atof(buf, &e)) < 0.000001  ||  errno  ||  !*e) {
+        tmo = 120.0/*2 min*/;
     }
     if (!ConnNetInfo_GetValue(0, "MX_PORT", buf, sizeof(buf), 0)
         ||  !(port = atoi(buf))  ||  port > 65535) {
@@ -252,7 +252,6 @@ SSendMailInfo* SendMailInfo_InitEx(SSendMailInfo* info,
 {
     if (info) {
         x_Sendmail_InitEnv();
-        info->magic_number    = MX_MAGIC_NUMBER;
         info->cc              = 0;
         info->bcc             = 0;
         s_MakeFrom(info->from, sizeof(info->from), user);
@@ -262,6 +261,7 @@ SSendMailInfo* SendMailInfo_InitEx(SSendMailInfo* info,
         info->mx_port         = s_MxPort;
         info->mx_timeout      = s_MxTmo;
         info->mx_options      = 0;
+        info->magic_cookie    = MX_MAGIC_COOKIE;
     }
     return info;
 }
@@ -398,8 +398,8 @@ const char* CORE_SendMailEx(const char*          to,
     SOCK sock = 0;
 
     info = uinfo ? uinfo : SendMailInfo_Init(&ainfo);
-    if (info->magic_number != MX_MAGIC_NUMBER)
-        SENDMAIL_RETURN(6, "Invalid magic number");
+    if (info->magic_cookie != MX_MAGIC_COOKIE)
+        SENDMAIL_RETURN(6, "Invalid magic cookie");
 
     if ((!to         ||  !*to)        &&
         (!info->cc   ||  !*info->cc)  &&
