@@ -1,4 +1,4 @@
-/* $Id: ncbi_service.c 351828 2012-02-01 17:45:08Z lavr $
+/* $Id: ncbi_service.c 373975 2012-09-05 15:32:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,11 +41,21 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_Service
 
+
+/*
+ * FIXME FIXME FIXME FIXME FIXME ---->
+ *
+ * NOTE: For fSERV_ReverseDns lookups the following rules apply to "skip"
+ *       contents:  a service would not be selected if there is a same-named
+ *       entry with matching host[:port] is found in the "skip" list, or
+ *       there is a nameless...
+ * NOTE:  Lookup by mask cancels fSERV_ReverseDns mode, if both are used.
+ */
+
+
 #define CONN_SERVICE_NAME  DEF_CONN_REG_SECTION "_" REG_CONN_SERVICE_NAME
 
 
-/* Use a property that "&a" for an array is equivalent to just "a", but for
- * pointers it will increase indirection, so SizeOf() will always be 1. */
 #define SizeOf(arr)  (sizeof(arr) / sizeof((arr)[0]))
 
 
@@ -64,6 +74,12 @@ ESwitch SERV_DoFastOpens(ESwitch on)
 }
 
 
+void SERV_InitFirewallMode(void)
+{
+    memset(s_FWPorts, 0, sizeof(s_FWPorts));
+}
+
+
 int/*bool*/ SERV_AddFirewallPort(unsigned short port)
 {
     unsigned int n = port / (sizeof(s_FWPorts[0]) << 3);
@@ -89,7 +105,7 @@ int/*bool*/ SERV_IsFirewallPort(unsigned short port)
 
 
 static char* s_ServiceName(const char* service,
-                           int/*bool*/ ismask, size_t depth)
+                           int/*bool*/ ismask, unsigned int depth)
 {
     char   buf[128];
     char   srv[128];
@@ -99,8 +115,8 @@ static char* s_ServiceName(const char* service,
     if (depth > 7) {
         assert(service  &&  *service);
         CORE_LOGF_X(7, eLOG_Error,
-                    ("[%s]  Maximal service name recursion depth reached: %lu",
-                     service, (unsigned long) depth));
+                    ("[%s]  Maximal service name recursion depth reached: %u",
+                     service, depth));
         return 0/*failure*/;
     }
     len = 0;
@@ -129,7 +145,7 @@ static char* s_ServiceName(const char* service,
             s = srv;
         }
         if (*s  &&  strcasecmp(s, service) != 0)
-            return s_ServiceName(s, ismask, ++depth);
+            return s_ServiceName(s, ismask, depth + 1);
     }
     return strdup(service);
 }
@@ -141,9 +157,9 @@ char* SERV_ServiceName(const char* service)
 }
 
 
-static int/*bool*/ s_AddSkipInfo(SERV_ITER   iter,
-                                 const char* name,
-                                 SSERV_Info* info)
+static int/*bool*/ s_AddSkipInfo(SERV_ITER      iter,
+                                 const char*    name,
+                                 SSERV_InfoCPtr info)
 {
     size_t n;
     assert(name);
@@ -155,18 +171,18 @@ static int/*bool*/ s_AddSkipInfo(SERV_ITER   iter,
             /* Replace older version */
             if (iter->last == iter->skip[n])
                 iter->last  = info;
-            free(iter->skip[n]);
+            free((void*) iter->skip[n]);
             iter->skip[n] = info;
             return 1;
         }
     }
     if (iter->n_skip == iter->a_skip) {
-        SSERV_Info** temp;
+        SSERV_InfoCPtr* temp;
         n = iter->a_skip + 10;
-        temp = (SSERV_Info**)
+        temp = (SSERV_InfoCPtr*)
             (iter->skip
-             ? realloc(iter->skip, n * sizeof(*temp))
-             : malloc (            n * sizeof(*temp)));
+             ? realloc((void*) iter->skip, n * sizeof(*temp))
+             : malloc (                    n * sizeof(*temp)));
         if (!temp)
             return 0;
         iter->skip = temp;
@@ -190,20 +206,20 @@ static int/*bool*/ s_IsMapperConfigured(const char* service, const char* key)
 }
 
 
-static SERV_ITER s_Open(const char*          service,
-                        unsigned/*bool*/     ismask,
-                        TSERV_Type           types,
-                        unsigned int         preferred_host,
-                        unsigned short       preferred_port,
-                        double               preference,
-                        const SConnNetInfo*  net_info,
-                        const SSERV_InfoCPtr skip[],
-                        size_t               n_skip,
-                        unsigned/*bool*/     external,
-                        const char*          arg,
-                        const char*          val,
-                        SSERV_Info**         info,
-                        HOST_INFO*           host_info)
+static SERV_ITER s_Open(const char*         service,
+                        unsigned/*bool*/    ismask,
+                        TSERV_Type          types,
+                        unsigned int        preferred_host,
+                        unsigned short      preferred_port,
+                        double              preference,
+                        const SConnNetInfo* net_info,
+                        SSERV_InfoCPtr      skip[],
+                        size_t              n_skip,
+                        unsigned/*bool*/    external,
+                        const char*         arg,
+                        const char*         val,
+                        SSERV_Info**        info,
+                        HOST_INFO*          host_info)
 {
     int/*bool*/ do_lbsmd = -1/*unassigned*/, do_dispd = -1/*unassigned*/;
     const SSERV_VTable* op;
@@ -219,7 +235,6 @@ static SERV_ITER s_Open(const char*          service,
     assert(ismask  ||  *s);
 
     iter->name              = s;
-    iter->type              = types & fSERV_All;
     iter->host              = (preferred_host == SERV_LOCALHOST
                                ? SOCK_GetLocalHostAddress(eDefault)
                                : preferred_host);
@@ -229,6 +244,7 @@ static SERV_ITER s_Open(const char*          service,
                                :  0.01 * (preference > 100.0
                                           ? 100.0
                                           : preference));
+    iter->types             = types;
     if (ismask)
         iter->ismask        = 1;
     if (types & fSERV_IncludeDown)
@@ -275,7 +291,7 @@ static SERV_ITER s_Open(const char*          service,
 
     if (net_info) {
         if (net_info->firewall)
-            iter->type |= fSERV_Firewall;
+            iter->types |= fSERV_Firewall;
         if (net_info->stateless)
             iter->stateless = 1;
         if (net_info->lb_disable)
@@ -311,7 +327,7 @@ static SERV_ITER s_Open(const char*          service,
 SERV_ITER SERV_OpenSimple(const char* service)
 {
     SConnNetInfo* net_info = ConnNetInfo_Create(service);
-    SERV_ITER iter = SERV_Open(service, fSERV_Any, 0, net_info);
+    SERV_ITER iter = SERV_Open(service, fSERV_Any, SERV_ANYHOST, net_info);
     ConnNetInfo_Destroy(net_info);
     return iter;
 }
@@ -330,12 +346,12 @@ SERV_ITER SERV_Open(const char*         service,
 }
 
 
-SERV_ITER SERV_OpenEx(const char*          service,
-                      TSERV_Type           types,
-                      unsigned int         preferred_host,
-                      const SConnNetInfo*  net_info,
-                      const SSERV_InfoCPtr skip[],
-                      size_t               n_skip)
+SERV_ITER SERV_OpenEx(const char*         service,
+                      TSERV_Type          types,
+                      unsigned int        preferred_host,
+                      const SConnNetInfo* net_info,
+                      SSERV_InfoCPtr      skip[],
+                      size_t              n_skip)
 {
     return s_Open(service, 0/*not mask*/, types,
                   preferred_host, 0/*preferred_port*/, 0.0/*preference*/,
@@ -345,17 +361,17 @@ SERV_ITER SERV_OpenEx(const char*          service,
 }
 
 
-SERV_ITER SERV_OpenP(const char*          service,
-                     TSERV_Type           types,
-                     unsigned int         preferred_host,
-                     unsigned short       preferred_port,
-                     double               preference,
-                     const SConnNetInfo*  net_info,
-                     const SSERV_InfoCPtr skip[],
-                     size_t               n_skip,
-                     int/*bool*/          external,
-                     const char*          arg,
-                     const char*          val)
+SERV_ITER SERV_OpenP(const char*         service,
+                     TSERV_Type          types,
+                     unsigned int        preferred_host,
+                     unsigned short      preferred_port,
+                     double              preference,
+                     const SConnNetInfo* net_info,
+                     SSERV_InfoCPtr      skip[],
+                     size_t              n_skip,
+                     int/*bool*/         external,
+                     const char*         arg,
+                     const char*         val)
 {
     return s_Open(service,
                   service  &&  (!*service  ||  strpbrk(service, "?*")), types,
@@ -373,18 +389,19 @@ static void s_SkipSkip(SERV_ITER iter)
         return;
     n = 0;
     while (n < iter->n_skip) {
-        SSERV_Info* temp = iter->skip[n];
+        SSERV_InfoCPtr temp = iter->skip[n];
         if (temp->time != NCBI_TIME_INFINITE
             &&  (!iter->time/*iterator reset*/
                  ||  ((temp->type != fSERV_Dns  ||  temp->host)
                       &&  temp->time < iter->time))) {
-            if (n < --iter->n_skip) {
-                memmove(iter->skip + n, iter->skip + n + 1,
-                        sizeof(*iter->skip)*(iter->n_skip - n));
+            if (--iter->n_skip > n) {
+                SSERV_InfoCPtr* ptr = iter->skip + n;
+                memmove((void*) ptr, (void*)(ptr + 1),
+                        (iter->n_skip - n) * sizeof(*ptr));
             }
             if (iter->last == temp)
                 iter->last  = 0;
-            free(temp);
+            free((void*) temp);
         } else
             n++;
     }
@@ -426,18 +443,18 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER   iter,
 }
 
 
-static SSERV_Info* s_GetInfo(const char*          service,
-                             TSERV_Type           types,
-                             unsigned int         preferred_host,
-                             unsigned short       preferred_port,
-                             double               preference,
-                             const SConnNetInfo*  net_info,
-                             const SSERV_InfoCPtr skip[],
-                             size_t               n_skip,
-                             int /*bool*/         external,
-                             const char*          arg,
-                             const char*          val,
-                             HOST_INFO*           host_info)
+static SSERV_Info* s_GetInfo(const char*         service,
+                             TSERV_Type          types,
+                             unsigned int        preferred_host,
+                             unsigned short      preferred_port,
+                             double              preference,
+                             const SConnNetInfo* net_info,
+                             SSERV_InfoCPtr      skip[],
+                             size_t              n_skip,
+                             int /*bool*/        external,
+                             const char*         arg,
+                             const char*         val,
+                             HOST_INFO*          host_info)
 {
     SSERV_Info* info = 0;
     SERV_ITER iter = s_Open(service, 0/*not mask*/, types,
@@ -445,7 +462,8 @@ static SSERV_Info* s_GetInfo(const char*          service,
                             net_info, skip, n_skip,
                             external, arg, val,
                             &info, host_info);
-    if (iter  &&  iter->op  &&  !info) {
+    assert(!iter  ||  iter->op);
+    if (iter  &&  !info) {
         /* All LOCAL/DISPD searches end up here, but none LBSMD ones */
         info = s_GetNextInfo(iter, host_info, 1/*internal*/);
     }
@@ -467,13 +485,13 @@ SSERV_Info* SERV_GetInfo(const char*         service,
 }
 
 
-SSERV_Info* SERV_GetInfoEx(const char*          service,
-                           TSERV_Type           types,
-                           unsigned int         preferred_host,
-                           const SConnNetInfo*  net_info,
-                           const SSERV_InfoCPtr skip[],
-                           size_t               n_skip,
-                           HOST_INFO*           host_info)
+SSERV_Info* SERV_GetInfoEx(const char*         service,
+                           TSERV_Type          types,
+                           unsigned int        preferred_host,
+                           const SConnNetInfo* net_info,
+                           SSERV_InfoCPtr      skip[],
+                           size_t              n_skip,
+                           HOST_INFO*          host_info)
 {
     return s_GetInfo(service, types,
                      preferred_host, 0/*preferred_host*/, 0.0/*preference*/,
@@ -483,18 +501,18 @@ SSERV_Info* SERV_GetInfoEx(const char*          service,
 }
 
 
-SSERV_Info* SERV_GetInfoP(const char*          service,
-                          TSERV_Type           types,
-                          unsigned int         preferred_host,
-                          unsigned short       preferred_port,
-                          double               preference,
-                          const SConnNetInfo*  net_info,
-                          const SSERV_InfoCPtr skip[],
-                          size_t               n_skip,
-                          int/*bool*/          external, 
-                          const char*          arg,
-                          const char*          val,
-                          HOST_INFO*           host_info)
+SSERV_Info* SERV_GetInfoP(const char*         service,
+                          TSERV_Type          types,
+                          unsigned int        preferred_host,
+                          unsigned short      preferred_port,
+                          double              preference,
+                          const SConnNetInfo* net_info,
+                          SSERV_InfoCPtr      skip[],
+                          size_t              n_skip,
+                          int/*bool*/         external, 
+                          const char*         arg,
+                          const char*         val,
+                          HOST_INFO*          host_info)
 {
     return s_GetInfo(service, types,
                      preferred_host, preferred_port, preference,
@@ -504,22 +522,25 @@ SSERV_Info* SERV_GetInfoP(const char*          service,
 }
 
 
-const SSERV_Info* SERV_GetNextInfoEx(SERV_ITER  iter,
-                                     HOST_INFO* host_info)
+SSERV_InfoCPtr SERV_GetNextInfoEx(SERV_ITER  iter,
+                                  HOST_INFO* host_info)
 {
-    return iter  &&  iter->op ? s_GetNextInfo(iter, host_info, 0) : 0;
+    assert(!iter  ||  iter->op);
+    return iter ? s_GetNextInfo(iter, host_info, 0) : 0;
 }
 
 
-const SSERV_Info* SERV_GetNextInfo(SERV_ITER iter)
+SSERV_InfoCPtr SERV_GetNextInfo(SERV_ITER iter)
 {
-    return iter  &&  iter->op ? s_GetNextInfo(iter, 0, 0) : 0;
+    assert(!iter  ||  iter->op);
+    return iter ? s_GetNextInfo(iter, 0,         0) : 0;
 }
 
 
 const char* SERV_MapperName(SERV_ITER iter)
 {
-    return iter  &&  iter->op ? iter->op->name : 0;
+    assert(!iter  ||  iter->op);
+    return iter ? iter->op->mapper : 0;
 }
 
 
@@ -532,7 +553,8 @@ const char* SERV_CurrentName(SERV_ITER iter)
 
 int/*bool*/ SERV_PenalizeEx(SERV_ITER iter, double fine, TNCBI_Time time)
 {
-    if (!iter  ||  !iter->op  ||  !iter->op->Feedback  ||  !iter->last)
+    assert(!iter  ||  iter->op);
+    if (!iter  ||  !iter->op->Feedback  ||  !iter->last)
         return 0/*false*/;
     return iter->op->Feedback(iter, fine, time ? time : 1/*NB: always != 0*/);
 }
@@ -546,7 +568,8 @@ int/*bool*/ SERV_Penalize(SERV_ITER iter, double fine)
 
 int/*bool*/ SERV_Rerate(SERV_ITER iter, double rate)
 {
-    if (!iter  ||  !iter->op  ||  !iter->op->Feedback  ||  !iter->last)
+    assert(!iter  ||  iter->op);
+    if (!iter  ||  !iter->op->Feedback  ||  !iter->last)
         return 0/*false*/;
     return iter->op->Feedback(iter, rate, 0/*i.e.rate*/);
 }
@@ -571,7 +594,7 @@ void SERV_Close(SERV_ITER iter)
         return;
     SERV_Reset(iter);
     for (i = 0;  i < iter->n_skip;  i++)
-        free(iter->skip[i]);
+        free((void*) iter->skip[i]);
     iter->n_skip = 0;
     if (iter->op) {
         if (iter->op->Close)
@@ -579,7 +602,7 @@ void SERV_Close(SERV_ITER iter)
         iter->op = 0;
     }
     if (iter->skip)
-        free(iter->skip);
+        free((void*) iter->skip);
     free((void*) iter->name);
     free(iter);
 }
@@ -590,7 +613,8 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text, int code)
     static const char used_server_info[] = "Used-Server-Info-";
     int retval = 0/*not updated yet*/;
 
-    if (iter  &&  iter->op  &&  text) {
+    assert(!iter  ||  iter->op);
+    if (iter  &&  text) {
         const char *c, *b;
         iter->time = (TNCBI_Time) time(0);
         for (b = text;  (c = strchr(b, '\n')) != 0;  b = c + 1) {
@@ -604,9 +628,9 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text, int code)
                 continue;
             memcpy(t, b, len);
             if (t[len - 1] == '\r')
-                t[len - 1] =  '\0';
+                t[len - 1]  = '\0';
             else
-                t[len] = '\0';
+                t[len    ]  = '\0';
             p = t;
             if (iter->op->Update  &&  iter->op->Update(iter, p, code))
                 retval = 1/*updated*/;
@@ -671,9 +695,9 @@ static void s_SetDefaultReferer(SERV_ITER iter, SConnNetInfo* net_info)
 {
     char* str, *referer = 0;
 
-    if (strcasecmp(iter->op->name, "DISPD") == 0)
+    if (strcasecmp(iter->op->mapper, "DISPD") == 0)
         referer = ConnNetInfo_URL(net_info);
-    else if ((str = strdup(iter->op->name)) != 0) {
+    else if ((str = strdup(iter->op->mapper)) != 0) {
         const char* host = net_info->client_host;
         const char* args = net_info->args;
         const char* name = iter->name;
@@ -714,8 +738,8 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
     static const char kSkipInfo[] = "Skip-Info-%u: ";
     static const char kAffinity[] = "Affinity: ";
     char buffer[128], *str;
+    TBSERV_TypeOnly t;
     size_t buflen, i;
-    TSERV_Type t;
     BUF buf = 0;
 
     /* Put client version number */
@@ -727,13 +751,14 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
         return 0;
     }
     if (iter) {
-        if (net_info && !net_info->http_referer && iter->op && iter->op->name)
+        assert(iter->op);
+        if (net_info  &&  !net_info->http_referer  &&  iter->op->mapper)
             s_SetDefaultReferer(iter, net_info);
         /* Accepted server types */
         buflen = sizeof(kAcceptedServerTypes) - 1;
         memcpy(buffer, kAcceptedServerTypes, buflen);
         for (t = 1;  t;  t <<= 1) {
-            if (iter->type & t) {
+            if (iter->types & t) {
                 const char* name = SERV_TypeStr((ESERV_Type) t);
                 size_t namelen = strlen(name);
                 if (!namelen  ||  buflen + 1 + namelen + 2 >= sizeof(buffer))
@@ -762,7 +787,7 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
                 return 0;
             }
         }
-        if (iter->type & fSERV_Firewall) {
+        if (iter->types & fSERV_Firewall) {
             /* Firewall */
             s_PrintFirewallPorts(buffer, sizeof(buffer), net_info);
             if (*buffer
@@ -775,9 +800,12 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
         }
         if (iter->pref  &&  (iter->host | iter->port)) {
             /* Preference */
-            buflen  = SOCK_HostPortToString(iter->host, iter->port,
-                                            buffer, sizeof(buffer));
-            buflen += sprintf(buffer + buflen, " %.2lf%%\r\n", iter->pref*1e2);
+            buflen = SOCK_HostPortToString(iter->host, iter->port,
+                                           buffer, sizeof(buffer));
+            buffer[buflen++] = ' ';
+            buflen = (size_t)(strcpy(NCBI_simple_ftoa(buffer + buflen,
+                                                      iter->pref * 100.0, 2),
+                                     "\r\n") - buffer) + 2/*"\r\n"*/;
             if (!BUF_Write(&buf, kPreference, sizeof(kPreference) - 1)  ||
                 !BUF_Write(&buf, buffer, buflen)) {
                 BUF_Destroy(buf);
@@ -842,29 +870,6 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
 }
 
 
-/*
- * Note parameters' ranges here:
- * 0.0 <= pref <= 1.0
- * 0.0 <  gap  <= 1.0
- * n >= 2
- * Hence, the formula below always yields in a value from the range [0..1].
- */
-double SERV_Preference(double pref, double gap, size_t n)
-{
-    double spread;
-    assert(0.0 <= pref && pref <= 1.0);
-    assert(0.0 <  gap  && gap  <= 1.0);
-    assert(n >= 2);
-    if (gap >= pref)
-        return gap;
-    spread = 14.0/(n + 12.0);
-    if (gap >= spread/((double) n))
-        return pref;
-    else
-        return 2.0/spread*gap*pref;
-}
-
-
 unsigned short SERV_ServerPort(const char*  name,
                                unsigned int host)
 {
