@@ -1,4 +1,4 @@
-/* $Id: ncbi_service_connector.c 362486 2012-05-09 17:25:02Z rafanovi $
+/* $Id: ncbi_service_connector.c 373973 2012-09-05 15:32:08Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,11 +42,8 @@
 #define NCBI_USE_ERRCODE_X   Connect_Service
 
 
-typedef unsigned short TBSERV_Type;
-
-
 typedef struct SServiceConnectorTag {
-    const char*    name;                /* Verbal connector type             */
+    const char*    type;                /* Verbal connector type             */
     const char*    descr;               /* Verbal connector description      */
     SConnNetInfo*  net_info;            /* Connection information            */
     const char*    user_header;         /* User header currently set         */
@@ -55,8 +52,8 @@ typedef struct SServiceConnectorTag {
     unsigned int   host;                /* Parsed connection info... (n.b.o) */
     unsigned short port;                /*                       ... (h.b.o) */
     unsigned short retry;               /* Open retry count since last okay  */
-    TBSERV_Type    types;               /* Server types, abridged to 16 bits */
-    unsigned char  reset;               /* Non-zero if iter was just reset   */
+    TSERV_TypeOnly types;               /* Server types w/o any specials     */
+    unsigned       reset:1;             /* Non-zero if iter was just reset   */
     ticket_t       ticket;              /* Network byte order (none if zero) */
     EIO_Status     status;              /* Status of last op                 */
     SSERVICE_Extra params;              /* Extra params as passed to ctor    */
@@ -88,7 +85,10 @@ extern "C" {
 
 static int/*bool*/ s_OpenDispatcher(SServiceConnector* uuu)
 {
-    if (!(uuu->iter = SERV_Open(uuu->service, uuu->types,
+    TSERV_Type types = uuu->types;
+    if (uuu->net_info->stateless)
+        types |= fSERV_Stateless;
+    if (!(uuu->iter = SERV_Open(uuu->service, types,
                                 SERV_LOCALHOST, uuu->net_info))) {
         CORE_LOGF_X(5, eLOG_Error,
                     ("[%s]  Service not found", uuu->service));
@@ -290,7 +290,7 @@ static const char* s_AdjustNetParams(const char*    service,
                                      EMIME_Type     mime_t,
                                      EMIME_SubType  mime_s,
                                      EMIME_Encoding mime_e,
-                                     char*          dynamic_header/*freed!*/)
+                                     const char*    extend_header)
 {
     const char *retval = 0;
 
@@ -304,164 +304,61 @@ static const char* s_AdjustNetParams(const char*    service,
     ConnNetInfo_DeleteAllArgs(net_info, cgi_args);
 
     if (ConnNetInfo_PrependArg(net_info, cgi_args, 0)) {
-        size_t sh_len = static_header  ? strlen(static_header)  : 0;
-        size_t dh_len = dynamic_header ? strlen(dynamic_header) : 0;
+        size_t sh_len = static_header ? strlen(static_header) : 0;
+        size_t eh_len = extend_header ? strlen(extend_header) : 0;
         char   c_t[MAX_CONTENT_TYPE_LEN];
-        size_t ct_len, len;
+        size_t len;
 
         if (s_IsContentTypeDefined(service, net_info, mime_t, mime_s, mime_e)
             ||  !MIME_ComposeContentTypeEx(mime_t, mime_s, mime_e,
                                            c_t, sizeof(c_t))) {
-            c_t[0] = '\0';
-            ct_len = 0;
+            *c_t = '\0';
+            len = 0;
         } else
-            ct_len = strlen(c_t);
-        if ((len = sh_len + dh_len + ct_len) != 0) {
-            char* temp = (char*) malloc(len + 1/*EOL*/);
+            len = strlen(c_t);
+        if ((len += sh_len + eh_len) != 0) {
+            char* temp = (char*) malloc(++len/*w/EOL*/);
             if (temp) {
-                strcpy(temp,          static_header  ? static_header  : "");
-                strcpy(temp + sh_len, dynamic_header ? dynamic_header : "");
-                strcpy(temp + sh_len + dh_len, c_t);
                 retval = temp;
+                memcpy(temp, static_header, sh_len);
+                temp += sh_len;
+                memcpy(temp, extend_header, eh_len);
+                temp += eh_len;
+                strcpy(temp, c_t);
             }
         } else
             retval = "";
     }
 
-    if (dynamic_header)
-        free(dynamic_header);
     return retval;
 }
 
 
-static const SSERV_Info* s_GetNextInfo(SServiceConnector* uuu)
+static SSERV_InfoCPtr s_GetNextInfo(SServiceConnector* uuu, int/*bool*/ http)
 {
-    int/*bool*/ reset = 0/*false*/;
-    SSERV_InfoCPtr info;
     for (;;) {
-        info = uuu->params.get_next_info
+        SSERV_InfoCPtr info = uuu->params.get_next_info
             ? (*uuu->params.get_next_info)(uuu->params.data, uuu->iter)
             : SERV_GetNextInfo(uuu->iter);
-        if (info  ||  reset)
+        if (info) {
+            if (http) {
+                /* Skip any 'stateful_capable' or unconnectable entries here,
+                 * which might have been left behind by either a failed
+                 * stateful dispatching with a fallback to stateless HTTP mode
+                 * or a too relaxed server type selection.
+                 */
+                if (info->sful  ||  info->type == fSERV_Dns)
+                    continue;
+            }
+            uuu->reset = 0/*false*/;
+            return info;
+        }
+        if (uuu->reset)
             break;
         SERV_Reset(uuu->iter);
-        reset = 1/*true*/;
+        uuu->reset = 1/*true*/;
     }
-    return info;
-}
-
-
-/* Although all additional HTTP tags that comprise the dispatching have default
- * values, which in most cases are fine with us, we will use these tags
- * explicitly to distinguish the calls originated within the service connector
- * from other calls (e.g. by Web browsers), and let the dispatcher to decide
- * whether to use more expensive dispatching (involving loopback connections)
- * in the latter case.
- */
-
-#ifdef __cplusplus
-extern "C" {
-    static int s_Adjust(SConnNetInfo*, void*, unsigned int);
-}
-#endif /*__cplusplus*/
-
-/*ARGSUSED*/
-/* This callback is only for services called via direct HTTP */
-static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
-                            void*         data,
-                            unsigned int  n)
-{
-    SServiceConnector* uuu = (SServiceConnector*) data;
-    const char*        user_header;
-    const SSERV_Info*  info;
-
-    assert(n != 0); /* paranoid assertion :-) */
-    if (uuu->net_info->firewall  &&  !uuu->net_info->stateless)
-        return 0; /*cannot adjust firewall stateful client*/
-
-    for (;;) {
-        if (!(info = s_GetNextInfo(uuu)))
-            return 0/*false - not adjusted*/;
-        /* Skip any 'stateful_capable' or unconnectable entries here, which
-         * might have been left behind by either a failed stateful dispatching
-         * with a fallback to stateless HTTP mode or a too relaxed server type
-         * selection */
-        if (!info->sful  &&  info->type != fSERV_Dns)
-            break;
-    }
-
-    {{
-        char* iter_header = SERV_Print(uuu->iter, 0, 0);
-        switch (info->type) {
-        case fSERV_Ncbid:
-            user_header = "Connection-Mode: STATELESS\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->service, net_info,
-                                            eReqMethod_Post,
-                                            NCBID_WEBPATH,
-                                            SERV_NCBID_ARGS(&info->u.ncbid),
-                                            uuu->net_info->args,
-                                            user_header, info->mime_t,
-                                            info->mime_s, info->mime_e,
-                                            iter_header);
-            break;
-        case fSERV_Http:
-        case fSERV_HttpGet:
-        case fSERV_HttpPost:
-            user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->service, net_info,
-                                            info->type == fSERV_HttpPost
-                                            ?  eReqMethod_Post
-                                            : (info->type == fSERV_HttpGet
-                                               ? eReqMethod_Get
-                                               : eReqMethod_Any),
-                                            SERV_HTTP_PATH(&info->u.http),
-                                            SERV_HTTP_ARGS(&info->u.http),
-                                            uuu->net_info->args,
-                                            user_header, info->mime_t,
-                                            info->mime_s, info->mime_e,
-                                            iter_header);
-            break;
-        case fSERV_Firewall:
-        case fSERV_Standalone:
-            user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->service, net_info,
-                                            eReqMethod_Post,
-                                            uuu->net_info->path, 0,
-                                            uuu->net_info->args,
-                                            user_header, info->mime_t,
-                                            info->mime_s, info->mime_e,
-                                            iter_header);
-            break;
-        default:
-            if (iter_header)
-                free(iter_header);
-            user_header = 0;
-            break;
-        }
-    }}
-    if (!user_header)
-        return 0/*false - not adjusted*/;
-
-    if (uuu->user_header) {
-        assert(*uuu->user_header);
-        ConnNetInfo_DeleteUserHeader(net_info, uuu->user_header);
-        free((void*) uuu->user_header);
-    }
-    if (*user_header) {
-        uuu->user_header = user_header;
-        if (!ConnNetInfo_OverrideUserHeader(net_info, user_header))
-            return 0/*false - not adjusted*/;
-    } else
-        uuu->user_header = 0;
-
-    if (info->type == fSERV_Ncbid  ||  (info->type & fSERV_Http)) {
-        SOCK_ntoa(info->host, net_info->host, sizeof(net_info->host));
-        net_info->port = info->port;
-    } else {
-        strcpy(net_info->host, uuu->net_info->host);
-        net_info->port = uuu->net_info->port;
-    }
-    return 1/*true - adjusted*/;
+    return 0;
 }
 
 
@@ -488,11 +385,11 @@ static char* s_HostPort(const char* host, unsigned short nport)
  * tunnel, but for buggy proxies that ignore HTTP body as connection data
  * (and thus violate the standard), this shortcut could not be utilized;
  * so the longer multi-step sequence was introduced below, instead.
- * Cf. ncbi_conn_stream.cpp: s_TunneledSocketConnector().
+ * Cf. ncbi_conn_stream.cpp: s_SocketConnectorBuilder().
  */
-static CONNECTOR s_CreateSocketConnector(const SConnNetInfo* net_info,
-                                         const void*         init_data,
-                                         size_t              init_size)
+static CONNECTOR s_SocketConnectorBuilder(const SConnNetInfo* net_info,
+                                          const void*         data,
+                                          size_t              size)
 {
     CONNECTOR   c;
     EIO_Status  status;
@@ -504,26 +401,13 @@ static CONNECTOR s_CreateSocketConnector(const SConnNetInfo* net_info,
     if (*net_info->http_proxy_host  &&  net_info->http_proxy_port) {
         status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
         assert(!sock ^ !(status != eIO_Success));
-        if (status == eIO_Success  &&  init_size) {
-            size_t size   = SOCK_OSHandleSize();
-            void*  handle = malloc(size);
-            status = SOCK_GetOSHandleEx(sock, handle, size, eTakeOwnership);
-            if (status == eIO_Success) {
-                SOCK_Close(sock);
-                status  = SOCK_CreateOnTopEx(handle, size, &sock,
-                                             init_data, init_size, flags);
-                if (status != eIO_Success) {
-                    SOCK_CloseOSHandle(handle, size);
-                    assert(!sock);
-                } else
-                    assert(sock);
-            } else {
-                SOCK_Abort(sock);
-                SOCK_Close(sock);
-                sock = 0;
-            }
-            if (handle)
-                free(handle);
+        if (status == eIO_Success  &&  size) {
+            SOCK s;
+            status = SOCK_CreateOnTopEx(sock, 0, &s,
+                                        data, size, flags);
+            assert(!s ^ !(status != eIO_Success));
+            SOCK_Destroy(sock);
+            sock = s;
         }
         proxy = 1/*true*/;
     }
@@ -531,9 +415,9 @@ static CONNECTOR s_CreateSocketConnector(const SConnNetInfo* net_info,
         const char* host = (net_info->firewall  &&  *net_info->proxy_host
                             ? net_info->proxy_host : net_info->host);
         if (!proxy  &&  net_info->debug_printout)
-            ConnNetInfo_LogEx(net_info, eLOG_Note, CORE_GetLOG());
+            ConnNetInfo_Log(net_info, eLOG_Note, CORE_GetLOG());
         status = SOCK_CreateEx(host, net_info->port, net_info->timeout, &sock,
-                               init_data, init_size, flags);
+                               data, size, flags);
         assert(!sock ^ !(status != eIO_Success));
     }
     hostport = s_HostPort(net_info->host, net_info->port);
@@ -547,9 +431,118 @@ static CONNECTOR s_CreateSocketConnector(const SConnNetInfo* net_info,
 }
 
 
+/* Although all additional HTTP tags that comprise the dispatching have their
+ * default values, which in most cases are fine with us, we will use these tags
+ * explicitly to distinguish the calls originated within the service connector
+ * from other calls (e.g. by Web browsers), and let the dispatcher decide
+ * whether to use more expensive dispatching (involving loopback connections)
+ * in the latter case.
+ */
+
+
+#ifdef __cplusplus
+extern "C" {
+    static int s_Adjust(SConnNetInfo*, void*, unsigned int);
+}
+#endif /*__cplusplus*/
+
+/*ARGSUSED*/
+/* NB: This callback is only for services called via direct HTTP */
+static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
+                            void*         data,
+                            unsigned int  n)
+{
+    SServiceConnector* uuu = (SServiceConnector*) data;
+    const char*        user_header;
+    char*              iter_header;
+    SSERV_InfoCPtr     info;
+
+    assert(n > 0  &&  (!net_info->firewall  ||  net_info->stateless));
+
+    if (uuu->retry >= uuu->net_info->max_try)
+        return 0/*false - too many errors*/;
+    uuu->retry++;
+
+    if (!(info = s_GetNextInfo(uuu, 1/*http*/)))
+        return 0/*false - not adjusted*/;
+
+    iter_header = SERV_Print(uuu->iter, 0, 0);
+    switch (info->type) {
+    case fSERV_Ncbid:
+        user_header = "Connection-Mode: STATELESS\r\n"; /*default*/
+        user_header = s_AdjustNetParams(uuu->service, net_info,
+                                        eReqMethod_Post,
+                                        NCBID_WEBPATH,
+                                        SERV_NCBID_ARGS(&info->u.ncbid),
+                                        uuu->net_info->args,
+                                        user_header, info->mime_t,
+                                        info->mime_s, info->mime_e,
+                                        iter_header);
+        break;
+    case fSERV_Http:
+    case fSERV_HttpGet:
+    case fSERV_HttpPost:
+        user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
+        user_header = s_AdjustNetParams(uuu->service, net_info,
+                                        info->type == fSERV_HttpPost
+                                        ?  eReqMethod_Post
+                                        : (info->type == fSERV_HttpGet
+                                           ? eReqMethod_Get
+                                           : eReqMethod_Any),
+                                        SERV_HTTP_PATH(&info->u.http),
+                                        SERV_HTTP_ARGS(&info->u.http),
+                                        uuu->net_info->args,
+                                        user_header, info->mime_t,
+                                        info->mime_s, info->mime_e,
+                                        iter_header);
+        break;
+    case fSERV_Firewall:
+    case fSERV_Standalone:
+        user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
+        user_header = s_AdjustNetParams(uuu->service, net_info,
+                                        eReqMethod_Post,
+                                        uuu->net_info->path, 0,
+                                        uuu->net_info->args,
+                                        user_header, info->mime_t,
+                                        info->mime_s, info->mime_e,
+                                        iter_header);
+        break;
+    default:
+        user_header = 0;
+        assert(0);
+        break;
+    }
+    if (iter_header)
+        free(iter_header);
+    if (!user_header)
+        return 0/*false - not adjusted*/;
+
+    if (uuu->user_header) {
+        assert(*uuu->user_header);
+        ConnNetInfo_DeleteUserHeader(net_info, uuu->user_header);
+        free((void*) uuu->user_header);
+    }
+    if (*user_header) {
+        uuu->user_header = user_header;
+        if (!ConnNetInfo_OverrideUserHeader(net_info, user_header))
+            return 0/*false - not adjusted*/;
+    } else
+        uuu->user_header = 0;
+
+    if (info->type == fSERV_Ncbid  ||  (info->type & fSERV_Http)) {
+        SOCK_ntoa(info->host, net_info->host, sizeof(net_info->host));
+        net_info->port = info->port;
+    } else {
+        strcpy(net_info->host, uuu->net_info->host);
+        net_info->port = uuu->net_info->port;
+    }
+    return 1/*true - adjusted*/;
+}
+
+
 static CONNECTOR s_Open(SServiceConnector* uuu,
                         const STimeout*    timeout,
-                        const SSERV_Info*  info,
+                        SSERV_InfoCPtr     info,
                         SConnNetInfo*      net_info)
 {
     int/*bool*/ but_last = 0/*false*/;
@@ -599,7 +592,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
             break;
         case fSERV_Standalone:
             if (!net_info->stateless)
-                return s_CreateSocketConnector(net_info, 0, 0);
+                return s_SocketConnectorBuilder(net_info, 0, 0);
             /* Otherwise, it will be a pass-thru connection via dispatcher */
             user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
             user_header = s_AdjustNetParams(uuu->service, net_info,
@@ -610,6 +603,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
             break;
         default:
             user_header = 0;
+            assert(0);
             break;
         }
     } else {
@@ -656,11 +650,11 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         if ((uh_len = strlen(user_header)) > 0) {
             char*  ih;
             size_t ih_len = strlen(iter_header);
-            if ((ih = (char*) realloc(iter_header, ih_len + uh_len + 1)) != 0){
-                strcpy(ih + ih_len, user_header);
+            if ((ih = (char*) realloc(iter_header, ++uh_len + ih_len)) != 0){
+                memcpy(ih + ih_len, user_header, uh_len);
                 iter_header = ih;
             }
-            free((char*) user_header);
+            free((void*) user_header);
         }
         user_header = iter_header;
     } else if (!*user_header)
@@ -676,7 +670,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
 
     ConnNetInfo_ExtendUserHeader
         (net_info, "User-Agent: NCBIServiceConnector/"
-         DISP_PROTOCOL_VERSION
+         HTTP_DISP_VERSION
 #ifdef NCBI_CXX_TOOLKIT
          " (CXX Toolkit)"
 #else
@@ -700,7 +694,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         c = HTTP_CreateConnectorEx(net_info, fHTTP_Flushable,
                                    s_ParseHeaderNoUCB, uuu/*user_data*/,
                                    0/*adjust*/, 0/*cleanup*/);
-        /* Wait for connection info back (error-transparent by DISPD.CGI) */
+        /* Wait for connection info back from dispatcher */
         if (c  &&  (status = CONN_Create(c, &conn)) == eIO_Success) {
             CONN_SetTimeout(conn, eIO_Open,      timeout);
             CONN_SetTimeout(conn, eIO_ReadWrite, timeout);
@@ -719,11 +713,10 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         if (!uuu->host)
             return 0/*failed, no connection info returned*/;
         if (uuu->host == (unsigned int)(-1)) {
-            /* Firewall mode only in stateful mode, fallback requested */
             assert(!info  ||  info->type == fSERV_Firewall);
             assert(!net_info->stateless);
             net_info->stateless = 1/*true*/;
-            /* Try to use stateless mode instead */
+            /* Fallback to try to use stateless mode instead */
             return s_Open(uuu, timeout, info, net_info);
         }
         SOCK_ntoa(uuu->host, net_info->host, sizeof(net_info->host));
@@ -734,10 +727,12 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
                         ("[%s]  Fallback port :%hu is not in the set",
                          uuu->service, uuu->port));
         }
-        net_info->port = uuu->port;
+        net_info->port    = uuu->port;
+        net_info->path[0] = '\0';
+        net_info->args[0] = '\0';
         ConnNetInfo_DeleteUserHeader(net_info, uuu->user_header);
-        return s_CreateSocketConnector(net_info, &uuu->ticket,
-                                       uuu->ticket ? sizeof(uuu->ticket) : 0);
+        return s_SocketConnectorBuilder(net_info, &uuu->ticket,
+                                        uuu->ticket ? sizeof(uuu->ticket) : 0);
     }
     return HTTP_CreateConnectorEx(net_info,
                                   (uuu->params.flags
@@ -754,9 +749,9 @@ static void s_Close(CONNECTOR       connector,
 {
     SServiceConnector* uuu = (SServiceConnector*) connector->handle;
 
-    if (uuu->name) {
-        free((void*) uuu->name);
-        uuu->name = 0;
+    if (uuu->type) {
+        free((void*) uuu->type);
+        uuu->type = 0;
     }
     if (uuu->descr) {
         free((void*) uuu->descr);
@@ -785,7 +780,7 @@ static void s_Close(CONNECTOR       connector,
 static const char* s_VT_GetType(CONNECTOR connector)
 {
     SServiceConnector* uuu = (SServiceConnector*) connector->handle;
-    return uuu->name ? uuu->name : uuu->service;
+    return uuu->type ? uuu->type : uuu->service;
 }
 
 
@@ -799,26 +794,24 @@ static char* s_VT_Descr(CONNECTOR connector)
 static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
 {
     SServiceConnector* uuu = (SServiceConnector*) connector->handle;
-    unsigned short max_try = uuu->net_info->max_try;
     SMetaConnector* meta = connector->meta;
     EIO_Status status = eIO_Closed;
 
-    if (max_try == 0)
-        max_try  = 1;
-    for (uuu->retry = 0;  uuu->retry < max_try;  uuu->retry++) {
-        const SSERV_Info* info;
+    for (uuu->retry = 0;  uuu->retry < uuu->net_info->max_try;  uuu->retry++) {
         SConnNetInfo* net_info;
+        SSERV_InfoCPtr info;
         CONNECTOR conn;
         int stateless;
 
-        assert(!uuu->meta.list  &&  !uuu->name  &&  !uuu->descr);
+        assert(!uuu->meta.list  &&  !uuu->type  &&  !uuu->descr);
 
         if (!uuu->iter  &&  !s_OpenDispatcher(uuu))
             break;
 
-        if (uuu->net_info->firewall  &&  strcasecmp(uuu->iter->name, "local"))
+        if (uuu->net_info->firewall
+            &&  strcasecmp(uuu->iter->op->mapper, "local") != 0) {
             info = 0;
-        else if (!(info = s_GetNextInfo(uuu)))
+        } else if (!(info = s_GetNextInfo(uuu, 0/*any*/)))
             break;
 
         if (!(net_info = ConnNetInfo_Clone(uuu->net_info))) {
@@ -853,18 +846,17 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
         CONN_SET_METHOD    (meta, read,   uuu->meta.read,  uuu->meta.c_read);
         CONN_SET_METHOD    (meta, status, uuu->meta.status,uuu->meta.c_status);
         if (uuu->meta.get_type) {
-            const char* type;
-            if ((type = uuu->meta.get_type(uuu->meta.c_get_type)) != 0) {
+            const char* temp;
+            if ((temp = uuu->meta.get_type(uuu->meta.c_get_type)) != 0) {
                 size_t slen = strlen(uuu->service);
-                size_t tlen = strlen(type);
-                char*  name = (char*) malloc(slen + tlen + 2);
-                if (name) {
-                    memcpy(name,        uuu->service, slen);
-                    name[slen++] = '/';
-                    memcpy(name + slen, type,         tlen);
-                    tlen += slen;
-                    name[tlen]   = '\0';
-                    uuu->name = name;
+                size_t tlen = strlen(temp);
+                char*  type = (char*) malloc(slen + tlen + 2);
+                if (type) {
+                    memcpy(type,        uuu->service, slen);
+                    type[slen++]      = '/';
+                    memcpy(type + slen, temp,         tlen);
+                    type[slen + tlen] = '\0';
+                    uuu->type = type;
                 }
             }
         }
@@ -939,7 +931,7 @@ static void s_Destroy(CONNECTOR connector)
         uuu->params.cleanup(uuu->params.data);
     s_CloseDispatcher(uuu);
     ConnNetInfo_Destroy(uuu->net_info);
-    assert(!uuu->name);
+    assert(!uuu->type);
     assert(!uuu->descr);
     free(uuu);
     free(connector);
@@ -1001,6 +993,8 @@ extern CONNECTOR SERVICE_CreateConnectorEx
         xxx->net_info->stateless = 1/*true*/;
     if ((types & fSERV_Firewall)  &&  !xxx->net_info->firewall)
         xxx->net_info->firewall = eFWMode_Adaptive;
+    if (xxx->net_info->max_try < 1)
+        xxx->net_info->max_try = 1;
     if (!s_OpenDispatcher(xxx)) {
         s_Destroy(ccc);
         return 0;
