@@ -1,4 +1,4 @@
-/* $Id: server.cpp 364729 2012-05-30 15:10:17Z ivanovp $
+/* $Id: server.cpp 388046 2013-02-04 21:53:37Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -171,6 +171,21 @@ CThreadInPool_ForServer::CAutoUnregGuard::~CAutoUnregGuard(void)
     m_Thread->x_UnregisterThread();
 }
 
+void
+CThreadInPool_ForServer::CountSelf(void)
+{
+    _ASSERT( !m_Counted );
+    m_Pool->m_ThreadCount.Add(1);
+    m_Counted = true;
+}
+
+CThreadInPool_ForServer::~CThreadInPool_ForServer(void)
+{
+    if (m_Counted) {
+        m_Pool->m_ThreadCount.Add(-1);
+    }
+}
+
 void
 CThreadInPool_ForServer::x_UnregisterThread(void)
 {
@@ -206,12 +221,15 @@ CThreadInPool_ForServer::Main(void)
     if (!m_Pool->m_ThrSuffix.empty()) {
         string thr_name = CNcbiApplication::Instance()->GetProgramDisplayName();
         thr_name += m_Pool->m_ThrSuffix;
-#ifdef NCBI_OS_LINUX
+#if defined(NCBI_OS_LINUX)  &&  defined(PR_SET_NAME)
         prctl(PR_SET_NAME, (unsigned long)thr_name.c_str(), 0, 0, 0);
 #endif
     }
 
-    m_Pool->Register(*this);
+    if ( !m_Pool->Register(*this) ) {
+        ERR_POST(Warning << "New worker thread blocked at the last minute.");
+        return NULL;
+    }
     CAutoUnregGuard guard(this);
 
     bool catch_all = TParamThreadPoolCatchExceptions::GetDefault();
@@ -222,12 +240,6 @@ CThreadInPool_ForServer::Main(void)
     return NULL;
 }
 
-void
-CThreadInPool_ForServer::OnExit(void)
-{
-    m_Pool->m_ThreadCount.Add(-1);
-}
-
 void
 CThreadInPool_ForServer::ProcessRequest(TItemHandle handle)
 {
@@ -272,8 +284,9 @@ CPoolOfThreads_ForServer::Spawn(unsigned int num_threads)
 {
     for (unsigned int i = 0; i < num_threads; i++)
     {
-        m_ThreadCount.Add(1);
-        NewThread()->Run();
+        CRef<TThread> thr(NewThread());
+        thr->CountSelf();
+        thr->Run();
     }
 }
 
@@ -320,12 +333,15 @@ CPoolOfThreads_ForServer::KillAllThreads(bool wait)
 }
 
 
-void
+bool
 CPoolOfThreads_ForServer::Register(TThread& thread)
 {
     CMutexGuard guard(m_Mutex);
-    if (!m_KilledAll) {
+    if (m_KilledAll) {
+        return false;
+    } else {
         m_Threads.push_back(CRef<TThread>(&thread));
+        return true;
     }
 }
 
