@@ -1,4 +1,4 @@
-/*  $Id: ncbi_stack_win32.cpp 310436 2011-06-30 17:34:16Z ivanov $
+/*  $Id: ncbi_stack_win32.cpp 357439 2012-03-22 18:37:10Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -446,6 +446,9 @@ private:
     } while(0)
 
 
+#pragma warning( push )
+#pragma warning( disable : 4748)
+
 CStackTraceImpl::CStackTraceImpl(void)
 {
     HANDLE curr_proc = GetCurrentProcess();
@@ -499,6 +502,8 @@ CStackTraceImpl::CStackTraceImpl(void)
     }
 }
 
+#pragma warning( pop )
+
 
 CStackTraceImpl::~CStackTraceImpl(void)
 {
