@@ -1,4 +1,4 @@
-/*  $Id: ncbiargs_p.hpp 272400 2011-04-07 15:17:19Z gouriano $
+/*  $Id: ncbiargs_p.hpp 361093 2012-04-30 13:55:18Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -173,6 +173,15 @@ private:
 };
 
 
+
+class CArg_Flag : public CArg_Boolean
+{
+public:
+    CArg_Flag(const string& name, bool value);
+    virtual bool HasValue(void) const;
+};
+
+
 class CArg_Dir : public CArg_String
 {
 public:
@@ -406,6 +415,19 @@ public:
 
 
 
+class CArgDesc_Opening : virtual public CArgDescMandatory
+{
+public:
+    CArgDesc_Opening(const string&            name,
+                 const string&            comment,
+                 CArgDescriptions::EType  type,
+                 CArgDescriptions::TFlags flags);
+    virtual ~CArgDesc_Opening(void);
+    virtual string GetUsageSynopsis(bool name_only = false) const;
+};
+
+
+
 class CArgDesc_PosOpt : virtual public CArgDescOptional,
                         public CArgDesc_Pos
 {
