@@ -1,4 +1,4 @@
-/*  $Id: ncbiobj.cpp 345525 2011-11-28 18:18:29Z vasilche $
+/*  $Id: ncbiobj.cpp 382092 2012-12-03 17:49:26Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -72,7 +72,11 @@
 
 
 #define USE_HEAPOBJ_LIST  0
-#if USE_HEAPOBJ_LIST
+#define USE_TLS_PTR 1
+#if USE_TLS_PTR
+#  include <corelib/ncbithr.hpp>
+#  include <vector>
+#elif USE_HEAPOBJ_LIST
 #  include <corelib/ncbi_safe_static.hpp>
 #  include <list>
 #  include <algorithm>
@@ -99,7 +103,9 @@ BEGIN_NCBI_SCOPE
 //
 
 
+#if !USE_TLS_PTR
 DEFINE_STATIC_FAST_MUTEX(sm_ObjectMutex);
+#endif
 
 #if defined(NCBI_COUNTER_NEED_MUTEX)
 // CAtomicCounter doesn't normally have a .cpp file of its own, so this
@@ -114,6 +120,139 @@ CAtomicCounter::TValue CAtomicCounter::Add(int delta) THROWS_NONE
 
 #endif
 
+#if USE_TLS_PTR
+
+static const CAtomicCounter::TValue kLastNewTypeMultiple = 1;
+static DECLARE_TLS_VAR(void*, s_LastNewPtr);
+static DECLARE_TLS_VAR(CAtomicCounter::TValue, s_LastNewType);
+typedef pair<void*, CAtomicCounter::TValue> TLastNewPtrMultipleInfo;
+typedef vector<TLastNewPtrMultipleInfo> TLastNewPtrMultiple;
+#ifdef NCBI_NO_THREADS
+static TLastNewPtrMultiple s_LastNewPtrMultiple;
+#else
+static TTlsKey s_LastNewPtrMultiple_key;
+#endif
+
+#ifdef NCBI_POSIX_THREADS
+static
+void sx_EraseLastNewPtrMultiple(void* ptr)
+{
+    delete (TLastNewPtrMultiple*)ptr;
+}
+#endif
+
+static
+TLastNewPtrMultiple& sx_GetLastNewPtrMultiple(void)
+{
+#ifdef NCBI_NO_THREADS
+    return s_LastNewPtrMultiple;
+#else
+    if ( !s_LastNewPtrMultiple_key ) {
+        DEFINE_STATIC_FAST_MUTEX(s_InitMutex);
+        NCBI_NS_NCBI::CFastMutexGuard guard(s_InitMutex);
+        if ( !s_LastNewPtrMultiple_key ) {
+            TTlsKey key = 0;
+            do {
+#  ifdef NCBI_WIN32_THREADS
+                _VERIFY((key = TlsAlloc()) != DWORD(-1));
+#  else
+                _VERIFY(pthread_key_create(&key, sx_EraseLastNewPtrMultiple)==0);
+#  endif
+            } while ( !key );
+#  ifndef NCBI_WIN32_THREADS
+            pthread_setspecific(key, 0);
+#  endif
+            s_LastNewPtrMultiple_key = key;
+        }
+    }
+    TLastNewPtrMultiple* set;
+#  ifdef NCBI_WIN32_THREADS
+    set = (TLastNewPtrMultiple*)TlsGetValue(s_LastNewPtrMultiple_key);
+#  else
+    set = (TLastNewPtrMultiple*)pthread_getspecific(s_LastNewPtrMultiple_key);
+#  endif
+    if ( !set ) {
+        set = new TLastNewPtrMultiple();
+#  ifdef NCBI_WIN32_THREADS
+        TlsSetValue(s_LastNewPtrMultiple_key, set);
+#  else
+        pthread_setspecific(s_LastNewPtrMultiple_key, set);
+#  endif
+    }
+    return *set;
+#endif
+}
+
+
+static
+void sx_PushLastNewPtrMultiple(void* ptr, CAtomicCounter::TValue type)
+{
+    _ASSERT(s_LastNewPtr);
+    TLastNewPtrMultiple& set = sx_GetLastNewPtrMultiple();
+    if ( s_LastNewType != kLastNewTypeMultiple ) {
+        set.push_back(TLastNewPtrMultipleInfo(s_LastNewPtr, s_LastNewType));
+        s_LastNewType = kLastNewTypeMultiple;
+    }
+    set.push_back(TLastNewPtrMultipleInfo(ptr, type));
+}
+
+static
+CAtomicCounter::TValue sx_PopLastNewPtrMultiple(void* ptr)
+{
+    TLastNewPtrMultiple& set = sx_GetLastNewPtrMultiple();
+    NON_CONST_ITERATE ( TLastNewPtrMultiple, it, set ) {
+        if ( it->first == ptr ) {
+            CAtomicCounter::TValue last_type = it->second;
+            swap(*it, set.back());
+            set.pop_back();
+            if ( set.empty() ) {
+                s_LastNewPtr = 0;
+            }
+            else {
+                s_LastNewPtr = set.front().first;
+            }
+            return last_type;
+        }
+    }
+    return 0;
+}
+
+static inline
+void sx_PushLastNewPtr(void* ptr, CAtomicCounter::TValue type)
+{
+    if ( s_LastNewPtr ) {
+        // multiple
+        sx_PushLastNewPtrMultiple(ptr, type);
+    }
+    else {
+        s_LastNewPtr = ptr;
+        s_LastNewType = type;
+    }
+}
+
+static inline
+CAtomicCounter::TValue sx_PopLastNewPtr(void* ptr)
+{
+    void* last_ptr = s_LastNewPtr;
+    if ( !last_ptr ) {
+        return 0;
+    }
+    CAtomicCounter::TValue last_type = s_LastNewType;
+    if ( last_type == kLastNewTypeMultiple ) {
+        // multiple ptrs
+        return sx_PopLastNewPtrMultiple(ptr);
+    }
+    else {
+        if ( s_LastNewPtr != ptr ) {
+            return 0;
+        }
+        s_LastNewPtr = 0;
+        return last_type;
+    }
+}
+
+#endif
+
 #if USE_HEAPOBJ_LIST
 static CSafeStaticPtr< list<const void*> > s_heap_obj;
 #endif
@@ -255,18 +394,23 @@ void* CObject::operator new(size_t size)
 #else
     void* ptr = ::operator new(size);
 
+#if USE_TLS_PTR
+    // just remember pointer in TLS
+    sx_PushLastNewPtr(ptr, eMagicCounterNew);
+#else// !USE_TLS_PTR
 #if USE_HEAPOBJ_LIST
     {{
         CFastMutexGuard LOCK(sm_ObjectMutex);
         s_heap_obj->push_front(ptr);
     }}
-#else// USE_HEAPOBJ_LIST
+#else// !USE_HEAPOBJ_LIST
     sx_FillNewMemory(ptr, size);
 #  if USE_COMPLEX_MASK
     GetSecondCounter(static_cast<CObject*>(ptr))->Set(eMagicCounterNew);
 #  endif// USE_COMPLEX_MASK
 #endif// USE_HEAPOBJ_LIST
     static_cast<CObject*>(ptr)->m_Counter.Set(eMagicCounterNew);
+#endif// USE_TLS_PTR
     return ptr;
 #endif
 }
@@ -274,14 +418,28 @@ void* CObject::operator new(size_t size)
 
 void CObject::operator delete(void* ptr)
 {
-#ifdef _DEBUG
-    CObject* objectPtr = static_cast<CObject*>(ptr);
-    TCount magic = objectPtr->m_Counter.Get();
+    // Can be called either from regular destruction (with counter initialized)
+    // or before CObject constructor is called (counter is not set yet).
+#if USE_TLS_PTR
+# ifdef _DEBUG
+    TCount magic = sx_PopLastNewPtr(ptr);
+    if ( !magic ) { // counter already initialized
+        magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+    }
+# else// !_DEBUG
+    // Just remove saved operator new info.
+    sx_PopLastNewPtr(ptr);
+# endif// _DEBUG
+#else// !USE_TLS_PTR
+# ifdef _DEBUG
+    TCount magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+# endif// _DEBUG
+#endif// USE_TLS_PTR
+
     // magic can be equal to:
     // 1. eMagicCounterDeleted when memory is freed after CObject destructor.
     // 2. eMagicCounterNew when memory is freed before CObject constructor.
     _ASSERT(magic == eMagicCounterDeleted  || magic == eMagicCounterNew);
-#endif
     ::operator delete(ptr);
 }
 
@@ -298,14 +456,18 @@ void* CObject::operator new(size_t size, void* place)
 // complement placement delete operator -> do nothing
 void CObject::operator delete(void* _DEBUG_ARG(ptr), void* /*place*/)
 {
-#ifdef _DEBUG
+    // Can be called either from regular destruction (with counter initialized)
+    // or before CObject constructor is called (counter is not set yet).
+#if !USE_TLS_PTR
+# ifdef _DEBUG
     CObject* objectPtr = static_cast<CObject*>(ptr);
     TCount magic = objectPtr->m_Counter.Get();
     // magic can be equal to:
     // 1. eMagicCounterDeleted when memory is freed after CObject destructor.
     // 2. 0 when memory is freed before CObject constructor.
     _ASSERT(magic == eMagicCounterDeleted  || magic == 0);
-#endif
+# endif// _DEBUG
+#endif// USE_TLS_PTR
 }
 
 
@@ -320,25 +482,44 @@ void* CObject::operator new(size_t size, CObjectMemoryPool* memory_pool)
     if ( !ptr ) {
         return operator new(size);
     }
+#if USE_TLS_PTR
+    // just remember pointer in TLS
+    sx_PushLastNewPtr(ptr, eMagicCounterPoolNew);
+#else// !USE_TLS_PTR
 #  if USE_COMPLEX_MASK
     GetSecondCounter(static_cast<CObject*>(ptr))->Set(eMagicCounterPoolNew);
 #  endif// USE_COMPLEX_MASK
     static_cast<CObject*>(ptr)->m_Counter.Set(eMagicCounterPoolNew);
+#endif// USE_TLS_PTR
     return ptr;
 }
 
 // complement pool delete operator
 void CObject::operator delete(void* ptr, CObjectMemoryPool* memory_pool)
 {
-#ifdef _DEBUG
-    CObject* objectPtr = static_cast<CObject*>(ptr);
-    TCount magic = objectPtr->m_Counter.Get();
+    // Can be called either from regular destruction (with counter initialized)
+    // or before CObject constructor is called (counter is not set yet).
+#if USE_TLS_PTR
+# ifdef _DEBUG
+    TCount magic = sx_PopLastNewPtr(ptr);
+    if ( !magic ) { // counter already initialized
+        magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+    }
+# else// !_DEBUG
+    // Just remove saved operator new info.
+    sx_PopLastNewPtr(ptr);
+# endif// _DEBUG
+#else// !USE_TLS_PTR
+# ifdef _DEBUG
+    TCount magic = static_cast<CObject*>(ptr)->m_Counter.Get();
+# endif//_DEBUG
+#endif// USE_TLS_PTR
+
     // magic can be equal to:
     // 1. eMagicCounterPoolDeleted when freed after CObject destructor.
     // 2. eMagicCounterPoolNew when freed before CObject constructor.
     _ASSERT(magic == eMagicCounterPoolDeleted ||
             magic == eMagicCounterPoolNew);
-#endif
     memory_pool->Deallocate(ptr);
 }
 
@@ -366,9 +547,41 @@ void CObject::operator delete[](void* ptr)
 }
 
 
+#ifdef _DEBUG
+# define ObjFatal Fatal
+#else
+# define ObjFatal Critical
+#endif
+
 // initialization in debug mode
 void CObject::InitCounter(void)
 {
+#if USE_TLS_PTR
+    if ( CAtomicCounter::TValue type = sx_PopLastNewPtr(this) ) {
+        switch ( type ) {
+        case eMagicCounterNew:
+            // allocated in heap
+            m_Counter.Set(eInitCounterInHeap);
+            break;
+        case eMagicCounterPoolNew:
+            // allocated in memory pool
+            m_Counter.Set(eInitCounterInPool);
+            break;
+        default:
+            ERR_POST_X(1, ObjFatal << "CObject::InitCounter: "
+                       "Bad s_LastNewType="<<type<<
+                       " at "<<StackTrace);
+            // something is broken in TLS data
+            // mark as not in heap
+            m_Counter.Set(eInitCounterNotInHeap);
+            break;
+        }
+    }
+    else {
+        // surely not in heap
+        m_Counter.Set(eInitCounterNotInHeap);
+    }
+#else
     // This code can't use Get(), which may block waiting for an
     // update that will never happen.
     // ATTENTION:  this code can cause UMR (Uninit Mem Read) -- it's okay here!
@@ -429,6 +642,7 @@ void CObject::InitCounter(void)
             m_Counter.Set(eInitCounterInPool);
         }
     }
+#endif
 }
 
 
@@ -444,12 +658,6 @@ CObject::CObject(const CObject& /*src*/)
 }
 
 
-#ifdef _DEBUG
-# define ObjFatal Fatal
-#else
-# define ObjFatal Critical
-#endif
-
 CObject::~CObject(void)
 {
     TCount count = m_Counter.Get();
@@ -587,6 +795,18 @@ void CObject::ReleaseReference(void) const
 void CObject::DoNotDeleteThisObject(void)
 {
     TCount count;
+#if USE_TLS_PTR
+    count = m_Counter.Get();
+    if ( ObjectStateValid(count) ) {
+        if ( ObjectStateCanBeDeleted(count) ) {
+            NCBI_THROW(CObjectException, eHeapState,
+                       "CObject::DoNotDeleteThisObject: "
+                       "CObject is allocated in heap");
+        }
+        // no-op
+        return;
+    }
+#else
     {{
         CFastMutexGuard LOCK(sm_ObjectMutex);
         count = m_Counter.Get();
@@ -597,6 +817,7 @@ void CObject::DoNotDeleteThisObject(void)
             return;
         }
     }}
+#endif
 
     if ( count == eMagicCounterDeleted ||
          count == eMagicCounterPoolDeleted ) {
@@ -617,6 +838,18 @@ void CObject::DoDeleteThisObject(void)
 {
 #ifndef USE_SINGLE_ALLOC
     TCount count;
+#if USE_TLS_PTR
+    count = m_Counter.Get();
+    if ( ObjectStateValid(count) ) {
+        if ( !ObjectStateCanBeDeleted(count) ) {
+            NCBI_THROW(CObjectException, eHeapState,
+                       "CObject::DoDeleteThisObject: "
+                       "CObject is not allocated in heap");
+        }
+        // no-op
+        return;
+    }
+#else
     {{
         CFastMutexGuard LOCK(sm_ObjectMutex);
         count = m_Counter.Get();
@@ -633,6 +866,7 @@ void CObject::DoDeleteThisObject(void)
             return;
         }
     }}
+#endif
     if ( ObjectStateValid(count) ) {
         ERR_POST_X(7, ObjFatal << "CObject::DoDeleteThisObject: "
                       "object was created without heap signature"<<StackTrace);
@@ -660,8 +894,9 @@ NCBI_PARAM_DEF_EX(bool, NCBI, ABORT_ON_COBJECT_THROW, false,
 void CObjectException::x_InitErrCode(CException::EErrCode err_code)
 {
     CCoreException::x_InitErrCode(err_code);
-    static NCBI_PARAM_TYPE(NCBI, ABORT_ON_COBJECT_THROW) sx_abort_on_throw;
-    if ( sx_abort_on_throw.Get() ) {
+    static const bool sx_abort_on_throw =
+        NCBI_PARAM_TYPE(NCBI, ABORT_ON_COBJECT_THROW)::GetDefault();
+    if ( sx_abort_on_throw ) {
         Abort();
     }
 }
@@ -678,24 +913,24 @@ void CObject::DebugDump(CDebugDumpContext ddc, unsigned int /*depth*/) const
 NCBI_PARAM_DECL(bool, NCBI, ABORT_ON_NULL);
 NCBI_PARAM_DEF_EX(bool, NCBI, ABORT_ON_NULL, false,
                   eParam_NoThread, NCBI_ABORT_ON_NULL);
-static NCBI_PARAM_TYPE(NCBI, ABORT_ON_NULL) sx_abort_on_null;
+static const bool sx_abort_on_null =
+    NCBI_PARAM_TYPE(NCBI, ABORT_ON_NULL)::GetDefault();
 
 void CObject::ThrowNullPointerException(void)
 {
-    if ( sx_abort_on_null.Get() ) {
+    if ( sx_abort_on_null ) {
         Abort();
     }
     NCBI_EXCEPTION_VAR(ex,
         CCoreException, eNullPtr, "Attempt to access NULL pointer.");
     ex.SetSeverity(eDiag_Critical);
     NCBI_EXCEPTION_THROW(ex);
-    // NCBI_THROW(CCoreException, eNullPtr, "Attempt to access NULL pointer.");
 }
 
 
 void CObject::ThrowNullPointerException(const type_info& type)
 {
-    if ( sx_abort_on_null.Get() ) {
+    if ( sx_abort_on_null ) {
         Abort();
     }
     NCBI_EXCEPTION_VAR(ex,
@@ -703,8 +938,6 @@ void CObject::ThrowNullPointerException(const type_info& type)
                        string("Attempt to access NULL pointer: ")+type.name());
     ex.SetSeverity(eDiag_Critical);
     NCBI_EXCEPTION_THROW(ex);
-    // NCBI_THROW(CCoreException, eNullPtr,
-    //            string("Attempt to access NULL pointer: ")+type.name());
 }
 
 
@@ -727,16 +960,26 @@ public:
 
         void Dump(void) const
         {
+            unsigned total_locks = 0;
             ITERATE ( TLocks, it, m_Locks ) {
+                ++total_locks;
                 LOG_POST("Locked<"<<sx_MonitorType->name()<<">"
                          "("<<it->first<<","<<m_Object<<")"
                          " @ " << *it->second);
             }
+            unsigned total_unlocks = 0;
             ITERATE ( TUnlocks, it, m_Unlocks ) {
+                ++total_unlocks;
                 LOG_POST("Unlocked<"<<sx_MonitorType->name()<<">"
                          "("<<it->first<<","<<m_Object<<")"
                          " @ " << *it->second);
             }
+            if ( total_locks ) {
+                LOG_POST("Total locks for "<<m_Object<<": "<<total_locks);
+            }
+            if ( total_unlocks ) {
+                LOG_POST("Total unlocks for "<<m_Object<<": "<<total_unlocks);
+            }
         }
         int LockCount(void) const
         {
@@ -912,6 +1155,7 @@ const char* CObjectException::GetErrCodeString(void) const
     case eRefOverflow:  return "eRefOverflow";
     case eNoRef:        return "eNoRef";
     case eRefUnref:     return "eRefUnref";
+    case eHeapState:    return "eHeapState";
     default:    return CException::GetErrCodeString();
     }
 }
@@ -1160,7 +1404,6 @@ void memchk(const void* ptr, char byte, size_t size)
 
 void* s_alloc_mem(size_t size, bool array) throw()
 {
-    //fprintf(stderr, "s_alloc_mem(%u, %d)\n", (unsigned)size, array);
     unsigned number = seq_number.Add(1);
     SAllocLog& log =
         start_log(number, array? SAllocLog::eNewArr: SAllocLog::eNew);
@@ -1198,7 +1441,6 @@ void* s_alloc_mem(size_t size, bool array) throw()
 
 void s_free_mem(void* ptr, bool array)
 {
-    //fprintf(stderr, "s_free_mem(%p, %d)\n", ptr, array);
     unsigned number = seq_number.Add(1);
     SAllocLog& log =
         start_log(number, array ? SAllocLog::eDeleteArr: SAllocLog::eDelete);
