@@ -1,4 +1,4 @@
-/* $Id: ncbistre.cpp 372997 2012-08-23 20:51:17Z rafanovi $
+/* $Id: ncbistre.cpp 386350 2013-01-17 19:10:11Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,43 +44,10 @@ BEGIN_NCBI_SCOPE
 
 
 #if defined(NCBI_OS_MSWIN) && defined(_UNICODE)
-
-CNcbiIfstream::CNcbiIfstream(
-    const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
- : IO_PREFIX::ifstream(_T_XCSTRING(_Filename), _Mode, _Prot)
+wstring ncbi_Utf8ToWstring(const char *utf8)
 {
+    return _T_XSTRING(utf8);
 }
-
-void CNcbiIfstream::open(
-    const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
-{
-    IO_PREFIX::ifstream::open(_T_XCSTRING(_Filename), _Mode, _Prot);
-}
-
-CNcbiOfstream::CNcbiOfstream(
-    const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
- : IO_PREFIX::ofstream(_T_XCSTRING(_Filename), _Mode, _Prot)
-{
-}
-
-void CNcbiOfstream::open(
-    const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
-{
-    IO_PREFIX::ofstream::open(_T_XCSTRING(_Filename), _Mode, _Prot);
-}
-
-CNcbiFstream::CNcbiFstream(
-    const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
- : IO_PREFIX::fstream(_T_XCSTRING(_Filename), _Mode, _Prot)
-{
-}
-
-void CNcbiFstream::open(
-    const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
-{
-    IO_PREFIX::fstream::open(_T_XCSTRING(_Filename), _Mode, _Prot);
-}
-
 #endif
 
 
