@@ -1,4 +1,4 @@
-/*  $Id: ncbitime.cpp 366018 2012-06-11 14:35:03Z ivanov $
+/*  $Id: ncbitime.cpp 388092 2013-02-05 14:48:19Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1125,8 +1125,13 @@ string CTime::AsString(const CTimeFormat& format, TSeconds out_tz) const
                   if (IsGmtTime()) {
                       break;
                   }
-                  TSeconds tz = (out_tz == eCurrentTimeZone) ?
-                                TimeZone() : out_tz;
+                  TSeconds tz = out_tz;
+                  if (out_tz == eCurrentTimeZone) {
+                      tz = TimeZone();
+                      if ( Daylight() ) {
+                          tz -= 3600;  // DST in effect
+                      }
+                  }
                   str += (tz > 0) ? '-' : '+';
                   if (tz < 0) tz = -tz;
                   int tzh = int(tz / 3600);
@@ -1874,9 +1879,9 @@ TSeconds CTime::TimeZoneDiff(void) const
     const CTime tg(GetGmtTime());
 
     TSeconds dSecs  = tl.Second() - tg.Second();
-    int      dMins  = tl.Minute() - tg.Minute();
-    int      dHours = tl.Hour()   - tg.Hour();
-    int      dDays  = tl.DiffWholeDays(tg);
+    TSeconds dMins  = tl.Minute() - tg.Minute();
+    TSeconds dHours = tl.Hour()   - tg.Hour();
+    TSeconds dDays  = tl.DiffWholeDays(tg);
     return ((dDays * 24 + dHours) * 60 + dMins) * 60 + dSecs;
 }
 
@@ -1902,9 +1907,9 @@ TSeconds CTime::DiffSecond(const CTime& from) const
         p2 = &from;
     }
     TSeconds dSecs  = p1->Second() - p2->Second();
-    int      dMins  = p1->Minute() - p2->Minute();
-    int      dHours = p1->Hour()   - p2->Hour();
-    int      dDays  = p1->DiffWholeDays(*p2);
+    TSeconds dMins  = p1->Minute() - p2->Minute();
+    TSeconds dHours = p1->Hour()   - p2->Hour();
+    TSeconds dDays  = p1->DiffWholeDays(*p2);
     return ((dDays * 24 + dHours) * 60 + dMins) * 60 + dSecs;
 }
 
