@@ -1,4 +1,4 @@
-/*  $Id: request_ctx.cpp 372362 2012-08-16 18:59:00Z rafanovi $
+/*  $Id: request_ctx.cpp 383502 2012-12-14 20:13:47Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,7 @@ CRequestContext::CRequestContext(void)
       m_ReqTimer(CStopWatch::eStop),
       m_BytesRd(0),
       m_BytesWr(0),
+      m_LogSite(kEmptyStr),
       m_PropSet(0),
       m_IsRunning(false),
       m_AutoIncOnPost(false)
@@ -110,6 +111,7 @@ void CRequestContext::Reset(void)
     UnsetRequestStatus();
     UnsetBytesRd();
     UnsetBytesWr();
+    UnsetLogSite();
     m_ReqTimer.Reset();
 }
 
@@ -230,7 +232,7 @@ bool CRequestContext::IsValidSessionID(const string& session_id)
             if (NStr::StringToUInt8(uid, NStr::fConvErr_NoThrow, 16) == 0  &&  errno !=0) {
                 return false;
             }
-            CTempString rqid(session_id, 18, session_id.size() - 20);
+            CTempString rqid(session_id, 17, session_id.size() - 20);
             if (NStr::StringToUInt(rqid, NStr::fConvErr_NoThrow) == 0  &&  errno != 0) {
                 return false;
             }
@@ -316,4 +318,45 @@ const char* CRequestContextException::GetErrCodeString(void) const
 }
 
 
+// NCBI_LOG_SITE logging.
+NCBI_PARAM_DECL(string, Log, Site);
+NCBI_PARAM_DEF_EX(string, Log, Site, kEmptyStr, eParam_NoThread, NCBI_LOG_SITE);
+typedef NCBI_PARAM_TYPE(Log, Site) TLogSite;
+
+
+string CRequestContext::GetApplicationLogSite(void)
+{
+    return TLogSite::GetDefault();
+}
+
+
+string CRequestContext::GetLogSite(void) const
+{
+    if ( x_IsSetProp(eProp_LogSite) ) {
+        return m_LogSite;
+    }
+    return TLogSite::GetDefault();
+}
+
+
+void CRequestContext::SetLogSite(const string& log_site)
+{
+    x_SetProp(eProp_LogSite);
+    m_LogSite = log_site;
+}
+
+
+bool CRequestContext::IsSetLogSite(void) const
+{
+    return x_IsSetProp(eProp_LogSite);
+}
+
+
+void CRequestContext::UnsetLogSite(void)
+{
+    x_UnsetProp(eProp_LogSite);
+    m_LogSite = kEmptyStr;
+}
+
+
 END_NCBI_SCOPE
