@@ -1,4 +1,4 @@
-/* $Id: stream_utils.cpp 341897 2011-10-25 00:56:50Z lavr $
+/* $Id: stream_utils.cpp 356642 2012-03-15 17:49:56Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,8 +31,10 @@
  *   2. Non-blocking read.
  *
  *   Reader-writer utilities:
- *   1. Append an IReader's contents into a string.
- *   2. Construct an IReader object from a string.
+ *   1. Construct an IReader object from an istream;
+ *   2. Construct an IWriter object from an ostream;
+ *   3. Construct an IReader object from a string;
+ *   4. Append entire IReader's contents to a string.
  *
  */
 
@@ -66,7 +68,7 @@ class CPushback_Streambuf : public CPushback_StreambufBase
 
 public:
     CPushback_Streambuf(istream& istream, CT_CHAR_TYPE* buf,
-                        streamsize buf_size, void* del_ptr);
+                        size_t buf_size, void* del_ptr);
     virtual ~CPushback_Streambuf();
 
 protected:
@@ -91,10 +93,10 @@ protected:
     // See comments in "connect/ncbi_conn_streambuf.hpp"
     virtual streamsize  _Xsgetn_s(CT_CHAR_TYPE* buf, size_t, streamsize n)
     { return xsgetn(buf, n); }
-#endif /*NCBI_OS_MSWIN*/
+#endif //NCBI_OS_MSWIN
 
 private:
-    void                 x_FillBuffer(streamsize max_size);
+    void                 x_FillBuffer(size_t max_size);
     void                 x_DropBuffer(void);
 
     istream&             m_Is;      // I/O stream this streambuf is attached to
@@ -102,17 +104,17 @@ private:
     CPushback_Streambuf* m_Next;    // of the kin in the sb chain
 
     CT_CHAR_TYPE*        m_Buf;
-    streamsize           m_BufSize;
+    size_t               m_BufSize;
     void*                m_DelPtr;
 
     static volatile int  sm_Index;
     static void          x_Callback(IOS_BASE::event, IOS_BASE&, int);
 
-    static const streamsize kMinBufSize;
+    static const size_t  kMinBufSize;
 };
 
 
-const streamsize CPushback_Streambuf::kMinBufSize = 4096;
+const size_t CPushback_Streambuf::kMinBufSize = 4096;
 
 
 volatile int CPushback_Streambuf::sm_Index = -1;  // uninited
@@ -131,7 +133,7 @@ void CPushback_Streambuf::x_Callback(IOS_BASE::event event,
 
 CPushback_Streambuf::CPushback_Streambuf(istream&      is,
                                          CT_CHAR_TYPE* buf,
-                                         streamsize    buf_size,
+                                         size_t        buf_size,
                                          void*         del_ptr)
     : m_Is(is), m_Next(0), m_Buf(buf), m_BufSize(buf_size), m_DelPtr(del_ptr)
 {
@@ -146,7 +148,7 @@ CPushback_Streambuf::CPushback_Streambuf(istream&      is,
                 DEFINE_STATIC_FAST_MUTEX(s_PushbackMutex);
                 CFastMutexGuard guard(s_PushbackMutex);
                 if (sm_Index == -1) {
-                    sm_Index =  IOS_BASE::xalloc();
+                    sm_Index  = IOS_BASE::xalloc();
                 }
             }
             m_Is.register_callback(x_Callback, sm_Index);
@@ -162,7 +164,7 @@ CPushback_Streambuf::CPushback_Streambuf(istream&      is,
 CPushback_Streambuf::~CPushback_Streambuf()
 {
     if (m_Is.pword(sm_Index) == this) {
-        m_Is.pword(sm_Index) =  0;
+        m_Is.pword(sm_Index)  = 0;
     }
     delete[] (CT_CHAR_TYPE*) m_DelPtr;
     delete m_Next;
@@ -226,17 +228,20 @@ CT_INT_TYPE CPushback_Streambuf::underflow(void)
     m_MIPSPRO_ReadsomeGptr = (CT_CHAR_TYPE*)(-1L);
 #endif //NCBI_COMPILER_MIPSPRO
 
-    x_FillBuffer(m_Sb->in_avail());
+    x_FillBuffer((size_t) m_Sb->in_avail());
     return gptr() < egptr() ? CT_TO_INT_TYPE(*gptr()) : CT_EOF;
 }
 
 
 streamsize CPushback_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 {
-    size_t n_total = 0;
-    while (m) {
+    streamsize n_total = 0;
+    while (m > 0) {
         if (gptr() < egptr()) {
-            size_t n       = (size_t) m;
+            size_t n       = sizeof(m) > sizeof(size_t)
+                &&  m > (streamsize) numeric_limits<size_t>::max()
+                ? numeric_limits<size_t>::max()
+                : (size_t) m;
             size_t n_avail = (size_t)(egptr() - gptr());
             size_t n_read  = n < n_avail ? n : n_avail;
             if (buf != gptr()) {  // either equal or non-overlapping
@@ -244,10 +249,10 @@ streamsize CPushback_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
             }
             gbump((int) n_read);
             m       -= (streamsize) n_read;
-            buf     += (streamsize) n_read;
+            buf     +=              n_read;
             n_total += (streamsize) n_read;
         } else {
-            x_FillBuffer(m);
+            x_FillBuffer((size_t) m);
             if (gptr() >= egptr()) {
                 break;
             }
@@ -294,7 +299,7 @@ streambuf* CPushback_Streambuf::setbuf(CT_CHAR_TYPE* /*buf*/,
 }
 
 
-void CPushback_Streambuf::x_FillBuffer(streamsize max_size)
+void CPushback_Streambuf::x_FillBuffer(size_t max_size)
 {
     _ASSERT(m_Sb);
     if ( !max_size ) {
@@ -304,14 +309,14 @@ void CPushback_Streambuf::x_FillBuffer(streamsize max_size)
     CPushback_Streambuf* sb = dynamic_cast<CPushback_Streambuf*> (m_Sb);
     if ( !sb ) {
         CT_CHAR_TYPE* bp = 0;
-        streamsize buf_size = m_DelPtr
-            ? (streamsize)(m_Buf - (CT_CHAR_TYPE*) m_DelPtr) + m_BufSize : 0;
+        size_t buf_size = m_DelPtr
+            ? (size_t)(m_Buf - (CT_CHAR_TYPE*) m_DelPtr) + m_BufSize : 0;
         if (buf_size < kMinBufSize) {
             buf_size = kMinBufSize;
             bp = new CT_CHAR_TYPE[buf_size];
         }
-        streamsize n = m_Sb->sgetn(bp ? bp : (CT_CHAR_TYPE*) m_DelPtr,
-                                   buf_size < max_size ? buf_size : max_size);
+        streamsize r = (streamsize)(buf_size < max_size ? buf_size : max_size);
+        streamsize n = m_Sb->sgetn(bp ? bp : (CT_CHAR_TYPE*) m_DelPtr, r);
         if (n <= 0) {
             // NB: For unknown reasons WorkShop6 can return -1 from sgetn :-/
             delete[] bp;
@@ -369,13 +374,20 @@ void CPushback_Streambuf::x_DropBuffer(void)
 
 void CStreamUtils::x_Pushback(CNcbiIstream& is,
                               CT_CHAR_TYPE* buf,
-                              streamsize    buf_size,
+                              streamsize    x_buf_size,
                               void*         del_ptr,
                               EPushback_How how)
 {
-    _ASSERT(!buf_size  ||  buf);
+    _ASSERT(!x_buf_size  ||  buf);
     _ASSERT(del_ptr <= buf);
 
+    if (sizeof(x_buf_size) > sizeof(size_t)
+        &&  x_buf_size > (streamsize) numeric_limits<size_t>::max()) {
+        NCBI_THROW(CCoreException, eInvalidArg,
+                   "Pushback data size too large");
+    }
+    size_t buf_size = (size_t) x_buf_size;
+
     CPushback_Streambuf* sb = dynamic_cast<CPushback_Streambuf*> (is.rdbuf());
 
     if (sb  &&  buf_size) {
@@ -400,8 +412,8 @@ void CStreamUtils::x_Pushback(CNcbiIstream& is,
                                   ? CPushback_Streambuf::kMinBufSize
                                   : CPushback_Streambuf::kMinBufSize >> 4))) {
             CT_CHAR_TYPE* bp = sb->gptr();
-            streamsize avail = (streamsize)(bp - sb->m_Buf);
-            streamsize take  = avail < buf_size ? avail : buf_size;
+            size_t avail = bp - sb->m_Buf;
+            size_t take  = avail < buf_size ? avail : buf_size;
             if (take) {
                 bp -= take;
                 buf_size -= take;
@@ -434,12 +446,12 @@ void CStreamUtils::x_Pushback(CNcbiIstream& is,
 
 #ifdef   NCBI_NO_READSOME
 #  undef NCBI_NO_READSOME
-#endif /*NCBI_NO_READSOME*/
+#endif //NCBI_NO_READSOME
 
-#if defined(NCBI_COMPILER_GCC)
+#if   defined(NCBI_COMPILER_GCC)
 #  if NCBI_COMPILER_VERSION < 300
 #    define NCBI_NO_READSOME 1
-#  endif /*NCBI_COMPILER_VERSION*/
+#  endif //NCBI_COMPILER_VERSION<300
 #elif defined(NCBI_COMPILER_MSVC)
     /* MSVC's readsome() is buggy [causes 1 byte reads] and is thus avoided.
      * Now when we have worked around the issue by implementing fastopen()
@@ -453,21 +465,21 @@ void CStreamUtils::x_Pushback(CNcbiIstream& is,
      * Don't use istream::readsome() but istream::read() instead in order to be
      * able to clear fake EOF caused by the unnecessary underflow() upcall.*/
 #  define NCBI_NO_READSOME 1
-#endif /*NCBI_COMPILER*/
+#endif //NCBI_COMPILER_...
 
 
 #ifndef NCBI_NO_READSOME
-static inline streamsize s_Readsome(CNcbiIstream& is,
+static inline streamsize x_Readsome(CNcbiIstream& is,
                                     CT_CHAR_TYPE* buf,
                                     streamsize    buf_size)
 {
 #  ifdef NCBI_COMPILER_WORKSHOP
     /* Rogue Wave does not always return correct value from is.readsome() :-/
      * In particular, when streambuf::showmanyc() returns 1 followed by a
-     * failed read() [that implements the extraction from the stream], which
-     * encounters the EOF, the readsome() will blindly return 1, and in
-     * general, always exactly the number of bytes showmanyc() reported,
-     * regardless of actually extracted by subsequent read operation.  Bug!
+     * failed read() [that implements an extraction from the stream] due to
+     * reaching the EOF, then readsome() will blindly return 1; and in general,
+     * returns always exactly the number of bytes that showmanyc()'s reported,
+     * regardless of the actually extracted ones by a subsequent read.  Bug!!
      * NOTE that showmanyc() does not guarantee the number of bytes that can
      * be read, but returns a best guess estimate [C++ Standard, footnote 275].
      */
@@ -475,20 +487,20 @@ static inline streamsize s_Readsome(CNcbiIstream& is,
     return n ? is.gcount() : 0;
 #  else
     return is.readsome(buf, buf_size);
-#  endif /*NCBI_COMPILER_WORKSHOP*/
+#  endif //NCBI_COMPILER_WORKSHOP
 }
-#endif /*NCBI_NO_READSOME*/
+#endif //NCBI_NO_READSOME
 
 
-static streamsize s_DoReadsome(CNcbiIstream& is,
-                               CT_CHAR_TYPE* buf,
-                               streamsize    buf_size)
+static streamsize s_Readsome(CNcbiIstream& is,
+                             CT_CHAR_TYPE* buf,
+                             streamsize    buf_size)
 {
     _ASSERT(buf  &&  buf_size);
 #ifdef NCBI_NO_READSOME
     if ( !is.good() ) {
         is.setstate(is.rdstate() | NcbiFailbit);
-        return 0; // simulate construction of sentry in real readsome()
+        return 0; // simulate construction of sentry in standard readsome()
     }
     // Special case: GCC had no readsome() prior to ver 3.0;
     // read() will set "eof" (and "fail") flag if gcount() < buf_size
@@ -502,7 +514,7 @@ static streamsize s_DoReadsome(CNcbiIstream& is,
     if (save)
         is.exceptions(NcbiGoodbit);
     is.read(buf, avail);
-    // readsome is not supposed to set a failbit on a stream initially good
+    // readsome() is not supposed to set a failbit on a stream initially good
     is.clear(is.rdstate() & ~NcbiFailbit);
     if (save)
         is.exceptions(save);
@@ -513,7 +525,7 @@ static streamsize s_DoReadsome(CNcbiIstream& is,
     return count;
 #else
     // Try to read data
-    streamsize n = s_Readsome(is, buf, buf_size);
+    streamsize n = x_Readsome(is, buf, buf_size);
     if (n != 0  ||  !is.good())
         return n;
     // No buffered data found, try to read from the real source [still good]
@@ -530,8 +542,8 @@ static streamsize s_DoReadsome(CNcbiIstream& is,
     if (buf_size == 1)
         return 1; // do not need more data
     // Read more data (up to "buf_size" bytes)
-    return s_Readsome(is, buf + 1, buf_size - 1) + 1;
-#endif /*NCBI_NO_READSOME*/
+    return x_Readsome(is, buf + 1, buf_size - 1) + 1;
+#endif //NCBI_NO_READSOME
 }
 
 
@@ -544,12 +556,12 @@ streamsize CStreamUtils::Readsome(CNcbiIstream& is,
         dynamic_cast<CMIPSPRO_ReadsomeTolerantStreambuf*> (is.rdbuf());
     if (sb)
         sb->MIPSPRO_ReadsomeBegin();
-#  endif /*NCBI_COMPILER_MIPSPRO*/
-    streamsize result = s_DoReadsome(is, buf, buf_size);
+#  endif //NCBI_COMPILER_MIPSPRO
+    streamsize result = s_Readsome(is, buf, buf_size);
 #  ifdef NCBI_COMPILER_MIPSPRO
     if (sb)
         sb->MIPSPRO_ReadsomeEnd();
-#  endif /*NCBI_COMPILER_MIPSPRO*/
+#  endif //NCBI_COMPILER_MIPSPRO
     return result;
 }
 
@@ -558,43 +570,82 @@ ERW_Result CStreamReader::Read(void*   buf,
                                size_t  count,
                                size_t* bytes_read)
 {
-    streamsize r = m_Stream->rdbuf()->sgetn(static_cast<char*>(buf), count);
+    streamsize r = m_Stream->good()
+        ? m_Stream->rdbuf()->sgetn(static_cast<char*>(buf), count) : 0;
+#ifdef NCBI_COMPILER_WORKSHOP
+    if (r < 0) {
+        r = 0; // NB: WS6 is known to return -1 from sgetn() :-/
+    }
+#endif //NCBI_COMPILER_WORKSHOP
     if ( bytes_read ) {
         *bytes_read = (size_t) r;
     }
-    return r ? eRW_Success : eRW_Eof;
+    if (!r) {
+        m_Stream->setstate(NcbiEofbit);
+        return eRW_Eof;
+    }
+    return eRW_Success;
 }
 
 
 ERW_Result CStreamReader::PendingCount(size_t* count)
 {
-    *count = m_Stream->rdbuf()->in_avail();
-    return eRW_Success;
+    IOS_BASE::iostate iostate = m_Stream->rdstate();
+    if (!(iostate & ~NcbiEofbit)) {  // NB: good() OR eof()
+        if (iostate) {
+            return eRW_Eof;
+        }
+        *count = (size_t)m_Stream->rdbuf()->in_avail();
+        return eRW_Success;
+    }
+    return eRW_Error;
 }
 
 
-void ExtractReaderContents(IReader& reader, string& s)
+ERW_Result CStreamWriter::Write(const void* buf,
+                                size_t      count,
+                                size_t*     bytes_written)
 {
-    size_t     n      = 0;
-    SIZE_TYPE  pos    = s.size();
-    ERW_Result status = eRW_Success;
+    streamsize w = m_Stream->good()
+        ? m_Stream->rdbuf()->sputn(static_cast<const char*>(buf), count) : 0;
+    if ( bytes_written ) {
+        *bytes_written = (size_t) w;
+    }
+    if (!w) {
+        m_Stream->setstate(NcbiBadbit);
+        return eRW_Error;
+    }
+    return eRW_Success;
+}
+ 
 
-    if (s.size() < 4096) {
+ERW_Result CStreamWriter::Flush(void)
+{
+    return m_Stream->flush() ? eRW_Success : eRW_Error;
+}
+
+
+void g_ExtractReaderContents(IReader& reader, string& s)
+{
+    SIZE_TYPE pos = s.size();
+    if (pos < 4096) {
         s.resize(4096);
     }
 
-    while (status == eRW_Success) {
-        pos += n;
+    ERW_Result status;
+    do {
         // Grow exponentially to avoid possible quadratic runtime,
         // adjusting size rather than capacity as the latter would
         // require gratuitous double-buffering.
         if (s.size() <= pos + 1024) {
             s.resize(s.size() * 2);
         }
+        size_t n;
         status = reader.Read(&s[pos], s.size() - pos, &n);
-    }
+        pos += n;
+    } while (status == eRW_Success);
     // shrink back to the actual size
-    s.resize(pos + n);
+    s.resize(pos);
     // XXX - issue diagnostic message or exception if status != eRW_Eof?
 }
 
@@ -609,14 +660,10 @@ ERW_Result CStringReader::Read(void* buf, size_t count, size_t* bytes_read)
         m_String.erase(0, m_Position);
         m_Position = 0;
     }
-    if (bytes_read) {
+    if ( bytes_read ) {
         *bytes_read = n;
     }
-    if (n == 0  &&  count > 0) {
-        return eRW_Eof;
-    } else {
-        return eRW_Success;
-    }
+    return count  &&  !n ? eRW_Eof : eRW_Success;
 }
 
 
