@@ -1,4 +1,4 @@
-/*  $Id: dbapi_conn_factory.cpp 343769 2011-11-09 16:51:52Z ivanovp $
+/*  $Id: dbapi_conn_factory.cpp 353121 2012-02-13 17:04:30Z ivanovp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -54,7 +54,8 @@ m_MaxNumOfValidationAttempts(1),
 m_MaxNumOfServerAlternatives(32),
 m_MaxNumOfDispatches(0),
 m_ConnectionTimeout(0),
-m_LoginTimeout(0)
+m_LoginTimeout(0),
+m_TryServerToo(false)
 {
     ConfigureFromRegistry(registry);
 }
@@ -90,6 +91,7 @@ CDBConnectionFactory::ConfigureFromRegistry(const IRegistry* registry)
             registry->GetBool(section_name, "TRY_SERVER_AFTER_SERVICE", false);
     } else {
         m_MaxNumOfConnAttempts = 1;
+        m_MaxNumOfValidationAttempts = 1;
         m_MaxNumOfServerAlternatives = 32;
         m_MaxNumOfDispatches = 0;
         m_ConnectionTimeout = 0;
