@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_context.cpp 369364 2012-07-18 17:40:00Z rafanovi $
+/*  $Id: dbapi_impl_context.cpp 368850 2012-07-12 19:36:18Z ivanovp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -441,8 +441,10 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
 
     // Precondition check.
     if (params.GetServerName().empty() ||
-            params.GetUserName().empty() ||
-            params.GetPassword().empty()) {
+        (!TDbapi_CanUseKerberos::GetDefault()
+         &&  (params.GetUserName().empty()
+              ||  params.GetPassword().empty())))
+    {
         string err_msg("Insufficient info/credentials to connect.");
 
         if (params.GetServerName().empty()) {
