@@ -1,4 +1,4 @@
-/* $Id: exception.cpp 343796 2011-11-09 18:12:57Z ivanovp $
+/* $Id: exception.cpp 357742 2012-03-26 20:17:21Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -458,7 +458,14 @@ CDB_UserHandler* CDB_UserHandler::SetDefault(CDB_UserHandler* h)
 }
 
 
-bool CDB_UserHandler::HandleAll(const TExceptions& exceptions)
+bool CDB_UserHandler::HandleAll(const TExceptions& /* exceptions */)
+{
+    return false;
+}
+
+bool CDB_UserHandler::HandleMessage(int /* severity */,
+                                    int /* msgnum */,
+                                    const string& /* message */)
 {
     return false;
 }
@@ -501,10 +508,19 @@ bool CDB_UserHandler_Diag::HandleIt(CDB_Exception* ex)
     if ( !ex )
         return true;
 
-    if ( m_Prefix.empty() ) {
-        ERR_POST_X(1, *ex);
-    } else {
-        ERR_POST_X(2, Severity(ex->GetSeverity()) << m_Prefix << " " << *ex);
+    if (ex->GetSeverity() == eDiag_Info) {
+        if ( m_Prefix.empty() ) {
+            ERR_POST_X(1, Info << ex->GetMsg());
+        } else {
+            ERR_POST_X(2, Info << m_Prefix << " " << ex->GetMsg());
+        }
+    }
+    else {
+        if ( m_Prefix.empty() ) {
+            ERR_POST_X(3, *ex);
+        } else {
+            ERR_POST_X(4, Severity(ex->GetSeverity()) << m_Prefix << " " << *ex);
+        }
     }
 
     return true;
