@@ -1,4 +1,4 @@
-/* $Id: context.cpp 169152 2009-08-26 16:47:29Z ivanovp $
+/* $Id: context.cpp 343769 2011-11-09 16:51:52Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -57,14 +57,6 @@
 
 BEGIN_NCBI_SCOPE
 
-/////////////////////////////////////////////////////////////////////////////
-inline
-CDiagCompileInfo GetBlankCompileInfo(void)
-{
-    return CDiagCompileInfo();
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 //
 //  CODBCContextRegistry (Singleton)
@@ -232,17 +224,18 @@ void CODBC_Reporter::ReportErrors(void) const
 
             switch( rc ) {
             case SQL_SUCCESS:
-                if(util::strncmp(SqlState, _T("HYT"), 3) == 0) { // timeout
+                if(util::strncmp(SqlState, _T_NCBI_ODBC("HYT"), 3) == 0) { // timeout
 
-                    CDB_TimeoutEx to(GetBlankCompileInfo(),
+                    CDB_TimeoutEx to(DIAG_COMPILE_INFO,
                                     0,
                                     err_msg.c_str(),
                                     NativeError);
 
                     m_HStack->PostMsg(&to);
                 }
-                else if(util::strncmp(SqlState, _T("40001"), 5) == 0) { // deadlock
-                    CDB_DeadlockEx dl(GetBlankCompileInfo(),
+				else if(util::strncmp(SqlState, _T_NCBI_ODBC("40001"), 5) == 0) {
+					// deadlock
+                    CDB_DeadlockEx dl(DIAG_COMPILE_INFO,
                                     0,
                                     err_msg.c_str());
                     m_HStack->PostMsg(&dl);
@@ -252,7 +245,7 @@ void CODBC_Reporter::ReportErrors(void) const
                 }
                 else if(NativeError != 5701
                     && NativeError != 5703 ){
-                    CDB_SQLEx se(GetBlankCompileInfo(),
+                    CDB_SQLEx se(DIAG_COMPILE_INFO,
                                 0,
                                 err_msg.c_str(),
                                 (NativeError == 0 ? eDiag_Info : eDiag_Warning),
@@ -271,7 +264,7 @@ void CODBC_Reporter::ReportErrors(void) const
                 err_msg += GetExtraMsg();
 
                 {
-                    CDB_DSEx dse(GetBlankCompileInfo(),
+                    CDB_DSEx dse(DIAG_COMPILE_INFO,
                                 0,
                                 err_msg.c_str(),
                                 eDiag_Warning,
@@ -286,7 +279,7 @@ void CODBC_Reporter::ReportErrors(void) const
                 err_msg += GetExtraMsg();
 
                 {
-                    CDB_ClientEx ce(GetBlankCompileInfo(),
+                    CDB_ClientEx ce(DIAG_COMPILE_INFO,
                                     0,
                                     err_msg.c_str(),
                                     eDiag_Warning,
