@@ -1,4 +1,4 @@
-/* $Id: variant.cpp 145603 2008-11-13 21:13:10Z ssikorsk $
+/* $Id: variant.cpp 331663 2011-08-18 15:37:29Z ivanovp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -23,7 +23,7 @@
 *
 * ===========================================================================
 *
-* File Name:  $Id: variant.cpp 145603 2008-11-13 21:13:10Z ssikorsk $
+* File Name:  $Id: variant.cpp 331663 2011-08-18 15:37:29Z ivanovp $
 *
 * Author:  Michael Kholodov
 *
@@ -375,7 +375,7 @@ string CVariant::GetString(void) const
                     break;
                 }
             case eDB_TinyInt:
-                s = NStr::IntToString((long)GetByte());
+                s = NStr::IntToString(GetByte());
                 break;
             case eDB_SmallInt:
                 s = NStr::IntToString(GetInt2());
