@@ -1,4 +1,4 @@
-/* $Id: cache.cpp 371868 2012-08-13 15:10:25Z rafanovi $
+/* $Id: cache.cpp 381650 2012-11-27 16:43:32Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,7 +42,7 @@ BEGIN_objects_SCOPE
 
 
 COrgRefCache::COrgRefCache( CTaxon1& host )
-    : m_host( host ), m_nCacheCapacity( 10 )
+    : m_host( host ), m_ppEntries( 0 ), m_nCacheCapacity( 10 )
 {
     return;
 }
@@ -392,10 +392,10 @@ s_NofTokens( const string& s )
 COrgMod::ESubtype
 COrgRefCache::GetSubtypeFromName( string& sName )
 {
-    static const string s_sSubspCf( " subsp. cf." );
-    static const string s_sSubspAff( " subsp. aff." );
-    static const string s_sCf( " cf." );
-    static const string s_sAff( " aff." );
+    static const char* s_sSubspCf  = " subsp. cf.";
+    static const char* s_sSubspAff = " subsp. aff.";
+    static const char* s_sCf       = " cf.";
+    static const char* s_sAff      = " aff.";
 
     string::size_type pos;
     if( sName.find('.') == string::npos ) {
@@ -761,6 +761,25 @@ COrgRefCache::BuildOrgRef( CTaxon1Node& node, COrg_ref& org, bool& is_species )
         }
     } else
         return false;
+
+    CRef<CTaxon1_info> pProp( new CTaxon1_info() );
+    pProp->SetIval1( node.GetTaxId() );
+    pProp->SetIval2( -2 ); // Get int property by name
+    pProp->SetSval( "pgcode" );
+
+    req.SetGetorgprop( *pProp );
+    try {
+	if( m_host.SendRequest( req, resp ) ) {
+	    if( resp.IsGetorgprop() ) { 
+		if( resp.GetGetorgprop().size() > 0 ) {
+		    CRef<CTaxon1_info> pInfo
+			= resp.GetGetorgprop().front();
+		    org.SetOrgname().SetPgcode( pInfo->GetIval2() );
+		}
+	    }
+	}
+    } catch( exception& e ) {
+    }
     return true;
 }
 
