@@ -1,4 +1,4 @@
-/*  $Id: data_source.cpp 366487 2012-06-14 16:53:31Z rafanovi $
+/*  $Id: data_source.cpp 373231 2012-08-27 20:52:58Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1412,6 +1412,9 @@ void CDataSource::GetSequenceTypes(const TIds& ids, TLoaded& loaded,
 
 void CDataSource::GetBlobs(TSeqMatchMap& match_map)
 {
+    if ( match_map.empty() ) {
+        return;
+    }
     if ( m_Loader ) {
         CDataLoader::TTSE_LockSets tse_sets;
         ITERATE(TSeqMatchMap, match, match_map) {
@@ -1631,8 +1634,9 @@ NCBI_PARAM_DEF_EX(unsigned, OBJMGR, BLOB_CACHE, 10,
 
 static unsigned s_GetCacheSize(void)
 {
-    static NCBI_PARAM_TYPE(OBJMGR, BLOB_CACHE) sx_Value;
-    return sx_Value.Get();
+    static const unsigned sx_Value =
+        NCBI_PARAM_TYPE(OBJMGR, BLOB_CACHE)::GetDefault();
+    return sx_Value;
 }
 
 
