@@ -1,4 +1,4 @@
-/*  $Id: scope_info.cpp 345676 2011-11-29 17:10:54Z vasilche $
+/*  $Id: scope_info.cpp 387351 2013-01-29 00:07:32Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -69,8 +69,9 @@ NCBI_PARAM_DEF_EX(bool, OBJMGR, SCOPE_AUTORELEASE, true,
 
 static bool s_GetScopeAutoReleaseEnabled(void)
 {
-    static NCBI_PARAM_TYPE(OBJMGR, SCOPE_AUTORELEASE) sx_Value;
-    return sx_Value.Get();
+    static const bool sx_Value =
+        NCBI_PARAM_TYPE(OBJMGR, SCOPE_AUTORELEASE)::GetDefault();
+    return sx_Value;
 }
 
 
@@ -80,8 +81,9 @@ NCBI_PARAM_DEF_EX(unsigned, OBJMGR, SCOPE_AUTORELEASE_SIZE, 10,
 
 static unsigned s_GetScopeAutoReleaseSize(void)
 {
-    static NCBI_PARAM_TYPE(OBJMGR, SCOPE_AUTORELEASE_SIZE) sx_Value;
-    return sx_Value.Get();
+    static const unsigned sx_Value =
+        NCBI_PARAM_TYPE(OBJMGR, SCOPE_AUTORELEASE_SIZE)::GetDefault();
+    return sx_Value;
 }
 
 
@@ -554,7 +556,9 @@ SSeqMatch_Scope CDataSource_ScopeInfo::x_GetSeqMatch(const CSeq_id_Handle& idh)
                 continue;
             if ( ret && ret.m_Seq_id.IsBetter(*it) ) // worse hit
                 continue;
-            ret = x_FindBestTSE(*it);
+            if ( SSeqMatch_Scope match = x_FindBestTSE(*it) ) {
+                ret = match;
+            }
         }
     }
     return ret;
@@ -647,6 +651,9 @@ void CDataSource_ScopeInfo::GetBlobs(TSeqMatchMap& match_map)
         ds_match_map.insert(CDataSource::TSeqMatchMap::value_type(
             it->first, SSeqMatch_DS()));
     }
+    if ( match_map.empty() ) {
+        return;
+    }
     GetDataSource().GetBlobs(ds_match_map);
     ITERATE(CDataSource::TSeqMatchMap, ds_match, ds_match_map) {
         if ( !ds_match->second ) {
@@ -1055,11 +1062,12 @@ bool CTSE_ScopeInfo::ContainsBioseq(const CSeq_id_Handle& id) const
 }
 
 
-bool CTSE_ScopeInfo::ContainsMatchingBioseq(const CSeq_id_Handle& id) const
+CSeq_id_Handle
+CTSE_ScopeInfo::ContainsMatchingBioseq(const CSeq_id_Handle& id) const
 {
     if ( CanBeUnloaded() ) {
         if ( ContainsBioseq(id) ) {
-            return true;
+            return id;
         }
         if ( id.HaveMatchingHandles() ) {
             CSeq_id_Handle::TMatches ids;
@@ -1067,12 +1075,12 @@ bool CTSE_ScopeInfo::ContainsMatchingBioseq(const CSeq_id_Handle& id) const
             ITERATE ( CSeq_id_Handle::TMatches, it, ids ) {
                 if ( *it != id ) {
                     if ( ContainsBioseq(*it) ) {
-                        return true;
+                        return *it;
                     }
                 }
             }
         }
-        return false;
+        return null;
     }
     else {
         return m_TSE_Lock->ContainsMatchingBioseq(id);
