@@ -1,4 +1,4 @@
-/*  $Id: seq_annot_info.cpp 369613 2012-07-20 18:37:00Z rafanovi $
+/*  $Id: seq_annot_info.cpp 386408 2013-01-17 21:29:50Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -65,7 +65,7 @@
 
 BEGIN_NCBI_SCOPE
 
-NCBI_DEFINE_ERR_SUBCODE_X(7);
+NCBI_DEFINE_ERR_SUBCODE_X(9);
 
 BEGIN_SCOPE(objects)
 
@@ -266,6 +266,22 @@ void CSeq_annot_Info::x_UpdateName(void)
 }
 
 
+string CSeq_annot_Info::GetDescription(void) const
+{
+    string ret;
+    if ( GetName().IsNamed() ) {
+        ret = GetName().GetName();
+    }
+    if ( HasTSE_Info() ) {
+        if ( !ret.empty() ) {
+            ret += '|';
+        }
+        ret += GetTSE_Info().GetDescription();
+    }
+    return ret;
+}
+
+
 CSeq_annot_Info::TAnnotIndex
 CSeq_annot_Info::x_GetSNPFeatCount(void) const
 {
@@ -653,7 +669,7 @@ void CSeq_annot_Info::x_Map(const CTSEAnnotObjectMapper& mapper,
         else {
             s << "unknown object";
         }
-        ERR_POST_X(6, "Failed to parse location of "<<s.rdbuf());
+        ERR_POST_X(6, "Failed to parse location of "<<s.rdbuf()<<" in "<<GetDescription());
         return;
     }
     mapper.Map(key, index);
@@ -719,9 +735,8 @@ void CSeq_annot_Info::x_InitFeatKeys(CTSE_Info& tse)
                 const CHandleRange& hr = hrit->second;
                 key.m_Range = hr.GetOverlappingRange();
                 if ( key.m_Range.Empty() ) {
-                    CNcbiOstrstream s;
-                    s << MSerial_AsnText << *info.GetFeatFast();
-                    ERR_POST_X(1, "Empty region in "<<s.rdbuf());
+                    ERR_POST_X(1, "Empty region in "<<GetDescription()<<" "<<
+                               MSerial_AsnText<<*info.GetFeatFast());
                     continue;
                 }
                 key.m_Handle = hrit->first;
@@ -784,9 +799,8 @@ void CSeq_annot_Info::x_InitGraphKeys(CTSE_Info& tse)
                 const CHandleRange& hr = hrit->second;
                 key.m_Range = hr.GetOverlappingRange();
                 if ( key.m_Range.Empty() ) {
-                    CNcbiOstrstream s;
-                    s << MSerial_AsnText << *info.GetGraphFast();
-                    ERR_POST_X(2, "Empty region in "<<s.rdbuf());
+                    ERR_POST_X(2, "Empty region in "<<GetDescription()<<" "<<
+                               MSerial_AsnText<<*info.GetGraphFast());
                     continue;
                 }
                 key.m_Handle = hrit->first;
@@ -837,9 +851,8 @@ void CSeq_annot_Info::x_InitAlignKeys(CTSE_Info& tse)
                 const CHandleRange& hr = hrit->second;
                 key.m_Range = hr.GetOverlappingRange();
                 if ( key.m_Range.Empty() ) {
-                    CNcbiOstrstream s;
-                    s << MSerial_AsnText << info.GetAlign();
-                    ERR_POST_X(3, "Empty region in "<<s.rdbuf());
+                    ERR_POST_X(3, "Empty region in "<<GetDescription()<<" "<<
+                               MSerial_AsnText<<info.GetAlign());
                     continue;
                 }
                 key.m_Handle = hrit->first;
@@ -891,9 +904,8 @@ void CSeq_annot_Info::x_InitLocsKeys(CTSE_Info& tse)
             const CHandleRange& hr = hrit->second;
             key.m_Range = hr.GetOverlappingRange();
             if ( key.m_Range.Empty() ) {
-                CNcbiOstrstream s;
-                s << MSerial_AsnText << info.GetLocs();
-                ERR_POST_X(4, "Empty region in "<<s.rdbuf());
+                ERR_POST_X(4, "Empty region in "<<GetDescription()<<" "<<
+                           MSerial_AsnText<<info.GetLocs());
                 continue;
             }
             key.m_Handle = hrit->first;
@@ -971,12 +983,7 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
         size_t keys_begin = m_ObjectIndex.GetKeys().size();
         index.m_AnnotObject_Info = &info;
         index.m_AnnotLocationIndex = 0;
-        CConstRef<CSeq_loc> loc;
-        try {
-            loc = m_Table_Info->GetColumn("Seq-table location").GetSeq_loc(0);
-        }
-        catch ( exception& /*ignored*/ ) {
-        }
+        CConstRef<CSeq_loc> loc = m_Table_Info->GetTableLocation();
         if ( !loc ) {
             return;
         }
@@ -986,9 +993,8 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
             const CHandleRange& hr = hrit->second;
             key.m_Range = hr.GetOverlappingRange();
             if ( key.m_Range.Empty() ) {
-                CNcbiOstrstream s;
-                s << MSerial_AsnText << *info.GetFeatFast();
-                ERR_POST_X(7, "Empty region in "<<s.rdbuf());
+                ERR_POST_X(7, "Empty region in "<<GetDescription()<<" "<<
+                           MSerial_AsnText<<*info.GetFeatFast());
                 continue;
             }
             key.m_Handle = hrit->first;
@@ -1030,6 +1036,9 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
         if ( info.IsRemoved() ) {
             continue;
         }
+        if ( m_Table_Info->RowIsDisabled(row) ) {
+            continue;
+        }
         size_t keys_begin = m_ObjectIndex.GetKeys().size();
         index.m_AnnotObject_Info = &info;
         bool partial = m_Table_Info->IsPartial(row);
@@ -1051,9 +1060,8 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
                         const CHandleRange& hr = hrit->second;
                         key.m_Range = hr.GetOverlappingRange();
                         if ( key.m_Range.Empty() ) {
-                            CNcbiOstrstream s;
-                            s << MSerial_AsnText << *info.GetFeatFast();
-                            ERR_POST_X(7, "Empty region in "<<s.rdbuf());
+                            ERR_POST_X(8, "Empty region in "<<GetDescription()<<" "<<
+                                       MSerial_AsnText<<*info.GetFeatFast());
                             continue;
                         }
                         key.m_Handle = hrit->first;
@@ -1093,20 +1101,20 @@ void CSeq_annot_Info::x_InitFeatTableKeys(CTSE_Info& tse)
 
 void CSeq_annot_Info::x_MapFeatById(const CFeat_id& id,
                                     CAnnotObject_Info& info,
-                                    bool xref)
+                                    EFeatIdType id_type)
 {
     if ( id.IsLocal() ) {
-        GetTSE_Info().x_MapFeatById(id.GetLocal(), info, xref);
+        GetTSE_Info().x_MapFeatById(id.GetLocal(), info, id_type);
     }
 }
 
 
 void CSeq_annot_Info::x_UnmapFeatById(const CFeat_id& id,
                                       CAnnotObject_Info& info,
-                                      bool xref)
+                                      EFeatIdType id_type)
 {
     if ( id.IsLocal() ) {
-        GetTSE_Info().x_UnmapFeatById(id.GetLocal(), info, xref);
+        GetTSE_Info().x_UnmapFeatById(id.GetLocal(), info, id_type);
     }
 }
 
@@ -1145,11 +1153,11 @@ void CSeq_annot_Info::x_MapFeatIds(CAnnotObject_Info& info)
 {
     const CSeq_feat& feat = *info.GetFeatFast();
     if ( feat.IsSetId() ) {
-        x_MapFeatById(feat.GetId(), info, true);
+        x_MapFeatById(feat.GetId(), info, eFeatId_id);
     }
     if ( feat.IsSetIds() ) {
         ITERATE ( CSeq_feat::TIds, it, feat.GetIds() ) {
-            x_MapFeatById(**it, info, true);
+            x_MapFeatById(**it, info, eFeatId_id);
         }
     }
     if ( info.GetFeatType() == CSeqFeatData::e_Gene ) {
@@ -1159,7 +1167,7 @@ void CSeq_annot_Info::x_MapFeatIds(CAnnotObject_Info& info)
         ITERATE ( CSeq_feat::TXref, it, feat.GetXref() ) {
             const CSeqFeatXref& xref = **it;
             if ( xref.IsSetId() ) {
-                x_MapFeatById(xref.GetId(), info, false);
+                x_MapFeatById(xref.GetId(), info, eFeatId_xref);
             }
         }
     }
@@ -1170,11 +1178,11 @@ void CSeq_annot_Info::x_UnmapFeatIds(CAnnotObject_Info& info)
 {
     const CSeq_feat& feat = *info.GetFeatFast();
     if ( feat.IsSetId() ) {
-        x_UnmapFeatById(feat.GetId(), info, true);
+        x_UnmapFeatById(feat.GetId(), info, eFeatId_id);
     }
     if ( feat.IsSetIds() ) {
         ITERATE ( CSeq_feat::TIds, it, feat.GetIds() ) {
-            x_UnmapFeatById(**it, info, true);
+            x_UnmapFeatById(**it, info, eFeatId_id);
         }
     }
     if ( info.GetFeatType() == CSeqFeatData::e_Gene ) {
@@ -1184,7 +1192,7 @@ void CSeq_annot_Info::x_UnmapFeatIds(CAnnotObject_Info& info)
         ITERATE ( CSeq_feat::TXref, it, feat.GetXref() ) {
             const CSeqFeatXref& xref = **it;
             if ( xref.IsSetId() ) {
-                x_UnmapFeatById(xref.GetId(), info, false);
+                x_UnmapFeatById(xref.GetId(), info, eFeatId_xref);
             }
         }
     }
@@ -1230,7 +1238,7 @@ void CSeq_annot_Info::x_MapAnnotObject(CAnnotObject_Info& info)
                 else {
                     s << "unknown annotation";
                 }
-                ERR_POST_X(5, "Empty region in "<<s.rdbuf());
+                ERR_POST_X(5, "Empty region in "<<GetDescription()<<" "<<s.rdbuf());
                 continue;
             }
             key.m_Handle = hrit->first;
@@ -1302,7 +1310,7 @@ void CSeq_annot_Info::x_RemapAnnotObject(CAnnotObject_Info& info)
                 else {
                     s << "unknown annotation";
                 }
-                ERR_POST_X(6, "Empty region in "<<s.rdbuf());
+                ERR_POST_X(9, "Empty region in "<<GetDescription()<<" "<<s.rdbuf());
                 continue;
             }
             key.m_Handle = hrit->first;
@@ -1696,5 +1704,131 @@ void CSeq_annot_Info::ReorderFtable(const vector<CSeq_feat_Handle>& feats)
 }
 
 
+void CSeq_annot_Info::AddFeatId(TAnnotIndex index,
+                                const CObject_id& id,
+                                EFeatIdType id_type)
+{
+    _ASSERT(size_t(index) < GetAnnotObjectInfos().size());
+    CAnnotObject_Info& info = m_ObjectIndex.GetInfos()[index];
+    _ASSERT(info.IsRegular());
+    _ASSERT(&info.GetSeq_annot_Info() == this);
+    CRef<CSeq_feat> feat(&const_cast<CSeq_feat&>(info.GetFeat()));
+    GetTSE_Info().x_MapFeatById(id, info, id_type);
+    CRef<CFeat_id> feat_id(new CFeat_id);
+    feat_id->SetLocal().Assign(id);
+    if ( id_type == eFeatId_xref ) {
+        CRef<CSeqFeatXref> feat_xref(new CSeqFeatXref);
+        feat_xref->SetId(*feat_id);
+        feat->SetXref().push_back(feat_xref);
+    }
+    else if ( !feat->IsSetId() ) {
+        feat->SetId(*feat_id);
+    }
+    else {
+        feat->SetIds().push_back(feat_id);
+    }
+}
+
+
+void CSeq_annot_Info::RemoveFeatId(TAnnotIndex index,
+                                   const CObject_id& id,
+                                   EFeatIdType id_type)
+{
+    _ASSERT(size_t(index) < GetAnnotObjectInfos().size());
+    CAnnotObject_Info& info = m_ObjectIndex.GetInfos()[index];
+    _ASSERT(info.IsRegular());
+    _ASSERT(&info.GetSeq_annot_Info() == this);
+    CRef<CSeq_feat> feat(&const_cast<CSeq_feat&>(info.GetFeat()));
+    bool removed_id = false;
+    if ( id_type == eFeatId_xref ) {
+        if ( feat->IsSetXref() ) {
+            NON_CONST_ITERATE ( CSeq_feat::TXref, it, feat->SetXref() ) {
+                const CSeqFeatXref& feat_xref = **it;
+                if ( !feat_xref.IsSetId() ) {
+                    continue;
+                }
+                const CFeat_id& feat_id = feat_xref.GetId();
+                if ( feat_id.IsLocal() && feat_id.GetLocal().Equals(id) ) {
+                    feat->SetXref().erase(it);
+                    if ( feat->SetXref().empty() ) {
+                        feat->ResetXref();
+                    }
+                    removed_id = true;
+                    break;
+                }
+            }
+        }
+    }
+    else {
+        if ( feat->IsSetId() && feat->GetId().IsLocal() &&
+             feat->GetId().GetLocal().Equals(id) ) {
+            feat->ResetId();
+            removed_id = true;
+        }
+        else if ( feat->IsSetIds() ) {
+            NON_CONST_ITERATE ( CSeq_feat::TIds, it, feat->SetIds() ) {
+                const CFeat_id& feat_id = **it;
+                if ( feat_id.IsLocal() && feat_id.GetLocal().Equals(id) ) {
+                    feat->SetIds().erase(it);
+                    if ( feat->SetIds().empty() ) {
+                        feat->ResetIds();
+                    }
+                    removed_id = true;
+                    break;
+                }
+            }
+        }
+    }
+    if ( !removed_id ) {
+        NCBI_THROW(CAnnotException, eFindFailed,
+                   "CSeq_feat_EditHandle::RemoveFeatId: Feat-id not found");
+    }
+    GetTSE_Info().x_UnmapFeatById(id, info, id_type);
+}
+
+
+void CSeq_annot_Info::ClearFeatIds(TAnnotIndex index,
+                                   EFeatIdType id_type)
+{
+    _ASSERT(size_t(index) < GetAnnotObjectInfos().size());
+    CAnnotObject_Info& info = m_ObjectIndex.GetInfos()[index];
+    _ASSERT(info.IsRegular());
+    _ASSERT(&info.GetSeq_annot_Info() == this);
+    CRef<CSeq_feat> feat(&const_cast<CSeq_feat&>(info.GetFeat()));
+    if ( id_type == eFeatId_xref ) {
+        if ( feat->IsSetXref() ) {
+            ERASE_ITERATE ( CSeq_feat::TXref, it, feat->SetXref() ) {
+                const CSeqFeatXref& feat_xref = **it;
+                if ( !feat_xref.IsSetId() ) {
+                    continue;
+                }
+                const CFeat_id& feat_id = feat_xref.GetId();
+                if ( feat_id.IsLocal() ) {
+                    GetTSE_Info().x_UnmapFeatById(feat_id.GetLocal(), info, id_type);
+                    VECTOR_ERASE(it, feat->SetXref());
+                }
+            }
+            feat->ResetXref();
+        }
+    }
+    else {
+        if ( feat->IsSetId() && feat->GetId().IsLocal() ) {
+            GetTSE_Info().x_UnmapFeatById(feat->GetId().GetLocal(), info, id_type);
+            feat->ResetId();
+        }
+        else if ( feat->IsSetIds() ) {
+            ERASE_ITERATE ( CSeq_feat::TIds, it, feat->SetIds() ) {
+                const CFeat_id& feat_id = **it;
+                if ( feat_id.IsLocal() ) {
+                    GetTSE_Info().x_UnmapFeatById(feat_id.GetLocal(), info, id_type);
+                    VECTOR_ERASE(it, feat->SetIds());
+                }
+            }
+            feat->ResetIds();
+        }
+    }
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
