@@ -1,4 +1,4 @@
-/*  $Id: seq_feat_handle.cpp 368318 2012-07-05 21:38:07Z rafanovi $
+/*  $Id: seq_feat_handle.cpp 382535 2012-12-06 19:21:37Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,7 @@
 #include <objmgr/impl/seq_annot_edit_commands.hpp>
 
 #include <objects/general/Dbtag.hpp>
+#include <objects/general/Object_id.hpp>
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
@@ -514,6 +515,153 @@ void CSeq_feat_EditHandle::AddDbxref(const string& db_name, int db_key)
 }
 
 
+void CSeq_feat_EditHandle::AddFeatId(const CObject_id& id)
+{
+    if ( !IsPlainFeat() ) {
+        NCBI_THROW(CObjMgrException, eInvalidHandle,
+                   "CSeq_feat_EditHandle::AddFeatId: not plain Seq-feat");
+    }
+    GetAnnot().x_GetInfo().AddFeatId(x_GetFeatIndex(), id, eFeatId_id);
+}
+
+
+void CSeq_feat_EditHandle::AddFeatId(int id)
+{
+    CObject_id obj_id;
+    obj_id.SetId(id);
+    AddFeatId(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::AddFeatId(const string& id)
+{
+    CObject_id obj_id;
+    obj_id.SetStr(id);
+    AddFeatId(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::AddFeatXref(const CObject_id& id)
+{
+    if ( !IsPlainFeat() ) {
+        NCBI_THROW(CObjMgrException, eInvalidHandle,
+                   "CSeq_feat_EditHandle::AddFeatXref: not plain Seq-feat");
+    }
+    GetAnnot().x_GetInfo().AddFeatId(x_GetFeatIndex(), id, eFeatId_xref);
+}
+
+
+void CSeq_feat_EditHandle::AddFeatXref(int id)
+{
+    CObject_id obj_id;
+    obj_id.SetId(id);
+    AddFeatXref(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::AddFeatXref(const string& id)
+{
+    CObject_id obj_id;
+    obj_id.SetStr(id);
+    AddFeatXref(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::RemoveFeatId(const CObject_id& id)
+{
+    if ( !IsPlainFeat() ) {
+        NCBI_THROW(CObjMgrException, eInvalidHandle,
+                   "CSeq_feat_EditHandle::RemoveFeatId: not plain Seq-feat");
+    }
+    GetAnnot().x_GetInfo().RemoveFeatId(x_GetFeatIndex(), id, eFeatId_id);
+}
+
+
+void CSeq_feat_EditHandle::RemoveFeatId(int id)
+{
+    CObject_id obj_id;
+    obj_id.SetId(id);
+    RemoveFeatId(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::RemoveFeatId(const string& id)
+{
+    CObject_id obj_id;
+    obj_id.SetStr(id);
+    RemoveFeatId(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::RemoveFeatXref(const CObject_id& id)
+{
+    if ( !IsPlainFeat() ) {
+        NCBI_THROW(CObjMgrException, eInvalidHandle,
+                   "CSeq_feat_EditHandle::RemoveFeatXref: not plain Seq-feat");
+    }
+    GetAnnot().x_GetInfo().RemoveFeatId(x_GetFeatIndex(), id, eFeatId_xref);
+}
+
+
+void CSeq_feat_EditHandle::RemoveFeatXref(int id)
+{
+    CObject_id obj_id;
+    obj_id.SetId(id);
+    RemoveFeatXref(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::RemoveFeatXref(const string& id)
+{
+    CObject_id obj_id;
+    obj_id.SetStr(id);
+    RemoveFeatXref(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::ClearFeatIds(void)
+{
+    if ( !IsPlainFeat() ) {
+        NCBI_THROW(CObjMgrException, eInvalidHandle,
+                   "CSeq_feat_EditHandle::ClearFeatIds: not plain Seq-feat");
+    }
+    GetAnnot().x_GetInfo().ClearFeatIds(x_GetFeatIndex(), eFeatId_id);
+}
+
+
+void CSeq_feat_EditHandle::ClearFeatXrefs(void)
+{
+    if ( !IsPlainFeat() ) {
+        NCBI_THROW(CObjMgrException, eInvalidHandle,
+                   "CSeq_feat_EditHandle::ClearFeatXrefs: not plain Seq-feat");
+    }
+    GetAnnot().x_GetInfo().ClearFeatIds(x_GetFeatIndex(), eFeatId_xref);
+}
+
+
+void CSeq_feat_EditHandle::SetFeatId(const CObject_id& id)
+{
+    ClearFeatIds();
+    AddFeatId(id);
+}
+
+
+void CSeq_feat_EditHandle::SetFeatId(int id)
+{
+    CObject_id obj_id;
+    obj_id.SetId(id);
+    SetFeatId(obj_id);
+}
+
+
+void CSeq_feat_EditHandle::SetFeatId(const string& id)
+{
+    CObject_id obj_id;
+    obj_id.SetStr(id);
+    SetFeatId(obj_id);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // CSeq_annot_ftable_CI
 
