@@ -1,4 +1,4 @@
-/*  $Id: seq_table_info.cpp 363670 2012-05-17 21:56:29Z rafanovi $
+/*  $Id: seq_table_info.cpp 386408 2013-01-17 21:29:50Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -906,6 +906,14 @@ void CSeqTableInfo::x_Initialize(const CSeq_table& feat_table)
                 m_Partial = CSeqTableColumnInfo(col);
                 continue;
             }
+            else if ( name == "disabled" ) {
+                if ( m_Disabled ) {
+                    NCBI_THROW_FMT(CAnnotException, eOtherError,
+                                   "Duplicate disabled column");
+                }
+                m_Disabled = CSeqTableColumnInfo(col);
+                continue;
+            }
             if ( !setter ) {
                 try {
                     setter = new CSeqTableSetAnyFeatField(name);
@@ -932,10 +940,14 @@ CSeqTableInfo::~CSeqTableInfo()
 }
 
 
-bool CSeqTableInfo::IsPartial(size_t row) const
+CConstRef<CSeq_loc> CSeqTableInfo::GetTableLocation(void) const
 {
-    bool val = false;
-    return m_Partial && m_Partial.GetBool(row, val) && val;
+    try {
+        return GetColumn("Seq-table location").GetSeq_loc(0);
+    }
+    catch ( exception& /*ignored*/ ) {
+        return null;
+    }
 }
 
 
