@@ -1,4 +1,4 @@
-/*  $Id: id_range.cpp 252199 2011-02-14 14:11:26Z vasilche $
+/*  $Id: id_range.cpp 386408 2013-01-17 21:29:50Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -384,12 +384,7 @@ void CSeqsRange::Add(const CSeq_table& table, const CBlobSplitterImpl& impl)
         Add(info->GetProduct(), table, impl);
     }
     else {
-        CConstRef<CSeq_loc> loc;
-        try {
-            loc = info->GetColumn("Seq-table location").GetSeq_loc(0);
-        }
-        catch ( CException& /*ignored*/ ) {
-        }
+        CConstRef<CSeq_loc> loc = info->GetTableLocation();
         if ( loc ) {
             Add(*loc, impl);
         }
