@@ -1,4 +1,4 @@
-/*  $Id: tse_chunk_info.cpp 200122 2010-08-04 17:57:31Z vasilche $
+/*  $Id: tse_chunk_info.cpp 382535 2012-12-06 19:21:37Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,7 @@
 #include <objmgr/impl/bioseq_set_info.hpp>
 #include <objmgr/impl/data_source.hpp>
 #include <objmgr/impl/annot_object.hpp>
+#include <objmgr/impl/annot_type_index.hpp>
 #include <objects/seq/Seq_literal.hpp>
 #include <objmgr/seq_map.hpp>
 #include <algorithm>
@@ -498,6 +499,136 @@ void CTSE_Chunk_Info::x_InitObjectIndexList(void)
 }
 
 
+static
+bool x_HasFeatType(const CTSE_Chunk_Info::TAnnotTypes& types,
+                   CSeqFeatData::E_Choice type)
+{
+    if ( type == CSeqFeatData::e_not_set ) {
+        return !types.empty();
+    }
+    if ( types.find(SAnnotTypeSelector(type)) != types.end() ) {
+        return true;
+    }
+    CAnnotType_Index::TIndexRange range =
+        CAnnotType_Index::GetFeatTypeRange(type);
+    for ( size_t index = range.first; index < range.second; ++index ) {
+        CSeqFeatData::ESubtype subtype =
+            CAnnotType_Index::GetSubtypeForIndex(index);
+        if ( types.find(SAnnotTypeSelector(subtype)) != types.end() ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+static
+bool x_HasFeatType(const CTSE_Chunk_Info::TAnnotTypes& types,
+                   CSeqFeatData::ESubtype subtype)
+{
+    if ( subtype == CSeqFeatData::eSubtype_any ) {
+        return !types.empty();
+    }
+    if ( types.find(SAnnotTypeSelector(subtype)) != types.end() ) {
+        return true;
+    }
+    CSeqFeatData::E_Choice type = CSeqFeatData::GetTypeFromSubtype(subtype);
+    if ( types.find(SAnnotTypeSelector(type)) != types.end() ) {
+        return true;
+    }
+    return false;
+}
+
+
+static
+bool x_HasFeatIds(const CTSE_Chunk_Info::TFeatIdsMap& types,
+                  CSeqFeatData::E_Choice type)
+{
+    if ( type == CSeqFeatData::e_not_set ) {
+        return !types.empty();
+    }
+    if ( types.find(SAnnotTypeSelector(type)) != types.end() ) {
+        return true;
+    }
+    CAnnotType_Index::TIndexRange range =
+        CAnnotType_Index::GetFeatTypeRange(type);
+    for ( size_t index = range.first; index < range.second; ++index ) {
+        CSeqFeatData::ESubtype subtype =
+            CAnnotType_Index::GetSubtypeForIndex(index);
+        if ( types.find(SAnnotTypeSelector(subtype)) != types.end() ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+static
+bool x_HasFeatIds(const CTSE_Chunk_Info::TFeatIdsMap& types,
+                  CSeqFeatData::ESubtype subtype)
+{
+    if ( subtype == CSeqFeatData::eSubtype_any ) {
+        return !types.empty();
+    }
+    if ( types.find(SAnnotTypeSelector(subtype)) != types.end() ) {
+        return true;
+    }
+    CSeqFeatData::E_Choice type = CSeqFeatData::GetTypeFromSubtype(subtype);
+    if ( types.find(SAnnotTypeSelector(type)) != types.end() ) {
+        return true;
+    }
+    return false;
+}
+
+
+bool CTSE_Chunk_Info::x_ContainsFeatType(CSeqFeatData::E_Choice type) const
+{
+    ITERATE ( TAnnotContents, it, m_AnnotContents ) {
+        if ( x_HasFeatType(it->second, type) ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+bool CTSE_Chunk_Info::x_ContainsFeatType(CSeqFeatData::ESubtype subtype) const
+{
+    ITERATE ( TAnnotContents, it, m_AnnotContents ) {
+        if ( x_HasFeatType(it->second, subtype) ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
+bool CTSE_Chunk_Info::x_ContainsFeatIds(CSeqFeatData::E_Choice type,
+                                        EFeatIdType id_type) const
+{
+    if ( !x_ContainsFeatType(type) ) {
+        return false;
+    }
+    if ( !m_ExplicitFeatIds ) {
+        return true;
+    }
+    return x_HasFeatIds(id_type == eFeatId_id? m_FeatIds: m_XrefIds, type);
+}
+
+
+bool CTSE_Chunk_Info::x_ContainsFeatIds(CSeqFeatData::ESubtype subtype,
+                                        EFeatIdType id_type) const
+{
+    if ( !x_ContainsFeatType(subtype) ) {
+        return false;
+    }
+    if ( !m_ExplicitFeatIds ) {
+        return true;
+    }
+    return x_HasFeatIds(id_type == eFeatId_id? m_FeatIds: m_XrefIds, subtype);
+}
+
+
 void CTSE_Chunk_Info::x_UpdateAnnotIndexContents(CTSE_Info& tse)
 {
     x_InitObjectIndexList();
@@ -522,18 +653,18 @@ void CTSE_Chunk_Info::x_UpdateAnnotIndexContents(CTSE_Info& tse)
     if ( m_ExplicitFeatIds ) {
         ITERATE ( TFeatIdsMap, it, m_FeatIds ) {
             ITERATE ( TFeatIdIntList, it2, it->second.m_IntList ) {
-                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), false);
+                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), eFeatId_id);
             }
             ITERATE ( TFeatIdStrList, it2, it->second.m_StrList ) {
-                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), false);
+                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), eFeatId_id);
             }
         }
         ITERATE ( TFeatIdsMap, it, m_XrefIds ) {
             ITERATE ( TFeatIdIntList, it2, it->second.m_IntList ) {
-                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), true);
+                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), eFeatId_xref);
             }
             ITERATE ( TFeatIdStrList, it2, it->second.m_StrList ) {
-                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), true);
+                tse.x_MapChunkByFeatId(*it2, it->first, GetChunkId(), eFeatId_xref);
             }
         }
     }
