@@ -1,4 +1,4 @@
-/*  $Id: tse_info.cpp 370666 2012-07-31 20:32:41Z vasilche $
+/*  $Id: tse_info.cpp 387293 2013-01-28 20:43:07Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -680,10 +680,11 @@ bool CTSE_Info::ContainsBioseq(const CSeq_id_Handle& id) const
 }
 
 
-bool CTSE_Info::ContainsMatchingBioseq(const CSeq_id_Handle& id) const
+CSeq_id_Handle
+CTSE_Info::ContainsMatchingBioseq(const CSeq_id_Handle& id) const
 {
     if ( ContainsBioseq(id) ) {
-        return true;
+        return id;
     }
     else if ( id.HaveMatchingHandles() ) {
         CSeq_id_Handle::TMatches ids;
@@ -691,12 +692,12 @@ bool CTSE_Info::ContainsMatchingBioseq(const CSeq_id_Handle& id) const
         ITERATE ( CSeq_id_Handle::TMatches, match_it, ids ) {
             if ( *match_it != id ) {
                 if ( ContainsBioseq(*match_it) ) {
-                    return true;
+                    return *match_it;
                 }
             }
         }
     }
-    return false;
+    return null;
 }
 
 
@@ -845,6 +846,9 @@ void CTSE_Info::x_SetBioseqIds(CBioseq_Info* info)
                            "\n  seq2: " + info->IdString());
             }
         }
+        if ( m_BioseqUpdater ) {
+            m_BioseqUpdater->Update(*info);
+        }
     }}
     // register this TSE in data source as containing the sequence
     if ( HasDataSource() ) {
@@ -924,6 +928,26 @@ void CTSE_Info::x_ResetDirtyAnnotIndexNoParent(void)
 }
 
 
+void CTSE_Info::UpdateFeatIdIndex(CSeqFeatData::E_Choice type,
+                                  EFeatIdType id_type) const
+{
+    if ( m_Split ) {
+        m_Split.GetNCObject().x_UpdateFeatIdIndex(type, id_type);
+    }
+    UpdateAnnotIndex();
+}
+
+
+void CTSE_Info::UpdateFeatIdIndex(CSeqFeatData::ESubtype subtype,
+                                  EFeatIdType id_type) const
+{
+    if ( m_Split ) {
+        m_Split.GetNCObject().x_UpdateFeatIdIndex(subtype, id_type);
+    }
+    UpdateAnnotIndex();
+}
+
+
 void CTSE_Info::UpdateAnnotIndex(const CSeq_id_Handle& id) const
 {
     x_GetRecords(id, false);
@@ -1306,10 +1330,11 @@ bool CTSE_Info::x_NeedsDelayedMainChunk(void) const
 void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
                                   const SFeatIdIndex& index,
                                   TFeatIdInt id,
-                                  bool xref) const
+                                  EFeatIdType id_type) const
 {
     if ( !index.m_Chunks.empty() ) {
         x_LoadChunks(index.m_Chunks);
+        UpdateAnnotIndex();
     }
     if ( !index.m_IndexInt ) {
         return;
@@ -1318,9 +1343,10 @@ void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
     for ( SFeatIdIndex::TIndexInt::const_iterator iter2 = index2.find(id);
           iter2 != index2.end() && iter2->first == id; ++iter2 ) {
         const SFeatIdInfo& info = iter2->second;
-        if ( info.m_Xref == xref ) {
+        if ( info.m_Type == id_type ) {
             if ( info.m_IsChunk ) {
                 x_LoadChunk(info.m_ChunkId);
+                UpdateAnnotIndex();
             }
             else {
                 objects.push_back(info.m_Info);
@@ -1340,23 +1366,23 @@ bool CTSE_Info::x_HasFeaturesWithId(CSeqFeatData::ESubtype subtype) const
 void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
                                   CSeqFeatData::ESubtype subtype,
                                   TFeatIdInt id,
-                                  bool xref) const
+                                  EFeatIdType id_type) const
 {
     TFeatIdIndex::const_iterator iter = m_FeatIdIndex.find(subtype);
     if ( iter == m_FeatIdIndex.end() ) {
         return;
     }
-    x_AddFeaturesById(objects, iter->second, id, xref);
+    x_AddFeaturesById(objects, iter->second, id, id_type);
 }
 
 
 void CTSE_Info::x_AddAllFeaturesById(TAnnotObjects& objects,
                                      TFeatIdInt id,
-                                     bool xref) const
+                                     EFeatIdType id_type) const
 {
     //LOG_POST_X(1, this << ": ""x_AddAllFeaturesWithId: " << id);
     ITERATE ( TFeatIdIndex, iter, m_FeatIdIndex ) {
-        x_AddFeaturesById(objects, iter->second, id, xref);
+        x_AddFeaturesById(objects, iter->second, id, id_type);
     }
 }
 
@@ -1364,10 +1390,11 @@ void CTSE_Info::x_AddAllFeaturesById(TAnnotObjects& objects,
 void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
                                   const SFeatIdIndex& index,
                                   const TFeatIdStr& id,
-                                  bool xref) const
+                                  EFeatIdType id_type) const
 {
     if ( !index.m_Chunks.empty() ) {
         x_LoadChunks(index.m_Chunks);
+        UpdateAnnotIndex();
     }
     if ( !index.m_IndexStr ) {
         return;
@@ -1376,9 +1403,10 @@ void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
     for ( SFeatIdIndex::TIndexStr::const_iterator iter2 = index2.find(id);
           iter2 != index2.end() && iter2->first == id; ++iter2 ) {
         const SFeatIdInfo& info = iter2->second;
-        if ( info.m_Xref == xref ) {
+        if ( info.m_Type == id_type ) {
             if ( info.m_IsChunk ) {
                 x_LoadChunk(info.m_ChunkId);
+                UpdateAnnotIndex();
             }
             else {
                 objects.push_back(info.m_Info);
@@ -1391,23 +1419,23 @@ void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
 void CTSE_Info::x_AddFeaturesById(TAnnotObjects& objects,
                                   CSeqFeatData::ESubtype subtype,
                                   const TFeatIdStr& id,
-                                  bool xref) const
+                                  EFeatIdType id_type) const
 {
     TFeatIdIndex::const_iterator iter = m_FeatIdIndex.find(subtype);
     if ( iter == m_FeatIdIndex.end() ) {
         return;
     }
-    x_AddFeaturesById(objects, iter->second, id, xref);
+    x_AddFeaturesById(objects, iter->second, id, id_type);
 }
 
 
 void CTSE_Info::x_AddAllFeaturesById(TAnnotObjects& objects,
                                      const TFeatIdStr& id,
-                                     bool xref) const
+                                     EFeatIdType id_type) const
 {
     //LOG_POST_X(1, this << ": ""x_AddAllFeaturesWithId: " << id);
     ITERATE ( TFeatIdIndex, iter, m_FeatIdIndex ) {
-        x_AddFeaturesById(objects, iter->second, id, xref);
+        x_AddFeaturesById(objects, iter->second, id, id_type);
     }
 }
 
@@ -1415,7 +1443,7 @@ void CTSE_Info::x_AddAllFeaturesById(TAnnotObjects& objects,
 CTSE_Info::SFeatIdIndex::TIndexInt&
 CTSE_Info::x_GetFeatIdIndexInt(CSeqFeatData::ESubtype type)
 {
-    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << id << " " << xref);
+    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << type);
     SFeatIdIndex& index = m_FeatIdIndex[type];
     if ( !index.m_IndexInt ) {
         index.m_IndexInt.reset(new SFeatIdIndex::TIndexInt);
@@ -1427,7 +1455,7 @@ CTSE_Info::x_GetFeatIdIndexInt(CSeqFeatData::ESubtype type)
 CTSE_Info::SFeatIdIndex::TIndexStr&
 CTSE_Info::x_GetFeatIdIndexStr(CSeqFeatData::ESubtype type)
 {
-    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << id << " " << xref);
+    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << type);
     SFeatIdIndex& index = m_FeatIdIndex[type];
     if ( !index.m_IndexStr ) {
         index.m_IndexStr.reset(new SFeatIdIndex::TIndexStr);
@@ -1438,26 +1466,26 @@ CTSE_Info::x_GetFeatIdIndexStr(CSeqFeatData::ESubtype type)
 
 void CTSE_Info::x_MapFeatById(TFeatIdInt id,
                               CAnnotObject_Info& info,
-                              bool xref)
+                              EFeatIdType id_type)
 {
-    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << id << " " << xref);
+    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << id << " " << id_type<<" "<<&info);
     SFeatIdIndex::TIndexInt& index =
         x_GetFeatIdIndexInt(info.GetFeatSubtype());
-    SFeatIdIndex::TIndexInt::value_type value(id, SFeatIdInfo(xref, &info));
+    SFeatIdIndex::TIndexInt::value_type value(id, SFeatIdInfo(id_type, &info));
     index.insert(value);
 }
 
 
 void CTSE_Info::x_UnmapFeatById(TFeatIdInt id,
                                 CAnnotObject_Info& info,
-                                bool xref)
+                                EFeatIdType id_type)
 {
-    //LOG_POST_X(3, this << ": ""x_UnmapFeatById: " << id << " " << xref);
+    //LOG_POST_X(3, this << ": ""x_UnmapFeatById: " << id << " " << id_type<<" "<<&info);
     SFeatIdIndex::TIndexInt& index =
         x_GetFeatIdIndexInt(info.GetFeatSubtype());
     for ( SFeatIdIndex::TIndexInt::iterator iter = index.lower_bound(id);
           iter != index.end() && iter->first == id; ++iter ) {
-        if ( iter->second.m_Info == &info && iter->second.m_Xref == xref ) {
+        if ( iter->second.m_Info == &info && iter->second.m_Type == id_type ) {
             index.erase(iter);
             return;
         }
@@ -1468,26 +1496,26 @@ void CTSE_Info::x_UnmapFeatById(TFeatIdInt id,
 
 void CTSE_Info::x_MapFeatById(const TFeatIdStr& id,
                               CAnnotObject_Info& info,
-                              bool xref)
+                              EFeatIdType id_type)
 {
-    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << id << " " << xref);
+    //LOG_POST_X(2, this << ": ""x_MapFeatById: " << id << " " << id_type<<" "<<&info);
     SFeatIdIndex::TIndexStr& index =
         x_GetFeatIdIndexStr(info.GetFeatSubtype());
-    SFeatIdIndex::TIndexStr::value_type value(id, SFeatIdInfo(xref, &info));
+    SFeatIdIndex::TIndexStr::value_type value(id, SFeatIdInfo(id_type, &info));
     index.insert(value);
 }
 
 
 void CTSE_Info::x_UnmapFeatById(const TFeatIdStr& id,
                                 CAnnotObject_Info& info,
-                                bool xref)
+                                EFeatIdType id_type)
 {
-    //LOG_POST_X(3, this << ": ""x_UnmapFeatById: " << id << " " << xref);
+    //LOG_POST_X(3, this << ": ""x_UnmapFeatById: " << id << " " << id_type<<" "<<&info);
     SFeatIdIndex::TIndexStr& index =
         x_GetFeatIdIndexStr(info.GetFeatSubtype());
     for ( SFeatIdIndex::TIndexStr::iterator iter = index.lower_bound(id);
           iter != index.end() && iter->first == id; ++iter ) {
-        if ( iter->second.m_Info == &info && iter->second.m_Xref == xref ) {
+        if ( iter->second.m_Info == &info && iter->second.m_Type == id_type ) {
             index.erase(iter);
             return;
         }
@@ -1498,26 +1526,26 @@ void CTSE_Info::x_UnmapFeatById(const TFeatIdStr& id,
 
 void CTSE_Info::x_MapFeatById(const TFeatId& id,
                               CAnnotObject_Info& info,
-                              bool xref)
+                              EFeatIdType id_type)
 {
     if ( id.IsId() ) {
-        x_MapFeatById(id.GetId(), info, xref);
+        x_MapFeatById(id.GetId(), info, id_type);
     }
     else {
-        x_MapFeatById(id.GetStr(), info, xref);
+        x_MapFeatById(id.GetStr(), info, id_type);
     }
 }
 
 
 void CTSE_Info::x_UnmapFeatById(const TFeatId& id,
                                 CAnnotObject_Info& info,
-                                bool xref)
+                                EFeatIdType id_type)
 {
     if ( id.IsId() ) {
-        x_UnmapFeatById(id.GetId(), info, xref);
+        x_UnmapFeatById(id.GetId(), info, id_type);
     }
     else {
-        x_UnmapFeatById(id.GetStr(), info, xref);
+        x_UnmapFeatById(id.GetStr(), info, id_type);
     }
 }
 
@@ -1549,10 +1577,10 @@ void CTSE_Info::x_UnmapFeatByLocus(const string& locus, bool tag,
 void CTSE_Info::x_MapChunkByFeatId(TFeatIdInt id,
                                    CSeqFeatData::ESubtype subtype,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     SFeatIdIndex::TIndexInt& index = x_GetFeatIdIndexInt(subtype);
-    SFeatIdIndex::TIndexInt::value_type value(id, SFeatIdInfo(xref, chunk_id));
+    SFeatIdIndex::TIndexInt::value_type value(id, SFeatIdInfo(id_type, chunk_id));
     index.insert(value);
 }
 
@@ -1560,28 +1588,28 @@ void CTSE_Info::x_MapChunkByFeatId(TFeatIdInt id,
 void CTSE_Info::x_MapChunkByFeatId(TFeatIdInt id,
                                    CSeqFeatData::E_Choice type,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     CAnnotType_Index::TIndexRange range =
         CAnnotType_Index::GetFeatTypeRange(type);
     for ( size_t index = range.first; index < range.second; ++index ) {
         CSeqFeatData::ESubtype subtype =
             CAnnotType_Index::GetSubtypeForIndex(index);
-        x_MapChunkByFeatId(id, subtype, chunk_id, xref);
+        x_MapChunkByFeatId(id, subtype, chunk_id, id_type);
     }
 }
 
 
-void CTSE_Info::x_MapChunkByFeatId(TFeatIdInt feat_id,
+void CTSE_Info::x_MapChunkByFeatId(TFeatIdInt id,
                                    const SAnnotTypeSelector& type,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     if ( type.GetFeatSubtype() != CSeqFeatData::eSubtype_any ) {
-        x_MapChunkByFeatId(feat_id, type.GetFeatSubtype(), chunk_id, xref);
+        x_MapChunkByFeatId(id, type.GetFeatSubtype(), chunk_id, id_type);
     }
     else {
-        x_MapChunkByFeatId(feat_id, type.GetFeatType(), chunk_id, xref);
+        x_MapChunkByFeatId(id, type.GetFeatType(), chunk_id, id_type);
     }
 }
 
@@ -1589,10 +1617,10 @@ void CTSE_Info::x_MapChunkByFeatId(TFeatIdInt feat_id,
 void CTSE_Info::x_MapChunkByFeatId(const TFeatIdStr& id,
                                    CSeqFeatData::ESubtype subtype,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     SFeatIdIndex::TIndexStr& index = x_GetFeatIdIndexStr(subtype);
-    SFeatIdIndex::TIndexStr::value_type value(id, SFeatIdInfo(xref, chunk_id));
+    SFeatIdIndex::TIndexStr::value_type value(id, SFeatIdInfo(id_type, chunk_id));
     index.insert(value);
 }
 
@@ -1600,28 +1628,28 @@ void CTSE_Info::x_MapChunkByFeatId(const TFeatIdStr& id,
 void CTSE_Info::x_MapChunkByFeatId(const TFeatIdStr& id,
                                    CSeqFeatData::E_Choice type,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     CAnnotType_Index::TIndexRange range =
         CAnnotType_Index::GetFeatTypeRange(type);
     for ( size_t index = range.first; index < range.second; ++index ) {
         CSeqFeatData::ESubtype subtype =
             CAnnotType_Index::GetSubtypeForIndex(index);
-        x_MapChunkByFeatId(id, subtype, chunk_id, xref);
+        x_MapChunkByFeatId(id, subtype, chunk_id, id_type);
     }
 }
 
 
-void CTSE_Info::x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+void CTSE_Info::x_MapChunkByFeatId(const TFeatIdStr& id,
                                    const SAnnotTypeSelector& type,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     if ( type.GetFeatSubtype() != CSeqFeatData::eSubtype_any ) {
-        x_MapChunkByFeatId(feat_id, type.GetFeatSubtype(), chunk_id, xref);
+        x_MapChunkByFeatId(id, type.GetFeatSubtype(), chunk_id, id_type);
     }
     else {
-        x_MapChunkByFeatId(feat_id, type.GetFeatType(), chunk_id, xref);
+        x_MapChunkByFeatId(id, type.GetFeatType(), chunk_id, id_type);
     }
 }
 
@@ -1629,13 +1657,13 @@ void CTSE_Info::x_MapChunkByFeatId(const TFeatIdStr& feat_id,
 void CTSE_Info::x_MapChunkByFeatId(const TFeatId& id,
                                    CSeqFeatData::ESubtype subtype,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     if ( id.IsId() ) {
-        x_MapChunkByFeatId(id.GetId(), subtype, chunk_id, xref);
+        x_MapChunkByFeatId(id.GetId(), subtype, chunk_id, id_type);
     }
     else {
-        x_MapChunkByFeatId(id.GetStr(), subtype, chunk_id, xref);
+        x_MapChunkByFeatId(id.GetStr(), subtype, chunk_id, id_type);
     }
 }
 
@@ -1643,27 +1671,27 @@ void CTSE_Info::x_MapChunkByFeatId(const TFeatId& id,
 void CTSE_Info::x_MapChunkByFeatId(const TFeatId& id,
                                    CSeqFeatData::E_Choice type,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
     if ( id.IsId() ) {
-        x_MapChunkByFeatId(id.GetId(), type, chunk_id, xref);
+        x_MapChunkByFeatId(id.GetId(), type, chunk_id, id_type);
     }
     else {
-        x_MapChunkByFeatId(id.GetStr(), type, chunk_id, xref);
+        x_MapChunkByFeatId(id.GetStr(), type, chunk_id, id_type);
     }
 }
 
 
-void CTSE_Info::x_MapChunkByFeatId(const TFeatId& feat_id,
+void CTSE_Info::x_MapChunkByFeatId(const TFeatId& id,
                                    const SAnnotTypeSelector& type,
                                    TChunkId chunk_id,
-                                   bool xref)
+                                   EFeatIdType id_type)
 {
-    if ( feat_id.IsId() ) {
-        x_MapChunkByFeatId(feat_id.GetId(), type, chunk_id, xref);
+    if ( id.IsId() ) {
+        x_MapChunkByFeatId(id.GetId(), type, chunk_id, id_type);
     }
     else {
-        x_MapChunkByFeatId(feat_id.GetStr(), type, chunk_id, xref);
+        x_MapChunkByFeatId(id.GetStr(), type, chunk_id, id_type);
     }
 }
 
@@ -1703,15 +1731,15 @@ void CTSE_Info::x_MapChunkByFeatType(const SAnnotTypeSelector& type,
 CTSE_Info::TAnnotObjects
 CTSE_Info::x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                              TFeatIdInt id,
-                             bool xref) const
+                             EFeatIdType id_type) const
 {
-    UpdateAnnotIndex();
     TAnnotObjects objects;
+    UpdateFeatIdIndex(subtype, id_type);
     if ( subtype == CSeqFeatData::eSubtype_any ) {
-        x_AddAllFeaturesById(objects, id, xref);
+        x_AddAllFeaturesById(objects, id, id_type);
     }
     else {
-        x_AddFeaturesById(objects, subtype, id, xref);
+        x_AddFeaturesById(objects, subtype, id, id_type);
     }
     return objects;
 }
@@ -1720,12 +1748,12 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
 CTSE_Info::TAnnotObjects
 CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
                              TFeatIdInt id,
-                             bool xref) const
+                             EFeatIdType id_type) const
 {
-    UpdateAnnotIndex();
     TAnnotObjects objects;
+    UpdateFeatIdIndex(type, id_type);
     if ( type == CSeqFeatData::e_not_set ) {
-        x_AddAllFeaturesById(objects, id, xref);
+        x_AddAllFeaturesById(objects, id, id_type);
     }
     else {
         CAnnotType_Index::TIndexRange range =
@@ -1733,7 +1761,7 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
         for ( size_t index = range.first; index < range.second; ++index ) {
             CSeqFeatData::ESubtype subtype =
                 CAnnotType_Index::GetSubtypeForIndex(index);
-            x_AddFeaturesById(objects, subtype, id, xref);
+            x_AddFeaturesById(objects, subtype, id, id_type);
         }
     }
     return objects;
@@ -1743,15 +1771,15 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
 CTSE_Info::TAnnotObjects
 CTSE_Info::x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                              const TFeatIdStr& id,
-                             bool xref) const
+                             EFeatIdType id_type) const
 {
-    UpdateAnnotIndex();
     TAnnotObjects objects;
+    UpdateFeatIdIndex(subtype, id_type);
     if ( subtype == CSeqFeatData::eSubtype_any ) {
-        x_AddAllFeaturesById(objects, id, xref);
+        x_AddAllFeaturesById(objects, id, id_type);
     }
     else {
-        x_AddFeaturesById(objects, subtype, id, xref);
+        x_AddFeaturesById(objects, subtype, id, id_type);
     }
     return objects;
 }
@@ -1760,12 +1788,12 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
 CTSE_Info::TAnnotObjects
 CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
                              const TFeatIdStr& id,
-                             bool xref) const
+                             EFeatIdType id_type) const
 {
-    UpdateAnnotIndex();
     TAnnotObjects objects;
+    UpdateFeatIdIndex(type, id_type);
     if ( type == CSeqFeatData::e_not_set ) {
-        x_AddAllFeaturesById(objects, id, xref);
+        x_AddAllFeaturesById(objects, id, id_type);
     }
     else {
         CAnnotType_Index::TIndexRange range =
@@ -1773,7 +1801,7 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
         for ( size_t index = range.first; index < range.second; ++index ) {
             CSeqFeatData::ESubtype subtype =
                 CAnnotType_Index::GetSubtypeForIndex(index);
-            x_AddFeaturesById(objects, subtype, id, xref);
+            x_AddFeaturesById(objects, subtype, id, id_type);
         }
     }
     return objects;
@@ -1783,14 +1811,14 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
 CTSE_Info::TAnnotObjects
 CTSE_Info::x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                              const TFeatId& id,
-                             bool xref) const
+                             EFeatIdType id_type) const
 {
     TAnnotObjects objects;
     if ( id.IsId() ) {
-        x_GetFeaturesById(subtype, id.GetId(), xref).swap(objects);
+        x_GetFeaturesById(subtype, id.GetId(), id_type).swap(objects);
     }
     else {
-        x_GetFeaturesById(subtype, id.GetStr(), xref).swap(objects);
+        x_GetFeaturesById(subtype, id.GetStr(), id_type).swap(objects);
     }
     return objects;
 }
@@ -1799,14 +1827,14 @@ CTSE_Info::x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
 CTSE_Info::TAnnotObjects
 CTSE_Info::x_GetFeaturesById(CSeqFeatData::E_Choice type,
                              const TFeatId& id,
-                             bool xref) const
+                             EFeatIdType id_type) const
 {
     TAnnotObjects objects;
     if ( id.IsId() ) {
-        x_GetFeaturesById(type, id.GetId(), xref).swap(objects);
+        x_GetFeaturesById(type, id.GetId(), id_type).swap(objects);
     }
     else {
-        x_GetFeaturesById(type, id.GetStr(), xref).swap(objects);
+        x_GetFeaturesById(type, id.GetStr(), id_type).swap(objects);
     }
     return objects;
 }
@@ -1871,6 +1899,31 @@ CTSE_Info::x_FindSeq_feat(const CSeq_id_Handle& loc_id,
 }
 
 
+void CTSE_Info::SetBioseqUpdater(CRef<CBioseqUpdater> updater)
+{
+    CFastMutexGuard guard(m_BioseqsMutex);
+    m_BioseqUpdater = updater;
+    set<CBioseq_Info*> seen;
+    NON_CONST_ITERATE ( TBioseqs, it, m_Bioseqs ) {
+        if ( seen.insert(it->second).second ) {
+            m_BioseqUpdater->Update(*it->second);
+        }
+    }
+}
+
+
+string CTSE_Info::GetDescription(void) const
+{
+    string ret;
+    ret = GetBlobId().ToString();
+    if ( GetName().IsNamed() ) {
+        ret += '/';
+        ret += GetName().GetName();
+    }
+    return ret;
+}
+
+
 CTSE_SetObjectInfo::CTSE_SetObjectInfo(void)
 {
 }
