@@ -1,4 +1,4 @@
-/*  $Id: tse_info_object.cpp 354114 2012-02-23 15:43:46Z vasilche $
+/*  $Id: tse_info_object.cpp 381777 2012-11-28 18:43:40Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -346,5 +346,15 @@ void CTSE_Info_Object::SetBioObjectId(const CBioObjectId& id)
     m_UniqueId = id;
 }
 
+
+string CTSE_Info_Object::GetDescription(void) const
+{
+    if ( HasParent_Info() ) {
+        return GetBaseParent_Info().GetDescription();
+    }
+    return string();
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
