@@ -1,4 +1,4 @@
-/* $Id: showdefline.cpp 361746 2012-05-03 20:00:22Z vasilche $
+/* $Id: showdefline.cpp 373571 2012-08-30 17:47:35Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: showdefline.cpp 361746 2012-05-03 20:00:22Z vasilche $";
+static char const rcsid[] = "$Id: showdefline.cpp 373571 2012-08-30 17:47:35Z zaretska $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -192,7 +192,7 @@ CShowBlastDefline::GetSeqIdList(const objects::CBioseq_Handle& bh,
             != string::npos) {
             vector<string> title_tokens;
             id_token = 
-                NStr::Tokenize(sequence::GetTitle(bh), " ", title_tokens)[0];
+                NStr::Tokenize(sequence::CDeflineGenerator().GenerateDefline(bh), " ", title_tokens)[0];
         }
         if (id_token != NcbiEmptyString) {
             // Create a new local id with a label containing the extracted
@@ -225,7 +225,7 @@ CShowBlastDefline::GetBioseqHandleDeflineAndId(const CBioseq_Handle& handle,
         list<CRef<objects::CSeq_id> > ids;
         GetSeqIdList(handle, ids);
         seqid = GetSeqIdListString(ids, show_gi);
-        defline = GetTitle(handle);
+        defline = sequence::CDeflineGenerator().GenerateDefline(handle);
     } else { 
         bdlRef->PutTargetGiFirst(this_gi_first);
         const list< CRef< CBlast_def_line > >& bdl = bdlRef->Get();
@@ -279,7 +279,6 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
     const CRef<CBlast_def_line_set> bdlRef = CSeqDB::ExtractBlastDefline(handle);
     const list< CRef< CBlast_def_line > > &bdl = (bdlRef.Empty()) ? list< CRef< CBlast_def_line > >() : bdlRef->Get();
        
-    const CBioseq::TId* ids = &handle.GetBioseqCore()->GetId();
     CRef<CSeq_id> wid;    
     sdl->defline = NcbiEmptyString;
  
@@ -312,43 +311,9 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
             }   
         }
     }        
-    //get id (sdl->id, sdl-gi)
-    if(bdl.empty()){
-        wid = FindBestChoice(*ids, CSeq_id::WorstRank);
-        sdl->id = wid;
-        sdl->gi = FindGi(*ids);    
-    } else {        
-        bool found = false;
-        for(list< CRef< CBlast_def_line > >::const_iterator iter = bdl.begin();
-            iter != bdl.end(); iter++){
-            const CBioseq::TId* cur_id = &((*iter)->GetSeqid());
-            int cur_gi =  FindGi(*cur_id);    
-            wid = FindBestChoice(*cur_id, CSeq_id::WorstRank);
-            if (!use_this_gi.empty()) {
-                ITERATE(list<int>, iter_gi, use_this_gi){
-                    if(cur_gi == *iter_gi){
-                        found = true;
-                        break;
-                    }
-                }
-            } else {
-                ITERATE(CBioseq::TId, iter_id, *cur_id) {
-                    if ((*iter_id)->Match(aln_id) 
-                      || (aln_id.IsGeneral() && aln_id.GetGeneral().CanGetDb() && 
-                         (*iter_id)->IsGeneral() && (*iter_id)->GetGeneral().CanGetDb() &&
-                         aln_id.GetGeneral().GetDb() == (*iter_id)->GetGeneral().GetDb())) {
-                        found = true;
-                    }
-                }
-            }
-            if(found){
-                sdl->id = wid;
-                sdl->gi = cur_gi;
-                ids = cur_id;
-                break;
-            }
-        }
-    }
+    //get id (sdl->id, sdl-gi)    
+    sdl->id = CAlignFormatUtil::GetDisplayIds(handle,aln_id,use_this_gi,sdl->gi);
+    sdl->alnIDFasta = aln_id.AsFastaString();
 
     //get linkout
     if((m_Option & eLinkout)){
@@ -408,6 +373,9 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
         string accession;
         sdl->id->GetLabel(&accession, CSeq_id::eContent);
         sdl->score_url = !useTemplates ? "<a href=#" : "";
+        if (!useTemplates && m_PositionIndex >= 0) {
+            sdl->score_url += "_" + NStr::IntToString(m_PositionIndex) + "_";
+        }
         sdl->score_url += sdl->gi == 0 ? accession : 
             NStr::IntToString(sdl->gi);
         sdl->score_url += !useTemplates ? ">" : "";
@@ -419,39 +387,24 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
         CAlignFormatUtil::SSeqURLInfo seqUrlInfo(user_url,m_BlastType,m_IsDbNa,m_Database,m_Rid,
                                                  m_QueryNumber,sdl->gi, accession, sdl->linkout,
                                                  blast_rank,false,(m_Option & eNewTargetWindow) ? true : false,seqRange,flip); 
-        seqUrlInfo.resourcesUrl = m_Reg.get() ? m_Reg->Get(m_BlastType, "RESOURCES_URL") : kEmptyStr;
+        seqUrlInfo.resourcesUrl = m_Reg.get() ? m_Reg->Get(m_BlastType, "RESOURCE_URL") : kEmptyStr;
         seqUrlInfo.useTemplates = useTemplates;        
         seqUrlInfo.advancedView = advancedView;
-        sdl->id_url = CAlignFormatUtil::GetIDUrl(&seqUrlInfo,aln_id,*m_ScopeRef);        
-        if (advancedView){
-            if (m_Option & eLinkout && (sdl->gi > 0) ) {                    
-                    sdl->linkout_list = CAlignFormatUtil::GetFullLinkoutUrl(bdl,
-                                           m_Rid, 
-                                           m_CddRid, 
-                                           m_EntrezTerm, 
-                                           handle.GetBioseqCore()->IsNa(),
-                                           sdl->gi,
-                                           true, false,
-                                           blast_rank,
-                                           m_LinkoutOrder,
-                                           seqUrlInfo.taxid,
-                                           m_Database,
-                                           m_QueryNumber,                                                 
-                                           user_url,
-                                           m_PreComputedResID, m_LinkoutDB,
-                                           m_MapViewerBuildName);
+
+        if(sdl->id->Which() == CSeq_id::e_Local && (m_Option & eHtml)){
+            //get taxid info for local blast db such as igblast db
+            ITERATE(list<CRef<CBlast_def_line> >, iter_bdl, bdl) {
+                if ((*iter_bdl)->IsSetTaxid() && (*iter_bdl)->CanGetTaxid()){
+                    seqUrlInfo.taxid = (*iter_bdl)->GetTaxid();
+                    break;
+                }
             }
-            //Links to GenBank,FASTA,Graphics
-            list <string> customLinksList = 
-            CAlignFormatUtil::GetCustomLinksList(&seqUrlInfo,
-                           aln_id,
-                           *m_ScopeRef);                           
-            sdl->linkout_list.splice(sdl->linkout_list.begin(),customLinksList);
         }
+        sdl->id_url = CAlignFormatUtil::GetIDUrl(&seqUrlInfo,aln_id,*m_ScopeRef);                
     }
 
     //get defline
-    sdl->defline = GetTitle(m_ScopeRef->GetBioseqHandle(*(sdl->id)));
+    sdl->defline = CDeflineGenerator().GenerateDefline(m_ScopeRef->GetBioseqHandle(*(sdl->id)));
     if (!(bdl.empty())) { 
         for(list< CRef< CBlast_def_line > >::const_iterator iter = bdl.begin();
             iter != bdl.end(); iter++){
@@ -531,6 +484,8 @@ CShowBlastDefline::CShowBlastDefline(const CSeq_align_set& seqalign,
         }
     }
     m_DeflineTemplates = NULL;
+    m_StartIndex = 0;
+    m_PositionIndex = -1;
 }
 
 CShowBlastDefline::~CShowBlastDefline()
@@ -1354,60 +1309,44 @@ CShowBlastDefline::SScoreInfo*
 CShowBlastDefline::x_GetScoreInfoForTable(const CSeq_align_set& aln, int blast_rank)
 {
     string evalue_buf, bit_score_buf, total_bit_score_buf, raw_score_buf;
-    int score = 0;
-    double bits = 0;
-    double evalue = 0;
-    int sum_n = 0;
-    int num_ident = 0;
-    SScoreInfo* score_info = NULL;
-
+        
     if(aln.Get().empty())
-        return score_info;
+        return NULL;
 
-    score_info = x_GetScoreInfo(*(aln.Get().front()), blast_rank); 
+    auto_ptr<SScoreInfo> score_info(new SScoreInfo);   
 
-    double total_bits = 0;
-    double highest_bits = 0;
-    double lowest_evalue = 0;
-    int highest_length = 1;
-    int highest_ident = 0;
-    int highest_identity = 0;
-    list<int> use_this_gi;   // Not used here, but needed for GetAlnScores.    
-    score_info->subjRange = CAlignFormatUtil::GetSeqAlignCoverageParams(aln,&score_info->master_covered_length,&score_info->flip);
-												
-    ITERATE(CSeq_align_set::Tdata, iter, aln.Get()) {
-        int align_length = CAlignFormatUtil::GetAlignmentLength(**iter, 
-                                                        m_TranslatedNucAlignment);
-        CAlignFormatUtil::GetAlnScores(**iter, score, bits, evalue, sum_n, 
-                                   num_ident, use_this_gi);  
-        use_this_gi.clear();
-    
-        total_bits += bits;
-    
-        if (100*num_ident/align_length > highest_identity) {
-            highest_length = align_length;
-            highest_ident = num_ident;
-            highest_identity = 100*num_ident/align_length;
-        }
-    
-        if (bits > highest_bits) {
-            highest_bits = bits;
-            lowest_evalue = evalue;
-        }       
+    CAlignFormatUtil::SSeqAlignSetCalcParams* seqSetInfo = CAlignFormatUtil::GetSeqAlignSetCalcParamsFromASN(aln);
+    if(seqSetInfo->hspNum == 0) {//calulated params are not in ASN - calculate now
+        seqSetInfo = CAlignFormatUtil::GetSeqAlignSetCalcParams(aln,m_QueryLength,m_TranslatedNucAlignment);
     }
-    score_info->match = highest_ident;      
-    score_info->align_length = highest_length;
 
-    // Really need to call this twice??
-    CAlignFormatUtil::GetScoreString(lowest_evalue, highest_bits, total_bits, score,
-                                     evalue_buf, bit_score_buf, total_bit_score_buf, raw_score_buf);
+    CAlignFormatUtil::GetScoreString(seqSetInfo->evalue, seqSetInfo->bit_score, seqSetInfo->total_bit_score, seqSetInfo->raw_score,
+                              evalue_buf, bit_score_buf, total_bit_score_buf,
+                              raw_score_buf);
+    score_info->id = seqSetInfo->id;
 
     score_info->total_bit_string = total_bit_score_buf; 
-    score_info->bit_string = bit_score_buf;
+    score_info->bit_string = bit_score_buf;    
     score_info->evalue_string = evalue_buf;
-    score_info->hspNum = aln.Size();	
+    score_info->percent_coverage = seqSetInfo->percent_coverage;
+    score_info->percent_identity = seqSetInfo->percent_identity;
+    score_info->hspNum = seqSetInfo->hspNum;
 
-    return score_info;
+    score_info->use_this_gi = seqSetInfo->use_this_gi;
+    score_info->sum_n = seqSetInfo->sum_n == -1 ? 1:seqSetInfo->sum_n ;
+
+    score_info->raw_score_string = raw_score_buf;//check if used
+    score_info->match = seqSetInfo->match; //check if used     
+    score_info->align_length = seqSetInfo->align_length;//check if used
+    score_info->master_covered_length = seqSetInfo->master_covered_length;//check if used
+
+    
+    score_info->subjRange = seqSetInfo->subjRange;    //check if used
+    score_info->flip = seqSetInfo->flip;//check if used		
+
+    score_info->blast_rank = blast_rank+1;		   
+
+    return score_info.release();
 }
 
 vector <CShowBlastDefline::SDeflineInfo*> 
@@ -1558,31 +1497,42 @@ string CShowBlastDefline::x_FormatDeflineTableLine(SDeflineInfo* sdl,SScoreInfo*
         defLine = CAlignFormatUtil::MapTemplate(defLine,"score_info",iter->bit_string);        
     }
     /*****************This block of code is for future use with AJAX begin***************************/ 
-    string deflId; 
+    string deflId,deflFrmID,deflFastaSeq,deflAccs; 
     if(sdl->gi == 0) {
         string accession;
         sdl->id->GetLabel(& deflId, CSeq_id::eContent);
+        deflFrmID =  CAlignFormatUtil::GetLabel(sdl->id);//Just accession without db part like GNOMON: or ti:
+        deflFastaSeq = sdl->id->AsFastaString();        
     }
     else {        
-      deflId = NStr::IntToString(sdl->gi);
+        deflFrmID = deflId = NStr::IntToString(sdl->gi);        
+        deflFastaSeq = "gi|" + NStr::IntToString(sdl->gi);
+        deflFastaSeq = sdl->alnIDFasta;
+        deflAccs = sdl->id->AsFastaString();
     }
+    
+    //If gi - deflFrmID and deflId are the same and equal to gi "555",deflFastaSeq will have "gi|555"
+    //If gnl - deflFrmID=number, deflId=ti:number,deflFastaSeq=gnl|xxx
+    //like  "268252125","ti:268252125","gnl|ti|961433.m" or "961433.m" and "GNOMON:961433.m" "gnl|GNOMON|961433.m"    
     defLine = CAlignFormatUtil::MapTemplate(defLine,"dfln_id",deflId);
+    defLine = CAlignFormatUtil::MapTemplate(defLine,"dflnFrm_id",deflFrmID);    
+    defLine = CAlignFormatUtil::MapTemplate(defLine,"dflnFASTA_id",deflFastaSeq);
+    defLine = CAlignFormatUtil::MapTemplate(defLine,"dflnAccs",deflAccs);    
     defLine = CAlignFormatUtil::MapTemplate(defLine,"dfln_rid",m_Rid);    
     defLine = CAlignFormatUtil::MapTemplate(defLine,"dfln_hspnum",iter->hspNum); 
-    defLine = CAlignFormatUtil::MapTemplate(defLine,"dfln_blast_rank",iter->blast_rank); 
+    defLine = CAlignFormatUtil::MapTemplate(defLine,"dfln_blast_rank",m_StartIndex + iter->blast_rank); 
+    
 	/*****************This block of code is for future use with AJAX end***************************/ 
 
     defLine = CAlignFormatUtil::MapTemplate(defLine,"total_bit_string",iter->total_bit_string);
         
-    int percent_coverage = 100*iter->master_covered_length/m_QueryLength;
-
-    defLine = CAlignFormatUtil::MapTemplate(defLine,"percent_coverage",NStr::IntToString(percent_coverage));
+    
+    defLine = CAlignFormatUtil::MapTemplate(defLine,"percent_coverage",NStr::IntToString(iter->percent_coverage));
 
     defLine = CAlignFormatUtil::MapTemplate(defLine,"evalue_string",iter->evalue_string);
 
-    if(m_Option & eShowPercentIdent){
-        int percent_identity = CAlignFormatUtil::GetPercentMatch(iter->match, iter->align_length);
-        defLine = CAlignFormatUtil::MapTemplate(defLine,"percent_identity",NStr::IntToString(percent_identity));
+    if(m_Option & eShowPercentIdent){        
+        defLine = CAlignFormatUtil::MapTemplate(defLine,"percent_identity",NStr::IntToString(iter->percent_identity));
     }
         
     if(m_Option & eShowSumN){     
