@@ -1,4 +1,4 @@
-/* $Id: tabular.cpp 367113 2012-06-20 19:26:08Z rafanovi $
+/* $Id: tabular.cpp 390712 2013-03-01 15:45:05Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 /// One line is printed for each alignment with tab-delimited fielalnVec. 
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: tabular.cpp 367113 2012-06-20 19:26:08Z rafanovi $";
+static char const rcsid[] = "$Id: tabular.cpp 390712 2013-03-01 15:45:05Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -49,7 +49,9 @@ static char const rcsid[] = "$Id: tabular.cpp 367113 2012-06-20 19:26:08Z rafano
 #include <objmgr/util/sequence.hpp>
 
 #include <objects/blastdb/Blast_def_line.hpp>
-#include <objects/blastdb/Blast_def_line_set.hpp>
+
+#include <objmgr/seqdesc_ci.hpp>
+#include <objects/seqfeat/Org_ref.hpp>
 
 #include <map>
 
@@ -57,6 +59,8 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 BEGIN_SCOPE(align_format)
 
+static const string NA = "N/A";
+
 void 
 CBlastTabularInfo::x_AddDefaultFieldsToShow()
 {
@@ -103,13 +107,15 @@ void CBlastTabularInfo::x_ResetFields()
 {
     m_QueryLength = m_SubjectLength = 0U;
     m_Score = m_AlignLength = m_NumGaps = m_NumGapOpens = m_NumIdent =
-        m_NumPositives = m_QueryStart = m_QueryEnd = m_SubjectStart = 
-        m_SubjectEnd = 0; 
+    m_NumPositives = m_QueryStart = m_QueryEnd = m_SubjectStart =
+    m_SubjectEnd = 0;
     m_BitScore = NcbiEmptyString;
     m_Evalue = NcbiEmptyString;
     m_QuerySeq = NcbiEmptyString;
     m_SubjectSeq = NcbiEmptyString;
     m_BTOP = NcbiEmptyString;
+    m_SubjectStrand = NcbiEmptyString;
+    m_QueryCovSeqalign = -1;
 }
 
 void CBlastTabularInfo::x_SetFieldDelimiter(EFieldDelimiter delim)
@@ -124,13 +130,15 @@ void CBlastTabularInfo::x_SetFieldDelimiter(EFieldDelimiter delim)
 CBlastTabularInfo::CBlastTabularInfo(CNcbiOstream& ostr, const string& format,
                                      EFieldDelimiter delim, 
                                      bool parse_local_ids)
-    : m_Ostream(ostr) 
+    : m_Ostream(ostr)
 {
     x_SetFieldsToShow(format);
     x_ResetFields();
     x_SetFieldDelimiter(delim);
     SetParseLocalIds(parse_local_ids);
     SetNoFetch(false);
+    m_QueryCovSubject.first = NA;
+    m_QueryCovSubject.second = -1;
 }
 
 CBlastTabularInfo::~CBlastTabularInfo()
@@ -239,6 +247,155 @@ void CBlastTabularInfo::x_PrintSubjectAllAccessions(void)
     }
 }
 
+void CBlastTabularInfo::x_PrintSubjectTaxIds()
+{
+	if(m_SubjectTaxIds.empty()) {
+		m_Ostream << NA;
+		return;
+	}
+
+    ITERATE(set<int>, iter, m_SubjectTaxIds) {
+        if (iter != m_SubjectTaxIds.begin())
+            m_Ostream << ";";
+       m_Ostream << *iter;
+    }
+}
+
+void CBlastTabularInfo::x_PrintSubjectBlastNames()
+{
+	if(m_SubjectBlastNames.empty()) {
+		m_Ostream << NA;
+		return;
+	}
+
+	ITERATE(set<string>, iter, m_SubjectBlastNames) {
+        if (iter != m_SubjectBlastNames.begin())
+            m_Ostream << ";";
+        m_Ostream << *iter;
+	}
+}
+
+void CBlastTabularInfo::x_PrintSubjectSuperKingdoms()
+{
+	if(m_SubjectSuperKingdoms.empty()) {
+		m_Ostream << NA;
+		return;
+	}
+
+	 ITERATE(set<string>, iter, m_SubjectSuperKingdoms) {
+		 if (iter != m_SubjectSuperKingdoms.begin())
+			 m_Ostream << ";";
+		 m_Ostream << *iter;
+	}
+}
+
+void CBlastTabularInfo::x_PrintSubjectSciNames()
+{
+	if(m_SubjectSciNames.empty()) {
+		m_Ostream << NA;
+		return;
+	}
+
+	ITERATE(vector<string>, iter, m_SubjectSciNames) {
+		 if (iter != m_SubjectSciNames.begin())
+			 m_Ostream << ";";
+		 m_Ostream << *iter;
+	}
+}
+
+void CBlastTabularInfo::x_PrintSubjectCommonNames()
+{
+	if(m_SubjectCommonNames.empty()) {
+		m_Ostream << NA;
+		return;
+	}
+
+	 ITERATE(vector<string>, iter, m_SubjectCommonNames) {
+		 if (iter != m_SubjectCommonNames.begin())
+			 m_Ostream << ";";
+		 m_Ostream << *iter;
+	}
+}
+
+void CBlastTabularInfo::x_PrintSubjectAllTitles ()
+{
+	if(m_SubjectDefline.NotEmpty() && m_SubjectDefline->CanGet() &&
+	   m_SubjectDefline->IsSet() && !m_SubjectDefline->Get().empty())
+	{
+		const list<CRef<CBlast_def_line> > & defline = m_SubjectDefline->Get();
+		list<CRef<CBlast_def_line> >::const_iterator iter = defline.begin();
+		for(; iter != defline.end(); ++iter)
+		{
+			if (iter != defline.begin())
+						 m_Ostream << "<>";
+
+			if((*iter)->IsSetTitle())
+			{
+				if((*iter)->GetTitle().empty())
+					m_Ostream << NA;
+				else
+					m_Ostream << (*iter)->GetTitle();
+			}
+			else
+				m_Ostream << NA;
+		}
+	}
+	else
+		m_Ostream << NA;
+
+}
+
+void CBlastTabularInfo::x_PrintSubjectTitle()
+{
+	if(m_SubjectDefline.NotEmpty() && m_SubjectDefline->CanGet() &&
+	   m_SubjectDefline->IsSet() && !m_SubjectDefline->Get().empty())
+	{
+		const list<CRef<CBlast_def_line> > & defline = m_SubjectDefline->Get();
+
+		if(defline.empty())
+			m_Ostream << NA;
+		else
+		{
+			if(defline.front()->IsSetTitle())
+			{
+				if(defline.front()->GetTitle().empty())
+					m_Ostream << NA;
+				else
+					m_Ostream << defline.front()->GetTitle();
+			}
+			else
+				m_Ostream << NA;
+		}
+	}
+	else
+		m_Ostream << NA;
+
+}
+
+void CBlastTabularInfo::x_PrintSubjectStrand()
+{
+	if(m_SubjectStrand != NcbiEmptyString)
+		m_Ostream << m_SubjectStrand;
+	else
+		m_Ostream << NA;
+}
+
+void CBlastTabularInfo::x_PrintSubjectCoverage(void)
+{
+	if(m_QueryCovSubject.second < 0)
+		m_Ostream << NA;
+	else
+		m_Ostream << NStr::IntToString(m_QueryCovSubject.second);
+}
+
+void CBlastTabularInfo::x_PrintSeqalignCoverage(void)
+{
+	if(m_QueryCovSeqalign < 0)
+		m_Ostream << NA;
+	else
+		m_Ostream << NStr::IntToString(m_QueryCovSeqalign);
+}
+
 CRef<CSeq_id> s_ReplaceLocalId(const CBioseq_Handle& bh, CConstRef<CSeq_id> sid_in, bool parse_local)
 {
 
@@ -251,7 +408,7 @@ CRef<CSeq_id> s_ReplaceLocalId(const CBioseq_Handle& bh, CConstRef<CSeq_id> sid_
             string id_token;
             vector<string> title_tokens;
             title_tokens = 
-                NStr::Tokenize(sequence::GetTitle(bh), " ", title_tokens);
+                NStr::Tokenize(sequence::CDeflineGenerator().GenerateDefline(bh), " ", title_tokens);
             if(title_tokens.empty()){
                 id_token = NcbiEmptyString;
             } else {
@@ -289,19 +446,27 @@ void CBlastTabularInfo::SetQueryId(const CBioseq_Handle& bh)
         
 void CBlastTabularInfo::SetSubjectId(const CBioseq_Handle& bh)
 {
-    m_SubjectIds.clear();
 
-    // Check if this Bioseq handle contains a Blast-def-line-set object.
-    // If it does, retrieve Seq-ids from all redundant sequences, and 
-    // print them separated by commas.
-    // Retrieve the CBlast_def_line_set object and save in a CRef, preventing
-    // its destruction; then extract the list of CBlast_def_line objects.
     const CRef<CBlast_def_line_set> bdlRef = 
         CSeqDB::ExtractBlastDefline(bh);
 
-    if (bdlRef && bdlRef->CanGet() && bdlRef->IsSet() && !bdlRef->Get().empty()){
+    x_SetSubjectId(bh, bdlRef);
+    
+}
+
+void CBlastTabularInfo::x_SetSubjectId(const CBioseq_Handle& bh, const CRef<CBlast_def_line_set> & bdlRef)
+{
+    m_SubjectIds.clear();
+
+    // Check if this Bioseq handle contains a Blast-def-line-set object.
+    // If it does, retrieve Seq-ids from all redundant sequences, and
+    // print them separated by commas.
+    // Retrieve the CBlast_def_line_set object and save in a CRef, preventing
+    // its destruction; then extract the list of CBlast_def_line objects.
+
+    if (bdlRef.NotEmpty() && bdlRef->CanGet() && bdlRef->IsSet() && !bdlRef->Get().empty()){
         vector< CConstRef<CSeq_id> > original_seqids;
-        
+
         ITERATE(CBlast_def_line_set::Tdata, itr, bdlRef->Get()) {
             original_seqids.clear();
             ITERATE(CBlast_def_line::TSeqid, id, (*itr)->GetSeqid()) {
@@ -313,7 +478,7 @@ void CBlastTabularInfo::SetSubjectId(const CBioseq_Handle& bh)
             m_SubjectIds.push_back(next_seqid_list);
         }
     } else {
-        // Blast-def-line is not filled, hence retrieve all Seq-ids directly 
+        // Blast-def-line is not filled, hence retrieve all Seq-ids directly
         // from the Bioseq handle's Seq-id.
         list<CRef<objects::CSeq_id> > subject_id_list;
         ITERATE(CBioseq_Handle::TId, itr, bh.GetId()) {
@@ -322,9 +487,124 @@ void CBlastTabularInfo::SetSubjectId(const CBioseq_Handle& bh)
         }
         m_SubjectIds.push_back(subject_id_list);
     }
-    
+
+
 }
 
+bool s_IsValidName(const string & name)
+{
+	if(name == "-")
+		return false;
+
+	if(name == "unclassified")
+		return false;
+
+	return true;
+}
+
+void CBlastTabularInfo::x_SetTaxInfo(const CBioseq_Handle & handle, const CRef<CBlast_def_line_set> & bdlRef)
+{
+	m_SubjectTaxIds.clear();
+	m_SubjectSciNames.clear();
+	m_SubjectCommonNames.clear();
+	m_SubjectBlastNames.clear();
+	m_SubjectSuperKingdoms.clear();
+
+    if (bdlRef.NotEmpty() && bdlRef->CanGet() && bdlRef->IsSet() && !bdlRef->Get().empty()){
+
+        ITERATE(CBlast_def_line_set::Tdata, itr, bdlRef->Get()) {
+        	if((*itr)->IsSetTaxid())
+        	{
+        		if((*itr)->GetTaxid())
+        			m_SubjectTaxIds.insert((*itr)->GetTaxid());
+        	}
+        }
+    }
+
+	if(m_SubjectTaxIds.empty())
+	{
+		CSeqdesc_CI desc_s(handle, CSeqdesc::e_Source);
+		for (;desc_s; ++desc_s)
+		{
+			 int t = desc_s->GetSource().GetOrg().GetTaxId();
+			 if(t)
+				  m_SubjectTaxIds.insert(t);
+		}
+
+		CSeqdesc_CI desc(handle, CSeqdesc::e_Org);
+		for (; desc; ++desc)
+		{
+			int t= desc->GetOrg().GetTaxId();
+			if(t)
+				m_SubjectTaxIds.insert(t);
+		}
+	}
+
+	if(m_SubjectTaxIds.empty())
+		return;
+
+   	if( x_IsFieldRequested(eSubjectSciNames) ||
+    	x_IsFieldRequested(eSubjectCommonNames) ||
+    	x_IsFieldRequested(eSubjectBlastNames) ||
+    	x_IsFieldRequested(eSubjectSuperKingdoms))
+   	{
+   		set<int>::iterator itr = m_SubjectTaxIds.begin();
+
+   		for(; itr !=  m_SubjectTaxIds.end(); ++itr)
+   		{
+    		try
+    		{
+    			SSeqDBTaxInfo taxinfo;
+    			CSeqDB::GetTaxInfo(*itr, taxinfo);
+    			m_SubjectSciNames.push_back(taxinfo.scientific_name);
+    			m_SubjectCommonNames.push_back(taxinfo.common_name);
+    			if(s_IsValidName(taxinfo.blast_name))
+    				m_SubjectBlastNames.insert(taxinfo.blast_name);
+
+    			if(s_IsValidName(taxinfo.s_kingdom))
+    				m_SubjectSuperKingdoms.insert(taxinfo.s_kingdom);
+
+    		} catch (const CException&) {
+    			//only put fillers in if we are going to show tax id
+    			// the fillers are put in so that the name list would
+    			// match the taxid list
+    			if(x_IsFieldRequested(eSubjectTaxIds) )
+    			{
+    				m_SubjectSciNames.push_back(NA);
+    				m_SubjectCommonNames.push_back(NA);
+    			}
+    		}
+    	}
+    }
+
+	return;
+
+}
+void CBlastTabularInfo::x_SetQueryCovSubject(const CSeq_align & align)
+{
+	int pct = -1;
+	if(align.GetNamedScore("seq_percent_coverage", pct))
+	{
+		m_QueryCovSubject.first = align.GetSeq_id(1).AsFastaString();
+		m_QueryCovSubject.second = pct;
+	}
+	else if(align.GetSeq_id(1).AsFastaString() != m_QueryCovSubject.first)
+	{
+		m_QueryCovSubject.first = NA;
+		m_QueryCovSubject.second = pct;
+	}
+
+}
+
+void CBlastTabularInfo::x_SetQueryCovSeqalign(const CSeq_align & align, int query_len)
+{
+	int len = abs((int) (align.GetSeqStop(0) - align.GetSeqStart(0))) + 1;
+	double tmp = 100.0 * len/(double) query_len;
+	if(tmp  < 99)
+		tmp +=0.5;
+
+	m_QueryCovSeqalign = (int)tmp;
+}
 
 int CBlastTabularInfo::SetFields(const CSeq_align& align, 
                                  CScope& scope, 
@@ -358,7 +638,8 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
     // Extract the full query id from the correspondintg Bioseq handle.
     if (x_IsFieldRequested(eQuerySeqId) || x_IsFieldRequested(eQueryGi) ||
         x_IsFieldRequested(eQueryAccession) ||
-        x_IsFieldRequested(eQueryAccessionVersion)) {
+        x_IsFieldRequested(eQueryAccessionVersion) ||
+        x_IsFieldRequested(eQueryCovSeqalign)) {
         try {
             // FIXME: do this only if the query has changed
             const CBioseq_Handle& query_bh = 
@@ -366,6 +647,7 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
             SetQueryId(query_bh);
             query_is_na = query_bh.IsNa();
             m_QueryLength = query_bh.GetBioseqLength();
+            x_SetQueryCovSeqalign(align, m_QueryLength);
         } catch (const CException&) {
             list<CRef<CSeq_id> > query_ids;
             CRef<CSeq_id> id(new CSeq_id());
@@ -376,20 +658,47 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
         }
     }
 
+    if (x_IsFieldRequested(eQueryCovSubject))
+    	x_SetQueryCovSubject(align);
+
     // Extract the full list of subject ids
-    if (x_IsFieldRequested(eSubjectSeqId) || 
+    bool setSubjectIds =  (x_IsFieldRequested(eSubjectSeqId) ||
         x_IsFieldRequested(eSubjectAllSeqIds) ||
         x_IsFieldRequested(eSubjectGi) ||
         x_IsFieldRequested(eSubjectAllGis) ||
         x_IsFieldRequested(eSubjectAccession) ||
         x_IsFieldRequested(eSubjectAllAccessions) ||
-        x_IsFieldRequested(eSubjAccessionVersion)) {
+        x_IsFieldRequested(eSubjAccessionVersion));
+
+    bool setSubjectTaxInfo = (x_IsFieldRequested(eSubjectTaxIds) ||
+    						  x_IsFieldRequested(eSubjectSciNames) ||
+    						  x_IsFieldRequested(eSubjectCommonNames) ||
+    						  x_IsFieldRequested(eSubjectBlastNames) ||
+    						  x_IsFieldRequested(eSubjectSuperKingdoms));
+
+    bool setSubjectTitle = (x_IsFieldRequested(eSubjectTitle) ||
+			  	  	  	  	x_IsFieldRequested(eSubjectAllTitles));
+
+    if(setSubjectIds || setSubjectTaxInfo || setSubjectTitle ||
+       x_IsFieldRequested(eSubjectStrand))
+    {
         try {
             const CBioseq_Handle& subject_bh = 
                 scope.GetBioseqHandle(align.GetSeq_id(1));
-            SetSubjectId(subject_bh);
+            CRef<CBlast_def_line_set> bdlRef =
+                CSeqDB::ExtractBlastDefline(subject_bh);
+            x_SetSubjectId(subject_bh, bdlRef);
             subject_is_na = subject_bh.IsNa();
             m_SubjectLength = subject_bh.GetBioseqLength();
+            if(setSubjectTaxInfo)
+            	x_SetTaxInfo(subject_bh, bdlRef);
+            if(setSubjectTitle)
+            {
+            	m_SubjectDefline.Reset();
+            	if(bdlRef.NotEmpty())
+            		m_SubjectDefline = bdlRef;
+            }
+
         } catch (const CException&) {
             list<CRef<CSeq_id> > subject_ids;
             CRef<CSeq_id> id(new CSeq_id());
@@ -398,6 +707,7 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
             SetSubjectId(subject_ids);
             bioseqs_found = false;
         }
+
     }
 
     // If Bioseq has not been found for one or both of the sequences, all
@@ -411,7 +721,9 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
         //_ASSERT(m_QueryId.front().NotEmpty());
         //m_QueryLength = sequence::GetLength(*m_QueryId.front(), &scope);
         m_QueryLength = sequence::GetLength(align.GetSeq_id(0), &scope);
+
     }
+
     if (x_IsFieldRequested(eSubjectLength) && m_SubjectLength == 0) {
         //_ASSERT(!m_SubjectIds.empty());
         //_ASSERT(!m_SubjectIds.front().empty());
@@ -428,7 +740,8 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
         x_IsFieldRequested(eSubjectSeq) || (x_IsFieldRequested(eNumIdentical) && num_ident > 0 ) ||
         x_IsFieldRequested(ePositives) || x_IsFieldRequested(eMismatches) || 
         x_IsFieldRequested(ePercentPositives) || x_IsFieldRequested(ePercentIdentical) ||
-        x_IsFieldRequested(eQueryFrame) || x_IsFieldRequested(eBTOP)) {
+        x_IsFieldRequested(eQueryFrame) || x_IsFieldRequested(eBTOP) ||
+        x_IsFieldRequested(eSubjectStrand)) {
 
     CRef<CSeq_align> finalAln(0);
    
@@ -561,7 +874,8 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
     }
 
     if (x_IsFieldRequested(eSubjectStart) || x_IsFieldRequested(eSubjectEnd) ||
-        x_IsFieldRequested(eSubjFrame) || x_IsFieldRequested(eFrames)) {
+        x_IsFieldRequested(eSubjFrame) || x_IsFieldRequested(eFrames) ||
+        x_IsFieldRequested(eSubjectStrand)) {
         // If subject is on a reverse strand, reverse its start and end
         // offsets. Also do that for a nucleotide-nucleotide search, if query
         // is on the reverse strand, because BLAST output always reverses
@@ -574,6 +888,14 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
             s_start = alnVec->GetSeqStart(kSubjectRow) + 1;
             s_end = alnVec->GetSeqStop(kSubjectRow) + 1;
         }
+
+        if(x_IsFieldRequested(eSubjectStrand))
+        {
+        	if((kTranslated) || (!subject_is_na))
+        		m_SubjectStrand = NA;
+        	else
+        		m_SubjectStrand = ((s_start - s_end) > 0 )? "minus":"plus";
+        }
     }
     SetEndpoints(q_start, q_end, s_start, s_end);
     
@@ -687,6 +1009,26 @@ void CBlastTabularInfo::x_PrintFieldNames()
             m_Ostream << "sbjct frame"; break; 
         case eBTOP:
             m_Ostream << "BTOP"; break;        
+        case eSubjectTaxIds:
+            m_Ostream << "subject tax ids"; break;
+        case eSubjectSciNames:
+        	m_Ostream << "subject sci names"; break;
+        case eSubjectCommonNames:
+        	m_Ostream << "subject com names"; break;
+        case eSubjectBlastNames:
+        	m_Ostream << "subject blast names"; break;
+        case eSubjectSuperKingdoms:
+        	m_Ostream << "subject super kingdoms"; break;
+        case eSubjectTitle:
+        	m_Ostream << "subject title"; break;
+        case eSubjectAllTitles:
+        	m_Ostream << "subject titles"; break;
+        case eSubjectStrand:
+        	m_Ostream << "subject strand"; break;
+        case eQueryCovSubject:
+        	m_Ostream << "% subject coverage"; break;
+        case eQueryCovSeqalign:
+        	m_Ostream << "% hsp coverage"; break;
         default:
             _ASSERT(false);
             break;
@@ -909,6 +1251,26 @@ CBlastTabularInfo::x_PrintField(ETabularField field)
         x_PrintSubjectFrame(); break;        
     case eBTOP:
         x_PrintBTOP(); break;        
+    case eSubjectTaxIds:
+    	x_PrintSubjectTaxIds(); break;
+    case eSubjectSciNames:
+    	x_PrintSubjectSciNames(); break;
+    case eSubjectCommonNames:
+    	x_PrintSubjectCommonNames(); break;
+    case eSubjectBlastNames:
+    	x_PrintSubjectBlastNames(); break;
+    case eSubjectSuperKingdoms:
+    	x_PrintSubjectSuperKingdoms(); break;
+    case eSubjectTitle:
+    	x_PrintSubjectTitle(); break;
+    case eSubjectAllTitles:
+    	x_PrintSubjectAllTitles(); break;
+    case eSubjectStrand:
+    	x_PrintSubjectStrand(); break;
+    case eQueryCovSubject:
+    	x_PrintSubjectCoverage(); break;
+    case eQueryCovSeqalign:
+    	x_PrintSeqalignCoverage(); break;
     default:
         _ASSERT(false);
         break;
@@ -945,6 +1307,7 @@ CIgBlastTabularInfo::PrintHeader(const string& program_version,
 int CIgBlastTabularInfo::SetMasterFields(const CSeq_align& align, 
                                  CScope& scope, 
                                  const string& chain_type,
+                                 const string& master_chain_type_to_show,
                                  CNcbiMatrix<int>* matrix)
 {
     int retval = 0;
@@ -962,7 +1325,7 @@ int CIgBlastTabularInfo::SetMasterFields(const CSeq_align& align,
     if (!hasSeq) x_AddFieldToShow(eQuerySeq);
     if (!hasQuerySeqId) x_AddFieldToShow(eQuerySeqId);
     if (!hasQueryStart) x_AddFieldToShow(eQueryStart);
-    retval = SetFields(align, scope, chain_type, matrix);
+    retval = SetFields(align, scope, chain_type, master_chain_type_to_show, matrix);
     if (!hasSeq) x_DeleteFieldToShow(eQuerySeq);
     if (!hasQuerySeqId) x_DeleteFieldToShow(eQuerySeqId);
     if (!hasQueryStart) x_DeleteFieldToShow(eQueryStart);
@@ -970,11 +1333,13 @@ int CIgBlastTabularInfo::SetMasterFields(const CSeq_align& align,
 };            
 
 int CIgBlastTabularInfo::SetFields(const CSeq_align& align,
-                                 CScope& scope, 
-                                 const string& chain_type,
-                                 CNcbiMatrix<int>* matrix)
+                                   CScope& scope, 
+                                   const string& chain_type,
+                                   const string& master_chain_type_to_show,
+                                   CNcbiMatrix<int>* matrix)
 {
     m_ChainType = chain_type;
+    m_MasterChainTypeToShow = master_chain_type_to_show;
     if (m_ChainType == "NA") m_ChainType = "N/A";
     return CBlastTabularInfo::SetFields(align, scope, matrix);
 };
@@ -994,10 +1359,11 @@ void CIgBlastTabularInfo::SetIgAnnotation(const CRef<blast::CIgAnnotation> &anno
     if (annot->m_FrameInfo[1] >= 0 && annot->m_FrameInfo[2] >= 0) {
         int off = annot->m_FrameInfo[1];
         int len = annot->m_FrameInfo[2] - off;
+        string seq_trans;
         if ( len % 3 == 0) {
             string seq_data(m_Query, off, len);
-            string seq_trans;
-            CSeqTranslator::Translate(seq_data, seq_trans);
+            CSeqTranslator::Translate(seq_data, seq_trans, 
+                                      CSeqTranslator::fIs5PrimePartial, NULL, NULL);
             if (seq_trans.find('*') != string::npos) {
                 SetFrame("IP");
             } else {
@@ -1006,10 +1372,79 @@ void CIgBlastTabularInfo::SetIgAnnotation(const CRef<blast::CIgAnnotation> &anno
         } else  {
             SetFrame("OF");
         }
+   
     } else {
         SetFrame("N/A");
     }
 
+    if (annot->m_FrameInfo[0] >= 0) {
+        int off = annot->m_FrameInfo[0];
+        int len = (annot->m_GeneInfo[1] - off)/3*3;
+        string seq_data(m_Query, off, len);
+        string seq_trans;
+        CSeqTranslator::Translate(seq_data, seq_trans, 
+                                  CSeqTranslator::fIs5PrimePartial, NULL, NULL);
+        if (seq_trans.find('*') != string::npos) {
+            m_OtherInfo.push_back("Yes");
+        } else {
+            m_OtherInfo.push_back("No");
+        }
+    } else {
+        m_OtherInfo.push_back("N/A");
+    }
+
+    if (annot->m_FrameInfo[2] >=0) {
+        int off = annot->m_FrameInfo[2];
+        int len = (annot->m_GeneInfo[5] - off)/3*3;
+        string seq_data(m_Query, off, len);
+        string seq_trans;
+        CSeqTranslator::Translate(seq_data, seq_trans, 
+                                  CSeqTranslator::fIs5PrimePartial, NULL, NULL);
+        if (seq_trans.find('*') == string::npos) {
+            m_OtherInfo.push_back("No");
+            if (m_FrameInfo == "IF" && m_OtherInfo[0] == "No") {
+                m_OtherInfo.push_back("Yes");
+            } else {
+                m_OtherInfo.push_back("No");
+            }
+        } else {
+            m_OtherInfo.push_back("Yes");
+            m_OtherInfo.push_back("No");
+        }
+    } else {
+        m_OtherInfo.push_back("N/A");
+        m_OtherInfo.push_back("N/A");
+    }
+
+    //stop codon anywhere between start of V and end of J
+    if (annot->m_FrameInfo[0] >= 0) {
+        int v_start = annot->m_FrameInfo[0];
+        int v_j_length = max(max(annot->m_GeneInfo[5], annot->m_GeneInfo[3]), annot->m_GeneInfo[1]) - annot->m_FrameInfo[0];
+  
+        string seq_data(m_Query, v_start, v_j_length);
+        string seq_trans;
+        CSeqTranslator::Translate(seq_data, seq_trans, 
+                                  CSeqTranslator::fIs5PrimePartial, NULL, NULL);
+       
+        if (seq_trans.find('*') == string::npos) {
+            m_OtherInfo.push_back("No");
+            if (m_FrameInfo == "IF" || m_FrameInfo == "IP") {
+                m_OtherInfo.push_back("Yes");
+            } else if (m_FrameInfo == "OF"){
+                m_OtherInfo.push_back("No");
+            } else {
+               m_OtherInfo.push_back("N/A"); 
+            }
+        } else {
+            m_OtherInfo.push_back("Yes");
+            m_OtherInfo.push_back("No");
+        }
+        
+    } else {
+        m_OtherInfo.push_back("N/A");
+        m_OtherInfo.push_back("N/A");
+    }
+ 
     // Domain info coordinates are inclusive (and always on positive strand)
     AddIgDomain("FWR1", annot->m_DomainInfo[0], annot->m_DomainInfo[1]+1,
                         annot->m_DomainInfo_S[0], annot->m_DomainInfo_S[1]+1);
@@ -1041,16 +1476,21 @@ void CIgBlastTabularInfo::PrintMasterAlign(const string &header) const
         }
         m_Ostream << header << "V(D)J rearrangement summary for query sequence ";
         m_Ostream << "(Top V gene match, ";
-        if (m_ChainType == "VH") m_Ostream << "Top D gene match, ";
-        m_Ostream << "Top J gene match, Chain type, V-J Frame, Strand):" << endl;
+        if (m_ChainType == "VH" || m_ChainType == "VD" || 
+            m_ChainType == "VB") m_Ostream << "Top D gene match, ";
+        m_Ostream << "Top J gene match, Chain type, stop codon, ";
+        m_Ostream << "V-J frame, Productive, Strand):" << endl;
         m_Ostream << m_VGene.sid << m_FieldDelimiter;
-        if (m_ChainType == "VH") m_Ostream << m_DGene.sid << m_FieldDelimiter;
+        if (m_ChainType == "VH"|| m_ChainType == "VD" || 
+            m_ChainType == "VB") m_Ostream << m_DGene.sid << m_FieldDelimiter;
         m_Ostream << m_JGene.sid << m_FieldDelimiter;
-        m_Ostream << m_ChainType << m_FieldDelimiter;
+        m_Ostream << m_MasterChainTypeToShow << m_FieldDelimiter;
+        m_Ostream << m_OtherInfo[3] << m_FieldDelimiter;
         if (m_FrameInfo == "IF") m_Ostream << "In-frame";
         else if (m_FrameInfo == "OF") m_Ostream << "Out-of-frame";
-        else if (m_FrameInfo == "IP") m_Ostream << "In-frame with stop codon";
+        else if (m_FrameInfo == "IP") m_Ostream << "In-frame";
         else m_Ostream << "N/A";
+        m_Ostream << m_FieldDelimiter << m_OtherInfo[4];
         m_Ostream << m_FieldDelimiter << ((m_IsMinusStrand) ? '-' : '+' ) << endl << endl;
         x_PrintIgGenes(false, header);
     }
@@ -1092,31 +1532,42 @@ void CIgBlastTabularInfo::PrintMasterAlign(const string &header) const
 void CIgBlastTabularInfo::PrintHtmlSummary() const
 {
     if (m_IsNucl) {
+        if (m_IsMinusStrand) {
+            m_Ostream << "<br>Note that your query represents the minus strand "
+                      << "of a V gene and has been converted to the plus strand. "
+                      << "The sequence positions refer to the converted sequence.\n\n";
+        }
         m_Ostream << "<br><br><br>V(D)J rearrangement summary for query sequence:\n";
         m_Ostream << "<pre><table border=1>\n";
         m_Ostream << "<tr><td>Top V gene match</td>";
-        if (m_ChainType == "VH") {  
+        if (m_ChainType == "VH" || m_ChainType == "VD" || 
+            m_ChainType == "VB") {  
             m_Ostream << "<td>Top D gene match</td>";
         }
         m_Ostream << "<td>Top J gene match</td>"
                   << "<td>Chain type</td>"
-                  << "<td>V-J Frame</td>"
+                  << "<td>stop codon</td>"
+                  << "<td>V-J frame</td>"
+                  << "<td>Productive</td>"
                   << "<td>Strand</td></tr>\n";
 
         m_Ostream << "<tr><td>"  << m_VGene.sid;
-        if (m_ChainType == "VH") { 
+        if (m_ChainType == "VH" || m_ChainType == "VD" || 
+            m_ChainType == "VB") { 
             m_Ostream << "</td><td>" << m_DGene.sid;
         }
         m_Ostream << "</td><td>" << m_JGene.sid
-                  << "</td><td>" << m_ChainType
+                  << "</td><td>" << m_MasterChainTypeToShow 
                   << "</td><td>";
+        m_Ostream << (m_OtherInfo[3]!="N/A" ? m_OtherInfo[3] : "") << "</td><td>";
         if (m_FrameInfo == "IF") {
             m_Ostream << "In-frame";
         } else if (m_FrameInfo == "OF") {
             m_Ostream << "Out-of-frame";
         } else if (m_FrameInfo == "IP") {
-            m_Ostream << "In-frame with stop codon";
+            m_Ostream << "In-frame";
         } 
+        m_Ostream << "</td><td>" << (m_OtherInfo[4]!="N/A" ? m_OtherInfo[4] : "");
         m_Ostream << "</td><td>" << ((m_IsMinusStrand) ? '-' : '+') 
                   << "</td></tr></table></pre>\n";
         x_PrintIgGenes(true, "");
@@ -1168,6 +1619,7 @@ void CIgBlastTabularInfo::x_ResetIgFields()
     m_VGene.Reset();
     m_DGene.Reset();
     m_JGene.Reset();
+    m_OtherInfo.clear();
 };
 
 void CIgBlastTabularInfo::x_PrintPartialQuery(int start, int end, bool isHtml) const
@@ -1215,7 +1667,9 @@ void CIgBlastTabularInfo::x_PrintIgGenes(bool isHtml, const string& header) cons
     if (b2 < 0) {
         b2 = b1;
         b3 = b1;
-        if (b3 > b4 && b4 > 0 && m_ChainType == "VH") {
+        if (b3 > b4 && b4 > 0 && (m_ChainType == "VH" || 
+                                  m_ChainType == "VD" || 
+                                  m_ChainType == "VB")) {
             b4 = b3;
         }
     }
@@ -1225,7 +1679,8 @@ void CIgBlastTabularInfo::x_PrintIgGenes(bool isHtml, const string& header) cons
         b5 = b3;
     }
 
-    if (m_ChainType == "VH") {
+    if (m_ChainType == "VH" || m_ChainType == "VD" || 
+            m_ChainType == "VB") {
         a1 = min(b1, b2);
         a2 = max(b1, b2);
         a3 = min(b3, b4);
@@ -1236,10 +1691,11 @@ void CIgBlastTabularInfo::x_PrintIgGenes(bool isHtml, const string& header) cons
     }
 
     if (isHtml) {
-        m_Ostream << "<br>Nucleotide details around V(D)J junctions:\n";
+        m_Ostream << "<br>V(D)J junction details based on top germline gene matches:\n";
         m_Ostream << "<pre><table border=1>\n";
         m_Ostream << "<tr><td>V region end</td>";
-        if (m_ChainType == "VH") {
+        if (m_ChainType == "VH" || m_ChainType == "VD" || 
+            m_ChainType == "VB") {
             m_Ostream << "<td>V-D junction*</td>"
                       << "<td>D region</td>"
                       << "<td>D-J junction*</td>";
@@ -1248,15 +1704,16 @@ void CIgBlastTabularInfo::x_PrintIgGenes(bool isHtml, const string& header) cons
         }
         m_Ostream << "<td>J region start</td></tr>\n<tr>";
     } else {
-        m_Ostream << header << "V(D)J junction details (V end, ";
-        if (m_ChainType == "VH")  m_Ostream << "V-D junction, D region, D-J junction, ";
+        m_Ostream << header << "V(D)J junction details based on top germline gene matches (V end, ";
+        if (m_ChainType == "VH" || m_ChainType == "VD" || 
+            m_ChainType == "VB")  m_Ostream << "V-D junction, D region, D-J junction, ";
         else m_Ostream << "V-J junction, ";
         m_Ostream << "J start).  Note that possible overlapping nucleotides at VDJ junction (i.e, nucleotides that could be assigned to either joining gene segment) are indicated in parentheses (i.e., (TACT)) but"
                   << " are not included under V, D, or J gene itself" << endl;
     }
 
     x_PrintPartialQuery(max(b0, a1 - 5), a1, isHtml); m_Ostream << m_FieldDelimiter;
-    if (m_ChainType == "VH") {
+    if (m_ChainType == "VH" || m_ChainType == "VD" || m_ChainType == "VB") {
         x_PrintPartialQuery(b1, b2, isHtml); m_Ostream << m_FieldDelimiter;
         x_PrintPartialQuery(a2, a3, isHtml); m_Ostream << m_FieldDelimiter;
         x_PrintPartialQuery(b3, b4, isHtml); m_Ostream << m_FieldDelimiter;
