@@ -1,4 +1,4 @@
-/*  $Id: vectorscreen.cpp 189834 2010-04-26 13:36:34Z madden $
+/*  $Id: vectorscreen.cpp 328328 2011-08-02 15:57:59Z jianye $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: vectorscreen.cpp 189834 2010-04-26 13:36:34Z madden $";
+static char const rcsid[] = "$Id: vectorscreen.cpp 328328 2011-08-02 15:57:59Z jianye $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -523,7 +523,8 @@ void CVecscreen::x_BuildNonOverlappingRange(vector<CRef<CSeq_align_set> >
             //merge if previous range intersect with current range
             if(j > 0){ 
                 prev_range = &(aln_info_vec[i].back()->range);
-                if(prev_range->IntersectingWith(cur_aln_info->range)){
+                if(prev_range->IntersectingWith(cur_aln_info->range) ||
+                   prev_range->AbuttingWith(cur_aln_info->range)){
                     aln_info_vec[i].back()->range = 
                         aln_info_vec[i].back()->range.CombinationWith(cur_aln_info->range);
                     delete cur_aln_info;
