@@ -1,4 +1,4 @@
-/*  $Id: aln_generators.cpp 350847 2012-01-24 19:35:33Z rafanovi $
+/*  $Id: aln_generators.cpp 360549 2012-04-24 15:29:03Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -64,21 +64,21 @@ USING_SCOPE(objects);
 
 
 CRef<CSeq_align>
-CreateSeqAlignFromAnchoredAln(const CAnchoredAln& anchored_aln,   ///< input
-                              CSeq_align::TSegs::E_Choice choice, ///< choice of alignment 'segs'
+CreateSeqAlignFromAnchoredAln(const CAnchoredAln& anchored_aln,
+                              CSeq_align::TSegs::E_Choice choice,
                               CScope* scope)
 {
     CRef<CSeq_align> sa(new CSeq_align);
     sa->SetType(CSeq_align::eType_not_set);
     sa->SetDim(anchored_aln.GetDim());
 
-    switch(choice)    {
+    switch(choice) {
     case CSeq_align::TSegs::e_Dendiag:
+        CreateDense_diagFromAnchoredAln(sa->SetSegs().SetDendiag(), anchored_aln, scope);
         break;
-    case CSeq_align::TSegs::e_Denseg: {
+    case CSeq_align::TSegs::e_Denseg:
         sa->SetSegs().SetDenseg(*CreateDensegFromAnchoredAln(anchored_aln, scope));
         break;
-    }
     case CSeq_align::TSegs::e_Std:
         break;
     case CSeq_align::TSegs::e_Packed:
@@ -101,6 +101,40 @@ CreateSeqAlignFromAnchoredAln(const CAnchoredAln& anchored_aln,   ///< input
 }
 
 
+CRef<CSeq_align>
+CreateSeqAlignFromPairwiseAln(const CPairwiseAln& pairwise_aln,
+                              CSeq_align::TSegs::E_Choice choice,
+                              CScope* scope)
+{
+    CRef<CSeq_align> sa(new CSeq_align);
+    sa->SetType(CSeq_align::eType_not_set);
+    sa->SetDim(2);
+
+    switch(choice) {
+    case CSeq_align::TSegs::e_Denseg:
+        sa->SetSegs().SetDenseg(*CreateDensegFromPairwiseAln(pairwise_aln, scope));
+        break;
+    case CSeq_align::TSegs::e_Disc:
+        sa->SetSegs().SetDisc(*CreateAlignSetFromPairwiseAln(pairwise_aln, scope));
+        break;
+    case CSeq_align::TSegs::e_Packed:
+        sa->SetSegs().SetPacked(*CreatePackedsegFromPairwiseAln(pairwise_aln, scope));
+        break;
+    case CSeq_align::TSegs::e_Spliced:
+        sa->SetSegs().SetSpliced(*CreateSplicedsegFromPairwiseAln(pairwise_aln, scope));
+        break;
+    case CSeq_align::TSegs::e_Std:
+    case CSeq_align::TSegs::e_Dendiag:
+    case CSeq_align::TSegs::e_Sparse:
+    case CSeq_align::TSegs::e_not_set:
+        NCBI_THROW(CAlnException, eInvalidRequest,
+                   "Unsupported CSeq_align::TSegs type.");
+        break;
+    }
+    return sa;
+}
+
+
 //#define _TRACE_CSegmentedRangeCollection
 #ifdef _TRACE_CSegmentedRangeCollection
 ostream& operator<<(ostream& out, const CRangeCollection<CPairwiseAln::TPos>& segmetned_range_coll)
@@ -179,6 +213,118 @@ public:
 };
 
 
+void
+CreateDense_diagFromAnchoredAln(CSeq_align::TSegs::TDendiag& dd,
+                                const CAnchoredAln& anchored_aln,
+                                CScope* scope) 
+{
+    const CAnchoredAln::TPairwiseAlnVector& pairwises = anchored_aln.GetPairwiseAlns();
+
+    typedef CSegmentedRangeCollection TAnchorSegments;
+    TAnchorSegments anchor_segments;
+    ITERATE(CAnchoredAln::TPairwiseAlnVector, pairwise_aln_i, pairwises) {
+        ITERATE (CPairwiseAln::TAlnRngColl, rng_i, **pairwise_aln_i) {
+            anchor_segments.insert(CPairwiseAln::TRng(rng_i->GetFirstFrom(), rng_i->GetFirstTo()));
+        }
+    }
+
+    CSeq_align::TSegs::TDendiag diags;
+    CDense_diag::TDim dim = anchored_aln.GetDim();
+    size_t numseg = anchor_segments.size();
+    for (size_t seg = 0; seg < numseg; ++seg) {
+        CRef<CDense_diag> diag(new CDense_diag);
+        diag->SetDim(dim);
+        diag->SetIds().resize(dim);
+        for (int row = 0;  row < dim;  ++row) {
+            CRef<CSeq_id> id(new CSeq_id);
+            id->Assign(anchored_aln.GetId(dim - row - 1)->GetSeqId());
+            diag->SetIds()[row] = id;
+        }
+        diag->SetStarts().resize(dim, kInvalidSeqPos);
+        diag->SetStrands().resize(dim);
+        diag->SetLen(anchor_segments[seg].GetLength());
+        diags.push_back(diag);
+    }
+
+    for (int row = 0;  row < dim;  ++row) {
+        size_t seg = 0;
+        CSeq_align::TSegs::TDendiag::iterator diag_it = diags.begin();
+
+        TAnchorSegments::const_iterator seg_i = anchor_segments.begin();
+        CPairwiseAln::TAlnRngColl::const_iterator aln_rng_i =
+            pairwises[dim - row - 1]->begin();
+        bool direct = aln_rng_i->IsDirect();
+        TSignedSeqPos left_delta = 0;
+        TSignedSeqPos right_delta = aln_rng_i->GetLength();
+        while (seg_i != anchor_segments.end()) {
+            _ASSERT(seg < numseg);
+            if (aln_rng_i != pairwises[dim - row - 1]->end()  &&
+                seg_i->GetFrom() >= aln_rng_i->GetFirstFrom()) {
+                _ASSERT(seg_i->GetToOpen() <= aln_rng_i->GetFirstToOpen());
+                if (seg_i->GetToOpen() > aln_rng_i->GetFirstToOpen()) {
+                    NCBI_THROW(CAlnException, eInternalFailure,
+                               "seg_i->GetToOpen() > aln_rng_i->GetFirstToOpen()");
+                }
+
+                // dec right_delta
+                _ASSERT(right_delta >= seg_i->GetLength());
+                if (right_delta < seg_i->GetLength()) {
+                    NCBI_THROW(CAlnException, eInternalFailure,
+                               "right_delta < seg_i->GetLength()");
+                }
+                right_delta -= seg_i->GetLength();
+
+                (*diag_it)->SetStarts()[row] = 
+                    (direct ?
+                     aln_rng_i->GetSecondFrom() + left_delta :
+                     aln_rng_i->GetSecondFrom() + right_delta);
+
+                // inc left_delta
+                left_delta += seg_i->GetLength();
+
+                if (right_delta == 0) {
+                    _ASSERT(left_delta == aln_rng_i->GetLength());
+                    ++aln_rng_i;
+                    if (aln_rng_i != pairwises[dim - row - 1]->end()) {
+                        direct = aln_rng_i->IsDirect();
+                        left_delta = 0;
+                        right_delta = aln_rng_i->GetLength();
+                    }
+                }
+            }
+            (*diag_it)->SetStrands()[row] =
+                (direct ? eNa_strand_plus : eNa_strand_minus);
+            ++seg_i;
+            ++seg;
+            ++diag_it;
+        }
+    }
+    // Cleanup: remove rows with gaps, remove one-row diags.
+    NON_CONST_ITERATE(CSeq_align::TSegs::TDendiag, diag_it, diags) {
+        size_t row = 0;
+        CDense_diag& diag = **diag_it;
+        CDense_diag::TStarts& starts = diag.SetStarts();
+        while (row < starts.size()) {
+            if (starts[row] == kInvalidSeqPos) {
+                starts.erase(starts.begin() + row);
+                CDense_diag::TIds& ids = diag.SetIds();
+                ids.erase(ids.begin() + row);
+                CDense_diag::TStrands& strands = diag.SetStrands();
+                strands.erase(strands.begin() + row);
+                continue;
+            }
+            ++row;
+        }
+        if (diag.GetStarts().size() < 2) continue;
+        diag.SetDim(starts.size());
+#if _DEBUG
+        diag.Validate();
+#endif
+        dd.push_back(*diag_it);
+    }
+}
+
+
 CRef<CDense_seg>
 CreateDensegFromAnchoredAln(const CAnchoredAln& anchored_aln,
                             CScope* scope) 
@@ -193,20 +339,20 @@ CreateDensegFromAnchoredAln(const CAnchoredAln& anchored_aln,
         }
     }
 
-    /// Create a dense-seg
+    // Create a dense-seg
     CRef<CDense_seg> ds(new CDense_seg);
 
-    /// Determine dimensions
+    // Determine dimensions
     CDense_seg::TNumseg& numseg = ds->SetNumseg();
     numseg = anchor_segments.size();
     CDense_seg::TDim& dim = ds->SetDim();
     dim = anchored_aln.GetDim();
 
-    /// Tmp vars
+    // Tmp vars
     CDense_seg::TDim row;
     CDense_seg::TNumseg seg;
 
-    /// Ids
+    // Ids
     CDense_seg::TIds& ids = ds->SetIds();
     ids.resize(dim);
     for (row = 0;  row < dim;  ++row) {
@@ -214,7 +360,7 @@ CreateDensegFromAnchoredAln(const CAnchoredAln& anchored_aln,
         SerialAssign<CSeq_id>(*ids[row], anchored_aln.GetId(dim - row - 1)->GetSeqId());
     }
 
-    /// Lens
+    // Lens
     CDense_seg::TLens& lens = ds->SetLens();
     lens.resize(numseg);
     TAnchorSegments::const_iterator seg_i = anchor_segments.begin();
@@ -224,11 +370,11 @@ CreateDensegFromAnchoredAln(const CAnchoredAln& anchored_aln,
 
     int matrix_size = dim * numseg;
 
-    /// Strands (just resize, will set while setting starts)
+    // Strands (just resize, will set while setting starts)
     CDense_seg::TStrands& strands = ds->SetStrands();
     strands.resize(matrix_size, eNa_strand_unknown);
 
-    /// Starts and strands
+    // Starts and strands
     CDense_seg::TStarts& starts = ds->SetStarts();
     starts.resize(matrix_size, -1);
     for (row = 0;  row < dim;  ++row) {
@@ -293,11 +439,11 @@ CRef<CDense_seg>
 CreateDensegFromPairwiseAln(const CPairwiseAln& pairwise_aln,
                             CScope* scope)
 {
-    /// Create a dense-seg
+    // Create a dense-seg
     CRef<CDense_seg> ds(new CDense_seg);
 
 
-    /// Determine dimensions
+    // Determine dimensions
     CDense_seg::TNumseg& numseg = ds->SetNumseg();
     numseg = pairwise_aln.size();
     ds->SetDim(2);
@@ -313,19 +459,19 @@ CreateDensegFromPairwiseAln(const CPairwiseAln& pairwise_aln,
     ids.resize(2);
 
 
-    /// Ids
+    // Ids
     ids[0].Reset(new CSeq_id);
     SerialAssign<CSeq_id>(*ids[0], pairwise_aln.GetFirstId()->GetSeqId());
     ids[1].Reset(new CSeq_id);
     SerialAssign<CSeq_id>(*ids[1], pairwise_aln.GetSecondId()->GetSeqId());
 
 
-    /// Tmp vars
+    // Tmp vars
     CDense_seg::TNumseg seg = 0;
     int matrix_pos = 0;
 
 
-    /// Main loop to set all values
+    // Main loop to set all values
     ITERATE(CPairwiseAln::TAlnRngColl, aln_rng_i, pairwise_aln) {
         starts[matrix_pos++] = aln_rng_i->GetFirstFrom();
         if ( !aln_rng_i->IsDirect() ) {
@@ -348,12 +494,12 @@ CreateDensegFromPairwiseAln(const CPairwiseAln& pairwise_aln,
 }
 
 
-CRef<CSpliced_seg>
-CreateSplicedsegFromAnchoredAln(const CAnchoredAln& anchored_aln,
-                                CScope* scope)
-{
-    _ASSERT(anchored_aln.GetDim() == 2);
+static const TSignedSeqPos kMaxSplicedExonIndelLength = 15;
 
+void InitSplicedsegFromPairwiseAln(CSpliced_seg& spliced_seg,
+                                   const CPairwiseAln& pairwise_aln,
+                                   CScope* scope)
+{
     // Sort by product positions (if minus strand, then reverse).
     // Make sure genomic positions are sorted in the corresponding direction too, no overlaps etc.
     // Small one-row gap -> indel.
@@ -361,52 +507,43 @@ CreateSplicedsegFromAnchoredAln(const CAnchoredAln& anchored_aln,
     // Other gaps -> intron, set 'partial' on both sides.
     // If the product does not start/end at the sequence extreme, set 'partial' for the exon.
 
-    // Create a spliced_seg
-    CRef<CSpliced_seg> spliced_seg(new CSpliced_seg);
-
-    CAnchoredAln::TDim anchor_row = anchored_aln.GetAnchorRow();
-    const CPairwiseAln& pairwise = *anchored_aln.GetPairwiseAlns()[1 - anchor_row];
     // Check strands - one per row.
-    _ASSERT((pairwise.GetFlags() & CPairwiseAln::fMixedDir) != CPairwiseAln::fMixedDir);
+    _ASSERT((pairwise_aln.GetFlags() & CPairwiseAln::fMixedDir) != CPairwiseAln::fMixedDir);
     // Product is nuc or prot.
-    _ASSERT(pairwise.GetFirstBaseWidth() == 1  ||
-            pairwise.GetFirstBaseWidth() == 3);
-    bool prot = pairwise.GetFirstBaseWidth() == 3;
+    _ASSERT(pairwise_aln.GetFirstBaseWidth() == 1  ||
+            pairwise_aln.GetFirstBaseWidth() == 3);
+    bool prot = pairwise_aln.GetFirstBaseWidth() == 3;
     // The other row is genomic.
-    _ASSERT(pairwise.GetSecondBaseWidth() == 1);
-    // The alignment should be sorted by now.
-    // pairwise.Sort();
+    _ASSERT(pairwise_aln.GetSecondBaseWidth() == 1);
 
     // Ids
-    CRef<CSeq_id> product_id(new CSeq_id); 
-    product_id->Assign(pairwise.GetFirstId()->GetSeqId());
-    spliced_seg->SetProduct_id(*product_id);
-    CRef<CSeq_id> genomic_id(new CSeq_id); 
-    genomic_id->Assign(pairwise.GetSecondId()->GetSeqId());
-    spliced_seg->SetGenomic_id(*genomic_id);
+    CRef<CSeq_id> product_id(new CSeq_id);
+    product_id->Assign(pairwise_aln.GetFirstId()->GetSeqId());
+    spliced_seg.SetProduct_id(*product_id);
+    CRef<CSeq_id> genomic_id(new CSeq_id);
+    genomic_id->Assign(pairwise_aln.GetSecondId()->GetSeqId());
+    spliced_seg.SetGenomic_id(*genomic_id);
 
     // Product type
-    spliced_seg->SetProduct_type(prot ?
+    spliced_seg.SetProduct_type(prot ?
         CSpliced_seg::eProduct_type_protein
         : CSpliced_seg::eProduct_type_transcript);
 
     // Exons
-    CSpliced_seg::TExons& exons = spliced_seg->SetExons();
+    CSpliced_seg::TExons& exons = spliced_seg.SetExons();
 
     typedef TSignedSeqPos                  TPos;
     typedef CRange<TPos>                   TRng; 
     typedef CAlignRange<TPos>              TAlnRng;
     typedef CAlignRangeCollection<TAlnRng> TAlnRngColl;
 
-    static const TPos kMaxIndelLength = 15;
-
     TPos last_prod_end = 0;
     TPos last_gen_end = 0;
     CRef<CSpliced_exon> exon;
-    CPairwiseAln::TAlnRngColl::const_iterator rg_it = pairwise.begin();
-    bool gen_direct = rg_it == pairwise.end() || rg_it->IsDirect();
+    CPairwiseAln::TAlnRngColl::const_iterator rg_it = pairwise_aln.begin();
+    bool gen_direct = rg_it == pairwise_aln.end() || rg_it->IsDirect();
     bool prod_direct = prot ||
-        rg_it == pairwise.end() || rg_it->IsFirstDirect();
+        rg_it == pairwise_aln.end() || rg_it->IsFirstDirect();
     // Adjust genomic strand - in CPairwiseAln it's relative to the first seq.
     if ( !prod_direct ) {
         gen_direct = !gen_direct;
@@ -415,12 +552,13 @@ CreateSplicedsegFromAnchoredAln(const CAnchoredAln& anchored_aln,
     TRng ex_prod_rg;
     TRng ex_gen_rg;
 
-    for (; rg_it != pairwise.end(); ++rg_it) {
+    // Main loop to set all values
+    ITERATE(CPairwiseAln::TAlnRngColl, rg_it, pairwise_aln) {
         const CPairwiseAln::TAlnRng& rg = *rg_it;
 
         // Unaligned ranges.
         TPos prod_skip, gen_skip;
-        if (rg_it == pairwise.begin()) {
+        if (rg_it == pairwise_aln.begin()) {
             prod_skip = 0;
             gen_skip = 0;
         }
@@ -432,7 +570,8 @@ CreateSplicedsegFromAnchoredAln(const CAnchoredAln& anchored_aln,
         }
 
         // Break exon, ignore long gaps between exons.
-        if (!exon  ||  prod_skip > kMaxIndelLength  ||  gen_skip > kMaxIndelLength) {
+        if (!exon  ||  prod_skip > kMaxSplicedExonIndelLength  ||
+            gen_skip > kMaxSplicedExonIndelLength) {
             if ( exon ) {
                 _ASSERT(exon->IsSetProduct_start());
                 _ASSERT(exon->IsSetGenomic_start());
@@ -572,20 +711,53 @@ CreateSplicedsegFromAnchoredAln(const CAnchoredAln& anchored_aln,
     }
 
 #ifdef _DEBUG
-    spliced_seg->Validate(true);
+    spliced_seg.Validate(true);
 #endif
+}
+
+
+CRef<CSpliced_seg>
+CreateSplicedsegFromAnchoredAln(const CAnchoredAln& anchored_aln,
+                                CScope* scope)
+{
+    _ASSERT(anchored_aln.GetDim() == 2);
+
+    // Sort by product positions (if minus strand, then reverse).
+    // Make sure genomic positions are sorted in the corresponding direction too, no overlaps etc.
+    // Small one-row gap -> indel.
+    // Large genomic gap -> intron (start new exon).
+    // Other gaps -> intron, set 'partial' on both sides.
+    // If the product does not start/end at the sequence extreme, set 'partial' for the exon.
+
+    // Create a spliced_seg
+    CRef<CSpliced_seg> spliced_seg(new CSpliced_seg);
+    CAnchoredAln::TDim anchor_row = anchored_aln.GetAnchorRow();
+    const CPairwiseAln& pairwise = *anchored_aln.GetPairwiseAlns()[1 - anchor_row];
+    InitSplicedsegFromPairwiseAln(*spliced_seg, pairwise, scope);
     return spliced_seg;
 }
 
 
-void
-s_TranslatePairwise(CPairwiseAln& out_pw,   ///< output pairwise (needs to be empty)
-                    const CPairwiseAln& pw, ///< input pairwise to translate from
-                    const CPairwiseAln& tr) ///< translating pairwise
+CRef<CSpliced_seg>
+CreateSplicedsegFromPairwiseAln(const CPairwiseAln& pairwise_aln,
+                                CScope* scope)
+{
+    // Create a dense-seg
+    CRef<CSpliced_seg> ss(new CSpliced_seg);
+    InitSplicedsegFromPairwiseAln(*ss, pairwise_aln, scope);
+    return ss;
+}
+
+
+void s_TranslatePairwise(
+    CPairwiseAln& out_pw,   // output pairwise (needs to be empty)
+    const CPairwiseAln& pw, // input pairwise to translate from
+    const CPairwiseAln& tr) // translating pairwise
 {
     ITERATE (CPairwiseAln, it, pw) {
         CPairwiseAln::TAlnRng ar = *it;
         ar.SetFirstFrom(tr.GetSecondPosByFirstPos(ar.GetFirstFrom()));
+        if (ar.GetFirstFrom() < 0) continue; // skip unaligned ranges
         out_pw.insert(ar);
     }
 }
@@ -594,58 +766,60 @@ s_TranslatePairwise(CPairwiseAln& out_pw,   ///< output pairwise (needs to be em
 typedef CAnchoredAln::TDim TDim;
 
 
-void
-CreateSeqAlignFromEachPairwiseAln
-(const CAnchoredAln::TPairwiseAlnVector pairwises, ///< input
- TDim anchor,                                      ///< choice of anchor
- vector<CRef<CSeq_align> >& out_seqaligns,         ///< output
- CSeq_align::TSegs::E_Choice choice,      ///< choice of alignment 'segs'
- CScope* scope)
+void CreateSeqAlignFromEachPairwiseAln(
+    const CAnchoredAln::TPairwiseAlnVector pairwises,
+    TDim                                   anchor,
+    vector<CRef<CSeq_align> >&             out_seqaligns,
+    CSeq_align::TSegs::E_Choice            choice,
+    CScope*                                scope)
 {
     out_seqaligns.resize(pairwises.size() - 1);
     for (TDim row = 0, sa_idx = 0;
          row < (TDim) pairwises.size();
-         ++row, ++sa_idx) {
-        if (row != anchor) {
-            CRef<CSeq_align> sa(new CSeq_align);
-            sa->SetType(CSeq_align::eType_partial);
-            sa->SetDim(2);
+         ++row) {
+        if (row == anchor) continue;
+        CRef<CSeq_align> sa(new CSeq_align);
+        sa->SetType(CSeq_align::eType_partial);
+        sa->SetDim(2);
 
-            const CPairwiseAln& pw = *pairwises[row];
-            CRef<CPairwiseAln> p(new CPairwiseAln(pairwises[anchor]->GetSecondId(),
-                                                  pw.GetSecondId(),
-                                                  pw.GetFlags()));
-            s_TranslatePairwise(*p, pw, *pairwises[anchor]);
+        const CPairwiseAln& pw = *pairwises[row];
+        CRef<CPairwiseAln> p(new CPairwiseAln(pairwises[anchor]->GetSecondId(),
+            pw.GetSecondId(),
+            pw.GetFlags()));
+        s_TranslatePairwise(*p, pw, *pairwises[anchor]);
 
-            switch(choice)    {
-            case CSeq_align::TSegs::e_Denseg: {
-                CRef<CDense_seg> ds = CreateDensegFromPairwiseAln(*p, scope);
-                sa->SetSegs().SetDenseg(*ds);
-                break;
-            }
-            case CSeq_align::TSegs::e_Disc: {
-                CRef<CSeq_align_set> disc = CreateAlignSetFromPairwiseAln(*p, scope);
-                sa->SetSegs().SetDisc(*disc);
-                break;
-            }
-            case CSeq_align::TSegs::e_Packed: {
-                CRef<CPacked_seg> ps = CreatePackedsegFromPairwiseAln(*p, scope);
-                sa->SetSegs().SetPacked(*ps);
-                break;
-            }
-            case CSeq_align::TSegs::e_Dendiag:
-            case CSeq_align::TSegs::e_Std:
-            case CSeq_align::TSegs::e_Spliced:
-            case CSeq_align::TSegs::e_Sparse:
-                NCBI_THROW(CAlnException, eInvalidRequest,
-                           "Unsupported CSeq_align::TSegs type.");
-            case CSeq_align::TSegs::e_not_set:
-            default:
-                NCBI_THROW(CAlnException, eInvalidRequest,
-                           "Invalid CSeq_align::TSegs type.");
-            }
-            out_seqaligns[sa_idx].Reset(sa);
+        switch(choice)    {
+        case CSeq_align::TSegs::e_Denseg: {
+            CRef<CDense_seg> ds = CreateDensegFromPairwiseAln(*p, scope);
+            sa->SetSegs().SetDenseg(*ds);
+            break;
         }
+        case CSeq_align::TSegs::e_Disc: {
+            CRef<CSeq_align_set> disc = CreateAlignSetFromPairwiseAln(*p, scope);
+            sa->SetSegs().SetDisc(*disc);
+            break;
+        }
+        case CSeq_align::TSegs::e_Packed: {
+            CRef<CPacked_seg> ps = CreatePackedsegFromPairwiseAln(*p, scope);
+            sa->SetSegs().SetPacked(*ps);
+            break;
+        }
+        case CSeq_align::TSegs::e_Spliced: {
+            CRef<CSpliced_seg> ss = CreateSplicedsegFromPairwiseAln(*p, scope);
+            sa->SetSegs().SetSpliced(*ss);
+            break;
+        }
+        case CSeq_align::TSegs::e_Dendiag:
+        case CSeq_align::TSegs::e_Std:
+        case CSeq_align::TSegs::e_Sparse:
+            NCBI_THROW(CAlnException, eInvalidRequest,
+                        "Unsupported CSeq_align::TSegs type.");
+        case CSeq_align::TSegs::e_not_set:
+        default:
+            NCBI_THROW(CAlnException, eInvalidRequest,
+                        "Invalid CSeq_align::TSegs type.");
+        }
+        out_seqaligns[sa_idx++].Reset(sa);
     }
 }
 
