@@ -1,4 +1,4 @@
-/*  $Id: seqdbatlas.cpp 189922 2010-04-26 19:02:12Z maning $
+/*  $Id: seqdbatlas.cpp 315260 2011-07-22 13:48:03Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,13 +32,13 @@
 /// classes, which provide control of a set of memory mappings.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: seqdbatlas.cpp 189922 2010-04-26 19:02:12Z maning $";
+static char const rcsid[] = "$Id: seqdbatlas.cpp 315260 2011-07-22 13:48:03Z camacho $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
 
-#include "seqdbatlas.hpp"
-#include "seqdbgeneral.hpp"
+#include <objtools/blast/seqdb_reader/impl/seqdbatlas.hpp>
+#include <objtools/blast/seqdb_reader/impl/seqdbgeneral.hpp>
 #include <memory>
 #include <algorithm>
 #include <objtools/blast/seqdb_reader/seqdbcommon.hpp>
@@ -339,7 +339,7 @@ bool CSeqDBAtlas::GetFileSizeL(const string & fname,
         if (file_length >= 0) {
             data.first  = true;
             data.second = SeqDB_CheckLength<Int8,TIndx>(file_length);
-            if (file_length > m_MaxFileSize) m_MaxFileSize = file_length;
+            if ((Uint8)file_length > m_MaxFileSize) m_MaxFileSize = file_length;
         } else {
             data.first  = false;
             data.second = 0;
@@ -1324,7 +1324,7 @@ bool CRegionMap::MapFile(CSeqDBAtlas * atlas)
         CHECK_MARKER();
         
         string msg("CSeqDBAtlas::MapFile: allocation failed for ");
-        msg += NStr::IntToString(rdsize);
+        msg += NStr::UInt8ToString(rdsize);
         msg += " bytes.";
         
         NCBI_THROW(CSeqDBException, eMemErr, msg);
