@@ -1,4 +1,4 @@
-/*  $Id: seqdbisam.cpp 220506 2011-01-20 22:26:16Z rafanovi $
+/*  $Id: seqdbisam.cpp 311249 2011-07-11 14:12:16Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,11 +32,11 @@
 /// index of some particular kind of identifiers.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: seqdbisam.cpp 220506 2011-01-20 22:26:16Z rafanovi $";
+static char const rcsid[] = "$Id: seqdbisam.cpp 311249 2011-07-11 14:12:16Z camacho $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
-#include "seqdbisam.hpp"
+#include <objtools/blast/seqdb_reader/impl/seqdbisam.hpp>
 #include <objects/seqloc/Seq_id.hpp>
 #include <objects/general/general__.hpp>
 #include <corelib/ncbiutil.hpp>
