@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 
-/*  $Id: seqdbvol.hpp 351200 2012-01-26 19:01:24Z maning $
+/*  $Id: seqdbvol.hpp 384347 2012-12-26 15:53:32Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,7 +40,7 @@
 
 #include <objtools/blast/seqdb_reader/impl/seqdbatlas.hpp>
 #include <objtools/blast/seqdb_reader/impl/seqdbgeneral.hpp>
-#include "seqdbtax.hpp"
+#include <objtools/blast/seqdb_reader/impl/seqdbtax.hpp>
 #include "seqdbcol.hpp"
 #include <objects/seq/seq__.hpp>
 
