@@ -1,4 +1,4 @@
-/*  $Id: build_db.cpp 368592 2012-07-10 14:40:31Z rafanovi $
+/*  $Id: build_db.cpp 387632 2013-01-30 22:55:42Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
   Code to build a database given various sources of sequence data.
 */
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: build_db.cpp 368592 2012-07-10 14:40:31Z rafanovi $";
+static char const rcsid[] = "$Id: build_db.cpp 387632 2013-01-30 22:55:42Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -292,14 +292,12 @@ void CBuildDatabase::x_DupLocal()
 
 static CConstRef<CBioseq> s_FixBioseqDeltas(CConstRef<objects::CBioseq> bs)
 {
-    if ((! bs->CanGetInst()) || bs->GetInst().CanGetSeq_data()) {
+    if ((! bs->CanGetInst()) || bs->GetInst().CanGetSeq_data() ||
+    	 ! bs->GetInst().CanGetExt() || ! bs->GetInst().GetExt().IsDelta()) {
         return bs;
     }
     
-    if (bs->CanGetInst() &&
-        bs->GetInst().CanGetExt() &&
-        bs->GetInst().GetExt().IsDelta() &&
-        bs->GetInst().CanGetMol() &&
+    if (bs->GetInst().CanGetMol() &&
         !CSeq_inst::IsNa(bs->GetInst().GetMol())) {
         
         NCBI_THROW(CMultisourceException, eArg,
