@@ -1,4 +1,4 @@
-/*  $Id: writedb_convert.cpp 282672 2011-05-13 17:53:00Z maning $
+/*  $Id: writedb_convert.cpp 387632 2013-01-30 22:55:42Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
 /// class for WriteDB.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: writedb_convert.cpp 282672 2011-05-13 17:53:00Z maning $";
+static char const rcsid[] = "$Id: writedb_convert.cpp 387632 2013-01-30 22:55:42Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -592,4 +592,26 @@ void WriteDB_Ncbi2naToBinary(const CSeq_inst & si,
     seq[last_byte] |= remainder;
 }
 
+void WriteDB_IupacnaToBinary(const CSeq_inst & si, string & seq, string & amb)
+{
+    const string & v = si.GetSeq_data().GetIupacna().Get();
+
+    _ASSERT(si.GetLength() == v.size());
+
+    string tmp;
+    // convert to string.
+    CSeqConvert::Convert(v,
+                         CSeqUtil::e_Iupacna,
+                         0,
+                         (int) v.size(),
+                         tmp,
+                         CSeqUtil::e_Ncbi4na);
+
+    WriteDB_Ncbi4naToBinary(tmp.c_str(),
+                            (int) tmp.size(),
+                            (int) si.GetLength(),
+                            seq,
+                            amb);
+}
+
 END_NCBI_SCOPE
