@@ -1,4 +1,4 @@
-/*  $Id: writedb_impl.cpp 349797 2012-01-13 14:28:18Z fongah2 $
+/*  $Id: writedb_impl.cpp 387632 2013-01-30 22:55:42Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
 /// class for WriteDB.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: writedb_impl.cpp 349797 2012-01-13 14:28:18Z fongah2 $";
+static char const rcsid[] = "$Id: writedb_impl.cpp 387632 2013-01-30 22:55:42Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -367,7 +367,7 @@ void CWriteDB_Impl::x_MakeAlias()
             if (dblist.size())
                 dblist += " ";
         
-            dblist += CWriteDB_File::MakeShortName(m_Dbname, i);
+            dblist += CDirEntry(CWriteDB_File::MakeShortName(m_Dbname, i)).GetName();
         }
     } else {
         dblist = m_Dbname;
@@ -633,6 +633,29 @@ x_SetDeflinesFromBinary(const string                   & bin_hdr,
     deflines.Reset(&* bdls);
 }
 
+
+static bool s_UseFastaReaderDeflines(CConstRef<CBioseq> & bioseq, CConstRef<CBlast_def_line_set> & deflines)
+{
+	if(deflines.Empty())
+		return false;
+
+	const CSeq_id * bioseq_id = bioseq->GetNonLocalId();
+
+	if(bioseq_id == NULL)
+		return true;
+
+	// Bioseq has non-local id, make sure at least one id is non-local from CFastaReader
+	// defline
+    ITERATE(list< CRef<CBlast_def_line> >, iter, deflines->Get()) {
+        CRef<CSeq_id> id = FindBestChoice((**iter).GetSeqid(), &CSeq_id::BestRank);
+        if (id.NotEmpty()  &&  !id->IsLocal()) {
+                return true;
+        }
+    }
+    return false;
+
+}
+
 void
 CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
                                  CConstRef<CBlast_def_line_set> & deflines,
@@ -677,6 +700,10 @@ CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
                                      parse_ids);
         }
         
+        if(!s_UseFastaReaderDeflines(bioseq, deflines)) {
+        	deflines.Reset();
+        }
+
         if (bin_hdr.empty() && deflines.Empty()) {
             x_BuildDeflinesFromBioseq(*bioseq,
                                       deflines,
@@ -864,6 +891,10 @@ void CWriteDB_Impl::x_CookSequence()
             WriteDB_Ncbi4naToBinary(si, m_Sequence, m_Ambig);
             break;
             
+        case CSeq_data::e_Iupacna:
+             WriteDB_IupacnaToBinary(si, m_Sequence, m_Ambig);
+             break;
+
         default:
             msg = "Need to write conversion for data type [";
             msg += NStr::IntToString((int) sd.Which());
@@ -1010,7 +1041,8 @@ void CWriteDB_Impl::x_Publish()
             
             for(size_t i = 0; i < m_ColumnTitles.size(); i++) {
                 m_Volume->CreateColumn(m_ColumnTitles[i],
-                                       m_ColumnMetas[i]);
+                                       m_ColumnMetas[i],
+                                       m_MaxFileSize);
             }
 #endif
         }
@@ -1553,9 +1585,9 @@ x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
 
         string title(fasta, 1, fasta.size());
         // Replace ^A with space
-        for(size_t pos=0; pos < title.size(); ++pos) {
-            if (title[pos] == '\001') title[pos] = ' ';
-        }
+        NStr::ReplaceInPlace(title, "\001", " ");
+        // Replace tabs with three spaces
+        NStr::ReplaceInPlace(title, "\t", "   ");
         defline->SetTitle(title);
 
         if (mship_i < membits.size()) {
