@@ -1,4 +1,4 @@
-/*  $Id: writedb_volume.cpp 216989 2010-12-13 14:02:20Z maning $
+/*  $Id: writedb_volume.cpp 387632 2013-01-30 22:55:42Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
 /// class for WriteDB.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: writedb_volume.cpp 216989 2010-12-13 14:02:20Z maning $";
+static char const rcsid[] = "$Id: writedb_volume.cpp 387632 2013-01-30 22:55:42Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -218,17 +218,23 @@ bool CWriteDB_Volume::WriteSequence(const string      & seq,
     
     // check the uniqueness of id
     if (m_Indices != CWriteDB::eNoIndex) {
+    	set<string>::size_type orig_size = m_IdSet.size();
+    	string id_u;
         ITERATE(TIdList, iter, idlist) {
             string id = (*iter)->AsFastaString();
-            string id_u = NStr::ToUpper(id);
+            id_u = NStr::ToUpper(id);
             if (m_IdSet.find(id_u) != m_IdSet.end() ) {
-                CNcbiOstrstream msg;
-                msg << "Error: Duplicate seq_ids are found: " << endl
-                    << id_u << endl;
-                NCBI_THROW(CWriteDBException, eArgErr, CNcbiOstrstreamToString(msg));
+            	continue;
             }
             m_IdSet.insert(id_u);
         }
+
+        if(m_IdSet.size() == orig_size) {
+        	CNcbiOstrstream msg;
+        	msg << "Error: Duplicate seq_ids are found: " << endl
+    	    << id_u << endl;
+        	NCBI_THROW(CWriteDBException, eArgErr, CNcbiOstrstreamToString(msg));
+        }
     }
 
     int off_hdr(0), off_seq(0), off_amb(0);
@@ -408,6 +414,7 @@ void CWriteDB_Volume::ListFiles(vector<string> & files) const
      (!defined(NCBI_COMPILER_MIPSPRO)) )
 int CWriteDB_Volume::CreateColumn(const string      & title,
                                   const TColumnMeta & meta,
+                                  Uint8               max_sz,
                                   bool                mbo)
 {
     int col_id = m_Columns.size();
@@ -436,13 +443,13 @@ int CWriteDB_Volume::CreateColumn(const string      & title,
                              m_Index,
                              title,
                              meta,
-                             m_MaxFileSize));
+                             max_sz));
 
     /* For support of multiple byte orders */
     if (mbo) new_col->AddByteOrder(m_DbName,
                              extn3,
                              m_Index,
-                             m_MaxFileSize);
+                             max_sz);
     
     // If the OID is not zero, then add all the blank records for the
     // prior OIDs to the new column.
