@@ -1,4 +1,4 @@
-/*  $Id: local_blastdb_adapter.cpp 368105 2012-07-02 20:06:14Z rafanovi $
+/*  $Id: local_blastdb_adapter.cpp 368230 2012-07-05 14:56:56Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  * Defines the CLocalBlastDbAdapter class
  */
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: local_blastdb_adapter.cpp 368105 2012-07-02 20:06:14Z rafanovi $";
+static char const rcsid[] = "$Id: local_blastdb_adapter.cpp 368230 2012-07-05 14:56:56Z camacho $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -55,7 +55,7 @@ CLocalBlastDbAdapter::GetSequenceType()
 int 
 CLocalBlastDbAdapter::GetTaxId(const CSeq_id_Handle& idh)
 {
-    int retval = kInvalidSeqPos;
+    int retval = static_cast<int>(kInvalidSeqPos);
     CConstRef<CSeq_id> id = idh.GetSeqId();
     if (id.NotEmpty()) {
         int oid = 0;
