@@ -1,4 +1,4 @@
-/*  $Id: reader_cache.cpp 348284 2011-12-27 20:42:58Z kazimird $
+/*  $Id: reader_cache.cpp 376549 2012-10-02 13:13:46Z ivanov $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -64,8 +64,9 @@ NCBI_PARAM_DEF_EX(int, GENBANK, CACHE_DEBUG, 0,
 
 int SCacheInfo::GetDebugLevel(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, CACHE_DEBUG) s_Value;
-    return s_Value.Get();
+    static const int s_Value =
+        NCBI_PARAM_TYPE(GENBANK, CACHE_DEBUG)::GetDefault();
+    return s_Value;
 }
 
 const int    SCacheInfo::IDS_MAGIC = 0x32fd0105;
@@ -793,7 +794,7 @@ bool CCacheReader::LoadChunk(CReaderRequestResult& result,
     }
 
     CLoadLockBlob blob(result, blob_id);
-    if ( CProcessor::IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( CProcessor::IsLoaded(result, blob_id, chunk_id, blob) ) {
         return true;
     }
 
