@@ -1,4 +1,4 @@
-/*  $Id: writer_cache.cpp 213894 2010-11-29 20:08:27Z vasilche $
+/*  $Id: writer_cache.cpp 330387 2011-08-11 16:49:59Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -234,10 +234,13 @@ void CCacheWriter::SaveStringGi(CReaderRequestResult& result,
         CStoreBuffer str;
         str.StoreInt4(ids->GetGi());
         try {
+            if ( GetDebugLevel() ) {
+                LOG_POST(Info<<"CCache:Write: "<<seq_id<<","<<GetGiSubkey());
+            }
             m_IdCache->Store(seq_id, 0, GetGiSubkey(),
                              str.data(), str.size());
         }
-        catch ( ... ) { // ignored
+        catch ( exception& ) { // ignored
         }
     }
 }
@@ -267,10 +270,13 @@ void CCacheWriter::SaveSeq_idGi(CReaderRequestResult& result,
         CStoreBuffer str;
         str.StoreInt4(ids->GetGi());
         try {
+            if ( GetDebugLevel() ) {
+                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetGiSubkey());
+            }
             m_IdCache->Store(GetIdKey(seq_id), 0, GetGiSubkey(),
                              str.data(), str.size());
         }
-        catch ( ... ) { // ignored
+        catch ( exception& ) { // ignored
         }
     }
 }
@@ -290,10 +296,13 @@ void CCacheWriter::SaveSeq_idAccVer(CReaderRequestResult& result,
             str = acc.AsString();
         }
         try {
+            if ( GetDebugLevel() ) {
+                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetAccVerSubkey());
+            }
             m_IdCache->Store(GetIdKey(seq_id), 0, GetAccVerSubkey(),
                              str.data(), str.size());
         }
-        catch ( ... ) { // ignored
+        catch ( exception& ) { // ignored
         }
     }
 }
@@ -310,10 +319,13 @@ void CCacheWriter::SaveSeq_idLabel(CReaderRequestResult& result,
     if ( ids->IsLoadedLabel() ) {
         const string& str = ids->GetLabel();
         try {
+            if ( GetDebugLevel() ) {
+                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetLabelSubkey());
+            }
             m_IdCache->Store(GetIdKey(seq_id), 0, GetLabelSubkey(),
                              str.data(), str.size());
         }
-        catch ( ... ) { // ignored
+        catch ( exception& ) { // ignored
         }
     }
 }
@@ -331,10 +343,13 @@ void CCacheWriter::SaveSeq_idTaxId(CReaderRequestResult& result,
         CStoreBuffer str;
         str.StoreInt4(ids->GetTaxId());
         try {
+            if ( GetDebugLevel() ) {
+                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetTaxIdSubkey());
+            }
             m_IdCache->Store(GetIdKey(seq_id), 0, GetTaxIdSubkey(),
                              str.data(), str.size());
         }
-        catch ( ... ) { // ignored
+        catch ( exception& ) { // ignored
         }
     }
 }
@@ -353,7 +368,7 @@ namespace {
                 try {
                     m_Cache->Remove(m_Key, m_Version, m_Subkey);
                 }
-                catch (...) { // ignored
+                catch ( exception& ) { // ignored
                 }
                 m_Cache = 0;
             }
@@ -384,30 +399,29 @@ void CCacheWriter::WriteSeq_ids(const string& key,
     }
 
     try {
+        if ( GetDebugLevel() ) {
+            LOG_POST(Info<<"CCache:Write: "<<key<<","<<GetSeq_idsSubkey());
+        }
         auto_ptr<IWriter> writer
             (m_IdCache->GetWriteStream(key, 0, GetSeq_idsSubkey()));
         if ( !writer.get() ) {
             return;
         }
 
-        {{
-            CWStream w_stream(writer.get());
-            CObjectOStreamAsnBinary obj_stream(w_stream);
-            static_cast<CObjectOStream&>(obj_stream).WriteUint4(ids->size());
-            ITERATE ( CLoadInfoSeq_ids, it, *ids ) {
-                obj_stream << *it->GetSeqId();
-            }
-        }}
-
-        writer.reset();
+        CWStream w_stream(writer.release(), 0, 0, CRWStreambuf::fOwnAll);
+        CObjectOStreamAsnBinary obj_stream(w_stream);
+        static_cast<CObjectOStream&>(obj_stream).WriteUint4(ids->size());
+        ITERATE ( CLoadInfoSeq_ids, it, *ids ) {
+            obj_stream << *it->GetSeqId();
+        }
     }
-    catch ( ... ) {
+    catch ( exception& ) {
         // In case of an error we need to remove incomplete data
         // from the cache.
         try {
             m_BlobCache->Remove(key, 0, GetSeq_idsSubkey());
         }
-        catch ( ... ) { // ignored
+        catch ( exception& ) { // ignored
         }
         // ignore cache write error - it doesn't affect application
     }
@@ -449,9 +463,12 @@ void CCacheWriter::SaveSeq_idBlob_ids(CReaderRequestResult& result,
         str.StoreString(true_subkey);
     }
     try {
+        if ( GetDebugLevel() ) {
+            LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<subkey);
+        }
         m_IdCache->Store(GetIdKey(seq_id), 0, subkey, str.data(), str.size());
     }
-    catch ( ... ) { // ignored
+    catch ( exception& ) { // ignored
     }
 }
 
@@ -467,10 +484,13 @@ void CCacheWriter::SaveBlobVersion(CReaderRequestResult& /*result*/,
     CStoreBuffer str;
     str.StoreInt4(version);
     try {
+        if ( GetDebugLevel() ) {
+            LOG_POST(Info<<"CCache:Write: "<<GetBlobKey(blob_id)<<","<<GetBlobVersionSubkey());
+        }
         m_IdCache->Store(GetBlobKey(blob_id), 0, GetBlobVersionSubkey(),
                          str.data(), str.size());
     }
-    catch ( ... ) { // ignored
+    catch ( exception& ) { // ignored
     }
 }
 
@@ -485,6 +505,12 @@ public:
         : m_Cache(cache), m_Key(key), m_Version(version), m_Subkey(subkey),
           m_Writer(cache->GetWriteStream(key, version, subkey))
         {
+            if ( SCacheInfo::GetDebugLevel() ) {
+                LOG_POST(Info<<"CCache:Write: "<<key<<","<<subkey<<","<<version);
+            }
+            if ( version == -1 ) {
+                ERR_POST("CCache:Write: "<<key<<","<<subkey<<","<<version);
+            }
             if ( m_Writer.get() ) {
                 m_Stream.reset(new CWStream(m_Writer.get()));
             }
@@ -529,7 +555,7 @@ public:
             try {
                 m_Cache->Remove(m_Key, m_Version, m_Subkey);
             }
-            catch (...) { // ignored
+            catch ( exception& ) { // ignored
             }
         }
 
@@ -566,7 +592,7 @@ CCacheWriter::OpenBlobStream(CReaderRequestResult& result,
         WriteProcessorTag(**stream, processor);
         return stream;
     }
-    catch ( ... ) { // ignored
+    catch ( exception& ) { // ignored
         return null;
     }
 }
