@@ -1,4 +1,4 @@
-/*  $Id: dispatcher.cpp 330387 2011-08-11 16:49:59Z vasilche $
+/*  $Id: dispatcher.cpp 390318 2013-02-26 21:04:57Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -129,8 +129,9 @@ void CGBRequestStatistics::PrintStatistics(void)
 inline
 int CReadDispatcher::CollectStatistics(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, READER_STATS) s_Value;
-    return s_Value.Get();
+    static const int s_Value =
+        NCBI_PARAM_TYPE(GENBANK, READER_STATS)::GetDefault();
+    return s_Value;
 }
 
 
@@ -482,22 +483,40 @@ namespace {
         TLock m_Lock;
     };
 
+    bool s_Blob_idsLoaded(CLoadLockBlob_ids& ids, CLoadLockSeq_ids& seq_ids)
+    {
+        if ( ids.IsLoaded() ) {
+            return true;
+        }
+        // check if seq-id is known as absent
+        if ( seq_ids.IsLoaded() &&
+             (seq_ids->GetState() & CBioseq_Handle::fState_no_data) ) {
+            // mark blob-ids as absent too
+            ids->SetState(seq_ids->GetState());
+            ids.SetLoaded();
+            return true;
+        }
+        return false;
+    }
+
     class CCommandLoadSeq_idBlob_ids : public CReadDispatcherCommand
     {
     public:
         typedef CSeq_id_Handle TKey;
+        typedef CLoadLockSeq_ids TPreLock;
         typedef CLoadLockBlob_ids TLock;
         CCommandLoadSeq_idBlob_ids(CReaderRequestResult& result,
                                    const TKey& key,
                                    const SAnnotSelector* sel)
             : CReadDispatcherCommand(result),
-              m_Key(key), m_Selector(sel), m_Lock(result, key, sel)
+              m_Key(key), m_Selector(sel),
+              m_PreLock(result, key), m_Lock(result, key, sel)
             {
             }
 
         bool IsDone(void)
             {
-                return m_Lock.IsLoaded();
+                return s_Blob_idsLoaded(m_Lock, m_PreLock);
             }
         bool Execute(CReader& reader)
             {
@@ -521,6 +540,7 @@ namespace {
     private:
         TKey m_Key;
         const SAnnotSelector* m_Selector;
+        TPreLock m_PreLock;
         TLock m_Lock;
     };
 
@@ -802,20 +822,21 @@ namespace {
     {
     public:
         typedef CSeq_id_Handle TKey;
+        typedef CLoadLockSeq_ids TSeqIds;
         typedef CLoadLockBlob_ids TIds;
         typedef CReadDispatcher::TContentsMask TMask;
         CCommandLoadSeq_idBlobs(CReaderRequestResult& result,
                                 const TKey& key, TMask mask,
                                 const SAnnotSelector* sel)
             : CReadDispatcherCommand(result),
-              m_Key(key), m_Ids(result, key, sel),
+              m_Key(key), m_SeqIds(result, key), m_Ids(result, key, sel),
               m_Mask(mask), m_Selector(sel)
             {
             }
 
         bool IsDone(void)
             {
-                return m_Ids.IsLoaded() &&
+                return s_Blob_idsLoaded(m_Ids, m_SeqIds) &&
                     s_AllBlobsAreLoaded(GetResult(),
                                         m_Ids, m_Mask, m_Selector);
             }
@@ -840,6 +861,7 @@ namespace {
         
     private:
         TKey m_Key;
+        TSeqIds m_SeqIds;
         TIds m_Ids;
         TMask m_Mask;
         const SAnnotSelector* m_Selector;
@@ -1037,8 +1059,9 @@ namespace {
             {
                 CReaderRequestResult& result = GetResult();
                 ITERATE(TIds, id, m_Ids) {
+                    CLoadLockSeq_ids seq_ids(result, *id);
                     CLoadLockBlob_ids blob_ids(result, *id, 0);
-                    if ( !blob_ids.IsLoaded() ) {
+                    if ( !s_Blob_idsLoaded(blob_ids, seq_ids) ) {
                         return false;
                     }
                     ITERATE ( CLoadInfoBlob_ids, it, *blob_ids ) {
@@ -1147,6 +1170,7 @@ void CReadDispatcher::Process(CReadDispatcherCommand& command,
                 }
                 else {
                     if ( retry_count >= max_retry_count &&
+                         !command.MayBeSkipped() &&
                          !reader.MayBeSkippedOnErrors() ) {
                         throw;
                     }
@@ -1157,6 +1181,7 @@ void CReadDispatcher::Process(CReadDispatcherCommand& command,
             catch ( CException& exc ) {
                 // error in the command
                 if ( retry_count >= max_retry_count &&
+                     !command.MayBeSkipped() &&
                      !reader.MayBeSkippedOnErrors() ) {
                     throw;
                 }
@@ -1166,6 +1191,7 @@ void CReadDispatcher::Process(CReadDispatcherCommand& command,
             catch ( exception& exc ) {
                 // error in the command
                 if ( retry_count >= max_retry_count &&
+                     !command.MayBeSkipped() &&
                      !reader.MayBeSkippedOnErrors() ) {
                     throw;
                 }
@@ -1176,7 +1202,8 @@ void CReadDispatcher::Process(CReadDispatcherCommand& command,
                 return;
             }
         } while ( retry_count < max_retry_count );
-        if ( !reader.MayBeSkippedOnErrors() ) {
+        if ( !command.MayBeSkipped() &&
+             !reader.MayBeSkippedOnErrors() ) {
             NCBI_THROW(CLoaderException, eLoaderFailed, command.GetErrMsg());
         }
     }
