@@ -1,4 +1,4 @@
-/*  $Id: gbloader.cpp 368697 2012-07-11 16:45:03Z vasilche $
+/*  $Id: gbloader.cpp 390318 2013-02-26 21:04:57Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -109,6 +109,7 @@ public:
         {
             return *m_Loader;
         }
+    virtual CGBDataLoader* GetLoaderPtr(void);
 
     //virtual TConn GetConn(void);
     //virtual void ReleaseConn(void);
@@ -715,6 +716,7 @@ CConstRef<CSeqref> CGBDataLoader::GetSatSatkey(const CSeq_id& id)
 CDataLoader::TBlobId CGBDataLoader::GetBlobId(const CSeq_id_Handle& sih)
 {
     CGBReaderRequestResult result(this, sih);
+    CLoadLockSeq_ids seq_ids(result, sih);
     CLoadLockBlob_ids blobs(result, sih, 0);
     m_Dispatcher->LoadSeq_idBlob_ids(result, sih, 0);
 
@@ -1206,6 +1208,7 @@ CGBDataLoader::x_GetRecords(const CSeq_id_Handle& sih,
     }
 
     CGBReaderRequestResult result(this, sih);
+    CLoadLockSeq_ids seq_ids(result, sih);
     m_Dispatcher->LoadBlobs(result, sih, mask, sel);
     CLoadLockBlob_ids blobs(result, sih, sel);
     _ASSERT(blobs.IsLoaded());
@@ -1243,6 +1246,7 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih)
     TNamedAnnotNames names;
 
     CGBReaderRequestResult result(this, sih);
+    CLoadLockSeq_ids seq_ids(result, sih);
     SAnnotSelector sel;
     sel.IncludeNamedAnnotAccession("NA*");
     CLoadLockBlob_ids blobs(result, sih, &sel);
@@ -1275,6 +1279,7 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih,
     TNamedAnnotNames names;
 
     CGBReaderRequestResult result(this, sih);
+    CLoadLockSeq_ids seq_ids(result, sih);
     SAnnotSelector sel;
     if ( !ExtractZoomLevel(named_acc, 0, 0) ) {
         sel.IncludeNamedAnnotAccession(CombineWithZoomLevel(named_acc, -1));
@@ -1307,10 +1312,16 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih,
 
 void CGBDataLoader::GetChunk(TChunk chunk)
 {
-    CGBReaderRequestResult result(this, CSeq_id_Handle());
-    m_Dispatcher->LoadChunk(result,
-                            GetRealBlobId(chunk->GetBlobId()),
-                            chunk->GetChunkId());
+    CReader::TChunkId id = chunk->GetChunkId();
+    if ( id == CProcessor::kMasterWGS_ChunkId ) {
+        CProcessor::LoadWGSMaster(this, chunk);
+    }
+    else {
+        CGBReaderRequestResult result(this, CSeq_id_Handle());
+        m_Dispatcher->LoadChunk(result,
+                                GetRealBlobId(chunk->GetBlobId()),
+                                id);
+    }
 }
 
 
@@ -1319,8 +1330,13 @@ void CGBDataLoader::GetChunks(const TChunkSet& chunks)
     typedef map<TBlobId, CReader::TChunkIds> TChunkIdMap;
     TChunkIdMap chunk_ids;
     ITERATE(TChunkSet, it, chunks) {
-        chunk_ids[(*it)->GetBlobId()].push_back(
-            (*it)->GetChunkId());
+        CReader::TChunkId id = (*it)->GetChunkId();
+        if ( id == CProcessor::kMasterWGS_ChunkId ) {
+            CProcessor::LoadWGSMaster(this, *it);
+        }
+        else {
+            chunk_ids[(*it)->GetBlobId()].push_back(id);
+        }
     }
     ITERATE(TChunkIdMap, it, chunk_ids) {
         CGBReaderRequestResult result(this, CSeq_id_Handle());
@@ -1337,6 +1353,7 @@ void CGBDataLoader::GetBlobs(TTSE_LockSets& tse_sets)
     TBlobContentsMask mask = fBlobHasCore;
     CReadDispatcher::TIds ids;
     ITERATE(TTSE_LockSets, tse_set, tse_sets) {
+        CLoadLockSeq_ids seq_ids_lock(result, tse_set->first);
         CLoadLockBlob_ids blob_ids_lock(result, tse_set->first, 0);
         ids.push_back(tse_set->first);
     }
@@ -1445,6 +1462,12 @@ CGBReaderRequestResult::~CGBReaderRequestResult(void)
 }
 
 
+CGBDataLoader* CGBReaderRequestResult::GetLoaderPtr(void)
+{
+    return m_Loader;
+}
+
+
 CRef<CLoadInfoSeq_ids>
 CGBReaderRequestResult::GetInfoSeq_ids(const string& key)
 {
