@@ -1,4 +1,4 @@
-/*  $Id: processors.cpp 370295 2012-07-27 16:38:16Z vasilche $
+/*  $Id: processors.cpp 390318 2013-02-26 21:04:57Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -41,6 +41,7 @@
 
 #include <objtools/data_loaders/genbank/reader_snp.hpp>
 #include <objtools/data_loaders/genbank/split_parser.hpp>
+#include <objtools/data_loaders/genbank/gbloader.hpp>
 #include <objtools/error_codes.hpp>
 
 #include <objmgr/impl/tse_split_info.hpp>
@@ -51,8 +52,7 @@
 #include <objects/seqsplit/ID2S_Feat_type_Info.hpp>
 #include <objects/seqsplit/ID2S_Chunk.hpp>
 // for read hooks setup
-#include <objects/general/Dbtag.hpp>
-#include <objects/general/Object_id.hpp>
+#include <objects/general/general__.hpp>
 #include <objects/seqfeat/Seq_feat.hpp>
 #include <objects/seqfeat/Gb_qual.hpp>
 #include <objects/seqfeat/Imp_feat.hpp>
@@ -91,6 +91,8 @@ NCBI_PARAM_DEF_EX(int, GENBANK, READER_STATS, 0,
                   eParam_NoThread, GENBANK_READER_STATS);
 NCBI_PARAM_DEF_EX(bool, GENBANK, CACHE_RECOMPRESS, true,
                   eParam_NoThread, GENBANK_CACHE_RECOMPRESS);
+NCBI_PARAM_DEF_EX(bool, GENBANK, ADD_WGS_MASTER, true,
+                  eParam_NoThread, GENBANK_ADD_WGS_MASTER);
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -218,8 +220,9 @@ namespace {
 inline
 int CProcessor::CollectStatistics(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, READER_STATS) s_Value;
-    return s_Value.Get();
+    static const int s_Value =
+        NCBI_PARAM_TYPE(GENBANK, READER_STATS)::GetDefault();
+    return s_Value;
 }
 
 
@@ -327,29 +330,41 @@ bool CProcessor::TryStringPack(void)
 
 bool CProcessor::TrySNPSplit(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, SNP_SPLIT) s_Value;
-    return s_Value.Get();
+    static const bool s_Value =
+        NCBI_PARAM_TYPE(GENBANK, SNP_SPLIT)::GetDefault();
+    return s_Value;
 }
 
 
 bool CProcessor::TrySNPTable(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, SNP_TABLE) s_Value;
-    return s_Value.Get();
+    static const bool s_Value =
+        NCBI_PARAM_TYPE(GENBANK, SNP_TABLE)::GetDefault();
+    return s_Value;
 }
 
 
 static bool s_UseMemoryPool(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, USE_MEMORY_POOL) s_Value;
-    return s_Value.Get();
+    static const bool s_Value =
+        NCBI_PARAM_TYPE(GENBANK, USE_MEMORY_POOL)::GetDefault();
+    return s_Value;
 }
 
 
 static bool s_CacheRecompress(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, CACHE_RECOMPRESS) s_Value;
-    return s_Value.Get();
+    static const bool s_Value =
+        NCBI_PARAM_TYPE(GENBANK, CACHE_RECOMPRESS)::GetDefault();
+    return s_Value;
+}
+
+
+static bool s_AddWGSMasterParam(void)
+{
+    static const bool s_Value =
+        NCBI_PARAM_TYPE(GENBANK, ADD_WGS_MASTER)::GetDefault();
+    return s_Value;
 }
 
 
@@ -410,12 +425,13 @@ CWriter* CProcessor::GetWriter(const CReaderRequestResult& result) const
 }
 
 
-bool CProcessor::IsLoaded(const TBlobId& /*blob_id*/,
+bool CProcessor::IsLoaded(CReaderRequestResult& result,
+                          const TBlobId& blob_id,
                           TChunkId chunk_id,
                           CLoadLockBlob& blob)
 {
     if ( chunk_id == kMain_ChunkId ) {
-        return blob.IsLoaded();
+        return result.IsBlobLoaded(blob_id);
     }
     else {
         return blob->GetSplitInfo().GetChunk(chunk_id).IsLoaded();
@@ -423,6 +439,220 @@ bool CProcessor::IsLoaded(const TBlobId& /*blob_id*/,
 }
 
 
+BEGIN_LOCAL_NAMESPACE;
+
+
+bool s_GoodLetters(CTempString s) {
+    ITERATE ( CTempString, it, s ) {
+        if ( !isalpha(*it & 0xff) ) {
+            return false;
+        }
+    }
+    return true;
+}
+
+
+bool s_GoodDigits(CTempString s) {
+    bool have_non_zero = false;
+    ITERATE ( CTempString, it, s ) {
+        if ( *it != '0' ) {
+            have_non_zero = true;
+            if ( !isdigit(*it & 0xff) ) {
+                return false;
+            }
+        }
+    }
+    return have_non_zero;
+}
+
+
+CSeq_id_Handle s_GetWGSMasterSeq_id(const CSeq_id_Handle& idh)
+{
+    CSeq_id_Handle master_idh;
+    if ( idh.Which() != CSeq_id::e_Genbank &&
+         idh.Which() != CSeq_id::e_Other ) {
+        return master_idh;
+    }
+    CConstRef<CSeq_id> id = idh.GetSeqId();
+    const CTextseq_id* text_id = id->GetTextseq_Id();
+    if ( !text_id || !text_id->IsSetAccession() ) {
+        return master_idh;
+    }
+
+    CTempString acc = text_id->GetAccession();
+    bool have_nz = NStr::StartsWith(acc, "NZ_");
+    SIZE_TYPE letters_pos = have_nz? 3: 0;
+    SIZE_TYPE digits_pos = letters_pos+4;
+    SIZE_TYPE digits_count = acc.size() - digits_pos;
+    if ( digits_count < 8 || digits_count > 10 ) {
+        return master_idh;
+    }
+    if ( !s_GoodLetters(acc.substr(letters_pos, 4)) ) {
+        return master_idh;
+    }
+    if ( !s_GoodDigits(acc.substr(digits_pos)) ) {
+        return master_idh;
+    }
+    int version = NStr::StringToNumeric<int>(acc.substr(digits_pos, 2));
+    Uint8 row_id = NStr::StringToNumeric<Uint8>(acc.substr(digits_pos+2));
+    if ( !version || !row_id ) {
+        return master_idh;
+    }
+    CSeq_id master_id;
+    CTextseq_id* master_text_id = 0;
+    if ( idh.Which() == CSeq_id::e_Genbank ) {
+        master_text_id = &master_id.SetGenbank();
+    }
+    else {
+        master_text_id = &master_id.SetOther();
+    }
+    string master_acc = acc.substr(0, digits_pos);
+    master_acc.resize(acc.size(), '0');
+    master_text_id->SetAccession(master_acc);
+    master_text_id->SetVersion(version);
+    master_idh = CSeq_id_Handle::GetHandle(master_id);
+    return master_idh;
+}
+
+
+inline
+int s_GetGoodDescrMask(void)
+{
+    return CSeqdesc::e_Pub | CSeqdesc::e_Comment | CSeqdesc::e_User;
+}
+
+
+bool s_IsGoodDescr(const CSeqdesc& desc)
+{
+    if ( desc.Which() == CSeqdesc::e_Pub ||
+         desc.Which() == CSeqdesc::e_Comment ) {
+        return true;
+    }
+    else if ( desc.Which() == CSeqdesc::e_User ) {
+        const CObject_id& type = desc.GetUser().GetType();
+        if ( type.Which() == CObject_id::e_Str ) {
+            const string& name = type.GetStr();
+            if ( name == "DBLink" ||
+                 name == "GenomeProjectsDB" ||
+                 name == "StructuredComment" ||
+                 name == "FeatureFetchPolicy" ) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+
+CRef<CSeq_descr> s_GetWGSMasterDescr(CDataLoader* loader,
+                                     const CSeq_id_Handle& master_idh)
+{
+    CRef<CSeq_descr> ret;
+    CDataLoader::TTSE_LockSet locks =
+        loader->GetRecordsNoBlobState(master_idh, CDataLoader::eBioseqCore);
+    ITERATE ( CDataLoader::TTSE_LockSet, it, locks ) {
+        CConstRef<CBioseq_Info> bs_info =
+            (*it)->FindMatchingBioseq(master_idh);
+        if ( !bs_info ) {
+            continue;
+        }
+        if ( bs_info->IsSetDescr() ) {
+            const CSeq_descr::Tdata& descr = bs_info->GetDescr().Get();
+            ITERATE ( CSeq_descr::Tdata, it, descr ) {
+                if ( s_IsGoodDescr(**it) ) {
+                    if ( !ret ) {
+                        ret = new CSeq_descr;
+                    }
+                    ret->Set().push_back(*it);
+                }
+            }
+        }
+        break;
+    }
+    return ret;
+}
+
+
+class CWGSBioseqUpdater_Base : public CBioseqUpdater
+{
+public:
+    CWGSBioseqUpdater_Base(const CSeq_id_Handle& master_idh)
+        : m_MasterId(master_idh)
+        {
+        }
+
+    bool HasMasterId(const CBioseq_Info& seq) const {
+        if ( m_MasterId ) {
+            const CBioseq_Info::TId& ids = seq.GetId();
+            ITERATE ( CBioseq_Info::TId, it, ids ) {
+                if ( s_GetWGSMasterSeq_id(*it) == m_MasterId ) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+    
+private:
+    CSeq_id_Handle m_MasterId;
+};
+
+
+class CWGSBioseqUpdaterChunk : public CWGSBioseqUpdater_Base
+{
+public:
+    CWGSBioseqUpdaterChunk(const CSeq_id_Handle& master_idh)
+        : CWGSBioseqUpdater_Base(master_idh)
+        {
+        }
+
+    virtual void Update(CBioseq_Info& seq) {
+        if ( HasMasterId(seq) ) {
+            // register master descr chunk
+            seq.x_AddDescrChunkId(s_GetGoodDescrMask(),
+                                  CProcessor::kMasterWGS_ChunkId);
+        }
+    }
+};
+
+
+class CWGSBioseqUpdaterDescr : public CWGSBioseqUpdater_Base
+{
+public:
+    CWGSBioseqUpdaterDescr(const CSeq_id_Handle& master_idh,
+                           CRef<CSeq_descr> descr)
+        : CWGSBioseqUpdater_Base(master_idh),
+          m_Descr(descr)
+        {
+        }
+
+    virtual void Update(CBioseq_Info& seq) {
+        if ( m_Descr && HasMasterId(seq) ) {
+            seq.AddSeq_descr(*m_Descr);
+        }
+    }
+
+private:
+    CRef<CSeq_descr> m_Descr;
+};
+
+
+class CWGSMasterChunkInfo : public CTSE_Chunk_Info
+{
+public:
+    CWGSMasterChunkInfo(const CSeq_id_Handle& master_idh)
+        : CTSE_Chunk_Info(CProcessor::kMasterWGS_ChunkId),
+          m_MasterId(master_idh)
+        {
+        }
+
+    CSeq_id_Handle m_MasterId;
+};
+
+
+END_LOCAL_NAMESPACE;
+
+
 void CProcessor::SetLoaded(CReaderRequestResult& result,
                            const TBlobId& blob_id,
                            TChunkId chunk_id,
@@ -442,6 +672,41 @@ void CProcessor::SetLoaded(CReaderRequestResult& result,
 }
 
 
+void CProcessor::LoadWGSMaster(CDataLoader* loader,
+                               CRef<CTSE_Chunk_Info> chunk)
+{
+    CSeq_id_Handle id = dynamic_cast<CWGSMasterChunkInfo&>(*chunk).m_MasterId;
+    CRef<CSeq_descr> descr = s_GetWGSMasterDescr(loader, id);
+    CRef<CBioseqUpdater> upd(new CWGSBioseqUpdaterDescr(id, descr));
+    const_cast<CTSE_Split_Info&>(chunk->GetSplitInfo()).x_SetBioseqUpdater(upd);
+    chunk->SetLoaded();
+}
+
+
+void CProcessor::AddWGSMaster(CReaderRequestResult& result,
+                              const TBlobId& /*blob_id*/,
+                              TChunkId chunk_id,
+                              CLoadLockBlob& blob)
+{
+    if ( chunk_id == kMain_ChunkId &&
+         !blob.IsLoaded() &&
+         s_AddWGSMasterParam() &&
+         result.GetLoaderPtr() ) {
+        CTSE_Info::TSeqIds ids;
+        blob->GetBioseqsIds(ids);
+        ITERATE ( CTSE_Info::TSeqIds, it, ids ) {
+            if ( CSeq_id_Handle id = s_GetWGSMasterSeq_id(*it) ) {
+                CRef<CTSE_Chunk_Info> chunk(new CWGSMasterChunkInfo(id));
+                blob->GetSplitInfo().AddChunk(*chunk);
+                CRef<CBioseqUpdater> upd(new CWGSBioseqUpdaterChunk(id));
+                blob->SetBioseqUpdater(upd);
+                break;
+            }
+        }
+    }
+}
+
+
 void CProcessor::SetSeq_entry(CReaderRequestResult& /*result*/,
                               const TBlobId& /*blob_id*/,
                               TChunkId chunk_id,
@@ -519,7 +784,7 @@ void CProcessor_ID1::ProcessObjStream(CReaderRequestResult& result,
                                       CObjectIStream& obj_stream) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_ID1: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -741,7 +1006,7 @@ void CProcessor_ID1_SNP::ProcessObjStream(CReaderRequestResult& result,
                                           CObjectIStream& obj_stream) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_ID1_SNP: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -831,7 +1096,7 @@ void CProcessor_SE::ProcessObjStream(CReaderRequestResult& result,
                                      CObjectIStream& obj_stream) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_SE: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -880,6 +1145,7 @@ void CProcessor_SE::ProcessObjStream(CReaderRequestResult& result,
     }}
 
     SetSeq_entry(result, blob_id, chunk_id, blob, seq_entry);
+    AddWGSMaster(result, blob_id, chunk_id, blob);
     SetLoaded(result, blob_id, chunk_id, blob);
 }
 
@@ -919,7 +1185,7 @@ void CProcessor_SE_SNP::ProcessObjStream(CReaderRequestResult& result,
                                          CObjectIStream& obj_stream) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_SE_SNP: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -1019,7 +1285,7 @@ void CProcessor_St_SE::ProcessObjStream(CReaderRequestResult& result,
                                         CObjectIStream& obj_stream) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_St_SE: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -1229,7 +1495,7 @@ void CProcessor_St_SE_SNPT::ProcessStream(CReaderRequestResult& result,
                                           CNcbiIstream& stream) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_St_SE_SNPT: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -1349,7 +1615,7 @@ void CProcessor_ID2::ProcessData(CReaderRequestResult& result,
                                  const CID2_Reply_Data* skel) const
 {
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_ID2: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -1512,6 +1778,7 @@ void CProcessor_ID2::ProcessData(CReaderRequestResult& result,
                        "CProcessor_ID2: "
                        "invalid data type: "<<data.GetData_type());
     }
+    AddWGSMaster(result, blob_id, chunk_id, blob);
     SetLoaded(result, blob_id, chunk_id, blob);
 }
 
@@ -1988,7 +2255,7 @@ void CProcessor_ExtAnnot::Process(CReaderRequestResult& result,
                        "bad blob "<<blob_id<<'/'<<chunk_id);
     }
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, chunk_id, blob) ) {
+    if ( IsLoaded(result, blob_id, chunk_id, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_ExtAnnot: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -2124,7 +2391,7 @@ void CProcessor_AnnotInfo::LoadBlob(CReaderRequestResult& result,
 {
     _ASSERT(info.IsSetAnnotInfo());
     CLoadLockBlob blob(result, blob_id);
-    if ( IsLoaded(blob_id, kMain_ChunkId, blob) ) {
+    if ( IsLoaded(result, blob_id, kMain_ChunkId, blob) ) {
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_AnnotInfo: "
                        "double load of "<<blob_id);
