@@ -1,4 +1,4 @@
-/*  $Id: reader_pubseq.cpp 366478 2012-06-14 16:50:26Z rafanovi $
+/*  $Id: reader_pubseq.cpp 376549 2012-10-02 13:13:46Z ivanov $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -69,7 +69,7 @@
 #define DEFAULT_NUM_CONN    2
 #define MAX_MT_CONN         5
 #define DEFAULT_ALLOW_GZIP  true
-#define DEFAULT_EXCL_WGS_MASTER false
+#define DEFAULT_EXCL_WGS_MASTER true
 
 #define NCBI_USE_ERRCODE_X   Objtools_Rd_Pubseq
 
@@ -88,8 +88,9 @@ NCBI_PARAM_DEF_EX(int, GENBANK, PUBSEQOS_DEBUG, 0,
 
 static int GetDebugLevel(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, PUBSEQOS_DEBUG) s_Value;
-    return s_Value.Get();
+    static const int s_Value =
+        NCBI_PARAM_TYPE(GENBANK, PUBSEQOS_DEBUG)::GetDefault();
+    return s_Value;
 }
 #else
 # define GetDebugLevel() (0)
