@@ -1,4 +1,4 @@
-/*  $Id: reader_id2_base.cpp 370295 2012-07-27 16:38:16Z vasilche $
+/*  $Id: reader_id2_base.cpp 390318 2013-02-26 21:04:57Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -91,8 +91,9 @@ NCBI_PARAM_DEF_EX(int, GENBANK, ID2_MAX_IDS_REQUEST_SIZE, 100,
 
 int CId2ReaderBase::GetDebugLevel(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, ID2_DEBUG) s_Value;
-    return s_Value.Get();
+    static const int s_Value =
+        NCBI_PARAM_TYPE(GENBANK, ID2_DEBUG)::GetDefault();
+    return s_Value;
 }
 
 
@@ -132,15 +133,19 @@ CId2ReaderBase::CDebugPrinter::~CDebugPrinter()
 // 1 = do not use packets or get-chunks requests
 static size_t GetMaxChunksRequestSize(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, ID2_MAX_CHUNKS_REQUEST_SIZE) s_Value;
-    return s_Value.Get();
+    static const size_t s_Value =
+        (size_t)
+        NCBI_PARAM_TYPE(GENBANK, ID2_MAX_CHUNKS_REQUEST_SIZE)::GetDefault();
+    return s_Value;
 }
 
 
 static size_t GetMaxIdsRequestSize(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, ID2_MAX_IDS_REQUEST_SIZE) s_Value;
-    return s_Value.Get();
+    static size_t s_Value =
+        (size_t)
+        NCBI_PARAM_TYPE(GENBANK, ID2_MAX_IDS_REQUEST_SIZE)::GetDefault();
+    return s_Value;
 }
 
 
@@ -842,12 +847,12 @@ bool CId2ReaderBase::LoadBlobs(CReaderRequestResult& result,
         if ( CProcessor_ExtAnnot::IsExtAnnot(blob_id) ) {
             const int chunk_id = CProcessor::kMain_ChunkId;
             CLoadLockBlob blob(result, blob_id);
-            if ( !CProcessor_ExtAnnot::IsLoaded(blob_id, chunk_id, blob) ) {
+            if ( !CProcessor::IsLoaded(result, blob_id, chunk_id, blob) ) {
                 dynamic_cast<const CProcessor_ExtAnnot&>
                     (m_Dispatcher->GetProcessor(CProcessor::eType_ExtAnnot))
                     .Process(result, blob_id, CProcessor::kMain_ChunkId);
             }
-            _ASSERT(CProcessor_ExtAnnot::IsLoaded(blob_id, chunk_id, blob));
+            _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
             continue;
         }
 
@@ -884,12 +889,12 @@ bool CId2ReaderBase::LoadBlob(CReaderRequestResult& result,
     if ( CProcessor_ExtAnnot::IsExtAnnot(blob_id) ) {
         conn.Release();
         const int chunk_id = CProcessor::kMain_ChunkId;
-        if ( !CProcessor_ExtAnnot::IsLoaded(blob_id, chunk_id, blob) ) {
+        if ( !CProcessor::IsLoaded(result, blob_id, chunk_id, blob) ) {
             dynamic_cast<const CProcessor_ExtAnnot&>
                 (m_Dispatcher->GetProcessor(CProcessor::eType_ExtAnnot))
                 .Process(result, blob_id, chunk_id);
         }
-        _ASSERT(CProcessor_ExtAnnot::IsLoaded(blob_id, chunk_id, blob));
+        _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
         return true;
     }
 
@@ -1360,7 +1365,9 @@ void CId2ReaderBase::x_ProcessPacket(CReaderRequestResult& result,
             }
         }
         reply.Reset();
-        x_EndOfPacket(conn);
+        if ( conn.IsAllocated() ) {
+            x_EndOfPacket(conn);
+        }
     }
     catch ( exception& /*rethrown*/ ) {
         if ( GetDebugLevel() >= eTraceError ) {
@@ -1926,33 +1933,17 @@ void CId2ReaderBase::x_ProcessGetBlob(
     TChunkId chunk_id = CProcessor::kMain_ChunkId;
     const CID2_Blob_Id& src_blob_id = reply.GetBlob_id();
     TBlobId blob_id = GetBlobId(src_blob_id);
-    CLoadLockBlob blob(result, blob_id);
+
     if ( src_blob_id.IsSetVersion() && src_blob_id.GetVersion() > 0 ) {
         SetAndSaveBlobVersion(result, blob_id, src_blob_id.GetVersion());
     }
-    if ( blob.IsLoaded() ) {
-        if ( blob->x_NeedsDelayedMainChunk() ) {
-            chunk_id = CProcessor::kDelayedMain_ChunkId;
-        }
-        else {
-            m_AvoidRequest |= fAvoidRequest_nested_get_blob_info;
-            ERR_POST_X(4, Info << "CId2ReaderBase: ID2-Reply-Get-Blob: "
-                          "blob already loaded: "<<blob_id);
-            return;
-        }
-    }
-
-    if ( blob->HasSeq_entry() ) {
-        ERR_POST_X(5, "CId2ReaderBase: ID2-Reply-Get-Blob: "
-                      "Seq-entry already loaded: "<<blob_id);
-        return;
-    }
 
     TBlobState blob_state = x_GetBlobState(main_reply);
     if ( blob_state & CBioseq_Handle::fState_no_data ) {
+        CLoadLockBlob blob(result, blob_id);
         blob.SetBlobState(blob_state);
         SetAndSaveNoBlob(result, blob_id, chunk_id, blob);
-        _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
+        _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
         return;
     }
 
@@ -1975,21 +1966,42 @@ void CId2ReaderBase::x_ProcessGetBlob(
         }
         ERR_POST_X(6, "CId2ReaderBase: ID2-Reply-Get-Blob: "
                    "no data in reply: "<<blob_id);
+        CLoadLockBlob blob(result, blob_id);
         SetAndSaveNoBlob(result, blob_id, chunk_id, blob);
-        _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
+        _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
         return;
     }
 
-    if ( blob_state ) {
-        m_Dispatcher->SetAndSaveBlobState(result, blob_id, blob, blob_state);
-    }
-
     if ( reply.GetSplit_version() != 0 ) {
         // split info will follow
         // postpone parsing this blob
         loaded_set.m_Skeletons[blob_id] = &data;
         return;
     }
+
+    CLoadLockBlob blob(result, blob_id);
+    if ( blob.IsLoaded() ) {
+        if ( blob->x_NeedsDelayedMainChunk() ) {
+            chunk_id = CProcessor::kDelayedMain_ChunkId;
+        }
+        else {
+            m_AvoidRequest |= fAvoidRequest_nested_get_blob_info;
+            ERR_POST_X(4, Info << "CId2ReaderBase: ID2-Reply-Get-Blob: "
+                          "blob already loaded: "<<blob_id);
+            return;
+        }
+    }
+
+    if ( blob->HasSeq_entry() ) {
+        ERR_POST_X(5, "CId2ReaderBase: ID2-Reply-Get-Blob: "
+                      "Seq-entry already loaded: "<<blob_id);
+        return;
+    }
+
+    if ( blob_state ) {
+        m_Dispatcher->SetAndSaveBlobState(result, blob_id, blob, blob_state);
+    }
+
     if ( reply.GetBlob_id().GetSub_sat() == CID2_Blob_Id::eSub_sat_snp ) {
         m_Dispatcher->GetProcessor(CProcessor::eType_Seq_entry_SNP)
             .ProcessBlobFromID2Data(result, blob_id, chunk_id, data);
@@ -1999,7 +2011,7 @@ void CId2ReaderBase::x_ProcessGetBlob(
             (m_Dispatcher->GetProcessor(CProcessor::eType_ID2))
             .ProcessData(result, blob_id, blob_state, chunk_id, data);
     }
-    _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
+    _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
 }
 
 
@@ -2010,7 +2022,17 @@ void CId2ReaderBase::x_ProcessGetSplitInfo(
     const CID2S_Reply_Get_Split_Info& reply)
 {
     TChunkId chunk_id = CProcessor::kMain_ChunkId;
-    TBlobId blob_id = GetBlobId(reply.GetBlob_id());
+    const CID2_Blob_Id& src_blob_id = reply.GetBlob_id();
+    TBlobId blob_id = GetBlobId(src_blob_id);
+    if ( src_blob_id.IsSetVersion() && src_blob_id.GetVersion() > 0 ) {
+        SetAndSaveBlobVersion(result, blob_id, src_blob_id.GetVersion());
+    }
+    if ( !reply.IsSetData() ) {
+        ERR_POST_X(11, "CId2ReaderBase: ID2S-Reply-Get-Split-Info: "
+                       "no data in reply: "<<blob_id);
+        return;
+    }
+
     CLoadLockBlob blob(result, blob_id);
     if ( !blob ) {
         ERR_POST_X(9, "CId2ReaderBase: ID2S-Reply-Get-Split-Info: "
@@ -2022,24 +2044,14 @@ void CId2ReaderBase::x_ProcessGetSplitInfo(
             chunk_id = CProcessor::kDelayedMain_ChunkId;
         }
         else {
+            m_AvoidRequest |= fAvoidRequest_nested_get_blob_info;
             ERR_POST_X(10, Info<<"CId2ReaderBase: ID2S-Reply-Get-Split-Info: "
                        "blob already loaded: " << blob_id);
             return;
         }
     }
-    if ( !reply.IsSetData() ) {
-        ERR_POST_X(11, "CId2ReaderBase: ID2S-Reply-Get-Split-Info: "
-                       "no data in reply: "<<blob_id);
-        return;
-    }
 
     TBlobState blob_state = x_GetBlobState(main_reply);
-    if ( blob_state & CBioseq_Handle::fState_no_data ) {
-        blob.SetBlobState(blob_state);
-        SetAndSaveNoBlob(result, blob_id, chunk_id, blob);
-        _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
-        return;
-    }
     {{
         SId2LoadedSet::TBlobStates::iterator iter =
             loaded_set.m_BlobStates.find(blob_id);
@@ -2047,6 +2059,12 @@ void CId2ReaderBase::x_ProcessGetSplitInfo(
             blob_state |= iter->second;
         }
     }}
+    if ( blob_state & CBioseq_Handle::fState_no_data ) {
+        blob.SetBlobState(blob_state);
+        SetAndSaveNoBlob(result, blob_id, chunk_id, blob);
+        _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
+        return;
+    }
 
     CConstRef<CID2_Reply_Data> skel;
     {{
@@ -2066,7 +2084,7 @@ void CId2ReaderBase::x_ProcessGetSplitInfo(
         .ProcessData(result, blob_id, blob->GetBlobState(), chunk_id,
                      reply.GetData(), reply.GetSplit_version(), skel);
 
-    _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
+    _ASSERT(CProcessor::IsLoaded(result, blob_id, chunk_id, blob));
     loaded_set.m_Skeletons.erase(blob_id);
 }
 
