@@ -1,4 +1,4 @@
-/*  $Id: reader_service.cpp 366472 2012-06-14 16:48:16Z rafanovi $
+/*  $Id: reader_service.cpp 370497 2012-07-30 16:22:04Z grichenk $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -81,8 +81,9 @@ NCBI_PARAM_DEF_EX(int, GENBANK, CONN_DEBUG, 0,
 
 static int s_GetDebugLevel(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, CONN_DEBUG) s_Value;
-    return s_Value.Get();
+    static const int s_Value =
+        NCBI_PARAM_TYPE(GENBANK, CONN_DEBUG)::GetDefault();
+    return s_Value;
 }
 
 
