@@ -1,4 +1,4 @@
-/*  $Id: reader_snp.cpp 129783 2008-06-04 13:28:42Z vasilche $
+/*  $Id: reader_snp.cpp 370497 2012-07-30 16:22:04Z grichenk $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -84,8 +84,9 @@ NCBI_PARAM_DEF_EX(bool, GENBANK, SNP_TABLE_STAT, false,
 
 static bool CollectSNPStat(void)
 {
-    static NCBI_PARAM_TYPE(GENBANK, SNP_TABLE_STAT) s_Value;
-    return s_Value.Get();
+    static const bool s_Value =
+        NCBI_PARAM_TYPE(GENBANK, SNP_TABLE_STAT)::GetDefault();
+    return s_Value;
 }
 
 
@@ -208,8 +209,8 @@ CSNP_Seq_feat_hook::~CSNP_Seq_feat_hook(void)
 
 
 #ifdef _DEBUG
-NCBI_PARAM_DECL(int, GENBANK, SNP_TABLE_DUMP);
-NCBI_PARAM_DEF_EX(int, GENBANK, SNP_TABLE_DUMP, 0,
+NCBI_PARAM_DECL(bool, GENBANK, SNP_TABLE_DUMP);
+NCBI_PARAM_DEF_EX(bool, GENBANK, SNP_TABLE_DUMP, 0,
                   eParam_NoThread, GENBANK_SNP_TABLE_DUMP);
 #endif
 
@@ -229,8 +230,9 @@ void CSNP_Seq_feat_hook::ReadContainerElement(CObjectIStream& in,
     }
     else {
 #ifdef _DEBUG
-        static NCBI_PARAM_TYPE(GENBANK, SNP_TABLE_DUMP) s_Value;
-        if ( s_Value.Get() > 0 ) {
+        static const bool s_Value =
+            NCBI_PARAM_TYPE(GENBANK, SNP_TABLE_DUMP)::GetDefault();
+        if ( s_Value ) {
             NcbiCerr <<
                 "CSNP_Seq_feat_hook::ReadContainerElement: complex SNP: " <<
                 SSNP_Info::s_SNP_Type_Label[snp_type] << ":\n" << 
