@@ -1,4 +1,4 @@
-/*  $Id: request_result.cpp 368697 2012-07-11 16:45:03Z vasilche $
+/*  $Id: request_result.cpp 390834 2013-03-02 19:33:56Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -285,14 +285,13 @@ CLoadInfoLock::CLoadInfoLock(CReaderRequestResult& owner,
 
 CLoadInfoLock::~CLoadInfoLock(void)
 {
-    ReleaseLock();
-    m_Owner.ReleaseLoadLock(m_Info);
 }
 
 
 void CLoadInfoLock::ReleaseLock(void)
 {
     m_Guard.Release();
+    m_Owner.ReleaseLoadLock(m_Info);
 }
 
 
@@ -528,6 +527,11 @@ CLoadLockBlob::CLoadLockBlob(TMutexSource& src, const CBlob_id& blob_id)
 }
 #endif
 
+CLoadLockBlob::CLoadLockBlob(void)
+{
+}
+
+
 CLoadLockBlob::CLoadLockBlob(CReaderRequestResult& src,
                              const CBlob_id& blob_id)
     : CTSE_LoadLock(src.GetBlobLoadLock(blob_id))
@@ -596,11 +600,17 @@ CReaderRequestResult::CReaderRequestResult(const CSeq_id_Handle& requested_id)
 
 CReaderRequestResult::~CReaderRequestResult(void)
 {
-    _ASSERT(m_LockMap.empty());
+    ReleaseLocks();
     _ASSERT(!m_AllocatedConnection);
 }
 
 
+CGBDataLoader* CReaderRequestResult::GetLoaderPtr(void)
+{
+    return 0;
+}
+
+
 void CReaderRequestResult::SetRequestedId(const CSeq_id_Handle& requested_id)
 {
     if ( !m_RequestedId ) {
@@ -638,8 +648,14 @@ bool CReaderRequestResult::IsBlobLoaded(const CBlob_id& blob_id)
     TBlobLoadInfo& info = x_GetBlobLoadInfo(blob_id);
     if ( !info.second ) {
         info.second = GetTSE_LoadLockIfLoaded(blob_id);
+        if ( !info.second ) {
+            return false;
+        }
     }
-    return info.second && info.second.IsLoaded();
+    if ( info.second.IsLoaded() ) {
+        return true;
+    }
+    return false;
 }
 
 
@@ -729,21 +745,20 @@ CRef<CTSE_Info> CReaderRequestResult::GetTSE_Info(const CBlob_id& blob_id)
 CRef<CLoadInfoLock>
 CReaderRequestResult::GetLoadLock(const CRef<CLoadInfo>& info)
 {
-    CLoadInfoLock*& lock_ptr = m_LockMap[info];
-    if ( !lock_ptr ) {
-        lock_ptr = new CLoadInfoLock(*this, info);
+    CRef<CLoadInfoLock>& lock = m_LockMap[info];
+    if ( !lock ) {
+        lock = new CLoadInfoLock(*this, info);
     }
     else {
-        _ASSERT(lock_ptr->Referenced());
+        _ASSERT(lock->Referenced());
     }
-    return Ref(lock_ptr);
+    return lock;
 }
 
 
 void CReaderRequestResult::ReleaseLoadLock(const CRef<CLoadInfo>& info)
 {
-    _ASSERT(m_LockMap[info] && !m_LockMap[info]->Referenced());
-    m_LockMap.erase(info);
+    m_LockMap[info] = null;
 }
 
 
@@ -798,11 +813,11 @@ void CReaderRequestResult::SaveLocksTo(TTSE_LockSet& locks)
 
 void CReaderRequestResult::ReleaseLocks(void)
 {
-    NON_CONST_ITERATE( TLockMap, it, m_LockMap ) {
-        it->second->ReleaseLock();
-    }
     m_BlobLoadLocks.clear();
     m_TSE_LockSet.clear();
+    NON_CONST_ITERATE ( TLockMap, it, m_LockMap ) {
+        it->second = null;
+    }
 }
 
 
