@@ -1,4 +1,4 @@
-/*  $Id: gff_formatter.cpp 202496 2010-08-26 13:32:10Z kornbluh $
+/*  $Id: gff_formatter.cpp 346734 2011-12-09 16:04:28Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -479,7 +479,7 @@ string CGFFFormatter::x_GetTranscriptID
 
     string transcript_id = gene_id;
     transcript_id += ":unknown_transcript_";
-    transcript_id += NStr::IntToString(m_Transcripts[gene_id].size());
+    transcript_id += NStr::NumericToString(m_Transcripts[gene_id].size());
     return transcript_id;
 }
 
