@@ -1,4 +1,4 @@
-/*  $Id: item_formatter.cpp 360035 2012-04-19 13:43:48Z kornbluh $
+/*  $Id: item_formatter.cpp 378707 2012-10-23 20:02:48Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -430,8 +430,9 @@ static string s_DoSup(const string& issue, const string& part_sup, const string&
     string str;
 
     if (!NStr::IsBlank(part_sup)) {
-        str += ' ';
-        str +=part_sup;
+        str += " (";
+        str += part_sup;
+        str += ')';
     }
 
     if (NStr::IsBlank(issue)  &&  NStr::IsBlank(part_supi)) {
