@@ -1,4 +1,4 @@
-/*  $Id: ostream_text_ostream.cpp 342280 2011-10-26 19:11:08Z kornbluh $
+/*  $Id: ostream_text_ostream.cpp 380171 2012-11-08 17:40:34Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -77,19 +77,5 @@ void COStreamTextOStream::AddLine(
     }
 }
 
-void COStreamTextOStream::AddCLine(
-    const char *line,
-    const CSerialObject* obj )
-{
-    m_Ostream << line << '\n';
-}
-
-void COStreamTextOStream::AddRawText(
-    const char *line,
-    const CSerialObject* obj )
-{
-    m_Ostream << line;
-}
-
 END_SCOPE(objects)
 END_NCBI_SCOPE
