@@ -1,4 +1,4 @@
-/*  $Id: qualifiers.cpp 361622 2012-05-03 13:26:51Z kornbluh $
+/*  $Id: qualifiers.cpp 388451 2013-02-07 19:49:24Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,7 @@
 #include <objects/seq/MolInfo.hpp>
 #include <objects/seq/seqport_util.hpp>
 #include <objmgr/seq_vector.hpp>
+#include <objmgr/util/seq_loc_util.hpp>
 
 #include <objtools/format/items/qualifiers.hpp>
 #include <objtools/format/context.hpp>
@@ -61,6 +62,7 @@ BEGIN_SCOPE(objects)
 
 const string IFlatQVal::kSpace     = " ";
 const string IFlatQVal::kSemicolon = ";";
+const string IFlatQVal::kSemicolonEOL = ";\n";
 const string IFlatQVal::kComma     = ",";
 const string IFlatQVal::kEOL       = "\n";
 
@@ -264,6 +266,43 @@ static size_t s_ComposeCodonRecognizedStr(const CTrna_ext& trna, string& recogni
     return size;
 }
 
+static bool s_AltitudeIsValid(const string & str )
+{
+    // this is the regex we're validating, but for speed and library dependency
+    // reasons, we're using a more raw approach:
+    //
+    // ^[+-]?[0-9]+(\.[0-9]+)?[ ]m\.$
+
+    // we use c-style strings so we don't have to worry about constantly
+    // checking if we're out of bounds: we get a '\0' which lets us be cleaner
+    const char *pch = str.c_str();
+    
+    // optional sign at start
+    if( *pch == '+' || *pch == '-' ) {
+        ++pch;
+    }
+
+    // then a number, maybe with one decimal point
+    if( ! isdigit(*pch) ) {
+        return false;
+    }
+    while( isdigit(*pch) ) {
+        ++pch;
+    }
+    if( *pch == '.' ) {
+        ++pch;
+        if( ! isdigit(*pch) ) {
+            return false;
+        }
+        while( isdigit(*pch) ) {
+            ++pch;
+        }
+    }
+
+    // should end with " m."
+    return NStr::Equal(pch, " m.");
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // CFormatQual - low-level formatted qualifier
 
@@ -830,7 +869,7 @@ void CFlatOrgModQVal::Format(TFlatQuals& q, const string& name,
                     AddPeriod(subname);
                 }
                 m_Prefix = &kEOL;
-                m_Suffix = &kEOL;
+                m_Suffix = ( add_period ? &kEOL : &kSemicolonEOL );
                 qual = x_AddFQ(q, "note", subname);
             } else {
                 qual = x_AddFQ(q, "note", name + ": " + subname, 
@@ -1189,7 +1228,7 @@ void CFlatSubSourceQVal::Format(TFlatQuals& q, const string& name,
                 if (add_period) {
                     AddPeriod(subname);
                 }
-                m_Suffix = &kEOL;
+                m_Suffix = ( add_period ? &kEOL : &kSemicolonEOL );
                 qual = x_AddFQ(q, "note", subname);
             } else {
                 qual = x_AddFQ(q, "note", name + ": " + subname);        
@@ -1223,6 +1262,15 @@ void CFlatSubSourceQVal::Format(TFlatQuals& q, const string& name,
             x_AddFQ(q, name, subname);
             break;
 
+        case CSubSource::eSubtype_altitude:
+            // skip invalid /altitudes
+            if( s_AltitudeIsValid(subname) || 
+                ( ! ctx.Config().IsModeRelease() && ! ctx.Config().IsModeEntrez() ) ) 
+            {
+                x_AddFQ(q, name, subname);
+            }
+            break;
+
         default:
             if ( ! subname.empty() ) {
                 ExpandTildes(subname, eTilde_space);
@@ -1393,12 +1441,13 @@ void CFlatModelEvQVal::Format
  CBioseqContext& ctx,
  IFlatQVal::TFlags flags) const
 {
-    size_t num_mrna = 0, num_prot = 0, num_est = 0;
+    size_t num_mrna = 0, num_prot = 0, num_est = 0, num_long_sra = 0;
+    size_t rnaseq_base_coverage = 0, rnaseq_biosamples_introns_full = 0;
     const string* method = 0;
 
     ITERATE (CUser_object::TData, it, m_Value->GetData()) {
         const CUser_field& field = **it;
-        if (!field.IsSetLabel()  &&  field.GetLabel().IsStr()) {
+        if (! field.IsSetLabel()  || !  field.GetLabel().IsStr()) {
             continue;
         }
         const string& label = field.GetLabel().GetStr();
@@ -1419,6 +1468,8 @@ void CFlatModelEvQVal::Format
                     num_est  = (*i)->GetData().GetInt();
                 } else if (label == "Protein") {
                     num_prot = (*i)->GetData().GetInt();
+                } else if (label == "long SRA read") {
+                    num_long_sra = (*i)->GetData().GetInt();
                 }
             }
         } else if (label == "mRNA") {
@@ -1427,6 +1478,16 @@ void CFlatModelEvQVal::Format
             num_est  = s_CountAccessions(field);
         } else if (label == "Protein") {
             num_prot = s_CountAccessions(field);
+        } else if (label == "long SRA read") {
+            num_long_sra = s_CountAccessions(field);
+        } else if (label == "rnaseq_base_coverage" ) {
+            if ( field.CanGetData() && field.GetData().IsInt() ) {
+                rnaseq_base_coverage = field.GetData().GetInt();
+            }
+        } else if (label == "rnaseq_biosamples_introns_full" ) {
+            if ( field.CanGetData() && field.GetData().IsInt() ) {
+                rnaseq_biosamples_introns_full = field.GetData().GetInt();
+            }
         }
     }
 
@@ -1437,29 +1498,63 @@ void CFlatModelEvQVal::Format
     }
     text << ".";
 
-    if (num_mrna > 0  ||  num_est > 0  ||  num_prot > 0) {
+    if (num_mrna > 0  ||  num_est > 0  ||  num_prot > 0 || num_long_sra > 0 ||
+        rnaseq_base_coverage > 0 ) 
+    {
         text << " Supporting evidence includes similarity to:";
     }
-    string prefix = " ";
-    if (num_mrna > 0) {
-        text << prefix << num_mrna << " mRNA";
-        if (num_mrna > 1) {
-            text << 's';
+    string section_prefix = " ";
+    // The countable section
+    if( num_mrna > 0  ||  num_est > 0  ||  num_prot > 0 || num_long_sra > 0 )
+    {
+        text << section_prefix;
+        string prefix = "";
+        if (num_mrna > 0) {
+            text << prefix << num_mrna << " mRNA";
+            if (num_mrna > 1) {
+                text << 's';
+            }
+            prefix = ", ";
         }
-        prefix = ", ";
+        if (num_est > 0) {
+            text << prefix << num_est << " EST";
+            if (num_est > 1) {
+                text << 's';
+            }
+            prefix = ", ";
+        }
+        if (num_prot > 0) {
+            text << prefix << num_prot << " Protein";
+            if (num_prot > 1) {
+                text << 's';
+            }
+            prefix = ", ";
+        }
+        if (num_long_sra > 0) {
+            text << prefix << num_long_sra << " long SRA read";
+            if (num_long_sra > 1) {
+                text << 's';
+            }
+            prefix = ", ";
+        }
+        section_prefix = ", and ";
     }
-    if (num_est > 0) {
-        text << prefix << num_est << " EST";
-        if (num_est > 1) {
-            text << 's';
-        }
-        prefix = ", ";
-    }
-    if (num_prot > 0) {
-        text << prefix << num_prot << " Protein";
-        if (num_prot > 1) {
-            text << 's';
+    // The RNASeq section
+    if( rnaseq_base_coverage > 0 )
+    {
+        text << section_prefix;
+
+        text << rnaseq_base_coverage << "% coverage by RNAseq alignments";
+        if( rnaseq_biosamples_introns_full > 0 ) {
+            text << ", including " << rnaseq_biosamples_introns_full;
+            text << " sample";
+            if( rnaseq_biosamples_introns_full > 1 ) {
+                text << 's';
+            }
+            text << " with support for all introns";
         }
+
+        section_prefix = ", and ";
     }
 
     x_AddFQ(q, name, CNcbiOstrstreamToString(text));
@@ -1542,9 +1637,25 @@ void CFlatAnticodonQVal::Format
     string locationString = CFlatSeqLoc(*m_Anticodon, ctx).GetString();
 
     CNcbiOstrstream text;
-    text << "(pos:" ;
-    text << locationString;
-    text << ",aa:" << m_Aa << ')' ;
+    text << "(pos:" << locationString;
+    text << ",aa:" << m_Aa;
+
+    CScope & scope = ctx.GetScope();
+    if( sequence::GetLength(*m_Anticodon, &scope) == 3 ) { // "3" because 3 nucleotides to an amino acid (and thus anticodon sequence)
+        
+        try {
+            CSeqVector seq_vector(*m_Anticodon, scope, CBioseq_Handle::eCoding_Iupac);
+            if( seq_vector.size() == 3 ) {
+                string seq("---");
+                seq_vector.GetSeqData(0, 3, seq);
+                NStr::ToLower(seq);
+                text << ",seq:" << seq;
+            }
+        } catch(...) {
+            // ignore any sort of error that occurs in this process
+        }
+    }
+    text << ')' ;
 
     x_AddFQ(q, name, CNcbiOstrstreamToString(text), CFormatQual::eUnquoted);
 }
