@@ -1,4 +1,4 @@
-/*  $Id: reference_item.cpp 361103 2012-04-30 14:18:40Z vasilche $
+/*  $Id: reference_item.cpp 380336 2012-11-09 20:35:09Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,7 @@
 #include <objects/general/Date.hpp>
 #include <objects/general/Date_std.hpp>
 #include <objects/medline/Medline_entry.hpp>
+#include <objects/mla/mla_client.hpp>
 #include <objects/pub/Pub.hpp>
 #include <objects/pub/Pub_equiv.hpp>
 #include <objects/pub/Pub_set.hpp>
@@ -54,6 +55,7 @@
 #include <objects/biblio/Imprint.hpp>
 #include <objects/submit/Submit_block.hpp>
 #include <objmgr/bioseq_ci.hpp>
+#include <objmgr/object_manager.hpp>
 #include <objmgr/util/sequence.hpp>
 #include <objmgr/util/seq_loc_util.hpp>
 
@@ -79,11 +81,15 @@ USING_SCOPE(sequence);
 class LessThan
 {
 public:
-    LessThan(bool serial_first, bool is_refseq);
+    enum ESerialFirst {
+        eSerialFirst_No = 0,
+        eSerialFirst_Yes
+    };
+    LessThan(ESerialFirst serial_first, bool is_refseq);
     bool operator()(const CRef<CReferenceItem>& ref1, const CRef<CReferenceItem>& ref2);
 private:
-    bool m_SerialFirst;
-    bool m_IsRefSeq;
+    ESerialFirst m_SerialFirst;
+    bool         m_IsRefSeq;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -464,7 +470,8 @@ static void s_MergeDuplicates
 void CReferenceItem::Rearrange(TReferences& refs, CBioseqContext& ctx)
 {
     {{
-        stable_sort(refs.begin(), refs.end(), LessThan(false, ctx.IsRefSeq()));
+        stable_sort(refs.begin(), refs.end(), 
+            LessThan(LessThan::eSerialFirst_No, ctx.IsRefSeq()));
     }}
 
     {{
@@ -480,7 +487,8 @@ void CReferenceItem::Rearrange(TReferences& refs, CBioseqContext& ctx)
 
     {{
         // re-sort, take serial number into consideration.
-        stable_sort(refs.begin(), refs.end(), LessThan(true, ctx.IsRefSeq()));
+        stable_sort(refs.begin(), refs.end(), 
+            LessThan(LessThan::eSerialFirst_Yes, ctx.IsRefSeq()));
     }}
     
     // assign final serial numbers
@@ -629,6 +637,79 @@ void CReferenceItem::x_GatherInfo(CBioseqContext& ctx)
         x_Init(**it, ctx);
     }
 
+    // if just pmid or just muid, we look it up, assuming the user has
+    // somehow given permission for remote lookups 
+    const static string kGbLoader = "GBLOADER";
+    if( IsJustUids() && 
+        ctx.GetScope().GetObjectManager().FindDataLoader(kGbLoader) ) 
+    {
+        // TODO: To avoid repeated connections, in the future we should have 
+        // one CMLAClient shared by the whole program
+        CMLAClient mlaClient;
+
+        vector< CRef<CPub> > new_pubs;
+
+        ITERATE( CPub_equiv::Tdata, it, pub.Get() ) {
+            const CPub & pub = **it;
+            CRef<CPub> new_pub;
+
+            try {
+                switch(pub.Which()) {
+                case CPub::e_Pmid:
+                    {
+                        const int pmid = pub.GetPmid().Get();
+
+                        CPubMedId req(pmid);
+                        CMLAClient::TReply reply;
+                        new_pub = mlaClient.AskGetpubpmid(req, &reply);
+                    }
+                    break;
+                case CPub::e_Muid:
+                    {
+                        const int muid = pub.GetMuid();
+
+                        const int pmid = mlaClient.AskUidtopmid(muid);
+                        if( pmid > 0 ) {
+                            CPubMedId req(pmid);
+                            CMLAClient::TReply reply;
+                            new_pub = mlaClient.AskGetpubpmid(req, &reply);
+                        }
+                    }
+                    break;
+                default:
+                    // ignore if type unknown
+                    break;
+                }
+            } catch(...) {
+                // don't worry if we can't look it up
+            }
+
+            if( new_pub ) {
+                // authors come back in a weird format that we need
+                // to convert to ISO
+                x_ChangeMedlineAuthorsToISO(new_pub);
+
+                new_pubs.push_back(new_pub);
+            }
+        }
+
+        if( ! new_pubs.empty() ) {
+            ITERATE( vector< CRef<CPub> >, new_pub_iter, new_pubs ) {
+                x_Init( **new_pub_iter, ctx );
+            }
+
+            // we have to add the new_pubs to m_Pubdesc->GetPub() but m_Pubdesc
+            // is const.  The solution is to copy it, modify the copy, and 
+            // set the copy to have CConstRef
+            CRef<CPubdesc> new_pubdesc( new CPubdesc );
+            new_pubdesc->Assign(*m_Pubdesc);
+            CPub_equiv::Tdata & new_pub_list = new_pubdesc->SetPub().Set();
+            copy( new_pubs.begin(), new_pubs.end(),
+                back_inserter(new_pub_list) );
+            m_Pubdesc = new_pubdesc;
+        }
+    }
+
     // gather Genbank specific fields (formats: Genbank, GBSeq, DDBJ)
     if ( ctx.IsGenbankFormat() ) {
         x_GatherRemark(ctx);
@@ -1198,6 +1279,121 @@ void CReferenceItem::x_CapitalizeTitleIfNecessary()
     }
 }
 
+void CReferenceItem::x_ChangeMedlineAuthorsToISO( CRef<CPub> pub )
+{
+    // leave early if it doesn't need to be changed
+    if( ! pub || ! pub->IsArticle() || ! pub->IsSetAuthors() || ! pub->GetAuthors().IsSetNames() ||
+        ! pub->GetAuthors().GetNames().IsMl() )
+    {
+        return;
+    }
+
+    // build our new authors list in here
+    CAuth_list::C_Names::TStd new_authors;
+
+    const CAuth_list::C_Names::TMl & ml_names = pub->GetAuthors().GetNames().GetMl();
+    ITERATE( CAuth_list::C_Names::TMl, ml_name_iter, ml_names ) {
+        string author_name = *ml_name_iter;
+
+        // we will fill in these 3 as we go along
+        string lastname;
+        string initials;
+        string suffix;
+
+        // this scope fills in lastname, initials, and suffix
+        {
+            NStr::TruncateSpacesInPlace(author_name);
+            vector<string> tokens;
+            NStr::Tokenize(author_name, " ", tokens, NStr::eMergeDelims);
+
+            // get suffix if it exists, and remove it from the list
+            if( tokens.size() >= 3 && 
+                ! x_StringIsJustCapitalLetters(tokens.back()) && 
+                x_StringIsJustCapitalLetters(tokens[tokens.size() - 2]) ) 
+            {
+                suffix = tokens.back();
+                tokens.pop_back();
+            }
+
+            // get initials if they exist, and remove them from the list
+            if( tokens.size() >= 2 &&
+                x_StringIsJustCapitalLetters(tokens.back()) )
+            {
+                initials = tokens.back();
+                tokens.pop_back();
+            }
+
+            // remaining pieces belong to the last name
+            lastname = NStr::Join(tokens, " ");
+        }
+
+        // put period in initials. e.g. "MJ" -> "M.J."
+        {
+            string new_initials;
+            ITERATE( string, ch_iter, initials ) {
+                new_initials += *ch_iter;
+                new_initials += '.';
+            }
+            // swap is faster than assignment
+            initials.swap(new_initials);
+        }
+
+        // a couple of static transformations for the suffix
+        typedef SStaticPair<const char*, const char*> TSufElem;
+        static const TSufElem sc_suf_map[] = {
+            { "1d",  "I" },
+            { "1st", "I" },
+            { "2d",  "II" },
+            { "2nd", "II" },
+            { "3d",  "III" },
+            { "3rd", "III" },
+            { "4th", "IV" },
+            { "5th", "V" },
+            { "6th", "VI" },
+            { "Jr",  "Jr." },
+            { "Sr",  "Sr."}
+        };
+        typedef CStaticArrayMap<const char *, const char *, PCase_CStr> TSufMap;
+        DEFINE_STATIC_ARRAY_MAP(TSufMap, sc_SufMap, sc_suf_map);
+
+        TSufMap::const_iterator suf_find_iter = sc_SufMap.find(suffix.c_str());
+        if( suf_find_iter != sc_SufMap.end() ) {
+            suffix = suf_find_iter->second;
+        }
+
+        CRef<CAuthor> new_author( new CAuthor );
+        CPerson_id_Base::TName & name = new_author->SetName().SetName();
+        name.SetLast( lastname );
+        if( ! initials.empty() ) {
+            name.SetInitials( initials );
+        }
+        if( ! suffix.empty() ) {
+            name.SetSuffix( suffix );
+        }
+
+        new_authors.push_back( new_author );
+    }
+    
+    copy( new_authors.begin(), 
+        new_authors.end(),
+        back_inserter( pub->SetArticle().SetAuthors().SetNames().SetStd() ) );
+}
+
+bool CReferenceItem::x_StringIsJustCapitalLetters( const string & str )
+{
+    if( str.empty() ) {
+        return false;
+    }
+
+    ITERATE(string, ch_iter, str ) {
+        if( ! isupper(*ch_iter) ) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 //
@@ -1460,7 +1656,7 @@ static CDate::ECompare s_CompareDates(const CDate& d1, const CDate& d2)
     return CDate::eCompare_same;
 }
 
-LessThan::LessThan(bool serial_first, bool is_refseq) :
+LessThan::LessThan(ESerialFirst serial_first, bool is_refseq) :
     m_SerialFirst(serial_first), m_IsRefSeq(is_refseq)
 {}
 
@@ -1469,7 +1665,7 @@ bool LessThan::operator()
 (const CRef<CReferenceItem>& ref1,
  const CRef<CReferenceItem>& ref2)
 {
-    if ( m_SerialFirst  &&  ref1->GetSerial() != ref2->GetSerial() ) {
+    if ( m_SerialFirst == eSerialFirst_Yes &&  ref1->GetSerial() != ref2->GetSerial() ) {
         return ref1->GetSerial() < ref2->GetSerial();
     }
 
