@@ -1,4 +1,4 @@
-/*  $Id: utils.cpp 366477 2012-06-14 16:49:45Z rafanovi $
+/*  $Id: utils.cpp 388127 2013-02-05 19:16:55Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1202,7 +1202,7 @@ const string strLinkBaseUSPTO(
     "http://patft.uspto.gov/netacgi/nph-Parser?patentnumber=" );
 
 const string strDocLink(
-    "http://www.ncbi.nlm.nih.gov/genome/guide/build.shtml" );
+    "http://www.ncbi.nlm.nih.gov/genome/annotation_euk/process/" );
 
 bool ConvertQuotesNotInHTMLTags( string &str )
 {   
