@@ -1,4 +1,4 @@
-/*  $Id: agp_util.cpp 356094 2012-03-12 14:14:32Z sapojnik $
+/*  $Id: agp_util.cpp 372558 2012-08-20 15:51:24Z sapojnik $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,7 +35,7 @@
 
 #ifndef i2s
 #define i2s(x) NStr::NumericToString(x)
-#define s2i(x) NStr::StringToNumeric(x)
+#define s2i(x) NStr::StringToNonNegativeInt(x)
 #endif
 
 BEGIN_NCBI_SCOPE
@@ -51,7 +51,7 @@ const CAgpErr::TStr CAgpErr::s_msg[]= {
     "column X is empty",
     "empty line",
     "invalid value for X",
-    "invalid linkage \"yes\" for gap_type ",
+    "invalid linkage",
 
     "X must be a positive integer not exceeding 2e+9",
     "object_end is less than object_beg",
@@ -571,11 +571,10 @@ int CAgpRow::ParseGapCols(bool log_errors)
             gap_type != eGapFragment &&
             gap_type != eGapScaffold )
         {
-            if(log_errors) m_AgpErr->Msg(CAgpErr::E_InvalidYes, GetGapType() );
-            return CAgpErr::E_InvalidYes;
+            if(log_errors) m_AgpErr->Msg(CAgpErr::E_InvalidLinkage, " \"yes\" for gap_type "+GetGapType() );
+            return CAgpErr::E_InvalidLinkage;
         }
     }
-
     if( log_errors && m_agp_version==eAgpVersion_auto ) {
         string msg;
         if( GetLinkageEvidence().size()==0 ) {
@@ -597,6 +596,12 @@ int CAgpRow::ParseGapCols(bool log_errors)
             // if(log_errors)
             m_AgpErr->Msg(CAgpErr::W_OldGapType, ". Recommended replacement: " + SubstOldGap(false) );
         }
+        if(!linkage && gap_type==eGapScaffold)
+        {
+            if(log_errors) m_AgpErr->Msg(CAgpErr::E_InvalidLinkage, " \"no\" for gap_type "+GetGapType() );
+            return CAgpErr::E_InvalidLinkage;
+        }
+
     }
     if(m_agp_version == eAgpVersion_1_1){
         // gap-type not in AGP 1.1
@@ -1074,12 +1079,12 @@ string CAgpReader::GetErrorMessage(const string& filename)
 
 void CAgpReader::x_CheckPragmaComment(void)
 {
-    const static string kAgpVersionCommentStart("##agp-version");
+    static const char* kAgpVersionCommentStart = "##agp-version";
     if( NStr::StartsWith(m_line, kAgpVersionCommentStart) ) {
         // skip whitespace before and after version number
         const SIZE_TYPE versionStartPos = m_line.find_first_not_of(
             " \t\v\f",
-            kAgpVersionCommentStart.length() );
+            strlen(kAgpVersionCommentStart) );
         const SIZE_TYPE versionEndPos = m_line.find_last_not_of(
             " \t\v\f" );
         string version;
