@@ -1,4 +1,4 @@
-/*  $Id: agp_validate_reader.cpp 356544 2012-03-14 20:02:47Z sapojnik $
+/*  $Id: agp_validate_reader.cpp 369200 2012-07-17 15:33:42Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1111,7 +1111,7 @@ bool CAgpValidateReader::x_PrintPatterns(
 {
   const int MaxPatterns=10;
   //const int MaxPatterns=2;
-  static const string SPACES="                                        ";
+  const string SPACES="                                        ";
 
   // Sorted by count to print most frequent first
   CAccPatternCounter::TMapCountToString cnt_pat; // multimap<int,string>
