@@ -1,4 +1,4 @@
-/*  $Id: fasta.cpp 372587 2012-08-20 18:38:55Z ucko $
+/*  $Id: fasta.cpp 381563 2012-11-26 18:14:23Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -127,6 +127,27 @@ inline unsigned char s_ASCII_ToUpper(unsigned char c)
     return s_ASCII_IsLower(c) ? c + 'A' - 'a' : c;
 }
 
+inline bool s_ASCII_IsAmbigNuc(unsigned char c)
+{
+    switch( s_ASCII_ToUpper(c) ) {
+    case 'U':
+    case 'R':
+    case 'Y':
+    case 'S':
+    case 'W':
+    case 'K':
+    case 'M':
+    case 'B':
+    case 'D':
+    case 'H':
+    case 'V':
+    case 'N':
+        return true;
+    default:
+        return false;
+    }
+}
+
 CFastaReader::CFastaReader(ILineReader& reader, TFlags flags)
     : m_LineReader(&reader), m_MaskVec(0), m_IDGenerator(new CSeqIdGenerator)
 {
@@ -488,7 +509,10 @@ bool CFastaReader::ParseIDs(const TStr& s)
     CBioseq::TId& ids = SetIDs();
     // CBioseq::TId  old_ids = ids;
     size_t count = 0;
-    CSeq_id::TParseFlags flags = CSeq_id::fParse_PartialOK; // be generous
+    // be generous overall, and give raw local IDs the benefit of the
+    // doubt for now
+    CSeq_id::TParseFlags flags
+        = CSeq_id::fParse_PartialOK | CSeq_id::fParse_AnyLocal;
     if (TestFlag(fParseRawID)) {
         flags |= CSeq_id::fParse_RawText;
     }
@@ -497,17 +521,15 @@ bool CFastaReader::ParseIDs(const TStr& s)
     } catch (CSeqIdException&) {
         // swap(ids, old_ids);
     }
-    if ( !count ) {
-        string str(s);
-        // Handled this way rather than by passing CSeq_id::fParse_ValidLocal
-        // so that subclasses can meaningfully overload IsValidLocalID.
-        if (IsValidLocalID(str)) {
-            ids.push_back(CRef<CSeq_id>(new CSeq_id(CSeq_id::e_Local, str)));
-        } else {
-            return false;
-        }
+    // recheck raw local IDs
+    if (count == 1  &&  ids.back()->IsLocal()
+        &&  !NStr::StartsWith(s, "lcl|", NStr::eNocase)
+        &&  !IsValidLocalID(s)) {
+        // swap(ids, old_ids);
+        ids.clear();
+        return false;
     }
-    return true;
+    return count > 0;
 }
 
 size_t CFastaReader::ParseRange(const TStr& s, TSeqPos& start, TSeqPos& end)
@@ -549,14 +571,38 @@ size_t CFastaReader::ParseRange(const TStr& s, TSeqPos& start, TSeqPos& end)
 
 void CFastaReader::ParseTitle(const TStr& s)
 {
+    const static size_t kWarnTitleLength = 1000;
+    if( s.length() > kWarnTitleLength ) {
+        ERR_POST_X(1, Warning
+            << "CFastaReader: Title is very long: " << s.length() 
+            << " characters (max is " << kWarnTitleLength << "),"
+            << " at line " << LineNumber());
+    }
+
+    const static size_t kWarnNumSeqCharsAtEnd = 20;
+    if( s.length() > kWarnNumSeqCharsAtEnd ) {
+        const string sEndOfTitle = 
+            s.substr(s.length() - kWarnNumSeqCharsAtEnd, kWarnNumSeqCharsAtEnd);
+        if( sEndOfTitle.find_first_not_of("ACGTacgt") == string::npos ) {
+            ERR_POST_X(1, Warning
+                << "CFastaReader: Title ends with at least " << kWarnNumSeqCharsAtEnd 
+                << " valid nucleotide characters.  Was the sequence accidentally put in the title line?"
+                << " at line " << LineNumber());
+        }
+    }
+
     CRef<CSeqdesc> desc(new CSeqdesc);
     desc->SetTitle().assign(s.data(), s.length());
     m_CurrentSeq->SetDescr().Set().push_back(desc);
 }
 
-bool CFastaReader::IsValidLocalID(const string& s)
+bool CFastaReader::IsValidLocalID(const TStr& s)
 {
-    return CSeq_id::IsValidLocalID(s);
+    if (TestFlag(fQuickIDCheck)) { // just check first character
+        return CSeq_id::IsValidLocalID(s.substr(0, 1));
+    } else {
+        return CSeq_id::IsValidLocalID(s);
+    }
 }
 
 void CFastaReader::GenerateID(void)
@@ -581,10 +627,16 @@ void CFastaReader::CheckDataLine(const TStr& s)
         return;
     }
     size_t good = 0, bad = 0, len = s.length();
+    const bool bIsNuc = ( m_CurrentSeq && m_CurrentSeq->IsSetInst() &&
+        m_CurrentSeq->GetInst().IsSetMol() &&  m_CurrentSeq->IsNa() );
+    size_t ambig_nuc = 0;
     for (size_t pos = 0;  pos < len;  ++pos) {
         unsigned char c = s[pos];
         if (s_ASCII_IsAlpha(c)  ||  c == '-'  ||  c == '*') {
             ++good;
+            if( bIsNuc && s_ASCII_IsAmbigNuc(c) ) {
+                ++ambig_nuc;
+            }
         } else if (isspace(c)  ||  (c >= '0' && c <= '9')) {
             // treat whitespace and digits as neutral
         } else if (c == ';') {
@@ -599,17 +651,34 @@ void CFastaReader::CheckDataLine(const TStr& s)
             ", there's a line that doesn't look like plausible data, but it's not marked as defline or comment.",
             LineNumber());
     }
+    // warn if more than a certain percentage is ambiguous nucleotides
+    const static size_t kWarnPercentAmbiguous = 40; // e.g. "40" means "40%"
+    const size_t percent_ambig = (ambig_nuc * 100) / good;
+    if( len > 3 && percent_ambig > kWarnPercentAmbiguous ) {
+        ERR_POST_X(1, Warning
+            << "CFastaReader: First data line in seq is about "
+            << percent_ambig << "% ambiguous nucleotides (shouldn't be over "
+            << kWarnPercentAmbiguous << "%)"
+            << " at line " << LineNumber() );
+    }
 }
 
 void CFastaReader::ParseDataLine(const TStr& s)
 {
     CheckDataLine(s);
 
-    size_t len = s.length();
+    size_t len = min(s.length(), s.find(';')); // ignore ;-delimited comments
     if (m_SeqData.capacity() < m_SeqData.size() + len) {
         // ensure exponential capacity growth to avoid quadratic runtime
         m_SeqData.reserve(2 * max(m_SeqData.capacity(), len));
     }
+    if ((GetFlags() & (fSkipCheck | fParseGaps | fValidate)) == fSkipCheck
+        &&  m_CurrentMask.Empty()) {
+        m_SeqData.append(s.data(), len);
+        m_CurrentPos += len;
+        return;
+    }
+        
     m_SeqData.resize(m_CurrentPos + len);
     for (size_t pos = 0;  pos < len;  ++pos) {
         unsigned char c = s[pos];
@@ -633,9 +702,6 @@ void CFastaReader::ParseDataLine(const TStr& s)
                 CloseMask();
             }
             ++m_CurrentPos;
-        } else if (c == ';') {
-            // comment -- ignore rest of line
-            break;
         } else if ( !isspace(c) ) {
             if (TestFlag(fValidate)) {
                 NCBI_THROW2(CBadResiduesException, eBadResidues,
@@ -765,7 +831,9 @@ void CFastaReader::AssembleSeq(void)
             inst.SetLength(GetCurrentPos(eRawPos));
             inst.SetRepr(CSeq_inst::eRepr_raw);
             CRef<CSeq_data> data(new CSeq_data(m_SeqData, format));
-            CSeqportUtil::Pack(data, inst.GetLength());
+            if ( !TestFlag(fLeaveAsText) ) {
+                CSeqportUtil::Pack(data, inst.GetLength());
+            }
             inst.SetSeq_data(*data);
         } else {
             inst.SetLength(GetCurrentPos(eRawPos));
