@@ -1,4 +1,4 @@
-/*  $Id: gff2_reader.cpp 346995 2011-12-13 14:34:50Z ludwigf $
+/*  $Id: gff2_reader.cpp 359378 2012-04-12 17:45:08Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -82,6 +82,7 @@
 #include <objects/seqfeat/Feat_id.hpp>
 #include <objects/seqset/Bioseq_set.hpp>
 
+#include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
 #include <objtools/readers/error_container.hpp>
@@ -607,30 +608,7 @@ bool CGff2Reader::x_FeatureSetLocation(
     CRef< CSeq_feat > pFeature )
 //  ----------------------------------------------------------------------------
 {
-    CRef< CSeq_id > pId;
-
-    const string& id_str = record.Id();
-    if (m_iFlags & fAllIdsAsLocal) {
-        pId.Reset(new CSeq_id(CSeq_id::e_Local, id_str));
-    } else {
-        bool is_numeric =
-            id_str.find_first_not_of("0123456789") == string::npos;
-
-        if (is_numeric && (m_iFlags & fNumericIdsAsLocal)) {
-            pId.Reset(new CSeq_id(CSeq_id::e_Local, id_str));
-        }
-        else {
-            try {
-                pId.Reset( new CSeq_id(id_str));
-                if (!pId || (pId->IsGi() && pId->GetGi() < 500) ) {
-                    pId = new CSeq_id(CSeq_id::e_Local, id_str);
-                }
-            }
-            catch (CException&) {
-                pId.Reset(new CSeq_id(CSeq_id::e_Local, id_str));
-            }
-        }
-    }
+    CRef< CSeq_id > pId = CReadUtil::AsSeqId(record.Id(), m_iFlags);
     CRef< CSeq_loc > pLocation( new CSeq_loc );
     pLocation->SetInt().SetId( *pId );
     pLocation->SetInt().SetFrom( record.SeqStart() );
@@ -748,6 +726,7 @@ bool CGff2Reader::x_FeatureSetDataGene(
     CRef< CSeq_feat > pFeature )
 //  ----------------------------------------------------------------------------
 {
+    //CGene_ref& gene = pFeature->SetData().SetGene();
     return true;
 }
 
@@ -769,6 +748,7 @@ bool CGff2Reader::x_FeatureSetDataCDS(
     CRef< CSeq_feat > pFeature )
 //  ----------------------------------------------------------------------------
 {
+    //CCdregion& cdr = pFeature->SetData().SetCdregion();
     return true;
 }
 
