@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 340591 2011-10-11 16:04:11Z ludwigf $
+/*  $Id: gff3_reader.cpp 360585 2012-04-24 17:36:54Z boukn $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -180,6 +180,24 @@ bool CGff3Reader::x_UpdateAnnot(
     record.GetAttribute("gbkey", gbkey);
     CRef< CSeq_feat > pFeature(new CSeq_feat);
 
+    //  Round trip info:
+    CRef< CGb_qual > pQual( new CGb_qual );
+    pQual->SetQual( "gff_source" );
+    pQual->SetVal( record.Source() );
+    pFeature->SetQual().push_back( pQual );
+
+    pQual.Reset( new CGb_qual );
+    pQual->SetQual( "gff_type" );
+    pQual->SetVal( record.Type() );
+    pFeature->SetQual().push_back( pQual );
+
+    if ( record.IsSetScore() ) {
+        pQual.Reset( new CGb_qual );
+        pQual->SetQual( "gff_score" );
+        pQual->SetVal( NStr::DoubleToString( record.Score() ) );
+        pFeature->SetQual().push_back( pQual );
+    }
+
     //  Special case: exon feature belonging to an RNA we have already seen
     if (record.Type() == "exon") {
         string parent;
