@@ -1,4 +1,4 @@
-/*  $Id: wiggle_reader.cpp 372344 2012-08-16 17:39:17Z rafanovi $
+/*  $Id: wiggle_reader.cpp 380852 2012-11-15 20:30:17Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -68,7 +68,9 @@ BEGIN_objects_SCOPE // namespace ncbi::objects::
 CWiggleReader::CWiggleReader(
     TFlags flags ) :
 //  ----------------------------------------------------------------------------
-    CReaderBase(flags){
+    CReaderBase(flags),
+    m_TrackType(eTrackType_invalid)
+{
     m_uLineNumber = 0;
     m_GapValue = 0.0; 
 }
@@ -98,13 +100,12 @@ CWiggleReader::ReadSeqAnnot(
     IErrorContainer* pErrorContainer ) 
 //  ----------------------------------------------------------------------------                
 {
+    m_ChromId.clear();
+    m_Values.clear();
     if (lr.AtEOF()) {
         return CRef<CSeq_annot>();
     }
     while ( xGetLine(lr) ) {
-        if ( xCommentLine() ) {
-            continue;
-        }
         CTempString s = xGetWord(pErrorContainer);
         if ( s == "browser" ) {
             xReadBrowser();
@@ -113,10 +114,23 @@ CWiggleReader::ReadSeqAnnot(
             xReadTrack(pErrorContainer);
         }
         else if ( s == "fixedStep" ) {
-            xReadFixedStep(lr, pErrorContainer);
+            SFixedStepInfo fixedStepInfo;
+            xGetFixedStepInfo(fixedStepInfo, pErrorContainer);
+            if (!m_ChromId.empty() && fixedStepInfo.mChrom != m_ChromId) {
+                cerr << fixedStepInfo.mChrom << endl;
+                lr.UngetLine();
+                return xGetAnnot();
+            }
+            xReadFixedStepData(fixedStepInfo, lr, pErrorContainer);
         }
         else if ( s == "variableStep" ) {
-            xReadVariableStep(lr, pErrorContainer);
+            SVarStepInfo varStepInfo;
+            xGetVarStepInfo(varStepInfo, pErrorContainer);
+            if (!m_ChromId.empty() && varStepInfo.mChrom != m_ChromId) {
+                lr.UngetLine();
+                return xGetAnnot();
+            }
+            xReadVariableStepData(varStepInfo, lr, pErrorContainer);
         }
         else {
             xReadBedLine(s, pErrorContainer);
@@ -260,8 +274,8 @@ void CWiggleReader::xPreprocessValues(SWiggleStat& stat)
 CRef<CSeq_id> CWiggleReader::xMakeChromId()
 //  =========================================================================
 {
-    CRef<CSeq_id> chrom_id(new CSeq_id(CSeq_id::e_Local, m_ChromId));
-    return chrom_id;
+    CRef<CSeq_id> id = CReadUtil::AsSeqId(m_ChromId);
+    return id;
 }
 
 //  =========================================================================
@@ -758,11 +772,13 @@ bool CWiggleReader::xGetLine(
     ILineReader& lr)
 //  =========================================================================
 {
-    if ( lr.AtEOF() ) {
-        return false;
+    while (!lr.AtEOF()) {
+        m_CurLine = *++lr;
+        if (!xCommentLine()) {
+            return true;
+        }
     }
-    m_CurLine = *++lr;
-    return true;
+	return false;
 }
 
 //  =========================================================================
@@ -779,6 +795,7 @@ CRef<CSeq_annot> CWiggleReader::xGetAnnot()
     else {
         pAnnot->SetData().SetSeq_table(*xMakeTable());
     }
+    m_ChromId.clear();
     return pAnnot;
 }
 
@@ -800,7 +817,7 @@ void CWiggleReader::xDumpChromValues(void)
         m_Annot->SetData().SetSeq_table(*xMakeTable());
     }
     if ( !m_SingleAnnot ) {
-//!        DumpAnnot();
+//        xDumpAnnot();
     }
     xResetChromValues();
 }
@@ -869,38 +886,40 @@ void CWiggleReader::xReadTrack(
     }
 }
 
-//  =========================================================================
-void CWiggleReader::xReadFixedStep(
-    ILineReader& lr,
+//  ----------------------------------------------------------------------------
+void CWiggleReader::xGetFixedStepInfo(
+    SFixedStepInfo& fixedStepInfo,
     IErrorContainer* pErrorContainer)
-//  =========================================================================
+//  ----------------------------------------------------------------------------
 {
-    if ( m_TrackType != eTrackType_wiggle_0 &&
-        m_TrackType != eTrackType_invalid ) {
-        CObjReaderLineException err(
-            eDiag_Warning,
-            0,
-            "Track \"type=wiggle_0\" is required");
-        xProcessError(err, pErrorContainer);
+    if ( m_TrackType != eTrackType_wiggle_0 ) {
+        if ( m_TrackType != eTrackType_invalid ) {
+            CObjReaderLineException err(
+                eDiag_Warning,
+                0,
+                "Track \"type=wiggle_0\" is required");
+            xProcessError(err, pErrorContainer);
+        }
+        else {
+            m_TrackType = eTrackType_wiggle_0;
+        }
     }
 
-    size_t start = 0;
-    size_t step = 0;
-    size_t span = 1;
+    fixedStepInfo.Reset();
     while ( xSkipWS() ) {
         CTempString name = xGetParamName(pErrorContainer);
         CTempString value = xGetParamValue(pErrorContainer);
         if ( name == "chrom" ) {
-            xSetChrom(value);
+            fixedStepInfo.mChrom = value;
         }
         else if ( name == "start" ) {
-            start = NStr::StringToUInt(value);
+            fixedStepInfo.mStart = NStr::StringToUInt(value);
         }
         else if ( name == "step" ) {
-            step = NStr::StringToUInt(value);
+            fixedStepInfo.mStep = NStr::StringToUInt(value);
         }
         else if ( name == "span" ) {
-            span = NStr::StringToUInt(value);
+            fixedStepInfo.mSpan = NStr::StringToUInt(value);
         }
         else {
             CObjReaderLineException err(
@@ -910,65 +929,78 @@ void CWiggleReader::xReadFixedStep(
             xProcessError(err, pErrorContainer);
         }
     }
-    if ( m_ChromId.empty() ) {
+    if ( fixedStepInfo.mChrom.empty() ) {
         CObjReaderLineException err(
             eDiag_Error,
             0,
             "Missing chrom parameter");
         xProcessError(err, pErrorContainer);
     }
-    if ( start == 0 ) {
+    if ( fixedStepInfo.mStart == 0 ) {
         CObjReaderLineException err(
             eDiag_Error,
             0,
             "Missing start value");
         xProcessError(err, pErrorContainer);
     }
-    if ( step == 0 ) {
+    if ( fixedStepInfo.mStep == 0 ) {
         CObjReaderLineException err(
             eDiag_Error,
             0,
             "Missing step value");
         xProcessError(err, pErrorContainer);
     }
+}
 
+//  =========================================================================
+void CWiggleReader::xReadFixedStepData(
+    const SFixedStepInfo& fixedStepInfo,
+    ILineReader& lr,
+    IErrorContainer* pErrorContainer)
+//  =========================================================================
+{
+    xSetChrom(fixedStepInfo.mChrom);
     SValueInfo value;
-    value.m_Pos = start-1;
-    value.m_Span = span;
+    value.m_Pos = fixedStepInfo.mStart-1;
+    value.m_Span = fixedStepInfo.mSpan;
     while ( xGetLine(lr) ) {
         if ( !xTryGetDouble(value.m_Value, pErrorContainer) ) {
             lr.UngetLine();
             break;
         }
         xAddValue(value);
-        value.m_Pos += step;
+        value.m_Pos += fixedStepInfo.mStep;
     }
 }
 
-//  =========================================================================
-void CWiggleReader::xReadVariableStep(
-    ILineReader& lr,
+//  ----------------------------------------------------------------------------
+void CWiggleReader::xGetVarStepInfo(
+    SVarStepInfo& varStepInfo,
     IErrorContainer* pErrorContainer)
-//  =========================================================================
+//  ----------------------------------------------------------------------------
 {
-    if ( m_TrackType != eTrackType_wiggle_0 &&
-        m_TrackType != eTrackType_invalid ) {
-        CObjReaderLineException err(
-            eDiag_Warning,
-            0,
-            "Track \"type=wiggle_0\" is required");
-        xProcessError(err, pErrorContainer);
+    if ( m_TrackType != eTrackType_wiggle_0 ) {
+        if ( m_TrackType != eTrackType_invalid ) {
+            CObjReaderLineException err(
+                eDiag_Warning,
+                0,
+                "Track \"type=wiggle_0\" is required");
+            xProcessError(err, pErrorContainer);
+        }
+        else {
+            m_TrackType = eTrackType_wiggle_0;
+        }
     }
 
-    size_t span = 1;
+    varStepInfo.Reset();
     while ( xSkipWS() ) {
         CTempString name = xGetParamName(pErrorContainer);
         CTempString value = xGetParamValue(pErrorContainer);
         if ( name == "chrom" ) {
-            xSetChrom(value);
+            varStepInfo.mChrom = value;
         }
         else if ( name == "span" ) {
-            span = NStr::StringToUInt(value);
+            varStepInfo.mSpan = NStr::StringToUInt(value);
         }
         else {
             CObjReaderLineException err(
@@ -978,15 +1010,25 @@ void CWiggleReader::xReadVariableStep(
             xProcessError(err, pErrorContainer);
         }
     }
-    if ( m_ChromId.empty() ) {
+    if ( varStepInfo.mChrom.empty() ) {
         CObjReaderLineException err(
             eDiag_Error,
             0,
             "Missing chrom parameter");
         xProcessError(err, pErrorContainer);
     }
+}
+
+//  =========================================================================
+void CWiggleReader::xReadVariableStepData(
+    const SVarStepInfo& varStepInfo,
+    ILineReader& lr,
+    IErrorContainer* pErrorContainer)
+//  =========================================================================
+{
+    xSetChrom(varStepInfo.mChrom);
     SValueInfo value;
-    value.m_Span = span;
+    value.m_Span = varStepInfo.mSpan;
     while ( xGetLine(lr) ) {
         if ( !xTryGetPos(value.m_Pos, pErrorContainer) ) {
             lr.UngetLine();
