@@ -1,4 +1,4 @@
-/*  $Id: mask_writer_int.cpp 255510 2011-02-24 17:19:39Z camacho $
+/*  $Id: mask_writer_int.cpp 390282 2013-02-26 19:09:04Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: mask_writer_int.cpp 255510 2011-02-24 17:19:39Z camacho $";
+static char const rcsid[] = "$Id: mask_writer_int.cpp 390282 2013-02-26 19:09:04Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -47,14 +47,15 @@ void CMaskWriterInt::Print( objects::CBioseq_Handle& bsh,
                                bool parsed_id )
 {
     PrintId( bsh, parsed_id );
+    os << endl;
     PrintMasks(os, mask);
 }
 
 void CMaskWriterInt::PrintMasks(CNcbiOstream& os, const TMaskList& mask)
 {
-    for( TMaskList::const_iterator i = mask.begin();
-         i != mask.end(); ++i )
+    ITERATE(TMaskList, i, mask) {
         os << i->first << " - " << i->second << "\n";
+    }
 }
 
 
