@@ -1,4 +1,4 @@
-/*  $Id: aliasinfo.cpp 329324 2011-08-09 16:35:21Z gouriano $
+/*  $Id: aliasinfo.cpp 376885 2012-10-04 18:09:44Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -61,7 +61,7 @@ typedef CAliasTypeInfoFunctions TFunc;
 
 CAliasTypeInfo::CAliasTypeInfo(const string& name, TTypeInfo type)
     : CParent(name, type->GetSize(), type),
-      m_DataOffset(0)
+      m_DataOffset(0), m_FullAlias(false)
 {
     InitAliasTypeInfoFunctions();
 }
