@@ -1,4 +1,4 @@
-/*  $Id: choicestr.cpp 203019 2010-08-31 12:57:32Z gouriano $
+/*  $Id: choicestr.cpp 338794 2011-09-22 15:43:54Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
 #include <corelib/ncbiutil.hpp>
 #include "exceptions.hpp"
 #include "type.hpp"
+#include "blocktype.hpp"
 #include "choicestr.hpp"
 #include "stdstr.hpp"
 #include "code.hpp"
@@ -65,8 +66,9 @@ BEGIN_NCBI_SCOPE
 CChoiceTypeStrings::CChoiceTypeStrings(const string& externalName,
                                        const string& className,
                                        const string& namespaceName,
+                                       const CDataType* dataType,
                                        const CComments& comments)
-    : CParent(externalName, className, namespaceName, comments),
+    : CParent(externalName, className, namespaceName, dataType, comments),
       m_HaveAssignment(false)
 {
 }
@@ -870,7 +872,11 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
             "\n"
             "void "<<methodPrefix<<"ThrowInvalidSelection("STATE_ENUM" index) const\n"
             "{\n"
-            "    throw NCBI_NS_NCBI::CInvalidChoiceSelection(DIAG_COMPILE_INFO, m_choice, index, sm_SelectionNames, sizeof(sm_SelectionNames)/sizeof(sm_SelectionNames[0]));\n"
+            "    throw NCBI_NS_NCBI::CInvalidChoiceSelection(DIAG_COMPILE_INFO";
+        if ( 1 ) { // add extra argument for better error message
+            methods << ", this";
+        }
+        methods << ", m_choice, index, sm_SelectionNames, sizeof(sm_SelectionNames)/sizeof(sm_SelectionNames[0]));\n"
             "}\n"
             "\n";
     }
@@ -1362,13 +1368,46 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
     methods <<
         "(\""<<GetExternalName()<<"\", "<<classPrefix<<GetClassNameDT()<<")\n"
         "{\n";
-    if ( !GetModuleName().empty() ) {
+
+    SInternalNames names;
+    string module_name = GetModuleName(&names);
+    if ( GetExternalName().empty() && !names.m_OwnerName.empty() ) {
         methods <<
-            "    SET_CHOICE_MODULE(\""<<GetModuleName()<<"\");\n";
+            "    SET_INTERNAL_NAME(\""<<names.m_OwnerName<<"\", ";
+        if ( !names.m_MemberName.empty() )
+            methods << "\""<<names.m_MemberName<<"\"";
+        else
+            methods << "0";
+        methods << ");\n";
+    }
+    if ( !module_name.empty() ) {
+        methods <<
+            "    SET_CHOICE_MODULE(\""<<module_name<<"\");\n";
+    }
+
+    ENsQualifiedMode defNsqMode = eNSQNotSet;
+    if (DataType()) {
+        defNsqMode = DataType()->IsNsQualified();
+        if (defNsqMode == eNSQNotSet) {
+            const CDataMember *dm = DataType()->GetDataMember();
+            if (dm && dm->Attlist()) {
+                defNsqMode = eNSUnqualified;
+            }
+        }
     }
     if ( !GetNamespaceName().empty() ) {
         methods <<
-            "    SET_NAMESPACE(\""<<GetNamespaceName()<<"\");\n";
+            "    SET_NAMESPACE(\""<<GetNamespaceName()<<"\")";
+        if (defNsqMode != eNSQNotSet) {
+            methods << "->SetNsQualified(";
+            if (defNsqMode == eNSQualified) {
+                methods << "true";
+            } else {
+                methods << "false";
+            }
+            methods << ")";
+        }
+        methods << ";\n";
     }
     if ( delayed ) {
         methods <<
@@ -1536,6 +1575,16 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
                 if (uniseq && uniseq->IsNonEmpty()) {
                     methods << "->SetNonEmpty()";
                 }
+                ENsQualifiedMode memNsqMode = i->dataType->IsNsQualified();
+                if (memNsqMode != eNSQNotSet && memNsqMode != defNsqMode) {
+                    methods << "->SetNsQualified(";
+                    if (memNsqMode == eNSQualified) {
+                        methods << "true";
+                    } else {
+                        methods << "false";
+                    }
+                    methods << ")";
+                }
             }
             methods << ";\n";
         }
