@@ -1,4 +1,4 @@
-/*  $Id: choicetype.cpp 282780 2011-05-16 16:02:27Z gouriano $
+/*  $Id: choicetype.cpp 382295 2012-12-04 20:44:50Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,6 +31,7 @@
 */
 
 #include <ncbi_pch.hpp>
+#include "exceptions.hpp"
 #include "choicetype.hpp"
 #include <serial/impl/autoptrinfo.hpp>
 #include <serial/impl/choice.hpp>
@@ -40,6 +41,7 @@
 #include "srcutil.hpp"
 #include <serial/impl/member.hpp>
 #include <typeinfo>
+#include "aliasstr.hpp"
 
 BEGIN_NCBI_SCOPE
 
@@ -204,7 +206,28 @@ CTypeInfo* CChoiceDataType::CreateTypeInfo(void)
 
 AutoPtr<CTypeStrings> CChoiceDataType::GenerateCode(void) const
 {
+#if 0
     return GetFullCType();
+#else
+    string alias = GetVar("_fullalias");
+    if (alias.empty()) {
+        return GetFullCType();
+    }
+    const CDataType* aliastype = ResolveGlobal(alias);
+    if (!aliastype) {
+        NCBI_THROW(CDatatoolException,eWrongInput,
+            "cannot create type info of _fullalias " + alias);
+    }
+    AutoPtr<CTypeStrings> dType = aliastype->GetRefCType();
+    dType->SetDataType(aliastype);
+    AutoPtr<CAliasTypeStrings> code(new CAliasTypeStrings(GlobalName(),
+                                                          ClassName(),
+                                                          *dType.release(),
+                                                          Comments()));
+    code->SetNamespaceName( GetNamespaceName());
+    code->SetFullAlias();
+    return AutoPtr<CTypeStrings>(code.release());
+#endif
 }
 
 AutoPtr<CTypeStrings> CChoiceDataType::GetRefCType(void) const
