@@ -1,4 +1,4 @@
-/*  $Id: datatool.cpp 366538 2012-06-14 20:27:59Z rafanovi $
+/*  $Id: datatool.cpp 382302 2012-12-04 20:46:40Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -81,7 +81,7 @@ int CDataTool::Run(void)
 
 CDataTool::CDataTool(void)
 {
-    SetVersion( CVersionInfo(2,5,0) );
+    SetVersion( CVersionInfo(2,7,0) );
 }
 
 void CDataTool::Init(void)
@@ -757,6 +757,11 @@ SourceFile::EType CDataTool::LoadDefinitions(
     return srctype;
 }
 
+string CDataTool::GetConfigValue(const string& section, const string& name) const
+{
+    return generator.GetConfig().Get(section,name);
+}
+
 END_NCBI_SCOPE
 
 int NcbiSys_main(int argc, ncbi::TXChar* argv[])
