@@ -1,4 +1,4 @@
-/*  $Id: dtdaux.cpp 282780 2011-05-16 16:02:27Z gouriano $
+/*  $Id: dtdaux.cpp 365689 2012-06-07 13:52:21Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -96,19 +96,23 @@ DTDEntity::EType DTDEntity::GetType(void) const
 }
 
 void DTDEntity::SetParseAttributes( const string& namespaceName,
-    bool elementForm, bool attributeForm)
+    bool elementForm, bool attributeForm,
+    map<string,string>& prefixToNamespace)
 {
     m_TargetNamespace = namespaceName;
     m_ElementFormDefault = elementForm;
     m_AttributeFormDefault = attributeForm;
+    m_PrefixToNamespace = prefixToNamespace;
 }
 
 void DTDEntity::GetParseAttributes( string& namespaceName,
-    bool& elementForm, bool& attributeForm) const
+    bool& elementForm, bool& attributeForm,
+    map<string,string>& prefixToNamespace) const
 {
     namespaceName = m_TargetNamespace;
     elementForm = m_ElementFormDefault;
     attributeForm = m_AttributeFormDefault;
+    prefixToNamespace = m_PrefixToNamespace;
 }
 
 /////////////////////////////////////////////////////////////////////////////
