@@ -1,7 +1,7 @@
 #ifndef DTDPARSER_HPP
 #define DTDPARSER_HPP
 
-/*  $Id: dtdparser.hpp 294599 2011-05-25 15:14:49Z gouriano $
+/*  $Id: dtdparser.hpp 365689 2012-06-07 13:52:21Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -76,10 +76,12 @@ public:
     virtual void EndCommentBlock(void);
 
 protected:
-    AutoPtr<CDataTypeModule> Module(const string& name);
+    void Module(AutoPtr<CFileModules>& modules, const string& name);
 
     virtual void BeginDocumentTree(void);
     virtual void BuildDocumentTree(CDataTypeModule& module);
+    virtual void BuildDataTree(AutoPtr<CFileModules>& modules,
+                               AutoPtr<CDataTypeModule>& module);
     void SkipConditionalSection(void);
 
     virtual string GetLocation(void);
@@ -109,7 +111,7 @@ protected:
     void ConsumeAttributeContent(DTDElement& node, const string& id_name);
     void ParseEnumeratedList(DTDAttribute& attrib);
 
-    void GenerateDataTree(CDataTypeModule& module);
+    void GenerateDataTree(CDataTypeModule& module, const string& name_space);
     void ModuleType(CDataTypeModule& module, const DTDElement& node);
     AutoPtr<CDataType> Type(const DTDElement& node,
                             DTDElement::EOccurrence occ,
