@@ -1,4 +1,4 @@
-/*  $Id: enumtype.cpp 332122 2011-08-23 16:26:09Z vasilche $
+/*  $Id: enumtype.cpp 382302 2012-12-04 20:46:40Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -37,12 +37,12 @@
 #include "module.hpp"
 #include "srcutil.hpp"
 #include <serial/impl/enumerated.hpp>
+#include "stdstr.hpp"
 
 BEGIN_NCBI_SCOPE
 
 CEnumDataType::CEnumDataType(void)
 {
-    ForbidVar("_type", "string");
 }
 
 const char* CEnumDataType::GetASNKeyword(void) const
@@ -139,7 +139,7 @@ void CEnumDataType::PrintXMLSchema(CNcbiOstream& out,
         if (mem->Optional()) {
             use = "optional";
             if (mem->GetDefault()) {
-                use += "\" default=\"" + mem->GetDefault()->GetXmlString();
+                use += "\" default=\"" + GetXmlValueName(mem->GetDefault()->GetXmlString());
             }
         } else {
             use = "required";
@@ -173,7 +173,7 @@ void CEnumDataType::PrintXMLSchema(CNcbiOstream& out,
     if (!inAttlist) {
         const CDataMember* mem = GetDataMember();
         if (mem && mem->Optional() && mem->GetDefault()) {
-            tmp += " default=\"" + mem->GetDefault()->GetXmlString() + "\"";
+            tmp += " default=\"" + GetXmlValueName(mem->GetDefault()->GetXmlString()) + "\"";
         }
     }
     opentag.push_back(tmp + ">");
@@ -227,7 +227,7 @@ void CEnumDataType::PrintDTDElement(CNcbiOstream& out, bool contents_only) const
         const CDataMember* mem = GetDataMember();
         out << "\n    " << tag << " (" << content << ") ";
         if (mem->GetDefault()) {
-            out << "\"" << mem->GetDefault()->GetXmlString() << "\"";
+            out << "\"" << GetXmlValueName(mem->GetDefault()->GetXmlString()) << "\"";
         } else {
             if (mem->Optional()) {
                 out << "#IMPLIED";
@@ -347,6 +347,11 @@ string CEnumDataType::GetDefaultString(const CDataValue& value) const
     }
 }
 
+string CEnumDataType::GetXmlValueName(const string& value) const
+{
+    return value;
+}
+
 CTypeInfo* CEnumDataType::CreateTypeInfo(void)
 {
     AutoPtr<CEnumeratedTypeValues>
@@ -408,6 +413,22 @@ AutoPtr<CTypeStrings> CEnumDataType::GetRefCType(void) const
 
 AutoPtr<CTypeStrings> CEnumDataType::GetFullCType(void) const
 {
+// in case client wants std type instead of enum.
+// I must be accurate here to not to mess with GetEnumCInfo()
+    string type = GetAndVerifyVar("_type");
+    if (!type.empty()) {
+        if (NStr::EndsWith(type, "string")) {
+            return AutoPtr<CTypeStrings>(
+                new CStringTypeStrings("NCBI_NS_STD::string",Comments(),true));
+        } else if (NStr::EndsWith(type, "CStringUTF8")) {
+            return AutoPtr<CTypeStrings>(
+                new CStringTypeStrings("NCBI_NS_NCBI::CStringUTF8",Comments(),true));
+        } else if (type == "double") {
+            return AutoPtr<CTypeStrings>(
+                new CStdTypeStrings(type,Comments(),true));
+        }
+    }
+
     SEnumCInfo enumInfo = GetEnumCInfo();
     AutoPtr<CEnumTypeStrings> 
         e(new CEnumTypeStrings(GlobalName(), enumInfo.enumName,
@@ -438,6 +459,21 @@ bool CIntEnumDataType::IsInteger(void) const
     return true;
 }
 
+string CIntEnumDataType::GetXmlValueName(const string& value) const
+{
+    try {
+// in case of named integers, value can be a name, not an integer
+        TEnumValueType d = (TEnumValueType)NStr::StringToInt(value);
+        ITERATE(TValues, v, GetValues()) {
+            if (v->GetValue() == d) {
+                return v->GetName();
+            }
+        }
+    } catch (...) {
+    }
+    return value;
+}
+
 const char* CBigIntEnumDataType::GetASNKeyword(void) const
 {
     return "BigInt";
