@@ -1,4 +1,4 @@
-/*  $Id: generate.cpp 210903 2010-11-09 13:14:51Z gouriano $
+/*  $Id: generate.cpp 366263 2012-06-13 14:08:32Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -728,9 +728,18 @@ void CCodeGenerator::GenerateModuleHPP(const string& path, list<string>& generat
     bool isfound = false;
     do {    
         isfound = false;
+        bool types_found = false;
         ITERATE ( TOutputFiles, filei, m_Files ) {
             CFileCode* code = filei->second.get();
-            module_name = code->GetPrimaryClass()->GetDoxygenModuleName();
+            list<CTypeStrings*> filetypes;
+            code->GetClasses( filetypes );
+            module_name.clear();
+            ITERATE(list<CTypeStrings*>, t, filetypes) {
+                string module = (*t)->GetDoxygenModuleName();
+                if (module_name.empty() || module.size() < module_name.size()) {
+                    module_name = module;
+                }
+            }
             if (current_module.empty()) {
                 if (modules.find(module_name) != modules.end()) {
                     continue;
@@ -766,6 +775,18 @@ void CCodeGenerator::GenerateModuleHPP(const string& path, list<string>& generat
                 *out <<
                     "void " << current_module << "_RegisterModuleClasses(void);\n\n";
             }
+            ITERATE(list<CTypeStrings*>, t, filetypes) {
+                if ((*t)->GetKind() == CTypeStrings::eKindObject) {
+                    CClassTypeStrings* classtype = dynamic_cast<CClassTypeStrings*>(*t);
+                    if (classtype && classtype->HaveTypeInfo()) {
+                        types_found = true;
+                    }
+                }
+            }
+        }
+        if (isfound && !types_found) {
+            generated.pop_back();
+            out->Discard();
         }
         if (out->is_open()) {
             ns.Reset(*out);
@@ -792,9 +813,18 @@ void CCodeGenerator::GenerateModuleCPP(const string& path, list<string>& generat
     bool isfound = false;
     do {    
         isfound = false;
+        bool types_found = false;
         ITERATE ( TOutputFiles, filei, m_Files ) {
             CFileCode* code = filei->second.get();
-            module_name = code->GetPrimaryClass()->GetDoxygenModuleName();
+            list<CTypeStrings*> filetypes;
+            code->GetClasses( filetypes );
+            module_name.clear();
+            ITERATE(list<CTypeStrings*>, t, filetypes) {
+                string module = (*t)->GetDoxygenModuleName();
+                if (module_name.empty() || module.size() < module_name.size()) {
+                    module_name = module;
+                }
+            }
             if (current_module.empty()) {
                 if (modules.find(module_name) != modules.end()) {
                     continue;
@@ -831,17 +861,29 @@ void CCodeGenerator::GenerateModuleCPP(const string& path, list<string>& generat
                 out_code <<
                     "void " << current_module << "_RegisterModuleClasses(void)\n{\n";
             }
-            if (code->GetPrimaryClass()->GetKind() == CTypeStrings::eKindObject) {
-                CClassTypeStrings* classtype = dynamic_cast<CClassTypeStrings*>(code->GetPrimaryClass());
-                if (classtype && classtype->HaveTypeInfo()) {
-                    out_inc <<
-                        "#include " << code->Include(code->GetUserHPPName()) << "\n";
-                    out_code << "    "
-                             << string(code->GetNamespace())
-                             << classtype->GetClassNameDT() << "::GetTypeInfo();\n";
+            set<string> user_includes;
+            ITERATE(list<CTypeStrings*>, t, filetypes) {
+                if ((*t)->GetKind() == CTypeStrings::eKindObject) {
+                    CClassTypeStrings* classtype = dynamic_cast<CClassTypeStrings*>(*t);
+                    if (classtype && classtype->HaveTypeInfo()) {
+                        types_found = true;
+                        string userhpp(code->Include(code->GetUserHPPName()));
+                        if (user_includes.find(userhpp) == user_includes.end()) {
+                            user_includes.insert(userhpp);
+                            out_inc <<
+                                "#include " << code->Include(code->GetUserHPPName()) << "\n";
+                        }
+                        out_code << "    "
+                                 << code->GetClassNamespace(*t).ToString()
+                                 << classtype->GetClassNameDT() << "::GetTypeInfo();\n";
+                    }
                 }
             }
         }
+        if (isfound && !types_found) {
+            generated.pop_back();
+            out->Discard();
+        }
         if (out->is_open()) {
             out_code << "}\n\n";
             ns.Reset(out_code);
