@@ -1,4 +1,4 @@
-/*  $Id: statictype.cpp 336735 2011-09-07 16:16:59Z vasilche $
+/*  $Id: statictype.cpp 382302 2012-12-04 20:46:40Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -171,9 +171,10 @@ void CStaticDataType::PrintDTDElement(CNcbiOstream& out, bool contents_only) con
 AutoPtr<CTypeStrings> CStaticDataType::GetFullCType(void) const
 {
     string type = GetAndVerifyVar("_type");
+    bool full_ns = !type.empty();
     if ( type.empty() )
         type = GetDefaultCType();
-    return AutoPtr<CTypeStrings>(new CStdTypeStrings(type,Comments()));
+    return AutoPtr<CTypeStrings>(new CStdTypeStrings(type,Comments(),full_ns));
 }
 
 const char* CNullDataType::GetASNKeyword(void) const
@@ -472,7 +473,8 @@ bool CStringDataType::CheckValue(const CDataValue& value) const
 TObjectPtr CStringDataType::CreateDefault(const CDataValue& value) const
 {
     if (m_Type == eStringTypeUTF8) {
-        return new (CStringUTF8*)(new CStringUTF8(dynamic_cast<const CStringDataValue&>(value).GetValue()));
+        return new (CStringUTF8*)(new CStringUTF8(
+            dynamic_cast<const CStringDataValue&>(value).GetValue(), eEncoding_UTF8));
     }
     return new (string*)(new string(dynamic_cast<const CStringDataValue&>(value).GetValue()));
 }
@@ -520,15 +522,16 @@ bool CStringDataType::NeedAutoPointer(TTypeInfo /*typeInfo*/) const
 AutoPtr<CTypeStrings> CStringDataType::GetFullCType(void) const
 {
     string type = GetAndVerifyVar("_type");
+    bool full_ns = !type.empty();
     if ( type.empty() )
         type = GetDefaultCType();
-    return AutoPtr<CTypeStrings>(new CStringTypeStrings(type,Comments()));
+    return AutoPtr<CTypeStrings>(new CStringTypeStrings(type,Comments(),full_ns));
 }
 
 const char* CStringDataType::GetDefaultCType(void) const
 {
     if (m_Type == eStringTypeUTF8) {
-        return "ncbi::CStringUTF8";
+        return "NCBI_NS_NCBI::CStringUTF8";
     }
     return "NCBI_NS_STD::string";
 }
@@ -560,9 +563,10 @@ bool CStringStoreDataType::NeedAutoPointer(TTypeInfo /*typeInfo*/) const
 AutoPtr<CTypeStrings> CStringStoreDataType::GetFullCType(void) const
 {
     string type = GetAndVerifyVar("_type");
+    bool full_ns = !type.empty();
     if ( type.empty() )
         type = GetDefaultCType();
-    return AutoPtr<CTypeStrings>(new CStringStoreTypeStrings(type,Comments()));
+    return AutoPtr<CTypeStrings>(new CStringStoreTypeStrings(type,Comments(),full_ns));
 }
 
 const char* CBitStringDataType::GetASNKeyword(void) const
@@ -600,7 +604,7 @@ AutoPtr<CTypeStrings> CBitStringDataType::GetFullCType(void) const
 
 const char* CBitStringDataType::GetDefaultCType(void) const
 {
-    return "ncbi::CBitString";
+    return "NCBI_NS_NCBI::CBitString";
 }
 
 const char* CBitStringDataType::GetXMLContents(void) const
@@ -739,7 +743,7 @@ bool CIntDataType::CheckValue(const CDataValue& value) const
 
 TObjectPtr CIntDataType::CreateDefault(const CDataValue& value) const
 {
-    return new Int4((Int4)dynamic_cast<const CIntDataValue&>(value).GetValue());
+    return new Int8((Int8)dynamic_cast<const CIntDataValue&>(value).GetValue());
 }
 
 string CIntDataType::GetDefaultString(const CDataValue& value) const
@@ -750,8 +754,8 @@ string CIntDataType::GetDefaultString(const CDataValue& value) const
 CTypeRef CIntDataType::GetTypeInfo(void)
 {
     if ( HaveModuleName() )
-        return UpdateModuleName(CStdTypeInfo<Int4>::CreateTypeInfo());
-    return &CStdTypeInfo<Int4>::GetTypeInfo;
+        return UpdateModuleName(CStdTypeInfo<Int8>::CreateTypeInfo());
+    return &CStdTypeInfo<Int8>::GetTypeInfo;
 }
 
 const char* CIntDataType::GetDefaultCType(void) const
@@ -865,16 +869,17 @@ TObjectPtr CAnyContentDataType::CreateDefault(const CDataValue& value) const
 
 AutoPtr<CTypeStrings> CAnyContentDataType::GetFullCType(void) const
 {
-// TO BE CHANGED !!!
+// TO BE CHANGED ?!!
     string type = GetAndVerifyVar("_type");
+    bool full_ns = !type.empty();
     if ( type.empty() )
         type = GetDefaultCType();
-    return AutoPtr<CTypeStrings>(new CAnyContentTypeStrings(type,Comments()));
+    return AutoPtr<CTypeStrings>(new CAnyContentTypeStrings(type,Comments(),full_ns));
 }
 
 const char* CAnyContentDataType::GetDefaultCType(void) const
 {
-    return "ncbi::CAnyContentObject";
+    return "NCBI_NS_NCBI::CAnyContentObject";
 }
 
 const char* CAnyContentDataType::GetASNKeyword(void) const
