@@ -1,7 +1,7 @@
 #ifndef STLSTR_HPP
 #define STLSTR_HPP
 
-/*  $Id: stlstr.hpp 282780 2011-05-16 16:02:27Z gouriano $
+/*  $Id: stlstr.hpp 382300 2012-12-04 20:46:15Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -61,6 +61,11 @@ public:
         {
             return m_TemplateName;
         }
+    const string& GetTemplateExtraParam(void) const
+        {
+            return m_ExtraParam;
+        }
+
     const CTypeStrings* GetArg1Type(void) const
         {
             return m_Arg1Type.get();
@@ -82,7 +87,10 @@ protected:
     virtual const CNamespace& GetTemplateNamespace(void) const;
 
 private:
+    void x_ParseTemplateName(const string& templateName);
+    
     string m_TemplateName;
+    string m_ExtraParam;
     AutoPtr<CTypeStrings> m_Arg1Type;
 };
 
