@@ -1,4 +1,4 @@
-/*  $Id: traversal_node.cpp 341838 2011-10-24 16:21:26Z kornbluh $
+/*  $Id: traversal_node.cpp 354091 2012-02-23 12:02:31Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -216,7 +216,7 @@ void CTraversalNode::GenerateCode( const string &func_class_name, CNcbiOstream&
         traversal_output_file << endl;
         traversal_output_file << "  try {" << endl;
         traversal_output_file << "    // Try to use an edit handle so we can update the object manager" << endl;
-        traversal_output_file << "    efh = CSeq_feat_EditHandle( m_NewCleanup.m_Scope->GetSeq_featHandle( arg0_raw ) );" << endl;
+        traversal_output_file << "    efh = CSeq_feat_EditHandle( m_Scope.GetSeq_featHandle( arg0_raw ) );" << endl;
         traversal_output_file << "    new_feat.Reset( new CSeq_feat );" << endl;
         traversal_output_file << "    new_feat->Assign( arg0_raw );" << endl;
         traversal_output_file << "  } catch(...) {" << endl;
