@@ -1,4 +1,4 @@
-/*  $Id: type.cpp 351888 2012-02-01 21:09:43Z gouriano $
+/*  $Id: type.cpp 382295 2012-12-04 20:44:50Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -556,6 +556,10 @@ string CDataType::FileName(void) const
         return GetParentType()->FileName();
     }
     if ( m_CachedFileName.empty() ) {
+        string prefix(GetModule()->GetSubnamespace());
+        if (!prefix.empty()) {
+            prefix += '_';
+        }
         const string file = GetVar("_file");
         if ( !file.empty() ) {
             m_CachedFileName = file;
@@ -568,7 +572,7 @@ string CDataType::FileName(void) const
             }
             m_CachedFileName =
                 Path(dir,
-                     MakeFileName(m_MemberName, 5 /* strlen("_.cpp") */ ));
+                     MakeFileName(prefix+m_MemberName, 5 /* strlen("_.cpp") */ ));
         }
     }
     return m_CachedFileName;
@@ -579,7 +583,12 @@ const CNamespace& CDataType::Namespace(void) const
     if ( !m_CachedNamespace.get() ) {
         const string ns = GetVar("_namespace");
         if ( !ns.empty() ) {
-            m_CachedNamespace.reset(new CNamespace(ns));
+            string sub_ns(GetModule()->GetSubnamespace());
+            if (sub_ns.empty()) {
+                m_CachedNamespace.reset(new CNamespace(ns));
+            } else {
+                m_CachedNamespace.reset(new CNamespace(ns + "::" + sub_ns));
+            }
         }
         else {
             if ( GetParentType() ) {
@@ -712,12 +721,14 @@ AutoPtr<CTypeStrings> CDataType::GenerateCode(void) const
         return AutoPtr<CTypeStrings>(code.release());
     }
     else {
+        string fullalias = GetVar("_fullalias");
         AutoPtr<CTypeStrings> dType = GetFullCType();
         AutoPtr<CAliasTypeStrings> code(new CAliasTypeStrings(GlobalName(),
                                                               ClassName(),
                                                               *dType.release(),
                                                               Comments()));
         code->SetNamespaceName( GetNamespaceName());
+        code->SetFullAlias(!fullalias.empty());
         return AutoPtr<CTypeStrings>(code.release());
     }
 }
