@@ -1,4 +1,4 @@
-/*  $Id: member.cpp 332122 2011-08-23 16:26:09Z vasilche $
+/*  $Id: member.cpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -587,12 +587,6 @@ void CMemberInfo::SetPathWriteHook(CObjectOStream* out, const string& path,
     m_WriteHookData.SetPathHook(out,path,hook);
 }
 
-void CMemberInfo::SetGlobalSkipHook(CSkipClassMemberHook* hook)
-{
-    CMutexGuard guard(GetTypeInfoMutex());
-    m_SkipHookData.SetGlobalHook(hook);
-}
-
 void CMemberInfo::SetLocalSkipHook(CObjectIStream& stream,
                                    CSkipClassMemberHook* hook)
 {
@@ -600,12 +594,6 @@ void CMemberInfo::SetLocalSkipHook(CObjectIStream& stream,
     m_SkipHookData.SetLocalHook(stream.m_ClassMemberSkipHookKey, hook);
 }
 
-void CMemberInfo::ResetGlobalSkipHook(void)
-{
-    CMutexGuard guard(GetTypeInfoMutex());
-    m_SkipHookData.ResetGlobalHook();
-}
-
 void CMemberInfo::ResetLocalSkipHook(CObjectIStream& stream)
 {
     CMutexGuard guard(GetTypeInfoMutex());
