@@ -1,4 +1,4 @@
-/*  $Id: objostr.cpp 359471 2012-04-13 12:48:55Z gouriano $
+/*  $Id: objostr.cpp 381682 2012-11-27 20:30:49Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -77,7 +77,7 @@ BEGIN_NCBI_SCOPE
 
 NCBI_PARAM_DECL(bool, SERIAL, FastWriteDouble);
 NCBI_PARAM_DEF(bool, SERIAL, FastWriteDouble, true);
-static NCBI_PARAM_TYPE(SERIAL, FastWriteDouble) s_FastWriteDouble;
+typedef NCBI_PARAM_TYPE(SERIAL, FastWriteDouble) TFastWriteDouble;
 
 
 CObjectOStream* CObjectOStream::Open(ESerialDataFormat format,
@@ -176,78 +176,89 @@ CObjectOStream* CObjectOStream::Open(ESerialDataFormat format,
 /////////////////////////////////////////////////////////////////////////////
 // data verification setup
 
-ESerialVerifyData CObjectOStream::ms_VerifyDataDefault = eSerialVerifyData_Default;
-static CStaticTls<int> s_VerifyTLS;
+
+NCBI_PARAM_ENUM_ARRAY(ESerialVerifyData, SERIAL, VERIFY_DATA_WRITE)
+{
+    {"NO",              eSerialVerifyData_No},
+    {"NEVER",           eSerialVerifyData_Never},
+    {"YES",             eSerialVerifyData_Yes},
+    {"ALWAYS",          eSerialVerifyData_Always},
+    {"DEFVALUE",        eSerialVerifyData_DefValue},
+    {"DEFVALUE_ALWAYS", eSerialVerifyData_DefValueAlways}
+};
+NCBI_PARAM_ENUM_DECL(ESerialVerifyData, SERIAL, VERIFY_DATA_WRITE);
+NCBI_PARAM_ENUM_DEF(ESerialVerifyData, SERIAL, VERIFY_DATA_WRITE, eSerialVerifyData_Default);
+typedef NCBI_PARAM_TYPE(SERIAL, VERIFY_DATA_WRITE) TSerialVerifyData;
 
 
 void CObjectOStream::SetVerifyDataThread(ESerialVerifyData verify)
 {
-    x_GetVerifyDataDefault();
-    ESerialVerifyData tls_verify = ESerialVerifyData(intptr_t(s_VerifyTLS.GetValue()));
-    if (tls_verify != eSerialVerifyData_Never &&
-        tls_verify != eSerialVerifyData_Always &&
-        tls_verify != eSerialVerifyData_DefValueAlways) {
-        if (tls_verify != verify &&
-            (verify == eSerialVerifyData_No || verify == eSerialVerifyData_Never)) {
-            ERR_POST_X_ONCE(2, Warning <<
-                "CObjectOStream::SetVerifyDataThread: data verification disabled");
+    ESerialVerifyData now = TSerialVerifyData::GetThreadDefault();
+    if (now != eSerialVerifyData_Never &&
+        now != eSerialVerifyData_Always &&
+        now != eSerialVerifyData_DefValueAlways) {
+        if (verify == eSerialVerifyData_Default) {
+            TSerialVerifyData::ResetThreadDefault();
+        } else {
+            if (verify != now && 
+                (verify == eSerialVerifyData_No || verify == eSerialVerifyData_Never)) {
+                ERR_POST_X_ONCE(2, Warning <<
+                    "CObjectOStream::SetVerifyDataThread: data verification disabled");
+            }
+            TSerialVerifyData::SetThreadDefault(verify);
         }
-        s_VerifyTLS.SetValue(reinterpret_cast<int*>(verify));
     }
 }
 
 void CObjectOStream::SetVerifyDataGlobal(ESerialVerifyData verify)
 {
-    x_GetVerifyDataDefault();
-    if (ms_VerifyDataDefault != eSerialVerifyData_Never &&
-        ms_VerifyDataDefault != eSerialVerifyData_Always &&
-        ms_VerifyDataDefault != eSerialVerifyData_DefValueAlways) {
-        if (ms_VerifyDataDefault != verify &&
-            (verify == eSerialVerifyData_No || verify == eSerialVerifyData_Never)) {
-            ERR_POST_X_ONCE(3, Warning <<
-                "CObjectOStream::SetVerifyDataGlobal: data verification disabled");
+    ESerialVerifyData now = TSerialVerifyData::GetDefault();
+    if (now != eSerialVerifyData_Never &&
+        now != eSerialVerifyData_Always &&
+        now != eSerialVerifyData_DefValueAlways) {
+        if (verify == eSerialVerifyData_Default) {
+            TSerialVerifyData::ResetDefault();
+        } else {
+            if (verify != now && 
+                (verify == eSerialVerifyData_No || verify == eSerialVerifyData_Never)) {
+                ERR_POST_X_ONCE(3, Warning <<
+                    "CObjectOStream::SetVerifyDataGlobal: data verification disabled");
+            }
+            TSerialVerifyData::SetDefault(verify);
         }
-        ms_VerifyDataDefault = verify;
     }
 }
 
 ESerialVerifyData CObjectOStream::x_GetVerifyDataDefault(void)
 {
-    ESerialVerifyData verify;
-    if (ms_VerifyDataDefault == eSerialVerifyData_Never ||
-        ms_VerifyDataDefault == eSerialVerifyData_Always ||
-        ms_VerifyDataDefault == eSerialVerifyData_DefValueAlways) {
-        verify = ms_VerifyDataDefault;
-    } else {
-        verify = ESerialVerifyData(intptr_t(s_VerifyTLS.GetValue()));
-        if (verify == eSerialVerifyData_Default) {
-            if (ms_VerifyDataDefault == eSerialVerifyData_Default) {
-
-                // change the default here, if you wish
-                ms_VerifyDataDefault = eSerialVerifyData_Yes;
-                //ms_VerifyDataDefault = eSerialVerifyData_No;
-
-                const char* str = getenv(SERIAL_VERIFY_DATA_WRITE);
-                if (str) {
-                    if (NStr::CompareNocase(str,"YES") == 0) {
-                        ms_VerifyDataDefault = eSerialVerifyData_Yes;
-                    } else if (NStr::CompareNocase(str,"NO") == 0) {
-                        ms_VerifyDataDefault = eSerialVerifyData_No;
-                    } else if (NStr::CompareNocase(str,"NEVER") == 0) {
-                        ms_VerifyDataDefault = eSerialVerifyData_Never;
-                    } else  if (NStr::CompareNocase(str,"ALWAYS") == 0) {
-                        ms_VerifyDataDefault = eSerialVerifyData_Always;
-                    } else  if (NStr::CompareNocase(str,"DEFVALUE") == 0) {
-                        ms_VerifyDataDefault = eSerialVerifyData_DefValue;
-                    } else  if (NStr::CompareNocase(str,"DEFVALUE_ALWAYS") == 0) {
-                        ms_VerifyDataDefault = eSerialVerifyData_DefValueAlways;
-                    }
+    ESerialVerifyData now = TSerialVerifyData::GetThreadDefault();
+    if (now == eSerialVerifyData_Default) {
+        now = TSerialVerifyData::GetDefault();
+        if (now == eSerialVerifyData_Default) {
+// this is to provide compatibility with old implementation
+            const char* str = getenv(SERIAL_VERIFY_DATA_WRITE);
+            if (str) {
+                if (NStr::CompareNocase(str,"YES") == 0) {
+                    now = eSerialVerifyData_Yes;
+                } else if (NStr::CompareNocase(str,"NO") == 0) {
+                    now = eSerialVerifyData_No;
+                } else if (NStr::CompareNocase(str,"NEVER") == 0) {
+                    now = eSerialVerifyData_Never;
+                } else  if (NStr::CompareNocase(str,"ALWAYS") == 0) {
+                    now = eSerialVerifyData_Always;
+                } else  if (NStr::CompareNocase(str,"DEFVALUE") == 0) {
+                    now = eSerialVerifyData_DefValue;
+                } else  if (NStr::CompareNocase(str,"DEFVALUE_ALWAYS") == 0) {
+                    now = eSerialVerifyData_DefValueAlways;
                 }
             }
-            verify = ms_VerifyDataDefault;
         }
     }
-    return verify;
+    if (now != eSerialVerifyData_Default) {
+        return now;
+    }
+    // change the default here, if you like
+    return eSerialVerifyData_Yes;
 }
 
 
@@ -260,7 +271,7 @@ CObjectOStream::CObjectOStream(ESerialDataFormat format,
       m_DataFormat(format),
       m_WriteNamedIntegersByValue(false),
       m_ParseDelayBuffers(eDelayBufferPolicyNotSet),
-      m_FastWriteDouble(s_FastWriteDouble.Get()),
+      m_FastWriteDouble(TFastWriteDouble::GetDefault()),
       m_VerifyData(x_GetVerifyDataDefault())
 {
 }
@@ -1122,7 +1133,7 @@ void CObjectOStream::SetFormattingFlags(TSerial_Format_Flags flags)
     TSerial_Format_Flags accepted =
         fSerial_AsnText_NoIndentation | fSerial_AsnText_NoEol;
     if (flags & ~accepted) {
-        ERR_POST_X_ONCE(12, Warning <<
+        ERR_POST_X_ONCE(13, Warning <<
             "CObjectOStream::SetFormattingFlags: ignoring unknown formatting flags");
     }
     SetUseIndentation((flags & fSerial_AsnText_NoIndentation) == 0);
