@@ -1,4 +1,4 @@
-/*  $Id: objostrasnb.cpp 361377 2012-05-01 17:30:53Z gouriano $
+/*  $Id: objostrasnb.cpp 370581 2012-07-31 15:09:30Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -256,8 +256,9 @@ NCBI_PARAM_DEF_EX(bool, SERIAL, WRITE_UTF8STRING_TAG, false,
 
 CObjectOStreamAsnBinary::TByte CObjectOStreamAsnBinary::MakeUTF8StringTag(void)
 {
-    static NCBI_PARAM_TYPE(SERIAL, WRITE_UTF8STRING_TAG) s_WriteUTF8StringTag;
-    ETagValue value = s_WriteUTF8StringTag.Get()? eUTF8String: eVisibleString;
+    static const bool s_WriteUTF8StringTag =
+        NCBI_PARAM_TYPE(SERIAL, WRITE_UTF8STRING_TAG)::GetDefault();
+    ETagValue value = s_WriteUTF8StringTag ? eUTF8String: eVisibleString;
     return MakeTagByte(eUniversal, ePrimitive, value);
 }
 
