@@ -1,4 +1,4 @@
-/*  $Id: typeinfo.cpp 339061 2011-09-26 14:09:07Z gouriano $
+/*  $Id: typeinfo.cpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -352,12 +352,6 @@ void CTypeInfo::SetPathWriteHook(CObjectOStream* out, const string& path,
     m_WriteHookData.SetPathHook(out,path,hook);
 }
 
-void CTypeInfo::SetGlobalSkipHook(CSkipObjectHook* hook)
-{
-    CMutexGuard guard(GetTypeInfoMutex());
-    m_SkipHookData.SetGlobalHook(hook);
-}
-
 void CTypeInfo::SetLocalSkipHook(CObjectIStream& stream,
                                  CSkipObjectHook* hook)
 {
@@ -366,12 +360,6 @@ void CTypeInfo::SetLocalSkipHook(CObjectIStream& stream,
     stream.AddMonitorType(this);
 }
 
-void CTypeInfo::ResetGlobalSkipHook(void)
-{
-    CMutexGuard guard(GetTypeInfoMutex());
-    m_SkipHookData.ResetGlobalHook();
-}
-
 void CTypeInfo::ResetLocalSkipHook(CObjectIStream& stream)
 {
     CMutexGuard guard(GetTypeInfoMutex());
