@@ -1,4 +1,4 @@
-/* $Id: streambuf.cpp 367949 2012-06-29 15:16:21Z ivanov $
+/* $Id: streambuf.cpp 386025 2013-01-15 19:02:03Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -74,12 +74,11 @@ CCompressionStreambuf::CCompressionStreambuf(
     }
 
     // Allocate memory for all buffers at one time
-    auto_ptr<CT_CHAR_TYPE> bp(new CT_CHAR_TYPE[size_t(read_bufsize + write_bufsize)]);
+    AutoArray<CT_CHAR_TYPE> bp(new CT_CHAR_TYPE[size_t(read_bufsize + write_bufsize)]);
     m_Buf = bp.get();
     if ( !m_Buf ) {
         return;
     }
-
     // Init processors and set the buffer pointers
     if ( m_Reader ) {
         m_Reader->Init();
@@ -294,12 +293,14 @@ int CCompressionStreambuf::Flush(CCompressionStream::EDirection dir)
             }
         }
     } while (sp->m_LastStatus == CP::eStatus_Repeat  ||
-            (out_avail  &&  sp->m_LastStatus == CP::eStatus_Overflow));
+            (out_avail  &&  (sp->m_LastStatus == CP::eStatus_Success || 
+                             sp->m_LastStatus == CP::eStatus_Overflow))
+            );
 
     // Flush underlying stream (on write)
     if (dir == CCompressionStream::eWrite) {
         if ( sp->m_LastStatus == CP::eStatus_EndOfData  ||
-            (sp->m_State == CSP::eFinalize && !out_avail)) {
+             sp->m_State == CSP::eFinalize) {
             if ( !WriteOutBufToStream(true /*force write*/) ) {
                 return -1;
             }
