@@ -1,4 +1,4 @@
-/*  $Id: format_guess.cpp 372350 2012-08-16 17:41:23Z rafanovi $
+/*  $Id: format_guess.cpp 390749 2013-03-01 18:22:26Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1067,6 +1067,65 @@ CFormatGuess::TestFormatAlignment(
 }
 
 //  -----------------------------------------------------------------------------
+ bool 
+ CFormatGuess::x_TestTableDelimiter(const string& delims)
+ {
+    list<string>::const_iterator iter = m_TestLines.begin();
+    list<string> toks;
+
+    // Merge delims if > 1.  Do not merge single delims (since they could 
+    // more easily represent blank fields
+    NStr::EMergeDelims  merge_delims = NStr::eMergeDelims;
+    if (delims.size() == 1)
+        merge_delims = NStr::eNoMergeDelims;
+
+
+    // Skip initial lines since not all headers start with comments like # or ;:
+    // Don't skip though if file is very short - add up to 3, 1 for each line 
+    // over 5:
+    for (size_t i=5; i<7; ++i)
+        if (m_TestLines.size() > i) ++iter;
+
+    /// determine the number of observed columns
+    size_t ncols = 0;
+    bool found = false;
+    for ( ;  iter != m_TestLines.end()  &&  ! found;  ++iter) {
+        if (iter->empty()  ||  (*iter)[0] == '#'  ||  (*iter)[0] == ';') {
+            continue;
+        }
+
+        toks.clear();
+        NStr::Split(*iter, delims, toks);
+        ncols = toks.size();
+        found = true;
+    }
+    if ( ncols < 2 ) {
+        return false;
+    }
+
+    size_t nlines = 1;
+    // verify that columns all have the same size
+    // we can add an exception for the last line
+    for ( ;  iter != m_TestLines.end();  ++iter) {
+        if (iter->empty()  ||  (*iter)[0] == '#'  ||  (*iter)[0] == ';') {
+            continue;
+        } 
+
+        toks.clear();
+        NStr::Split(*iter, delims, toks);
+        if (toks.size() != ncols) {
+            list<string>::const_iterator it = iter;
+            ++it;
+            if (it != m_TestLines.end() || (m_iTestDataSize < s_iTestBufferSize) ) {
+                return false;
+            }
+        } else {
+            ++nlines;
+        }
+    }
+    return ( nlines >= 2 );
+ }
+
 bool
 CFormatGuess::TestFormatTable(
     EMode /* not used */ )
@@ -1093,47 +1152,19 @@ CFormatGuess::TestFormatTable(
     //  - the sample contains at least two non-comment lines.
     //
 
-    list<string>::const_iterator iter = m_TestLines.begin();
-    list<string> toks;
+    //' ' ' \t' '\t' ',' '|'
+    if (x_TestTableDelimiter(" "))
+        return true;
+    else if (x_TestTableDelimiter(" \t"))
+        return true;
+    else if (x_TestTableDelimiter("\t"))
+        return true;
+    else if (x_TestTableDelimiter(","))
+        return true;
+    else if (x_TestTableDelimiter("|"))
+        return true;
 
-    /// determine the number of observed columns
-    size_t ncols = 0;
-    bool found = false;
-    for ( ;  iter != m_TestLines.end()  &&  ! found;  ++iter) {
-        if (iter->empty()  ||  (*iter)[0] == '#'  ||  (*iter)[0] == ';') {
-            continue;
-        }
-
-        toks.clear();
-        NStr::Split(*iter, " \t,", toks);
-        ncols = toks.size();
-        found = true;
-    }
-    if ( ncols < 2 ) {
-        return false;
-    }
-
-    size_t nlines = 1;
-    // verify that columns all have the same size
-    // we can add an exception for the last line
-    for ( ;  iter != m_TestLines.end();  ++iter) {
-        if (iter->empty()  ||  (*iter)[0] == '#'  ||  (*iter)[0] == ';') {
-            continue;
-        }
-
-        toks.clear();
-        NStr::Split(*iter, " \t,", toks);
-        if (toks.size() != ncols) {
-            list<string>::const_iterator it = iter;
-            ++it;
-            if (it != m_TestLines.end() || (m_iTestDataSize < s_iTestBufferSize) ) {
-                return false;
-            }
-        } else {
-            ++nlines;
-        }
-    }
-    return ( nlines >= 2 );
+    return false;
 }
 
 //  -----------------------------------------------------------------------------
@@ -1338,6 +1369,17 @@ CFormatGuess::TestFormatBed15(
         if ( columns.size() != columncount ) {
             return false;
         } else {
+            if (!s_IsTokenPosInt(columns[1]) ||   //chr start
+                !s_IsTokenPosInt(columns[2]) ||   //chr end
+                !s_IsTokenPosInt(columns[4]) ||   //score
+                !s_IsTokenPosInt(columns[6]) ||   //thick draw start
+                !s_IsTokenPosInt(columns[7]))     //thick draw end
+                    return false;
+            string strand = NStr::TruncateSpaces(columns[5]);
+            
+            if (strand != "+" && strand != "-")
+                return false;
+
             LineFound = true;
         }
     }
@@ -1361,6 +1403,17 @@ CFormatGuess::TestFormatWiggle(
                 return true;
             }
         }
+        if ( NStr::StartsWith(*it, "fixedStep") ) { /* MSS-140 */
+            if ( NStr::Find(*it, "chrom=")  &&  NStr::Find(*it, "start=") ) {
+                return true;
+            } 
+        }
+        if ( NStr::StartsWith(*it, "variableStep") ) { /* MSS-140 */
+            if ( NStr::Find(*it, "chrom=") ) {
+                return true;
+            }
+            return true;
+        }
     }
     return false;
 }
@@ -1665,17 +1718,17 @@ CFormatGuess::IsSampleNewick(
         //  Strip out comments:
         string trimmed;
         bool in_comment = false;
-        for ( size_t ii=0; line[ii] != 0; ++ii ) {
+        for ( size_t ii=0; line.c_str()[ii] != 0; ++ii ) {
             if ( ! in_comment ) {
-                if ( line[ii] != '[' ) {
-                    trimmed += line[ii];
+                if ( line.c_str()[ii] != '[' ) {
+                    trimmed += line.c_str()[ii];
                 }
                 else {
                     in_comment = true;
                 }
             }
             else /* in_comment */ {
-                if ( line[ii] == ']' ) {
+                if ( line.c_str()[ii] == ']' ) {
                     in_comment = false;
                 }
             }
@@ -1686,10 +1739,10 @@ CFormatGuess::IsSampleNewick(
         //  Compress quoted labels:
         string trimmed;
         bool in_quote = false;
-        for ( size_t ii=0; line[ii] != 0; ++ii ) {
+        for ( size_t ii=0; line.c_str()[ii] != 0; ++ii ) {
             if ( ! in_quote ) {
-                if ( line[ii] != '\'' ) {
-                    trimmed += line[ii];
+                if ( line.c_str()[ii] != '\'' ) {
+                    trimmed += line.c_str()[ii];
                 }
                 else {
                     in_quote = true;
@@ -1697,7 +1750,7 @@ CFormatGuess::IsSampleNewick(
                 }
             }
             else { /* in_quote */
-                if ( line[ii] == '\'' ) {
+                if ( line.c_str()[ii] == '\'' ) {
                     in_quote = false;
                 }
             }
@@ -1708,21 +1761,21 @@ CFormatGuess::IsSampleNewick(
         //  Strip distance markers:
         string trimmed;
         size_t ii=0;
-        while ( line[ii] != 0 ) {
-            if ( line[ii] != ':' ) {
-                trimmed += line[ii++];
+        while ( line.c_str()[ii] != 0 ) {
+            if ( line.c_str()[ii] != ':' ) {
+                trimmed += line.c_str()[ii++];
             }
             else {
                 ii++;
-                if ( line[ii] == '-'  || line[ii] == '+' ) {
+                if ( line.c_str()[ii] == '-'  || line.c_str()[ii] == '+' ) {
                     ii++;
                 }
-                while ( '0' <= line[ii] && line[ii] <= '9' ) {
+                while ( '0' <= line.c_str()[ii] && line.c_str()[ii] <= '9' ) {
                     ii++;
                 }
-                if ( line[ii] == '.' ) {
+                if ( line.c_str()[ii] == '.' ) {
                     ii++;
-                    while ( '0' <= line[ii] && line[ii] <= '9' ) {
+                    while ( '0' <= line.c_str()[ii] && line.c_str()[ii] <= '9' ) {
                         ii++;
                     }
                 }
@@ -1736,8 +1789,8 @@ CFormatGuess::IsSampleNewick(
             return false;
         }
         size_t paren_count = 1;
-        for ( size_t ii=1; line[ii] != 0; ++ii ) {
-            switch ( line[ii] ) {
+        for ( size_t ii=1; line.c_str()[ii] != 0; ++ii ) {
+            switch ( line.c_str()[ii] ) {
                 default: 
                     break;
                 case '(':
@@ -2313,7 +2366,7 @@ bool CFormatGuess::IsLineHgvs(
     const string& line )
 {
     // This simple check can mistake Newwick, so Newwick is checked first
-    //  /:(g|c|r|p|m|mt)\./  as in NC_000001.9:g.1234567C>T
+    //  /:(g|c|r|p|m|mt|n)\./  as in NC_000001.9:g.1234567C>T
     int State = 0;
     ITERATE(string, Iter, line) {
         char Char = *Iter;
@@ -2331,9 +2384,12 @@ bool CFormatGuess::IsLineHgvs(
                 Char == 'c' ||
                 Char == 'r' ||
                 Char == 'p' ||
-               (Char == 'm' && Next == 't') ||
+                Char == 'n' ||
                 Char == 'm' ) {
                 State = 2;
+                if (Char=='m' && Next == 't') {
+                    ++Iter;
+                }
             }
         } else if(State == 2) {
             if(Char == '.') 
