@@ -1,4 +1,4 @@
-/*  $Id: ncbi_url.cpp 342890 2011-11-01 18:41:53Z grichenk $
+/*  $Id: ncbi_url.cpp 369170 2012-07-17 13:20:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -552,8 +552,8 @@ const CUrlArgs& CUrl::GetArgs(void) const
 
 IUrlEncoder* CUrl::GetDefaultEncoder(void)
 {
-    static CDefaultUrlEncoder s_DefaultEncoder;
-    return &s_DefaultEncoder;
+    static CSafeStaticPtr<CDefaultUrlEncoder> s_DefaultEncoder;
+    return &s_DefaultEncoder.Get();
 }
 
 
