@@ -1,4 +1,4 @@
-/*  $Id: strbuffer.cpp 348644 2012-01-03 15:53:45Z vasilche $
+/*  $Id: strbuffer.cpp 355801 2012-03-08 16:20:00Z ivanovp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -553,7 +553,7 @@ void CIStreamBuffer::SetStreamPos(CNcbiStreampos pos)
             NCBI_THROW(CIOException,eRead,"stream position is out of buffer");
         }
         m_BufferPos = pos;
-        m_CurrentPos = m_Buffer + pos;
+        m_CurrentPos = m_Buffer + (size_t)pos;
         m_Line = 1;
     }
 }
@@ -727,7 +727,7 @@ COStreamBuffer::COStreamBuffer(CNcbiOstream& out, bool deleteOut)
       m_CurrentPos(m_Buffer),
       m_BufferEnd(m_Buffer + KInitialBufferSize),
       m_Line(1), m_LineLength(0),
-      m_BackLimit(0), m_UseIndentation(true),
+      m_BackLimit(0), m_UseIndentation(true), m_UseEol(true),
       m_CanceledCallback(0)
 {
 }
@@ -985,6 +985,9 @@ void COStreamBuffer::PutUint8(Uint8 v)
 void COStreamBuffer::PutEolAtWordEnd(size_t lineLength)
     THROWS1((CIOException, bad_alloc))
 {
+    if (!GetUseEol()) {
+        return;
+    }
     Reserve(1);
     size_t linePos = m_LineLength;
     char* pos = m_CurrentPos;
