@@ -1,4 +1,4 @@
-/*  $Id: thread_pool_old.cpp 301002 2011-06-06 17:18:53Z ivanovp $
+/*  $Id: thread_pool_old.cpp 384924 2013-01-03 16:08:30Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -48,7 +48,7 @@ protected:
 };
 
 
-void CStdPoolOfThreads::KillAllThreads(bool wait)
+void CStdPoolOfThreads::KillAllThreads(TKillFlags flags)
 {
     TACValue n, old_max;
     {{
@@ -58,6 +58,17 @@ void CStdPoolOfThreads::KillAllThreads(bool wait)
         n = m_ThreadCount.Get(); // Capture for use without mutex
     }}
 
+    {{
+        size_t n2 = m_Threads.size();
+        if (n != n2) {
+            ERR_POST(Warning << "Registered " << n2 << " threads but expected "
+                     << n);
+            if (n2 > n) {
+                n = n2;
+            }
+        }
+    }}
+
     CRef<CStdRequest> poison(new CFatalRequest);
 
     for (TACValue i = 0;  i < n;  ) {
@@ -70,17 +81,17 @@ void CStdPoolOfThreads::KillAllThreads(bool wait)
         }
     }
     NON_CONST_ITERATE(TThreads, it, m_Threads) {
-        if (wait) {
+        if ((flags & fKill_Wait) != 0) {
             (*it)->Join();
         } else {
             (*it)->Detach();
         }
     }
     m_Threads.clear();
-    {{
+    if ((flags & fKill_Reopen) != 0) {
         CMutexGuard guard(m_Mutex);
         m_MaxThreads = old_max;
-    }}
+    }
 }
 
 
@@ -89,6 +100,9 @@ void CStdPoolOfThreads::Register(TThread& thread)
     CMutexGuard guard(m_Mutex);
     if (m_MaxThreads > 0) {
         m_Threads.push_back(CRef<TThread>(&thread));
+    } else {
+        NCBI_THROW(CThreadException, eRunError,
+                   "No more threads allowed in pool.");
     }
 }
 
@@ -108,7 +122,7 @@ void CStdPoolOfThreads::UnRegister(TThread& thread)
 CStdPoolOfThreads::~CStdPoolOfThreads()
 {
     try {
-        KillAllThreads(false);
+        KillAllThreads(0);
     } catch(...) {}    // Just to be sure that we will not throw from the destructor.
 }
 
